/*********************************************************************\

    File        :   placer_main.c

    Date        :   30-06-94
    Version     :   1

    Auteur      :   BEAURIN oliver

    Description :   Placer

\*********************************************************************/

/*---------------------------------------------------------\
                     includes
\---------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <strings.h>

#include MUT_H
#include MLO_H
#include MPH_H
#include APR_H

#include GFP_H

#include "dpr.h"
#include "placer_structure.h"
#include "placer_utils.h"
#include "placer_decoup.h"
#include "placer_create_grid.h"
#include "placer_create_con.h"
#include "placer_dprfile.h"

/*---------------------------------------------------------\
                     defines
\---------------------------------------------------------*/
#define WIDTH_OF_GLUE_PTYPE 0xbaffe

/*---------------------------------------------------------\
                     locals vars
\---------------------------------------------------------*/
static chain_list *model_already_done = NULL;

/*---------------------------------------------------------\
                    found_nb_slices
\---------------------------------------------------------*/
long found_nb_slices ( ptplacement )
placement *ptplacement;
{
    column  *ptcolumn;
    slice   *ptslice;
    long     nbslicesmax;
    long     nbslices;

    if( !ptplacement->COLUMNS )
    {
        return 0L;
    }

    nbslicesmax = -1L;

    for( ptcolumn = ptplacement->COLUMNS;
         ptcolumn;
         ptcolumn = ptcolumn->NEXT )
    {
        for( ptslice = ptcolumn->SLICES, nbslices = -1L;
             ptslice;
             ptslice = ptslice->NEXT)
        {
            if( ptslice->INDEX > nbslices )
                nbslices = ptslice->INDEX;
        }
 
        if(nbslices > nbslicesmax)
            nbslicesmax = nbslices;
    }

    return nbslicesmax + 1;

} /* end of found_nb_slices */

/*---------------------------------------------------------\
                    found_nb_cols
\---------------------------------------------------------*/
long found_nb_cols( ptplacement )
placement *ptplacement;
{
    column  *ptcolumn;
    long     nbcols;

    if( !ptplacement->COLUMNS )
    {
        return 0L;
    }

    for( ptcolumn = ptplacement->COLUMNS, nbcols = 0L;
         ptcolumn;
         ptcolumn = ptcolumn->NEXT, nbcols++ )
	{
		ptcolumn->INDEX = nbcols;
	}
    
    return nbcols;

} /* end of found_nb_cols */

/*---------------------------------------------------------\
                    make_slices_array
\---------------------------------------------------------*/
void make_slices_array ( ptplacement )
placement  *ptplacement;
{
    long slicei;
    long nbslices;

    nbslices = ptplacement->NB_SLICES;

    ptplacement->SLICESHEIGHTS = (long * )mbkalloc( ( nbslices + 1 ) * sizeof( long ) );
    ptplacement->YSLICE = (long * )mbkalloc( ( nbslices + 1 ) * sizeof( long ) );
    ptplacement->TYPES = (char * )mbkalloc( ( nbslices + 1 ) * sizeof( char ) );

    for( slicei = 0; slicei <= nbslices;  slicei++ )
    {
        ptplacement->SLICESHEIGHTS[ slicei ] = -1;
        ptplacement->TYPES[ slicei ] = -1;
    }

} /* end of make_slices_array */

/*---------------------------------------------------------\
                     check_column_slice_heights
\---------------------------------------------------------*/
void check_column_slice_heights( ptplacement, ptcolumn )
placement  *ptplacement;
column     *ptcolumn;
{
    slice   *ptslice;
    long     colwidth;

    colwidth = 0;

    for( ptslice = ptcolumn->SLICES;
         ptslice;
         ptslice = ptslice->NEXT)
    {
        if( ptslice->WIDTH > colwidth )
            colwidth = ptslice->WIDTH;

        if( ptplacement->SLICESHEIGHTS[ ptslice->INDEX ] == -1 )
        {
            ptplacement->SLICESHEIGHTS[ ptslice->INDEX ] = ptslice->HEIGHT;
        }
        else
        {
            if( ptplacement->SLICESHEIGHTS[ ptslice->INDEX ] != ptslice->HEIGHT )
            {
                fprintf( stderr, "*** dpp error *** : All slice don't have the same height.\n" );
                exit( 1 );
            }
        }
    }
    ptcolumn->WIDTH = colwidth; 

} /* end of check_column_slice_heights */

/*---------------------------------------------------------\
                    check_slices_heights
\---------------------------------------------------------*/
void check_slices_heights ( ptplacement )
placement  *ptplacement;
{
    column  *ptcolumn;

    if( !ptplacement->COLUMNS )
    {
        return;
    }

    for( ptcolumn = ptplacement->COLUMNS;
         ptcolumn;
         ptcolumn = ptcolumn->NEXT )
    {
        check_column_slice_heights( ptplacement, ptcolumn );
    }
    
} /* end of check_slices_heights */

/*---------------------------------------------------------\
                    check_alims
\---------------------------------------------------------*/
void check_alims( ptplacement )
placement  *ptplacement;
{
    column  *ptcolumn;
    slice   *ptslice;
    char     type;

    if( !ptplacement->COLUMNS )
    {
        return;
    }

    for( ptcolumn = ptplacement->COLUMNS;
         ptcolumn;
         ptcolumn = ptcolumn->NEXT )
    {
        for( ptslice = ptcolumn->SLICES;
             ptslice;
             ptslice = ptslice->NEXT)
        {
            if( ( ptslice->MODEL_NAME ) && ( ptslice->INDEX & 1 ) )
            {
                type = ptslice->TYPE_UP;
            }
            else
            {
                type = ptslice->TYPE_DOWN;
            }
            if( ptplacement->TYPES[ ptslice->INDEX ] == -1 )
            {
                ptplacement->TYPES[ ptslice->INDEX ] = type;
            }
            else
            {
                if( ptplacement->TYPES[ ptslice->INDEX ] != type )
                {
                    fprintf( stderr, "*** dpp error *** : Alims correspondances error in %s slice : %ld.\n", ptcolumn->INSTANCE_NAME, ptslice->INDEX );
                    exit( 1 );
                }
            }
        }
    }
    
} /* end of check_alims */

/*---------------------------------------------------------\
                    fill_yslice_array
\---------------------------------------------------------*/
void fill_yslice_array ( ptplacement )
placement  *ptplacement;
{
    long slicei;
    long yslice;
    long nbslices;

    nbslices = ptplacement->NB_SLICES;

    yslice = 0L;
    for( slicei = 0; slicei <= nbslices; slicei ++ )
    {
        ptplacement->YSLICE[ slicei ] = yslice;
        if( ptplacement->SLICESHEIGHTS[ slicei ] != -1L )
            yslice += ptplacement->SLICESHEIGHTS[ slicei ];
    }

} /* end of fill_yslice_array */

/*---------------------------------------------------------\
                    in_already_done
\---------------------------------------------------------*/
char in_already_done ( name )
char *name;
{
    chain_list *ptchain;

    for( ptchain = model_already_done; ptchain; ptchain = ptchain->NEXT )
    {
        if( (char *)ptchain->DATA == name )
            return TRUE;
    }

    return FALSE;

} /* end of in_already_done */

/*---------------------------------------------------------\
                    make_cols
\---------------------------------------------------------*/
void make_cols( ptplacement, ptcolumn )
placement  *ptplacement;
column     *ptcolumn;
{
    phfig_list *ptphcol;
    slice      *ptslice;
    long        nbslices;

    nbslices = ptplacement->NB_SLICES;

    if( !ptcolumn )
    {
        fprintf( stderr, "*** dpp bug *** : i am sorry but you find a bug (make_cols 1), i can't continue...\n" );
        exit( 1 );
    }

    if( ptcolumn->TYPE != COL_COLUMN )
    {
        fprintf( stderr, "*** dpp bug *** : i am sorry but you find a bug (make_cols 2), i can't continue...\n" );
        exit( 1 );
    }

    if( in_already_done( ptcolumn->MODEL_NAME ) )
    {
        if( verbose_mode == VERBOSE_ALL )
        {
            fprintf( stdout, "        model already create.\n" );
            fflush( stdout );
        }
        return;
    }

    delphfig( ptcolumn->MODEL_NAME );
    ptphcol = addphfig( ptcolumn->MODEL_NAME );

    model_already_done = addchain( model_already_done, ptcolumn->MODEL_NAME );

    defab( ptphcol,
           0L, 0L,
           ptcolumn->WIDTH * SCALE_X,
           ptplacement->YSLICE[ nbslices ] * SCALE_X );
    
    for( ptslice = ptcolumn->SLICES;
         ptslice;
         ptslice = ptslice->NEXT)
    {
        if( ptslice->MODEL_NAME )
        {
            addphins( ptphcol,
                      ptslice->MODEL_NAME,
                      ptslice->INSTANCE_NAME,
                      ( ptslice->INDEX & 1 ) ? SYM_Y : NOSYM,
                      0L,
                      ptplacement->YSLICE[ ptslice->INDEX ] * SCALE_X );
        }
    }

    savephfig( ptphcol );

} /* end of make_cols */

/*---------------------------------------------------------\
                    make_phfig_placement
\---------------------------------------------------------*/
void make_phfig_placement ( ptplacement, ptlofig )
placement  *ptplacement;
lofig_list *ptlofig;
{
    locon_list *ptlocon;
    locon_list *ptloconi;
    chain_list *ptchain;
    phfig_list *ptphfig;
    column     *ptcolumn;
    long        X;
    long        Index;
    con_NS     *ptconNS;
    char        found_NS_con;
    con_EW     *ptconEW;
    con_EW     *conEW_min;
    con_EW     *conEW_max;
    column     *conEW_colmin;
    column     *conEW_colmax;
    long        conEW_minX;
    long        conEW_maxX;
    long        nbslices;
    long        slicei;
    long        track;
    ptype_list *ptype;
    lofig_ab   *defab_struct;
	ulocon_place_list *locon_place;

    nbslices = ptplacement->NB_SLICES;
 
    if( !ptplacement->COLUMNS )
    {
        return;
    }

    ptphfig = addphfig( ptplacement->MODEL_NAME );

    X     = 0L;

    for( ptcolumn = ptplacement->COLUMNS;
         ptcolumn;
         ptcolumn = ptcolumn->NEXT )
    {
        Index = ptcolumn->INDEX;

        if( ptcolumn->TYPE == COL_EMPTY )
        {
            continue;
        }

        ptcolumn->X = X * SCALE_X;

        addphins( ptphfig,
                  ptcolumn->MODEL_NAME,
                  ptcolumn->INSTANCE_NAME,
                  NOSYM,
                  X * SCALE_X,
                  0L );

        X = X + ptcolumn->WIDTH;
    }

    ptype = getptype( ptlofig->USER, (long)C_GfpUT_UserLoFigAB );
    if( ptype != NULL )
    {
        defab_struct = (lofig_ab *)( ptype->DATA );

        defab( ptphfig,
               0L, defab_struct->DY1 * SCALE_X,
               X * SCALE_X,
               ( ptplacement->YSLICE[ nbslices ] + defab_struct->DY2 ) * SCALE_X );
    }
    else
    {
        defab( ptphfig,
               0L, 0L,
               X * SCALE_X,
               ptplacement->YSLICE[ nbslices ] * SCALE_X );
    }


    for( ptlocon=ptlofig->LOCON; ptlocon; ptlocon=ptlocon->NEXT )
    {
        if( isvdd( ptlocon->NAME ) || isvss( ptlocon->NAME ) )
        {
            continue;
        }


        ptchain = (chain_list *)( getptype( ptlocon->SIG->USER, (long)LOFIGCHAIN)->DATA );

        found_NS_con = FALSE;
        conEW_minX   = Index;
        conEW_maxX   = -1L;
        conEW_min    = NULL;
        conEW_max    = NULL;
        conEW_colmin = NULL;
        conEW_colmax = NULL;

        while( ptchain )
        {
            ptloconi = (locon_list *)( ptchain->DATA );
            ptchain  = ptchain->NEXT;

            if( ptloconi->TYPE != INTERNAL )
            {
                continue;
            }

            if( isvdd( ptloconi->NAME ) || isvss( ptloconi->NAME ) )
            {
                continue;
            }

            ptcolumn = (column *)getptype( ( (loins_list *)ptloconi->ROOT)->USER, (long)COLUMN_PTYPE )->DATA;
            if( ( ptconNS = getcon_NS( ptcolumn->CON_NS, ptloconi->NAME ) ) != NULL )
            {
                addphcon( ptphfig,
                          ptconNS->TYPE,
                          ptlocon->NAME,
                          ptconNS->POSX + ptcolumn->X,
                          ( ptconNS->TYPE == NORTH ) ? ptplacement->YSLICE[ nbslices ] * SCALE_X : 0L,
                          ALU1,
                          ptconNS->WIDTH );

                ptype = getptype( ptlocon->USER, (long)C_GfpUT_UserLoConPlace );
                if( ptype != NULL )
                {
                    fprintf( stderr, "*** dpp warning *** : unable to move control connector %s\n", ptlocon->NAME );
                }
                found_NS_con = TRUE;
            }
            else if( ( ptconEW = getcon_EW( ptcolumn->CON_EW, ptloconi->NAME ) ) != NULL )
            {
                if( ptcolumn->INDEX <= conEW_minX )
                {
                    conEW_minX   = ptcolumn->INDEX;
                    conEW_colmin = ptcolumn;
                    conEW_min    = ptconEW;
                }
                if( ptcolumn->INDEX >= conEW_maxX )
                {
                    conEW_maxX   = ptcolumn->INDEX;
                    conEW_colmax = ptcolumn;
                    conEW_max    = ptconEW;
                }
            }
            else
            {
                fprintf( stderr, "*** dpp bug *** : i am sorry but you find a bug (make_phfig_placement 2), i can't continue...\n" );
				fprintf( stderr, "Column Model    :\"%s\"\n", ptcolumn->MODEL_NAME);
				fprintf( stderr, "Column Instance :\"%s\"\n", ptcolumn->INSTANCE_NAME);
				fprintf( stderr, "Figure Connector:\"%s\"\n", ptlocon->NAME);
				fprintf( stderr, "Column Connector:\"%s\"\n", ptloconi->NAME);
                exit( 1 );
            }
        }

        if( !found_NS_con )
        {
            if( ( conEW_max == NULL ) || ( conEW_min == NULL ) )
            {
                fprintf( stderr,
                         "*** dpp error *** : Unable to found physical connector for logical connector : %s\n",
                         ptlocon->NAME );
                exit( 1 );
            }

            ptype = getptype( ptlocon->USER, (long)C_GfpUT_UserLoConPlace );
            if( ptype != NULL )
            {
                locon_place = (ulocon_place_list *)( ptype->DATA );

                while( locon_place )
                {
                    if( ( locon_place->SIDE == C_UtFDP_SideTypeNorth )
                      ||( locon_place->SIDE == C_UtFDP_SideTypeSouth ) )
                    {
                        addphcon( ptphfig,
                                  ( locon_place->SIDE == C_UtFDP_SideTypeSouth ) ? SOUTH : NORTH,
                                  ptlocon->NAME,
                                  0L,
                                  ( locon_place->SIDE == C_UtFDP_SideTypeNorth ) ? ptplacement->YSLICE[ nbslices ] * SCALE_X
                                                                                 : 0L,
                                  ALU1,
                                  SCALE_X );
                    }
                    else
                    {
                        if( ( locon_place->SLICE >= ptplacement->NB_SLICES )
                          ||( locon_place->SLICE < 0 ) )
                        {
                            slicei = -1L;
                        }
                        else
                        {
                            slicei = locon_place->SLICE;
                        }
                        if( slicei < 0 )
                        {
                            slicei = YtoSlice( ptplacement, conEW_colmin, conEW_min );
                        }
                        if( ( locon_place->TRACK > ( ( ptplacement->SLICESHEIGHTS[ slicei ] - 15L ) / 5L ) )
                          ||( locon_place->TRACK < 0L ) )
                        {
                            track = -1L;
                        }
                        else
                        {
                            track = locon_place->TRACK;
                        }
                        if( verbose_mode == VERBOSE_ALL )
                        {
                            if ( track == -1 )
                            {
                                fprintf( stdout,
                                         "     Moving connector %s on %s side, slice %ld, default track.\n",
                                         ptlocon->NAME,
                                         ( locon_place->SIDE == C_UtFDP_SideTypeEast ) ? "EAST" : "WEST",
                                         slicei );
                                fflush( stdout );
                            }
                            else
                            {
                                fprintf( stdout,
                                         "     Moving connector %s on %s side, slice %ld, track %ld.\n",
                                         ptlocon->NAME,
                                         ( locon_place->SIDE == C_UtFDP_SideTypeEast ) ? "EAST" : "WEST",
                                         slicei, track );
                                fflush( stdout );
                            }
                        }
                        if( track < 0 )
                        {
                            track = 0L;
                        }
                        else
                        {
                            track = 5L * track + ( ( slicei & 1 ) ? 7L : 8L );
                        }
                        addphcon( ptphfig,
                                  ( locon_place->SIDE == C_UtFDP_SideTypeEast ) ? EAST : WEST,
                                  ptlocon->NAME,
                                  ( locon_place->SIDE == C_UtFDP_SideTypeEast ) ? X * SCALE_X : 0L,
                                  ( ptplacement->YSLICE[ slicei ] + track ) * SCALE_X,
                                  ALU1,
                                  2 * SCALE_X );
                    }
                    locon_place = locon_place->NEXT;
                }
            } else if( conEW_minX < ( Index - conEW_maxX - 1 ) )
            {
                addphcon( ptphfig,
                          WEST,
                          ptlocon->NAME,
                          0L,
                          ptplacement->YSLICE[ YtoSlice( ptplacement, conEW_colmin, conEW_min ) ] * SCALE_X,
                          ALU2,
                          2 * SCALE_X );
            }
            else
            {
                addphcon( ptphfig,
                          EAST,
                          ptlocon->NAME,
                          X * SCALE_X,
                          ptplacement->YSLICE[ YtoSlice( ptplacement, conEW_colmax, conEW_max ) ] * SCALE_X,
                          ALU2,
                          2 * SCALE_X );
            }
        }
    }

    PlCon_MakePlacement( ptphfig->NAME );

    savephfig( ptphfig );

} /* end of make_phfig_placement */

/*---------------------------------------------------------\
                     create_column
\---------------------------------------------------------*/
void create_column( ptplacement )
placement  *ptplacement;
{
    column *ptcolumn;
    slice  *ptslice;
    long    nbslices;

    nbslices = ptplacement->NB_SLICES;

    for( ptcolumn = ptplacement->COLUMNS; ptcolumn; ptcolumn=ptcolumn->NEXT )
    {
        if( ptcolumn->TYPE != COL_GLUE )
            continue;


        if( islogic_column( ptcolumn->MODEL_NAME, nbslices ) )
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout,
                        "     -> Indexed column     : '%s'.\n",
                         ptcolumn->INSTANCE_NAME );
                fflush( stdout );
            }
            create_column_two_level( ptcolumn, ptcolumn->MODEL_NAME );

            for( ptslice = ptcolumn->SLICES; ptslice; ptslice = ptslice->NEXT )
            {
                if( ptslice->INDEX >= nbslices )
                {
                    fprintf( stderr, "*** dpp error *** : logic column have more slice than datapath.\n" );
                    exit( 1 );
                }
            }

            check_column_slice_heights( ptplacement, ptcolumn );
            fill_yslice_array( ptplacement );
            make_cols( ptplacement, ptcolumn ); 
            create_con_list_for_column( ptcolumn );
        }
        else if( nbslices >= nbcells_of_lofig( ptcolumn->MODEL_NAME ) )
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout,
                        "     -> Non indexed column : '%s'.\n",
                         ptcolumn->INSTANCE_NAME );
                fflush( stdout );
            }
        }
    }
    
} /* end of create_column */

/*---------------------------------------------------------\
                     create_glue_logic_bloc
\---------------------------------------------------------*/
void create_glue_logic_bloc( ptplacement, nb_ite, nb_cells )
placement  *ptplacement;
long        nb_ite;
long        nb_cells;
{
    phfig_list   *ptphfig;
    column       *ptcolumn;
    column       *ptcolumnnext;
    PlaceConList *ptplacecon;
    long          nbcolumns;
    long          nbslices;

    nbslices = ptplacement->NB_SLICES;

    for( ptcolumn = ptplacement->COLUMNS; ptcolumn; ptcolumn=ptcolumn->NEXT )
    {
        if( ptcolumn->TYPE != COL_GLUE )
            continue;


		if( verbose_mode == VERBOSE_ALL )
        {
			fprintf( stdout,
					"     -> Logic Bloc  : '%s'.\n",
					 ptcolumn->INSTANCE_NAME );
            fflush( stdout );
        }

        if( !in_already_done( ptcolumn->MODEL_NAME ) )
        {
            model_already_done = addchain( model_already_done, ptcolumn->MODEL_NAME );

            ptplacecon = create_con_for_glue_placing( ptplacement, ptcolumn );
            nbcolumns  = glue_logic_placer( ptcolumn->MODEL_NAME, nbslices, nb_cells, nb_ite, NULL, ptplacecon );

            ptphfig       = getphfig( ptcolumn->MODEL_NAME );
            ptphfig->USER = addptype( ptphfig->USER, WIDTH_OF_GLUE_PTYPE, (void *)nbcolumns );
        }
        else
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout, "        Model already placed.\n" );
                fflush( stdout );
            }

            ptphfig   = getphfig( ptcolumn->MODEL_NAME );
            nbcolumns = (long)( getptype( ptphfig->USER, (long)WIDTH_OF_GLUE_PTYPE)->DATA );
        }

        nbcolumns--;

        for( ptcolumnnext = ptcolumn->NEXT; ptcolumnnext; ptcolumnnext=ptcolumnnext->NEXT )
        {
            ptcolumnnext->INDEX += nbcolumns;
        }

        for( ; nbcolumns != 0; nbcolumns-- )
        {
            ptcolumn->NEXT        = addcolumn( ptcolumn->NEXT, NULL, NULL, NULL, COL_EMPTY );
            ptcolumn->NEXT->INDEX = ptcolumn->INDEX + nbcolumns;
        }

        ptcolumn->SLICES = create_slice_list_for_glue( ptcolumn->MODEL_NAME,
                                                       ptcolumn->INSTANCE_NAME,
                                                       nbslices );

		if( verbose_mode == VERBOSE_ALL )
        {
			fprintf( stdout, "\n" );
            fflush( stdout );
        }

        check_column_slice_heights( ptplacement, ptcolumn );
        fill_yslice_array( ptplacement );
        create_con_list_for_placed_bloc( ptcolumn );
    }
    
} /* end of create_glue_logic_bloc */

/*---------------------------------------------------------\
                     placer
\---------------------------------------------------------*/
void placer( lofig_name, dp_width, nb_ite, nb_cells )
char *lofig_name;
long  dp_width;
long  nb_ite;
long  nb_cells;
{
    lofig_list *ptlofig;
    loins_list *ptloins;
    placement  *ptplacement;
    long        nbslices;
    long        operator_type;

    fprintf( stdout, " * Placement step.\n" );
    if( verbose_mode )
    {
        fprintf( stdout, "   o Getting netlist.\n" );
        fflush( stdout );
    }
    ptlofig = getlofig( lofig_name, 'A' );
    lofigchain( ptlofig );

    if( verbose_mode )
    {
        fprintf( stdout, "   o Getting dpr file.\n" );
        fflush( stdout );
    }
    PlCon_Init();
    UtFDP_LoadPlacement( lofig_name );

    ptlofig = getlofig( lofig_name, 'A' );
    if( verbose_mode )
    {
        fprintf( stdout, "   o Building symbolic grid.\n" );
        fflush( stdout );
    }

    ptplacement = addplacement( lofig_name );

    for( ptloins = ptlofig->LOINS; ptloins; ptloins = ptloins->NEXT )
    {
        operator_type = UtHir_GetModelType( ptloins->FIGNAME );

        if( operator_type == C_UtHir_TypeOperatorRouted )
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout,
                         "     -> Routed bloc : '%s' (%s).\n",
                         ptloins->INSNAME, ptloins->FIGNAME );
                fflush( stdout );
            }

            ptplacement->COLUMNS = create_column_one_level(
                                          ptplacement->COLUMNS,
                                          ptloins->FIGNAME,
                                          ptloins->INSNAME,
                                          FALSE );

            create_con_list_for_routed_bloc( ptplacement->COLUMNS );
        }
        else if( operator_type == C_UtHir_TypeOperatorPlaced )
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout,
                        "     -> Placed bloc : '%s' (%s).\n",
                         ptloins->INSNAME, ptloins->FIGNAME );
                fflush( stdout );
            }

            ptplacement->COLUMNS = create_column_one_level(
                                          ptplacement->COLUMNS,
                                          ptloins->FIGNAME,
                                          ptloins->INSNAME,
                                          TRUE );
            create_con_list_for_placed_bloc( ptplacement->COLUMNS );
        }
        else if( operator_type == C_UtHir_TypeOperatorUser )
        {
            if( verbose_mode == VERBOSE_ALL )
            {
                fprintf( stdout,
                         "     -> Glue bloc   : '%s' (%s).\n",
                         ptloins->INSNAME, ptloins->FIGNAME );
                fflush( stdout );
            }

            ptplacement->COLUMNS = addcolumn( ptplacement->COLUMNS,
                                              NULL,
                                              ptloins->FIGNAME,
                                              ptloins->INSNAME,
                                              COL_GLUE );
        }
        else
        {
            fprintf( stderr,
                     "*** dpp error *** : two level hierarchy not respected in %s for instance %s(%s)\n",
                     ptlofig->NAME, ptloins->INSNAME, ptloins->FIGNAME );
            exit( 1 );
        }

        ptloins->USER = addptype( ptloins->USER, COLUMN_PTYPE, (void *)ptplacement->COLUMNS );
		ptplacement->COLUMNS->LOINS = ptloins;
    }

    nbslices = found_nb_slices( ptplacement );

    if( ( nbslices == 0 ) && ( dp_width == 0 ) )
    {
        fprintf( stderr,
                 "*** dpp error *** : unable to found DP width, please use the -w option.\n" );
        exit( 1 );
    }

    ptplacement->NB_SLICES = (nbslices > dp_width ) ? nbslices : dp_width;

    if( verbose_mode )
    {
        fprintf( stdout, "   o Ckecking slices heights.\n" );
        fflush( stdout );
    }

    make_slices_array( ptplacement );
    check_slices_heights( ptplacement ); /* and fill col width field */
    fill_yslice_array( ptplacement );

    if( verbose_mode )
    {
        fprintf( stdout, "\n" );
        fprintf( stdout, "   o Nb slices : %d.\n", (int)ptplacement->NB_SLICES );
        fprintf( stdout, "   o Nb cols   : %d.\n", (int)found_nb_cols( ptplacement ) );
        fprintf( stdout, "\n" );
        fprintf( stdout, "   o Creating column.\n" );
        fflush( stdout );
    }
    create_column( ptplacement );

    if( verbose_mode )
    {
        fprintf( stdout, "   o Creating glue logic blocs.\n" );
        fflush( stdout );
    }
    create_glue_logic_bloc( ptplacement, nb_ite, nb_cells );

    if( verbose_mode )
    {
        fprintf( stdout, "   o Checking alims correspondances.\n" );
        fflush( stdout );
    }
    if( debug_mode )
    {
        display_placement( ptplacement );
    }
    check_alims( ptplacement );

    if( verbose_mode )
    {
        fprintf( stdout, "   o Creating global phfig view.\n" );
        fflush( stdout );
    }
    make_phfig_placement( ptplacement, ptlofig );

    if( debug_mode )
    {
        display_placement( ptplacement );
    }

    if( verbose_mode )
    {
        fprintf( stdout, "   o Freeing symbolic grid.\n" );
        fflush( stdout );
    }
    freeplacement( ptplacement );
    freechain( model_already_done );
    model_already_done = NULL;

/* penser a liberer les lofigch, ptlofigchain ET les ptypes columns */

    fprintf( stdout, " * Placement done.\n" );
    fflush( stdout );

    return;
} /* end of placer */
