/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                  Setup.c                          |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <Xm/Xm.h>
# include <Xm/PushBG.h>

# include MUT_H
# include MPH_H
# include RDS_H
# include RPR_H
# include GTB_H
# include GSB_H
# include GMX_H
# include GMF_H
# include GMV_H
# include GMT_H
# include GMS_H
# include GMH_H

# include "GMS_setup.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/
  
  static FILE *FileConfig;

/*------------------------------------------------------------\
|                                                             |
|                          Functions                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      Set Panel New Values                   |
|                                                             |
\------------------------------------------------------------*/

void DrealSetPanelValues( Panel, Values )

  DrealPanelItem *Panel;
  int            *Values;
{
  rdsbegin();

  Panel->COMPUTE = 1;
  Panel->X       = Values[0];
  Panel->Y       = Values[1];
  Panel->WIDTH   = Values[2];
  Panel->HEIGHT  = Values[3];
  Panel->MANAGED = Values[4];

  XtVaSetValues( Panel->PANEL,
                 XmNx        , Values[0],
                 XmNy        , Values[1],
                 XmNwidth    , Values[2],
                 XmNheight   , Values[3],
                 NULL );

  if ( Panel->MANAGED )
  {
    DrealEnterPanel( Panel );
  }
  else
  {
    XtUnmanageChild( Panel->PANEL );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      Write TopLevel Values                  |
|                                                             |
\------------------------------------------------------------*/

void DrealWriteTopLevelValues()

{
  Dimension Values[5];

  rdsbegin();

  XtVaGetValues( DrealTopLevel,
                 XmNx        , &Values[0],
                 XmNy        , &Values[1],
                 XmNwidth    , &Values[2],
                 XmNheight   , &Values[3],
                 NULL );

  Values[0] = Values[0] - DREAL_TOPLEVEL_TRANSLATE_X;
  Values[1] = Values[1] - DREAL_TOPLEVEL_TRANSLATE_Y;
  Values[4] = 1;
  
  fprintf( FileConfig, "VERSION: %s\n", DREAL_VERSION );
  fprintf( FileConfig, "X: %d, Y: %d, WIDTH: %d, HEIGHT: %d, MANAGED: %d\n",
           Values[0], Values[1], Values[2], Values[3], Values[4] );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       Write Panel Values                    |
|                                                             |
\------------------------------------------------------------*/

void DrealWritePanelValues( Panel )

  DrealPanelItem *Panel;
{
  Dimension Values[5];

  rdsbegin();

  if ( Panel->COMPUTE == 0 )
  {
    XtVaGetValues( Panel->PANEL,
                   XmNx        , &Values[0],
                   XmNy        , &Values[1],
                   XmNwidth    , &Values[2],
                   XmNheight   , &Values[3],
                   NULL );

    Values[0] = Values[0] - DREAL_PANEL_TRANSLATE_X;
    Values[1] = Values[1] - DREAL_PANEL_TRANSLATE_Y;
  }
  else
  {
    Values[0] = Panel->X;
    Values[1] = Panel->Y;
    Values[2] = Panel->WIDTH;
    Values[3] = Panel->HEIGHT;
  }

  Values[4] = Panel->MANAGED;

  fprintf( FileConfig, "X: %d, Y: %d, WIDTH: %d, HEIGHT: %d, MANAGED: %d\n",
           Values[0], Values[1], Values[2], Values[3], Values[4] );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        Read Panel Values                    |
|                                                             |
\------------------------------------------------------------*/

void DrealReadPanelValues( Panel )

  DrealPanelItem *Panel;
{
  int Values[5];

  rdsbegin();

  fscanf( FileConfig, "X: %d, Y: %d, WIDTH: %d, HEIGHT: %d, MANAGED: %d\n",
          &Values[0], &Values[1], &Values[2], &Values[3], &Values[4] );

  DrealSetPanelValues( Panel, Values );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       Read  Panel Values                    |
|                                                             |
\------------------------------------------------------------*/

char DrealReadTopLevelValues()
{
  int Values[5];
  char Version[64];

  rdsbegin();

  fscanf( FileConfig, "VERSION: %s\n", Version );

  if ( strcmp( Version, DREAL_VERSION ) )
  {
    rdsend();
    return( 0 );
  }

  fscanf( FileConfig, "X: %d, Y: %d, WIDTH: %d, HEIGHT: %d, MANAGED: %d\n",
          &Values[0], &Values[1], &Values[2], &Values[3], &Values[4] );

  XtVaSetValues( DrealTopLevel,
                 XmNx        , Values[0],
                 XmNy        , Values[1],
                 XmNwidth    , Values[2],
                 XmNheight   , Values[3],
                 NULL );

  rdsend();
  return( 1 );
}

/*------------------------------------------------------------\
|                                                             |
|                         ReadActiveLayers                    |
|                                                             |
\------------------------------------------------------------*/

void DrealReadActiveLayers()

{
  char Layer;
  int  Value;

  rdsbegin();

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    fscanf( FileConfig, "ACTIVE: %d\n", &Value );

    DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] = Value;
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    fscanf( FileConfig, "ACTIVE: %d\n", &Value );

    DREAL_RDS_ACTIVE_NAME_TABLE[ Layer ] = Value;
  }

  fscanf( FileConfig, "FILLMODE: %d\n", &Value );

  DREAL_FILL_MODE = Value;

  fscanf( FileConfig, "INTERFACE: %d\n", &Value );

  DREAL_INSTANCE_INTERFACE = Value;

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        WriteActiveLayers                    |
|                                                             |
\------------------------------------------------------------*/

void DrealWriteActiveLayers()

{
  char Layer;
  int  Value;

  rdsbegin();

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    fprintf( FileConfig, "ACTIVE: %d\n", 
             DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] );
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    fprintf( FileConfig, "ACTIVE: %d\n", 
             DREAL_RDS_ACTIVE_NAME_TABLE[ Layer ] );
  }

  fprintf( FileConfig, "FILLMODE: %d\n",
           DREAL_FILL_MODE );

  fprintf( FileConfig, "INTERFACE: %d\n",
           DREAL_INSTANCE_INTERFACE );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                   DrealDefaultTopLevelValues                |
|                                                             |
\------------------------------------------------------------*/

void DrealDefaultTopLevelValues()

{
  rdsbegin();

  XtVaSetValues( DrealTopLevel,
                 XmNheight   , DREAL_TOPLEVEL_HEIGHT,
                 XmNwidth    , DREAL_TOPLEVEL_WIDTH,
                 XmNx        , DREAL_TOPLEVEL_X,
                 XmNy        , DREAL_TOPLEVEL_Y,
                 NULL
               );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        DrealDefaultConfig                   |
|                                                             |
\------------------------------------------------------------*/

void DrealDefaultConfig()

{
  char      Layer;

  rdsbegin();

  DrealDefaultTopLevelValues();

  DrealSetPanelValues( &DrealViewArrowsPanel, 
                        DrealViewArrowsDefaultValues );
  DrealSetPanelValues( &DrealViewZoomPanel, 
                        DrealViewZoomDefaultValues );
  DrealSetPanelValues( &DrealViewLayerPanel, 
                        DrealViewLayerDefaultValues );
  DrealSetPanelValues( &DrealViewGridPanel, 
                        DrealViewGridDefaultValues );
  DrealSetPanelValues( &DrealViewMapPanel, 
                        DrealViewMapDefaultValues );
  DrealSetPanelValues( &DrealToolsMessagePanel,
                        DrealToolsMessageDefaultValues );
  DrealSetPanelValues( &DrealHelpPresentPanel,
                        DrealHelpPresentDefaultValues );

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] = 1;
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    DREAL_RDS_ACTIVE_NAME_TABLE[ Layer ] = 1;
  }

  DREAL_FILL_MODE = DREAL_FILL_MODE_PATTERN;
  DREAL_INSTANCE_INTERFACE = DREAL_TRUE;

  DrealInitializeLayer();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                   DrealLoadTopLevelConfig                   |
|                                                             |
\------------------------------------------------------------*/

void DrealLoadTopLevelConfig()

{
  rdsbegin();

  FileConfig = fopen( DREAL_GMS_FILE_NAME, "r" );

  if ( FileConfig == (FILE *)NULL )
  {
    DrealDefaultTopLevelValues();
  }
  else
  {
    if ( ! DrealReadTopLevelValues() )
    {
      DrealDefaultTopLevelValues();
    }

    fclose( FileConfig );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                          DrealLoadConfig                    |
|                                                             |
\------------------------------------------------------------*/

void DrealLoadConfig( Message )

  char Message;
{
  rdsbegin();

  FileConfig = fopen( DREAL_GMS_FILE_NAME, "r" );

  if ( FileConfig == (FILE *)NULL )
  {
    if ( Message )
    {
      DrealErrorMessage( DrealMainWindow, 
                         "Unable to open config file !" );
    }
    else
    {
      DrealDefaultConfig();
    }
  }
  else
  {
    if ( DrealReadTopLevelValues() )
    {
      DrealReadPanelValues( &DrealViewArrowsPanel       );
      DrealReadPanelValues( &DrealViewZoomPanel         );
      DrealReadPanelValues( &DrealViewLayerPanel        );
      DrealReadPanelValues( &DrealViewGridPanel         );
      DrealReadPanelValues( &DrealViewMapPanel          );
      DrealReadPanelValues( &DrealToolsMessagePanel     );
      DrealReadPanelValues( &DrealHelpPresentPanel      );

      DrealReadActiveLayers();
      DrealInitializeLayer();
    }
    else
    if ( Message )
    {
      DrealErrorMessage( DrealMainWindow,
                         "Bad version, unable to open config file !" );
    }

    fclose( FileConfig );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                          DrealSaveConfig                    |
|                                                             |
\------------------------------------------------------------*/

void DrealSaveConfig()

{
  rdsbegin();

  FileConfig = fopen( DREAL_GMS_FILE_NAME, "w" );

  if ( FileConfig == (FILE *)NULL )
  {
    DrealErrorMessage( DrealMainWindow,
                       "Unable to open config file !" );
  }
  else
  {
    DrealWriteTopLevelValues();

    DrealWritePanelValues( &DrealViewArrowsPanel       );
    DrealWritePanelValues( &DrealViewZoomPanel         );
    DrealWritePanelValues( &DrealViewLayerPanel        );
    DrealWritePanelValues( &DrealViewGridPanel         );
    DrealWritePanelValues( &DrealViewMapPanel          );
    DrealWritePanelValues( &DrealToolsMessagePanel     );
    DrealWritePanelValues( &DrealHelpPresentPanel      );

    DrealWriteActiveLayers();

    fclose( FileConfig );
  }

  rdsend();
}
