/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                   Event.c                         |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/

/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <X11/Intrinsic.h>
# include <X11/StringDefs.h>
# include <Xm/Xm.h>

 
# include MUT_H
# include MPH_H
# include RDS_H
# include RPR_H
# include GSB_H
# include GRD_H
# include GMX_H
# include GTB_H
# include GMV_H
# include GMS_H
# include GMH_H
# include "GMX_motif.h"
# include "GMX_grid.h"
# include "GMX_cursor.h"
# include "GMX_event.h"
# include "GMX_message.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Translation                        |
|                                                             |
\------------------------------------------------------------*/

  static String DrealEventTranslation =

    "<Btn1Down>:    CallbackEvent( 0 )\n\
     <Btn2Down>:    CallbackEvent( 1 )\n\
     <Btn3Down>:    CallbackEvent( 2 )\n\
     <Motion>:      CallbackEvent( 3 )\n\
     <EnterWindow>: CallbackEvent( 4 )\n\
     <LeaveWindow>: CallbackEvent( 5 )";

/*------------------------------------------------------------\
|                                                             |
|                         Event Count                         |
|                                                             |
\------------------------------------------------------------*/

  static char DrealMaxEventEditTable[] =

  {
    2   /* DREAL_EDIT_MEASURE           */
  };

  static char DrealInputEventEditTable[] =

  {
    DREAL_INPUT_HALF_BOX  /* DREAL_EDIT_MEASURE           */
  };

  static char DrealMaxEventZoomTable[] =

  {
    1,  /* DREAL_ZOOM_CENTER            */
    2,  /* DREAL_ZOOM_IN                */
    2   /* DREAL_ZOOM_PAN               */
  };

  static char DrealInputEventZoomTable[] =

  {
    DREAL_INPUT_POINT,  /* DREAL_ZOOM_CENTER            */
    DREAL_INPUT_BOX,    /* DREAL_ZOOM_IN                */
    DREAL_INPUT_LINE    /* DREAL_ZOOM_PAN               */
  };

/*------------------------------------------------------------\
|                                                             |
|                          Translation                        |
|                                                             |
\------------------------------------------------------------*/

/*------------------------------------------------------------\
|                                                             |
|                      Expose Edit Zoom                       |
|                                                             |
\------------------------------------------------------------*/

  static long     DrealLambdaEventZoomX[ 2 ] = { 0, 0 };
  static long     DrealLambdaEventZoomY[ 2 ] = { 0, 0 };
  static char     DrealMaxEventZoom          = 0;
         char     DrealCountEventZoom        = 0;

  static long     DrealLambdaEventEditX[ 2 ] = { 0, 0 };
  static long     DrealLambdaEventEditY[ 2 ] = { 0, 0 };
  static char     DrealMaxEventEdit          = 2;
         char     DrealCountEventEdit        = 0;

  static char     DrealFirstExpose           = DREAL_TRUE;

/*------------------------------------------------------------\
|                                                             |
|                          Input                              |
|                                                             |
\------------------------------------------------------------*/

 static char DrealInputMode     = DREAL_INPUT_HALF_BOX;
 static char DrealSaveInputMode = DREAL_INPUT_HALF_BOX;
 static char DrealEditMode      = DREAL_EDIT_MEASURE;
 static char DrealSaveEditMode  = DREAL_EDIT_MEASURE;

/*------------------------------------------------------------\
|                                                             |
|                          Edit Message                       |
|                                                             |
\------------------------------------------------------------*/

  static void (*DrealPromptMessage)()     = DrealPromptEditMeasure;
  static void (*DrealSavePromptMessage)() = DrealPromptEditMeasure;

/*------------------------------------------------------------\
|                                                             |
|                           Functions                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                     DrealChangeEditMode                     |
|                                                             |
\------------------------------------------------------------*/
 
void DrealChangeEditMode( EditMode , PromptMessage )
 
   char  EditMode;
   void (*PromptMessage)();
{
  rdsbegin();

  if ( EditMode & DREAL_ZOOM_MARK )
  {
    EditMode = EditMode & ~DREAL_ZOOM_MARK;

    DrealCountEventZoom = 0;
    DrealMaxEventZoom   = DrealMaxEventZoomTable[ EditMode ];

    if ( ! ( DrealEditMode & DREAL_ZOOM_MARK ) )
    {
      DrealSaveEditMode      = DrealEditMode;
      DrealSaveInputMode     = DrealInputMode;
      DrealSavePromptMessage = DrealPromptMessage;
    }

    DrealInputMode     = DrealInputEventZoomTable[ EditMode ];
    DrealPromptMessage = PromptMessage;
    DrealEditMode      = EditMode | DREAL_ZOOM_MARK;

    DrealChangeCursorType( NULL, NULL, 0, DrealInputMode );
  }  
  else
  {
    if ( DrealEditMode & DREAL_ZOOM_MARK )
    {
      DrealSaveEditMode      = EditMode;
      DrealSaveInputMode     = DrealInputEventEditTable[ EditMode ];
      DrealSavePromptMessage = PromptMessage;
      DrealCountEventEdit    = 0;
      DrealMaxEventEdit      = DrealMaxEventEditTable[ EditMode ];
    }
    else
    {
      if ( DrealEditMode != EditMode )
      {
        DrealCountEventEdit = 0;
        DrealMaxEventEdit   = DrealMaxEventEditTable[ EditMode ];
        DrealEditMode       = EditMode;
        DrealInputMode      = DrealInputEventEditTable[ EditMode ];
        DrealPromptMessage  = PromptMessage;

        DrealChangeCursorType( NULL, NULL, 0, DrealInputMode );
      }
    }
  }

  DrealPromptMessage();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                   DrealContinueEditMode                     |
|                                                             |
\------------------------------------------------------------*/

void DrealContinueEditMode( EditMode , PromptMessage, CountEvent )

   char  EditMode;
   void (*PromptMessage)();
   char  CountEvent;
{
  rdsbegin();

  if ( DrealEditMode & DREAL_ZOOM_MARK )
  {
    DrealSaveEditMode      = EditMode;
    DrealSaveInputMode     = DrealInputEventEditTable[ EditMode ];
    DrealSavePromptMessage = PromptMessage;
    DrealCountEventEdit    = CountEvent;
    DrealMaxEventEdit      = DrealMaxEventEditTable[ EditMode ];
  }
  else
  {
    DrealCountEventEdit = CountEvent;
    DrealMaxEventEdit   = DrealMaxEventEditTable[ EditMode ];
    DrealEditMode       = EditMode;
    DrealInputMode      = DrealInputEventEditTable[ EditMode ];
    DrealPromptMessage  = PromptMessage;

    DrealChangeCursorType( DrealLambdaEventEditX,
                           DrealLambdaEventEditY,
                           DrealCountEventEdit,
                           DrealInputMode );

  }

  DrealPromptMessage();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealTreatEventZoom                    |
|                                                             |
\------------------------------------------------------------*/

void DrealTreatEventZoom()

{
  rdsbegin();

  DrealChangeCursorType( DrealLambdaEventEditX,
                         DrealLambdaEventEditY,
                         DrealCountEventEdit,
                         DrealSaveInputMode );

  switch ( DrealEditMode )
  {
    case DREAL_ZOOM_CENTER :
                              
      DrealZoomCenter( DrealLambdaEventZoomX[ 0 ], 
                       DrealLambdaEventZoomY[ 0 ] );
      break;

    case DREAL_ZOOM_IN     :

      DrealZoomIn( DrealLambdaEventZoomX[ 0 ],
                   DrealLambdaEventZoomY[ 0 ],
                   DrealLambdaEventZoomX[ 1 ],
                   DrealLambdaEventZoomY[ 1 ] );
      break;

    case DREAL_ZOOM_PAN    :

      DrealZoomPan( DrealLambdaEventZoomX[ 0 ],
                    DrealLambdaEventZoomY[ 0 ],
                    DrealLambdaEventZoomX[ 1 ],
                    DrealLambdaEventZoomY[ 1 ] );
      break;
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealTreatEventEdit                    |
|                                                             |
\------------------------------------------------------------*/

void DrealTreatEventEdit()

{ 
  long DeltaX;
  long DeltaY;
  long X1;
  long X2;
  long Y1;
  long Y2;

  rdsend();

  X1 = DrealLambdaEventEditX[0];
  X2 = DrealLambdaEventEditX[1];
  Y1 = DrealLambdaEventEditY[0];
  Y2 = DrealLambdaEventEditY[1];

  if ( ( DrealInputMode == DREAL_INPUT_LSTRING ) || 
       ( DrealInputMode == DREAL_INPUT_ORTHO   ) ||
       ( DrealInputMode == DREAL_INPUT_SORTHO  ) )
  {
    DeltaX = X1 - X2;
    DeltaY = Y1 - Y2;

    if ( DeltaX < 0 ) DeltaX = - DeltaX;
    if ( DeltaY < 0 ) DeltaY = - DeltaY;

    if ( DeltaX > DeltaY )
    {
      Y2 = Y1;

      DrealLambdaEventEditX[0] = X2;
      DrealLambdaEventEditY[0] = Y2;
    }  
    else
    {   
      X2 = X1;

      DrealLambdaEventEditX[0] = X2;
      DrealLambdaEventEditY[0] = Y2;
    }
  }

  if ( DrealInputMode == DREAL_INPUT_LSTRING )
  {   
    DrealChangeCursorType( DrealLambdaEventEditX,
                           DrealLambdaEventEditY,
                           1, DrealInputMode );
 
    DrealCountEventEdit = 1;
  }
  else
  {
    DrealChangeCursorType( NULL, NULL, 0,
                           DrealInputMode );

    DrealCountEventEdit = 0;
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackEvent                         |
|                                                             |
\------------------------------------------------------------*/

void CallbackEvent ( MyWidget, Event, Args, Argc )

     Widget        MyWidget;
     XButtonEvent *Event;
     String       *Args;
     int          *Argc;
{
  Display  *EventDisplay;
  char      MouseEvent;
  Position  OldLambdaCursorX;
  Position  OldLambdaCursorY;

  rdsbegin();

  EventDisplay = Event->display;

  MouseEvent = Args[ 0 ][ 0 ] - '0';

  if ( MouseEvent == DREAL_LEAVE )
  {
    DrealUndisplayCursor();
    DrealCursorInside = DREAL_FALSE;
  }
  else
  {
    OldLambdaCursorX = DrealLambdaCursorX;
    OldLambdaCursorY = DrealLambdaCursorY;

    DrealComputeCursor( Event->x, Event->y );

    switch ( MouseEvent )
    {
      case DREAL_B1UP   : 

        if ( DrealEditMode & DREAL_ZOOM_MARK )
        {
          if ( DrealCountEventZoom < DrealMaxEventZoom )
          {
            DrealLambdaEventZoomX[ DrealCountEventZoom ] = DrealLambdaCursorX;
            DrealLambdaEventZoomY[ DrealCountEventZoom ] = DrealLambdaCursorY;

            DrealCountEventZoom = DrealCountEventZoom + 1;

            DrealPointCursor();
          
            if ( DrealCountEventZoom == DrealMaxEventZoom )
            {
              DrealTreatEventZoom();

              DrealInputMode      = DrealSaveInputMode;
              DrealEditMode       = DrealSaveEditMode;
              DrealPromptMessage  = DrealSavePromptMessage;
              DrealSaveEditMode   = DREAL_EDIT_MEASURE;
              DrealSaveInputMode  = DREAL_INPUT_HALF_BOX;
              DrealCountEventZoom = 0;
              DrealMaxEventZoom   = 0;

              DrealComputeCursor( Event->x, Event->y );
              DrealDisplayCursor();
            }
          }
        }
        else
        {
          if ( DrealCountEventEdit < DrealMaxEventEdit )
          {
            DrealLambdaEventEditX[ DrealCountEventEdit ] = DrealLambdaCursorX;
            DrealLambdaEventEditY[ DrealCountEventEdit ] = DrealLambdaCursorY;

            DrealCountEventEdit = DrealCountEventEdit + 1;

            DrealPointCursor();

            if ( DrealCountEventEdit == DrealMaxEventEdit )
            {
              DrealTreatEventEdit();

              DrealDisplayCursor();
            }
          }
        }

        DrealPromptMessage();

        break;

      case DREAL_B2UP   :

        break;

      case DREAL_B3UP   :

        if ( DrealEditMode & DREAL_ZOOM_MARK )
        {
          if ( DrealCountEventZoom != 0 )
          {
            DrealCountEventZoom = DrealCountEventZoom - 1;

            DrealResetCursor();
          }
          else
          { 
            DrealChangeCursorType( DrealLambdaEventEditX,
                                   DrealLambdaEventEditY,
                                   DrealCountEventEdit,
                                   DrealSaveInputMode );

            DrealInputMode          = DrealSaveInputMode;
            DrealEditMode           = DrealSaveEditMode;
            DrealPromptMessage      = DrealSavePromptMessage;
            DrealSaveEditMode       = DREAL_EDIT_MEASURE;
            DrealSaveInputMode      = DREAL_INPUT_HALF_BOX;
            DrealSavePromptMessage  = DrealPromptEditMeasure;
            DrealCountEventZoom     = 0;
            DrealMaxEventZoom       = 0;
          }
        }
        else
        {
          if ( DrealCountEventEdit != 0 )
          {
            DrealCountEventEdit = DrealCountEventEdit - 1;

            DrealResetCursor();
          }
          else
          {
            DrealChangeCursorType( NULL, NULL, 0,
                                   DREAL_INPUT_HALF_BOX );

            DrealInputMode      = DREAL_INPUT_HALF_BOX;
            DrealEditMode       = DREAL_EDIT_MEASURE;
            DrealPromptMessage  = DrealPromptEditMeasure;
            DrealCountEventEdit = 0;
            DrealMaxEventEdit   = 2;
          }
        }

        DrealPromptMessage();

        break; 

      case DREAL_MOTION :

        break;

      case DREAL_ENTER  : DrealCursorInside = DREAL_TRUE;
                          DrealDisplayCursor();
                          break;
    }

    if ( ( OldLambdaCursorX != DrealLambdaCursorX ) ||
         ( OldLambdaCursorY != DrealLambdaCursorY ) )
    {
      DrealDisplayCoordinates(); 
      DrealDisplayCursor();
    }
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackResize                        |
|                                                             |
\------------------------------------------------------------*/

void CallbackResize( MyWidget, ClientData, CallData )

     Widget                       MyWidget;
     XtPointer                    ClientData;
     XmDrawingAreaCallbackStruct *CallData;
{
  rdsbegin();

  if ( DrealFirstExpose == DREAL_FALSE )
  {
    DrealSetMouseCursor( DrealGraphicWindow, DREAL_WATCH_CURSOR );
    DrealUndisplayCursor();

    DrealResizeGraphicWindow();
    DrealRefreshGraphicWindow( 0, 0,
                               DrealGraphicDx, DrealGraphicDy );
    DrealSetMouseCursor( DrealGraphicWindow, DREAL_NORMAL_CURSOR );
    DrealDisplayCursor();
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackExpose                        |
|                                                             |
\------------------------------------------------------------*/

void CallbackExpose( MyWidget, ClientData, CallData )

     Widget                       MyWidget;
     XtPointer                    ClientData;
     XmDrawingAreaCallbackStruct *CallData;
{
  XExposeEvent *ExposeEvent;

  rdsbegin();

  ExposeEvent = (XExposeEvent *)CallData->event;

  if ( DrealFirstExpose == DREAL_TRUE )
  {
    DrealFirstExpose = DREAL_FALSE;

    DrealInitializeGraphicWindow();
    DrealUndisplayCursor();

    DrealLoadConfig( DREAL_FALSE );

    if ( DrealFigureRds != (rdsfig_list *)NULL )
    {
      DrealSetMouseCursor( DrealGraphicWindow, DREAL_WATCH_CURSOR );

      DrealChangeTopLevelTitle( DrealFigureRds->NAME );
      DrealInitializeZoom();
      DrealDisplayToolsMessage();
      DrealDisplayFigure( 0, 0, DrealGraphicDx, DrealGraphicDy );

      DrealSetMouseCursor( DrealGraphicWindow, DREAL_NORMAL_CURSOR );
    }
    else
    {
      DrealChangeTopLevelTitle( (char *)NULL );
    }

    DrealRefreshGraphicWindow( 0, 0, DrealGraphicDx, DrealGraphicDy );
    DrealDisplayCursor();

    DrealEnterPresentPanel();
  }
  else
  {
    DrealUndisplayCursor();
    DrealRefreshGraphicWindow( ExposeEvent->x, ExposeEvent->y,
                               ExposeEvent->width, ExposeEvent->height );
    DrealDisplayCursor();
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     DrealInitializeEvent                    |
|                                                             |
\------------------------------------------------------------*/

void DrealInitializeEvent()

{
  XtActionsRec NewActions;

  rdsbegin();

  XtVaSetValues( DrealGraphicWindow,
                 XmNtranslations,
                 XtParseTranslationTable( DrealEventTranslation ),
                 NULL
               );

  NewActions.string = "CallbackEvent";
  NewActions.proc   =  CallbackEvent;

  XtAppAddActions( DrealApplication, &NewActions, 1 );

  XtAddCallback( DrealGraphicWindow, 
                 XmNresizeCallback, 
                 CallbackResize, NULL );

  XtAddCallback( DrealGraphicWindow, 
                 XmNexposeCallback, 
                 CallbackExpose, NULL );

  rdsend();
}
