/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                   GRD101.h                        |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/

# ifndef DREAL_GRD
# define DREAL_GRD 

# define DREAL_MODE (sizeof( drealrec ))

# define DREAL_MAX_REC       8
# define DREAL_ONE_WINDOW    (long)0x00010000
# define DREAL_DRAW_TEXT     (long)0x00020000

/*------------------------------------------------------------\
|                                                             |
|                            Macros                           |
|                                                             |
\------------------------------------------------------------*/

# define DREAL_WINDOW( R )                                    \
                                                              \
    (((drealrec *)((char *)(R)+sizeof(rdsrec_list)))->WINDOW  )

/*------------------------------------------------------------\
|                                                             |
|                            Macros                           |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Windows                          |
|                                                             |
\------------------------------------------------------------*/
 
# define IsDrealOneWindow( Rectangle )         \
                                               \
   ( ( Rectangle )->FLAGS & DREAL_ONE_WINDOW   )

# define SetDrealOneWindow( Rectangle )        \
                                               \
   ( ( Rectangle )->FLAGS |= DREAL_ONE_WINDOW  )
 
# define ClearDrealOneWindow( Rectangle )      \
                                               \
   ( ( Rectangle )->FLAGS &= ~DREAL_ONE_WINDOW )

/*------------------------------------------------------------\
|                                                             |
|                           DrawText                          |
|                                                             |
\------------------------------------------------------------*/

# define IsDrealDrawText( Rectangle )         \
                                              \
   ( ( Rectangle )->FLAGS & DREAL_DRAW_TEXT   )

# define SetDrealDrawText( Rectangle )        \
                                              \
   ( ( Rectangle )->FLAGS |= DREAL_DRAW_TEXT  )

# define ClearDrealDrawText( Rectangle )      \
                                              \
   ( ( Rectangle )->FLAGS &= ~DREAL_DRAW_TEXT )

/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                       Dreal Windows                         |
|                                                             |
\------------------------------------------------------------*/

  typedef struct drealwinrec   

  {
    struct drealwinrec *NEXT; 
    rdsrec_list        *RECTAB[ DREAL_MAX_REC ];

  } drealwinrec;

  typedef struct drealwin

  {
    struct drealwinrec *LAYERTAB[ RDS_MAX_LAYER ];

  } drealwin;

  typedef struct drealrecwin 

  {
    struct drealrecwin *NEXT;
    drealwin           *WINDOW;
    
  } drealrecwin;

/*------------------------------------------------------------\
|                                                             |
|                         X Rectangle                         |
|                                                             |
\------------------------------------------------------------*/

  typedef struct drealrec

  {
    drealrecwin  *WINDOW;

  } drealrec;

/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/

  extern char         DrealComputedBound;
  extern long         DrealBoundXmin;
  extern long         DrealBoundYmin;
  extern long         DrealBoundXmax;
  extern long         DrealBoundYmax; 
  extern long         DrealWindowXmin;
  extern long         DrealWindowYmin;
  extern long         DrealWindowXmax;
  extern long         DrealWindowYmax;
  extern long         DrealWindowDx;
  extern long         DrealWindowDy;
  extern long         DrealWindowSide;
  extern long         DrealWindowSize;
  extern drealwin   **DrealWindowTable;
  extern rdsfig_list *DrealFigureRds;

/*------------------------------------------------------------\
|                                                             |
|                          Functions                          |
|                                                             |
\------------------------------------------------------------*/

  extern rdsfig_list *DrealAddFigure();
  extern void         DrealDelFigure();
  extern void         DrealFlattenFigure();

/*------------------------------------------------------------\
|                                                             |
|                    Window Acces Functions                   |
|                                                             |
\------------------------------------------------------------*/

  extern void DrealInitializeWindowTable();

/*------------------------------------------------------------\
|                                                             |
|                    Error Message Functions                  |
|                                                             |
\------------------------------------------------------------*/

  extern char *DrealGetErrorMessage();

/*------------------------------------------------------------\
|                                                             |
|                    Informations Functions                   |
|                                                             |
\------------------------------------------------------------*/

  extern char *DrealGetInformations();

# endif
