/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                  Error.c                          |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <stdlib.h>
# include <signal.h>
# include <setjmp.h>
# include MUT_H
# include MPH_H
# include RDS_H
# include RPR_H
# include RFM_H
# include RTL_H
# include GSB_H 
# include GRD_H
# include "GRD_error.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/

  static FILE   *DrealStreamErr;
  static FILE   *DrealStreamOut;
  static FILE   *DrealStreamAll;

  static char    DrealErrFileName[ 40 ];
  static char    DrealOutFileName[ 40 ];
  static char    DrealAllFileName[ 40 ];

  static char    DrealNormalMode = 1;

/*------------------------------------------------------------\
|                                                             |
|                      Dreal Error Message                    |
|                                                             |
\------------------------------------------------------------*/

  static char *DrealErrorBuffer = (char *)NULL;

/*------------------------------------------------------------\
|                                                             |
|                      Dreal Informations                     |
|                                                             |
\------------------------------------------------------------*/

  static char DrealInformationsBuffer[ DREAL_INFORMATIONS_BUFFER_SIZE ];

/*------------------------------------------------------------\
|                                                             |
|                           Functions                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                 Dreal Error Message Functions               |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                   DrealInitializeErrorMessage               |
|                                                             |
\------------------------------------------------------------*/

void DrealInitializeErrorMessage( Debug )
{
  rdsbegin();

  DrealNormalMode = ! Debug;

  if ( DrealNormalMode )
  {
    sprintf( DrealOutFileName, "/usr/tmp/out_%d",getpid() ); 
  }

  sprintf( DrealErrFileName, "/usr/tmp/err_%d",getpid() );
  sprintf( DrealAllFileName, "/usr/tmp/all_%d",getpid() );

  DrealStreamErr = freopen( DrealErrFileName, "w+", stderr);
  DrealStreamAll = fopen  ( DrealAllFileName, "w+"        );

  if ( DrealNormalMode )
  {
    DrealStreamOut = freopen( DrealOutFileName, "w+", stdout);
  }
  else
  {
    DrealStreamOut = DrealStreamErr;
  }

  if ( ( DrealStreamAll == (FILE *)NULL ) ||
       ( DrealStreamOut == (FILE *)NULL ) ||
       ( DrealStreamErr == (FILE *)NULL ) )
  {
    fprintf( stdout, "Grm: Unable to open trace window !\n");
    exit(1);
  }

  if ( DrealNormalMode )
  {
    unlink( DrealOutFileName ); 
  }

  unlink( DrealErrFileName );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealFlushErrorMessage                 |
|                                                             |
\------------------------------------------------------------*/

void DrealFlushErrorMessage()
{
  int Data;

  rdsbegin();

  fflush( DrealStreamErr );
  fseek( DrealStreamErr, 0L, 0 );

  while( ( Data = fgetc( DrealStreamErr ) ) != EOF )
  {
    fputc( Data, DrealStreamAll );
  }

  fclose( DrealStreamErr );

  DrealStreamErr = freopen( DrealErrFileName, "w+", stderr );

  if ( DrealNormalMode )
  {
    fflush( DrealStreamOut );
    fseek( DrealStreamOut, 0L, 0 );

    while( ( Data = fgetc( DrealStreamOut ) ) != EOF )
    {
      fputc( Data, DrealStreamAll );
    } 

    fclose( DrealStreamOut );

    DrealStreamOut = freopen( DrealOutFileName, "w+", stdout );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealGetErrorMessage                   |
|                                                             |
\------------------------------------------------------------*/

char *DrealGetErrorMessage()
{
  char *Message;
  long  Length;
  long  Index;
  int   Data;

  rdsbegin();

  DrealFlushErrorMessage();

  fflush( DrealStreamAll );

  Length = ftell( DrealStreamAll );

  if ( DrealErrorBuffer != (char *)NULL )
  {
    rdsfreeblock( DrealErrorBuffer );
  }

  DrealErrorBuffer = rdsallocblock( Length + 1 );
  Index            = 0;

  while ( ( ( Data = fgetc( DrealStreamAll ) ) != EOF ) &&
          ( Index  < Length ) )
  {
    DrealErrorBuffer[ Index++ ] = (char)Data;
  }

  rewind( DrealStreamAll );

  DrealErrorBuffer[ Index ] = '\0';

  if ( Index == 0 )
  {
    Message = (char *)NULL;
  }
  else
  {
    Message = DrealErrorBuffer;
  }

  rdsend();
  return( Message );
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealGetInformations                   |
|                                                             |
\------------------------------------------------------------*/

char *DrealGetInformations()

{
  char *Scan;

  rdsbegin();

  DrealComputeBound();

  Scan = DrealInformationsBuffer;

  if ( DrealFigureRds != (rdsfig_list *)NULL )
  {
    sprintf( Scan, "  FIGURE : %s\n\n", DrealFigureRds->NAME );
  }
  else
  {
    sprintf( Scan, "  FIGURE : No current figure !\n\n" );
  }

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  BOUNDING BOX : \n\n  XMIN : %.3f\n  YMIN : %.3f\n  XMAX : %.3f\n  YMAX : %.3f\n\n",
           (float)( DrealBoundXmin ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealBoundYmin ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealBoundXmax ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealBoundYmax ) * DREAL_PHYSICAL_GRID_TO_MICRON );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  WINDOWS : \n\n  XMIN : %.3f\n  YMIN : %.3f\n  XMAX : %.3f\n  YMAX : %.3f\n  DX : %d\n  DY : %d\n  SIDE : %.3f\n\n",
           (float)( DrealWindowXmin ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealWindowYmin ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealWindowXmax ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           (float)( DrealWindowYmax ) * DREAL_PHYSICAL_GRID_TO_MICRON,
           DrealWindowDx,
           DrealWindowDy,
           (float)( DrealWindowSide ) * DREAL_PHYSICAL_GRID_TO_MICRON );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  RDS TECHNOLOGY : %s\n\n", RDS_TECHNO_NAME );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  DREAL TECHNOLOGY : %s\n\n", DREAL_TECHNO_NAME );


  rdsend();
  return( DrealInformationsBuffer );
}

