/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                   Rds.c                           |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <stdlib.h>
# include <signal.h>
# include <setjmp.h>
# include MUT_H
# include MPH_H
# include RDS_H
# include RTL_H
# include RUT_H
# include RPR_H
# include RFM_H
# include GSB_H 
# include GRD_H
# include "GRD_error.h"
# include "GRD_window.h"
# include "GRD_rds.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                       For Set Long Jump                     |
|                                                             |
\------------------------------------------------------------*/

  static jmp_buf DrealJumpBuffer;
  static void    (*OldExitHandler)() = NULL;

/*------------------------------------------------------------\
|                                                             |
|                         Dreal Figure                        |
|                                                             |
\------------------------------------------------------------*/

  rdsfig_list *DrealFigureRds     = (rdsfig_list *)NULL;

/*------------------------------------------------------------\
|                                                             |
|                 Dreal Exit Handler Functions                |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      DrealExitHandler                       |
|                                                             |
\------------------------------------------------------------*/

void DrealExitHandler()
{
  DrealFlushErrorMessage();

  signal( SIGTERM, OldExitHandler );
  longjmp( DrealJumpBuffer, 1);
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealActiveExitHandler                 |
|                                                             |
\------------------------------------------------------------*/

void DrealActiveExitHandler()
{
  OldExitHandler = signal( SIGTERM, DrealExitHandler );
}

/*------------------------------------------------------------\
|                                                             |
|                       DrealResetExitHandler                 |
|                                                             |
\------------------------------------------------------------*/
 
void DrealResetExitHandler() 
{ 
  signal( SIGTERM, OldExitHandler );
}

/*------------------------------------------------------------\
|                                                             |
|                     Dreal Add Functions                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                     Dreal Add Figure                        |
|                                                             |
\------------------------------------------------------------*/

rdsfig_list *DrealAddFigure( Name )

  char *Name;
{
  char         Layer;
  rdsrec_list *Rectangle;
  rdsins_list *Instance;

  rdsbegin();

  DrealFigureRds = Drealgetrdsfig( Name, 'V', DREAL_MODE );

  if ( DrealFigureRds != (rdsfig_list *)NULL )
  {
    DrealComputeBound();
    DrealCheckWindow();

    for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
    {
      for ( Rectangle  = DrealFigureRds->LAYERTAB[ Layer ];
            Rectangle != (rdsrec_list *)NULL;
            Rectangle  = Rectangle->NEXT )
      {
        DrealInsertRectangle( Rectangle );
      }
    }

    for ( Instance  = DrealFigureRds->INSTANCE;
          Instance != (rdsins_list *)NULL;
          Instance  = Instance->NEXT )
    {
      for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
      {
        for ( Rectangle  = Instance->LAYERTAB[ Layer ];
              Rectangle != (rdsrec_list *)NULL;
              Rectangle  = Rectangle->NEXT )
        {
          DrealInsertRectangle( Rectangle );
        }
      }
    }
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    Dreal Delete Functions                   |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      Dreal Del Figure                       |
|                                                             |
\------------------------------------------------------------*/

void DrealDelFigure()

{
  char         Layer;
  rdsrec_list *DelRec;
  rdsrec_list *ScanRec;
  rdsins_list *ScanIns;
  rdsins_list *DelIns;
  rdsfig_list *ScanFig;
  drealrecwin *RecWin;

  rdsbegin();

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    ScanRec  = DrealFigureRds->LAYERTAB[ Layer ];  

    while ( ScanRec != (rdsrec_list *)NULL )
    {
      DelRec   = ScanRec;
      ScanRec  = ScanRec->NEXT;

      DrealEraseRecWin( DelRec );

      freerdsrec( DelRec, DREAL_MODE );
    }

    DrealFigureRds->LAYERTAB[ Layer ] = (rdsrec_list *)NULL;
  }

  ScanIns  = DrealFigureRds->INSTANCE;

  while ( ScanIns != (rdsins_list *)NULL )
  {
    DelIns   = ScanIns;
    ScanIns  = ScanIns->NEXT;

    for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
    {
      ScanRec  = DrealFigureRds->LAYERTAB[ Layer ];  

      while ( ScanRec != (rdsrec_list *)NULL )
      {
        DelRec   = ScanRec;
        ScanRec  = ScanRec->NEXT;

        DrealEraseRecWin( DelRec );
      
        freerdsrec( DelRec, DREAL_MODE );
      }
    }

    freerdsins( DelIns );
  }

  DrealFigureRds->INSTANCE = (rdsins_list *)NULL;
  DrealEraseWindow();

  while ( HEAD_RDSFIG != (rdsfig_list *)NULL )
  {
    delrdsfig( HEAD_RDSFIG->NAME );
  }

  DrealFigureRds = (rdsfig_list *)NULL;

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                  Dreal Flatten Functions                    |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                    Dreal Flatten Figure                     |
|                                                             |
\------------------------------------------------------------*/

void DrealFlattenFigure()
{
  rdsfig_list *Figure;
  rdsins_list *ScanIns;
  rdsrec_list *ScanRec;
  rdsrec_list *DelRec;
  char         Layer;

  rdsbegin();

  for ( ScanIns  = DrealFigureRds->INSTANCE;
        ScanIns != (rdsins_list *)NULL;
        ScanIns  = ScanIns->NEXT )
  {
    for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
    {
      ScanRec = ScanIns->LAYERTAB[ Layer ];

      ScanIns->LAYERTAB[ Layer ] = (rdsrec_list *)NULL;

      while ( ScanRec  != (rdsrec_list *)NULL )
      {
        DelRec  = ScanRec;
        ScanRec = ScanRec->NEXT;

        DrealEraseRectangle( DelRec );

        freerdsrec( DelRec, DREAL_MODE );
      }
    }
  }

  Figure = addrdsfig( "_ludo_", DREAL_MODE );
  Figure->INSTANCE = DrealFigureRds->INSTANCE;
  DrealFigureRds->INSTANCE = (rdsins_list *)NULL;

  Drealrflattenrdsfig( Figure, RDS_YES, RDS_NO );

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    ScanRec = Figure->LAYERTAB[ Layer ]; 

    while ( ScanRec != (rdsrec_list *)NULL )
    {
      DrealInsertRectangle( ScanRec );

      if ( ScanRec->NEXT == (rdsrec_list *)NULL )
      {
        ScanRec->NEXT = DrealFigureRds->LAYERTAB[ Layer ];
        DrealFigureRds->LAYERTAB[ Layer ] = Figure->LAYERTAB[ Layer ];
        Figure->LAYERTAB[ Layer ] = (rdsrec_list *)NULL;

        break;
      }

      ScanRec = ScanRec->NEXT;
    }
  }

  delrdsfig( "_ludo_" );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    Drealgetrdsfig                           |
|                                                             |
\------------------------------------------------------------*/

rdsfig_list *Drealgetrdsfig( FigureName, Mode )

   char        *FigureName;
   char         Mode;
{
  rdsfig_list *FigureRds;

  rdsbegin();

  FigureName = namealloc( FigureName );

  DrealActiveExitHandler();

  if ( setjmp( DrealJumpBuffer ) == 0 )
  {
    FigureRds = getrdsfig( FigureName, Mode, DREAL_MODE );

    DrealResetExitHandler();

    rdsend();
    return( FigureRds );
  }
  else
  {
    delrdsfig( FigureName );

    rdsend();
    return( (rdsfig_list *)NULL );
  }
}

/*------------------------------------------------------------\
|                                                             |
|                    Drealrflattenrdsfig                      |
|                                                             |
\------------------------------------------------------------*/

void Drealrflattenrdsfig( Figure, Concat, Catal )

   rdsfig_list *Figure;
   char         Concat;
   char         Catal;
{
  rdsbegin();

  DrealActiveExitHandler();

  if ( setjmp( DrealJumpBuffer ) == 0 )
  {
    rflattenrdsfig( Figure, Concat, Catal );

    DrealResetExitHandler();
  }

  rdsend();
}
