/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                 Window.c                          |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <stdlib.h>
# include MUT_H
# include MPH_H
# include RDS_H
# include RPR_H
# include RFM_H
# include GSB_H
# include GRD_H
# include "GRD_window.h"

/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/

  long         DrealWindowXmin  = 0;
  long         DrealWindowYmin  = 0;
  long         DrealWindowXmax  = 0;
  long         DrealWindowYmax  = 0;
  long         DrealWindowDx    = 0;
  long         DrealWindowDy    = 0;
  long         DrealWindowSide  = 0;
  long         DrealWindowSize  = 0;
  drealwin   **DrealWindowTable = (drealwin **)NULL;

  long         DrealBoundXmin   = 0;
  long         DrealBoundYmin   = 0;
  long         DrealBoundXmax   = 0;
  long         DrealBoundYmax   = 0;

  char         DrealComputedBound = 0;

/*------------------------------------------------------------\
|                                                             |
|                          Functions                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                       Alloc Functions                       |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                        DrealAllocWin                        |
|                                                             |
\------------------------------------------------------------*/

drealwin *DrealAllocWin()
{
  return((drealwin *)rdsalloc(sizeof(drealwin), 0));
}

/*------------------------------------------------------------\
|                                                             |
|                       DrealAllocWinRec                      |
|                                                             |
\------------------------------------------------------------*/

drealwinrec *DrealAllocWinRec()
{
  return((drealwinrec *)rdsalloc(sizeof(drealwinrec), 1));
}

/*------------------------------------------------------------\
|                                                             |
|                       DrealAllocRecWin                      |
|                                                             |
\------------------------------------------------------------*/

drealrecwin *DrealAllocRecWin()
{
  return((drealrecwin *)rdsalloc(sizeof(drealrecwin), 1));
}

/*------------------------------------------------------------\
|                                                             |
|                        Free Functions                       |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         DrealFreeWin                        |
|                                                             |
\------------------------------------------------------------*/

void DrealFreeWin( FreeWin )

   drealwin *FreeWin;
{
  rdsfree((char *)FreeWin, 0);
}

/*------------------------------------------------------------\
|                                                             |
|                        DrealFreeWinRec                      |
|                                                             |
\------------------------------------------------------------*/

void DrealFreeWinRec( FreeWinRec )

   drealwinrec *FreeWinRec;
{
  rdsfree((char *)FreeWinRec, sizeof(drealwinrec));
}

/*------------------------------------------------------------\
|                                                             |
|                        DrealFreeRecWin                      |
|                                                             |
\------------------------------------------------------------*/

void DrealFreeRecWin( FreeRecWin )

   drealrecwin *FreeRecWin;
{
  rdsfree((char *)FreeRecWin, sizeof(drealrecwin));
}

/*------------------------------------------------------------\
|                                                             |
|                        Init Functions                       |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Drealgetenv                        |
|                                                             |
\------------------------------------------------------------*/

long Drealgetenv( Name )
 
  char *Name;
{
  char *String;

  rdsbegin();

  String = getenv( Name );

  if ( String == (char *)NULL )
  {
    rdsend();
    return( 0 );
  }

  rdsend();
  return( atoi( String ) );
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealInitializeWindow                  |
|                                                             |
\------------------------------------------------------------*/

void DrealInitializeWindow()
{
  rdsbegin();

  if ( DrealWindowTable == (drealwin **)NULL )
  {
    DrealWindowDx   = Drealgetenv( DREAL_WINDOW_DX   );
    DrealWindowDy   = Drealgetenv( DREAL_WINDOW_DY   );
    DrealWindowXmin = Drealgetenv( DREAL_WINDOW_XMIN );
    DrealWindowYmin = Drealgetenv( DREAL_WINDOW_YMIN );
    DrealWindowSide = Drealgetenv( DREAL_WINDOW_SIDE );

    if ( ( DrealWindowDx   <= 0 ) ||
         ( DrealWindowDy   <= 0 ) ||
         ( DrealWindowSide <= 0 ) )
    {
      DrealWindowDx   = DREAL_DEFAULT_WINDOW_DX;
      DrealWindowDy   = DREAL_DEFAULT_WINDOW_DY;
      DrealWindowSize = DREAL_DEFAULT_WINDOW_DX * DREAL_DEFAULT_WINDOW_DY;
      DrealWindowXmin = DREAL_DEFAULT_WINDOW_XMIN * RDS_PHYSICAL_GRID;
      DrealWindowYmin = DREAL_DEFAULT_WINDOW_YMIN * RDS_PHYSICAL_GRID;
      DrealWindowXmax = DREAL_DEFAULT_WINDOW_XMAX * RDS_PHYSICAL_GRID;
      DrealWindowYmax = DREAL_DEFAULT_WINDOW_YMAX * RDS_PHYSICAL_GRID;
      DrealWindowSide = DREAL_DEFAULT_WINDOW_SIDE * RDS_PHYSICAL_GRID;
    }
    else
    {
      DrealWindowSize = DrealWindowDx * DrealWindowDy;
      DrealWindowSide = DrealWindowSide * RDS_PHYSICAL_GRID;
      DrealWindowXmin = DrealWindowXmin * RDS_PHYSICAL_GRID;
      DrealWindowYmin = DrealWindowYmin * RDS_PHYSICAL_GRID;
      DrealWindowXmax = DrealWindowXmin + ( DrealWindowDx * DrealWindowSide );
      DrealWindowYmax = DrealWindowYmin + ( DrealWindowDy * DrealWindowSide );
    }

    DrealWindowTable = 

      (drealwin **)rdsalloc( sizeof(drealwin *) * 
                             DrealWindowSize, 0 );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       Purge Functions                       |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                       DrealEraseWindow                      |
|                                                             |
\------------------------------------------------------------*/

void DrealEraseWindow()

{
  long         Offset;
  drealwin    *ScanWin;
  drealwin    *DelWin;
  drealwinrec *ScanWinRec;
  drealwinrec *DelWinRec;
  char         Layer;

  rdsbegin();

  for ( Offset = 0; Offset < DrealWindowSize; Offset++ ) 
  {
    if ( DrealWindowTable[ Offset ] != (drealwin *)NULL )
    {
      ScanWin = DrealWindowTable[ Offset ];

      for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
      {
        ScanWinRec  = ScanWin->LAYERTAB[ Layer ];

        while ( ScanWinRec != (drealwinrec *)NULL )
        {
          DelWinRec   = ScanWinRec;
          ScanWinRec  = ScanWinRec->NEXT;

          DrealFreeWinRec( DelWinRec );
        }
      }

      DrealFreeWin( ScanWin );
       
      DrealWindowTable[ Offset ] = (drealwin *)NULL;
    }
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    DrealInsertRectangle                     |
|                                                             |
\------------------------------------------------------------*/

void DrealInsertRectangle( Rectangle )

   rdsrec_list *Rectangle;
{
  drealwin       *LinkWin;
  drealrecwin    *RecWin;
  drealwinrec    *WinRec;
  char            Layer;
  char            Index;
  long            Offset;
 
  long            X1;
  long            X2;
  long            Y1;
  long            Y2;
  long            FirstX;

  rdsbegin();

  Y1 = ( Rectangle->Y - 1             - DrealWindowYmin ) / DrealWindowSide;
  Y2 = ( Rectangle->Y + Rectangle->DY - DrealWindowYmin ) / DrealWindowSide;
  X1 = ( Rectangle->X - 1             - DrealWindowXmin ) / DrealWindowSide;
  X2 = ( Rectangle->X + Rectangle->DX - DrealWindowXmin ) / DrealWindowSide; 

  if ( X1 < 0              ) X1 = 0;
  if ( X2 < 0              ) X2 = 0;
  if ( X1 >= DrealWindowDx ) X1 = DrealWindowDx - 1;
  if ( X2 >= DrealWindowDx ) X2 = DrealWindowDx - 1;

  if ( Y1 < 0              ) Y1 = 0;
  if ( Y2 < 0              ) Y2 = 0;
  if ( Y1 >= DrealWindowDy ) Y1 = DrealWindowDy - 1;
  if ( Y2 >= DrealWindowDy ) Y2 = DrealWindowDy - 1;

  FirstX = X1;

  while ( Y1 <= Y2 )
  {
    X1 = FirstX;

    while ( X1 <= X2 )
    {
      Offset = Y1 * DrealWindowDx + X1;

      if ( DrealWindowTable[ Offset ] == (drealwin *)NULL )
      {
        DrealWindowTable[ Offset ] = DrealAllocWin();
      }

      LinkWin = DrealWindowTable[ Offset ];

      if ( DREAL_WINDOW( Rectangle ) == (drealrecwin *)NULL )
      {
        DREAL_WINDOW( Rectangle ) = (drealrecwin *)LinkWin;

        SetDrealOneWindow( Rectangle );
      }
      else
      {   
        if ( IsDrealOneWindow( Rectangle ) )
        {
          ClearDrealOneWindow( Rectangle );

          RecWin                    = DrealAllocRecWin();
          RecWin->WINDOW            = (drealwin *)DREAL_WINDOW( Rectangle );
          DREAL_WINDOW( Rectangle ) = RecWin;
        }

        RecWin                    = DrealAllocRecWin();
        RecWin->WINDOW            = LinkWin;
        RecWin->NEXT              = DREAL_WINDOW( Rectangle );
        DREAL_WINDOW( Rectangle ) = RecWin;
      }

      Layer = GetRdsLayer( Rectangle );
 
      WinRec = LinkWin->LAYERTAB[ Layer ];
 
      if ( WinRec == (drealwinrec *)NULL )
      {
        WinRec                     = DrealAllocWinRec();
        LinkWin->LAYERTAB[ Layer ] = WinRec;
        WinRec->RECTAB[ 0 ]        = Rectangle;
      }
      else
      {   
        for ( Index = 0; Index < DREAL_MAX_REC; Index++ )
        {
          if ( WinRec->RECTAB[ Index ] == (rdsrec_list *)NULL ) break;
        }

        if ( Index == DREAL_MAX_REC )
        {
          WinRec                     = DrealAllocWinRec();
          WinRec->NEXT               = LinkWin->LAYERTAB[ Layer ];
          LinkWin->LAYERTAB[ Layer ] = WinRec;
          WinRec->RECTAB[ 0 ]        = Rectangle;
        }
        else
        {
          WinRec->RECTAB[ Index ] = Rectangle;
        }
      }
        
      X1 = X1 + 1;
    }
 
    Y1  = Y1 + 1;
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     DrealEraseRectangle                     |
|                                                             |
\------------------------------------------------------------*/

void DrealEraseRectangle( Rectangle )

  rdsrec_list *Rectangle;
{
  drealrecwin  StaticRecWin;
  drealrecwin *ScanRecWin;
  drealrecwin *DelRecWin;
  drealwinrec *ScanWinRec; 
  drealwinrec *FirstWinRec;
  drealwin    *ScanWin;

  char         Layer;
  char         Index;
  char         Found;

  rdsbegin();

  if ( IsDrealOneWindow( Rectangle ) )
  {
    StaticRecWin.NEXT   = (drealrecwin *)NULL;
    StaticRecWin.WINDOW = (drealwin *)DREAL_WINDOW( Rectangle );
    ScanRecWin          = &StaticRecWin;
  }
  else
  {
    ScanRecWin = DREAL_WINDOW( Rectangle );
  }

  Layer = GetRdsLayer( Rectangle );

  while ( ScanRecWin != (drealrecwin *)NULL )
  {
    DelRecWin  = ScanRecWin;
    ScanRecWin = ScanRecWin->NEXT;

    ScanWin = DelRecWin->WINDOW;
    Found   = DREAL_MAX_REC;

    FirstWinRec = ScanWin->LAYERTAB[ Layer ];

    for ( ScanWinRec  = FirstWinRec;
          ScanWinRec != (drealwinrec *)NULL;
          ScanWinRec  = ScanWinRec->NEXT )
    {
      for ( Index = 0; Index < DREAL_MAX_REC; Index++ )
      {
        if ( ScanWinRec->RECTAB[ Index ] == Rectangle )
        {
          Found = Index; break;
        }
      }

      if ( Found != DREAL_MAX_REC ) break;
    }

    if ( ScanWinRec == FirstWinRec )
    {
      ScanWinRec->RECTAB[ Found ] = (rdsrec_list *)NULL;
    }
    else
    {
      for ( Index = 0; Index < DREAL_MAX_REC; Index++ )
      {
        if ( FirstWinRec->RECTAB[ Index ] != (rdsrec_list *)NULL ) break;
      }

      ScanWinRec->RECTAB[ Found ]  = FirstWinRec->RECTAB[ Index ];
      FirstWinRec->RECTAB[ Index ] = (rdsrec_list *)NULL;
    }

    for ( Index = 0; Index < DREAL_MAX_REC; Index++ )
    {
      if ( FirstWinRec->RECTAB[ Index ] != (rdsrec_list *)NULL ) break;
    }

    if ( Index == DREAL_MAX_REC )
    {
      ScanWin->LAYERTAB[ Layer ] = FirstWinRec->NEXT;

      DrealFreeWinRec( FirstWinRec );
    }
  
    if ( DelRecWin != &StaticRecWin )
    {
      DrealFreeRecWin( DelRecWin );
    }
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     DrealEraseRecWin                        |
|                                                             |
\------------------------------------------------------------*/
 
void DrealEraseRecWin( Rectangle )

  rdsrec_list *Rectangle;
{ 
  drealrecwin *ScanRecWin;
  drealrecwin *DelRecWin;

  rdsbegin();

  if ( ! IsDrealOneWindow( Rectangle ) )
  { 
    ScanRecWin = DREAL_WINDOW( Rectangle );

    do
    {
      DelRecWin  = ScanRecWin;
      ScanRecWin = ScanRecWin->NEXT;

      DrealFreeRecWin( DelRecWin );
    }
    while ( ScanRecWin != (drealrecwin *)NULL );
  } 

  DREAL_WINDOW( Rectangle ) = (drealrecwin *)NULL;

  rdsend();
} 

/*------------------------------------------------------------\
|                                                             |
|                     DrealViewWindow                         |
|                                                             |
\------------------------------------------------------------*/

# ifdef DEBUG

void DrealViewWindow()
{
  drealwin       *ScanWin;
  drealwinrec    *ScanWinRec;
  rdsrec_list    *Rectangle;
  long            X;
  long            Y;
  long            Xmin;
  long            Ymin;
  long            Xmax;
  long            Ymax;
  long            Counter;
  long            MaskCounter;
  char            Layer;
  char            ScanRec;

  rdsbegin();
 
  fprintf( stdout, "\n\t--> Window\n" );
  fprintf( stdout, "\n\tXMIN : %d"  , DrealWindowXmin );
  fprintf( stdout, "\n\tXMAX : %d"  , DrealWindowXmax );
  fprintf( stdout, "\n\tYMIN : %d"  , DrealWindowYmin );
  fprintf( stdout, "\n\tYMAX : %d"  , DrealWindowYmax );
  fprintf( stdout, "\n\tDX   : %d"  , DrealWindowDx   );
  fprintf( stdout, "\n\tDY   : %d"  , DrealWindowDy   );
  fprintf( stdout, "\n\tSIDE : %d"  , DrealWindowSide );
  fprintf( stdout, "\n\tSIZE : %d\n", DrealWindowSize );
 
  fprintf( stdout, "\n\t--> Dump all windows\n" );
 
  Ymin = DrealWindowYmin;
  Ymax = Ymin + DrealWindowSide;
  Y    = 0;
 
  MaskCounter = 0;

  while ( Y < DrealWindowDy )
  {
    Xmin = DrealWindowXmin;
    Xmax = Xmin + DrealWindowSide ;
    X    = 0;
 
    ScanWin = DrealWindowTable[ Y * DrealWindowDx ];

    while ( X < DrealWindowDx )
    {
      fprintf( stdout, "\n\t--> Window[ %d, %d ]\n", X, Y );

      fprintf( stdout, "\n\tXMIN : %d"  , Xmin );
      fprintf( stdout, "\n\tYMIN : %d"  , Ymin );
      fprintf( stdout, "\n\tXMAX : %d"  , Xmax );
      fprintf( stdout, "\n\tYMAX : %d\n", Ymax );

      if ( ScanWin != (drealwin *)NULL )
      {
        Counter = 0;

        for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
        {
          for ( ScanWinRec  = ScanWin->LAYERTAB[ Layer ];
                ScanWinRec != (drealwinrec *)NULL;
                ScanWinRec  = ScanWinRec->NEXT )
          {
            for ( ScanRec = 0; ScanRec < DREAL_MAX_REC ; ScanRec++ )
            {
              Rectangle = ScanWinRec->RECTAB[ ScanRec ];

              if ( Rectangle != (rdsrec_list *)NULL )
              {
                if ( ( Rectangle->X > Xmax                 ) ||
                     ( Rectangle->Y > Ymax                 ) ||
                     ( Rectangle->X + Rectangle->DX < Xmin ) ||
                     ( Rectangle->Y + Rectangle->DY < Ymin ) )
                {
                  viewrdsrec( Rectangle );
                }
                else
                {   
                  Counter = Counter + 1;
                }
              }  
            }  
          }
        }

        MaskCounter = MaskCounter + Counter;

        fprintf( stdout, "\n\t<-- Total rectangles %d", Counter );
      }

      Xmin = Xmax;
      Xmax = Xmax + DrealWindowSide;
      X    = X + 1;

      ScanWin = DrealWindowTable[ Y * DrealWindowDx + X ];
    }

    Ymin = Ymax;
    Ymax = Ymax + DrealWindowSide;
    Y    = Y + 1;
  }

  fprintf( stdout, "\n<-- Total Rectangles %d\n", MaskCounter );

  rdsend();
}

# endif

/*------------------------------------------------------------\
|                                                             |
|                     DrealComputeBound                       |
|                                                             |
\------------------------------------------------------------*/

void DrealComputeBound()

{
  rdsrec_list *Rec;
  rdsins_list *Ins;
  char         FirstBound;
  char         Layer;

  if ( DrealFigureRds == (rdsfig_list *)NULL )
  {
    DrealComputedBound = 0; return;
  }

  rdsbegin();
  FirstBound = 1;

  for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
  {
    for ( Rec  = DrealFigureRds->LAYERTAB[ Layer ];
          Rec != (rdsrec_list *)NULL;
          Rec  = Rec->NEXT )
    {
      if ( FirstBound )
      {
        FirstBound = 0;

        DrealBoundXmin = Rec->X;
        DrealBoundYmin = Rec->Y;
        DrealBoundXmax = Rec->DX + DrealBoundXmin;
        DrealBoundYmax = Rec->DY + DrealBoundYmin;
      }
      else
      {
        if ( DrealBoundXmin > Rec->X ) DrealBoundXmin = Rec->X;
        if ( DrealBoundYmin > Rec->Y ) DrealBoundYmin = Rec->Y;
        if ( DrealBoundXmax < ( Rec->X + Rec->DX ) ) DrealBoundXmax = Rec->X + Rec->DX;
        if ( DrealBoundYmax < ( Rec->Y + Rec->DY ) ) DrealBoundYmax = Rec->Y + Rec->DY;
      }
    }
  }

  for ( Ins  = DrealFigureRds->INSTANCE;
        Ins != (rdsins_list *)NULL;
        Ins  = Ins->NEXT )
  {
    for ( Layer = 0; Layer < RDS_MAX_LAYER; Layer++ )
    {
      for ( Rec  = Ins->LAYERTAB[ Layer ];
            Rec != (rdsrec_list *)NULL;
            Rec  = Rec->NEXT )
      {
        if ( FirstBound )
        {
          FirstBound = 0;

          DrealBoundXmin = Rec->X;
          DrealBoundYmin = Rec->Y;
          DrealBoundXmax = Rec->DX + DrealBoundXmin;
          DrealBoundYmax = Rec->DY + DrealBoundYmin;
        }
        else
        {
          if ( DrealBoundXmin > Rec->X ) DrealBoundXmin = Rec->X;
          if ( DrealBoundYmin > Rec->Y ) DrealBoundYmin = Rec->Y;
          if ( DrealBoundXmax < ( Rec->X + Rec->DX ) ) DrealBoundXmax = Rec->X + Rec->DX;
          if ( DrealBoundYmax < ( Rec->Y + Rec->DY ) ) DrealBoundYmax = Rec->Y + Rec->DY;
        }
      }
    }
  }

  DrealComputedBound = ( ! FirstBound );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealCheckWindow                       |
|                                                             |
\------------------------------------------------------------*/

void DrealCheckWindow()
{
  long         Border;
  long         SideX;
  long         SideY;
  char         Compute;

  rdsbegin();

  Compute = 0;
  Border  = DREAL_WINDOW_BORDER * RDS_PHYSICAL_GRID;

  if ( ( DrealBoundXmin - Border ) <= DrealWindowXmin )
  {
    DrealWindowXmin = DrealBoundXmin - Border;
    Compute         = 1;
  }

  if ( ( DrealBoundYmin - Border ) <= DrealWindowYmin )
  {
    DrealWindowYmin = DrealBoundYmin - Border;
    Compute         = 1;
  }

  if ( ( DrealBoundXmax + Border ) >= DrealWindowXmax )
  {
    DrealWindowXmax = DrealBoundXmax + Border;
    Compute         = 1;
  }

  if ( ( DrealBoundYmax + Border ) >= DrealWindowYmax )
  {
    DrealWindowYmax = DrealBoundYmax + Border;
    Compute         = 1;
  }

  if ( Compute )
  {
    DrealEraseWindow();
    rdsfree( DrealWindowTable, 0 );

    DrealWindowDx = DREAL_DEFAULT_WINDOW_DX;
    DrealWindowDy = DREAL_DEFAULT_WINDOW_DY;

    SideX = 1 + ( DrealWindowXmax - DrealWindowXmin ) / DrealWindowDx;
    SideY = 1 + ( DrealWindowYmax - DrealWindowYmin ) / DrealWindowDy;

    if ( SideX < SideY )
    {
      DrealWindowSide = SideY;
    }
    else
    {
      DrealWindowSide = SideX;
    }

    DrealWindowDx = 1 + ( DrealWindowXmax - DrealWindowXmin ) / DrealWindowSide;
    DrealWindowDy = 1 + ( DrealWindowYmax - DrealWindowYmin ) / DrealWindowSide;

    DrealWindowXmax = DrealWindowXmin + ( DrealWindowDx * DrealWindowSide );
    DrealWindowYmax = DrealWindowYmin + ( DrealWindowDy * DrealWindowSide );
    DrealWindowSize = DrealWindowDx * DrealWindowDy;

    DrealWindowTable = 

      (drealwin **)rdsalloc( sizeof(drealwin *) * 
                             DrealWindowSize, 0 );
  }

  rdsend();
}
