/*------------------------------------------------------------\
|                                                             |
| Tool    :                   DREAL                           |
|                                                             |
| File    :                   Panel.c                         |
|                                                             |
| Authors :              Jacomme Ludovic                      |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <Xm/Xm.h>
# include <Xm/PushBG.h>
# include <Xm/CascadeBG.h>

# include MPH_H
# include RDS_H
# include RPR_H
# include GSB_H
# include GTB_H
# include GMX_H
# include GMV_H

# include "GMV_panel.h"
# include "GMV_dialog.h"
# include "GMV_view.h"
# include "GMV_message.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                        Panel Pixmap                         |
|                                                             |
\------------------------------------------------------------*/

  static char DrealPixmapDownArrow[] =

  { 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xfe, 0xff, 0x00, 0x00, 0x02, 0x80, 0x00, 0x00, 0x02, 0x80, 0x00,
    0x00, 0x02, 0x80, 0x00, 0x00, 0x02, 0x80, 0x00, 0x00, 0x02, 0x80, 0x00,
    0x00, 0x02, 0x80, 0x00, 0x00, 0x02, 0x80, 0x00, 0x00, 0x02, 0x80, 0x00,
    0xf0, 0x03, 0x80, 0x1f, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10,
    0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00, 0x08,
    0x40, 0x00, 0x00, 0x04, 0x80, 0x00, 0x00, 0x02, 0x00, 0x01, 0x00, 0x01,
    0x00, 0x02, 0x80, 0x00, 0x00, 0x04, 0x40, 0x00, 0x00, 0x08, 0x20, 0x00,
    0x00, 0x10, 0x10, 0x00, 0x00, 0x20, 0x08, 0x00, 0x00, 0x40, 0x04, 0x00,
    0x00, 0x80, 0x02, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
  };

  static char DrealPixmapLeftArrow[] =

  {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x80, 0x0f, 0x00, 0x00, 0x40, 0x08, 0x00, 0x00, 0x20, 0x08, 0x00,
    0x00, 0x10, 0x08, 0x00, 0x00, 0x08, 0x08, 0x00, 0x00, 0x04, 0xf8, 0x1f,
    0x00, 0x02, 0x00, 0x10, 0x00, 0x01, 0x00, 0x10, 0x80, 0x00, 0x00, 0x10,
    0x40, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10,
    0x08, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00, 0x10,
    0x40, 0x00, 0x00, 0x10, 0x80, 0x00, 0x00, 0x10, 0x00, 0x01, 0x00, 0x10,
    0x00, 0x02, 0x00, 0x10, 0x00, 0x04, 0xf8, 0x1f, 0x00, 0x08, 0x08, 0x00,
    0x00, 0x10, 0x08, 0x00, 0x00, 0x20, 0x08, 0x00, 0x00, 0x40, 0x08, 0x00,
    0x00, 0x80, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
  };

  static char DrealPixmapRightArrow[] =

  {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xf0, 0x01, 0x00, 0x00, 0x10, 0x02, 0x00, 0x00, 0x10, 0x04, 0x00,
    0x00, 0x10, 0x08, 0x00, 0x00, 0x10, 0x10, 0x00, 0xf8, 0x1f, 0x20, 0x00,
    0x08, 0x00, 0x40, 0x00, 0x08, 0x00, 0x80, 0x00, 0x08, 0x00, 0x00, 0x01,
    0x08, 0x00, 0x00, 0x02, 0x08, 0x00, 0x00, 0x04, 0x08, 0x00, 0x00, 0x08,
    0x08, 0x00, 0x00, 0x10, 0x08, 0x00, 0x00, 0x08, 0x08, 0x00, 0x00, 0x04,
    0x08, 0x00, 0x00, 0x02, 0x08, 0x00, 0x00, 0x01, 0x08, 0x00, 0x80, 0x00,
    0x08, 0x00, 0x40, 0x00, 0xf8, 0x1f, 0x20, 0x00, 0x00, 0x10, 0x10, 0x00,
    0x00, 0x10, 0x08, 0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x10, 0x02, 0x00,
    0x00, 0xf0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
  };

  static char DrealPixmapUpArrow[] =

  {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x80, 0x00, 0x00, 0x00, 0x40, 0x01, 0x00, 0x00, 0x20, 0x02, 0x00,
    0x00, 0x10, 0x04, 0x00, 0x00, 0x08, 0x08, 0x00, 0x00, 0x04, 0x10, 0x00,
    0x00, 0x02, 0x20, 0x00, 0x00, 0x01, 0x40, 0x00, 0x80, 0x00, 0x80, 0x00,
    0x40, 0x00, 0x00, 0x01, 0x20, 0x00, 0x00, 0x02, 0x10, 0x00, 0x00, 0x04,
    0x08, 0x00, 0x00, 0x08, 0x08, 0x00, 0x00, 0x08, 0x08, 0x00, 0x00, 0x08,
    0x08, 0x00, 0x00, 0x08, 0xf8, 0x01, 0xc0, 0x0f, 0x00, 0x01, 0x40, 0x00,
    0x00, 0x01, 0x40, 0x00, 0x00, 0x01, 0x40, 0x00, 0x00, 0x01, 0x40, 0x00,
    0x00, 0x01, 0x40, 0x00, 0x00, 0x01, 0x40, 0x00, 0x00, 0x01, 0x40, 0x00,
    0x00, 0x01, 0x40, 0x00, 0x00, 0xff, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
  };

/*------------------------------------------------------------\
|                                                             |
|                        Panel Arrows                         |
|                                                             |
\------------------------------------------------------------*/

   static char *DrealPanelArrowsButtonName[] = 

   {
     "Left",  
     "Up",
     "Down",
     "Right",
     "Move Set",
     "Close"
   };

   static DrealPanelButtonItem DrealViewArrowsButton[] =

         {
           {
             &DrealPanelArrowsButtonName[0],
             NULL, DrealPixmapLeftArrow, 32, 32,
             NULL,
             NULL,
             0, 0,
             1, 2,
             CallbackLeftArrow,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelArrowsButtonName[1],
             NULL, DrealPixmapUpArrow, 32, 32,
             NULL,
             NULL,
             1, 0,
             1, 2,
             CallbackUpArrow,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelArrowsButtonName[2],
             NULL, DrealPixmapDownArrow, 32, 32,
             NULL,
             NULL,
             0, 2,
             1, 2,
             CallbackDownArrow,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelArrowsButtonName[3],
             NULL, DrealPixmapRightArrow, 32, 32,
             NULL,
             NULL,
             1, 2,
             1, 2,
             CallbackRightArrow,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelArrowsButtonName[4],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 4,
             2, 1,
             CallbackMoveSet,
             (XtPointer)NULL,
             (Widget)NULL
           } 
           , 
           {
             &DrealPanelArrowsButtonName[5],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 5,
             2, 1,
             CallbackCloseArrows,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           NULL
         };

   DrealPanelItem DrealViewArrowsPanel =

         {
           "Arrows",
           1,
           1,
           DREAL_VIEW_ARROWS_X,
           DREAL_VIEW_ARROWS_Y, 
           90,
           125,
           2,  
           6,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           DrealViewArrowsButton
         };

   int DrealViewArrowsDefaultValues[ 5 ] =

         {
           DREAL_VIEW_ARROWS_X, 
           DREAL_VIEW_ARROWS_Y,
           90, 125, 1
         };

/*------------------------------------------------------------\
|                                                             |
|                        Panel Zoom                           |
|                                                             |
\------------------------------------------------------------*/

 static char *DrealPanelZoomButtonName[] =

   {
     "Refresh",
     "UnZoom",
     "Zoom",
     "Mooz",
     "Zoom Set",
     "Zoom In",
     "Center",
     "Goto",
     "Pan",
     "Fit",
     "Close"
   };


 DrealPanelButtonItem DrealViewZoomButton[] =

         {
           {
             &DrealPanelZoomButtonName[0],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 0,
             1, 1,
             CallbackZoomRefresh,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[1],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 1,
             1, 1,
             CallbackZoomPrevious,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[2],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 2,
             1, 1,
             CallbackZoomMore,
             (XtPointer)NULL,
             (Widget)NULL
           } 
           ,
           {
             &DrealPanelZoomButtonName[3],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 3,
             1, 1,
             CallbackZoomLess,
             (XtPointer)NULL,
             (Widget)NULL
           } 
           ,
           {
             &DrealPanelZoomButtonName[4],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 4,
             1, 1,
             CallbackZoomSet,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[5],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 5,
             1, 1,
             CallbackZoomIn,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[6],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 6,
             1, 1,
             CallbackZoomCenter,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[7],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 7,
             1, 1,
             CallbackZoomGoto,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[8],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 8,
             1, 1,
             CallbackZoomPan,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealPanelZoomButtonName[9],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 9,
             1, 1,
             CallbackZoomFit,
             (XtPointer)NULL,
             (Widget)NULL
           } 
           , 
           {
             &DrealPanelZoomButtonName[10],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 10,
             1, 1,
             CallbackCloseZoom,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           NULL
         };

   DrealPanelItem DrealViewZoomPanel =

         {
           "Zoom",
           1,
           1,
           DREAL_VIEW_ZOOM_X,
           DREAL_VIEW_ZOOM_Y, 
           90,
           275,
           1,  
           11,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           DrealViewZoomButton
         };

   int DrealViewZoomDefaultValues[ 5 ] =

         {
           DREAL_VIEW_ZOOM_X, 
           DREAL_VIEW_ZOOM_Y,
           90, 275, 1
         };

/*------------------------------------------------------------\
|                                                             |
|                        Panel Layer                          |
|                                                             |
\------------------------------------------------------------*/

  static char DrealViewLayerTranslate [ DREAL_RDS_MAX_LAYER ] =

  {
    RDS_NWELL,
    RDS_PWELL,
    RDS_NIMP,
    RDS_PIMP,
    RDS_ACTIV,
    RDS_NDIF,
    RDS_PDIF,
    RDS_NTIE,
    RDS_PTIE,
    RDS_POLY,
    RDS_GATE,
    RDS_TPOLY,
    RDS_CONT,
    RDS_ALU1,
    RDS_TALU1,
    RDS_VIA1,
    RDS_ALU2,
    RDS_TALU2,
    RDS_VIA2,
    RDS_ALU3,
    RDS_TALU3,
    RDS_VIA3,
    RDS_ALU4,
    RDS_CPAS,
    RDS_REF,
    RDS_ABOX
  };

  static char *DrealViewLayerButtonName[] =

  {
    "All visible",
    "All invisible",
    "Fig",
    "Inst",
    "FCon",
    "ICon",
    "FSeg",
    "ISeg",
    "FRef",
    "IRef",
    "Pattern",
    "Interface",
    "Apply",
    "Close"
  };

 static char *DrealViewLayerFillModeName[] =

  {
    "Outline",
    "Fill",
    "Pattern"
  };

  static char *DrealViewLayerInterfaceName[] =

  {
    "No interface",
    "Interface"
  };

  static char *DrealViewLayerButtonBackground = "Black";
  static char *DrealViewLayerButtonForeground = "White";

     DrealPanelButtonItem DrealViewLayerButton[] =

         {
           {
             &DrealViewLayerButtonName[0],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 0,
             2, 1,
             CallbackLayerAllVisible,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[1],
             NULL, NULL, 0, 0, 
             NULL, 
             NULL, 
             2, 0, 
             2, 1, 
             CallbackLayerAllInvisible, 
             (XtPointer)NULL, 
             (Widget)NULL 
           } 
           , 
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][0],
             NULL, NULL, 0, 0, 
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][2], 
             0, 1, 
             1, 1, 
             CallbackLayerVisible, 
             (XtPointer)DREAL_RDS_NWELL,
             (Widget)NULL 
           } 
           , 
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][0],
             NULL, NULL, 0, 0, 
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NWELL][2], 
             1, 1, 
             1, 1, 
             CallbackLayerInvisible, 
             (XtPointer)DREAL_RDS_NWELL,
             (Widget)NULL 
           } 
           , 
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][0],
             NULL, NULL, 0, 0, 
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][2],
             2, 1, 
             1, 1,
             CallbackLayerVisible, 
             (XtPointer)DREAL_RDS_PWELL,
             (Widget)NULL 
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][0],
             NULL, NULL, 0, 0, 
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PWELL][2],
             3, 1, 
             1, 1,
             CallbackLayerInvisible, 
             (XtPointer)DREAL_RDS_PWELL,  
             (Widget)NULL 
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][2],
             0, 2,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_NIMP,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NIMP][2],
             1, 2,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_NIMP,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][2],
             2, 2,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_PIMP,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PIMP][2],
             3, 2,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_PIMP,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][2],
             0, 3,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ACTIV,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ACTIV][2],
             1, 3,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ACTIV,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][2],
             2, 3,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_NDIF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NDIF][2],
             3, 3,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_NDIF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][2],
             0, 4,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_PDIF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PDIF][2],
             1, 4,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_PDIF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][2],
             2, 4,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_NTIE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_NTIE][2],
             3, 4,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_NTIE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][2],
             0, 5,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_PTIE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_PTIE][2],
             1, 5,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_PTIE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][2],
             2, 5,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_POLY,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_POLY][2],
             3, 5,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_POLY,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][2],
             0, 6,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_GATE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_GATE][2],
             1, 6,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_GATE,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][2],
             2, 6,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_TPOLY,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TPOLY][2],
             3, 6,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_TPOLY,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][2],
             0, 7,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_CONT,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CONT][2],
             1, 7,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_CONT,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][2],
             2, 7,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ALU1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU1][2],
             3, 7,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ALU1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][2],
             0, 8,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_TALU1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU1][2],
             1, 8,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_TALU1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][2],
             2, 8,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_VIA1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA1][2],
             3, 8,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_VIA1,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][2],
             0, 9,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ALU2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU2][2],
             1, 9,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ALU2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][2],
             2, 9,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_TALU2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU2][2],
             3, 9,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_TALU2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][2],
             0, 10,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_VIA2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA2][2],
             1, 10,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_VIA2,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][2],
             2, 10,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ALU3,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU3][2],
             3, 10,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ALU3,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][2],
             0, 11,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_TALU3,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_TALU3][2],
             1, 11,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_TALU3,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][2],
             2, 11,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_VIA3,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_VIA3][2],
             3, 11,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_VIA3,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][2],
             0, 12,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ALU4,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ALU4][2],
             1, 12,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ALU4,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][2],
             2, 12,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_CPAS,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_CPAS][2],
             3, 12,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_CPAS,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][2],
             0, 13,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_REF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_REF][2],
             1, 13,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_REF,
             (Widget)NULL
           } 
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][2],
             2, 13,
             1, 1,
             CallbackLayerVisible,
             (XtPointer)DREAL_RDS_ABOX,
             (Widget)NULL
           }
           ,
           {
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][0],
             NULL, NULL, 0, 0,
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][1],
             &DREAL_RDS_LAYER_NAME_TABLE[RDS_ABOX][2],
             3, 13,
             1, 1,
             CallbackLayerInvisible,
             (XtPointer)DREAL_RDS_ABOX,
             (Widget)NULL
           } 
           ,
           {
             &DrealViewLayerButtonName[2],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             0, 14,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_FIGURE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[2],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             1, 14,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_FIGURE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[3],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             2, 14,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_INSTANCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[3],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             3, 14,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_INSTANCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[4],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             0, 15,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_FIGURE_CONNECTOR_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[4],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             1, 15,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_FIGURE_CONNECTOR_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[5],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             2, 15,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_INSTANCE_CONNECTOR_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[5],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             3, 15,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_INSTANCE_CONNECTOR_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[6],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             0, 16,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_FIGURE_SEGMENT_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[6],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             1, 16,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_FIGURE_SEGMENT_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[7],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             2, 16,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_INSTANCE_SEGMENT_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[7],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             3, 16,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_INSTANCE_SEGMENT_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[8],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             0, 17,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_FIGURE_REFERENCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[8],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             1, 17,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_FIGURE_REFERENCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[9],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             2, 17,
             1, 1,
             CallbackNameVisible,
             (XtPointer)DREAL_INSTANCE_REFERENCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[9],
             NULL, NULL, 0, 0,
             &DrealViewLayerButtonForeground,
             &DrealViewLayerButtonBackground,
             3, 17,
             1, 1,
             CallbackNameInvisible,
             (XtPointer)DREAL_INSTANCE_REFERENCE_NAME,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[10],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 18,
             2, 1,
             CallbackLayerFillMode,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[11],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             2, 18,
             2, 1,
             CallbackLayerInterface,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[12],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 19,
             2, 1,
             CallbackLayerApply,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           {
             &DrealViewLayerButtonName[13],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             2, 19,
             2, 1,
             CallbackCloseLayer,
             (XtPointer)NULL,
             (Widget)NULL
           }
           ,
           NULL
         };


     DrealPanelItem DrealViewLayerPanel =

         {
           "Layer",
           1,
           0,
           DREAL_VIEW_LAYER_X,
           DREAL_VIEW_LAYER_Y,
           180,
           500,
           4,
           20,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           DrealViewLayerButton
         };

  int DrealViewLayerDefaultValues[ 5 ] =

         {
           DREAL_VIEW_LAYER_X,
           DREAL_VIEW_LAYER_Y,
           180, 500, 0
         };

   static char FirstEnterLayerPanel = 1;

/*------------------------------------------------------------\
|                                                             |
|                        Panel Grid                           |
|                                                             |
\------------------------------------------------------------*/

 static char *DrealPanelGridButtonName[] =

   {
     "Grid X",
     "Grid Y",
     "Grid Off",
     "Close"
   };


 DrealPanelButtonItem DrealViewGridButton[] =

         {
           {
             &DrealPanelGridButtonName[0],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 0,
             1, 1,
             CallbackSetGridX,
             (XtPointer)NULL,
             (Widget)NULL
           }
           , 
           {
             &DrealPanelGridButtonName[1],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 1,
             1, 1,
             CallbackSetGridY,
             (XtPointer)NULL,
             (Widget)NULL   
           }
           ,
           {
             &DrealPanelGridButtonName[2],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 2,
             1, 1,
             CallbackGridOnOff,
             (XtPointer)NULL,
             (Widget)NULL   
           }
           ,
           {
             &DrealPanelGridButtonName[3],
             NULL, NULL, 0, 0,
             NULL,
             NULL,
             0, 3,
             1, 1,
             CallbackCloseGrid,
             (XtPointer)NULL,
             (Widget)NULL   
           }
           ,
           NULL
         };

     DrealPanelItem DrealViewGridPanel =

         {
           "Grid",
           1,
           0,
           DREAL_VIEW_GRID_X,
           DREAL_VIEW_GRID_Y, 
           90,
           100,
           1,  
           4,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           (Widget)NULL,
           DrealViewGridButton
         };

  int DrealViewGridDefaultValues[ 5 ] =

         {
           DREAL_VIEW_GRID_X,
           DREAL_VIEW_GRID_Y, 
           90, 100, 0
         };

/*------------------------------------------------------------\
|                                                             |
|                          Functions                          |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      Callback For Arrows                    |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      CallbackLeftArrow                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackLeftArrow( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomLeft();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     CallbackRightArrow                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackRightArrow( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomRight();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackUpArrow                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackUpArrow( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomUp();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackDownArrow                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackDownArrow( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomDown();

  rdsend();
}


/*------------------------------------------------------------\
|                                                             |
|                       CallbackMoveSet                       |
|                                                             |
\------------------------------------------------------------*/

void CallbackMoveSet( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealEnterDialog( &DrealViewMoveSetDialog );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackCloseArrows                    |
|                                                             |
\------------------------------------------------------------*/

void CallbackCloseArrows( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealExitPanel( &DrealViewArrowsPanel );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                         Zoom Callback                       |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                       CallbackZoomRefresh                   |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomRefresh( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomRefresh();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     CallbackZoomPrevious                    |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomPrevious( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomUndo();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackZoomLess                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomLess( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomLess();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomMore                     |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomMore( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomMore();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackZoomSet                       |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomSet( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealEnterDialog( &DrealViewZoomSetDialog );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackCloseZoom                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackCloseZoom( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealExitPanel( &DrealViewZoomPanel );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomIn                       |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomIn( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealChangeEditMode( DREAL_ZOOM_IN,
                       DrealPromptZoomIn );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomCenter                   |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomCenter( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealChangeEditMode( DREAL_ZOOM_CENTER,
                       DrealPromptZoomCenter );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomGoto                     |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomGoto( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealEnterDialog( &DrealViewZoomGotoDialog );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomPan                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomPan( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealChangeEditMode( DREAL_ZOOM_PAN,
                       DrealPromptZoomPan );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                        CallbackZoomFit                      |
|                                                             |
\------------------------------------------------------------*/

void CallbackZoomFit( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomFit();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      Callback For Layer                     |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                    CallbackLayerAllVisible                  |
|                                                             |
\------------------------------------------------------------*/

void CallbackLayerAllVisible( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealLayerAllVisible();

  rdsend();
}

/*------------------------------------------------------------\ 
|                                                             | 
|                  CallbackLayerAllInvisible                  | 
|                                                             | 
\------------------------------------------------------------*/ 
 
void CallbackLayerAllInvisible( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData; 
{
  rdsbegin();

  DrealLayerAllInvisible();

  rdsend();
}

/*------------------------------------------------------------\ 
|                                                             | 
|                      CallbackLayerVisible                   | 
|                                                             | 
\------------------------------------------------------------*/ 
 
void CallbackLayerVisible( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData; 
{
  char Layer;

  rdsbegin();

  Layer = DrealViewLayerTranslate[ (int)ClientData ];

  if ( DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] == 0 )
  {
    DrealSetLayerVisible( ClientData );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackNameVisible                   |
|                                                             |
\------------------------------------------------------------*/

void CallbackNameVisible( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  if ( DREAL_RDS_ACTIVE_NAME_TABLE[ (int)ClientData ] == 0 )
  {
    DrealSetNameVisible( ClientData );
  }

  rdsend();
}

/*------------------------------------------------------------\ 
|                                                             | 
|                     CallbackLayerInvisible                  | 
|                                                             | 
\------------------------------------------------------------*/ 
 
void CallbackLayerInvisible( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData; 
{
  char Layer;

  rdsbegin();

  Layer = DrealViewLayerTranslate[ (int)ClientData ];

  if ( DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] == 1 )
  { 
    DrealSetLayerInvisible( ClientData );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackNameInvisible                  |
|                                                             |
\------------------------------------------------------------*/

void CallbackNameInvisible( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  if ( DREAL_RDS_ACTIVE_NAME_TABLE[ (int)ClientData ] == 1 )
  {
    DrealSetNameInvisible( ClientData );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackLayerFillMode                 |
|                                                             |
\------------------------------------------------------------*/

void CallbackLayerFillMode( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DREAL_FILL_MODE = ( DREAL_FILL_MODE + 1 ) % 3;

  DrealChangeFillMode( DREAL_FILL_MODE );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackLayerInterface                |
|                                                             |
\------------------------------------------------------------*/

void CallbackLayerInterface( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DREAL_INSTANCE_INTERFACE = ! DREAL_INSTANCE_INTERFACE;
  DrealChangeInterface( DREAL_INSTANCE_INTERFACE );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackLayerApply                    |
|                                                             |
\------------------------------------------------------------*/

void CallbackLayerApply( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealZoomRefresh();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                       CallbackCloseLayer                    |
|                                                             |
\------------------------------------------------------------*/

void CallbackCloseLayer( MyWidget, ClientData, CallData )

     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealExitPanel( &DrealViewLayerPanel );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealChangeFillMode                    |
|                                                             |
\------------------------------------------------------------*/

void DrealChangeFillMode( FillMode )

  char FillMode;
{
  int      Index;
  XmString Label;

  rdsbegin();

  Index = 2 + ( ( DREAL_RDS_MAX_LAYER + DREAL_MAX_ACTIVE_NAME ) << 1 );
  Label = XmStringCreateSimple( DrealViewLayerFillModeName[ FillMode ] );

  XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON,
                 XmNlabelString, Label,
                 XmNresizable  , False,
                 NULL );

  XmStringFree( Label );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealChangeInterface                   |
|                                                             |
\------------------------------------------------------------*/

void DrealChangeInterface( Interface )

  char Interface;
{
  int      Index;
  XmString Label;

  rdsbegin();

  Index = 3 + ( ( DREAL_RDS_MAX_LAYER + DREAL_MAX_ACTIVE_NAME ) << 1 );
  Label = XmStringCreateSimple( DrealViewLayerInterfaceName[ Interface ] );

  XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON,
                 XmNlabelString, Label,
                 XmNresizable  , False,
                 NULL );

  XmStringFree( Label );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealSetLayerVisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealSetLayerVisible( Layer )

  int Layer;
{
  int   Index;
  char *Background;
  char *Foreground;

  rdsbegin();

  Index = 2 + ( Layer << 1 );
  Layer = DrealViewLayerTranslate[ Layer ];

  if ( DrealViewLayerButton[ Index ].BUTTON != NULL )
  {
    DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] = 1;

    XtVaSetValues( DrealViewLayerButton[ Index + 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNbackground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index + 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNforeground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    Background = *DrealViewLayerButton[ Index ].BACKGROUND;
    Foreground = *DrealViewLayerButton[ Index ].FOREGROUND;

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg            ,
                   XmNbackground           ,
                   XmRString               ,
                   Background              ,
                   strlen( Background ) + 1,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg            ,
                   XmNforeground           ,
                   XmRString               ,
                   Foreground              ,
                   strlen( Foreground ) + 1,
                   NULL
                 );
  }

  rdsend();
} 

/*------------------------------------------------------------\
|                                                             |
|                       DrealSetNameVisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealSetNameVisible( Name )

  int Name;
{
  int   Index;
  char *Background;
  char *Foreground;

  rdsbegin();

  Index = 2 + ( ( DREAL_RDS_MAX_LAYER + Name ) << 1 );

  if ( DrealViewLayerButton[ Index ].BUTTON != NULL )
  {
    DREAL_RDS_ACTIVE_NAME_TABLE[ Name ] = 1;

    XtVaSetValues( DrealViewLayerButton[ Index + 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNbackground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index + 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNforeground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    Background = *DrealViewLayerButton[ Index ].BACKGROUND;
    Foreground = *DrealViewLayerButton[ Index ].FOREGROUND;

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg            ,
                   XmNbackground           ,
                   XmRString               ,
                   Background              ,
                   strlen( Background ) + 1,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg            ,
                   XmNforeground           ,
                   XmRString               ,
                   Foreground              ,
                   strlen( Foreground ) + 1,
                   NULL
                 );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    DrealSetLayerInvisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealSetLayerInvisible( Layer )

  int Layer;
{
  int   Index;
  char *Background;
  char *Foreground;

  rdsbegin();

  Index = 3 + ( Layer << 1 );
  Layer = DrealViewLayerTranslate[ Layer ];

  if ( DrealViewLayerButton[ Index ].BUTTON != NULL )
  {
    DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] = 0;

    XtVaSetValues( DrealViewLayerButton[ Index - 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNbackground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index - 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNforeground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    Background = *DrealViewLayerButton[ Index ].BACKGROUND;
    Foreground = *DrealViewLayerButton[ Index ].FOREGROUND;

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg                          ,
                   XmNbackground                         ,
                   XmRString                             ,
                   Background                            ,
                   strlen( Background ) + 1              ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg                          ,
                   XmNforeground                         ,
                   XmRString                             ,
                   Foreground                            ,
                   strlen( Foreground ) + 1              ,
                   NULL
                 );
  } 

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                     DrealSetNameInvisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealSetNameInvisible( Name )

  int Name;
{
  int   Index;
  char *Background;
  char *Foreground;

  rdsbegin();

  Index = 3 + ( ( DREAL_RDS_MAX_LAYER + Name ) << 1 );

  if ( DrealViewLayerButton[ Index ].BUTTON != NULL )
  {
    DREAL_RDS_ACTIVE_NAME_TABLE[ Name ] = 0;

    XtVaSetValues( DrealViewLayerButton[ Index - 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNbackground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index - 1 ].BUTTON,
                   XtVaTypedArg                            ,
                   XmNforeground                           ,
                   XmRString                               ,
                   DREAL_BLACK_COLOR                       ,
                   strlen( DREAL_BLACK_COLOR ) + 1         ,
                   NULL
                 );

    Background = *DrealViewLayerButton[ Index ].BACKGROUND;
    Foreground = *DrealViewLayerButton[ Index ].FOREGROUND;

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg                          ,
                   XmNbackground                         ,
                   XmRString                             ,
                   Background                            ,
                   strlen( Background ) + 1              ,
                   NULL
                 );

    XtVaSetValues( DrealViewLayerButton[ Index ].BUTTON  ,
                   XtVaTypedArg                          ,
                   XmNforeground                         ,
                   XmRString                             ,
                   Foreground                            ,
                   strlen( Foreground ) + 1              ,
                   NULL
                 );
  } 

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      DrealLayerAllVisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealLayerAllVisible()

{
  int   Layer;

  rdsbegin();

  for ( Layer = DREAL_RDS_NWELL; Layer < DREAL_RDS_MAX_LAYER; Layer++ )
  {
    DrealSetLayerVisible( Layer );
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    DrealSetNameVisible( Layer );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    DrealInitializeLayer                     |
|                                                             |
\------------------------------------------------------------*/

void DrealInitializeLayer()

{
  int   Index;
  char  Layer;

  rdsbegin();

  for ( Index = DREAL_RDS_NWELL; Index < DREAL_RDS_MAX_LAYER; Index++ )
  {
    Layer = DrealViewLayerTranslate[ Index ];

    if ( DREAL_RDS_ACTIVE_LAYER_TABLE[ Layer ] == 0 )
    {
      DrealSetLayerInvisible( Index );
    }
    else
    {
      DrealSetLayerVisible( Index );
    }
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    if ( DREAL_RDS_ACTIVE_NAME_TABLE[ Layer ] == 0 )
    {
      DrealSetNameInvisible( Layer );
    }
    else
    {
      DrealSetNameVisible( Layer );
    }
  }

  DrealChangeFillMode( DREAL_FILL_MODE );
  DrealChangeInterface( DREAL_INSTANCE_INTERFACE );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                    DrealLayerAllInvisible                   |
|                                                             |
\------------------------------------------------------------*/

void DrealLayerAllInvisible()

{
  int   Layer;

  rdsbegin();

  for ( Layer = RDS_NWELL; Layer < DREAL_RDS_MAX_LAYER; Layer++ )
  {
    DrealSetLayerInvisible( Layer );
  }

  for ( Layer = 0; Layer < DREAL_MAX_ACTIVE_NAME; Layer++ )
  {
    DrealSetNameInvisible( Layer );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      Callback For Grid                      |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                      CallbackSetGridX                       |
|                                                             |
\------------------------------------------------------------*/
 
void CallbackSetGridX( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealEnterDialog( &DrealViewGridXDialog );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackSetGridY                       |
|                                                             |
\------------------------------------------------------------*/
 
void CallbackSetGridY( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealEnterDialog( &DrealViewGridYDialog );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackGridOnOff                      |
|                                                             |
\------------------------------------------------------------*/
 
void CallbackGridOnOff( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  XmString Label;

  rdsbegin();

  DrealLambdaUserGrid = ! DrealLambdaUserGrid;

  if ( DrealLambdaUserGrid == DREAL_FALSE )
  {  
    Label = XmStringCreateSimple( "Grid Off" );
  }  
  else
  {   
    Label = XmStringCreateSimple( "Grid On" );
  }  

  XtVaSetValues( MyWidget, XmNlabelString, Label,
                           XmNresizable  , False,
                           NULL );
  XmStringFree( Label );

  DrealZoomRefresh();

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      CallbackCloseGrid                      |
|                                                             |
\------------------------------------------------------------*/
 
void CallbackCloseGrid( MyWidget, ClientData, CallData )
 
     Widget  MyWidget;
     caddr_t ClientData;
     caddr_t CallData;
{
  rdsbegin();

  DrealExitPanel( &DrealViewGridPanel );

  rdsend();
}
