
## ************************ Read all the arguments ************************


## Check if there is at least one argument.
if ($#argv == 0) then 
	${ECHO} "usage : fpgen [-gv] program (without extension)";
	exit( 0 );
endif


## Defaults values
@   NAMEARG = 0
@   TALK    = 0
@   ERASE   = 1
@   EXECUTE = 1
@   FAILED  = 0
set INCDEV  = ""
set INCMAC  = ""
set LIBDEV  = ""
set LIBMAC  = ""

@ i = 1
while($i <= $#argv)
	switch(${argv[$i]})
		case -v:
@			TALK = 1
			breaksw
		case -g:
@			ERASE = 0
			breaksw
		case -gv:
		case -vg:
@			TALK  = 1
@			ERASE = 0
			breaksw
		case -E:
@			EXECUTE = 0
			breaksw
		case --dev:
set			INCDEV  = ${FITMK_FITIDP}
set			LIBDEV  = ${FITMK_FITLDP}
			breaksw
		case --mach:
set			INCMAC  = ${FITMK_FITIMP}
set			LIBMAC  = ${FITMK_FITLMP}
			breaksw
		default:
@			NAMEARG = ${i}
			break
	endsw

@	i += 1
end


## ************************ Check reads arguments *************************


# Check is the file name has been read.
if (${NAMEARG} == 0) then 
	${ECHO} "usage : fpgen [-gv] program (without extension)";
	exit( 1 );
endif


set FITMK_FPGICP = "${INCDEV} ${INCMAC} ${FITMK_ALCICP}"
set FITMK_FPGLBP = "${LIBDEV} ${LIBMAC} ${FITMK_ALCLBP}"

if (${INCDEV} != "") then
	${ECHO} "- INC = ${FITMK_FPGICP}.";
	${ECHO} "- LIB = ${FITMK_FPGLBP}.";
endif


${RM} ${argv[$NAMEARG]}.frr ${argv[$NAMEARG]}.o >& /dev/null

${ECHO} ""
${ECHO} "             @@@@@@@@@                @@@@ @                        "
${ECHO} "               @@    @              @@    @@                        "
${ECHO} "               @@     @            @@      @                        "
${ECHO} "               @@       @@@ @@@   @@             @@@@@   @@@ @@@    "
${ECHO} "               @@   @    @@@  @@  @@            @     @   @@@   @   "
${ECHO} "               @@@@@@    @@    @@ @@     @@@@@ @@     @@  @@    @@  "
${ECHO} "               @@   @    @@    @@ @@     @ @@  @@@@@@@@@  @@    @@  "
${ECHO} "               @@        @@    @@ @@    @  @@  @@         @@    @@  "
${ECHO} "               @@        @@    @@  @@      @@  @@      @  @@    @@  "
${ECHO} "               @@        @@@  @@    @@    @@    @@    @@  @@    @@  "
${ECHO} "             @@@@@@      @@ @@@       @@@@        @@@@   @@@@  @@@@ "
${ECHO} "                         @@                                         "
${ECHO} "                        @@@@                                        "
${ECHO} ""
${ECHO} "                           Fitpath Generation Language              "
${ECHO} ""
${ECHO} "                    Alliance CAD System 3.0,      fpgen 1.05        "
${ECHO} "                    Copyright (c) 93-95, MASI, CAO-VLSI Team        "
${ECHO} "                    E-mail support:     cao-vlsi@masi.ibp.fr        "
${ECHO} ""
${ECHO} ""


## ******************* Check MBK Environment Variables ********************


# Set all the MBK variables to their defaults values.
# (if they have not been sets before the call of this batch)
if (! ${?MBK_IN_LO})      setenv MBK_IN_LO      vst
if (! ${?MBK_OUT_LO})     setenv MBK_OUT_LO     vst
if (! ${?MBK_IN_PH})      setenv MBK_IN_PH      ap
if (! ${?MBK_OUT_PH})     setenv MBK_OUT_PH     ap
if (! ${?MBK_CATAL_NAME}) setenv MBK_CATAL_NAME CATAL
if (! ${?MBK_CATA_LIB})   setenv MBK_CATA_LIB   .
if (! ${?MBK_WORK_LIB})   setenv MBK_WORK_LIB   .

# Print current MBK environment
if (${TALK}) then
	${ECHO}    "Current MBK environment :"
	${ECHO}    "- MBK_IN_LO      : ${MBK_IN_LO}"
	${ECHO}    "- MBK_OUT_LO     : ${MBK_OUT_LO}"
	${ECHO}    "- MBK_IN_PH      : ${MBK_IN_PH}"
	${ECHO}    "- MBK_OUT_PH     : ${MBK_OUT_PH}"
	${ECHO}    "- MBK_WORK_LIB   : ${MBK_WORK_LIB}"
	${ECHO} -n "- MBK_CATA_LIB   : "
	${ECHO} ${MBK_CATA_LIB} |                                  \
		awk 'BEGIN { FS=":"; }                                 \
				   { for( i=1; i<=NF; i++)                     \
					 {                                         \
						printf("%s\n", $i);                    \
							if (i < NF)                        \
								printf("                   "); \
					  }                                        \
				   }' -
	${ECHO}    "- MBK_CATAL_NAME : ${MBK_CATAL_NAME}"
	${ECHO}    ""
	${ECHO}    ""
endif


## ********************** Compilation And Execution ***********************


if (${TALK} && ! ${FAILED}) then
	${ECHO} -n "            ";
	${ECHO} -n "Compiling ${argv[$NAMEARG]}.c:";
endif
${CC}  ${FITMK_FPGICP} -c -g -o ${argv[$NAMEARG]}.o \
				                ${argv[$NAMEARG]}.c >& ${argv[$NAMEARG]}.frr;
if (${status}) then
@	FAILED = 1

	${ECHO} "Compilation aborted.";
else
	if (${TALK}) ${ECHO} "Done."
endif


if (! ${FAILED}) then
	if (${TALK}) then
		${ECHO} -n "              ";
		${ECHO} -n "Linking ${argv[$NAMEARG]}.o with FpGen and GenLib:";
	endif
	${CC} -g -o ${argv[$NAMEARG]} \
				${argv[$NAMEARG]}.o  ${FITMK_FPGLBP} \
									 ${FITMK_FITLLS} \
        		        	         ${FITMK_ALCLBP} \
        		        	         ${FITMK_ALCGGN} \
        		        	         ${FITMK_ALCAPR} \
        		        	         ${FITMK_ALCMGN} \
        		        	         ${FITMK_ALCMLO} \
        		        	         ${FITMK_ALCMPH} \
        		        	         ${FITMK_ALCICN} \
								 >>& ${argv[$NAMEARG]}.frr;
	if (${status}) then
@		FAILED = 1

		${ECHO} "Link aborted.";
	else
		if (${TALK}) ${ECHO} "Done."
	endif
endif

${RM} -f ${argv[$NAMEARG]}.o >& /dev/null;


if (! ${FAILED}) then
	if (${EXECUTE}) then
		if (${TALK}) then
			${ECHO} -n "            "
			${ECHO}    "Executing ${argv[$NAMEARG]}.";
			${ECHO}    ""
			${ECHO}    ""
			${ECHO} -n "          "
			${ECHO}    "Output from ${argv[$NAMEARG]}:";
		endif
		${argv[$NAMEARG]} |& tee -a ${argv[$NAMEARG]}.frr;
@		EXIT_CODE = ${status}

		if (${EXIT_CODE}) then
			${ECHO}    ""
			${ECHO} -n "Abnormal termination, "
			${ECHO}    "program exit with code:${EXIT_CODE}.";
			${ECHO}    ""
			${ECHO}    ""

@			FAILED = 1
		else
			if (${TALK}) then
				${ECHO}    "Execution completed.";
				${ECHO}    ""
				${ECHO}    ""
			endif
		endif
	else
		if (${TALK}) then
			${ECHO} ""
			${ECHO} ""
			${ECHO} "Execution skipped.";
			${ECHO} ""
			${ECHO} ""
		endif
	endif
endif


if (${ERASE} && ${EXECUTE}) then
	if (${TALK}) then
		${ECHO} -n " Removing binary file ${argv[$NAMEARG]}:";
	endif
	${RM} -f ${argv[$NAMEARG]} >& /dev/null;
	if (${TALK}) ${ECHO} "Done."
endif

if (${FAILED}) then
	${ECHO} -n "     "
	${ECHO}    "Errors report in ${argv[$NAMEARG]}.frr file.";

	exit(1);
endif

if (${ERASE}) then
	if (${TALK}) ${ECHO} -n "Removing history file ${argv[$NAMEARG]}.frr:";
	${RM} -f ${argv[$NAMEARG]}.frr >& /dev/null;
	if (${TALK}) ${ECHO} "Done."
endif


exit(0);
