#include  <stdio.h>
#include  <string.h>
#include  <varargs.h>

#include   MUT_H
#include   MLO_H
#include   MPH_H
#include   MGN_H
#include   GFP_H
#include  "FpGenMBK.h"
#include  "FpGenMGN.h"


/*          Alliance CAD System 3.0                               */
/*             FpGen Package 1.05                                 */
/*                                                                */
/*  Copyright(c) 93-95, MASI, CAO-VLSI Team                       */
/*                                                                */
/*  Author      :          Jean-Paul CHAPUT                       */
/*  E-mail      :      cao-vlsi@masi.ibp.fr                       */
/* ************************************************************** */
/*  Module      :      "FpGenMGN.c"                               */




/*  Function  :  'FgnMGN_Trace'                                             */
/* ------------------------------------------------------------------------ */
/*  Activate or disable the trace mode.                                     */

extern void  FgnMGN_Trace( aFile, aLine, aMode )
  char *aFile;
  long  aLine;
  long  aMode;
{
  char *ModeName[2];


  ModeName[0] = "OFF";
  ModeName[1] = "ON";

  fflush( stdout );
  fflush( stderr );

  switch( aMode )
  {
    case C_FgnMGN_TraceModeOn:
    case C_FgnMGN_TraceModeOff:

      fprintf( stderr, "FpGen:DP_TRACE:" );
      fprintf( stderr, "Trace mode %s at line %ld of file \"%s\".\n",
                        ModeName[aMode], aLine, aFile );

      fpgen_Glob.TraceMode = aMode;
      break;

    default:

      fprintf( stderr, "*** FpGen Warning ***:DP_TRACE" );
      fprintf( stderr, "Invalid trace mode %ld at line %ld of file \"%s\".\n",
                       aMode, aLine, aFile );
      fprintf( stderr, "                      " );
      fprintf( stderr, "Trace mode desabled.\n" );

      fpgen_Glob.TraceMode = C_FgnMGN_TraceModeOff;
      break;
  }
}


/*  Function  :  'DP_DEFLOFIG'.                                             */
/* ======================================================================== */
/*  Open a new Data-Path logical figure.                                    */

extern void  DP_DEFLOFIG(aFigName, aMaxWidth, aMsb0)
  char *aFigName;  /* Name of the newly created logical figure. */
  long  aMaxWidth; /* Maximum width of the data-path buses.     */
  long  aMsb0;     /* Index zero for the MSB.                   */
{
  static char  FuncName[] = "DP_DEFLOFIG";

  /* Set name of current function. */
  fpgen_Glob.CurrFunc = FuncName;

  FgnMBK_TraceHeader();
  FgnMBK_TracePrintF( "%20s : DP_DEFLOFIG.\n", "Interface-Function" );


  /* Actualize the number of calls to "DP_DEFLOFIG". */
  fpgen_Glob.CallDefLo += 1;

  /* First call to "DP_DEFLOFIG". */
  if ( fpgen_Glob.CallDefLo == 1L )
  {
    /* Initialisation of FpGen and Ut modules. */
    UtBas_Init();
    UtEnv_Init();
    UtTec_Init( UtEnv_Environ.RDS_TECHNO_NAME );
    UtFlt_Init();
    UtXlg_Init();
    UtMLo_Init();
    UtMPh_Init();
    UtHir_Init();
    UtUsr_Init();
    FgnMBK_Init();
  }

  FgnMBK_TracePrintF( "%20s : ", "(char *)aFigName" );
  FgnMBK_TracePrintF( "(0x%08lX) ", aFigName );
  FgnMBK_TracePrintF(  "\"%s\".\n", aFigName );

  FgnMBK_TracePrintF( "%20s : ",        "(long)aMaxWidth" );
  FgnMBK_TracePrintF( "(0x--------)  %ld.\n", aMaxWidth  );

  FgnMBK_TracePrintF( "%20s : ", "(long)aMSB0" );
  FgnMBK_TracePrintF( "(0x--------)  %ld.\n", aMsb0 );


  /* Check the number of call to the function. */
  if ( fpgen_Glob.CallDefLo != fpgen_Glob.CallSavLo + 1 )
    GfpUT_PrintError( fpgen_Glob.ModuleId,
                      E_FgnMBK_DEFLOCall,
                      fpgen_Glob.CurrFunc,
                      aFigName );
  
  /* Check of argument "MaxWidth". */
  if (   ( aMaxWidth < 1                        )
      || ( aMaxWidth > C_GfpGN_LimitsMaxWidthDP ) )
    GfpUT_PrintError( fpgen_Glob.ModuleId,
                      E_FgnMBK_DEFLOWidthMax,
                      fpgen_Glob.CurrFunc,
                      aFigName ,
                      aMaxWidth,
                      C_GfpGN_LimitsMaxWidthDP );

  /* Check the parity of "MaxWidth". */
  if ( aMaxWidth % 2 )
    GfpUT_PrintError( fpgen_Glob.ModuleId,
                      E_FgnMBK_DEFLOWidthEven,
                      fpgen_Glob.CurrFunc,
                      aFigName,
                      aMaxWidth );

  /* Set global variable "WidthDP". */
  fpgen_Glob.WidthDP = aMaxWidth;

  /* Set global variable "Msb0". */
  switch( aMsb0 )
  {
    case LSB_INDEX_ZERO:
    case MSB_INDEX_ZERO:

      fpgen_Glob.Msb0 = aMsb0;
      break;

    default:

      GfpUT_PrintError( fpgen_Glob.ModuleId,
                        E_FgnMBK_DEFLOLsbPos,
                        fpgen_Glob.CurrFunc,
                        aFigName,
                        aMsb0 );
  }

  /* Set mode of CTRL terminals management.
   * (default mode : EXPLICIT)
   */
  fpgen_Glob.ConMode = C_FgnMGN_ConModeEXPLICIT;

  /* Set the current "lofig".
   * (copy of the GENLIB's WORK_LOFIG for checks)
   */
  DEF_LOFIG( aFigName );

  /* Perform additionnal MBK operations of DP_DEFLOFIG.  */
  FgnMBK_DefLoFig();

  FgnMBK_TraceFlush();
}


/*  Function  :  'DP_SAVLOFIG'.                                             */
/* ======================================================================== */
/*  Close and save on disk the previously generated Data-Path.              */

extern void  DP_SAVLOFIG()
{
  static char  FuncName[] = "DP_SAVLOFIG";

  /* Set name of current function. */
  fpgen_Glob.CurrFunc = FuncName;


  FgnMBK_TraceHeader();
  FgnMBK_TracePrintF( "%20s : DP_SAVELOFIG.\n", "Interface-Function" );

  /* Actualize the number of calls to "DP_SAVLOFIG". */
  fpgen_Glob.CallSavLo += 1;

  /* If the function has been call before "DP_DEFLOFIG"
   *    - Load 'FPGEN' specific error table.
   *    - Process error : "No previous call to "DP_DEFLOFIG".
   */
  if ( fpgen_Glob.CallDefLo == 0 )
  {
    FgnMBK_Init();
    GfpUT_PrintError( fpgen_Glob.ModuleId,
                      E_FgnMBK_SharedMissDefLo,
                      fpgen_Glob.CurrFunc );
  }

  /* Check the number of call to the function. */
  if ( fpgen_Glob.CallDefLo != fpgen_Glob.CallSavLo )
    GfpUT_PrintError( fpgen_Glob.ModuleId,
                      E_FgnMBK_SAVLOCall,
                       fpgen_Glob.CurrFunc );

  /* Perform MBK additionnal operations of DP_SAVLOFIG. */
  FgnMBK_SavLoFig();

  DP_FLATTEN();
  SAVE_LOFIG();

  FgnMBK_TraceFlush();
}


/*  Fonction  :  'DP_CONMODE'.                                    */
/* -------------------------------------------------------------- */
/*  Reset the management mode of CTRL terminals after the first
 *  call to "DP_DEFLOFIG".
 */

extern void  DP_CONMODE(aConMode)
	long  aConMode;
{
	static char  FuncName[] = "DP_CONMODE";

	/* Set the current function name. */
	fpgen_Glob.CurrFunc = FuncName;

	/* If the function has been call before "DP_DEFLOFIG"       */
	/*    - Load 'fpgen' specific error table.                  */
	/*    - Process error : "No previous call to "DP_DEFLOFIG". */
	if (fpgen_Glob.CallDefLo == 0)
	{
		FgnMBK_Init();
		GfpUT_PrintError(fpgen_Glob.ModuleId,
					E_FgnMBK_SharedMissDefLo,
						 fpgen_Glob.CurrFunc);
	}

	switch( aConMode )
	{
		case C_FgnMGN_ConModeDEFAULT  :
	 	case C_FgnMGN_ConModeEXPLICIT :
		case C_FgnMGN_ConModeIMPLICIT :
			fpgen_Glob.ConMode = aConMode;
			break;
		default :
			GfpUT_PrintError(fpgen_Glob.ModuleId,
						  E_FgnMBK_CONMODEUnknow,
							 fpgen_Glob.CurrFunc, aConMode);
	}
}


/*  Function  :  'DP_LOCON'.                                      */
/* -------------------------------------------------------------- */
/*  Add a new logical connector to the current figure.            */

extern void  DP_LOCON(aConName,
                      aConDir ,
					  aConSig )
	char *aConName;  /* Name of the connector (vectors allowed). */
	char  aConDir;   /* Direction of the connector.              */
	char *aConSig;   /* Signal associated to the connector.      */
{
	static char  FuncName[] = "DP_LOCON";


  FgnMBK_TraceHeader();
  FgnMBK_TracePrintF( "%20s : DP_LOCON.\n", "Interface-Function" );


	/* Set the current function name. */
	fpgen_Glob.CurrFunc = FuncName;

    /* If the function have been call before DP_DEFLO :      */
	/*     - Initialisation of error data base.              */
	/*     - Process error : "no previous call to DP_DEFLO". */
	if (fpgen_Glob.CallDefLo == 0)
	{
		FgnMBK_Init();
		GfpUT_PrintError(fpgen_Glob.ModuleId,
					E_FgnMBK_SharedMissDefLo,
						 fpgen_Glob.CurrFunc);
	}

    FgnMBK_TracePrintF( "%20s : ", "(char *)aCon" );
    FgnMBK_TracePrintF( "(0x%08lX) ", aConName );
    FgnMBK_TracePrintF(  "\"%s\".\n", aConName );

    FgnMBK_TracePrintF( "%20s : ", "(long)aDir" );
    FgnMBK_TracePrintF( "(0x--------) \'%c\'.\n", aConDir  );

    FgnMBK_TracePrintF( "%20s : ", "(char *)aSig" );
    FgnMBK_TracePrintF( "(0x%08lX) ", aConSig );
    FgnMBK_TracePrintF(  "\"%s\".\n", aConSig );

	/* Detect power supplies. */
	if (!FgnMBK_LoConPower(aConName))
	{
	    /* Encapsulated GENLIB LOCON function call. */
	    LOCON(aConName, aConDir, aConSig);
	}

  FgnMBK_TraceFlush();
}


/*  Fonction  :  'DP_IMPORT'.                                     */
/* -------------------------------------------------------------- */
/*  Instanciate a data-path or a column in the current data-path.
 *    It must have his data terminals build with virtual 
 *  connectors. Implicits terminals are automatically detect and
 *  added to the interface.
 */

extern void  DP_IMPORT(va_alist)
	va_dcl
{
	static       char  FuncName[] = "DP_IMPORT";

			  va_list  ArgList;

	/* Set name of current function. */
	fpgen_Glob.CurrFunc = FuncName;

    /* If the function have been call before DP_DEFLO :      */
	/*     - Initialisation of error data base.              */
	/*     - Process error : "no previous call to DP_DEFLO". */
	if (fpgen_Glob.CallDefLo == 0)
	{
		FgnMBK_Init();
		GfpUT_PrintError(fpgen_Glob.ModuleId,
					E_FgnMBK_SharedMissDefLo,
						 fpgen_Glob.CurrFunc);
	}

	/* Get the list of variable arguments. */
	va_start( ArgList);

	/* Call to the importation function. */
	FgnMBK_Import( ArgList);

	/* Clean the list of variable arguments. */
	va_end( ArgList);
}


/*  Function  :  'DP_FLATTEN'.                                    */
/* -------------------------------------------------------------- */
/*  Flatten the current logical figure to a normalized 2 levels
 *  netlist. It is just an encapsulation of the Data-Path UTili-
 *  ties function 'UtHir_HierarchFlatten'.
 */

extern void  DP_FLATTEN()
{ 
	static char          FuncName[] = "DP_FLATTEN";

	/* Set the current function name. */
	fpgen_Glob.CurrFunc = FuncName;

    /* If the function have been call before DP_DEFLO :      */
	/*     - Initialisation of error data base.              */
	/*     - Process error : "no previous call to DP_DEFLO". */
	if (fpgen_Glob.CallDefLo == 0)
	{
		FgnMBK_Init();
		GfpUT_PrintError(fpgen_Glob.ModuleId,
					E_FgnMBK_SharedMissDefLo,
						 fpgen_Glob.CurrFunc);
	}

	/* Call to flatten function. */
	FgnMBK_Flatten();
}


/*  Function  :  'DP_INV'.                                        */
/* -------------------------------------------------------------- */
/*  Inverter/Driver generator.                                    */

extern void  DP_INV(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionInv, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_BUFF'.                                       */
/* -------------------------------------------------------------- */
/*  Inverter/Driver generator.                                    */

extern void  DP_BUFF(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionBuff, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_AND2'.                                       */
/* -------------------------------------------------------------- */
/*  And2 generator.                                               */

extern void  DP_AND2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionAnd2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_AND3'.                                       */
/* -------------------------------------------------------------- */
/*  And3 generator.                                               */

extern void  DP_AND3(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionAnd3, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NAND2'.                                      */
/* -------------------------------------------------------------- */
/*  Nand2 generator.                                              */

extern void  DP_NAND2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNand2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_NAND3'.                                      */
/* -------------------------------------------------------------- */
/*  Nand3 generator.                                              */

extern void  DP_NAND3(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNand3, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_OR2'.                                        */
/* -------------------------------------------------------------- */
/*  Or2 generator.                                                */

extern void  DP_OR2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionOr2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_OR3'.                                        */
/* -------------------------------------------------------------- */
/*  Or3 generator.                                                */

extern void  DP_OR3(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionOr3, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NOR2'.                                       */
/* -------------------------------------------------------------- */
/*  Nor2 generator.                                               */

extern void  DP_NOR2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNor2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NOR3'.                                       */
/* -------------------------------------------------------------- */
/*  Nor3 generator.                                               */

extern void  DP_NOR3(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNor3, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_XOR2'.                                       */
/* -------------------------------------------------------------- */
/*  Xor2 generator.                                               */

extern void  DP_XOR2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionXor2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_XNOR2'.                                      */
/* -------------------------------------------------------------- */
/*  Xnor2 generator.                                              */

extern void  DP_XNOR2(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic boolean operators generator. */
	FgnMBK_GenericMF( C_FgnMBK_FunctionXnor2, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NOR2MASK'.                                   */
/* -------------------------------------------------------------- */

extern void  DP_NOR2MASK(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNor2mask, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_XNOR2MASK'.                                  */
/* -------------------------------------------------------------- */

extern void  DP_XNOR2MASK(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionXnor2mask, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NAND2MASK'.                                  */
/* -------------------------------------------------------------- */

extern void  DP_NAND2MASK(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNand2mask, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NMUX2CS'.                                    */
/* -------------------------------------------------------------- */

extern void  DP_NMUX2CS(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic macro-function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNmux2cs, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_MUX2CS'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_MUX2CS(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic macro-function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionMux2cs, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_MUX3CD'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_MUX3CD(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic macro-function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionMux3cd, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_MUX3CS'.                                               */
/* ------------------------------------------------------------------------ */

extern void  DP_MUX3CS( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic macro-function generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionMux3cs, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_MUX4CS'.                                               */
/* ------------------------------------------------------------------------ */

extern void  DP_MUX4CS( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic macro-function generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionMux4cs, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_MUX5CS'.                                               */
/* ------------------------------------------------------------------------ */

extern void  DP_MUX5CS( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic macro-function generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionMux5cs, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_MUX6CS'.                                               */
/* ------------------------------------------------------------------------ */

extern void  DP_MUX6CS( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic macro-function generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionMux6cs, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_BUSE'.                                       */
/* -------------------------------------------------------------- */
/*  Non inverter tristate generator.                              */

extern void  DP_BUSE(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionBuse, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_NBUSE'.                                      */
/* -------------------------------------------------------------- */
/*  Non inverter tristate generator.                              */

extern void  DP_NBUSE(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNbuse, ArgList);

	/* Clean the list of variable arguments. */
	va_end(ArgList);
}


/*  Function  :  'DP_ADD2S'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_ADD2S(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionAdd2s, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_ADD2F'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_ADD2F(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionAdd2f, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_ADSB2F'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_ADSB2F(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionAdsb2f, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_SHIFT'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_SHIFT(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionShift, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_CONST'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_CONST(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Nacro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionConst, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NUL'.                                        */
/* -------------------------------------------------------------- */

extern void  DP_NUL(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNul, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PLATCH1'.                                    */
/* -------------------------------------------------------------- */

extern void  DP_PLATCH1(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPl1, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_NPLATCH1'.                                   */
/* -------------------------------------------------------------- */

extern void  DP_NPLATCH1(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionNpl1, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMSNW'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_PMSNW(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPmsnw, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMSNWQ'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_PMSNWQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPmsnwq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMS'.                                        */
/* -------------------------------------------------------------- */

extern void  DP_PMS(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPms, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMSQ'.                                       */
/* -------------------------------------------------------------- */

extern void  DP_PMSQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPmsq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMST'.                                       */
/* -------------------------------------------------------------- */

extern void  DP_PMST(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPmst, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PMSTQ'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_PMSTQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPmstq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PDFFNW'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_PDFFNW(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPdffnw, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PDFFNWQ'.                                    */
/* -------------------------------------------------------------- */

extern void  DP_PDFFNWQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPdffnwq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PDFF'.                                                 */
/* ------------------------------------------------------------------------ */

extern void  DP_PDFF( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic register file generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionPdff, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_PDFFR'.                                                */
/* ------------------------------------------------------------------------ */

extern void  DP_PDFFR( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic register file generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionPdffr, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_PDFFQ'.                                      */
/* -------------------------------------------------------------- */

extern void  DP_PDFFQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPdffq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_PDFFT'.                                                */
/* ------------------------------------------------------------------------ */

extern void  DP_PDFFT(va_alist)
  va_dcl
{
  va_list  ArgList;

  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic register file generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionPdfft, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_PDFFRT'.                                               */
/* ------------------------------------------------------------------------ */

extern void  DP_PDFFRT( va_alist )
  va_dcl
{
  va_list  ArgList;


  /* Get the list of variable arguments. */
  va_start( ArgList );

  /* Call the generic register file generator. */
  FgnMBK_GenericMF( C_FgnMBK_FunctionPdffrt, ArgList );

  /* Clean the list of variable arguments. */
  va_end( ArgList );
}


/*  Function  :  'DP_PDFFTQ'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_PDFFTQ(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic register file generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionPdfftq, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG1C0'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG1C0(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg1c0, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG1CC'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG1CC(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg1cc, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG2CC'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG2CC(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg2cc, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG2C0'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG2C0(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg2c0, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG1F0'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG1F0(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg1f0, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG1FC'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG1FC(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg1fc, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG2FC'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG2FC(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg2fc, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}


/*  Function  :  'DP_RFG2F0'.                                     */
/* -------------------------------------------------------------- */

extern void  DP_RFG2F0(va_alist)
	va_dcl
{
	va_list  ArgList;

	/* Get the list of variable arguments. */
	va_start(ArgList);

	/* Call the generic Macro-Function generator. */
	FgnMBK_GenericMF(C_FgnMBK_FunctionRfg2f0, ArgList);

	/* Clean the list of variable arguments. */
	va_end(  ArgList);
}
