#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>

#include   GFP_H


/*          Alliance CAD System 3.0                                         */
/*             FpGen Package 1.05                                           */
/*                                                                          */
/*  Copyright(c) 93-95, MASI, CAO-VLSI Team                                 */
/*                                                                          */
/*  Author      :          Jean-Paul CHAPUT                                 */
/*  E-mail      :      cao-vlsi@masi.ibp.fr                                 */
/* ************************************************************************ */
/*  Module      :      "GenericGNg.c"                                       */


/* Internal constants of module. */
#define       IS_GenGNg_ArgLc_PROG         "prog"
#define       IS_GenGNg_ArgLc_FUNCT        "funct"
#define       IS_GenGNg_ArgLc_DPWTH        "dpwth"
#define       IS_GenGNg_ArgLc_WIDTH        "width"
#define       IS_GenGNg_ArgLc_SLICE        "slice"
#define       IS_GenGNg_ArgLc_DRIVE        "drive"
#define       IS_GenGNg_ArgLc_MASK         "mask"
#define       IS_GenGNg_ArgLc_MSB0         "-msb0"
#define       IS_GenGNg_ArgLc_VHDL         "-vhdl"
#define       IS_GenGNg_ArgLc_ICON         "-icon"
#define       IS_GenGNg_ArgLc_LAYOUT       "-layout"
#define       IS_GenGNg_ArgLc_NETLIST      "-netlist"
#define       IS_GenGNg_ArgLc_POWER        "--merge-power"
#define       IS_GenGNg_ArgLc_NAME         "-o"

#define       IC_GenGNg_ModeLinked          0
#define       IC_GenGNg_ModeGeneric         1


/* Error constants of module. */
#define       IE_GenGNg_ArgLc_PROG          1
#define       IE_GenGNg_ArgLc_FUNCT         2
#define       IE_GenGNg_ArgLc_DPWTH         3
#define       IE_GenGNg_ArgLc_WIDTH         4
#define       IE_GenGNg_ArgLc_SLICE         5
#define       IE_GenGNg_ArgLc_DRIVE         6
#define       IE_GenGNg_ArgLc_MASK          7
#define       IE_GenGNg_ArgLc_NAME          8
#define       IE_GenGNg_ArgLc_NOVIEW        9
#define       IE_GenGNg_ArgLc_UNKNOW       10


/*  Function  :  'main'                                           */
/* -------------------------------------------------------------- */

int main(argc,argv)
   int  argc;
  char *argv[];
{
  long  ModuleId;
  long  Mode;
  char *Prog;
  char *ModelName = (char*)NULL;
  long  Function  = C_GfpGN_FunctionUnknow;
  long  WidthDP;
  long  WidthOp;
  long  Slice;
  long  Drive;
  long  *XlMask   = (long*)NULL;
  long  FlagMSB0;
  long  FlagVHDL;
  long  FlagNetlist;
  long  FlagLayout;
  long  FlagIcon;
  long  FlagPower;
  char *TmpPt;
  long  ArgPos;
  long  ArgCount;


  /* Initialize the utilities module. */
  UtBas_Init();
  UtUsr_Init();
  UtEnv_Init();
  UtTec_Init( UtEnv_Environ.RDS_TECHNO_NAME );
  UtXlg_Init();
  UtFlt_Init();
  UtMLo_Init();
  UtMPh_Init();
  UtHir_Init();

  /* Attribution of a module identifier. */
  ModuleId = GfpUT_GetModuleId("GenericGNg", F_GfpUT_ModuleAdd);

  /* Errors associated to a missing argument. */
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_PROG,
        "Bad executable name:\"%s\".");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_FUNCT,
        "Missing argument:\"%s\" (operator to generate).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_DPWTH,
        "Missing argument:\"%s\" (data-path bus wide).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_WIDTH,
        "Missing argument:\"%s\" (operator bus wide).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_SLICE,
        "Missing argument:\"%s\" (LSB startind slice).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_DRIVE,
        "Missing argument:\"%s\" (output drive power).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_MASK,
        "Missing argument:\"%s\" (mask to apply).");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_NOVIEW,
        "No view to generate (use \"-vhdl\", \"-netlist\" or \"-layout\").");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_UNKNOW,
        "Invalid argument:\"%s\".");
  GfpUT_AddError(ModuleId, IE_GenGNg_ArgLc_NAME,
        "Missing model name.");


  /* Transmit the command line informations. */
  GfpUT_InitArgLc(argc, argv);

  /* Read all the floating arguments (switchs). */
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_MSB0   , &FlagMSB0   );
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_VHDL   , &FlagVHDL   );
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_NETLIST, &FlagNetlist);
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_LAYOUT , &FlagLayout );
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_ICON   , &FlagIcon   );
  (void)GfpUT_GetArgLcByName(IS_GenGNg_ArgLc_POWER  , &FlagPower  );


  /* Read the executable name. */
  Prog = GfpUT_GetArgLcByPos(0, &ArgCount);

  if (!strcmp(Prog,"GenericGNg"))
  {
    /* Behave as the generic generator. */
    Mode = IC_GenGNg_ModeGeneric;
    
    Prog = GfpUT_GetArgLcByPos(1, &ArgCount);
    if (ArgCount != 1)
      GfpUT_PrintError(ModuleId,
                       IE_GenGNg_ArgLc_FUNCT,
                       IS_GenGNg_ArgLc_FUNCT);
  }
  else
    /* Behave like a specific generator. */
    Mode = IC_GenGNg_ModeLinked;

  /* Check the executable name. */
  if (!strcmp(Prog,  "invg"    )) Function = C_GfpGN_FunctionInv;
  if (!strcmp(Prog, "buffg"    )) Function = C_GfpGN_FunctionBuff;
  if (!strcmp(Prog,"nand3g"    )) Function = C_GfpGN_FunctionNand3;
  if (!strcmp(Prog,"nand2g"    )) Function = C_GfpGN_FunctionNand2;
  if (!strcmp(Prog, "and3g"    )) Function = C_GfpGN_FunctionAnd3;
  if (!strcmp(Prog, "and2g"    )) Function = C_GfpGN_FunctionAnd2;
  if (!strcmp(Prog, "nor3g"    )) Function = C_GfpGN_FunctionNor3;
  if (!strcmp(Prog, "nor2g"    )) Function = C_GfpGN_FunctionNor2;
  if (!strcmp(Prog,  "or3g"    )) Function = C_GfpGN_FunctionOr3;
  if (!strcmp(Prog,  "or2g"    )) Function = C_GfpGN_FunctionOr2;
  if (!strcmp(Prog,"xnor2g"    )) Function = C_GfpGN_FunctionXnor2;
  if (!strcmp(Prog, "xor2g"    )) Function = C_GfpGN_FunctionXor2;
  if (!strcmp(Prog,"nand2maskg")) Function = C_GfpGN_FunctionNand2mask;
  if (!strcmp(Prog, "nor2maskg")) Function = C_GfpGN_FunctionNor2mask;
  if (!strcmp(Prog,"xnor2maskg")) Function = C_GfpGN_FunctionXnor2mask;
  if (!strcmp(Prog,"constg"    )) Function = C_GfpGN_FunctionConst;
  if (!strcmp(Prog,"nbuseg"    )) Function = C_GfpGN_FunctionNbuse;
  if (!strcmp(Prog, "buseg"    )) Function = C_GfpGN_FunctionBuse;
  if (!strcmp(Prog,"nmux2csg"  )) Function = C_GfpGN_FunctionNmux2cs;
  if (!strcmp(Prog, "mux2csg"  )) Function = C_GfpGN_FunctionMux2cs;
  if (!strcmp(Prog, "mux3cdg"  )) Function = C_GfpGN_FunctionMux3cd;
  if (!strcmp(Prog, "mux3csg"  )) Function = C_GfpGN_FunctionMux3cs;
  if (!strcmp(Prog, "mux4csg"  )) Function = C_GfpGN_FunctionMux4cs;
  if (!strcmp(Prog, "mux5csg"  )) Function = C_GfpGN_FunctionMux5cs;
  if (!strcmp(Prog, "mux6csg"  )) Function = C_GfpGN_FunctionMux6cs;
  if (!strcmp(Prog, "add2sg"   )) Function = C_GfpGN_FunctionAdd2s;
  if (!strcmp(Prog, "nulg"     )) Function = C_GfpGN_FunctionNul;
  if (!strcmp(Prog, "npl1g"    )) Function = C_GfpGN_FunctionNpl1;
  if (!strcmp(Prog,  "pl1g"    )) Function = C_GfpGN_FunctionPl1;
  if (!strcmp(Prog,  "pmsnwg"  )) Function = C_GfpGN_FunctionPmsnw;
  if (!strcmp(Prog,  "pmsnwqg" )) Function = C_GfpGN_FunctionPmsnwq;
  if (!strcmp(Prog,  "pmsg"    )) Function = C_GfpGN_FunctionPms;
  if (!strcmp(Prog,  "pmsqg"   )) Function = C_GfpGN_FunctionPmsq;
  if (!strcmp(Prog,  "pmstg"   )) Function = C_GfpGN_FunctionPmst;
  if (!strcmp(Prog,  "pmstqg"  )) Function = C_GfpGN_FunctionPmstq;
  if (!strcmp(Prog, "pdffnwg"  )) Function = C_GfpGN_FunctionPdffnw;
  if (!strcmp(Prog, "pdffnwqg" )) Function = C_GfpGN_FunctionPdffnwq;
  if (!strcmp(Prog, "pdffg"    )) Function = C_GfpGN_FunctionPdff;
  if (!strcmp(Prog, "pdffrg"   )) Function = C_GfpGN_FunctionPdffr;
  if (!strcmp(Prog, "pdffqg"   )) Function = C_GfpGN_FunctionPdffq;
  if (!strcmp(Prog, "pdfftg"   )) Function = C_GfpGN_FunctionPdfft;
  if (!strcmp(Prog, "pdffrtg"  )) Function = C_GfpGN_FunctionPdffrt;
  if (!strcmp(Prog, "pdfftqg"  )) Function = C_GfpGN_FunctionPdfftq;

  if (Function == C_GfpGN_FunctionUnknow)
    GfpUT_PrintError(ModuleId, IE_GenGNg_ArgLc_PROG, Prog);


  /* Read the standart fixed arguments (value). */

  /* Check if the data-path bus wide argument has been read. */
  TmpPt = GfpUT_GetArgLcByPos(1 + Mode, &ArgCount);
  if (ArgCount != 1)
    GfpUT_PrintError(ModuleId,
                     IE_GenGNg_ArgLc_DPWTH,
                     IS_GenGNg_ArgLc_DPWTH);
  else
    WidthDP = atol(TmpPt);

  /* Check if the operator bus wide argument has been read. */
  TmpPt = GfpUT_GetArgLcByPos(2 + Mode, &ArgCount);
  if (ArgCount != 1)
    GfpUT_PrintError(ModuleId,
                     IE_GenGNg_ArgLc_WIDTH,
                     IS_GenGNg_ArgLc_WIDTH);
  else
    WidthOp = atol(TmpPt);

  /* Check if the LSB starting slice argument has been read. */
  TmpPt = GfpUT_GetArgLcByPos(3 + Mode, &ArgCount);
  if (ArgCount != 1)
    GfpUT_PrintError(ModuleId,
                     IE_GenGNg_ArgLc_SLICE,
                     IS_GenGNg_ArgLc_SLICE);
  else
    Slice = atol(TmpPt);


  switch(Function)
  {
    case C_GfpGN_FunctionInv:
    case C_GfpGN_FunctionBuff:
    case C_GfpGN_FunctionNand3:
    case C_GfpGN_FunctionNand2:
    case C_GfpGN_FunctionAnd3:
    case C_GfpGN_FunctionAnd2:
    case C_GfpGN_FunctionNor3:
    case C_GfpGN_FunctionNor2:
    case C_GfpGN_FunctionOr3:
    case C_GfpGN_FunctionOr2:
      /* Check if the output drive power argument has been read. */
      TmpPt = GfpUT_GetArgLcByPos(4+Mode, &ArgCount);
      if (ArgCount != 1)
        GfpUT_PrintError(ModuleId,
                         IE_GenGNg_ArgLc_DRIVE,
                         IS_GenGNg_ArgLc_DRIVE);
      else
        Drive = atol(TmpPt);
      break;
    case C_GfpGN_FunctionXor2:
    case C_GfpGN_FunctionXnor2:
      Drive = 1;
      break;
    case C_GfpGN_FunctionNand2mask:
    case C_GfpGN_FunctionNor2mask:
    case C_GfpGN_FunctionXnor2mask:
    case C_GfpGN_FunctionConst:
      /* Check if the mask/constant argument has been read. */
      TmpPt = GfpUT_GetArgLcByPos(4+Mode, &ArgCount);
      if (ArgCount != 1)
        GfpUT_PrintError(ModuleId,
                         IE_GenGNg_ArgLc_MASK,
                         IS_GenGNg_ArgLc_MASK);
      else
      {
        /* Check the syntax of the string. */
        (void)GfpUT_XlStrCheck(TmpPt);

        XlMask = GfpUT_XlAlloc();

        /* Convert the string in an extended long. */
        GfpUT_XlStrToXl(TmpPt, XlMask);
      }
      break;
    case C_GfpGN_FunctionNbuse:
    case C_GfpGN_FunctionBuse:
    case C_GfpGN_FunctionNmux2cs:
    case C_GfpGN_FunctionMux2cs:
    case C_GfpGN_FunctionMux3cd:
    case C_GfpGN_FunctionMux3cs:
    case C_GfpGN_FunctionMux4cs:
    case C_GfpGN_FunctionMux5cs:
    case C_GfpGN_FunctionMux6cs:
    case C_GfpGN_FunctionAdd2s:
    case C_GfpGN_FunctionNul:
    case C_GfpGN_FunctionNpl1:
    case C_GfpGN_FunctionPl1:
    case C_GfpGN_FunctionPmsnw:
    case C_GfpGN_FunctionPmsnwq:
    case C_GfpGN_FunctionPms:
    case C_GfpGN_FunctionPmsq:
    case C_GfpGN_FunctionPmst:
    case C_GfpGN_FunctionPmstq:
    case C_GfpGN_FunctionPdffnw:
    case C_GfpGN_FunctionPdffnwq:
    case C_GfpGN_FunctionPdff:
    case C_GfpGN_FunctionPdffr:
    case C_GfpGN_FunctionPdffq:
    case C_GfpGN_FunctionPdfft:
    case C_GfpGN_FunctionPdffrt:
    case C_GfpGN_FunctionPdfftq:
      break;
  }

  /* Check if at least one view is asked. */
  if (   (!FlagVHDL   )
    && (!FlagNetlist)
    && (!FlagLayout )
    && (!FlagIcon   ))
    GfpUT_PrintError(ModuleId, IE_GenGNg_ArgLc_NOVIEW);


  /* Try to read the model name. */
  if ((ArgPos = GfpUT_GetArgLcByName("-o", &ArgCount))
                             != C_GfpUT_ArgLcPosUnknow)
  {
    ModelName = GfpUT_GetArgLcByPos(ArgPos + 1, &ArgCount);

    if (ArgCount != 1)
      GfpUT_PrintError(ModuleId, IE_GenGNg_ArgLc_NAME);
  }


  /* Look for unexamined arguments. */
  if ((TmpPt=GfpUT_GetArgLcRemain()) != (char*)NULL)
    GfpUT_PrintError(ModuleId, IE_GenGNg_ArgLc_UNKNOW, TmpPt);    


  switch(Function)
  {
    case C_GfpGN_FunctionInv:
    case C_GfpGN_FunctionBuff:
    case C_GfpGN_FunctionNand3:
    case C_GfpGN_FunctionNand2:
    case C_GfpGN_FunctionAnd3:
    case C_GfpGN_FunctionAnd2:
    case C_GfpGN_FunctionNor3:
    case C_GfpGN_FunctionNor2:
    case C_GfpGN_FunctionOr3:
    case C_GfpGN_FunctionOr2:
    case C_GfpGN_FunctionXor2:
    case C_GfpGN_FunctionXnor2:
      GfpGN_GenericDispatch(Function   ,
                            ModelName  ,
                            WidthDP    ,
                            WidthOp    ,
                            Slice      ,
                            Drive      ,
                            FlagMSB0   ,
                            FlagVHDL   ,
                            FlagNetlist,
                            FlagLayout ,
                            FlagIcon   ,
                            FlagPower  );
      break;
    case C_GfpGN_FunctionNand2mask:
    case C_GfpGN_FunctionNor2mask:
    case C_GfpGN_FunctionXnor2mask:
    case C_GfpGN_FunctionConst:
      GfpGN_GenericDispatch(Function   ,
                            ModelName  ,
                            WidthDP    ,
                            WidthOp    ,
                            Slice      ,
                            XlMask     ,
                            FlagMSB0   ,
                            FlagVHDL   ,
                            FlagNetlist,
                            FlagLayout ,
                            FlagIcon   ,
                            FlagPower  );
      break;
    case C_GfpGN_FunctionNbuse:
    case C_GfpGN_FunctionBuse:
    case C_GfpGN_FunctionNmux2cs:
    case C_GfpGN_FunctionMux2cs:
    case C_GfpGN_FunctionMux3cd:
    case C_GfpGN_FunctionMux3cs:
    case C_GfpGN_FunctionMux4cs:
    case C_GfpGN_FunctionMux5cs:
    case C_GfpGN_FunctionMux6cs:
    case C_GfpGN_FunctionAdd2s:
    case C_GfpGN_FunctionNul:
    case C_GfpGN_FunctionNpl1:
    case C_GfpGN_FunctionPl1:
    case C_GfpGN_FunctionPmsnw:
    case C_GfpGN_FunctionPmsnwq:
    case C_GfpGN_FunctionPms:
    case C_GfpGN_FunctionPmsq:
    case C_GfpGN_FunctionPmst:
    case C_GfpGN_FunctionPmstq:
    case C_GfpGN_FunctionPdffnw:
    case C_GfpGN_FunctionPdffnwq:
    case C_GfpGN_FunctionPdff:
    case C_GfpGN_FunctionPdffr:
    case C_GfpGN_FunctionPdffq:
    case C_GfpGN_FunctionPdfft:
    case C_GfpGN_FunctionPdffrt:
    case C_GfpGN_FunctionPdfftq:
      GfpGN_GenericDispatch(Function   ,
                            ModelName  ,
                            WidthDP    ,
                            WidthOp    ,
                            Slice      ,
                            FlagMSB0   ,
                            FlagVHDL   ,
                            FlagNetlist,
                            FlagLayout ,
                            FlagIcon   ,
                            FlagPower  );
      break;
  }

  /* Free the Mask, if it has been used. */
  if (XlMask != (long*)NULL) GfpUT_XlFree(XlMask);

  exit(0);
}
