/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  synthese FPGA                                              */
/*    Fichier :  fpga.c                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  Eudes Prado Lopes                       le : 20 /10/1993 */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include LOG_H
#include BEH_H
#include MLO_H
#include "../mapping/types.h"
#include "bool2xil.h"
#include "bool2act.h"
#include "../compil/lax_param.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "fpga.h"
#include "fp_type.h"
#include "mapp3000.h"



/*======================================================================/
/		                MAIN					/
/======================================================================*/

main(argc,argv)
int argc ;
char *argv[];
{
int trace;
lofig_list *fig;
beout_list *out;
bebus_list *bus;

mbkenv();
VHB_HEDFIG = NULL;
fp_sys = initializeFPSystem();
fpMajSystem(fp_sys);

                /* verification de la validite des arguments */
 
if (!verifArgsFPGA(fp_sys,argc,argv)) 
   {
   exit(-1);
   }

initializeBdd(1);

                /* initialisation selon le mode (batch ou non) */
 
if (!strcmp(argv[1],"-o") || !strcmp(argv[1],"-l"))
   {
   int vhb_reorder = 0;
 
                /* initialisation du systeme */
 
   fpAddInfoSystem(fp_sys,1,argc,argv,"1.01");

   if (FP_BANN)
      displayFpgaSystem(fp_sys);

   trace = 1;
   if (FP_TRACE == 2)
      trace = 2;
 
   if (FP_OPTION == 'l')
      FP_AUX = 1;
 
		/* compilation de la befig */

   if (FP_MAKEORDER >= 1)
      vhb_reorder = 2;
 
   fp_sys->orderPI = 
       (chain_list *) compileVHDL(FP_VHDL,FP_ORDERPI,FP_ORDER,FP_PAR,
                                  FP_AUX,1,vhb_reorder,1,trace,nameInt,NULL); 

   if (FP_ORDERPI == NULL)
      {
      printf("*** Compilation aborted...\n");
      exit(-1);
      }
 
  /*printf("********************** BUS Mapping ********************  \n");*/

   bus = VHB_HEDFIG->BEBUS;
   while (bus) 
   {
    printf("ERROR :%s is a bus and buses are not allowed yet \n",bus->NAME);
#ifdef LAX
            return 0;
#else
            exit(-1) ;
#endif 
    bus = bus->NEXT;
   }


   FP_BEH = VHB_HEDFIG;
   
                /* verification de la coherence */
 
   verifCohBeh(FP_BEH,FP_DELAYPI,FP_OPTIMPO);



  
   if (FP_OPTION == 'l')	/* mode local */
      FP_AUX = 1;

   if (FP_TRACE)
      displayBeh(FP_BEH,0);

	/* Graph Display */
    
   if (FP_TRACE == 3)
   {
   beout_list *out = (FP_BEH)->BEOUT;
   printf(" vai na displayMultiGraph \n");
   while (out)
	{
	displayGraphicBdd(out->NODE);
	out = out->NEXT;
	}

   }

if (FP_TRACE)
{
bereg_list *reg;
biabl_list *biabl;
printf("Debug  des registres \n");
reg = (FP_BEH)->BEREG;
while (reg)
   {
   printf("reg->NAME is %s \n", reg->NAME);
   biabl = reg->BIABL;
   printf("biabl->CNDABL = ");
   while(biabl)
     {
     displayExpr(biabl->CNDABL);
     biabl = biabl->NEXT;
     }
   printf("\n");
   printf("biabl->VALABL = ");
   while(biabl)
     {
     displayExpr(biabl->VALABL);
     biabl = biabl->NEXT;
     }
   printf("\n");

   reg = reg->NEXT;
   }
}

   fig = (lofig_list *) mappingFpga();
   if (FP_TRACE >= 2)
      displayMultiGraph(FP_BEH);

   }
else
   {
                /* version interactive */
 
   if (argc == 2)
      {
      fpAddInfoSystem(fp_sys,0,argc,argv,"FPGA Synthesis - V 1.01");
      }
   else 
      fpAddInfoSystem(fp_sys,2,argc,argv,"FPGA Synthesis - V 1.01");
 
   BVL_LINNUM = 0;

   FPPRINCIPAL();
   }
printf("\n");
exit(0);
}
