/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :synthetiseur de FPGA					    */
/*    Fichier :annealing                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :Burgun L.                              le : 03/04/1992     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H
#include <math.h>
#include "../mapping/types.h"
#include "../compil/lax_param.h"
#include "fp_type.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "./fpga.h"

 
/*-------------------------------------------------------------------------
localOptFPGA    : optimisation locale d'une befig
---------------------------------------------------------------------------
retour          : un void.
---------------------------------------------------------------------------*/
void localOptFPGA(trace)
int trace;
{
beaux_list *aux;
chain_list *saveAux = NULL;
int numberNode;
befig_list *beh;
chain_list *saveaux = NULL;
int numCLB,depthCLB;
chain_list *orderPI;
char *carint;
befig_list *CBeh;
int numCLBC,depthCLBC;
 
beh = FP_BEH;

			/* recopie de FP_BEH dans CBeh */

CBeh = (befig_list *) copyBeh(beh);
CBeh->CIRCUI =  initializeCct("figure",(beh->CIRCUI)->countI,50);
cpOrderCct(beh->CIRCUI,CBeh->CIRCUI);

		/* on elimine les auxiliaires trop petits */

/* compactBeh(CBeh,FP_GUARDEDAUX);*/

                     /* sauvegarde du fichier */

if (FP_TRACE)
   {
   carint = (char *) mbkalloc(strlen(CBeh->NAME) + 2);
   sprintf(carint,"%sd",CBeh->NAME);
   decompileVHDL(CBeh,carint,1);
   mbkfree(carint);
   }

		/* reconstruction des graphes */

orderPI = (chain_list *) makeOrderAbl(CBeh,FP_ORDER,0);
aux = CBeh->BEAUX;
while (aux)
   {
   orderPI = addchain(orderPI,aux->NAME);
   aux = aux->NEXT ;
   }
        
regenereBddBeh(CBeh,orderPI);
makeBddBeh(CBeh,CBeh->CIRCUI,0,trace,50,100,NULL);


	/* on choisit entre beh et compactBeh */


switch (FP_FAMILY)
   {
   case X3000 :
      costX3000Beh(FP_BEH,&numCLB,&depthCLB);
      costX3000Beh(CBeh,&numCLBC,&depthCLBC);
      break;
   case X4000 :
      costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
      costFpgaBeh(CBeh,costX4000,&numCLBC,NULL,&depthCLBC);
      break;
   case ACT1 :
      costACT1Beh(FP_BEH,&numCLB,&depthCLB);
      costACT1Beh(CBeh,&numCLBC,&depthCLBC);
      break;
   case ACT2 :
      costACT1Beh(FP_BEH,&numCLB,&depthCLB);
      costACT1Beh(CBeh,&numCLBC,&depthCLBC);
      break;
   case XNODE :
      costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
      costFpgaBeh(CBeh,countNode,&numCLBC,depthNode,&depthCLBC);
      break;
   default :
      costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
      costFpgaBeh(CBeh,costFPGA,&numCLBC,NULL,&depthCLBC,FP_AUX);
   }

if (FP_TRACE)
   {
   printf(" beh         :   blocks = %d     depth = %d\n",numCLB,depthCLB);
   printf(" compact beh :   blocks = %d     depth = %d\n",numCLBC,depthCLBC);
   }
 
if (numCLB > numCLBC)
   {
   FP_BEH = CBeh;
   beh = CBeh;
   }

aux = beh->BEAUX;
saveAux = NULL;               /* on vide les auxinit eventuels */
while (aux)
  {
  saveAux = addchain(saveAux,aux->NAME);
  aux = aux->NEXT;
  }

{
chain_list *intermediate = FP_GUARDEDAUX;
while (intermediate)
   {
   if (existAuxBeh(FP_BEH,intermediate->DATA))
      {  
      saveAux = addchain(saveAux,intermediate->DATA);
      }  
   else
      {  
      PRINTF("optimization : Warning \n---> bad parameters file\n");
      PRINTF("---> signal %s unknown\n");
      }  
   intermediate = intermediate->NEXT;
   }
}

#ifdef LAX
   PRINTF("INITIAL COST :\n");
#else
   printf("=============================  INITIAL COST  ==================================\n");
#endif

if (FP_FAMILY == LUT4 || FP_FAMILY == LUT5 || FP_FAMILY == LUT6)
   {
   costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
   PRINTF("Total number of LUT   = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X3000)
   {
   costX3000Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of X3000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X4000)
   {
   costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
   PRINTF("Total number of X4000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == ACT1 || FP_FAMILY == ACT2)
   {
   costACT1Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of ACT1  = %d\n",numCLB);
   PRINTF("Depth in ACT1         = %d\n",depthCLB);
   }

costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
PRINTF("Total number of BDD   = %d\n",numCLB);
PRINTF("Depth in BDD          = %d\n",depthCLB);


#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif

                        /* re-ordonnancement */

numberNode = 5;
 
if (FP_MAKEORDER == 2)
   {
   switch (FP_FAMILY)
      {
      case X3000 :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,costX3000);
         break;
      case X4000 :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,costX4000);
         break;
      case ACT1 :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,costACT1);
         break;
      case ACT2 :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,costACT1);
         break;
      case XNODE :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,countNode);
         break;
      default :
         fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,100,1,costFPGA);
      }
   }

else             /* on recupere l'ordre de la couche circuit */
   {
   char **ptChar;
   int c;
 
   ptChar = (FP_CIRCUIT)->pNameI;               /* les entrees */
 
   fp_sys->orderPI = NULL;
   for (c = ((FP_CIRCUIT)->countI)-2; c != 0; c--)
      {  
      fp_sys->orderPI =  addchain(fp_sys->orderPI,*ptChar);
      ptChar++;
      }  
   fp_sys->orderPI = reverse(fp_sys->orderPI);
   }
 
makeBddBeh(FP_BEH,FP_CIRCUIT,0,trace,200,1000000,NULL);
 
removeAuxBeh(FP_BEH,saveAux);
}

/*-------------------------------------------------------------------------
globalOptFPGA   : optimisation globale d'une befig
---------------------------------------------------------------------------
retour          : un void.
---------------------------------------------------------------------------*/
void globalOptFPGA(trace)
int trace;
{
tabSolutionFPGA *tabSol;
chain_list *order;
int level;
int amplitude;
int numberSol,indiceMin,coutMin;
int i;
int existDC;
chain_list *lst, *lstExpr;
chain_list *saveAux;
int numberInput = 0;
int numberNode = 0;
int numCLB,depthCLB;

        /* sauvegarde dans saveAux des auxiliaires a garder */
 
saveAux = (chain_list *) createAuxInitBeh(FP_BEH);
 

		/* prise en compte des Don't care */

        /* existe-t-il des constantes 'd' dans les expressions ? */
 
lstExpr = exprToCLBeh(FP_BEH);
lst = lstExpr;
existDC = 0;
while (lst)
   {
   if (searchExpr((chain_list *)lst->DATA,"'d'"))
      existDC = 1;
   lst = lst->NEXT;
   }
freechain(lstExpr);
 
if (existDC)    /* appel au traitement des don't care */
   {
   for (i=1;i<3;i++)
      {

        /* premiere phase d'ordonnancement avant la prise en compte des DC */

      if (FP_MAKEORDER == 2)
         {
         switch (FP_FAMILY)
            {
            case X3000 :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,costX3000); 
             break;
            case X4000 :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,costX4000); 
             break;
            case ACT1 :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,costACT1); 
             break;
            case ACT2 :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,costACT1); 
             break;
            case XNODE :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,countNode); 
             break;
            default :
             order = makeOrderBddBeh(FP_BEH,0,NULL,NULL,1,5,1,trace,50,1,costFPGA); 
            }
         }
      makeBddBeh(FP_BEH,FP_CIRCUIT,0,trace,200,1000000,NULL);
      }

   switch (FP_FAMILY)
      {
      case X3000 :
         traitementDCBeh(FP_BEH,trace,costX3000,1);
         break;
      case X4000 :
         traitementDCBeh(FP_BEH,trace,costX4000,1);
         break;
      case ACT1 :
         traitementDCBeh(FP_BEH,trace,costACT1,1);
         break;
      case ACT2 :
         traitementDCBeh(FP_BEH,trace,costACT1,1);
         break;
      case XNODE :
         traitementDCBeh(FP_BEH,trace,countNode,1);
         break;
      default :
         traitementDCBeh(FP_BEH,trace,costFPGA,1);
      }
   }

 
                /* on met dans saveaux les variables intermediaires
                   que l'on conserve */
 
   {
   chain_list *intermediate = FP_GUARDEDAUX;
   while (intermediate)
      {  
      if (existAuxBeh(FP_BEH,intermediate->DATA))
         {
         saveAux = addchain(saveAux,intermediate->DATA);
         }
      else
         {
         PRINTF("optimization : Warning \n---> bad parameters file\n");
         PRINTF("---> signal %s unknown\n");
         }
      intermediate = intermediate->NEXT;
      numberInput++;
      }  
   }

		/* on elimine les autres variables */

 
removeAuxBeh(FP_BEH,saveAux);
bddToAblBeh(FP_BEH,0,FP_TRACE);
          
#ifdef LAX
   PRINTF("INITIAL COST :\n");
#else
   printf("=============================  INITIAL COST  ==================================\n");
#endif

if (FP_FAMILY == LUT4 || FP_FAMILY == LUT5 || FP_FAMILY == LUT6)
   {
   costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
   PRINTF("Total number of LUT   = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X3000)
   {
   costX3000Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of X3000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X4000)
   {
   costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
   PRINTF("Total number of X4000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == ACT1 || FP_FAMILY == ACT2)
   {
   costACT1Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of ACT1  = %d\n",numCLB);
   PRINTF("Depth in ACT1         = %d\n",depthCLB);
   }

costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
PRINTF("Total number of BDD   = %d\n",numCLB);
PRINTF("Depth in BDD          = %d\n",depthCLB);


#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif


tabSol = (tabSolutionFPGA *) mbkalloc (100 * sizeof(tabSolutionFPGA));
numberSol = 0;
 
                /* on evalue la solution initiale */

switch (FP_FAMILY)
   {
   case X3000 :
        costX3000Beh(FP_BEH,&numCLB,&depthCLB);
        break;
   case X4000 :
        costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
        break;
   case ACT1 :
        costACT1Beh(FP_BEH,&numCLB,&depthCLB);
        break;
   case ACT2 :
        costACT1Beh(FP_BEH,&numCLB,&depthCLB);
        break;
   case XNODE :
        costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
        break;
   default :
        costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
   }
 
tabSol[numberSol].numCLB = numCLB;
tabSol[numberSol].depthCLB = depthCLB;
tabSol[numberSol].order = copyChain_list(FP_ORDERPI);

if (FP_TRACE)
   {
   chain_list *ordering;
 
   PRINTF("Total number of Blocks   = %d\n",numCLB);
   PRINTF("Depth in blocks          = %d\n",depthCLB);
 
   ordering = tabSol[numberSol].order;
   PRINTF("Order                    =");
   while (ordering)
      {
      PRINTF(" %s",ordering->DATA);
      ordering = ordering->NEXT;
      }
   PRINTF("\n");
   }
numberSol++;

                /* on positionne les variables du recuit simule */

numberInput = numberInput + countInputBeh(FP_BEH);
amplitude = (int) log((double) numberInput);
if (amplitude < 4)
   amplitude = 4;
numberNode = 5;

if (FP_MAKEORDER < 2)
   (fp_sys->par)->level = 0;
level = FP_LEVEL;
if (level < amplitude)
   amplitude = level - 1;

numberInput = numberInput / 2;
if (numberInput < 2)
   numberInput = 2;

 
if (trace)
   printf("\n============== Simulated Anealing ===============\n\n");


while (amplitude >= 0 && level >= 1)
   {
   for (i=2;i>0;i--)
      {
		/* calcul de l'ordonnancement par gradient */

      if (i == 2)
         {
         switch (FP_FAMILY)
            {
            case X3000 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,costX3000);
               break;
            case X4000 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,costX4000);
               break;
            case ACT1 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,costACT1);
               break;
            case ACT2 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,costACT1);
               break;
            case XNODE :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,countNode);
               break;
            default :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,numberInput,1,costFPGA);
            }
         }
      else
         {
         switch (FP_FAMILY)
            {
            case X3000 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,costX3000);
               break;
            case X4000 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,costX4000);
               break;
            case ACT1 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,costACT1);
               break;
            case ACT2 :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,costACT1);
               break;
            case XNODE :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,countNode);
               break;
            default :
               fp_sys->orderPI = makeOrderBddBeh(FP_BEH,FP_OPTIM,FP_OPTIMPO,
                            FP_DELAYPI,1,numberNode,1,trace,2000,1,costFPGA);
            }
         }

      makeBddBeh(FP_BEH,FP_CIRCUIT,0,trace,200,1000000,NULL);

		      	/* calcul d'une nouvelle solution */

      if (i == 1)  
         {
         int ruledc = RULES_DC;

			/* sans don't care */

         RULES_DC = 0;
         switch (FP_FAMILY)
            {
            case X3000 :
               costX3000Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case X4000 :
               costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
               break;
            case ACT1 :
               costACT1Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case ACT2 :
               costACT1Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case XNODE :
               costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
               break;
            default :
               costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
            }
 

         tabSol[numberSol].numCLB = numCLB;
         tabSol[numberSol].depthCLB = depthCLB;
         tabSol[numberSol].order = copyChain_list(FP_ORDERPI);

         if (FP_TRACE)
            {
            chain_list *ordering;
 
            PRINTF("Total number of Blocks   = %d\n",numCLB);
            PRINTF("Depth in blocks          = %d\n",depthCLB);
 
            ordering = tabSol[numberSol].order;
            PRINTF("Order                    =");
            while (ordering)
               {
               PRINTF(" %s",ordering->DATA);
               ordering = ordering->NEXT;
               }
            PRINTF("\n");
            }
         numberSol++;

			/* avec don't care */

         RULES_DC = 1;
         switch (FP_FAMILY)
            {
            case X3000 :
               costX3000Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case X4000 :
               costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
               break;
            case ACT1 :
               costACT1Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case ACT2 :
               costACT1Beh(FP_BEH,&numCLB,&depthCLB);
               break;
            case XNODE :
               costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);
               break;
            default :
               costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
            }
 

         tabSol[numberSol].numCLB = numCLB;
         tabSol[numberSol].depthCLB = depthCLB;
         tabSol[numberSol].order = copyChain_list(FP_ORDERPI);

         if (FP_TRACE)
            {
            chain_list *ordering;
 
            PRINTF("Total number of Blocks   = %d\n",numCLB);
            PRINTF("Depth in blocks          = %d\n",depthCLB);
 
            ordering = tabSol[numberSol].order;
            PRINTF("Order                    =");
            while (ordering)
               {
               PRINTF(" %s",ordering->DATA);
               ordering = ordering->NEXT;
               }
            PRINTF("\n");
            }
         numberSol++;

      		/* on restore RULES_DC */

         RULES_DC = ruledc;
         }
      }

   		/* recuit simule */

   if (amplitude > 0 && level > 1)
      {
      fp_sys->orderPI = randomOrder(FP_ORDERPI,amplitude);
      regenereBddBeh(FP_BEH,FP_ORDERPI);
      resetBdd();
      makeBddBeh(FP_BEH,FP_CIRCUIT,0,trace,200,1000000,NULL);
      }
   if (amplitude > 1)
      amplitude--;
   level--;
   }

		/* choix de la meilleure solution */

coutMin = 10000000;

for (i=0;i<numberSol;i++)
   {
   int coutI = tabSol[i].numCLB + (tabSol[i].depthCLB*FP_OPTIM);

   if (FP_TRACE)
      printf("== Solution %d :  Depth = %d  Blocks = %d\n",i,tabSol[i].depthCLB,tabSol[i].numCLB);
   if (coutMin > coutI)
      {
      coutMin = coutI;
      indiceMin = i;
      }
   }

		/* on utilise les DC si la solution est paire */

if (indiceMin % 2 == 1 && indiceMin > 0)
   RULES_DC = 0;
else
   RULES_DC = 1;
 
                /* regeneration des graphes */
 
if (FP_TRACE)
   printf("Graph regeneration %d \n",indiceMin);
 
fp_sys->orderPI = tabSol[indiceMin].order;

regenereBddBeh(FP_BEH,FP_ORDERPI);
resetBdd();
makeBddBeh(FP_BEH,FP_CIRCUIT,0,trace,200,1000000,NULL);
removeAuxBeh(FP_BEH,saveAux);

if (FP_TRACE)
   displayCct(FP_CIRCUIT,1);
mbkfree(tabSol);
}


/*-------------------------------------------------------------------------
annealingFPGA 	: recuit simule pour l'optimisation FPGA 
---------------------------------------------------------------------------
retour		: un void.
---------------------------------------------------------------------------*/
void annealingFPGA(beh,trace)
befig_list *beh;
int trace;
{
int numCLB,depthCLB;

printf("Running the FPGA Optimizer on '%s'...\n",FP_BEH->NAME);

if (FP_GUARDEDAUX)
   printf("Warning : some intermediate signals are kept\n");

if (FP_AUX)
   localOptFPGA(trace);
else
   globalOptFPGA(trace);
 


#ifdef LAX
   PRINTF("FINAL COST :\n");
#else
   printf("==============================  FINAL COST  ===================================\n");
#endif

if (FP_FAMILY == LUT4 || FP_FAMILY == LUT5 || FP_FAMILY == LUT6)
   {
   costFpgaBeh(FP_BEH,costFPGA,&numCLB,NULL,&depthCLB,FP_AUX);
   PRINTF("Total number of LUT   = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X3000)
   {
   costX3000Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of X3000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == X4000)
   {
   costFpgaBeh(FP_BEH,costX4000,&numCLB,NULL,&depthCLB);
   PRINTF("Total number of X4000 = %d\n",numCLB);
   PRINTF("Depth in CLB          = %d\n",depthCLB);
   }

if (FP_FAMILY == ACT1 || FP_FAMILY == ACT2)
   {
   costACT1Beh(FP_BEH,&numCLB,&depthCLB);
   PRINTF("Total number of ACT1  = %d\n",numCLB);
   PRINTF("Depth in ACT1         = %d\n",depthCLB);
   }

costFpgaBeh(FP_BEH,countNode,&numCLB,depthNode,&depthCLB);

PRINTF("Total number of BDD   = %d\n",numCLB);
PRINTF("Depth in BDD          = %d\n",depthCLB);


#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif

printf("End of the FPGA Optimizer\n\n");
}
