/* ###----------------------------------------------------### */
/*    file  : lex.c                                           */
/*    description : ce fichier analyse le flux d'entrees d'une*/
/*    fonction booleenne au plus 5 varaibles d'un caractere   */
/*    et renvoie les tokens au yacc .                         */
/*    yacc utilise des fonctions qui creent ABL de la fonction*/ 
/*    booleenne, Ces fonctions sont definies dans ce fichier  */
/*    date  : 18.1.95                                         */
/*    autor : Behzad SAFARI                                   */
/* ###----------------------------------------------------### */


#include MUT_H
#include LOG_H

#include <stdio.h>
#include <stdlib.h>
#include "y.tab.h"
#include "lex.h"

#define addabl(DATA,NEXT) addchain(NEXT,DATA)

static char *CHAIN;        /* variable globale contenant la chaine */
static chain_list *ABL;    /* variable globale pointant sur ABL    */



/* ###----------------------------------------------------### */
/*    fonction  : ablatom                                     */
/*    description : cette fonction cree abl d'une variable    */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *ablatom(data,next)
void *data;
chain_list *next;

  {
  ABL = addabl((void *)data,next);
  return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : ablor                                       */
/*    description : cette fonction cree abl d'une fonction    */
/*    booleenns OR .                                          */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *ablor(data,next)
void *data;
chain_list *next;

  {
  ABL = addabl((void *)addabl((void *)OR,NULL),
        addabl((void *)data,addabl((void *)next,NULL)));

  return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : abland                                      */
/*    description : cette fonction cree abl d'une fonction    */
/*    booleenns AND .                                         */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *abland(data,next)
void *data;
chain_list *next;

  {
  ABL = addabl((void *)addabl((void *)AND,NULL),
        addabl((void *)data,addabl((void *)next,NULL)));

   return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : ablxor                                      */
/*    description : cette fonction cree abl d'une fonction    */
/*    booleenns AND .                                         */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *ablxor(data,next)
void *data;
chain_list *next;

  {
  ABL =  addabl((void *)addabl((void *)OR,NULL),
         addabl((void *)addabl((void *)addabl((void *)AND,NULL),
         addabl ((void *)addabl((void *)addabl((void *)NOT,NULL),
         addabl((void *)data,NULL)),addabl((void *)next,NULL))),
         addabl((void *)addabl((void *)addabl((void *)AND,NULL),
         addabl ((void *)addabl((void *)addabl((void *)NOT,NULL),
         addabl((void *)next,NULL)),addabl((void *)data,NULL))),NULL)));

  return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : ablnot                                      */
/*    description : cette fonction cree abl d'une fonction    */
/*    booleenns NOT .                                         */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *ablnot(data,next)
void *data;
chain_list *next;

  {
  ABL =  addabl((void *)addabl((void *)NOT,NULL),
         addabl((void *)next,NULL));

  return ABL;
  }
/* ###----------------------------------------------------### */
/*    fonction  : ablstable                                   */
/*    description : cette fonction cree abl d'une operande    */
/*    STABLE utilisee en VHDL  .                              */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */
chain_list *ablstable(data,next)
void *data;
chain_list *next;

  {
  ABL =  addabl((void *)addabl((void *)STABLE,NULL),
         addabl((void *)next,NULL));

  return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : booltoabl                                   */
/*    description : cette fonction recoit l'equation booleenne*/
/*    sous forme d'une chaine de caracteres et appelle yyparse*/
/*    et elle renvoit un pointeur sur abl cree .              */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

chain_list *booltoabl (str)
char *str;

  {
     /* ###------------------------------------------### */
     /*     CHAIN est une variable globale au fichier    */
     /*     qui contient la chaine de caracteres de      */
     /*     l'equation booleenne. On a besoin de cette   */
     /*     variable pour pouvoir l'utiliser dans yylex .*/
     /* ###------------------------------------------### */

  CHAIN = str;

    yyparse();
    return ABL;
  }

/* ###----------------------------------------------------### */
/*    fonction  : yylex                                       */
/*    description : cette fonction analyse le flux d'entree   */
/*    d'une fonction booleenne et renvoit les tokens au yacc  */
/*    date  : 18.1.95                                         */
/* ###----------------------------------------------------### */

int yylex()
 {
   char ch[20];
   int i     ;
   char c    ;

while( (c = *(CHAIN ++)) != '\0')
{
while(!isspace(c))
   {

  if (isalpha(c) || c == '_')
                   {
                   i = 0;
                   do
                       ch[i++] = c;

                   while(isalnum(c = *(CHAIN ++)) && !isspace(c) || c == '_');
                   CHAIN --;
                   ch[i]='\0';
                   yylval.s=namealloc(ch);
                   return ID ;
                   }
 
   switch (c)
              {
              case '+': {return _OR ;}
              case '*': {return _AND;}
              case '~': {return _NOT;}
              case '@': {return _XOR;}
              case '!': {return _STABLE;}
              case '(': {return '(';}
              case ')': {return ')';}

              default : yyerror("bad char");
              }
   }
}
return 0;
 }


int yywrap()

  {
  return 1;
  }


void yyerror(s)
char *s;

  {
  fprintf(stderr,"%s \n",s);
  exit(0);
  }

