#include <stdio.h>
#include <stdlib.h>
#include <varargs.h>
#include <ctype.h>

#include MUT_H
#include MLO_H
#include MLU_H
#include MPH_H
#include MPU_H

#include "rin.h"
#include "frin.h"

static lofig_list *lf;
static phfig_list *pf, *pc;

/******************************
 *
 * print out an error an exists.
 * could probably be wise to detect all the errors before quiting, as
 * I formerly did in bbr, but I'm lazy today.
 *
 ****************************************************/
void error(va_alist)
va_dcl
{
va_list args;
int errn;
char *s;
static char *msg[] = {
   "No core found in logical figure '%s'",
   "No '%s.%s' file found for pad placement",
   "More than one possible core in figure '%s'",
   "No physical instance '%s' found in figure '%s'",
   "No logical instance '%s' found in figure '%s'",
   "No logical pad connector '%s' in instance '%s' found in figure '%s'",
   "No north connector '%s' in figure '%s' found during external copy",
   };

   va_start(args);
   errn = va_arg(args, int);
   s = msg[errn];
   (void)vfprintf(stderr, s, args);
   fputs("\n", stderr);
   va_end(args);
   exit(2);
}

/***********************************
 *
 * type the figures of the `.rin' file as pads
 * only the physical figure is typed, so I'll have to do a getphfig
 * before any pad or not pad verification (even in unexpected parts of
 * the code).
 *
 **************************************************/
#define PADS (long)81294
#define typepad(figure) (figure->USER = addptype(figure->USER, PADS, NULL))
#define ispad(figname)  (getptype(getphfig(figname, 'P')->USER, PADS))
#define freepad(figure) (figure->USER = delptype(figure->USER, PADS))

void getpadmodel(insname)
char *insname;
{
loins_list *li;
phfig_list *pad;

   li = getloins(lf, insname);
   pad = getphfig(li->FIGNAME, 'P');
   typepad(pad);
}

/***********************************
 *
 * add the pads into the chip level physical figure:
 * the pads orientation is as follows for correct position.
 * north row: NOSYM
 * south row: SYM_Y
 * west row : ROT_P
 * east row : SY_RP
 * that's why the geop array below look like that!
 *
 **************************************************/

static char geop[4] = {NOSYM, SYM_Y, ROT_P, SY_RP};
static int rowsize[2][4]; /* number of pads and size of each pad row */

void addpad(index, insname)
int index;
char *insname;
{
loins_list *li;
phins_list *lastpi = pf->PHINS;
phfig_list *pad, *lastpad;
int x, y;

   li = getloins(lf, insname);
   pad = getphfig(li->FIGNAME, 'P');
   lastpad = getphfig(lastpi->FIGNAME, 'P');

   /* lastpi:
      If we place the first pad of a row, ie the previous instance is
      the core of the chip or a pad with a new orientation, we must
      choose the coordinates.
      Otherwise, it's fairly straight forward: abut the pads in the
      given order. */

   if (lastpi->TRANSF == geop[index] && ispad(lastpi->FIGNAME)) {
      if (index == LNORTH || index == LSOUTH) {
         x = lastpi->XINS + lastpad->XAB2 - lastpad->XAB1, y = lastpi->YINS;
         rowsize[1][index] = x + (pad->XAB2 - pad->XAB1);
      } else {
         x = lastpi->XINS, y = lastpi->YINS + lastpad->XAB2 - lastpad->XAB1;
         rowsize[1][index] = y + (pad->XAB2 - pad->XAB1);
      }
   } else {
      /* no core coordinates:
         I KNOW that I put the core in 0,0, so there is no need to
         take its instanciation coordinates into account. */

      switch (index) {
      case LNORTH: x = 0, y = 0 + (pc->YAB2 - pc->YAB1);
                   break;
      case LSOUTH: x = 0, y = 0 - (pad->YAB2 - pad->YAB1);
                   break;
      case LWEST : x = 0 - (pad->YAB2 - pad->YAB1), y = 0;
                   break;
      case LEAST : x = 0 + (pc->XAB2 - pc->XAB1), y = 0;
                   break;
      }
   }
   rowsize[0][index]++;
   addphins(pf, li->FIGNAME, insname, geop[index], x, y);
}

/****************************
 *
 * adjust the length of each row and col.
 * car must be taken to ensure that each pad is place on a grid that
 * is an integer number of SCALE_X's.
 * Beside that, it's fairly straight forward.
 *
 ***************************************************/
#define adjustrow(index) \
{\
   additive = (dx - rowsize[1][index]) / SCALE_X;\
   quotien = additive / rowsize[0][index];\
   rest    = additive % rowsize[0][index];\
   for (pi = pf->PHINS; pi; pi = pi->NEXT) {\
      if (pi->TRANSF != geop[index] || !ispad(pi->FIGNAME))\
         continue;\
      pi->XINS += additive * SCALE_X;\
      if (index == LNORTH)\
         pi->YINS = dy;\
      additive -= (quotien + (rest-- > 0 ? 1 : 0));\
   }\
}

#define adjustcol(index) \
{\
   additive = (dy - rowsize[1][index]) / SCALE_X;\
   quotien = additive / rowsize[0][index];\
   rest    = additive % rowsize[0][index];\
   for (pi = pf->PHINS; pi; pi = pi->NEXT) {\
      if (pi->TRANSF != geop[index] || !ispad(pi->FIGNAME))\
         continue;\
      if (index == LEAST)\
         pi->XINS = dx;\
      pi->YINS += additive * SCALE_X;\
      additive -= (quotien + (rest-- > 0 ? 1 : 0));\
   }\
}


static void adjustpads(pf)
phfig_list *pf;
{
int dx, dy; /* the core starts in 0,0 so delta are sufficient */
int additive, quotien, rest;
phfig_list *pad;
phins_list *pi;

   /* row:
      find the largest row in both directions. */
   dx = rowsize[1][LNORTH] > rowsize[1][LSOUTH] ? rowsize[1][LNORTH]
                                                : rowsize[1][LSOUTH];
   dy = rowsize[1][LEAST] > rowsize[1][LWEST]   ? rowsize[1][LEAST]
                                                : rowsize[1][LWEST];
   /* pad limited circuit in the x direction: stretch the core */
   if (dx > (pc->XAB2 - pc->XAB1)) {
      additive = (dx - (pc->XAB2 - pc->XAB1)) / SCALE_X;
      if (additive%2) /* odd number, can't split it in the middle */
         pc->XAB2 += ((additive/2) + 1) * SCALE_X,
         pc->XAB1 -= (additive/2) * SCALE_X;
      else
         pc->XAB2 += (additive/2) * SCALE_X,
         pc->XAB1 -= (additive/2) * SCALE_X;
   } else { /* core limited in that direction: stretch the rows */
      dx = (pc->XAB2 - pc->XAB1);
   }

   /* pad limited circuit in the y direction */
   if (dy > (pc->YAB2 - pc->YAB1)) {
      additive = (dy - (pc->YAB2 - pc->YAB1)) / SCALE_X;
      if (additive%2) /* odd number, can't split it in the middle */
         pc->YAB2 += ((additive/2) + 1) * SCALE_X,
         pc->YAB1 -= (additive/2) * SCALE_X;
      else
         pc->YAB2 += (additive/2) * SCALE_X,
         pc->YAB1 -= (additive/2) * SCALE_X;
   } else { /* core limited in that direction */
      dy = (pc->YAB2 - pc->YAB1);
   }

   /* the --[LNORTH] is due to a piquet et interval pb */
   --rowsize[0][LNORTH];
   --rowsize[0][LSOUTH];
   --rowsize[0][LWEST];
   --rowsize[0][LEAST];
   adjustrow(LNORTH);
   adjustrow(LSOUTH);
   adjustcol(LWEST);
   adjustcol(LEAST);

   /* final abbox:
      I must fit the chip ab box on the pads boundary, and that's what
      I'm doing right now */
   pad = getphfig(pf->PHINS->FIGNAME, 'P');
   pf->XAB1 = -(pad->YAB2 - pad->YAB1);
   pf->XAB2 = 0 + (pc->XAB2 - pc->XAB1) + (pad->YAB2 - pad->YAB1);
   pf->YAB1 = -(pad->YAB2 - pad->YAB1);
   pf->YAB2 = 0 + (pc->YAB2 - pc->YAB1) + (pad->YAB2 - pad->YAB1);
}

static locon_list *getloinscon(instance, conname)
loins_list *instance;
char *conname;
{
locon_list *c;

   for (c = instance->LOCON; c; c = c->NEXT)
      if (c->NAME == conname)
         return c;
   return NULL;
}

static locon_list *getlofigcon(figure, signal)
lofig_list *figure;
losig_list *signal;
{
locon_list *c;

   for (c = figure->LOCON; c; c = c->NEXT)
      if (c->SIG == signal)
         return c;
   return NULL;
}

static phcon_list *getphfigcon(pad, name)
phfig_list *pad;
char *name;
{
phcon_list *con;

   for (con = pad->PHCON; con; con = con->NEXT)
      if (con->NAME == name && con->ORIENT == NORTH)
         return con;
   return NULL;
}

/****************************
 *
 * this has two functions:
 * 1) create the physical connectors on the abbox of the core.
 *    Each connector of a pad that is connected to the internal block
 *    is copied down into the existing core.
 * 2) since we may have several connectors that are connected together
 *    in a pad, ie alu1 and alu2 are both available, we have to make a
 *    choice to satisfy cheops priviledged directions.
 *
 ************************************************************/
static void copydowncon(pf, lf, pc)
phfig_list *pf, *pc;
lofig_list *lf;
{
phins_list *pi;
phfig_list *pad;
phcon_list *con, *scon, *next;
int x, y, face;
int i = 0;
loins_list *li;
locon_list *lc, *fc;
char *conname;

   /* core:
      new connectors are added on the core to have a cheops compatible
      physical and logical view of the object. */
   for (pi = pf->PHINS; pi; pi = pi->NEXT) {
     if (!ispad(pi->FIGNAME))
         continue;
      pad = getphfig(pi->FIGNAME, 'P');
      li = getloins(lf, pi->INSNAME);
      switch (pi->TRANSF) {
      case NOSYM: face = NORTH;
                  break;
      case SYM_Y: face = SOUTH;
                  break;
      case ROT_P: face = WEST;
                  break;
      case SY_RP: face = EAST;
      }
      for (con = pad->PHCON; con; con = con->NEXT) {
         if (con->ORIENT == SOUTH) {
            if ((lc = getloinscon(li, con->NAME)) == NULL)
               error(NO_LO_PAD_CON, con->NAME, li->INSNAME, lf->NAME);
            xyflat(&x, &y, con->XCON, con->YCON, pi->XINS, pi->YINS,
                   pad->XAB1, pad->YAB1, pad->XAB2, pad->YAB2,
                   pi->TRANSF);
            /* this is a copy_down, ie we must move the connector
               according to the instanciated model, not the current model */
            x += 0 + pc->XAB1;
            y += 0 + pc->YAB1;

            conname = getsigname(lc->SIG);
            /* conname = lc->NAME; check if that works some days */
            addphcon(pc, face, conname, x, y, con->LAYER, con->WIDTH); 
            if (!getlofigcon(lf, lc->SIG))
               addlocon(lf, conname, lc->SIG, UNKNOWN);
         }
      }
   }

   /* check for doubles:
      cheops doesn't like same connectors in metal 1/ metal 2 at the
      same coordinates, so we have to destroy the one present using a
      orientation criteria.
      Note that this is the simplest solution I though of, not the
      most elegant or fast one. */
   for (con = pc->PHCON; con->NEXT; con = next)
      for (scon = con->NEXT; scon; scon = scon->NEXT) {
         if (con->XCON == scon->XCON
             && con->YCON == scon->YCON
             && con->NAME == scon->NAME) {
            if ((con->LAYER == ALU2
                   && (con->ORIENT == NORTH || con->ORIENT == SOUTH))
                 || (con->LAYER == ALU1
                   && (con->ORIENT == EAST || con->ORIENT == WEST)))
               scon->LAYER = con->LAYER;
            next = con->NEXT;
            delphcon(pc, con);
         } else
            next = con->NEXT;
      }
}

static phcon_list *getnorthphcon(pad, name)
phfig_list *pad;
char *name;
{
phcon_list *con;

   for (con = pad->PHCON; con; con = con->NEXT)
      if (con->NAME == name && con->ORIENT == NORTH)
         return con;
   return NULL;
}

/*******************************************
 *
 * In order for the chip to be usable, we have to put connectors with
 * a relevant name on the pads outer world connectors.
 * Relevant means with the name given in the port declaration of the
 * netlist, for sure.
 *
 *****************************************************************/
static void copyupcon(lf, pf)
lofig_list *lf;
phfig_list *pf;
{
loins_list *li;
phins_list *pi;
phcon_list *con;
chain_list *c;
locon_list *lc, *tlc;
phfig_list *pad;
int x, y, face;

   /* chip:
      the logical connectors of the chip must be copied up on the
      physical interface in order to validate the layout of the chip.
      So go ahead, lets copy them up. */

   lofigchain(lf);
   for (lc = lf->LOCON; lc; lc = lc->NEXT) {
      c = (chain_list *)(getptype(lc->SIG->USER, LOFIGCHAIN)->DATA);
      for (; c; c = c->NEXT) {
         tlc = ((locon_list *)c->DATA);
         if (tlc->TYPE == INTERNAL) {
            li = (loins_list *)(tlc->ROOT);
            if (!ispad(li->FIGNAME))
               continue; /* should happen for the core only */
            pi = getphins(pf, li->INSNAME);
            pad = getphfig(li->FIGNAME, 'P');
            if ((con = getnorthphcon(pad, tlc->NAME)) == NULL)
               continue;
            xyflat(&x, &y, con->XCON, con->YCON, pi->XINS, pi->YINS,
                   pad->XAB1, pad->YAB1, pad->XAB2, pad->YAB2,
                   pi->TRANSF);
            switch (pi->TRANSF) {
            case NOSYM: face = NORTH;
                        break;
            case SYM_Y: face = SOUTH;
                        break;
            case ROT_P: face = WEST;
                        break;
            case SY_RP: face = EAST;
            }
            addphcon(pf, face, lc->NAME, x, y, con->LAYER, con->WIDTH); 
         }
      }
   }
}

/************************
 * 
 * we're building the netlist of the core from the netlist of the
 * chip, so we must get rid of the unuseful (at a certain time)
 * logical connectors of the chip.
 *
 ********************************************************/
static void killlocon(lf)
lofig_list *lf;
{
locon_list *lc, *next;

   for (lc = lf->LOCON; lc; lc = (locon_list *)next) {
      next = lc->NEXT;
      dellocon(lf, lc->NAME);
   }
}


/* Longitudinal extension to be substracted from the power rail second
   coordinates. */
#define EXT(layer) (layer == ALU2 ? SCALE_X : SCALE_X/2)

/* Avoid the creation of 0 length wires */
#define nzphseg(figure, layer, width, x0, y0, x1, y1, name)          \
         do {                                                        \
            if (x0 != x1 || y0 != y1)                                \
               addphseg(figure, layer, width, x0, y0, x1, y1, name); \
         } while(0)
   

/********************************
 *
 * we assume that:
 * every supply connector of every pad is at the same y coordinate and
 * has the same width than of any other pad, and that the netlist has
 * previously been checked to see that no vdd or vss or ck are routed
 * together.
 *
 **************************************************************/
static void drawpower(pf)
phfig_list *pf;
{
phins_list *pi, *lastpi = NULL;
phfig_list *pad, *lastpad;
phcon_list *con;
int x, y;
int i = 0;

   for (pi = pf->PHINS; pi; pi = pi->NEXT) {
      /* ispad():
         we've found the core, by construction our last instance, so
         we have to finish the last corner and then we're done with
         the power rail construction. */
      if (!ispad(pi->FIGNAME)) {
         lastpad = getphfig(lastpi->FIGNAME, 'P');
         for (con = lastpad->PHCON; con; con = con->NEXT) {
            if (con->ORIENT == WEST) {
               xyflat(&x, &y,
                      con->XCON, con->YCON,
                      lastpi->XINS, lastpi->YINS,
                      lastpad->XAB1, lastpad->YAB1,
                      lastpad->XAB2, lastpad->YAB2,
                      lastpi->TRANSF);

               if (lastpi->TRANSF == NOSYM || lastpi->TRANSF == SYM_Y)
                     nzphseg(pf, con->LAYER, con->WIDTH,
                              x, y,
                              x - (con->YCON - lastpad->YAB1) - con->WIDTH/2 + EXT(con->LAYER), y,
                              con->NAME);
               else
                     nzphseg(pf, con->LAYER, con->WIDTH,
                              x, y,
                              x, y - (con->YCON - lastpad->YAB1) - con->WIDTH/2 + EXT(con->LAYER),
                              con->NAME); 
            }
         }
         break;
      }
      pad = getphfig(pi->FIGNAME, 'P');
      /* Add left corners:
         I check that we're on the end of a row, and since i've been
         copying from right to left until now, I have to add the wires
         on the left of the last instance of a row. */
      if (lastpi && pi->TRANSF != lastpi->TRANSF) {
         lastpad = getphfig(lastpi->FIGNAME, 'P');
         for (con = lastpad->PHCON; con; con = con->NEXT) {
            if (con->ORIENT == WEST) {
               xyflat(&x, &y,
                      con->XCON, con->YCON,
                      lastpi->XINS, lastpi->YINS,
                      lastpad->XAB1, lastpad->YAB1,
                      lastpad->XAB2, lastpad->YAB2,
                      lastpi->TRANSF);

               if (lastpi->TRANSF == NOSYM || lastpi->TRANSF == SYM_Y) 
                     nzphseg(pf, con->LAYER, con->WIDTH,
                              x, y,
                              x - (con->YCON - lastpad->YAB1) - con->WIDTH/2 + EXT(con->LAYER), y,
                              con->NAME); 
               else
                     nzphseg(pf, con->LAYER, con->WIDTH,
                              x, y,
                              x, y - (con->YCON - lastpad->YAB1) - con->WIDTH/2 + EXT(con->LAYER),
                              con->NAME);
            }
         }
         lastpi = NULL; /* starting a new row */
      }

      /* Normal case:
         I copy each connector coordinate and place a wire regarding
         the old instance coordinate.
         The right most connection is a special case too. */
      for (con = pad->PHCON; con; con = con->NEXT) {
         if (con->ORIENT == EAST) {
            xyflat(&x, &y,
                   con->XCON, con->YCON,
                   pi->XINS, pi->YINS,
                   pad->XAB1, pad->YAB1,
                   pad->XAB2, pad->YAB2,
                   pi->TRANSF);

            if (pi->TRANSF == NOSYM || pi->TRANSF == SYM_Y) {
               if (lastpi == NULL)
                   nzphseg(pf, con->LAYER, con->WIDTH,
                            x, y,
                            x + (con->YCON - pad->YAB1) + con->WIDTH/2 - EXT(con->LAYER), y,
                            con->NAME); 
               else
                   nzphseg(pf, con->LAYER, con->WIDTH,
                            x, y, lastpi->XINS, y,
                            con->NAME); 
            } else {
               if (lastpi == NULL)
                   nzphseg(pf, con->LAYER, con->WIDTH,
                            x, y,
                            x, y + (con->YCON - pad->YAB1) + con->WIDTH/2 - EXT(con->LAYER),
                            con->NAME); 
               else
                   nzphseg(pf, con->LAYER, con->WIDTH,
                            x, y, x, lastpi->YINS,
                            con->NAME); 
            }
         }
      }
      lastpi = pi;
   }
}

/**************************
 *
 * find the core out of this mess!
 *
 *****************************************************/

static loins_list *getcore(lf, pc)
lofig_list *lf;
phfig_list **pc;
{
loins_list *li, *core;
int index = 0;

   /* find the core :
      I assume that everything that I have not read from the .rin file
      at the chip level is (are) the core block(s). */

   for (li = lf->LOINS; li; li = li->NEXT) {
      if (!ispad(li->FIGNAME)) {
         core = li;
         index ++;
      }
   }

   if (!index)
      error(NO_LOGICAL_CORE, lf->NAME);
   /* two case:
      I got a core level, a la bbr + ring, and well, I got to build a
      inside core to pad netlist.
      Otherwise, I've got this netlist, but I have to modify it
      slightly, and to create the corresponding hierarchical level
      physically.  */
   else if (index == 1) {
      *pc = getphfig(core->FIGNAME, 'A');
      addphins(pf, core->FIGNAME, core->INSNAME, NOSYM, 0, 0);
      return core;
   } else {
      error(N_LOGICAL_CORE, lf->NAME);
      /* To be continued if necessary, but I doubt */
      *pc = addphfig("niet");
      addphins(pf, li->FIGNAME, li->INSNAME, NOSYM, 0, 0);
   }
}

/**************************
 * 
 * do a bunch of verifications on the pads, the pads netlist, the chip
 * and core netlist, etc.
 *
 ******************************************/
static checkpads(lf, pf)
lofig_list *lf;
phfig_list *pf;
{
loins_list *li;
phins_list *pi;
ptype_list *pt;

   /* Instances:
      check for consistency regarding the number and names of logical
      instances and physical instances.
      Brute force 2n^2 algo.
      I first implemented something with the ptypes to do a n + n^2
      but since we have few instances, the complexity and malloc and
      free do not worth it! */
   for (li = lf->LOINS; li; li = li->NEXT) {
      for (pi = pf->PHINS; pi; pi = pi->NEXT)
         if (pi->INSNAME == li->INSNAME)
            break;
      if (!pi)
         error(NO_PHYS_INS, li->INSNAME, lf->NAME);
   }

   for (pi = pf->PHINS; pi; pi = pi->NEXT) {
      for (li = lf->LOINS; li; li = li->NEXT)
         if (pi->INSNAME == li->INSNAME)
            break;
      if (!li)
         error(NO_LOGI_INS, pi->INSNAME, pf->NAME);
   }

   /* for each pad, check that a given gabarit is followed. */
   /* for each pad, check that the power signals are looking fine. */
   /* for each pad, check that the logical instances and the physical
       models are compliants. */
}

static makefeeds(core)
phfig_list *core;
{
phfig_list *pf;
phins_list *pi;
phseg_list *ps, *pps;
int i;

   SEPAR = '_';
   for (pi = core->PHINS; pi; pi = pi->NEXT) {
      pf = getphfig(pi->FIGNAME, 'P');
      i = 0;
      for (ps = pf->PHSEG; ps; ps = pps) {
         if (ps->LAYER == TALU1 || ps->LAYER == TALU2) {
            if (ps->TYPE == UP || ps->TYPE == DOWN) {
               addphcon(pf, SOUTH,
                        nameindex("feed", i),
	        ps->X1, ps->Y1, ps->LAYER, ps->WIDTH); 
               addphcon(pf, NORTH,
                        nameindex("feed", i++),
	        ps->X2, ps->Y2, ps->LAYER, ps->WIDTH); 
            } else {
               addphcon(pf, WEST,
                        nameindex("feed", i),
	        ps->X1, ps->Y1, ps->LAYER, ps->WIDTH); 
               addphcon(pf, EAST,
                        nameindex("feed", i++),
	        ps->X2, ps->Y2, ps->LAYER, ps->WIDTH); 
            }
            pps = ps->NEXT;
            delphseg(pf, ps);
         } else
            pps = ps->NEXT;
      }
      savephfig(pf);
   }
   SEPAR = '.';
}

/******************************
 *
 * destroy all that is not necessary for the core netlist.
 * The core from cheops point of view, for sure.
 *
 ************************************************************/
static void buildnetlist(lf, core)
lofig_list *lf;
loins_list *core;
{
loins_list *li;
losig_list *ls;
void *next;

   for (li = lf->LOINS; li; li = (loins_list *)next) {
      if (ispad(li->FIGNAME)) {
         next = li->NEXT;
         delloins(lf, li->INSNAME);
      } else
         next = li->NEXT;
   }
   flattenlofig(lf, core->INSNAME, NO);
   /* I must delete all the signal on the which zero connectors are
      connected, because I've done a harsh job on this netlist */
   lofigchain(lf);
   for (ls = lf->LOSIG; ls; ls = (losig_list *)next) {
      if (getptype(ls->USER, (long)LOFIGCHAIN)->DATA == NULL) {
         next = ls->NEXT;
         dellosig(lf, ls->INDEX);
      } else
         next = ls->NEXT;
   }
}

/****************************`
 *
 * main routine that calls the other ones, but nothing else!
 *
 ****************************************************************/
void buildchip(name)
char *name;
{
loins_list *li;

   lf = getlofig(name, 'A');
   pf = addphfig(name);
   /* get the circuit core:
      first preload the .rin file in order to see mark the models of
      the instances contained in this file as pads.
      (So I do not relie on the catalog)
      second scan the list of instances to find out our core. */
   preloadpads(name);
   li = getcore(lf, &pc);
   /* get the pad placement:
      only an encapsulation of yyparse, because I don't want to have
      to `sed' all the sources, the `.l' and `.y' are enough. */
   loadpads(name);

   /* checks:
      we must verify some properties on the pads, and on the netlist
      connecting the pads together. */
   checkpads(lf, pf);

   /* fill in row:
      this builds the chips rows of pads, with evently spaced pads if
      needed. */
   adjustpads(pf);

   /* rails:
      are drawn using this function. */
   drawpower(pf);

   /* kill connectors:
      copy all external logical connectors of the chip into the layout
      and then remove all the existing external connectors of the
      chip, since these connectors are not relevant to the core
      routing. */
   copyupcon(lf, pf);
   killlocon(lf);

   /* connectors and connections:
      the pad connectors are copied down into the physical core, and
      copied up into the chip netlist. */
   copydowncon(pf, lf, pc);
   
   /* feed through:
      Cheops assumes that a feed through is a pair of connectors on
      both edges of a block, whereas we say that it is a single wire. */
   makefeeds(pc);


   /* flatten the core:
      we need a block level netlist for cheops, since there is no core
      notion in its world. */
   buildnetlist(lf, li);
}

/******************
 *
 * give me your guess!
 *
 ****************************************/
void savechip()
{
   savephfig(pc);
   savephfig(pf);
   sortlocon(&lf->LOCON);
   sortlosig(&lf->LOSIG);
   lf->NAME = pc->NAME;
   savelofig(lf);
}
