/***************************************************************************
 *
 * Makes the output of cheops compliant with our methodology
 *
 * The edge function is due to Luis Lucas, slighlty modified to use
 * dymanic allocation.
 * It's in O(n^2), but the number of non-minimal segments is small, so
 * who cares.
 * Frederic Petrot
 *
 **************************************************************************/
#include <stdio.h>
#include MUT_H
#include MLO_H
#include MPH_H

static void edge(phfig)
phfig_list *phfig;
{
phfig_list *phinsfig;
phcon_list *phcon;
phseg_list *phseg;
phins_list *phins;

phseg_list  **hor;
phseg_list  **ver;

int width,h,v,i,j;

   for(phins=phfig->PHINS;phins;phins=phins->NEXT){
      phinsfig=getphfig(phins->FIGNAME,'P');
      for(phcon=phinsfig->PHCON;phcon;phcon=phcon->NEXT){int x,y;
         xyflat(&x,&y,phcon->XCON,phcon->YCON,phins->XINS,phins->YINS,
                      phinsfig->XAB1,phinsfig->YAB1,
                      phinsfig->XAB2,phinsfig->YAB2,
                      phins->TRANSF);
         for(phseg=phfig->PHSEG;phseg;phseg=phseg->NEXT){
            if(x==phseg->X1&&y==phseg->Y1) {
                if(!isvdd(phcon->NAME)&&!isvss(phcon->NAME))break;
                if(isvdd(phcon->NAME)&&isvdd(phseg->NAME))break;
                if(isvss(phcon->NAME)&&isvss(phseg->NAME))break;
                fprintf(stderr,"*** %s of %s at %d %d is connected to %s\n",
                      phcon->NAME,phins->INSNAME,
                      x/SCALE_X,y/SCALE_X,
                      phseg->NAME);
            }
            if(x==phseg->X2&&y==phseg->Y2) {
                if(!isvdd(phcon->NAME)&&!isvss(phcon->NAME))break;
                if(isvdd(phcon->NAME)&&isvdd(phseg->NAME))break;
                if(isvss(phcon->NAME)&&isvss(phseg->NAME))break;
                fprintf(stderr,"*** %s of %s at %d %d is connected to %s\n",
                      phcon->NAME,phins->INSNAME,
                      phcon->XCON/SCALE_X,phcon->YCON/SCALE_X,
                      phseg->NAME);
           }
         }
         if(phseg!=NULL) continue;
         if(phcon->LAYER==TALU2||phcon->LAYER==TALU1) continue;
         fprintf(stderr,"*** %s of %s at %d %d badly connected\n",
                phcon->NAME,phins->INSNAME,
                x/SCALE_X,y/SCALE_X);
      }
   }
   width=3*SCALE_X;
   h=v=0;
   for(phseg=phfig->PHSEG;phseg;phseg=phseg->NEXT){
      if(phseg->WIDTH>width&&(phseg->TYPE==LEFT||phseg->TYPE==RIGHT)) h++;
      if(phseg->WIDTH>width&&(phseg->TYPE==UP||phseg->TYPE==DOWN)) v++;
   }
   hor = (phseg_list **)mbkalloc(h * sizeof(phseg_list *));
   ver = (phseg_list **)mbkalloc(v * sizeof(phseg_list *));
   h=v=0;
   for(phseg=phfig->PHSEG;phseg;phseg=phseg->NEXT){
      if(phseg->WIDTH>width&&(phseg->TYPE==LEFT||phseg->TYPE==RIGHT)) 
         hor[h++]=phseg;
      if(phseg->WIDTH>width&&(phseg->TYPE==UP||phseg->TYPE==DOWN)) 
         ver[v++]=phseg;
   }

   for(i=0;i<h;i++){
      for(j=0;j<v;j++){
         if(hor[i]->LAYER!=ver[j]->LAYER) continue;
         if(  (hor[i]->X1==ver[j]->X1&&hor[i]->Y1==ver[j]->Y1)
            ||(hor[i]->X1==ver[j]->X2&&hor[i]->Y1==ver[j]->Y2)){
            hor[i]->X1-=((ver[j]->WIDTH/2)-SCALE_X);
           }
         if(  (hor[i]->X2==ver[j]->X1&&hor[i]->Y2==ver[j]->Y1)
            ||(hor[i]->X2==ver[j]->X2&&hor[i]->Y2==ver[j]->Y2)){
            hor[i]->X2+=((ver[j]->WIDTH/2)-SCALE_X);
           }
        }
     }
}

/***********************************************************
 *
 * No feed through connectors in Alliance
 * This is unuseful since the model of the instances have been alive
 * in a different format.
 *
 ************************/
killfeed(phfig)
phfig_list *phfig;
{
phins_list *pi;
phfig_list *pf;
phcon_list *con, *ard;

   for (pi = pf->PHINS; pi; pi = pi->NEXT) {
      pf = getphfig(pi->FIGNAME, 'A');
      for (con = pf->PHCON; con; con = ard)
         if (con->LAYER == TALU2 || con->LAYER == TALU1) {
            ard = con->NEXT;
            delphcon(pf, con);
         } else
            ard = con->NEXT;
      savephfig(pf);
   }
}

main(argc, argv)
int argc;
char **argv;
{
phfig_list *pf;

   if(argc != 2) {
      fprintf(stderr, "Usage: %s core\n", argv[0]);
      exit(1);
   }
   mbkenv();
   alliancebanner("FROut", "1.0", "Floorplan Router Output", "94, 95", ALC);
   pf = getphfig(argv[1], 'A');
   edge(pf);
   savephfig(pf);
   exit(0);
}
