/*------------------------------------------------------------\
|                                                             |
| Tool    :                   GRAAL                           |
|                                                             |
| File    :                  Error.c                          |
|                                                             |
| Authors :      Venot Frederic and Jacomme Ludovic           |
|                                                             |
| Date    :                  01.08.93                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                         Include Files                       |
|                                                             |
\------------------------------------------------------------*/

# include <stdio.h>
# include <stdlib.h>
# include <signal.h>
# include <setjmp.h>
# include MUT_H
# include MPH_H
# include RDS_H
# include RPR_H
# include RFM_H
# include RTL_H
# include GSB_H 
# include GRM_H
# include "GRM_error.h"

/*------------------------------------------------------------\
|                                                             |
|                           Constants                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                            Types                            |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                          Variables                          |
|                                                             |
\------------------------------------------------------------*/

  static FILE   *GraalStreamErr;
  static FILE   *GraalStreamOut;
  static FILE   *GraalStreamAll;

  static char    GraalErrFileName[ 40 ];
  static char    GraalOutFileName[ 40 ];
  static char    GraalAllFileName[ 40 ];

  static char    GraalNormalMode = 1;

/*------------------------------------------------------------\
|                                                             |
|                      Graal Error Message                    |
|                                                             |
\------------------------------------------------------------*/

  static char *GraalErrorBuffer = (char *)NULL;

/*------------------------------------------------------------\
|                                                             |
|                      Graal Informations                     |
|                                                             |
\------------------------------------------------------------*/

  static char GraalInformationsBuffer[ GRAAL_INFORMATIONS_BUFFER_SIZE ];

/*------------------------------------------------------------\
|                                                             |
|                           Functions                         |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                 Graal Error Message Functions               |
|                                                             |
\------------------------------------------------------------*/
/*------------------------------------------------------------\
|                                                             |
|                   GraalInitializeErrorMessage               |
|                                                             |
\------------------------------------------------------------*/

void GraalInitializeErrorMessage( Debug )

  char Debug;
{
  rdsbegin();

  GraalNormalMode = ! Debug;

  if ( GraalNormalMode )
  {
    sprintf( GraalOutFileName, "/usr/tmp/out_%d",getpid() ); 
  }

  sprintf( GraalErrFileName, "/usr/tmp/err_%d",getpid() );
  sprintf( GraalAllFileName, "/usr/tmp/all_%d",getpid() );

  GraalStreamErr = freopen( GraalErrFileName, "w+", stderr);
  GraalStreamAll = fopen  ( GraalAllFileName, "w+"        );

  if ( GraalNormalMode )
  {
    GraalStreamOut = freopen( GraalOutFileName, "w+", stdout);
  }
  else
  {
    GraalStreamOut = GraalStreamErr;
  }

  if ( ( GraalStreamAll == (FILE *)NULL ) ||
       ( GraalStreamOut == (FILE *)NULL ) ||
       ( GraalStreamErr == (FILE *)NULL ) )
  {
    fprintf( stdout, "Grm: Unable to open trace window !\n");
    exit( 1 );
  }

  if ( GraalNormalMode )
  {
    unlink( GraalOutFileName ); 
  }

  unlink( GraalErrFileName );

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      GraalFlushErrorMessage                 |
|                                                             |
\------------------------------------------------------------*/

void GraalFlushErrorMessage()
{
  int Data;

  rdsbegin();

  fflush( GraalStreamErr );
  fseek( GraalStreamErr, 0L, 0 );

  while( ( Data = fgetc( GraalStreamErr ) ) != EOF )
  {
    fputc( Data, GraalStreamAll );
  }

  fclose( GraalStreamErr );

  GraalStreamErr = freopen( GraalErrFileName, "w+", stderr );

  if ( GraalNormalMode )
  {
    fflush( GraalStreamOut );
    fseek( GraalStreamOut, 0L, 0 );

    while( ( Data = fgetc( GraalStreamOut ) ) != EOF )
    {
      fputc( Data, GraalStreamAll );
    } 
  
    fclose( GraalStreamOut );

    GraalStreamOut = freopen( GraalOutFileName, "w+", stdout );
  }

  rdsend();
}

/*------------------------------------------------------------\
|                                                             |
|                      GraalGetErrorMessage                   |
|                                                             |
\------------------------------------------------------------*/

char *GraalGetErrorMessage()

{
  char *Message;
  long  Length;
  long  Index;
  int   Data;

  rdsbegin();

  GraalFlushErrorMessage();

  fflush( GraalStreamAll );

  Length = ftell( GraalStreamAll );

  if ( GraalErrorBuffer != (char *)NULL )
  {
    rdsfreeblock( GraalErrorBuffer );
  }

  GraalErrorBuffer = rdsallocblock( Length + 1 );
  Index            = 0;

  while ( ( ( Data = fgetc( GraalStreamAll ) ) != EOF ) &&
          ( Index  < Length ) )
  {
    GraalErrorBuffer[ Index++ ] = (char)Data;
  }

  rewind( GraalStreamAll );

  GraalErrorBuffer[ Index ] = '\0';

  if ( Index == 0 )
  {
    Message = (char *)NULL;
  }
  else
  {
    Message = GraalErrorBuffer;
  }

  rdsend();
  return( Message );
}

/*------------------------------------------------------------\
|                                                             |
|                      GraalGetInformations                   |
|                                                             |
\------------------------------------------------------------*/

char *GraalGetInformations()

{
  char *Scan;

  rdsbegin();

  GraalComputeBound();

  Scan = GraalInformationsBuffer;

  if ( GraalFigureMbk != (phfig_list *)NULL )
  {
    sprintf( Scan, "  FIGURE : %s\n\n  XAB1 : %d\n  YAB1 : %d\n  XAB2 : %d\n  YAB2 : %d\n\n",
             GraalFigureMbk->NAME,
             GraalFigureMbk->XAB1  / SCALE_X,
             GraalFigureMbk->YAB1  / SCALE_X,
             GraalFigureMbk->XAB2  / SCALE_X,
             GraalFigureMbk->YAB2  / SCALE_X  );
  }
  else
  {
    sprintf( Scan, "  FIGURE : No current figure !\n\n" );
  }

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  BOUNDING BOX : \n\n  XMIN : %d\n  YMIN : %d\n  XMAX : %d\n  YMAX : %d\n\n",
           GraalBoundXmin / RDS_LAMBDA, GraalBoundYmin / RDS_LAMBDA,
           GraalBoundXmax / RDS_LAMBDA, GraalBoundYmax / RDS_LAMBDA );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  WINDOWS : \n\n  XMIN : %d\n  YMIN : %d\n  XMAX : %d\n  YMAX : %d\n  DX : %d\n  DY : %d\n  SIDE : %d\n\n",
           GraalWindowXmin / RDS_LAMBDA, GraalWindowYmin / RDS_LAMBDA,
           GraalWindowXmax / RDS_LAMBDA, GraalWindowYmax / RDS_LAMBDA,
           GraalWindowDx               , GraalWindowDy,
           GraalWindowSide / RDS_LAMBDA );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  RDS TECHNOLOGY : %s\n\n", RDS_TECHNO_NAME );

  Scan = Scan + strlen( Scan );

  sprintf( Scan, "  GRAAL TECHNOLOGY : %s\n\n", GRAAL_TECHNO_NAME );

  rdsend();
  return( GraalInformationsBuffer );
}
