/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  NetOptim                                                   */
/*    Fichier :  no_asga.c                                                  */
/*                                                                          */
/*    (c) copyright 1992 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : N. Dictus                             le : 07/02/1995     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include MLO_H
#include LOG_H
#include MPH_H
#include BEH_H
#include "types_map.h"
#include "../net/no_type.h"
#include "../net/no_system.h"
#include "neto.h"
#include "../mapping/generic.h"
#include "util.h"
#include "no_asga.h"
#include "no_optim.h"

pTH phTH;

/*---------------------------------------------------------------------------
displayTab    : affichage de la structure ptype_list (tableau trie en fonction
		de Y et de X)
-----------------------------------------------------------------------------
retour        : affichage
---------------------------------------------------------------------------*/
void displayTab(tab)
ptype_list *tab;
{
ptype_list *t;

printf("---------- Tableau Placement ----------\n");
for(t = tab; t; t = t->NEXT)
   {
   ptype_list *elt;

   printf(" Y=%ld", t->TYPE);
   for(elt = (ptype_list *)t->DATA; elt; elt = elt->NEXT)
      {
      phins_list *phins = (phins_list *)elt->DATA;

      printf(" %s[%s](%ld)", phins->INSNAME, phins->FIGNAME, elt->TYPE);

	/* verification : les ecarts sont-ils bien calcules ? */
      if (elt->NEXT)
         if ((elt->TYPE+cellLoadSurface(phins->FIGNAME)*60) != elt->NEXT->TYPE)
            printf("ERROR calcul decalage : %s[%s](%ld)\n",
                   phins->INSNAME, phins->FIGNAME, t->TYPE);
      }
   printf("\n");
   }
printf("---------- END Tableau Placement ----------\n");
}

/*---------------------------------------------------------------------------
getTab   : recherche d'un elt dans la ptype_list tab triee suivant le TYPE: 
		TYPE = coordonnee Y de l'instance
		DATA = ptype_list triee des instances de coordonnee Y :
			TYPE = coordonnee X de l'instance
			DATA = pointeur sur l'instance (phins_list *)
-----------------------------------------------------------------------------
retour      : modele est inseree apres phins + instances a droite sont decalees
---------------------------------------------------------------------------*/
ptype_list *getTab(tab, phins)
ptype_list *tab;
phins_list *phins;
{
ptype_list *eltY, *eltX;

	/* recherche de la bonne ligne*/
eltY = getptype(tab, phins->YINS);

if (!eltY)
   {
   printf("ERROR getTab : coordinate Y=%ld does not exist\n", phins->YINS);
   exit(-1);
   }

	/* recherche de la bonne colone */
eltX = getptype((ptype_list *)eltY->DATA, phins->XINS);

if (!eltX)
   {
   printf("ERROR getTab : coordinate X=%ld (Y=%ld) does not exist\n",
          phins->XINS, phins->YINS);
   exit(-1);
   }

return eltX;
}

/*---------------------------------------------------------------------------
modifTab : insertion ou modif d'un elt dans la ptype_list tab triee
	   suivant le TYPE: 
		TYPE = coordonnee Y de l'instance
		DATA = ptype_list triee des instances de coordonnee Y :
			TYPE = coordonnee X de l'instance
			DATA = pointeur sur l'instance (phins_list *)
-----------------------------------------------------------------------------
retour   : modele est inseree apres phins + instances a droite sont decalees
---------------------------------------------------------------------------*/
void modifTab(modif, tab, loins, model)
int modif;
ptype_list *tab;
loins_list *loins;
char *model;
{
ptype_list *eltX;
long decalage = 0, coordX;
ptype_list *lst, *newPtype;
loins_list *buffer;
phins_list *phins = (phins_list *)searchTH(phTH, loins->INSNAME);

eltX = getTab(tab, phins);

if (eltX->NEXT)
   coordX = eltX->NEXT->TYPE;
else
   coordX = eltX->TYPE + (cellLoadSurface(phins->FIGNAME) * 5) / 21;

if (modif == 1)
   {
	/* adaptation en puissance du modele */
   addPuissante(NO_LOG, loins, model);
   decalage = (cellLoadSurface(model) - cellLoadSurface(loins->FIGNAME)) * 60;
   }
else
   {
	/* insertion du buffer dans la lofig */
   buffer = addBuffer(NO_LOG, loins, NULL, model);

	/* ajout de l'instance dans la netlist placee */
   NO_PHG->PHINS = addphins(NO_PHG, model, buffer->INSNAME,
                            phins->TRANSF, coordX, phins->YINS);

   if (NO_PHG->PHINS->INSNAME != buffer->INSNAME)
      printf("ERROR : Instance inseree n'est pas en tete\n");

	/* insertion dans la ligne */
   newPtype = addptype((ptype_list *)NULL, coordX, NO_PHG->PHINS);
   newPtype->NEXT = eltX->NEXT;
   eltX->NEXT = newPtype;

	/* calcul du decalage */
   decalage = cellLoadSurface(model) * 60;
   }

	/* on effectue les decalages des instances a droite de la modif */
for(lst = (modif == 1) ? eltX->NEXT : newPtype->NEXT; lst; lst = lst->NEXT)
   {
   lst->TYPE += decalage;
   ((phins_list *)lst->DATA)->XINS += decalage;
   }
}

/*---------------------------------------------------------------------------
addTab    : ajout d'un element dans la ptype_list tab triee suivant le TYPE: 
		TYPE = coordonnee Y de l'instance
		DATA = ptype_list triee des instances de coordonnee Y :
			TYPE = coordonnee X de l'instance
			DATA = pointeur sur l'instance (phins_list *)
-----------------------------------------------------------------------------
retour      : phins est inseree a sa place 
	      ATTENTION : l'element de coordonnee doit deja exister (creerTab)
---------------------------------------------------------------------------*/
void addTab(tab, phins)
ptype_list *tab;
phins_list *phins;
{
ptype_list *elt;

	/* recherche de la bonne ligne*/
elt = getptype(tab, phins->YINS);

if (!elt)
   {
   printf("ERROR addTab : coordinate Y=%ld does not exist\n", phins->YINS);
   exit(-1);
   }

	/* insertion dans la ligne */
elt->DATA = (void *)tripType((ptype_list *)elt->DATA,
                             phins->XINS, (void *)phins, NUP);
}

/*---------------------------------------------------------------------------
creerTab    : creation d'une ptype_list triee suivant le TYPE: 
		TYPE = coordonnee Y de l'instance
		DATA = ptype_list triee des instances de coordonnee Y :
			TYPE = coordonnee X de l'instance
			DATA = pointeur sur l'instance (phins_list *)
-----------------------------------------------------------------------------
retour      : la liste de ptype
---------------------------------------------------------------------------*/
ptype_list *creerTab()
{
phins_list *phins;
ptype_list *tableau = (ptype_list *)NULL;
pTH yTh = createTH(10);

for(phins = NO_PHG->PHINS; phins; phins = phins->NEXT)
   {
   if (searchTH(yTh, phins->YINS) == EMPTYTH)
      {
	/* la ligne de coordonne Y n'existe pas encore */
      tableau = tripType(tableau, phins->YINS, (void *)NULL, NDOWN);
      addTH(yTh, phins->YINS, 1);
      }
	/* ajout de l'element dans le tableau */
   addTab(tableau, phins);
   }

destroyTH(yTh);
return tableau;
}

/*---------------------------------------------------------------------------
fillTH    : remplit la TH : key = loins->INSNAME - value = *phins
-----------------------------------------------------------------------------
retour    : la TH est a jour.
---------------------------------------------------------------------------*/
pTH fillTH(lofig, phfig)
lofig_list *lofig;
phfig_list *phfig;
{
loins_list *loins;

pTH hash = createTH(2000);
for(loins = lofig->LOINS; loins; loins = loins->NEXT)
   {
   phins_list *phins;
 
   for(phins = phfig->PHINS; phins && phins->INSNAME != loins->INSNAME;
       phins = phins->NEXT)
      ;
   if (phins->INSNAME == loins->INSNAME)
      addTH(hash, loins->INSNAME, (int)phins);
   else
      printf("ERROR : %s does not exist in the placement\n",
             phins->INSNAME);
   }
return hash;
}

/*---------------------------------------------------------------------------
displayPlace    : affiche les instances d'une netlist placee
-----------------------------------------------------------------------------
retour          : affichage
---------------------------------------------------------------------------*/
void displayPlace(netlist)
phfig_list *netlist;
{
chain_list *mod;
phins_list *phins;
phseg_list *seg;
phvia_list *via;
phref_list *ref;
phcon_list *con;
int cptPhins = 0;

printf("---------- phfig %s (X1=%ld Y1=%ld X2=%ld Y2=%ld) ----------\n",
       netlist->NAME, netlist->XAB1, netlist->YAB1,
       netlist->XAB2, netlist->YAB2);

for(mod = netlist->MODELCHAIN; mod; mod = mod->NEXT)
   printf("  Modele %s\n", (char *)mod->DATA);

for(phins = netlist->PHINS; phins; phins = phins->NEXT)
   printf("  %d) Inst %s[%s] - X=%ld  Y=%ld\n", ++cptPhins,
          phins->INSNAME, phins->FIGNAME, phins->XINS, phins->YINS);

if (netlist->PHCON)
   printf("CONNECTORS !\n");

if (netlist->PHSEG)
   printf("SEGMENTS !\n");

if (netlist->PHVIA)
   printf("VIAS !\n");

if (netlist->PHREF)
   printf("REFS !\n");

printf("---------- END phfig %s ----------\n", netlist->NAME);
}

/*---------------------------------------------------------------------------
  --------------------------------------------------------------------------- */
int busy(loins, sCin, capaRout, newModel)
loins_list *loins;
long sCin, capaRout;
char **newModel;
{
if (sCin > 200)
   {
   *newModel = namealloc("b1_y");
   return 2;
   }
else
   if (sCin > 100)
      {
      if (loins->FIGNAME == namealloc("n1_y"))
         *newModel = namealloc("np1_y");
      else
         if (loins->FIGNAME == namealloc("a2_y"))
            *newModel = namealloc("a2p_y");
         else
            if (loins->FIGNAME == namealloc("na2_y"))
               *newModel = namealloc("na2p_y");
            else
               {
               *newModel = namealloc("p1_y");
               return 2;
               }
      return 1;
      }
return 0;
}

/*---------------------------------------------------------------------------
calculLargeurCapa :
-----------------------------------------------------------------------------
retour            :
---------------------------------------------------------------------------*/
void calculLargeurCapa(lofig, tab)
lofig_list *lofig;
ptype_list *tab;
{
loins_list *loins;

for(loins = lofig->LOINS; loins; loins = loins->NEXT)
   if (cellLoadType(loins->FIGNAME) != TIE)
      {
      char *newModel = (char *)NULL;
      int chgt = 0;
      locon_list *con, *conOut = (locon_list *)NULL;
      long sCin = 0;
      long capaRout = 0;
      long minX = 0, minY = 0;
      long maxX = 0, maxY = 0;
      int nbFanout = 0;
      phins_list *phins = (phins_list *)searchTH(phTH, loins->INSNAME);

      printf(" INS %s[%s] X=%ld  Y=%ld\n",
             phins->INSNAME, phins->FIGNAME,
             phins->XINS, phins->YINS);

      minX = phins->XINS;
      minY = phins->YINS;
      maxX = phins->XINS;
      maxY = phins->YINS;
      for(con = loins->LOCON; con && !conOut; con = con->NEXT)
         {
         if ((con->DIRECTION == OUT) ||
             (con->DIRECTION == INOUT) ||
             (con->DIRECTION == TRISTATE) ||
             (con->DIRECTION == TRANSCV))
            {
            ptype_list *liste;
            chain_list *lstCon;

                /* connecteur de sortie */
            conOut = con;
            liste = getptype(conOut->SIG->USER, (long)LOFIGCHAIN);

            if (!liste)
               {
               PRINTF("ASGA ERROR : No type LOFIGCHAIN for %s\n",
                      loins->INSNAME);
               exit(1);
               }

           /* pour le connecteur de sortie, calculer son fanout (somme CIN) */
            for(lstCon = (chain_list *)liste->DATA; lstCon;
                lstCon = lstCon->NEXT)
               {
               locon_list *conAux = (locon_list *)lstCon->DATA;

               if (conAux->ROOT != (void *)loins)
                  {
                  if (conAux->TYPE == EXTERNAL)
                     {
                      	/* connecteur externe */
                     printf("  in %s\n", conAux->NAME);

                     sCin += cellLoadCin((char *)NULL, conAux->NAME);
                     nbFanout++;
                     }
                  else
                   	/* connecteur interne */
                     if (conAux->DIRECTION == IN)
                        {
                        loins_list *cellOut = (loins_list *)conAux->ROOT;
                        phins_list *phOut   = (phins_list *)searchTH(phTH,
                                                        cellOut->INSNAME);
  
                        printf("  %s[%s] X=%ld  Y=%ld\n",
                               phOut->INSNAME, phOut->FIGNAME,
                               phOut->XINS, phOut->YINS);

                           /* portes attaquees par l'instance a traiter */
                        sCin += cellLoadCin(cellOut->FIGNAME, conAux->NAME);
                        nbFanout++;
                        if (phOut->XINS < minX)
                           minX = phOut->XINS;
                        if (phOut->YINS < minY)
                           minY = phOut->YINS;
                        if (phOut->XINS > maxX)
                           maxX = phOut->XINS;
                        if (phOut->YINS > maxY)
                           maxY = phOut->YINS;
                        }
                     else
                        {
                        if ((conAux->DIRECTION != TRANSCV) &&
                            (conAux->DIRECTION != TRISTATE))
                           PRINTF("TIMING ANALYSIS : Unknown connector '%s' of %s[%s]\n",
                                  conAux->NAME, ((loins_list *)conAux->ROOT)->INSNAME,
                               ((loins_list *)conAux->ROOT)->FIGNAME);
                        }
                  }
               }   	/* fin du calcul de la somme CIN */
            }
         }		/* fin parcours des connecteurs de l'instance */

      chgt = busy(loins, sCin, capaRout, &newModel);

      printf("   X=%ld  Y=%ld\n   BUSY Chgt %d - sCin=%ld - newModel=%s\n",
             maxX-minX, maxY-minY,
             chgt, sCin, newModel);

      if (chgt > 0)
         modifTab(chgt, tab, loins, newModel);
      }			/* fin parcours des instances */
}

/*---------------------------------------------------------------------------
mainOptimAsga   :
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void mainOptimAsga()
{
ptype_list *tabl;

phTH = fillTH(NO_LOG, NO_PHG);

/*
displayPlace(NO_PHG);
*/
displayTab(tabl = creerTab());

calculLargeurCapa(NO_LOG, tabl);

	/* !!! Il faut verifier l'abutment box !!! */
/*
displayTab(tabl);
displayPlace(NO_PHG);
*/
destroyTH(phTH);
/*
*/
}
