/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  NetOptim                                                   */
/*    Fichier :  util.c                                                     */
/*                                                                          */
/*    (c) copyright 1992 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : N. Dictus                             le : 10/04/1992     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include MLO_H
#include LOG_H
#include MPH_H
#include BEH_H
#include "types_map.h"
#include "no_optim.h"
#include "util.h"
#include "incdelai.h"
#include "../net/no_type.h"
#include "../net/no_system.h"
#include "../mapping/generic.h"

/*---------------------------------------------------------------------------
cheminCritique : affichage du chemin critique
-----------------------------------------------------------------------------
retour         : nom des portes des sorties vers les entrees
---------------------------------------------------------------------------*/
void cheminCritique(inst, type, delai, min)
loins_list *inst;
int type;	/* NUP ou NDOWN */
long delai;
short min;
{
ptype_list *chemin;
chain_list *lstCon;
chain_list *l, *listePorte = (chain_list *)NULL;
char *nameConExt = NULL;
loins_list *reg = (loins_list *)NULL;
int nbCouches = 1;
int typeParam = type;
loins_list *porte = inst;
int val;
losig_list *sigOut;
ptype_list *liste;

val = searchTH(loconOutTH, inst->INSNAME);
if (val != EMPTYTH)
   {
   sigOut = ((locon_list *)val)->SIG;
   liste = getptype(sigOut->USER, (long)LOFIGCHAIN);

   if (!liste)
      {
      printf("ERROR cheminCritique : no type LOFIGCHAIN for %s[%s]\n",
             inst->INSNAME, inst->FIGNAME);
      exit(-1);
      }
   }
else
   {
   printf("ERROR cheminCritique : no type LOCONOUT for %s[%s]\n",
          inst->INSNAME, inst->FIGNAME);
   exit(-1);
   }

PRINTI("\n");
for(lstCon = (chain_list *)liste->DATA; lstCon; lstCon = lstCon->NEXT)
   {
   locon_list *conAux = (locon_list *)lstCon->DATA;

   if (conAux->ROOT != (void *)inst)
      if (conAux->TYPE == EXTERNAL)
         nameConExt = conAux->NAME;
      else
         if (cellLoadType(((loins_list *)conAux->ROOT)->FIGNAME) == REG)
            reg = ((loins_list *)conAux->ROOT);
   }

if (!(reg || nameConExt))
      {
      PRINTF("ERROR Timing : critical path neither on register nor on external connector (on %s)\n", inst->INSNAME);
      exit(1);
      }

listePorte = addchain(listePorte, (void *)porte);
if (min == 0)
   {
   while (chemin = getptype(porte->USER, (long)((type == NUP)?LOINSUP:LOINSDW)))
      {
      if ((type == NUP) && (cellLoadPolarity(porte->FIGNAME) == NEG))
         type= NDOWN;
      else
         if ((type == NDOWN) && (cellLoadPolarity(porte->FIGNAME) == NEG))
            type= NUP;
      porte = (loins_list *)chemin->DATA;
      listePorte = addchain(listePorte, (void *)porte);
      nbCouches++;
      }
   chemin = getptype(porte->USER, (long)((type == NUP)?LOCONUP:LOCONDW));
   }
else
   {
		/* chemin critique minimal */
   while (chemin = getptype(porte->USER, (long)((type == NUP)?LOINSMINUP:LOINSMINDW)))
      {
      if ((type== NUP) && (cellLoadPolarity(porte->FIGNAME) == NEG))
         type= NDOWN;
      else
         if ((type== NDOWN) && (cellLoadPolarity(porte->FIGNAME) == NEG))
            type= NUP;
      porte = (loins_list *)chemin->DATA;
      listePorte = addchain(listePorte, (void *)porte);
      nbCouches++;
      }
   chemin = getptype(porte->USER, (long)((type == NUP)?LOCONMINUP:LOCONMINDW));
   }

PRINTI("\t%sCritical path %s = %d ps\n",
       (min)?"Minimal ":"", (typeParam == NUP)?" UP ":"DOWN", delai);

if (chemin)
   {
	/* connecteur en debut de chemin */
   locon_list *con = (locon_list *)chemin->DATA;

   PRINTI("\t\tfrom external connector %s\n", con->NAME);
   }
else
   {
	/* porte en debut de chemin */
   if (cellLoadType(porte->FIGNAME) == REG)
      {
      PRINTI("\t\tfrom register %s\n", porte->INSNAME);

	    /* suppression du registre dans le chemin critique */
      nbCouches--;
      listePorte = listePorte->NEXT; 
      }
   else
      if (cellLoadType(porte->FIGNAME) == CONST)
         PRINTI("\t\tfrom gate %s\n", porte->INSNAME);

   }

if (nameConExt)
   PRINTI("\t\tto external connector %s\n", nameConExt);
else
   if (reg)
      PRINTI("\t\tto register %s\n", reg->INSNAME);

if (chemin)
   PRINTI("[%s]->", ((locon_list *)chemin->DATA)->NAME);
else
   if (cellLoadType(porte->FIGNAME) == REG)
      PRINTI("[%s]->", porte->INSNAME);

for(l = listePorte; l; l = l->NEXT)
   {
   loins_list *gate = (loins_list *)l->DATA;

   if (NO_TRACE >= 1)
      displayTimingInst(gate, 0, 1);
   else
      PRINTI("%s->", gate->INSNAME);
/*
*/
   }
if (nameConExt)
   PRINTI("[%s]\n", nameConExt);
else
   PRINTI("[%s]\n", reg->INSNAME);
PRINTI("\t=> %d gates\n\n", nbCouches);
}

/*---------------------------------------------------------------------------
computFanout	: calcul le fanout d'une porte
-----------------------------------------------------------------------------
retour          : ptype_list de loins_list (DATA) et cin (TYPE)
		  chargeOut = somme des capas vues a la sortie
                  nbFanout = nombre de portes vues a la sortie
---------------------------------------------------------------------------*/
ptype_list *computFanout(porte, con, sCin, nbFanout, buffer)
loins_list *porte;
locon_list *con;
long *sCin, *nbFanout, *buffer;
{
losig_list *sigOut;
ptype_list *liste;
chain_list *lstCon;
ptype_list *lstFanout = (ptype_list *)NULL;

if (porte)
   {
   int val = searchTH(loconOutTH, porte->INSNAME);

   if (val == EMPTYTH)
      {
      printf("ERROR computFanout : no type LOCONOUT for %s[%s]\n",
             porte->INSNAME, porte->FIGNAME);
      exit(-1);
      }
   else
      {
      sigOut = ((locon_list *)val)->SIG;
      }
   }
else
   sigOut = con->SIG;

liste = getptype(sigOut->USER, (long)LOFIGCHAIN);

if (!liste)
   {
   PRINTF("ERROR computFanout : No type LOFIGCHAIN for %s\n",
          getsigname(sigOut));
   exit(-1);
   }

	/* initialisation de sCin, buffer et nbFanout a 0 */
*sCin = 0;
*buffer = 0;
*nbFanout = 0;

	/* parcours du fanout et calcul de la somme des capas */
for(lstCon = (chain_list *)liste->DATA; lstCon; lstCon = lstCon->NEXT)
   {
   locon_list *conAux = (locon_list *)lstCon->DATA;

   if (conAux->ROOT != (void *)porte)
      {
      if (conAux->TYPE == EXTERNAL)
         {
                /* connecteur externe */
         *sCin += cellLoadCin((char *)NULL, conAux->NAME);
         }
      else  
         {
                /* connecteur interne */
         if (conAux->DIRECTION == IN)
            {
            loins_list *cellOut = (loins_list *)conAux->ROOT;
            long cin = cellLoadCin(cellOut->FIGNAME, conAux->NAME);
 
            *sCin += cin;
            (*nbFanout)++;
            lstFanout = addptype(lstFanout, cin, (void *)cellOut);
            if (cellLoadType(cellOut->FIGNAME) == BUFF)
               (*buffer)++;
            }
         else
            PRINTI("ERROR computFanout : Unknown internal connector '%s'\n",
                   conAux->NAME);
         }
      }
   }
return lstFanout;
}

/*---------------------------------------------------------------------------
resistanceNot   :  renvoie la resistance de sortie du NOT (ou 0 si pas de NOT)
-----------------------------------------------------------------------------
retour          : int
---------------------------------------------------------------------------*/
long resistanceNot(front)
int front;
{
long rNot = 99999;
cellList *cells = NO_CELLS;

        /* recherche de la cellule logique NOT */
while (cells && cells->nomlog != NOT1)
      cells = cells->NEXT;

        /* si NOT existe, calcul de la + ptite resistance parmi tous les not */
if (cells)
   {
   ptype_list *c;

   for(c = cells->surface_tech ;c; c = c->NEXT)
      {
      long res;
      char *RES = (char *)mbkalloc(7 + strlen(cells->entrees->name));

      if (front == NDOWN)
         {
         sprintf(RES, "rdown_%s",cells->entrees->name);
         res = (long)searchGeneric(cells, c->DATA, RES);
         if (res < rNot)
            rNot = res;
#if MACHINE != pc
         mbkfree(RES);
#endif
         }
      else
         {
         sprintf(RES, "rup_%s",cells->entrees->name);
         res = (long)searchGeneric(cells, c->DATA, RES);
         if (res < rNot)
            rNot = res;
#if MACHINE != pc
         mbkfree(RES);
#endif
         }
      }
   return rNot;
   }
else
   return 1000;
}

/*---------------------------------------------------------------------------
getInfoInst  :
-----------------------------------------------------------------------------
retour : un long 
---------------------------------------------------------------------------*/
long getInfoInst(inst, typeInfo)
loins_list *inst;
long typeInfo;
{
int val = searchTH(loconOutTH, inst->INSNAME);

if (val != EMPTYTH)
   {
   locon_list *conOut = (locon_list *)val;
   ptype_list *listeInfo = getptype(conOut->SIG->USER, typeInfo);

   if (listeInfo)
      return (long)listeInfo->DATA;
   PRINTF("ERROR : no timing informations on %s [%s]\n",
          inst->INSNAME, inst->FIGNAME);
   exit(1);
   }
else
   {
   PRINTF("ERROR : no timing informations on %s [%s]\n",
          inst->INSNAME, inst->FIGNAME);
   exit(1);
   }
}

/*---------------------------------------------------------------------------
getDelayIn  : retourne le delai d'une entree retardee
-----------------------------------------------------------------------------
retour : un long ou 0.
---------------------------------------------------------------------------*/
long getDelayIn(liste, conName)
ptype_list *liste;
char *conName;
{
ptype_list *l;

for(l = liste; l; l = l->NEXT)
   if ((char *)l->DATA == conName)
      return l->TYPE;
return (long)0;
}

/*---------------------------------------------------------------------------
getResistIn : retourne le delai d'une entree retardee
-----------------------------------------------------------------------------
retour : un long ou 0.
---------------------------------------------------------------------------*/
long getResistIn(liste, conName)
ptype_list *liste;
char *conName;
{
ptype_list *l;

for(l = liste; l; l = l->NEXT)
   if ((char *)l->DATA == conName)
      return l->TYPE;
return (long)0;
}

/*---------------------------------------------------------------------------
tripType : ajoute dans une ptype_list un elt a sa place
           (tri des TYPE)
-----------------------------------------------------------------------------
retour   : ptype_list triee suivant sens (NUP ou NDOWN)
---------------------------------------------------------------------------*/
ptype_list *tripType(liste, type, data, sens)
ptype_list *liste;
long type;
void *data;
int sens;
{
if (!liste)
   return addptype(liste, type, data);
 
if ((sens == NUP) && (type < liste->TYPE))
   return addptype(liste, type, data);
else
   if ((sens == NDOWN) && (type >= liste->TYPE))
      return addptype(liste, type, data);
   else
      {
      ptype_list *p1 = liste;
      ptype_list *p2 = liste->NEXT;
      ptype_list *new = (ptype_list *)mbkalloc(sizeof(ptype_list));

      while (((sens == NUP  ) && p2 && (p2->TYPE < type)) || 
             ((sens == NDOWN) && p2 && (p2->TYPE > type)))
         {
         p2 = p2->NEXT;
         p1 = p1->NEXT;
         }
      new->TYPE = type;
      new->DATA = data;
      new->NEXT = p2;
      p1->NEXT = new;
      return liste;
      }
}

/*---------------------------------------------------------------------------
displayTimingInst : affiche les retards, temps requis, slack d'une instance
-----------------------------------------------------------------------------
retour        : RIEN. affichage.
---------------------------------------------------------------------------*/
void displayTimingInst(inst, tout, del)
loins_list *inst;
  /* si tout = 1 tous les signaux sont affiches, sinon que la sortie */
short tout, del;
{
locon_list *con;
ptype_list *userCriticUp = getptype(inst->USER, (long)LOINSUP);
ptype_list *userCriticDw = getptype(inst->USER, (long)LOINSDW);

for(con = inst->LOCON; con; con = con->NEXT)
   if (!isvdd(con->NAME) && !isvss(con->NAME))
      if (con->DIRECTION == OUT || con->DIRECTION == TRANSCV ||
          con->DIRECTION == TRISTATE || tout)
         {
         ptype_list *userUp = getptype(con->SIG->USER, (long)RETARDUP);
         ptype_list *userDw = getptype(con->SIG->USER, (long)RETARDDW);
         ptype_list *profond = getptype(con->SIG->USER, (long)PROF);
         long capa   = (long)searchTH(capaTH, getsigname(con->SIG));
      
         ptype_list *requiUp = getptype(con->SIG->USER, (long)REQUIUP);
         ptype_list *requiDw = getptype(con->SIG->USER, (long)REQUIDW);
         ptype_list *slackUp = getptype(con->SIG->USER, (long)SLACKUP);
         ptype_list *slackDw = getptype(con->SIG->USER, (long)SLACKDW);
         ptype_list *critic   = getptype(con->SIG->USER, (long)CRITIC);

         if (userUp)
            PRINTI("  %s [%s/%s] -\tDELAY  - UP = %d - DW = %d - CAPA = %d\n",
                   inst->INSNAME, inst->FIGNAME, con->NAME,
                   (long)userUp->DATA, (long)userDw->DATA,
                   capa);
         else
            PRINTF("ERROR : no delay for %s [%s/%s]\n",
                   inst->INSNAME, inst->FIGNAME, con->NAME);
         if (!del)
            {
            if (requiUp)
               PRINTI("\t\t\tREQUI  - UP = %d - DW = %d\n",
                      (long)requiUp->DATA, (long)requiDw->DATA);
            else
               PRINTF("ERROR : no required time for %s [%s/%s]\n",
                      inst->INSNAME, inst->FIGNAME, con->NAME);
            if (slackUp)
               {
               PRINTI("\t\t\tSLACK  - UP = %d - DW = %d\n",
                      (long)slackUp->DATA, (long)slackDw->DATA);
               PRINTI("\t\t\tRESIST - UP = %d - DW = %d\n",
                      cellLoadR(inst->FIGNAME, (char *)NULL, NUP),
                      cellLoadR(inst->FIGNAME, (char *)NULL, NDOWN));
               }
            else
               PRINTF("ERROR : no slack for %s [%s/%s]\n",
                      inst->INSNAME, inst->FIGNAME, con->NAME);
            if (profond && critic)
               PRINTI("\t\t\tDEPTH  = %d -  CRITICALITY = %d\n",
                      (long)profond->DATA, (long)critic->DATA);
            else
               PRINTF("ERROR : no depth or criticality for %s [%s/%s]\n",
                      inst->INSNAME, inst->FIGNAME, con->NAME);
            }
         }
if (cellLoadType(inst->FIGNAME) != REG && cellLoadType(inst->FIGNAME) != CONST)
   {
   if (!del)
      {
      if (userCriticUp)
         PRINTI("\t\t\tCRITIC - UP = %s", 
                ((loins_list *)userCriticUp->DATA)->INSNAME);
      else
         PRINTI("\t\t\tCRITIC - UP = %s", 
                ((locon_list *)getptype(inst->USER, (long)LOCONUP)->DATA)->NAME);
      if (userCriticDw)
         PRINTI(" - DW = %s\n", ((loins_list *)userCriticDw->DATA)->INSNAME);
      else
         PRINTI(" - DW = %s\n", ((locon_list *)getptype(inst->USER, (long)LOCONDW)->DATA)->NAME);
      }
   }
}

/*---------------------------------------------------------------------------
displayTiming : affiche des retards sur les signaux des instances de la LOFIG
-----------------------------------------------------------------------------
retour        : RIEN. affichage
---------------------------------------------------------------------------*/
void displayTiming(lofig, tout)
lofig_list *lofig;
short tout;  /* tout = 1, tous les signaux sont affiches, sinon que la sortie */
{
loins_list *inst;

PRINTF("Display timing results\n");

for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   displayTimingInst(inst, tout, 0);

PRINTF("End display timing results\n");
}

/*---------------------------------------------------------------------------
afficheChemins	: affiche les retards sur toutes les sorties
-----------------------------------------------------------------------------*/
void afficheChemins(fig)
lofig_list *fig;
{
loins_list *inst;

for(inst = fig->LOINS; inst; inst = inst->NEXT)
   {
   if (cellLoadType(inst->FIGNAME) == REG)
      {
/*
      int val = searchTH(loconOutTH, inst->INSNAME);
      losig_list *sigOut = ((locon_list *)val)->SIG;
      ptype_list *liste = getptype(sigOut->USER, (long)LOFIGCHAIN);
      chain_list *lstCon;
*/
      }
   else
      {
      int val = searchTH(loconOutTH, inst->INSNAME);
      losig_list *sigOut = ((locon_list *)val)->SIG;

      if (sigOut->TYPE == EXTERNAL)
         {
         long delaiUp = (long)(getptype(sigOut->USER, (long)RETARDUP)->DATA);
         long delaiDw = (long)(getptype(sigOut->USER, (long)RETARDDW)->DATA);

         if (delaiUp > delaiDw)
            cheminCritique(inst, NUP, delaiUp, 0);
         else
            cheminCritique(inst, NDOWN, delaiDw, 0);
         }
      }
   }
}

/*---------------------------------------------------------------------------
countType	: 
-----------------------------------------------------------------------------
retour		: ptype_list
---------------------------------------------------------------------------*/
ptype_list *countType(liste, data)
ptype_list *liste;
void *data;
{
ptype_list *l;

for(l = liste; (l && (l->DATA != data)); l = l->NEXT)
   ;
if (l)
   {
   l->TYPE = l->TYPE + 1;
   return liste;
   }
else
   return addptype(liste, 1, data);
}

/*---------------------------------------------------------------------------
displayFanout	: affiche le fanout de chaque porte
-----------------------------------------------------------------------------
retour		: void
---------------------------------------------------------------------------*/
void displayFanout(lofig, level)
lofig_list *lofig;
short level;
{
locon_list *con;
loins_list *inst;
int nbPbs = 0;

statistics(lofig, 0);

	/* parcours des connecteurs de la figure */
for(con = lofig->LOCON; con; con = con->NEXT)
   {
   int nbOut = 0;
   int sCin = 0;
   ptype_list *fanout = (ptype_list *)NULL;
   ptype_list *fan = (ptype_list *)NULL;
   ptype_list *liste;
   chain_list *l;

   if (!isvdd(con->NAME) && !isvss(con->NAME))
      {
      if (con->DIRECTION == IN)
         {
         if (level < 3)
            PRINTI(" CONNECTOR : %s\n", con->NAME);

		/* liste des liaisons signaux --> connecteurs */
         liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

         for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
            {
            locon_list *conAux = (locon_list *)(l->DATA);

            if (conAux->ROOT != (void *)lofig)
               {
               if (conAux->TYPE == EXTERNAL)
                  {
			/* connecteurs externes : E/S primaires */
                  PRINTI("\tExternal %s\n", conAux->NAME);
                  }
               else
                  {
   			/* connecteur interne */
                  loins_list *cellOut = (loins_list *)conAux->ROOT;

                        /* portes attaquees par l'instance a traiter */
                  sCin += cellLoadCin(cellOut->FIGNAME, conAux->NAME);
                  nbOut++;
                  fanout = countType(fanout, (void *)cellOut->FIGNAME);
                  }
               }
            }
         PRINTI("   %d gates driven - Sun cin = %d\n", nbOut, sCin);
         for(fan = fanout; fan; fan = fan->NEXT)
            PRINTI("\tModel %s = %d\n", (char *)fan->DATA, fan->TYPE);
         }
      }
   }	/* fin parcours connecteurs */

	/* parcours des instances de la figure */
for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   {
   int nbOut = 0;
   int sCin = 0;
   ptype_list *fanout = (ptype_list *)NULL;
   ptype_list *fan = (ptype_list *)NULL;

   if (level < 3) PRINTI("  INSTANCE : %s [%s] Fanout Max = %d\n",
                          inst->INSNAME,inst->FIGNAME,
                          cellLoadFanout(inst->FIGNAME));

	/* parcours des connecteurs de l'instance */
   for(con = inst->LOCON; con; con = con->NEXT)
      {
      ptype_list *liste;
      chain_list *l;

      if (!isvdd(con->NAME) && !isvss(con->NAME))
         {
         if (con->DIRECTION == OUT)
            {
		/* liste des liaisons signaux --> connecteurs */
            liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

            for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
               {
               locon_list *conAux = (locon_list *)(l->DATA);

               if (conAux->ROOT != (void *)inst)
                  {
                  if (conAux->TYPE == EXTERNAL)
                     {
			/* connecteurs externes : E/S primaires */
                     PRINTI("\tExternal %s\n", conAux->NAME);
                     }
                  else
                     {
   			/* connecteur interne */
                     loins_list *cellOut = (loins_list *)conAux->ROOT;

                        /* portes attaquees par l'instance a traiter */
                     sCin += cellLoadCin(cellOut->FIGNAME, conAux->NAME);
                     nbOut++;
                     fanout = countType(fanout, (void *)cellOut->FIGNAME);
                     }
                  }
               }
            }
         }
      }	/* fin parcours connecteurs */
   PRINTI("   %d gates driven - Sun cin = %d\n", nbOut, sCin);
   for(fan = fanout; fan; fan = fan->NEXT)
      PRINTI("\tModel %s = %d\n", (char *)fan->DATA, fan->TYPE);
   }
}

/*---------------------------------------------------------------------------
displayLofig	: affiche une lofig (INSTANCE - CONNECTEUR - SIGNAL - 
                                     CONNECTEUR - INSTANCE)
                  level = 0 -> liste des instances et leurs interconnexions
                  level = 1 -> liste des instances et leurs interconnexions
                               + somme des Cin
                  level = 2 -> liste des instances et leurs interconnexions
                               + Resistance * somme des Cin
                  level = 3 -> liste des instances a problemes
                  level = 4 -> liste des instances et les portes qu'elles
                               attaquent
-----------------------------------------------------------------------------
retour		: void
---------------------------------------------------------------------------*/
void displayLofig(lofig, level)
lofig_list *lofig;
short level;
{
locon_list *con;
loins_list *inst;
int nbPbs = 0;

statistics(lofig, 0);

	/* parcours des instances de la figure */
for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   {
   int nbOut = 0;
   ptype_list *fanout = NULL;

   if (level < 3) PRINTI("  INSTANCE : %s [%s] Fanout Max = %d\n",
                          inst->INSNAME,inst->FIGNAME,
                          cellLoadFanout(inst->FIGNAME));

	/* parcours des connecteurs de l'instance */
   for(con = inst->LOCON; con; con = con->NEXT)
      {
      ptype_list *liste;
      chain_list *l;

      if (!isvdd(con->NAME) && !isvss(con->NAME))
         {
         if (level < 3) 
            PRINTI("   CONNECTOR : %s (%c) --- SIGNAL %s --- ",
                   con->NAME,con->DIRECTION,
                   (char *)(con->SIG->NAMECHAIN->DATA));

         if (con->DIRECTION == OUT && level < 3)
            PRINTI("CELLS :");

		/* liste des liaisons signaux --> connecteurs */
         liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

         for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
            {
            if (((locon_list *)(l->DATA))->ROOT != (void *)inst)
               {
               if (((locon_list *)(l->DATA))->TYPE == EXTERNAL)
                  {
			/* connecteurs externes : E/S primaires */
                  if (level < 3)
                     if (con->DIRECTION == OUT)
                        PRINTI(" Extern");
                     else
                        PRINTI("CELLS : Extern");
                  if (con->DIRECTION == OUT)
                     {
                     fanout = addptype(fanout,
                                       (long)(((locon_list *)(l->DATA))->NAME),
                                       (void *)NULL);
                     nbOut++;
                     }
                  }
               else
                  {
   			/* connecteur interne */
                  loins_list *cellOut =
                                 (loins_list *)((locon_list *)(l->DATA))->ROOT;

		    /* DETERMINE LE FANOUT DE CHAQUE CELLULE */

                  if (con->DIRECTION == OUT &&
                      ((locon_list *)(l->DATA))->DIRECTION == IN)
                     {
                     fanout = addptype(fanout,
                                       (long)(((locon_list *)(l->DATA))->NAME),
                                       (void *)cellOut);
                     nbOut++;
                     if (level < 3) PRINTI(" %s,",cellOut->INSNAME);
                     }
                  }
               }
            }
         if (level < 3) PRINTI("\n");
         }
      }	/* fin parcours connecteurs */

   if (fanout)
      {
      int somCin = 0, nbDrived = 0;
      int charge = 0;
      ptype_list *f;

      if (level == 4)
         {
         PRINTI("  INSTANCE : %s [%s] Fanout Max = %d, drives %d gate%s :\n",
                inst->INSNAME,inst->FIGNAME,cellLoadFanout(inst->FIGNAME),
                nbOut,(nbOut>1)?"s":"");
         }
      if (level < 3) PRINTI("     %s drives %d gate%s :",
                             inst->INSNAME,nbOut,(nbOut > 1)? "s":"");
      for(f = fanout; f; f = f->NEXT)
         {
         loins_list *att = (loins_list *)f->DATA;

		/* compteur de portes attaquees */
         nbDrived ++;

         if (level < 3) 
            if (att)
               PRINTI(" %s/%s -",att->INSNAME, (char *)(f->TYPE));
            else
               PRINTI(" EXTERN/%s -",(char *)f->TYPE);

         if (level == 4) 
            if (att)
               PRINTI("    %s [%s/%s] Cin = %d\n",att->INSNAME,att->FIGNAME,
                      (char *)(f->TYPE),
                      cellLoadCin(att->FIGNAME,(char *)(f->TYPE)));
            else
               PRINTI("    EXTERN [CONNECTOR %s] Cin = %d\n", (char *)(f->TYPE),
                      cellLoadCin((char *)NULL,(char *)f->TYPE));
         if (level > 0)
            if (att)
               somCin += cellLoadCin(att->FIGNAME, (char *)f->TYPE);
            else
               somCin += cellLoadCin((char *)NULL, (char *)f->TYPE);
         }
      if (level == 0)
         PRINTI("\n");
      if (level == 1)
         PRINTI("\n     Sum Cin = %d\n", 
                somCin);
      charge = MAX(cellLoadR(inst->FIGNAME,NULL,NDOWN),
                   cellLoadR(inst->FIGNAME,NULL,NUP)) * somCin ;
      if (level == 2)
         {
         PRINTI("\n     R Max x Sum Cin = %d x %d = %d - RC Max = %d\n",
                MAX(cellLoadR(inst->FIGNAME,NULL,NDOWN),
                    cellLoadR(inst->FIGNAME,NULL,NUP)),
                somCin, charge, 800);
         }
      if (level >= 3 && somCin > cellLoadFanout(inst->FIGNAME))
         {
         nbPbs++;

         if (level == 3)
            {
            PRINTI("  INSTANCE : %s [%s] Fanout max = %d\n",
                   inst->INSNAME,inst->FIGNAME, cellLoadFanout(inst->FIGNAME));
            PRINTI("      Sum Cin = %d, %d gate%s driven\n",
                   somCin, nbDrived, nbDrived>1?"s":"");
            }
         }
      if (level == 4)
         PRINTI("      Sum Cin = %d%s\n",somCin,
                (somCin>cellLoadFanout(inst->FIGNAME))?"  ! ##### !":"");
      }
   if (level < 3) 
      PRINTI("\n");
   }
if (level >= 3)
   PRINTI("\n Number of problem%s = %d\n\n",nbPbs>1?"s":"",nbPbs);
}

/*--------------------------------------------------------------------------
displaySol 	: affichage des solutions d'optimisation
----------------------------------------------------------------------------
retour 		: 
--------------------------------------------------------------------------*/
void displaySol(sol)
sol_list *sol;
{
ptype_list *p, *name;

printf("Solution ");
switch (sol->TYPE) 
  {
  case BUN : printf("Buffer with NOT\n"); break;
  case BUF : printf("BUFFER\n"); break;
  case DUP : printf("DUPLICATION\n"); break;
  }
printf("Number : %d\n", sol->NOMBRE);
printf("Cost in area : %d\n", sol->SURFACE);
if (sol->ADAPT)
   printf("cellule adaptee = %s\n", sol->ADAPT);
printf("Cost in cin : %d\n", sol->DIFFCIN);
printf("somme cin : %d\n", sol->SUMCIN);

for(p = sol->ARBRE, name = sol->NAME; p; p = p->NEXT, name = name->NEXT)
   {
   ptype_list *q;

   printf("     * %ld [%s]\n",name->TYPE, (char *)name->DATA);
   for(q = (ptype_list *)p->DATA; q; q=q->NEXT)
      if (q->DATA)
         printf("   porte %s - in %s - cin %d\n",((loins_list *)q->DATA)->INSNAME,
             (char *)q->TYPE, cellLoadCin(((loins_list *)q->DATA)->FIGNAME,
                                          (char *)q->TYPE));
      else
         printf("   con = %s - cin %d\n",q->TYPE,
                cellLoadCin((char *)NULL, (char *)q->TYPE));
   }
}
