/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  NetOptim                                                   */
/*    Fichier :  no_optim.c                                                 */
/*                                                                          */
/*    (c) copyright 1992 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : N. Dictus                             le : 10/04/1992     */
/*                                                                          */
/*    Modifie par : N. Dictus (ajout buffers inverseurs)le : 22/12/1994     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include MLO_H
#include LOG_H
#include MPH_H
#include BEH_H
#include "types_map.h"
#include "../net/no_type.h"
#include "../net/no_system.h"
#include "neto.h"
#include "../mapping/generic.h"
#include "incdelai.h"
#include "no_optim.h"
#include "no_global.h"

/*---------------------------------------------------------------------------
addmodel        : ajoute un modele s'il n'existe pas dans la liste des modeles
-----------------------------------------------------------------------------
retour          : chain_list * des modeles utilises dans la lofig
---------------------------------------------------------------------------*/
chain_list *addmodel(liste, modele)
chain_list *liste;
char *modele;
{
chain_list *l;

for(l = liste; l; l = l->NEXT)
   if (modele == (char *)l->DATA)
      return liste;
return addchain(liste, (void *)modele);
}

/*---------------------------------------------------------------------------
clearmodel : remet a jour la liste des modeles
-----------------------------------------------------------------------------
retour     : 
---------------------------------------------------------------------------*/
void clearmodel(ptfig)
lofig_list *ptfig;
{
loins_list *liste;

freechain(ptfig->MODELCHAIN);
ptfig->MODELCHAIN = (chain_list *)NULL;

for(liste = ptfig->LOINS; liste; liste = liste->NEXT)
   ptfig->MODELCHAIN = addmodel(ptfig->MODELCHAIN, liste->FIGNAME);
}

/*---------------------------------------------------------------------------
adaptePuissance : recherche la porte la plus appropriee parmi les cellules phy
                  fanout >= sCin & surface minimale
-----------------------------------------------------------------------------
retour          : char * = le nom de modele de la porte puissante
---------------------------------------------------------------------------*/
char *adaptePuissance(liste, sCin)
cellList *liste;
long sCin;
{
char *porteOK = (char *)(liste->resistance_tech->DATA);
int cout = cellLoadSurface(porteOK);
int minDiff = cellLoadFanout(porteOK) - sCin;
ptype_list *p;

for(p = liste->resistance_tech->NEXT; p; p = p->NEXT)
      {
      int diff = cellLoadFanout((char *)p->DATA) - sCin;

      if (diff >= 0)
         {
                /* cellule resoud le probleme */
         if (minDiff < 0)
            {
            minDiff = diff;
            cout = cellLoadSurface((char *)p->DATA);
            porteOK = (char *)p->DATA;
            }
         else
           if (cellLoadSurface((char *)p->DATA) < cout)
              {
              minDiff = diff;
              cout = cellLoadSurface((char *)p->DATA);
              porteOK = (char *)p->DATA;
              }
            else
              if (cellLoadSurface((char *)p->DATA) == cout)
                 if (cellLoadFanout((char *)p->DATA) < cellLoadFanout(porteOK))
                    {
                        /* si surface egale alors plus petit fanout */
                    minDiff = diff;
                    cout = cellLoadSurface((char *)p->DATA);
                    porteOK = (char *)p->DATA;
                    }
         }
      else
         if ((minDiff < 0) && (diff > minDiff))
            {
            minDiff = diff;
            cout = cellLoadSurface((char *)p->DATA);
            porteOK = (char *)p->DATA;
            }
      }
return porteOK;
}

/*---------------------------------------------------------------------------
addPuissante    : remplace par une porte puissante
-----------------------------------------------------------------------------
retour          : void
---------------------------------------------------------------------------*/
void addPuissante(ptfig, inst, puissante)
lofig_list *ptfig;
loins_list *inst;
char *puissante;
{
PRINTI("    ==> POWER GATE %s [%s / area = %d / fanout = %d]\n",inst->INSNAME,
       puissante, cellLoadSurface(puissante), cellLoadFanout(puissante));
inst->FIGNAME = puissante;

ptfig->MODELCHAIN = addmodel(ptfig->MODELCHAIN, puissante);
}

/*---------------------------------------------------------------------------
addInversCond   : Ajout d'un buffer inverseur conditionnel
-----------------------------------------------------------------------------
retour          : pointeur sur l'instance de l'inverseur ajoute
---------------------------------------------------------------------------*/
loins_list *addInversCond(ptfig, inst, inv, conIn, nameBuff, listeCond)
lofig_list *ptfig;
loins_list *inst;       /* instance a bufferiser */
loins_list *inv;        /* invers. apres lequel on ajoute le buffer inverseur */
locon_list *conIn;      /* connecteur bufferiser */
char *nameBuff;         /* nom du modele du buffer a ajouter */
ptype_list *listeCond;	/* liste des instances a ne pas bufferiser */
{
lofig_list *figBuffer = getlofig(nameBuff,'P');
locon_list *conAux, *conBuffer, *conBufOut;
locon_list *conOutInv = (locon_list *)searchTH(loconOutTH, inv->INSNAME);
losig_list *sigOutInv = conOutInv->SIG;
losig_list *newSig, *oldSig;
loins_list *instBuff;
chain_list *signal = (chain_list *)NULL, *lstCon;
ptype_list *l, *liste;
char *nomSig;
char *nameIns = mbkalloc(12);
int cinBuff = 0;
long capaInst = 0, capaBuff = 0, capaInv;

        /* calcul du 1er index non utilise */
indexMaximum++;

        /* creation d'un nouveau signal */
nomSig = mbkalloc(12);
sprintf(nomSig, "netop%d", indexMaximum);
newSig = addlosig(ptfig, indexMaximum,
                         addchain((chain_list *)NULL, (void *)nomSig),
                         INTERNAL, 0.0);

/* DEBUG
printf("INST %s [%s]\n", inv->INSNAME, inv->FIGNAME);
for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
   printf("con : %s == %s (%d)\n",conBuffer->NAME,
          getsigname(conBuffer->SIG), conBuffer->SIG->INDEX);
*/

   /* creation de la liste des signaux attaches aux connecteurs du buffer */
	/* ATTENTION : !!!!! parcours des connecteurs du modele !!!!! */
for(conBuffer = figBuffer->LOCON; conBuffer; conBuffer = conBuffer->NEXT)
   {
   if (isvdd(conBuffer->NAME) || isvss(conBuffer->NAME))
      {
      conBuffer->DIRECTION = IN;

        /* recherche des signaux vss ou vdd dans inv */
      for(conAux = inv->LOCON; conAux; conAux = conAux->NEXT)
         if (((isvdd(getsigname(conAux->SIG))) && isvdd(conBuffer->NAME)) ||
             ((isvss(getsigname(conAux->SIG))) && isvss(conBuffer->NAME)))
            signal = addchain(signal, (void *)conAux->SIG);
      }
   else
      {
      chain_list *out = cellLoadOut(figBuffer->NAME);
      Alist      *in  =  cellLoadIn(figBuffer->NAME);
      locon_list *con;

      if (memberChain_list(conBuffer->NAME,out))
         {
                /* connecteur de sortie */
         conBuffer->DIRECTION = OUT;

         if (inst)
            {
                /* on ajoute un buffer apres l'inverseur */
            locon_list *conOutG = (locon_list *)searchTH(loconOutTH,
                                                         inst->INSNAME);
            oldSig = conOutG->SIG;
            signal = addchain(signal, (void *)oldSig);
            conOutG->SIG = newSig;
            addTH(genUpTH, getsigname(newSig), (int)inst);
            addTH(genDwTH, getsigname(newSig), (int)inst);
            }
         else
            {
                /* l'inverseur inv est drive par un connecteur */
            char *nameSig = getsigname(newSig);

            oldSig = conIn->SIG;
            freechain(newSig->NAMECHAIN);
            newSig->NAMECHAIN = addchain((chain_list *)NULL,
                                         getsigname(conIn->SIG));
            newSig->INDEX = conIn->SIG->INDEX;
            freechain(conIn->SIG->NAMECHAIN);
            conIn->SIG->NAMECHAIN = addchain((chain_list *)NULL,
                                             nameSig);
            conIn->SIG->INDEX = indexMaximum;
            conIn->SIG->TYPE = INTERNAL;
            signal = addchain(signal, (void *)conIn->SIG);
            conIn->SIG = newSig;
            addTH(genConTH, getsigname(newSig), (int)conIn);
            conIn->SIG->TYPE = EXTERNAL;
            }
         }
      else
         {
                /* connecteur d'entree */
         cinBuff = cellLoadCin(nameBuff, conBuffer->NAME);
         conBuffer->DIRECTION = IN;
         if (memberAlist(conBuffer->NAME,in))
            signal = addchain(signal, (void *)sigOutInv);
         }
      }
   }

        /* remise a jour de l'ordre des connecteurs */
signal = reverse(signal);

        /* verif !!!!
if (0)
   {
   chain_list *s;

   printf("BUFFER %s \n", figBuffer->NAME);
   for(conBuffer = figBuffer->LOCON,s = signal; conBuffer && s;
       conBuffer = conBuffer->NEXT, s = s->NEXT)
         printf("con : %s == %s (%d)\n",conBuffer->NAME,
                (char *)(getsigname((losig_list *)(s->DATA))),
                (char *)(((losig_list *)(s->DATA))->INDEX));
   }

if (inst)
   {
   printf("INST %s [%s]\n", inst->INSNAME, inst->FIGNAME);
   for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
      printf("con : %s == %s (%d)\n",conBuffer->NAME,
             conBuffer->SIG->NAMECHAIN->DATA, conBuffer->SIG->INDEX);
   }
*/

        /* ajout du buffer dans la lofig */
sprintf(nameIns, "netopi%d",indexMaximum);
instBuff = addloins(ptfig, nameIns, figBuffer, signal);

addTH(genUpTH, getsigname(oldSig), (int)instBuff);
addTH(genDwTH, getsigname(oldSig), (int)instBuff);
 
	/* recherche du connecteur de sortie du buffer */
for(conBuffer = instBuff->LOCON; conBuffer; conBuffer = conBuffer->NEXT)
   if (conBuffer->DIRECTION == OUT)
      {
      conBufOut = conBuffer;
      addTH(loconOutTH, instBuff->INSNAME, (int)conBuffer);
      }

	/* on remet les instances de listeCond a la sortie de inst ou conIn */
for(l = listeCond; l; l= l->NEXT)
   {
   loins_list *instDirect = ((loins_list *)l->DATA);

   printf("  remis en direct %s\n", instDirect->INSNAME);
   for(conAux = instDirect->LOCON; conAux; conAux = conAux->NEXT)
      if (conAux->DIRECTION == IN && conAux->SIG == oldSig)
         {
         conAux->SIG = newSig;
		/* calcul de la somme des capas drivees par inst */
         capaInst += cellLoadCin(instDirect->FIGNAME, conAux->NAME);
         }
   }
	/* memorisation de la somme des capas drivees par inst */
addTH(capaTH, getsigname(newSig), capaInst);
if (inst)
   majDelaiInst(inst, capaInst);
else
   majDelaiCon(conIn, capaInst);

	/* ajout de cinBuff a la somme des capas drivees par inv */
capaInv = (long)searchTH(capaTH, getsigname(sigOutInv)) + cinBuff;

	/* memorisation de la somme des capas drivees par inv */
addTH(capaTH, getsigname(sigOutInv), capaInv);
majDelaiInst(inv, capaInv);

lofigchain(ptfig);
liste = getptype(conBufOut->SIG->USER, (long)LOFIGCHAIN);

        /* parcours du fanout du buffer et calcul de la somme des capas */
for(lstCon = (chain_list *)liste->DATA; lstCon; lstCon = lstCon->NEXT)
   {
   locon_list *conAux = (locon_list *)lstCon->DATA;

   if (conAux->ROOT != (void *)instBuff)
      {
      if (conAux->TYPE == EXTERNAL)
         {
             /* connecteur externe */
         capaBuff += cellLoadCin((char *)NULL, conAux->NAME);
         addTH(pathTH, instBuff->INSNAME, 1);
         }
      else
             /* connecteur interne */
         if (conAux->DIRECTION == IN)
            {
            loins_list *cellOut = (loins_list *)conAux->ROOT;

            capaBuff += cellLoadCin(cellOut->FIGNAME, conAux->NAME);
            }
         else
            PRINTI("\nERROR add Buffer : Unknown connector '%s'\n",
                   conAux->NAME);
      }
   }

addTH(capaTH, getsigname(oldSig), capaBuff);
majDelaiInst(instBuff, capaBuff);

cptBuffer++;

printf("capa Inst %d, capa Buff %d\n", capaInst, capaBuff);
        /* Affichage */
if (inst)
   PRINTI("    ==> BUFFER added after %s : %s [%s / area = %d]\n",
          inst->INSNAME,
          instBuff->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));
else
   PRINTI("    ==> BUFFER added after input %s : %s [%s / area = %d]\n",
          conIn->NAME,
          instBuff->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));

        /* chainages signaux -> connecteurs */

freechain(signal);
dellofig(figBuffer->NAME);
return instBuff;
}

/*---------------------------------------------------------------------------
addBufferCond   : Ajout d'un buffer conditionnel
-----------------------------------------------------------------------------
retour          : instance representant le buffer ajoute
---------------------------------------------------------------------------*/
loins_list *addBufferCond(ptfig, inst, conIn, nameBuff, listeCond)
lofig_list *ptfig;
loins_list *inst;       /* instance apres laquelle on ajoute le buffer ou NULL*/
locon_list *conIn;      /* connecteur IN apres lequel on ajoute le buffer ou NULL */
char *nameBuff;         /* nom du modele du buffer a ajouter */
ptype_list *listeCond;	/* liste des instances a ne pas bufferiser */
{
lofig_list *figBuffer = getlofig(nameBuff,'P');
losig_list *newSig;
locon_list *conAux;
locon_list *conBuffer, *conBufOut;
chain_list *signal = (chain_list *)NULL, *lstCon;
ptype_list *l, *liste;
loins_list *instBuff;
int cinBuff = 0;
losig_list *oldSig;
char *nomSig;
long capaInst = 0, capaBuff = 0;
char *nameIns = mbkalloc(12);

        /* calcul du 1er index non utilise */
indexMaximum++;

        /* creation d'un nouveau signal */
nomSig = mbkalloc(12);
sprintf(nomSig, "netop%d", indexMaximum);
newSig = addlosig(ptfig, indexMaximum,
                        addchain((chain_list *)NULL,
                                 (void *)nomSig),
                        INTERNAL, 0.0);

/* DEBUG
if (inst)
   {
   printf("INST %s [%s]\n", inst->INSNAME, inst->FIGNAME);
   for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
      printf("con : %s == %s (%d)\n",conBuffer->NAME,
             conBuffer->SIG->NAMECHAIN->DATA, conBuffer->SIG->INDEX);
   }
*/

   /* creation de la liste des signaux attaches aux connecteurs du buffer */
	/* ATTENTION : !!!!! parcours des connecteurs du modele !!!!! */
for(conBuffer = figBuffer->LOCON; conBuffer; conBuffer = conBuffer->NEXT)
   {
   if (isvdd(conBuffer->NAME) || isvss(conBuffer->NAME))
      {
      conBuffer->DIRECTION = IN;

        /* recherche des signaux vss ou vdd dans inst */

      if (inst)
         {
         for(conAux = inst->LOCON; conAux; conAux = conAux->NEXT)
            if (((isvdd(getsigname(conAux->SIG))) && isvdd(conBuffer->NAME)) ||
                ((isvss(getsigname(conAux->SIG))) && isvss(conBuffer->NAME)))
               signal = addchain(signal, (void *)conAux->SIG);
         }
      else
         {
         conAux = ptfig->LOCON;
         while (conAux &&
                !((isvdd(getsigname(conAux->SIG)) && 
                  isvdd(conBuffer->NAME))  ||
                 (isvss(getsigname(conAux->SIG)) && 
                  isvss(conBuffer->NAME))))
           conAux = conAux->NEXT;
         if (conAux)
            signal = addchain(signal, (void *)conAux->SIG);
         else
            {
            printf("\nERROR : no VDD or VSS in %s\n", ptfig->NAME);
            exit(1);
            }
         }
      }
   else
      {
      chain_list *out = cellLoadOut(figBuffer->NAME);
      Alist      *in  =  cellLoadIn(figBuffer->NAME);
      locon_list *con;

      if (memberChain_list(conBuffer->NAME,out))
         {
                /* connecteur de sortie */
         conBuffer->DIRECTION = OUT;
         if (inst)
            {
                /* on ajoute un buffer apres une instance */
            for(con = inst->LOCON; con ; con = con->NEXT)
               if (con->DIRECTION == OUT)
                  {
                  oldSig = con->SIG;
                  signal = addchain(signal, (void *)oldSig);
                  con->SIG = newSig;
                  addTH(genUpTH, getsigname(newSig), (int)inst);
                  addTH(genDwTH, getsigname(newSig), (int)inst);
/*
                  newSig->USER = addptype((ptype_list *)NULL, (long)GENERUP, (void *)inst);
*/
                  }
            }
         else
            {
                /* on ajoute un buffer apres un connecteur */
            char *nameSig = mbkalloc(12);

            oldSig = conIn->SIG;
                  /* Inversion des noms et index de signaux (bug du driver) */
            freechain(newSig->NAMECHAIN);
            newSig->NAMECHAIN = addchain((chain_list *)NULL,
                                         conIn->SIG->NAMECHAIN->DATA);
            newSig->INDEX = conIn->SIG->INDEX;
            freechain(conIn->SIG->NAMECHAIN);
            conIn->SIG->INDEX = indexMaximum;
            sprintf(nameSig, "netops%d",indexMaximum);
            conIn->SIG->NAMECHAIN = addchain((chain_list *)NULL, (void *)nameSig);

            conIn->SIG->TYPE = INTERNAL;
            signal = addchain(signal, (void *)(conIn->SIG));
            conIn->SIG = newSig;
            addTH(genConTH, getsigname(newSig), (int)conIn);
            conIn->SIG->TYPE = EXTERNAL;
            }
         }
      else
         {
                /* connecteur d'entree */
         cinBuff = cellLoadCin(nameBuff, conBuffer->NAME);
         conBuffer->DIRECTION = IN;
         if (memberAlist(conBuffer->NAME,in))
            signal = addchain(signal, (void *)(newSig));
         }
      }
   }

        /* remise a jour de l'ordre des connecteurs */
signal = reverse(signal);

        /* verif !!!!
if (1)
   {
   chain_list *s;

   printf("BUFFER %s \n", figBuffer->NAME);
   for(conBuffer = figBuffer->LOCON,s = signal; conBuffer && s;
       conBuffer = conBuffer->NEXT, s = s->NEXT)
         printf("con : %s == %s (%d)\n",conBuffer->NAME,
                (char *)(((losig_list *)(s->DATA))->NAMECHAIN->DATA),
                (char *)(((losig_list *)(s->DATA))->INDEX));
   }

if (inst)
   {
   printf("INST %s [%s]\n", inst->INSNAME, inst->FIGNAME);
   for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
      printf("con : %s == %s (%d)\n",conBuffer->NAME,
             conBuffer->SIG->NAMECHAIN->DATA, conBuffer->SIG->INDEX);
   }
*/

        /* ajout du buffer dans la lofig */
sprintf(nameIns, "netopi%d",indexMaximum);
instBuff = addloins(ptfig, nameIns, figBuffer, signal);

addTH(genUpTH, getsigname(oldSig), (int)instBuff);
addTH(genDwTH , getsigname(oldSig), (int)instBuff);
 
	/* recherche du connecteur de sortie du buffer */
for(conBuffer = instBuff->LOCON; conBuffer; conBuffer = conBuffer->NEXT)
   if (conBuffer->DIRECTION == OUT)
      {
      conBufOut = conBuffer;
      addTH(loconOutTH, instBuff->INSNAME, (int)conBuffer);
      }
/*
      instBuff->USER = addptype((ptype_list *)NULL, (long)LOCONOUT,
                               (void *)conBuffer);
   oldSig->USER = addptype((ptype_list *)NULL, (long)GENERUP, (void *)instBuff);
*/

	/* on remet les instances de listeCond a la sortie de inst ou conIn */
for(l = listeCond; l; l= l->NEXT)
   {
/*
   printf("  %s\n", ((loins_list *)l->DATA)->INSNAME);
*/
   for(conAux = ((loins_list *)l->DATA)->LOCON; conAux; conAux = conAux->NEXT)
      if (conAux->DIRECTION == IN && conAux->SIG == oldSig)
         {
         conAux->SIG = newSig;
		/* calcul de la somme des capas drivees par inst + cinBuff */
         capaInst += cellLoadCin(((loins_list *)l->DATA)->FIGNAME, conAux->NAME);
         }
   }
	/* ajout de cinBuff a la somme des capas drivees par inst ou conIn */
capaInst += cinBuff;
	/* memorisation de la somme des capas drivees par inst */
addTH(capaTH, getsigname(newSig), capaInst);
if (inst)
   majDelaiInst(inst, capaInst);
else
   majDelaiCon(conIn, capaInst);

lofigchain(ptfig);
liste = getptype(conBufOut->SIG->USER, (long)LOFIGCHAIN);

                /* parcours du fanout du buffer et calcul de la somme des capas */
for(lstCon = (chain_list *)liste->DATA; lstCon; lstCon = lstCon->NEXT)
   {
   locon_list *conAux = (locon_list *)lstCon->DATA;

   if (conAux->ROOT != (void *)instBuff)
      {
      if (conAux->TYPE == EXTERNAL)
         {
             /* connecteur externe */
         capaBuff += cellLoadCin((char *)NULL, conAux->NAME);
         addTH(pathTH, instBuff->INSNAME, 1);
         }
      else
             /* connecteur interne */
         if (conAux->DIRECTION == IN)
            {
            loins_list *cellOut = (loins_list *)conAux->ROOT;

            capaBuff += cellLoadCin(cellOut->FIGNAME, conAux->NAME);
            }
         else
            PRINTI("\nERROR add Buffer : Unknown connector '%s'\n",
                   conAux->NAME);
      }
   }

addTH(capaTH, getsigname(oldSig), capaBuff);
majDelaiInst(instBuff, capaBuff);

cptBuffer++;

/*
printf("capa Inst %d, capa Buff %d\n", capaInst, capaBuff);
*/
        /* Affichage */
if (inst)
   PRINTI("    ==> (%d) BUFFER added after %s : %s [%s / area = %d]\n", cptBuffer,
          inst->INSNAME,
          instBuff->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));
else
   PRINTI("    ==> (%d) BUFFER added after input %s : %s [%s / area = %d]\n", cptBuffer,
          conIn->NAME,
          instBuff->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));

        /* chainages signaux -> connecteurs */

freechain(signal);
dellofig(figBuffer->NAME);
return instBuff;
}

/*---------------------------------------------------------------------------
addBuffer       : Ajout d'un buffer
-----------------------------------------------------------------------------
retour          : instance du buffer ajoute
---------------------------------------------------------------------------*/
loins_list *addBuffer(ptfig, inst, conIn, nameBuff)
lofig_list *ptfig;
loins_list *inst;       /* instance apres laquelle on ajoute le buffer ou NULL*/
locon_list *conIn;      /* connecteur IN apres lequel on ajoute le buffer */
char *nameBuff;         /* nom du modele du buffer a ajouter */
{
lofig_list *figBuffer = getlofig(nameBuff,'P');
losig_list *newSig;
locon_list *conAux;
locon_list *conBuffer;
chain_list *signal = (chain_list *)NULL;
loins_list *instBuf;
int cinBuff = 0;
char *nameSig = mbkalloc(12);

        /* calcul du 1er index non utilise */
indexMaximum++;
                        /* creation d'un nouveau signal */
sprintf(nameSig, "netops%d",indexMaximum);
newSig = addlosig(ptfig, indexMaximum,
                        addchain((chain_list *)NULL,
                           (void *)nameSig),
                        INTERNAL, 0.0);
/*
if (inst)
   {
   printf("INST %s [%s]\n", inst->INSNAME, inst->FIGNAME);
   for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
      printf("con : %s == %s (%d)\n",conBuffer->NAME,
             conBuffer->SIG->NAMECHAIN->DATA, conBuffer->SIG->INDEX);
   }

printf("indexMaximum = %d\n", indexMaximum);
*/

   /* creation de la liste des signaux attaches aux connecteurs du buffer */
for(conBuffer = figBuffer->LOCON; conBuffer; conBuffer = conBuffer->NEXT)
   if (isvdd(conBuffer->NAME) || isvss(conBuffer->NAME))
      {
      conBuffer->DIRECTION = IN;

        /* recherche des signaux vss ou vdd dans inst */

      if (inst)
         {
         for(conAux = inst->LOCON; conAux; conAux = conAux->NEXT)
            if (((isvdd(getsigname(conAux->SIG))) && isvdd(conBuffer->NAME)) ||
                ((isvss(getsigname(conAux->SIG))) && isvss(conBuffer->NAME)))
               signal = addchain(signal, (void *)conAux->SIG);
         }
      else
         {
         conAux = ptfig->LOCON;
         while (conAux &&
                !((isvdd((char *)(conAux->SIG->NAMECHAIN->DATA)) && 
                  isvdd(conBuffer->NAME))  ||
                 (isvss((char *)(conAux->SIG->NAMECHAIN->DATA)) && 
                  isvss(conBuffer->NAME))))
           conAux = conAux->NEXT;
         if (conAux)
            signal = addchain(signal, (void *)conAux->SIG);
         else
            {
            printf("vdd or vss not found\n");
            exit(1);
            }
         }
      }
   else
      {
      chain_list *out = cellLoadOut(figBuffer->NAME);
      Alist      *in  =  cellLoadIn(figBuffer->NAME);
      locon_list *con;

      if (memberChain_list(conBuffer->NAME,out))
         {
                /* connecteur de sortie */
         conBuffer->DIRECTION = OUT;
         if (inst)
            {
                /* on ajoute un buffer apres une instance */
            for(con = inst->LOCON; con ; con = con->NEXT)
               if (con->DIRECTION == OUT)
                  {
                  signal = addchain(signal, (void *)(con->SIG));
                  con->SIG = newSig;
                  }
            }
         else
            {
            char *nameSig = mbkalloc(12);

                    /* on ajoute un buffer apres un connecteur */
        /* Inversion des noms et des index de signaux (bug du driver ...) */
            freechain(newSig->NAMECHAIN);
            newSig->NAMECHAIN = addchain((chain_list *)NULL, conIn->SIG->NAMECHAIN->DATA);
            newSig->INDEX = conIn->SIG->INDEX;
            freechain(conIn->SIG->NAMECHAIN);
            conIn->SIG->INDEX = indexMaximum;
            sprintf(nameSig, "netops%d", indexMaximum);
            conIn->SIG->NAMECHAIN = addchain((chain_list *)NULL,
                                       (void *)nameSig);
            conIn->SIG->TYPE = INTERNAL;
            signal = addchain(signal, (void *)(conIn->SIG));
            conIn->SIG = newSig;
            conIn->SIG->TYPE = EXTERNAL;
            }
         }
      else
         {
                /* connecteur d'entree */
         cinBuff = cellLoadCin(nameBuff, conBuffer->NAME);
         conBuffer->DIRECTION = IN;
         if (memberAlist(conBuffer->NAME,in))
            signal = addchain(signal, (void *)(newSig));
         }
      }

        /* remise a jour de l'ordre des connecteurs */
signal = reverse(signal);

        /* verif !!!!
if (1)
   {
   chain_list *s;

   printf("BUFFER %s \n", figBuffer->NAME);
   for(conBuffer = figBuffer->LOCON,s = signal; conBuffer && s;
       conBuffer = conBuffer->NEXT, s = s->NEXT)
         printf("con : %s == %s (%d)\n",conBuffer->NAME,
                (char *)(((losig_list *)(s->DATA))->NAMECHAIN->DATA),
                (char *)(((losig_list *)(s->DATA))->INDEX));
   }

if (inst)
   {
   printf("INST %s [%s]\n", inst->INSNAME, inst->FIGNAME);
   for(conBuffer = inst->LOCON; conBuffer ; conBuffer = conBuffer->NEXT)
      printf("con : %s == %s (%d)\n",conBuffer->NAME,
             conBuffer->SIG->NAMECHAIN->DATA, conBuffer->SIG->INDEX);
   }
*/

        /* ajout du buffer dans la lofig */
{
char *nameIns = mbkalloc(12);

sprintf(nameIns, "netopi%d",indexMaximum);
instBuf = addloins(ptfig, nameIns, figBuffer, signal);
/*
*/
if (NO_OPTION == 'f')
   addTH(dejaTraite, (char *)namealloc(nameIns), 1);
}

        /* Affichage */
if (inst)
   PRINTI("    ==> BUFFER added after %s : %s [%s / area = %d]\n",
          inst->INSNAME,
          instBuf->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));
else
   PRINTI("    ==> BUFFER added after %s : %s [%s / area = %d]\n",
          conIn->NAME,
          instBuf->INSNAME,
          nameBuff,
          cellLoadSurface(nameBuff));

        /* chainages signaux -> connecteurs */

lofigchain(ptfig);
dellofig(figBuffer->NAME);

/*
NO_LOG->NAME = NO_FILEOUT;
printf("Saving %s/%s...\n", WORK_LIB, NO_LOG->NAME);
savelofig(NO_LOG);
*/

return instBuf;
}

/*---------------------------------------------------------------------------
dupliqueInst    : duplique une instance de porte (nb fois)
-----------------------------------------------------------------------------
retour          : void (insertion d'instances dans la lofig)
---------------------------------------------------------------------------*/
void dupliqueInst(ptfig, inst, solution)
lofig_list *ptfig;
loins_list *inst;       /* instance a dupliquer */
sol_list *solution;
{
locon_list *conFig, *conI, *conF;
int nbExt = 0;
losig_list *sigOut = NULL;
ptype_list *fan, *instDupl, *p;

indexMaximum++;

if (inst->FIGNAME != (char *)solution->NAME->DATA)
   addPuissante(ptfig, inst, (char *)solution->NAME->DATA);

PRINTI("    ==> %d DUPLICATION%s of %s ", solution->NOMBRE-1,
       (solution->NOMBRE-1>1)?"S":"", inst->INSNAME);
fflush(stdout);

        /* duplication (nb -1 fois) */
for(fan = solution->ARBRE->NEXT, instDupl = solution->NAME->NEXT;
    fan;
    fan = fan->NEXT, instDupl = instDupl->NEXT)
   {
   chain_list *signal = NULL, *s;
   lofig_list *fig;
   losig_list *newSig;
   char *nameSig = mbkalloc(12);

                /* creation d'un nouveau signal */
   sprintf(nameSig, "netops%d",indexMaximum);
   newSig = addlosig(ptfig, indexMaximum,
                                 addchain((chain_list *)NULL,
                                          (void *)nameSig),
                                 INTERNAL, 0.0);

   fig = getlofig((char *)instDupl->DATA,'P');

/*
   printf(".");
   fflush(stdout);
*/

  /* creation de la liste des signaux attaches aux connecteurs de l'instance */

   for(conFig = inst->LOCON, conF = fig->LOCON; conFig ;
       conFig = conFig->NEXT, conF = conF->NEXT)
      {
      if (conFig->NAME != conF->NAME)
         {
         printf("erreur dans les connecteurs de la lofig %s\n", inst->INSNAME);
         exit(-1);
         }
                /* Ajout de la direction des connecteurs */
      conF->DIRECTION = conFig->DIRECTION;
      if (conFig->DIRECTION == OUT)
         {
                /* connecteur de sortie */

         signal = addchain(signal, (void *)newSig);
         sigOut = conFig->SIG;
         }
      else
        /* connecteur vss, vdd ou entree */
         signal = addchain(signal, (void *)(conFig->SIG));
      }

        /* remise a jour de l'ordre des connecteurs de l'instance creee */
   signal = reverse(signal);

  /* mise a jour des signaux sur les portes attaquees par le nveau signal */
   for(p = (ptype_list *)(fan->DATA); p; p = p->NEXT)
      if (p->DATA)
         {
                /* on attaque une instance */
         for(conI = ((loins_list *)(p->DATA))->LOCON; conI; conI = conI->NEXT)
            if (conI->NAME == (char *)(p->TYPE))
               conI->SIG = newSig;
         }
      else
         {
         char *nameSig = mbkalloc(12);

                /* on attaque un connecteur EXTERNE */
         nbExt ++;
         if (nbExt > 1)
            printf("WARNING : la porte %s attaque au moins deux connecteurs EXT\n",inst->INSNAME);

         printf(" connector EXTERNAL %s",(char *)(p->TYPE));

         newSig->TYPE = EXTERNAL;
         newSig->NAMECHAIN = addchain((chain_list *)NULL,
                                      (void *)(sigOut->NAMECHAIN->DATA));
         newSig->INDEX = sigOut->INDEX;
         sprintf(nameSig, "netops%d",indexMaximum);
         sigOut->NAMECHAIN = addchain((chain_list *)NULL,
                                      (void *)nameSig);
         sigOut->TYPE = INTERNAL;
         sigOut->INDEX = indexMaximum;

                /* recherche du connecteur EXT dans la lofig */
         for(conI = ptfig->LOCON; conI; conI = conI->NEXT)
            if (conI->NAME == (char *)(p->TYPE))
               conI->SIG = newSig;
         }

        /* ajout de l'instance dans la lofig */
   {
   char *nameIns = mbkalloc(12);

   sprintf(nameIns, "netopi%d",indexMaximum);
   addloins(ptfig,nameIns, fig, signal);
   if (NO_OPTION == 'f')
      addTH(dejaTraite, (char *)namealloc(nameIns), 1);
   }

   indexMaximum++;
   dellofig(fig->NAME);

        /* verif !!!!
for(conFig = inst->LOCON,s = signal; conFig && s;
    conFig = conFig->NEXT, s = s->NEXT)
   printf("dupl con : %s == %s (%d)\n",conFig->NAME,
          (char *)(((losig_list *)(s->DATA))->NAMECHAIN->DATA),
          (char *)(((losig_list *)(s->DATA))->INDEX));
         */
   }
PRINTI("\n");

        /* chainages signaux -> connecteurs */
lofigchain(ptfig);
}

/*---------------------------------------------------------------------------
void freeSol    : liberation memoire des solutions
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void freeSol(sol)
sol_list *sol;
{
sol_list *auxSol;

while (sol)
   {
   auxSol = sol->NEXT;
   freeptype(sol->NAME);
   freeptype(sol->ARBRE);
   mbkfree((void *)sol);
   sol = auxSol;
   }
}

/*---------------------------------------------------------------------------
triInstance     : repartit les instances du fanout en sous-listes
-----------------------------------------------------------------------------
retour          : liste de solutions, le champ ARBRE a la forme suivante :
                 ptype_list *
                  TYPE = Somme Cin
                  DATA = ptype_list *
                          TYPE = char * : nom du connecteur attaque
                          DATA = loins_list *
---------------------------------------------------------------------------*/
sol_list *triInstance(fanout, nameCell, type, sol)
ptype_list *fanout;
char *nameCell;
short type;
sol_list *sol;
{
long valMax = cellLoadFanout(nameCell);
ptype_list *tri = addptype((ptype_list *)NULL, (long)0, (void *)NULL);
ptype_list *p;
sol_list *newSol = (sol_list*)mbkalloc(sizeof(sol_list));
cellList *lstCells = (cellList *)NULL;

while (fanout)
      {
      long cin = 0;

                /* connecteur INTERNE : recherche du Cin de la porte */
      if (fanout->DATA)
         cin = cellLoadCin(((loins_list *)fanout->DATA)->FIGNAME,
                           (char *)fanout->TYPE);
      else
         cin = cellLoadCin((char *)NULL, (char *)fanout->TYPE);

      if ((tri->DATA == NULL) || ((cin + tri->TYPE) <= valMax))
         {
         tri->DATA = (void *)addptype((ptype_list *)tri->DATA,
                                      fanout->TYPE, fanout->DATA);
         tri->TYPE += cin;
         fanout = fanout->NEXT;
         }
      else
         {
         short nonInsere = 1;
         ptype_list *triParcours;

         triParcours = tri->NEXT;
         while (nonInsere && triParcours)
               if ((triParcours->DATA == NULL) ||
                   ((cin + triParcours->TYPE) <= valMax))
                  {
                  triParcours->DATA = (void *)addptype((ptype_list *)triParcours->DATA,
                                               fanout->TYPE, fanout->DATA);
                  if (fanout->DATA)
                     cin = cellLoadCin(((loins_list *)fanout->DATA)->FIGNAME,
                                       (char *)fanout->TYPE);
                  else
                     cin = cellLoadCin((char *)NULL, (char *)fanout->TYPE);
                  triParcours->TYPE += cin;
                  fanout = fanout->NEXT;
                  nonInsere = 0;
                  }
               else
                  triParcours = triParcours->NEXT;
         if (nonInsere == 1)
            tri = addptype(tri, (long)0, (void *)NULL);
         }
      }

newSol->NAME = (ptype_list *)NULL;
newSol->SURFACE = 0;
newSol->SUMCIN  = 0;
newSol->NOMBRE  = 0;
lstCells = (cellList *)searchTH(NO_CELLTH, nameCell);
for(p = tri; p ; p = p->NEXT)
   {
   newSol->NOMBRE++;
   newSol->NAME = addptype(newSol->NAME, p->TYPE,
                           (void *)adaptePuissance(lstCells, p->TYPE));
   newSol->SURFACE += cellLoadSurface((char *)newSol->NAME->DATA);
   if (cellLoadIn((char *)newSol->NAME->DATA))
      newSol->SUMCIN  += cellLoadCin((char *)newSol->NAME->DATA,
                                     cellLoadIn((char *)newSol->NAME->DATA)->name);
   }
newSol->TYPE = type;
newSol->NAME = (ptype_list *)reverse((chain_list *)newSol->NAME);
newSol->ARBRE = tri;
newSol->NEXT = sol;
return newSol;
}

/*---------------------------------------------------------------------------
addConDir       : ajoute la direction des connecteurs d'une instance
-----------------------------------------------------------------------------
retour          : void
void addConDir(inst)
loins_list *inst;
{
locon_list *con;
Alist * in =  cellLoadIn(inst->FIGNAME);
chain_list * out = cellLoadOut(inst->FIGNAME);

for(con = inst->LOCON; con; con = con->NEXT)
   {
   if (memberChain_list(con->NAME,out))
      con->DIRECTION = OUT;
   else
      if (memberAlist(con->NAME,in))
         con->DIRECTION = IN;
      else
         if (!isvdd(con->NAME) && !isvss(con->NAME))
            {
            printf("addConDir : Unknown direction of %s in %s\n",
                   con->NAME, inst->FIGNAME);
            exit(1);
            }
         else 
            con->DIRECTION = IN;
   }
}
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
calculProf : calcul de la profondeur pour les instances d'une LOFIG
-----------------------------------------------------------------------------
retour       : la profondeur est memorisee dans le champ USER des signaux
---------------------------------------------------------------------------*/
void calculProf(lofig)
lofig_list *lofig;
{
loins_list *inst;
locon_list *con;
 
prepareLofig(lofig);
profondeurCircuit = 0;
instProf = NULL;
 
        /* Parcours des connecteurs externes d'entree (profondeur = 0) */
for(con = lofig->LOCON; con; con = con->NEXT)
   if (con->DIRECTION == IN && !isvdd(con->NAME) && !isvss(con->NAME))
      {
      con->SIG->USER = addptype(con->SIG->USER, (long)PROF, (void *)0);
      addTH(genConTH, getsigname(con->SIG), (int)con);
/*
      con->SIG->USER = addptype(con->SIG->USER, (long)GENCON, (void *)con);
*/
      }
 
        /* parcours des instances de la figure */
for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   {
   locon_list *conOut = (locon_list *)searchTH(loconOutTH, inst->INSNAME);
 
   if (!getptype(conOut->SIG->USER,(long)PROF))
      profondeur(inst);
   }
 
printf("PROFONDEUR MAX = %d sur %s[%s]\n\n", profondeurCircuit, instProf->INSNAME,
                                       instProf->FIGNAME);
}
 
/*---------------------------------------------------------------------------
profondeur      :  
-----------------------------------------------------------------------------
retour          : 
---------------------------------------------------------------------------*/
void profondeur(inst)
loins_list *inst;
{
if (cellLoadType(inst->FIGNAME) == REG)
   {
   locon_list *conOut = (locon_list *)searchTH(loconOutTH, inst->INSNAME);

        /* memorisation du delai sur le signal de sortie de inst */
   conOut->SIG->USER = addptype(conOut->SIG->USER, (long)PROF, (void *)0);
   return;
   }
if (cellLoadType(inst->FIGNAME) == BUS)
   {
   PRINTF("\nERROR : The gate %s [%s] is not treated actually\n",
          inst->INSNAME, inst->FIGNAME);
   exit(1);
   }
else
   {
   int userOut = searchTH(loconOutTH, inst->INSNAME);
   locon_list *conOut;
   losig_list *sigOut;
   ptype_list *liste;

   long maxProfond = 0;
   locon_list *con;

   if (userOut != EMPTYTH)
      {
      conOut = (locon_list *)userOut;
      sigOut = conOut->SIG;
      liste = getptype(sigOut->USER, (long)LOFIGCHAIN);
      }
   else
      {
      PRINTF("TIMING ANALYSIS : No type LOCONOUT for %s\n", inst->INSNAME);
      exit(1);
      }
 
   if (!liste)
      {
      PRINTF("TIMING ANALYSIS : No type LOFIGCHAIN for %s\n", inst->INSNAME);      exit(1);
      }
 
   if (NO_TRACE >= 2)
      printf(" Computing profondeur on %s [%s]\n", inst->INSNAME, inst->FIGNAME);
 
        /* Parcours des connecteurs de l'instance a traiter pour
        trouver les connecteurs d'entree et recuperer les profondeurs
        des signaux sur ces connecteurs */
 
   for(con = inst->LOCON; con; con = con->NEXT)
      {
      if (con->DIRECTION == IN &&
          !isvdd(con->NAME) && !isvss(con->NAME))
         {
         loins_list *cellGenere = (loins_list *)NULL;
         locon_list *conGenere = (locon_list *)NULL;
         long profond = 0;
         ptype_list *user;
         int gen;
 
         if (user = getptype(con->SIG->USER, (long)PROF))
            {
                   /* profondeur deja calculee ... */
            profond = (long)user->DATA;
            if ((gen = searchTH(genUpTH, getsigname(con->SIG))) != EMPTYTH)
                        /* C'est une porte qui genere le signal */
               cellGenere = (loins_list *)gen;
            else
                        /* C'est un connecteur qui genere le signal */
               conGenere = (locon_list *)searchTH(genConTH, getsigname(con->SIG));
            }
         else
            {
                /* On recherche la porte qui genere le signal */
            if ((gen = searchTH(genUpTH, getsigname(con->SIG))) != EMPTYTH)
               {
                        /* C'est une porte qui genere le signal */
               cellGenere = (loins_list *)gen;

                   /* appel recursif pour la porte qui genere le signal */
               profondeur(cellGenere);

               if (user = getptype(con->SIG->USER, (long)PROF))
                  {
                        /* on recupere la profondeur tout juste calculee */
                  profond = (long)user->DATA;
                  }
               else
                  {
                  PRINTF("\nERROR profondeur : no profondeur on inputs of %s\n", inst->INSNAME);
                  exit(1);
                  }
               }
            else
               {
               PRINTF("\nERROR TIMING ANALYSIS : no delay on %s[%s]\n", inst->INSNAME, con->NAME);
               }   
            }   /* else : profondeur pas encore calculee */

        if (profond > maxProfond)
            maxProfond = profond;
         }      /* if connecteur d'entree */
      else
         if (con->DIRECTION != OUT && !isvdd(con->NAME) && !isvss(con->NAME))
            PRINTF("TIMING ANALYSIS : Unknown direction for connector %s of %s\n", con->NAME, inst->INSNAME);
 
      }         /* parcours des connecteurs de l'instance (for) */
 
   maxProfond++;

   if (NO_TRACE >= 2)
      printf("   profondeur of %s [%s] = %d \n",
             inst->INSNAME, inst->FIGNAME, maxProfond);
 
        /* memorisation des retards critiques sur le signal de sortie de inst */
   sigOut->USER = addptype(sigOut->USER, (long)PROF, (void *)maxProfond);
 
   if (maxProfond > profondeurCircuit)
      {
      profondeurCircuit = maxProfond;
      instProf = inst;
      }
   }
}

/*---------------------------------------------------------------------------
statistics      :  informations sur la LOFIG : surface, modeles ...
-----------------------------------------------------------------------------
retour          : int = la surface en pitchs
---------------------------------------------------------------------------*/
int statistics(lofig, surfInit)
lofig_list *lofig;
int surfInit;
{
loins_list *inst;
chain_list *mod;
int countMod = 0;
int totalSurf = 0;
int totalInst = 0;
float plus = 0;
char *modTie = namealloc("tie_y");

if (NO_TRACE >= 2)
   PRINTF("\nFIGNAME : %s\n",lofig->NAME);

        /* parcours des modeles et calcul du nombre d'instances par modele */

for(mod = lofig->MODELCHAIN; mod; mod = mod->NEXT)
   {
   if ((char *)mod->DATA != modTie)
      {
      int countInst = 0;

      for(inst = lofig->LOINS; inst; inst = inst->NEXT)
         if (inst->FIGNAME == (char *)mod->DATA)
            countInst++;
#ifdef LAX
      PRINTI("  Model = %s (area=%d) ==> %d cell%s\n",(char *)(mod->DATA),
             cellLoadSurface((char *)mod->DATA),
             countInst, (countInst > 1) ? "s" : "");
#else
      if (NO_TRACE >= 1)
         PRINTI("  Model = %s (area=%d) ==> %d cell%s\n",(char *)(mod->DATA),
                cellLoadSurface((char *)mod->DATA),
                countInst, (countInst > 1) ? "s" : "");
#endif
      countMod++;
      totalInst += countInst;
      totalSurf += countInst * cellLoadSurface((char *)mod->DATA);
      }
   }

#ifdef LAX
PRINTI("   %d models - %d cells - %d pitchs\n",
       countMod, totalInst, totalSurf);
#endif

PRINTF("   %d models - %d cells - %d pitchs",
       countMod, totalInst, totalSurf);

if (surfInit)
   plus = (float)(100 * (totalSurf - surfInit)) / surfInit;

if (plus)
   PRINTF(" (+%5.2f %%)\n\n", plus);
else
   PRINTF("\n\n");

nbGates = totalInst;
return totalSurf;
}

/*---------------------------------------------------------------------------
capaNot         :  renvoie la capa d'entree du NOT (ou 0 si pas de NOT)
-----------------------------------------------------------------------------
retour          : int
---------------------------------------------------------------------------*/
long capaNot()
{
long cinNot = 0;
cellList *cells = NO_CELLS;

        /* recherche de la cellule logique NOT */
while (cells && cells->nomlog != NOT1)
      cells = cells->NEXT;

        /* si elle existe, calcul de la plus petite capa parmi tous les not */
if (cells)
   {
   ptype_list *c;

   for(c = cells->surface_tech ;c; c = c->NEXT)
      {
      char *CIN = (char *)mbkalloc(5 + strlen(cells->entrees->name));
      long cin;

      sprintf(CIN, "cin_%s",cells->entrees->name);
      cin = (long)searchGeneric(cells, c->DATA, CIN);
      if (cinNot == 0 || cinNot > cin)
         cinNot = cin;
#if MACHINE != pc
      mbkfree((void *)CIN);
#endif
      }
   }
return cinNot;
}

/*---------------------------------------------------------------------------
triConptype     : ajoute un elt d'une ptype_list a sa place
-----------------------------------------------------------------------------
retour          : ptype_list triee en ordre decroissant.
---------------------------------------------------------------------------*/
ptype_list *triConptype(liste, name, data)
ptype_list *liste;
char *name;
void *data;
{
int type = 0;
int tete = 0;

if (data)
   type = cellLoadCin(((loins_list *)data)->FIGNAME, name) ;
else
   type = cellLoadCin((char *)NULL, name);

if (!liste)
   return addptype(liste,(long)name,data);

if (liste->DATA)
   tete = cellLoadCin(((loins_list *)liste->DATA)->FIGNAME,(char *)liste->TYPE);
else
   tete = cellLoadCin((char *)NULL, (char *)liste->TYPE);
if (type >= tete)
   return addptype(liste,(long)name,data);
else
   {
   ptype_list *p1 = liste;
   ptype_list *p2 = liste->NEXT;
   ptype_list *new = (ptype_list *)mbkalloc(sizeof(ptype_list));

   while (p2 && cellLoadCin(((loins_list *)p2->DATA)->FIGNAME,
                             (char *)p2->TYPE) > type)
         {
         p2 = p2->NEXT;
         p1 = p1->NEXT;
         }
   new->TYPE = (long)name;
   new->DATA = data;
   new->NEXT = p2;
   p1->NEXT = new;
   return liste;
   }
}

/*---------------------------------------------------------------------------
capaEntrees     : ajoute la valeur des capa sur les connecteurs de LOFIG
-----------------------------------------------------------------------------
retour          : void
---------------------------------------------------------------------------*/
void capaEntrees()
{
locon_list *conFig;
int cinNot = capaNot();

for(conFig = NO_LOG->LOCON; conFig; conFig = conFig->NEXT)
   {
   if (!isvdd(conFig->NAME) &&
       !isvss(conFig->NAME) &&
       conFig->DIRECTION == IN)
      {
      int sCin = 0;
      chain_list *l;
      ptype_list *liste = getptype(conFig->SIG->USER,(long)LOFIGCHAIN);

      for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
                /* recherche portes attaquees (et pas connecteurs EXT) */
         if (((locon_list *)(l->DATA))->TYPE == INTERNAL &&
             ((locon_list *)(l->DATA))->DIRECTION == IN)
            {
            loins_list *cellOut =
                     (loins_list *)((locon_list *)(l->DATA))->ROOT;

            sCin += cellLoadCin(cellOut->FIGNAME,
                                ((locon_list *)l->DATA)->NAME);
            }
      }
   }
}

/*---------------------------------------------------------------------------
optimEntrees    :
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void optimEntrees(conIn, fanoutMax)
char *conIn;
long fanoutMax;
{
locon_list *conFig = NO_LOG->LOCON;
ptype_list *liste, *fanout = NULL;
chain_list *l;
int sCin = 0;
int nbPortes = 0;

while(conFig && conFig->NAME != conIn)
     conFig = conFig->NEXT;
if (!conFig)
   {
   printf("Connector '%s' does not exist as primary input\n", conIn);
   printf("(bad parameter file)\n");
   exit(1);
   }

liste = getptype(conFig->SIG->USER,(long)LOFIGCHAIN);

for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
   if (((locon_list *)(l->DATA))->ROOT != (void *)NO_LOG)
      {
                /* recherche portes attaquees (et pas connecteurs EXT) */
      if (((locon_list *)(l->DATA))->TYPE == INTERNAL &&
          ((locon_list *)(l->DATA))->DIRECTION == IN)
         {
         loins_list *cellOut =
                  (loins_list *)((locon_list *)(l->DATA))->ROOT;

         sCin += cellLoadCin(cellOut->FIGNAME,
                             ((locon_list *)l->DATA)->NAME);
         nbPortes++;
                /* portes attaquees par le connecteur a traiter */
         fanout = triConptype(fanout,
                              ((locon_list *)(l->DATA))->NAME,
                              (void *)cellOut);
         }
      }
if (sCin > fanoutMax)
   {
   sol_list *solutions = (sol_list *)NULL, *lstSol = (sol_list *)NULL;
   sol_list *minSol = (sol_list *)NULL;
   ptype_list *cellsPhy = (ptype_list *)NULL;
   cellList *cells = NO_CELLS;

   if (NO_TRACE >= 1)
      printf("  # Solving %s [Input Connector]\n     %d gates driven - sum Cin = %d - fanout max = %d\n",
             conIn, nbPortes, sCin, fanoutMax);

                /* SOLUTIONS avec BUFFERS */
   while (cells && cells->type != 't')
      cells = cells->NEXT;
   if (cells)
      for(cellsPhy = cells->resistance_tech; cellsPhy;
          cellsPhy=cellsPhy->NEXT)
         {
         char *nameCellsPhy = (char *)cellsPhy->DATA;

         solutions = triInstance(fanout, nameCellsPhy, BUF, solutions);
         if (nbPortes == 1)
            solutions->DIFFCIN = cellLoadFanout(nameCellsPhy)-sCin;
         else
            solutions->DIFFCIN = 0;
         }

                /* SOLUTIONS avec BUFFERS de NOT */
      cells = NO_CELLS;
      while (cells && cells->nomlog != NOT1)
         cells = cells->NEXT;
      if (cells)
         {
         for(cellsPhy = cells->resistance_tech; cellsPhy;
             cellsPhy=cellsPhy->NEXT)
            {
            char *nameCellsPhy = (char *)cellsPhy->DATA;
            char *insertNot = (char *)NULL;

            solutions = triInstance(fanout, nameCellsPhy, BUN, solutions);
            insertNot = adaptePuissance(cells,
                         solutions->NOMBRE*cellLoadCin(nameCellsPhy,
                                      cellLoadIn(nameCellsPhy)->name));
            solutions->SURFACE += cellLoadSurface(insertNot);
            solutions->ADAPT = insertNot;
            if (nbPortes == 1)
               solutions->DIFFCIN = cellLoadFanout(nameCellsPhy)-sCin;
            else
               solutions->DIFFCIN = 0;
            }
         }

	/* recherche de la meilleure solution */
      if (solutions)
         {
         minSol = solutions;
         for(lstSol = solutions->NEXT; lstSol; lstSol = lstSol->NEXT)
             if ((lstSol->SURFACE <= minSol->SURFACE) &&
                 (lstSol->DIFFCIN >= 0))
                minSol = lstSol;
         }

                /* mise en oeuvre de la solution */
      if (minSol)
         {
         if (minSol->TYPE == BUF)
            {
            loins_list *instBuff = addBuffer(NO_LOG, (loins_list *)NULL,
                                       conFig, (char *)minSol->NAME->DATA);
            if (minSol->NOMBRE > 1)
               dupliqueInst(NO_LOG, instBuff,minSol);
            }
         if (minSol->TYPE == BUN)
            {
            loins_list *instNot1 = addBuffer(NO_LOG, (loins_list *)NULL,
                                       conFig, minSol->ADAPT);
            loins_list *instNot2 = addBuffer(NO_LOG, instNot1,
                                       (locon_list *)NULL, (char *)minSol->NAME->DATA);
            if (minSol->NOMBRE > 1)
               dupliqueInst(NO_LOG, instNot2, minSol);
            }
         }
      else
         {
         printf("    ==> NO SOLUTION for %s [input connector]\n", conFig->NAME);
         printf("     !!! Merci de me faire une fiche probleme. NATHALIE !!!\n");
         }
      freeSol(solutions);
   }
   freeptype(fanout);
}

/*---------------------------------------------------------------------------
optimiseInst    : optimisation selon le fanout maximum pour une cellule
-----------------------------------------------------------------------------
retour          : void
---------------------------------------------------------------------------*/
void optimiseInst(instTraite)
loins_list *instTraite;
{
if ((NO_OPTION == 'g') || (searchTH(dejaTraite, instTraite->INSNAME) == EMPTYTH))
   {
   ptype_list *fanout = NULL, *fan;
   int nbPortes = 0;
   locon_list *con;
   int sCin = 0;
   int savesCin = 0;

        /* mise dans la table de hachage des instances deja traitees */
   if (NO_OPTION == 'f')
      addTH(dejaTraite, instTraite->INSNAME, 1);

        /* parcours des connecteurs de l'instance : calcul du fanout */
   if (NO_OPTION != 'g')
      for(con = instTraite->LOCON; con; con = con->NEXT)
         {
        /* recherche du connecteur de sortie */
         if (con->DIRECTION == OUT)
            {
            chain_list *l;

                /* liste des liaisons signaux --> connecteurs */
            ptype_list *liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

            for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
               if (((locon_list *)(l->DATA))->ROOT != (void *)instTraite)
                  {
                        /* recherche portes attaquees (pas con EXTERNE) */
                                /* Pour appel RECURSIF !!! */
                  if (((locon_list *)(l->DATA))->TYPE == INTERNAL &&
                      ((locon_list *)(l->DATA))->DIRECTION == IN)
                     {
                     loins_list *cellOut =
                              (loins_list *)((locon_list *)(l->DATA))->ROOT;

                                /* portes attaquees par l'instance a traiter */
                     fanout = triConptype(fanout,
                                          ((locon_list *)(l->DATA))->NAME,
                                          (void *)cellOut);
                     }
                  }
             }
         }         /* fin parcours connecteurs */

        /* appel recursif sur les instances du fanout */
   if (NO_OPTION != 'g')
      {
      for(fan = fanout; fan; fan = fan->NEXT)
         if (searchTH(dejaTraite, ((loins_list *)fan->DATA)->INSNAME) == EMPTYTH)
            optimiseInst((loins_list *)fan->DATA);

      freeptype(fanout);
      fanout = NULL;
      }

        /* calcul du fanout de l'instance a traiter :
                portes attaquees + connecteurs externes, somme des Cin */

      for(con = instTraite->LOCON; con; con = con->NEXT)
         {
                /* recherche des connecteurs de sortie */
         if (con->DIRECTION == OUT)
            {
                /* liste des liaisons signaux --> connecteurs */
            ptype_list *liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);
            chain_list *l;

            for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
               if (((locon_list *)(l->DATA))->ROOT != (void *)instTraite)
                  {
                        /* connecteur externe */
                  if (((locon_list *)(l->DATA))->TYPE == EXTERNAL)
                     {
                     fanout = triConptype(fanout,
                                          ((locon_list *)(l->DATA))->NAME,
                                          (void *)NULL);
                     sCin += cellLoadCin((char *)NULL, ((locon_list *)l->DATA)->NAME);
                     nbPortes++;
                     }
                  else
                     if (((locon_list *)(l->DATA))->DIRECTION == IN)
                        {
                                /* connecteur interne et entree de porte */
                        loins_list *cellOut =
                                 (loins_list *)((locon_list *)(l->DATA))->ROOT;

                        fanout = triConptype(fanout,
                                             ((locon_list *)(l->DATA))->NAME,
                                             (void *)cellOut);
                        sCin += cellLoadCin(cellOut->FIGNAME,
                                            ((locon_list *)l->DATA)->NAME);
                        nbPortes++;
                        }
                  }
            }
         }

   if (sCin > cellLoadFanout(instTraite->FIGNAME))
      {
        /* la porte est trop chargee */

      char *portePuissanteOK = NULL;

      if (NO_TRACE >= 1)
         printf("  # Solving gate %s [%s / Area = %d]\n     %d gate%s driven - Sum Cin = %d - fanout max = %d\n",
             instTraite->INSNAME, instTraite->FIGNAME, cellLoadSurface(instTraite->FIGNAME),
             nbPortes, nbPortes>1?"s":"",
             sCin, cellLoadFanout(instTraite->FIGNAME));

        /* Recherche d'une SOLUTION */

      savesCin = sCin;
                /* SOLUTION en portes PUISSANTES */
      portePuissanteOK = adaptePuissance((cellList *)searchTH(NO_CELLTH, instTraite->FIGNAME), (long)sCin);
      if (cellLoadFanout(portePuissanteOK) - sCin >= 0)
         {
                /* test porte puissante = OK */
         addPuissante(NO_LOG, instTraite, portePuissanteOK);
         }
      else
         {
         sol_list *solutions = (sol_list *)NULL, *lstSol = (sol_list *)NULL;
         sol_list *minSol = (sol_list *)NULL;
         ptype_list *cellsPhy = (ptype_list *)NULL;
         cellList *cells = NO_CELLS;

         sCin = savesCin;

                /* SOLUTIONS avec BUFFERS */
         while (cells && cells->type != 't')
            cells = cells->NEXT;
         if (cells)
            for(cellsPhy = cells->resistance_tech; cellsPhy;
                cellsPhy=cellsPhy->NEXT)
               {
               char *nameCellsPhy = (char *)cellsPhy->DATA;
               cellList *cellsInst;
               char *bonnePuissance = (char *)NULL;

               solutions = triInstance(fanout, nameCellsPhy, BUF, solutions);
               cellsInst = (cellList *)searchTH(NO_CELLTH, instTraite->FIGNAME);
               bonnePuissance = adaptePuissance(cellsInst, solutions->SUMCIN);
               solutions->SURFACE += cellLoadSurface(bonnePuissance);
               solutions->ADAPT = bonnePuissance;
               if (nbPortes == 1)
                  solutions->DIFFCIN = cellLoadFanout(nameCellsPhy)-sCin;
               else
                  solutions->DIFFCIN = 0;
               }

                /* SOLUTIONS de BUFFERS avec des NOT */
         cells = (cellList *)searchTH(NO_CELLTH, instTraite->FIGNAME);
         if (cells->nomlog != NOT1)
            {
            cells = NO_CELLS;
            while (cells && cells->nomlog != NOT1)
               cells = cells->NEXT;
            if (cells)
               {

               for(cellsPhy = cells->resistance_tech; cellsPhy;
                   cellsPhy=cellsPhy->NEXT)
                  {
                  char *nameCellsPhy = (char *)cellsPhy->DATA;
                  char *insertNot = (char *)NULL;

                  solutions = triInstance(fanout, nameCellsPhy, BUN, solutions);
                  insertNot = adaptePuissance(cells, solutions->SUMCIN);
                  solutions->SURFACE += cellLoadSurface(insertNot);
                  solutions->SURFACE += cellLoadSurface(instTraite->FIGNAME);
                  solutions->ADAPT = insertNot;
                  if (nbPortes == 1)
                     solutions->DIFFCIN = cellLoadFanout(nameCellsPhy)-sCin;
                  else
                     solutions->DIFFCIN = 0;
                  }
               }
            }

                /* SOLUTIONS de DUPLICATIONS */
         if ((cellLoadType(instTraite->FIGNAME) != REG) && (nbPortes > 1))
            {
            cells = (cellList *)searchTH(NO_CELLTH, instTraite->FIGNAME);
            for(cellsPhy = cells->resistance_tech; cellsPhy;
                cellsPhy=cellsPhy->NEXT)
               {
               char *nameCellsPhy = (char *)cellsPhy->DATA;

               solutions = triInstance(fanout, nameCellsPhy, DUP, solutions);
               solutions->ADAPT = (char *)NULL;
               solutions->DIFFCIN = 0;
               }
            }

                /* Choix d'une solution */
         if (solutions)
            {
            minSol = solutions;
            for(lstSol = solutions->NEXT; lstSol; lstSol = lstSol->NEXT)
               {
               if (minSol->DIFFCIN < 0)
                  {
                  if (lstSol->DIFFCIN > minSol->DIFFCIN)
                      minSol = lstSol;
                  }
               else
                  if (lstSol->DIFFCIN >= 0)
                     if (lstSol->SURFACE < minSol->SURFACE)
                        minSol = lstSol;
                     else
                        if (lstSol->SURFACE == minSol->SURFACE)
                           if (lstSol->NOMBRE < minSol->NOMBRE)
                              minSol = lstSol;
               }
            }
                /* mise en oeuvre de la solution */
         if (minSol)
            {
            if (minSol->TYPE == BUF)
               {
               loins_list *instBuff = addBuffer(NO_LOG, instTraite,
                                          (locon_list *)NULL, (char *)minSol->NAME->DATA);
               if (minSol->NOMBRE > 1)
                  dupliqueInst(NO_LOG, instBuff,minSol);
               if (minSol->ADAPT != instTraite->FIGNAME)
                  addPuissante(NO_LOG, instTraite, minSol->ADAPT);
               }
            if (minSol->TYPE == DUP)
               {
/*
               if (instTraite->FIGNAME != minSol->NAME)
                  addPuissante(NO_LOG, instTraite, minSol->NAME);
               if (minSol->NOMBRE > 1)
*/
                  dupliqueInst(NO_LOG, instTraite, minSol);
               }
            if (minSol->TYPE == BUN)
               {
               loins_list *instNot1 = addBuffer(NO_LOG, instTraite,
                                          (locon_list *)NULL, minSol->ADAPT);
               loins_list *instNot2 = addBuffer(NO_LOG, instNot1,
                                          (locon_list *)NULL, (char *)minSol->NAME->DATA);
               if (minSol->NOMBRE > 1)
                  dupliqueInst(NO_LOG, instNot2, minSol);
               }
            }
         else
            {
            printf("    ==> NO SOLUTION for %s\n", instTraite->INSNAME);
            printf("     !!! Merci de me faire une fiche probleme. NATHALIE !!!\n");
            }
                /* LIBERERATION DES SOLUTIONS */
         freeSol(solutions);
         }
      }
   freeptype(fanout);
   }
}

/*---------------------------------------------------------------------------
optimFanMax     : optimisation selon le fanout maximum
-----------------------------------------------------------------------------
retour          : void
---------------------------------------------------------------------------*/
void optimFanMax()
{
loins_list *inst;
long diffMax = 0;
loins_list *instMax = NULL;
int nbPbs = 0;

        /* parcours des instances de la figure */
for(inst = NO_LOG->LOINS; inst; inst = inst->NEXT)
   {
   if (cellLoadType(inst->FIGNAME) == BUS)
     {
        /* remettre a jour dir con BUS */
     }
   else
     if (searchTH(dejaTraite,inst->INSNAME) == EMPTYTH)
      {
      long somCin = 0;
      long diffCapa = 0;
      locon_list *con;

        /* parcours des connecteurs de l'instance : calcul du fanout */
      for(con = inst->LOCON; con; con = con->NEXT)
         {
                /* recherche du connecteur de sortie */
         if (con->DIRECTION == OUT)
            {
            chain_list *l;
                /* liste des liaisons signaux --> connecteurs */
            ptype_list *liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

            for(l = (chain_list *)(liste->DATA); l; l = l->NEXT)
               if (((locon_list *)(l->DATA))->ROOT != (void *)inst)
                  {
                  if (((locon_list *)(l->DATA))->TYPE == EXTERNAL)
                        /* connecteur externe */
                     somCin += (long)cellLoadCin((char *)NULL,
                                                 ((locon_list *)l->DATA)->NAME);
                  else
                     if (((locon_list *)(l->DATA))->DIRECTION == IN)
                        {
                                /* connecteur interne et entree de porte */
                        loins_list *cellOut =
                         (loins_list *)((locon_list *)(l->DATA))->ROOT;

                                /* calcul de la somme des Cin */
                        somCin += (long)cellLoadCin(cellOut->FIGNAME,
                                                    ((locon_list *)l->DATA)->NAME);
                        }
                  }
            }
         }

      if ((somCin > 0) && (cellLoadFanout(inst->FIGNAME) != 0) &&
          ((diffCapa = somCin - (long)cellLoadFanout(inst->FIGNAME)) > 0))
         {
                /* l'instance  ne respecte pas le fanout max */
         nbPbs++;

                /* recherche du + fort pb non encore traite */
         if (diffCapa > diffMax)
            {
            diffMax = diffCapa;
            instMax = inst;
            }
         }
      }
   }    /* fin parcours inst */

if (instMax)
   {
   if (NO_TRACE >= 1)
      PRINTF("\nNumber of problems : %d : cell %s [%s] - Sum Cin =  %d\n",
             nbPbs, instMax->INSNAME, instMax->FIGNAME,
             diffMax + cellLoadFanout(instMax->FIGNAME));

   optimiseInst(instMax);
   optimFanMax();
   }
}

/*---------------------------------------------------------------------------
mainInfo :
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void mainInfo(type)
char type;
{
if (getenv("NO_CAPA") == NULL || atoi(getenv("NO_CAPA")) == 0)
   NO_CAPA = 0;
else
   NO_CAPA = 1;

if (type == 'f')
   {
   displayLofig(NO_LOG, 3);

	/* affiche le fanout de chaque porte */
/*
   displayFanout(NO_LOG, 0);
*/
   }
else
   {
   int oldTrace = NO_TRACE;

   NO_TRACE = 1;
   statistics(NO_LOG, 0);
   NO_TRACE = oldTrace;

   prepareLofig(NO_LOG);

   if (NO_TRACE >= 1)
      {
      calcul_retard(NO_LOG, 0);
      calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 1);
      }
   else
      calcul_retard(NO_LOG, 2);
   cheminCritique(porteCriticMinUp, NUP, delaiCriticMinUp, 1);
   cheminCritique(porteCriticMinDw, NDOWN, delaiCriticMinDw, 1);
   printf("\n Maximal depth = %d gates on %s\n\n",
          profondeurCircuit, instProf->INSNAME);

   if (NO_TRACE >= 1)
      afficheChemins(NO_LOG);

   if (NO_TRACE >= 2)
      {
      displayTiming(NO_LOG, 0);
      }
   }
}

/*---------------------------------------------------------------------------
mainOptimFanMax :
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void mainOptimFanMax()
{
ptype_list *p;
locon_list *con;
FILE *fichStat;
char *fileStat;

int surfaceInit = statistics(NO_LOG, 0);

NO_CAPA = 0;
dejaTraite = createTH(500);
optimFanMax();
printf("\n");

	/* optimisation des entrees primaires */
if (NO_CAPAPI)
   {
   for(p = NO_CAPAPI; p ; p = p->NEXT)
      {
      optimEntrees((char *)p->DATA, p->TYPE);
      addTH(dejaTraite, (char *)p->DATA, 1);
      }
   }
for(con = NO_LOG->LOCON; con; con = con->NEXT)
   if ((con->DIRECTION == IN) && !isvdd(con->NAME) && !isvss(con->NAME))
      if (searchTH(dejaTraite, con->NAME) == EMPTYTH)
         optimEntrees(con->NAME, (long)(4*capaNot()));
   
destroyTH(dejaTraite);
capaEntrees();
clearmodel(NO_LOG);

statistics(NO_LOG, surfaceInit);

NO_LOG->NAME = NO_FILEOUT;
printf("Saving %s/%s...\n", WORK_LIB, NO_LOG->NAME);
savelofig(NO_LOG);

        /* Generation du fichier de statistiques */
fileStat = (char *)mbkalloc(strlen( WORK_LIB)+strlen(NO_LOG->NAME)+8);
sprintf(fileStat, "%s/%s.stat",  WORK_LIB, NO_LOG->NAME);
fichStat = fopen(fileStat, "w");
if (fichStat)
   {
   ptype_list *liste;
   chain_list *l;

   printf("\nGeneration of statistics file : %s\n", fileStat);

   for(con = NO_LOG->LOCON; con; con = con->NEXT)
      if ((con->DIRECTION == IN) && !isvdd(con->NAME) && !isvss(con->NAME))
         {
         fprintf(fichStat, "Connector name : %s (IN) - Capa = %d\n",
                 con->NAME, (int)((con->SIG->CAPA)*1000));
         }
      else
         if (con->DIRECTION == OUT)
            {
            liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

            for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
               {
               loins_list *inst;

               if (((locon_list *)(l->DATA))->TYPE != EXTERNAL)
                  {
                  inst = (loins_list *)((locon_list *)(l->DATA))->ROOT;

                  fprintf(fichStat, "Connector name : %s (OUT) [%s] - max fanout = %d\n",
                          con->NAME, inst->FIGNAME, cellLoadFanout(inst->FIGNAME));
                  }
               }
            }
   fclose(fichStat);
#if MACHINE != pc
   mbkfree((void *)fileStat);
#endif
   }
}

/*---------------------------------------------------------------------------
mainOptimGlobal   :
-----------------------------------------------------------------------------
retour          :
---------------------------------------------------------------------------*/
void mainOptimGlobal()
{
int NO_FANIN, NO_BUFF, NO_POWER;
int surfaceInit = statistics(NO_LOG, 0);

NO_CAPA = 0;

	/* par defaut pas d'optimisation en fanin */
if (getenv("NO_FANIN") == NULL || atoi(getenv("NO_FANIN")) != 0)
   NO_FANIN = 1;
else
   NO_FANIN = 0;

if (getenv("NO_BUFF") == NULL || atoi(getenv("NO_BUFF")) != 0)
   NO_BUFF = 1;
else
   NO_BUFF = 0;

if (getenv("NO_POWER") == NULL || atoi(getenv("NO_POWER")) != 0)
   NO_POWER = 1;
else
   NO_POWER = 0;

listeCritic = (ptype_list *)NULL;

prepareLofig(NO_LOG);

/*
if (NO_FANIN)
   {
   calcul_retard(NO_LOG, 2);
   assignFaninLofig(NO_LOG);
   }
*/

if (NO_POWER)
   {
   calcul_retard(NO_LOG, 1);
   calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 2);

   puissanceLofig(10);
   }

if (NO_BUFF)
   {
   calcul_retard(NO_LOG, 1);
   calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 1);

/*
   inverseurLofig(2, 2, 20, 10);
*/
   bufferLofig(2, 2, 10, 50);

   calcul_retard(NO_LOG, 0);
	/* portes dont aspect  critique < 10 sont optimisees */
   calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 3);

   bufferLofig(2, 2, 100, 5);

   {
   ptype_list *pinst;
   int cptCell = 0;
   short OK = 1, encore = 0;

   calcul_retard(NO_LOG, 1);
   calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 2);

   for(pinst = listeCritic; pinst; pinst = pinst->NEXT)
      {
      loins_list *inst = (loins_list *)pinst->DATA;
      locon_list *conOut = (locon_list *)searchTH(loconOutTH, inst->INSNAME);
      ptype_list *critic  = getptype(conOut->SIG->USER, (long)CRITIC);
      ptype_list *slackUp = getptype(conOut->SIG->USER, (long)SLACKUP);
      ptype_list *slackDw = getptype(conOut->SIG->USER, (long)SLACKDW);
      long capa = searchTH(capaTH, getsigname(conOut->SIG));

      cptCell++;

      if (((int)critic->DATA > 0) && ((int)critic->DATA < 50))
         {
/*
         printf("  %d) Inst %s[%s] capa=%ld  critic=%ld  Slack Up %ld  Dw %ld\n",
                cptCell, inst->INSNAME, inst->FIGNAME, capa, (long)critic->DATA,
                (long)slackUp->DATA, (long)slackDw->DATA);
*/
         optimiseInst(inst);
         }
      }

   }

   }

/*
if (NO_POWER)
   {
   calcul_retard(NO_LOG, 1);
   calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 2);

   puissanceLofig(800);
   }
*/

/*
assignFaninLofig(NO_LOG);
*/

/*
calcul_retard(NO_LOG, 2);
calcul_requi(NO_LOG, delaiCriticUp, delaiCriticDw, 1);
*/

/*
cheminCritique(porteCriticUp, NUP, delaiCriticUp, 0);
cheminCritique(porteCriticDw, NDOWN, delaiCriticDw, 0);
*/

clearmodel(NO_LOG);
statistics(NO_LOG, surfaceInit);

/*
NO_LOG->NAME = NO_FILEOUT;
printf("Saving %s/%s...\n", WORK_LIB, NO_LOG->NAME);
savelofig(NO_LOG);
*/
}
