/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  optimiseur de netlist                                      */
/*    Fichier :  no_systemp.c                                               */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  N. DICTUS                            le : 04/10/1993     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdlib.h>
#include MUT_H
#include MLO_H
#include MPH_H
#include LOG_H
#include "ctype.h"
#include "../load/types_map.h"
#include "no_type.h"
#include "no_system.h"
#include "enveloppe.h"
#include "../load/no_optim.h"
#include "../load/incdelai.h"

pTH dejaTraite;

/*----------------------------------------------------------------------------
initializeNOParam : initialise les parametres de Netoptim.
------------------------------------------------------------------------------
retour		: un pointeur sur une structure param.
------------------------------------------------------------------------------*/
param *initializeNOParam(mode,level,intermediate,delayPI,earlyPO,numN,numP,maxSlope,impedancePI,capaPO,capaPI,aux)
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
param *par = (param *) malloc (sizeof(param));

par->mode = mode;
par->level = level;
par->numTransN = numN;
par->numTransP = numP;
par->aux = aux;
par->maxSlopeTime = maxSlope;
par->intermediate = intermediate;
par->delayPI = delayPI;
par->earlyPO = earlyPO;
par->impedancePI = impedancePI;
par->capaPO = capaPO;
par->capaPI = capaPI;
return(par);
}

/*----------------------------------------------------------------------------
initializeNOSystem	: alloue no_sys *
------------------------------------------------------------------------------
retour			: un pointeur sur une structure systeme. 
------------------------------------------------------------------------------*/
no_system *initializeNOSystem()
{
no_system *no;

no = (no_system *) malloc (sizeof(no_system));

no->name	= (char *) malloc (100);
no->file	= (char *) malloc (100);
no->fileOut	= (char *) malloc (100);
no->filePar	= (char *) malloc (100);
no->fileLog	= (char *) malloc (100);
no->fileParam	= (char *) malloc (100);
no->par_lib	= (char *) malloc (10);
no->version	= (char *) malloc(100);
no->cata_lib	= (ptype_list *)NULL;

return( no );
}

/*----------------------------------------------------------------------------
majSysCellNeto :
------------------------------------------------------------------------------
retour		: 
------------------------------------------------------------------------------*/
short majSysCellNeto(lofig, trace, option, cataLib, c4Lib, xilLib,
                     actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                     maxSlope,impedancePI,capaPO,capaPI,aux)
lofig_list *lofig;
int trace;
char option;
char *cataLib, *c4Lib, *xilLib, *actLib;
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
losig_list *sigAux;
int i;

NO_LOG   = lofig;
NO_TRACE = trace;
NO_OPTION = option;
	/* !!! NO_CATA_LIB est une chain_list de path !... */
NO_CATA_LIB = addptype((ptype_list *)NULL, 0, (void *)cataLib);
NO_CATA_LIB = addptype(NO_CATA_LIB, 0, (void *)c4Lib);
NO_CATA_LIB = addptype(NO_CATA_LIB, 0, (void *)xilLib);
NO_CATA_LIB = addptype(NO_CATA_LIB, 0, (void *)actLib);

NO_PAR = initializeNOParam(mode,level,intermediate,delayPI,earlyPO,numN,numP,
                           maxSlope,impedancePI,capaPO,capaPI,aux);

	/* init systeme de chargement cellules */
cellInit();

	/* preparation de la lofig */
PRINTF("Flattening %s...\n", lofig->NAME);
rflattenlofig(lofig, YES, YES);
if ((!lofig->LOINS) || (lofig->LOTRS))
   {
   printf("ERROR : Transistor level description : verify your CATAL file !\n",
          lofig->NAME);
#ifdef LAX
   LXU_Error("Transistor level description : verify your CATAL file !");
#endif
   return 0;
   }

lofigchain(lofig);

indexMaximum = returnIndexMax(lofig);

if (cellLoad(lofig->MODELCHAIN) != 0)
   {
   calculEnveloppe(NO_CELLS);
   if (!restorealldir(lofig))
      {
      PRINTF("Adding connector direction failed on %s\n\n", lofig->NAME);
      exit(1);
      }

        /* mise a jour de la direction des connecteurs EXTERNES */
   addConDirLofig(lofig);

   return 1;
   }

PRINTF("\nERROR : cells library not found in %s\n\n", getenv("MBK_CATA_LIB"));
return 0;
}

/*----------------------------------------------------------------------------
laxOptimFanout :
------------------------------------------------------------------------------
retour		: 
------------------------------------------------------------------------------*/
lofig_list *laxOptimFanout(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux)
lofig_list *lofig;
int trace;
char option;
char *cataLib, *c4Lib, *xilLib, *actLib;
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
locon_list *con;
int surfaceInit ;

if (majSysCellNeto(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux))
   {
   surfaceInit = statistics(NO_LOG, 0);

   dejaTraite = createTH(500);

	/* optimisation de la lofig */
   PRINTF("Fanout optimization on %s\n", NO_LOG->NAME);
   optimFanMax();

	/* optimisation des entrees primaires */
   PRINTF("Inputs optimization on %s\n", NO_LOG->NAME);
   if (NO_CAPAPI)
      {
      ptype_list *p;

      for(p = NO_CAPAPI; p ; p = p->NEXT)
         {
         optimEntrees((char *)p->DATA, p->TYPE);
         addTH(dejaTraite, (char *)p->DATA, 1);
         }
      }

   for(con = NO_LOG->LOCON; con; con = con->NEXT)
      if ((con->DIRECTION == IN) && !isvdd(con->NAME) && !isvss(con->NAME))
         if (searchTH(dejaTraite, con->NAME) == EMPTYTH)
            optimEntrees(con->NAME, 4*capaNot());
   
   destroyTH(dejaTraite);
   capaEntrees();
   clearmodel(lofig);

   statistics(lofig, surfaceInit);
   return lofig;
   }
return (lofig_list*)NULL;
}

/*----------------------------------------------------------------------------
laxOptimGlobal :
------------------------------------------------------------------------------
retour		: 
------------------------------------------------------------------------------*/
lofig_list *laxOptimGlobal(lofig, trace, option, cataLib, c4Lib, xilLib,
                           actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                           maxSlope,impedancePI,capaPO,capaPI,aux)
lofig_list *lofig;
int trace;
char option;
char *cataLib, *c4Lib, *xilLib, *actLib;
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
loins_list *inst;

if (majSysCellNeto(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux))
   {
   mainOptimGlobal();
   return lofig;
   }

return (lofig_list*)NULL;
}

/*----------------------------------------------------------------------------
laxCalcul_delai :
------------------------------------------------------------------------------
retour		: 
----------------------------------------------------------------------------*/
lofig_list *laxCalcul_delai(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux)
lofig_list *lofig;
int trace;
char option;
char *cataLib, *c4Lib, *xilLib, *actLib;
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
ptype_list *p;

if (majSysCellNeto(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux))
   {
   statistics(lofig, 0);
   prepareLofig(lofig);
   calcul_retard(lofig, 2);
   return lofig;
   }

return (lofig_list*)NULL;
}

/*----------------------------------------------------------------------------
laxDisplayFanout :
------------------------------------------------------------------------------
retour		: 
------------------------------------------------------------------------------*/
lofig_list *laxDisplayFanout(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux)
lofig_list *lofig;
int trace;
char option;
char *cataLib, *c4Lib, *xilLib, *actLib;
int mode, level, numN, numP, aux;
chain_list *intermediate, *earlyPO;
ptype_list *delayPI, *impedancePI, *capaPO, *capaPI;
long maxSlope;
{
if (majSysCellNeto(lofig, trace, option, cataLib, c4Lib, xilLib,
                   actLib, mode,level,intermediate,delayPI,earlyPO,numN,numP,
                   maxSlope,impedancePI,capaPO,capaPI,aux))
   {
   displayLofig(lofig, 3);
   return lofig;
   }
return (lofig_list*)NULL;
}
