/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  Synthetiseur Logique                                       */
/*    Fichier :  sl_DC.c                                                    */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  L. Burgun                            le : 03/09/1991     */
/*                                                                          */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H

	/* fonction de simplification par DC */
      
char *getenv();
int costX3000();
/*------------------------------------------------------------------------------
depthNodeBdd : calcul la profondeur d'un BDD
------------------------------------------------------
parametres 	 : un pNode
-------------------------------------------------------
return 		 : int
------------------------------------------------------------------------------*/
int depthNodeBdd(pt)
pNode pt;
{
int depthlow,depthhigh;

if (pt->index < 2) return 0;
depthlow = depthNodeBdd(pt->low);
depthhigh = depthNodeBdd(pt->high);
if (depthlow > depthhigh) 
   return 1 + depthlow;
else
   return 1 + depthhigh;
}

/*------------------------------------------------------------------------------
costDepthTab : calcul le cout en nombre de litteraux 
-------------------------------------------------------
parametres 	 : une table de pNode
-------------------------------------------------------
return 		 : int
------------------------------------------------------------------------------*/
int costDepthTab(tab,len)
pNode *tab;
int len;
{
int i;
int count = 0;
int depth;

markAllBdd(0);

for (i=0;i<len;i++)
   {
   depth = depthNodeBdd(tab[i]); 
   if (count < depth)
      count = depth;
   }
markAllBdd(0);
return(count);
}


/*------------------------------------------------------------------------------
costTab : calcul le cout en nombre de litteraux 
-------------------------------------------------------
parametres 	 : une table de pNode
-------------------------------------------------------
return 		 : int
------------------------------------------------------------------------------*/
int costTab(tab,len,func)
pNode *tab;
int len;
int (*func)();
{
int i;
int count = 0;

markAllBdd(0);

if ((* func) == costX3000)
   {
   chain_list *tete, *lstCLB4 = NULL;

   for (i=0;i<len;i++)
      {
      count = count + costX3000(tab[i],&lstCLB4); 
      }

   tete = lstCLB4;
   while (lstCLB4)
      {
      if (lstCLB4->DATA)
         freechain(lstCLB4->DATA);
      lstCLB4 = lstCLB4->NEXT;
      }
   freechain(tete);
   }
else
   {
   for (i=0;i<len;i++)
      {
      count = count + (* func)(tab[i]); 
      }
   }
markAllBdd(0);
return(count);
}

/*------------------------------------------------------------------------------
sigmaDFBeh     : calcul des DC provenant des entrees 
-------------------------------------------------------
parametres 	 : une befig
-------------------------------------------------------
return 		 : void
------------------------------------------------------------------------------*/
pNode sigmaDFBeh(support,Fon,beh,dejaTraite)
chain_list *support;
pNode Fon;
befig_list *beh;
pTH dejaTraite;
{
pNode ret = zero;
chain_list *supp = support;
beaux_list *aux;
pNode obs;
pNode inter;

	/* parcours du support */

while (supp)
   {
   pNode deja = (pNode) searchTH(dejaTraite,(char *) supp->DATA);


   if (deja != (pNode) EMPTYTH)      /* noeud deja traite */
      {
      aux = beh->BEAUX;
      while (aux && strcmp(aux->NAME,supp->DATA))
         {
         aux = aux->NEXT;
         }

		/* calcul de la fonction d'observabilite */

      obs = applyBinBdd(XOR,
                        constraintBdd(Fon,(pNode) aux->NODE), 
                        constraintBdd(Fon,notBdd((pNode) aux->NODE)));

		/* ajout a la fonction de retour */

      ret = applyBinBdd(OR,ret,applyBinBdd(AND,deja,obs));
      }
   else
      {
      pNode reportDC;

      aux = beh->BEAUX;
      while (aux && strcmp(aux->NAME,supp->DATA))
         {
         aux = aux->NEXT;
         }

      if (aux)
         {

         if (searchExpr(aux->ABL,"'d'"))
            {
            chain_list *expr;

            if (!aux->NODE)
               aux->NODE = ablToBddCct(beh->CIRCUI,aux->ABL); 

            expr = substExpr(aux->ABL,"'d'",createAtom("'1'"));

            deja = applyBinBdd(AND,notBdd(aux->NODE),
                                     ablToBddCct(beh->CIRCUI,expr)); 
            freeExpr(expr);
            }
         else
            deja = zero; 

/*
        printf("DC local : ");
        displayExpr(bddToAblCct(beh->CIRCUI,deja));
*/

		/* report des DC provenant du support */

         reportDC = sigmaDFBeh(supportChain_listExpr(aux->ABL),deja,
                                  beh,dejaTraite);
         if (reportDC != zero)
            deja = applyBinBdd(OR,deja,reportDC);

/*
        printf("DC en fn des entrees : ");
        displayExpr(bddToAblCct(beh->CIRCUI,deja));
*/

		/* calcul de la fonction d'observabilite */

         obs = applyBinBdd(XOR,
                           constraintBdd(Fon,(pNode) aux->NODE), 
                           constraintBdd(Fon,notBdd((pNode) aux->NODE)));

		/* ajout a la fonction de retour */

         ret = applyBinBdd(OR,ret,
                  applyBinBdd(AND,deja,obs));

/*
        printf("Observabilite : ");
        displayExpr(bddToAblCct(beh->CIRCUI,ret));
*/
         addTH(dejaTraite,supp->DATA,deja);
         }
      }

/*
printf("NAME = %s\n",supp->DATA);
displayExpr(bddToAblCct(beh->CIRCUI,ret));
*/

   supp = supp->NEXT;
   }


freechain(support);
return ret;
}


/*------------------------------------------------------------------------------
CalculDCBeh      : calcule les DC d'une befig 
-------------------------------------------------------
parametres 	 : une befig, un tableau de on, de dc et un compteur
-------------------------------------------------------
return 		 : void
------------------------------------------------------------------------------*/
void calculDCBeh(beh,pON,pDC)
befig_list *beh;
pNode *pON, *pDC;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
biabl_list *biabl;
binode_list *binode;
int i=0;

		/* ajout dans la couche circuit des BDD des auxiliaires */

aux = beh->BEAUX;
while (aux)
   {
   if (aux->NODE)
      addOutputCct(beh->CIRCUI,aux->NAME,aux->NODE);
   aux = aux->NEXT;
   }

		/* traitement des sorties primaires */

out = beh->BEOUT;
while (out) 
   {
   if (out->ABL)
      {
      pON[i] = out->NODE;

		/* recherche des DC locaux */

      if (searchExpr(out->ABL,"'d'"))
         {
         chain_list *expr;

         expr = substExpr(out->ABL,"'d'",createAtom("'1'"));

         pDC[i] = applyBinBdd(AND,notBdd(pON[i]),
                                  ablToBddCct(beh->CIRCUI,expr)); 
         freeExpr(expr);
         }
      else
         pDC[i] = zero; 

		/* report des DC provenant du support */
/*
      reportDC = sigmaDFBeh(supportChain_listExpr(out->ABL),out->NODE,
                               beh,dejaTraite);
      if (reportDC != zero)
         pDC[i] = applyBinBdd(OR,pDC[i],reportDC);
*/

      i++;
      }
   out = out->NEXT;
   }

reg = beh->BEREG;
while (reg) 
   {
   biabl = reg->BIABL;
   binode = reg->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         pON[i] = binode->VALNODE;

		/* recherche des DC locaux */

         if (searchExpr(biabl->VALABL,"'d'"))
            {
            chain_list *expr;

            expr = substExpr(biabl->VALABL,"'d'",createAtom("'1'"));

            pDC[i] = applyBinBdd(AND,notBdd(pON[i]),
                                     ablToBddCct(beh->CIRCUI,expr)); 
            freeExpr(expr);
            }
         else
            pDC[i] = zero; 

		/* report des DC provenant du support */
/*
         reportDC = sigmaDFBeh(supportChain_listExpr(biabl->VALABL),
                               binode->VALNODE,beh,dejaTraite);

         if (reportDC != zero)
            pDC[i] = applyBinBdd(OR,pDC[i],reportDC);
*/

         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   reg = reg->NEXT;
   }

bus = beh->BEBUS;
while (bus) 
   {
   biabl = bus->BIABL;
   binode = bus->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         pON[i] = binode->VALNODE;

		/* recherche des DC locaux */

         if (searchExpr(biabl->VALABL,"'d'"))
            {
            chain_list *expr;

            expr = substExpr(biabl->VALABL,"'d'",createAtom("'1'"));

            pDC[i] = applyBinBdd(AND,notBdd(pON[i]),
                                     ablToBddCct(beh->CIRCUI,expr)); 
            freeExpr(expr);
            }
         else
            pDC[i] = zero; 

		/* report des DC provenant du support */

/*
         reportDC = sigmaDFBeh(supportChain_listExpr(biabl->VALABL),
                               binode->VALNODE,beh,dejaTraite);

         if (reportDC != zero)
            pDC[i] = applyBinBdd(OR,pDC[i],reportDC);
*/

         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   bus = bus->NEXT;
   }

bux = beh->BEBUX;
while (bux) 
   {
   biabl = bux->BIABL;
   binode = bux->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         pON[i] = binode->VALNODE;

		/* recherche des DC locaux */

         if (searchExpr(biabl->VALABL,"'d'"))
            {
            chain_list *expr;

            expr = substExpr(biabl->VALABL,"'d'",createAtom("'1'"));

            pDC[i] = applyBinBdd(AND,notBdd(pON[i]),
                                     ablToBddCct(beh->CIRCUI,expr)); 
            freeExpr(expr);
            }
         else
            pDC[i] = zero; 

		/* report des DC provenant du support */

/*
         reportDC = sigmaDFBeh(supportChain_listExpr(biabl->VALABL),
                               binode->VALNODE,beh,dejaTraite);

         if (reportDC != zero)
            pDC[i] = applyBinBdd(OR,pDC[i],reportDC);
*/

         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   bux = bux->NEXT;
   }
}


/*------------------------------------------------------------------------------
numberBddBeh     : calcule le nombre de BDD (out + reg + bux + bus)
-------------------------------------------------------
parametres 	 : une befig
-------------------------------------------------------
return 		 : int
------------------------------------------------------------------------------*/
int numberBddBeh(beh)
befig_list *beh;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
biabl_list *biabl;
binode_list *binode;
int numberGraph = 0;

out = beh->BEOUT;
while (out) 
   {
   if (out->ABL)
      {
      numberGraph++;
      }
   out = out->NEXT;
   }

reg = beh->BEREG;
while (reg) 
   {
   biabl = reg->BIABL;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         numberGraph++;
         }
      biabl = biabl->NEXT;
      }
   reg = reg->NEXT;
   }

bus = beh->BEBUS;
while (bus) 
   {
   biabl = bus->BIABL;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         numberGraph++;
         }
      biabl = biabl->NEXT;
      }
   bus = bus->NEXT;
   }

bux = beh->BEBUX;
while (bux) 
   {
   biabl = bux->BIABL;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         numberGraph++;
         }
      biabl = biabl->NEXT;
      }
   bux = bux->NEXT;
   }
return numberGraph;
}

/*------------------------------------------------------------------------------
traitementDCBeh     : procedure de prise en compte des Don't care
-------------------------------------------------------
parametres 	 : une befig
-------------------------------------------------------
return 		 : void
------------------------------------------------------------------------------*/
void traitementDCBeh(beh,trace,func,level)
befig_list *beh;
int trace;
int (*func)();
int level;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
biabl_list *biabl;
binode_list *binode;
pNode *pDC;
pNode *pON;
int numberGraph=0;
int i,j;
pNode simplifPlusDcZeroBdd();
pNode restrictBdd();
pNode cnstBdd();
pNode cliqueBdd();
pNode jacoBdd();
pTH dejaTraite;
pNode reportDC;
pElemTH pEl;
int coutInitial;
int coutFinal;
int boucle;

int DCFUN,CLIQUE,JACO,SIMPLE;

		/* dejaTraite sert a la re-ecriture des expressions */

dejaTraite = createTH(100);

		/* calcul du nombre de graphes */

printf("Running Don't Care Minimization...\n");

numberGraph = numberBddBeh(beh);;

pDC = (pNode *) mbkalloc (numberGraph*sizeof(pNode));
pON = (pNode *) mbkalloc (numberGraph*sizeof(pNode));

		/* mise a jour des tables */


if (trace)
   printf("Computing the Don't Care of each signal...\n");


calculDCBeh(beh,pON,pDC);

coutInitial = costTab(pON,numberGraph,func);

if (trace)
   printf("depth max = %d init = %d\n\n",costDepthTab(pON,numberGraph),coutInitial);

if (trace)
   printf("Applying the Don't Care simplification function...\n");


if (getenv("MBK_X") != NULL)
   {
   if (!strcmp(getenv("MBK_X"),"0"))
      {
      CLIQUE = 0;
      JACO = 0;
      printf("DC local --> FUNC = ");
      if (getenv("MBK_FUN") == NULL || !strcmp(getenv("MBK_FUN"),"0"))
         {
         DCFUN = 0;
         printf("restrict\n");
         }
      if (!strcmp(getenv("MBK_FUN"),"1"))
         {
         DCFUN = 1;
         printf("simplifPlus\n");
         }
      if (!strcmp(getenv("MBK_FUN"),"2"))
         {
         DCFUN = 2;
         printf("simplif\n");
         }
      if (!strcmp(getenv("MBK_FUN"),"3"))
         {
         DCFUN = 3;
         printf("constraint\n");
         }
      if (!strcmp(getenv("MBK_FUN"),"4"))
         {
         DCFUN = 4;
         printf("simplifOne\n");
         }
      }
   else
      {
      SIMPLE = 0;
      printf("DC global --> ALGO = ");
      if (!strcmp(getenv("MBK_X"),"1"))
         {
         CLIQUE = 1;
         JACO = 0;
         printf("clique on Multi-BDD\n");
         }
      if (!strcmp(getenv("MBK_X"),"2"))
         {
         CLIQUE = 0;
         JACO = 1;
         printf("jaco on Multi-BDD\n");
         }
      if (!strcmp(getenv("MBK_X"),"3"))
         {
         CLIQUE = 1;
         JACO = 0;
         SIMPLE = 1;
         printf("clique on BDD\n");
         }
      if (!strcmp(getenv("MBK_X"),"4"))
         {
         CLIQUE = 0;
         JACO = 1;
         SIMPLE = 1;
         printf("jaco on BDD\n");
         }
      }
   }
else
   {
   CLIQUE = 0;
   JACO = 0;
   DCFUN = 2;
   }

		/*-- ALGO des cliques ----*/

if (CLIQUE)
   {
   if (SIMPLE)
      {
      for (i=0; i < numberGraph; i++)
         {
         printf("===================================== [%d] =====\n",i);
      
         printf("pON :\n");
         displayBdd(pON[i],1);
         displayGraphicBdd(pON[i]);

         printf("pDC :\n");
         displayBdd(pDC[i],1);

         printf("pON simplifie par simplif :\n");
         displayBdd(simplifDcZeroBdd(pON[i],pDC[i]),1);

         pON[i] = cliqueBdd(pON[i],pDC[i]);

         printf("resultat pON :\n");
         displayBdd(pON[i],1);
         }
      }
   else
      {
      cliqueMultiBdd(pON,pDC,numberGraph);

/*
      for (i=0; i < numberGraph; i++)
         {
         printf("===================================== [%d] =====\n",i);
         printf("resultat pON :\n");
         displayBdd(pON[i],1);
         }
*/
      }
   }

		/*-- ALGO de Jacobi ----*/

if (JACO)
   {
   if (SIMPLE)
      {
      for (i=0; i < numberGraph; i++)
         {
         printf("===================================== [%d] =====\n",i);
      
         printf("pON :\n");
         displayGraphicBdd(pON[i]);

         printf("pDC :\n");
         displayGraphicBdd(pDC[i]);

         printf("pON simplifie par simplifZero :\n");
         displayGraphicBdd(simplifDcZeroBdd(pON[i],pDC[i]));

         printf("pON + pDC simplifie par simplifOne :\n");
         displayGraphicBdd(simplifDcOneBdd(applyBinBdd(OR,pDC[i],pON[i]),pDC[i]));

         pON[i] = jacoBdd(pON[i],pDC[i]);

         printf("resultat pON :\n");
         displayGraphicBdd(pON[i]);
         }
      }
   else
      {
      jacoMultiBdd(pON,pDC,numberGraph);

/*
      for (i=0; i < numberGraph; i++)
         {
         printf("===================================== [%d] =====\n",i);
         printf("resultat pON :\n");
         displayBdd(pON[i],1);
         }
*/
      }
   }

if (!JACO && !CLIQUE)
   {
   boucle = level;

   for (j=1; j <= boucle; j++)
      {
      for (i=0; i < numberGraph; i++)
         {
		/* le i-eme graphe possede un DC */

         if (pDC[i] != zero)
            {
            pNode pInter;
            pInter = pON[i];

			/* simplification de pON[i] */

            if (DCFUN == 0) pON[i] = restrictBdd(pON[i],notBdd(pDC[i]));
            if (DCFUN == 1) pON[i] = simplifPlusDcZeroBdd(pON[i],pDC[i]);
            if (DCFUN == 2) pON[i] = simplifDcZeroBdd(pON[i],pDC[i]);
            if (DCFUN == 3) pON[i] = cnstBdd(pON[i],notBdd(pDC[i]));
            if (DCFUN == 4) pON[i] = simplifDcOneBdd(applyBinBdd(OR,pDC[i],pON[i]),pDC[i]);

			/* minimisation successive */

            if (level == 2)
               {
               coutFinal = costTab(pON,numberGraph,func);

   
               if (coutInitial < coutFinal)    /* pas d'amelioration */
                  {
                  pON[i] = pInter;
                  }
               else      /* on enleve au DC la partie utilisee dans ON */
                  {
                  if (trace) 
                     {
                     printf("cost(%s) %d --> ",beh->NAME,coutInitial);
                     printf("%d\n",coutFinal);
                     }
                  pDC[i] = applyBinBdd(AND,pDC[i],notBdd(pON[i]));
                  coutInitial = coutFinal;
                  }
               }
            } 
         }
      }
   }

if (CLIQUE || (level == 1))
   {
   coutFinal = costTab(pON,numberGraph,func);
   if (trace) 
      {
      printf("cost(%s) %d --> ",beh->NAME,coutInitial);
      printf("%d\n",coutFinal);
      printf("depth max = %d\n\n",costDepthTab(pON,numberGraph));
      }
   }
		/* regeneration des equations simplifiees */

		/* on vide la table des sorties de la couches circuit */

pEl = ((beh->CIRCUI)->pTO)->pElem;

for (i = 0; i < ((beh->CIRCUI)->pTO)->length; i++)
   {
   pEl->value = EMPTYTH;
   pEl++;
   }

i = 0;

out = beh->BEOUT;
while (out) 
   {
   if (out->ABL)
      {
      if (out->NODE != pON[i])
         {
         out->NODE = pON[i];
         freeExpr(out->ABL);
         out->ABL = bddToAblCct(beh->CIRCUI,pON[i]);
         }
      i++;
      }
   out = out->NEXT;
   }

reg = beh->BEREG;
while (reg) 
   {
   biabl = reg->BIABL;
   binode = reg->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         if (binode->VALNODE != pON[i])
            {
            binode->VALNODE = pON[i];
            freeExpr(biabl->VALABL);
            biabl->VALABL = bddToAblCct(beh->CIRCUI,pON[i]);
            }
         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   reg = reg->NEXT;
   }

bus = beh->BEBUS;
while (bus) 
   {
   biabl = bus->BIABL;
   binode = bus->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         if (binode->VALNODE != pON[i])
            {
            binode->VALNODE = pON[i];
            freeExpr(biabl->VALABL);
            biabl->VALABL = bddToAblCct(beh->CIRCUI,pON[i]);
            }
         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   bus = bus->NEXT;
   }

bux = beh->BEBUX;
while (bux) 
   {
   biabl = bux->BIABL;
   binode = bux->BINODE;
   while (biabl)
      {
      if (biabl->VALABL)
         {
         if (binode->VALNODE != pON[i])
            {
            binode->VALNODE = pON[i];
            freeExpr(biabl->VALABL);
            biabl->VALABL = bddToAblCct(beh->CIRCUI,pON[i]);
            }
         i++;
         }
      biabl = biabl->NEXT;
      binode = binode->NEXT;
      }
   bux = bux->NEXT;
   }


mbkfree(pON);
mbkfree(pDC);
destroyTH(dejaTraite);
}
