/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : synthetiseur ACTEL                                          */
/*    Fichier : bool2act.c                                                  */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : DICTUS N.                             le : 14/06/1994     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
#include MUT_H
#include MLO_H
#include LOG_H
#include BEH_H
#include ICN_H
#include IAC_H
#include ICU_H
#include "bool2act.h"
#include "../compil/lax_param.h"
#include "fp_type.h"

#define CM8A 100
#define INV 101
#define DL1 102
#define DF1 103

/*------------------------------------------------------------------------------
padIcon          : genere un icon de buffer pour ACTEL (pad)
--------------------------------------------------------------------------------
parametres       : type = IN ou OUT
--------------------------------------------------------------------------------
return           : un icon
------------------------------------------------------------------------------*/
struct IconGate *padIcon(type, store_it)
char type;
int store_it;        /* 1 if the gate's icon have to be stored, 0 else */
{
struct IconGate *gate_icon;

gate_icon = addicongate( (type == IN) ? "inbuf" : "outbuf",
                         360, 0,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
                         500, -200,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
                         510, 200,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL);
if (type == IN)
   {
        /*sortie */
   addiconcon(gate_icon, "y",
              1000, 0, SAX_EAST, OUT,
              SAX_LEFT, SAX_DOWN, SAX_HORIZONTAL, 1000, -50);
        /* entree */
   addiconcon(gate_icon, "pad",
              0, 0, SAX_WEST, IN,
              SAX_RIGHT, SAX_DOWN, SAX_HORIZONTAL, 0, -50);
   }
else
   {
        /*sortie */
   addiconcon(gate_icon, "pad",
              1000, 0, SAX_EAST, OUT,
              SAX_LEFT, SAX_DOWN, SAX_HORIZONTAL, 1000, -50);
        /* entree */
   addiconcon(gate_icon, "d",
              0, 0, SAX_WEST, IN,
              SAX_RIGHT, SAX_DOWN, SAX_HORIZONTAL, 0, -50);
   }
 
addiconline( gate_icon, 300, 200, 200, 200);
addiconline( gate_icon, 300, -200, 200, -200);
addiconline( gate_icon, 700, 0, 1000, 0);
addiconline( gate_icon, 300, 200, 700, 0);
addiconline( gate_icon, 300, -200, 700, 0);
addiconline( gate_icon, 200, 0, 0, 0);
addiconline( gate_icon, 200, -200, 600, 0);
addiconline( gate_icon, 200, 200, 600, 0);
addiconline( gate_icon, 200, -200, 200, 200);
update_icon( gate_icon);
 
if (store_it)
   saveicon( gate_icon);
 
return gate_icon;
}

/*------------------------------------------------------------------------------
cstIcon    	: genere un icone de type constante
--------------------------------------------------------------------------------
parametres      : la constante (0 ou 1) et la suvegarde sur disque
--------------------------------------------------------------------------------
return          : une  structure icon
------------------------------------------------------------------------------*/
struct IconGate *cstIcon(cte, nom, sortie, store_it)
short cte;
char *nom, *sortie;     /* nom de la cellule et de l'entree */
int store_it;   /* 1 if the gate's icon have to be stored, 0 else */
{
struct IconGate *gate_icon;


if (cte == 1)
   {
   gate_icon = addicongate(nom, 440, 20,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL,
                         -60, -880,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL,
                         260, -60,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL);
   if (!gate_icon)
      return geticongate(nom);

   addiconcon( gate_icon, sortie, 400, -500, SAX_EAST, OUT,
                    SAX_CENTER, SAX_RIGHT, SAX_HORIZONTAL, 400, -550);
 
   addiconline( gate_icon, 200, -500, 400, -500);
   addiconline( gate_icon, 200, -700, -200, -700);
   addiconline( gate_icon, 200, -300, 200, -700);
   addiconline( gate_icon, -200, -300, 200, -300);
   addiconline( gate_icon, -200, -700, -200, -300);
   addiconline( gate_icon, -100, -400, 100, -400);
   addiconline( gate_icon, 0, -600, -100, -500);
   addiconline( gate_icon, 0, -600, 0, -400);
   }
else
   {
   gate_icon = addicongate(nom, 290, 210,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL,
                         -120, -280,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL,
                         -30, 320,
                         SAX_UP, SAX_RIGHT, SAX_HORIZONTAL);
 
   if (!gate_icon)
      return geticongate(nom);
 
   addiconcon( gate_icon, sortie,
                    400, 0, SAX_EAST, OUT,
                    SAX_CENTER, SAX_RIGHT, SAX_HORIZONTAL, 400, -50);
 
   addiconline( gate_icon, 100, -100, -100, 100);
   addiconline( gate_icon, 200, 0, 400, 0);
   addiconline( gate_icon, 200, -200, -200, -200);
   addiconline( gate_icon, 200, 200, 200, -200);
   addiconline( gate_icon, -200, 200, 200, 200);
   addiconline( gate_icon, -200, -200, -200, 200);
   addiconcircle( gate_icon, -100, -100, 200, 200);
   }
 
if (store_it)
   saveicon( gate_icon);
 
return gate_icon;
}
 
/*------------------------------------------------------------------------------
createAlim	 : genere une lofig de type actel ALIM GND/VCC
--------------------------------------------------------------------------------
parametres 	 : type = 1 ou 0.
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *createAlim(type)
short type;
{
lofig_list *liste;
int index = 0;
lofig_list *modelFig;
char *save_WORK_LIB;
char *nameModel = (type == 0) ? namealloc("gnd") : namealloc("vcc");

	/* recherche du model en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == nameModel)
      return liste;

	/* modele n'existe pas : on le cree */
modelFig = addlofig(nameModel);

	/* ajout de la sortie */
addlocon(modelFig, namealloc("y"),
         addlosig(modelFig, index++, 
                  addchain((chain_list *)NULL, namealloc("y")),
                  EXTERNAL, (float) 0),
         OUT);

save_WORK_LIB = WORK_LIB;
WORK_LIB = (char *) genNAME("%s",FP_ACTEL_LIB);

	/* Generation de l'icon corresponsant sur disque */
delicongate(ICONGATE_MODELNAME(cstIcon(type, nameModel, "y", 1)));

savelofig(modelFig);
WORK_LIB = save_WORK_LIB;
return modelFig;
}

/*------------------------------------------------------------------------------
createModelPad	 : genere une lofig de type actel Pad I/O
--------------------------------------------------------------------------------
parametres 	 : type = IN ou OUT.
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *createModelPad(type)
char type;
{
lofig_list *liste;
int index = 0;
lofig_list *modelFig;
bepor_list *port, *auxPort = (bepor_list *)NULL;
char *save_WORK_LIB;
char *nameModel = (type == IN) ? namealloc("inbuf") : namealloc("outbuf");

	/* recherche du model en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == nameModel)
      return liste;

	/* modele n'existe pas : on le cree */
modelFig = addlofig(nameModel);

	/* ajout des entree / sortie */
if (type == IN)
   {
	/* INBUF */
   addlocon(modelFig, namealloc("pad"),
            addlosig(modelFig, index++, 
                     addchain((chain_list *)NULL, namealloc("pad")),
                     EXTERNAL, (float) 0),
            IN);
   addlocon(modelFig, namealloc("y"),
            addlosig(modelFig, index++, 
                     addchain((chain_list *)NULL, namealloc("y")),
                     EXTERNAL, (float) 0),
            OUT);
   }
else
   if (type == OUT)
      {
	/* OUTBUF */
      addlocon(modelFig, namealloc("d"),
               addlosig(modelFig, index++, 
                        addchain((chain_list *)NULL, namealloc("d")),
                        EXTERNAL, (float) 0),
               IN);
      addlocon(modelFig, namealloc("pad"),
               addlosig(modelFig, index++, 
                        addchain((chain_list *)NULL, namealloc("pad")),
                        EXTERNAL, (float) 0),
               OUT);
      }
   else
      printf("Warning : createModelPad : direction inexpected %c\n", type);

	/* Generation de l'icon corresponsant sur disque */
save_WORK_LIB = WORK_LIB;
WORK_LIB = (char *) genNAME("%s",FP_ACTEL_LIB);

	/* Generation de l'icon corresponsant sur disque */
delicongate(ICONGATE_MODELNAME(padIcon(type, 1)));

savelofig(modelFig);
WORK_LIB = save_WORK_LIB;
return modelFig;
}

/*------------------------------------------------------------------------------
createModelFpga	 : genere une lofig de type actel a partir de son modele VHDL.
--------------------------------------------------------------------------------
parametres 	 : befig. les entrees sont declarees avant les sorties
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *createModelFpga(befig)
befig_list *befig;
{
lofig_list *liste;
int index = 0;
lofig_list *modelFig;
char *save_WORK_LIB;
bepor_list *port, *auxPort = (bepor_list *)NULL;
berin_list *in, *auxIn = (berin_list *)NULL;

	/* recherche du model en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == befig->NAME)
      return liste;

	/* modele n'existe pas : on le cree */
modelFig = addlofig(befig->NAME);

	/* reverse pas physique */
for(port = befig->BEPOR; port; port = port->NEXT)
   auxPort = beh_addbepor(auxPort, port->NAME,
                          port->DIRECTION, port->TYPE);

	/* ajout des entrees / sorties */
for(port = auxPort; port; port = port->NEXT)
   {
	/* creation du connecteur et du signal  de l'entree/sortie */
   if ((port->DIRECTION == IN) || (port->DIRECTION == OUT))
      addlocon(modelFig, port->NAME,
               addlosig(modelFig, index++, 
                        addchain((chain_list *)NULL, port->NAME),
                        EXTERNAL, (float) 0),
               port->DIRECTION);
   else
      printf("Warning : Unknown direction port : %s\n", port->NAME);
   }

save_WORK_LIB = WORK_LIB;
WORK_LIB = (char *) genNAME("%s",FP_ACTEL_LIB);

	/* reverse des berin pour le bon ordre des icon !!! */
for(in = befig->BERIN; in; in = in->NEXT)
   auxIn = beh_addberin(auxIn, in->NAME);

savelofig(modelFig);
WORK_LIB = save_WORK_LIB;
return modelFig;
}

/*------------------------------------------------------------------------------
bool2Act	 : genere une lofig a partir d'une befig telle quelle.
		   (un noeud du reseau booleen correspond a un bloc actel)
--------------------------------------------------------------------------------
parametres 	 : une befig, une table des operateurs.
                   table : operateur --> befig correspondante.
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *bool2Act(beh, operName)
befig_list *beh;
char **operName;
{
lofig_list *fig = (lofig_list *)NULL;
bepor_list *ent;
bepor_list *port;
chain_list *sigchain = (chain_list *)NULL;
beaux_list *gate;
beout_list *sortie;
bereg_list *reg;
losig_list *sigZero = (losig_list *)NULL;
losig_list *sigOne  = (losig_list *)NULL;
loins_list *instZero = (loins_list *)NULL;
loins_list *instOne  = (loins_list *)NULL;
pTH sigTH;
int indexSig = 1;
lofig_list *liste;
char *INBUF = namealloc("inbuf");
loins_list *inst;
locon_list *con;

printf("bool2Act : creating %s\n", beh->NAME);

/*
displayBeh(beh, 0);
*/

	/* recherche de la lofig en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == beh->NAME)
      return liste;

	/* table hash : nom connecteur externe --> pointeur du signal */
sigTH = createTH(200);

	/* lofig n'existe pas : on la cree */
fig = addlofig(beh->NAME);

	/* --- ajout des connecteurs externes --- */
for(port = beh->BEPOR; port; port = port->NEXT)
   if (port->DIRECTION == IN)
      {
      chain_list *sigChain = (chain_list *)NULL;

	/* creation du signal correspond au connecteur (pad cote externe) */
      losig_list *sig = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL, port->NAME), 
                                 EXTERNAL, (float) 0);

	/* creation du signal correspond au pad cote interne */
      losig_list *sigOut = addlosig(fig, indexSig++,
                                    addchain((chain_list *)NULL,
                                             (char *)genNAME("buf%s", port->NAME)),
                                    INTERNAL, (float )0);
      lofig_list *modelPad = getlofig(INBUF, 'E');

      sigChain = addchain(sigChain, sig);
      sigChain = addchain(sigChain, sigOut);
      fig->LOINS = addloins(fig, (char *)genNAME("inbuf%s", port->NAME),
                            modelPad, sigChain);

	/* ajout du connecteur dans la lofig */
      addlocon(fig,port->NAME,sig, port->DIRECTION);
      addTH(sigTH, port->NAME, (int)sigOut);
      }

	/* ajoute des instances BEREG */
for(reg = beh->BEREG; reg; reg = reg->NEXT)
   {
   int cptBiabl = 0;
   lofig_list *model;
   int val;
   losig_list *sig = (losig_list *)NULL;
   biabl_list *biabl;
   short bascule = 0;

/*
   printf("REG %s\n", reg->NAME);
*/

   sigchain = (chain_list *)NULL;

   if (reg->BIABL->NEXT)
      {
      printf("Error : latch %s with more than one BIABL\n", reg->NAME);
      return (lofig_list *)NULL;
      }

	/* parcours de l'expression pour les signaux sur les entrees */
   for(biabl = reg->BIABL; biabl; biabl = biabl->NEXT)
      {
      chain_list *exp;
      short cptArg = 0;

      cptBiabl++;

      if (ATOM(biabl->VALABL))
         {
         printf("Error : latch %s has an atomic VALUE", reg->NAME);
         displayExpr(biabl->VALABL);
         return (lofig_list *)NULL;
         }

      for(exp = CDR(biabl->VALABL); exp; exp = exp->NEXT)
         {
         if (ATOM(CAR(exp)))
            {
            cptArg++;
            if (!strcmp(VALUE_ATOM(CAR(exp)), "'1'"))
               {
               if (!sigZero)
                  {
                  sigZero = addlosig(fig, indexSig++,
                                     addchain((chain_list *)NULL,
                                              (char *)genNAME("vcc%d", indexSig)),
                                     INTERNAL, (float)0);
                  fig->LOINS = addloins(fig, (char *)genNAME("vcc%d", indexSig),
                                        getlofig(namealloc("vcc"), 'E'),
                                        addchain((chain_list *)NULL,
                                                 (void *)sigZero));
                  }
               sigchain = addchain(sigchain, (void *)sigZero);
               }
            else
               if (!strcmp(VALUE_ATOM(CAR(exp)), "'0'"))
                  {
                  if (!sigOne)
                     {
                     sigOne = addlosig(fig, indexSig++,
                                       addchain(NULL, (char *)genNAME("gnd%d", indexSig)),
                                       INTERNAL, (float)0);
                     fig->LOINS = addloins(fig, (char *)genNAME("gnd%d", indexSig),
                                           getlofig(namealloc("gnd"), 'E'),
                                           addchain((chain_list *)NULL,
                                                    (void *)sigOne));
                     }
                  sigchain = addchain(sigchain, (void *)sigOne);
                  }
               else
                  if ((val = searchTH(sigTH, VALUE_ATOM(CAR(exp)))) != EMPTYTH)
                     sigchain = addchain(sigchain, (void *)val);
                  else
                     {
                     sig = addlosig(fig, indexSig++,
                                    addchain((chain_list *)NULL,
                                             VALUE_ATOM(CAR(exp))), 
                                    INTERNAL, (float) 0);
                     sigchain = addchain(sigchain, (void *)sig);
                     addTH(sigTH, VALUE_ATOM(CAR(exp)), (int)sig);
                     }
            }
         else
            {
                /* on n'a pas un atome dans l'abl du registre */
            printf("Error : Not atomic expression in REG %s", reg->NAME);
            displayExpr(biabl->VALABL);
            return (lofig_list *)NULL;
            }
         }

	/* verification du nombre d'arguments */
/*
      for (ent = (operName[OPER(biabl->VALABL)])->BEPOR; ent; ent = ent->NEXT)
          if (ent->DIRECTION == IN)
             cptArg--;
      if (cptArg != 0)
         {
         printf("Error : bad arguments number in REG %s", reg->NAME);
         displayExpr(biabl->VALABL);
         return (lofig_list *)NULL;
         }
*/

	/* signal sur la sortie du registre */
      if ((val = searchTH(sigTH, reg->NAME)) != EMPTYTH)
         sigchain = addchain(sigchain, (void *)val);
      else
         {
         sig = addlosig(fig, indexSig++,
                        addchain((chain_list *)NULL, reg->NAME), 
                        INTERNAL, (float) 0);

         sigchain = addchain(sigchain, (void *)sig);
         addTH(sigTH, reg->NAME, (int)sig);
         }

      model = getlofig(operName[OPER(biabl->VALABL)], 'E');
/*
      model = createModelFpga(operName[OPER(biabl->VALABL)]);
*/
      fig->LOINS = addloins(fig, reg->NAME, model, sigchain);
      }
   }

	/* ajoute des instances BEAUX */
for(gate = beh->BEAUX; gate; gate = gate->NEXT)
   {
   int val;
   sigchain = (chain_list *)NULL;

/*
   printf("AUX %s (%d) :", gate->NAME, OPER(gate->ABL));
   displayExpr(gate->ABL);
*/
   if (!ATOM(gate->ABL))
      {
      chain_list *exp;
      losig_list *sig = (losig_list *)NULL;
      lofig_list *model;
      short cptArg = 0;

	/* parcours de l'expr pour les signaux sur les entrees */
      for(exp = CDR(gate->ABL); exp; exp = exp->NEXT)
         {
         if (ATOM(CAR(exp)))
            {
            cptArg++;
            if (!strcmp(VALUE_ATOM(CAR(exp)), "'1'"))
               {
               if (!sigZero)
                  {
                  sigZero = addlosig(fig, indexSig++,
                                     addchain((chain_list *)NULL,
                                              (char *)genNAME("vcc%d", indexSig)),
                                     INTERNAL, (float)0);
                  fig->LOINS = addloins(fig, (char *)genNAME("vcc%d", indexSig),
                                        getlofig(namealloc("vcc"), 'E'),
                                        addchain((chain_list *)NULL,
                                                 (void *)sigZero));
                  }
               sigchain = addchain(sigchain, (void *)sigZero);
               }
            else
               if (!strcmp(VALUE_ATOM(CAR(exp)), "'0'"))
                  {
                  if (!sigOne)
                     {
                     sigOne = addlosig(fig, indexSig++,
                                       addchain(NULL, (char *)genNAME("gnd%d", indexSig)),
                                       INTERNAL, (float)0);
                     fig->LOINS = addloins(fig, (char *)genNAME("gnd%d", indexSig),
                                           getlofig(namealloc("gnd"), 'E'),
                                           addchain((chain_list *)NULL,
                                                    (void *)sigOne));
                     }
                  sigchain = addchain(sigchain, (void *)sigOne);
                  }
               else
                  if ((val = searchTH(sigTH, VALUE_ATOM(CAR(exp)))) != EMPTYTH)
                     sigchain = addchain(sigchain, (void *)val);
                  else
                     {
                     sig = addlosig(fig, indexSig++, 
                                    addchain((chain_list *)NULL,
                                             VALUE_ATOM(CAR(exp))),
                                    INTERNAL, (float) 0);
                     sigchain = addchain(sigchain, (void *)sig);
                     addTH(sigTH, VALUE_ATOM(CAR(exp)), (int)sig);
                     }
            }
         else
            {
		/* on n'a pas un atome dans l'abl de l'auxiliaire */
            printf("Error : Not atomic expression in AUX %s", gate->NAME);
            displayExpr(gate->ABL);
            return (lofig_list *)NULL;
            }
         }

        /* verification du nombre d'arguments */
/*
      for (ent =  (operName[OPER(gate->ABL)])->BEPOR; ent; ent = ent->NEXT)
          if (ent->DIRECTION == IN)
             cptArg--;
      if (cptArg != 0)
         {
         printf("Error : bad arguments (%d) number in AUX %s : ", cptArg, gate->NAME);
         displayExpr(gate->ABL);
         return (lofig_list *)NULL;
         }
*/

	/* signal sur la sortie */
      if ((val = searchTH(sigTH, gate->NAME)) != EMPTYTH)
            sigchain = addchain(sigchain, (void *)val);
      else
         {
         sig = addlosig(fig, indexSig++,
                        addchain((chain_list *)NULL, gate->NAME), 
                        INTERNAL, (float) 0);

         sigchain = addchain(sigchain, (void *)sig);
         addTH(sigTH, gate->NAME, (int)sig);
         }

      model = getlofig(operName[OPER(gate->ABL)], 'E');
/*
      model = createModelFpga(operName[OPER(gate->ABL)]);
*/
      fig->LOINS = addloins(fig, gate->NAME, model, sigchain);
      }
   else
      {
	/* cas atomique */
      printf("Error : AUX %s is an atomic expression = ", gate->NAME);
      displayExpr(gate->ABL);
      return (lofig_list *)NULL;
      }
   }
	/* ajoute des instances BEOUT */
for(sortie = beh->BEOUT; sortie; sortie = sortie->NEXT)
   {
   int val;
   losig_list *sigAux = (losig_list *)NULL;
   chain_list *lstChain = (chain_list *)NULL;
   lofig_list *modelPad = getlofig(namealloc("outbuf"), 'E');
   losig_list *sig;

   sigchain = (chain_list *)NULL;

/*
   printf("OUT %s (%d)", sortie->NAME, OPER(sortie->ABL));
   displayExpr(sortie->ABL);
*/

   if (!ATOM(sortie->ABL))
      {
      chain_list *exp;
      lofig_list *model;
      short cptArg = 0;
      losig_list *sig = (losig_list *)NULL;

	/* parcours de l'expr pour les signaux sur les entrees */
      for(exp = CDR(sortie->ABL); exp; exp = exp->NEXT)
         {
         if (ATOM(CAR(exp)))
            {
            cptArg++;
            if (!strcmp(VALUE_ATOM(CAR(exp)), "'1'"))
               {
               if (!sigZero)
                  {
                  sigZero = addlosig(fig, indexSig++,
                                     addchain((chain_list *)NULL,
                                              (char *)genNAME("vcc%d", indexSig)),
                                     INTERNAL, (float)0);
                  fig->LOINS = addloins(fig,
                                       (char *)genNAME("vcc%d", indexSig),
                                       getlofig(namealloc("vcc"), 'E'),
                                       addchain((chain_list *)NULL,
                                                (void *)sigZero));
                  }
               sigchain = addchain(sigchain, (void *)sigZero);
               }
            else
               if (!strcmp(VALUE_ATOM(CAR(exp)), "'0'"))
                  {
                  if (!sigOne)
                     {
                     sigOne = addlosig(fig, indexSig++,
                                       addchain((chain_list *)NULL,
                                                (char *)genNAME("gnd%d", indexSig)),
                                       INTERNAL, (float)0);
                     fig->LOINS = addloins(fig, (char *)genNAME("gnd%d", indexSig),
                                           getlofig(namealloc("gnd"), 'E'),
                                           addchain((chain_list *)NULL,
                                                    (void *)sigOne));
                     }
                  sigchain = addchain(sigchain, (void *)sigOne);
                  }
               else
                  if ((val = searchTH(sigTH, VALUE_ATOM(CAR(exp)))) != EMPTYTH)
                     sigchain = addchain(sigchain, (void *)val);
                  else
                     {
                     sig = addlosig(fig, indexSig++,
                                    addchain((chain_list *)NULL,
                                             VALUE_ATOM(CAR(exp))),
                                    INTERNAL, (float) 0);
                     sigchain = addchain(sigchain, (void *)sig);
                     addTH(sigTH, VALUE_ATOM(CAR(exp)), (int)sig);
                     }
            }
         else
            {
		/* on n'a pas un atome dans l'abl de la sortie */
            printf("Error : Not atomic expression in OUT %s", sortie->NAME);
            displayExpr(sortie->ABL);
            return (lofig_list *)NULL;
            }
         }

        /* verification du nombre d'arguments */
/*
      for (ent =  operName[OPER(sortie->ABL)]->BEPOR; ent; ent = ent->NEXT)
          if (ent->DIRECTION == IN)
             cptArg--;
      if (cptArg != 0)
         {
         printf("Error : bad arguments number in OUT %s", sortie->NAME);
         displayExpr(sortie->ABL);
         return (lofig_list *)NULL;
         }
*/

	/* Ajout du signal sur la sortie */
      sigAux = addlosig(fig, indexSig++,
                        addchain((chain_list *)NULL,
                                 genNAME("aux%s",sortie->NAME)), 
                        INTERNAL, (float) 0);

      sigchain = addchain(sigchain, (void *)sigAux);
      model = getlofig(operName[OPER(sortie->ABL)], 'E');
/*
      model = createModelFpga(operName[OPER(sortie->ABL)]);
*/
      fig->LOINS = addloins(fig, (char *)genNAME("out%s", sortie->NAME),
                            model, sigchain);
      }
   else
      {
	/* cas atomique : sortie reliee a une registre ou un aux */
      int valAux;

      if (!strcmp(VALUE_ATOM(sortie->ABL), "'1'"))
         {
         if (!sigZero)
            {
            sigZero = addlosig(fig, indexSig++,
                               addchain((chain_list *)NULL,
                                        (char *)genNAME("vcc%d", indexSig)),
                               INTERNAL, (float)0);
            fig->LOINS = addloins(fig,
                                 (char *)genNAME("vcc%d", indexSig),
                                 getlofig(namealloc("vcc"), 'E'),
                                 addchain((chain_list *)NULL,
                                          (void *)sigZero));
            }
         sigAux =sigZero;
         }
      else
         if (!strcmp(VALUE_ATOM(sortie->ABL), "'0'"))
            {
            if (!sigOne)
               {
               sigOne = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL,
                                          (char *)genNAME("gnd%d", indexSig)),
                                 INTERNAL, (float)0);
               fig->LOINS = addloins(fig, (char *)genNAME("gnd%d", indexSig),
                                     getlofig(namealloc("gnd"), 'E'),
                                     addchain((chain_list *)NULL,
                                              (void *)sigOne));
               }
            sigAux = sigOne;
            }
         else
            {
            if ((valAux = searchTH(sigTH, VALUE_ATOM(sortie->ABL))) == EMPTYTH)
               printf("Warning : OUT %s (%s) is not in BEFIG\n", sortie->NAME, 
                      VALUE_ATOM(sortie->ABL));
            sigAux = (losig_list *)valAux;
            }
      }
        /* creation du signal correspond au connecteur (pad cote externe) */
   sig = addlosig(fig, indexSig++,
                  addchain((chain_list *)NULL, sortie->NAME),
                  EXTERNAL, (float) 0);
   lstChain = addchain(lstChain, sigAux);
   lstChain = addchain(lstChain, sig);
   fig->LOINS = addloins(fig, (char *)genNAME("outbuf%s", sortie->NAME),
                         modelPad, lstChain);
   addlocon(fig, sortie->NAME, sig, OUT);
   addTH(sigTH, sortie->NAME, (int)sigAux);
   }

destroyTH(sigTH);

	/* elimination des buffers d'entrees inutiles */
lofigchain(fig);
for(inst = fig->LOINS; inst; inst = inst->NEXT)
   if (inst->FIGNAME == INBUF)
      for(con = inst->LOCON; con; con = con->NEXT)
         if (con->DIRECTION == OUT)
            {
            ptype_list *liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);

            if (!((chain_list *)liste->DATA)->NEXT)
               {
               printf("Elimination de %s\n", inst->INSNAME);
               delloins(fig, inst->INSNAME);
               }
            }
/*
displayLofigAct(fig);
*/

return fig;
}

/*---------------------------------------------------------------------------
displayLofig	: affiche une lofig (INSTANCE - CONNECTEUR - SIGNAL - 
                                     CONNECTEUR - INSTANCE)
-----------------------------------------------------------------------------
retour		: void
---------------------------------------------------------------------------*/
void displayLofigAct(lofig)
lofig_list *lofig;
{
loins_list *inst;

	/* parcours des instances de la figure */
for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   {
   int nbOut = 0;
   locon_list *con;
   ptype_list *fanout = NULL;

   printf("  INSTANCE : %s [%s]\n", inst->INSNAME, inst->FIGNAME);

	/* parcours des connecteurs de l'instance */
   for(con = inst->LOCON; con; con = con->NEXT)
      {
      ptype_list *liste;
      chain_list *l;

      if (!isvdd(con->NAME) && !isvss(con->NAME))
         {
         printf("   CONNECTOR : %s (%c) --- SIGNAL %s --- \n",
                con->NAME,con->DIRECTION, (char *)(con->SIG->NAMECHAIN->DATA));

		/* liste des liaisons signaux --> connecteurs */
         if (0)
         {
         liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);
         for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
            {
            if (((locon_list *)(l->DATA))->ROOT != (void *)inst)
               {
               locon_list *conAux = (locon_list *)l->DATA;

               if (conAux->TYPE == EXTERNAL)
                  {
			/* connecteurs externes : E/S primaires */
                  printf(" Extern %s", conAux->NAME);
                  }
               else
                  {
   			/* connecteur interne */
                  loins_list *cellOut = (loins_list *)conAux->ROOT;

		    /* DETERMINE LE FANOUT DE CHAQUE CELLULE */

                  if (con->DIRECTION == OUT)
                     {
                     nbOut++;
                     printf("  %s[%s],", cellOut->INSNAME, cellOut->FIGNAME);
                     }
                  else
                     if (conAux->DIRECTION == OUT)
                        printf("  %s[%s],", cellOut->INSNAME, cellOut->FIGNAME);
                  }
               }
            }
         printf("\n");
         }
         }
      }		/* fin parcours connecteurs */

   if (0 && nbOut > 0)
      {
      printf("     %s drives %d gate%s\n",
             inst->INSNAME, nbOut, (nbOut > 1)? "s":"");
      }
   printf("\n");
   }
}

/*---------------------------------------------------------------------------
driverAct	: driver ADL
-----------------------------------------------------------------------------
parametre	: lofig a driver
-----------------------------------------------------------------------------
retour		: void. un fichier .adl est genere dans MBK_WORK_LIB
---------------------------------------------------------------------------*/
void driverAct(lofig)
lofig_list *lofig;
{
loins_list *inst;
locon_list *con;
losig_list *sig;
FILE *fich;
char *fileStat = (char *)mbkalloc(strlen(WORK_LIB) + strlen(lofig->NAME) + 
                                  strlen(OUT_LO) + 4);

sprintf(fileStat, "%s/%s.adl", WORK_LIB, lofig->NAME, OUT_LO);

printf("Generating file '%s'\n", fileStat);

if (fich = fopen(fileStat, "w"))
   {
   lofigchain(lofig);

   fprintf(fich, "DEF MAPTOP;");

	/* parcours des connecteurs de la figure */
   for(con = lofig->LOCON; con; con = con->NEXT)
      {
      if (con->NEXT)
         fprintf(fich, " %s,", con->NAME);
      else
         fprintf(fich, " %s.", con->NAME);
      }
   fprintf(fich, "\n");

	/* parcours des instances de la figure */
   for(inst = lofig->LOINS; inst; inst = inst->NEXT)
      {
      fprintf(fich, "USE ADLIB:%s; %s.\n", inst->FIGNAME, inst->INSNAME);
      }

	/* parcours des signaux de la figure */
   for(sig = lofig->LOSIG; sig; sig = sig->NEXT)
      {
      ptype_list *liste = getptype(sig->USER, (long)LOFIGCHAIN);
      chain_list *l;

      fprintf(fich, "NET %s;", (char *)genNAME("DEF_NET_%d", sig->INDEX));

      for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
         {
         locon_list *conAux = (locon_list *)l->DATA;
      
         if (conAux->TYPE == INTERNAL)
            if (l->NEXT)
               fprintf(fich, " %s: %s,", ((loins_list *)conAux->ROOT)->INSNAME, conAux->NAME);
            else
               fprintf(fich, " %s: %s.", ((loins_list *)conAux->ROOT)->INSNAME, conAux->NAME);
         else
            if (l->NEXT)
               fprintf(fich, " %s,", conAux->NAME);
            else
               fprintf(fich, " %s.", conAux->NAME);
         }
      fprintf(fich, "\n");
      }
   fprintf(fich, "END.\n");
   }
else
   printf("Can't open file %s\n", fileStat);
}
