/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  preuve formelle 				       	    */
/*    Fichier :  proof.c                                                    */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  L. Burgun                            le : 09/08/1991     */
/*                                                                          */
/*    Modifie par :                                     le : 13/11/1991     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include LOG_H
#include BEH_H
#include "../compil/lax_param.h"
#include "../synthe/sl_type.h"
#include "../mapping/types.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "proof.h"

		/* on ne verifie pas si les fichiers existent ...*/

#define VERIFILE 0

char AuVol();

/*----------------------------------------------------------------------------
verifArgsProof	: verifie les arguments de la ligne de commande.
------------------------------------------------------------------------------
retour		: un entier a 1 si OK, 1 sinon.
------------------------------------------------------------------------------*/

int verifArgsProof(argc,argv)
int argc;
char *argv[];
{
FILE *fic;
FILE *fopen();

if (argc < 3  || argc > 6)
   {
   printf("\n");
   printf("proof 2.02   \nusage  :   proof -option");
   printf(" <file_entity1> <file_entity2> \n");
   printf("option :  -d   display the logical functions\n");
   printf("          -a   the common auxiliary signals are kept\n");
   printf("          -p   the polarity of the triggered edges is inversed \n");
   printf("               within the first behavioural description\n");
   return(0) ;
   }
if (argc > 3)
   {
   if(strcmp(argv[1],"-d") && strcmp(argv[1],"-a") && strcmp(argv[1],"-p"))
   	{
	printf("\nproof 2.02    : unknown option '%s'\n\n",argv[1]) ;
	return(0) ;
	}
   }
if (argc > 4)
   {
   if(strcmp(argv[2],"-d") && strcmp(argv[2],"-a") && strcmp(argv[2],"-p"))
   	{
	printf("\nproof 2.02    : unknown option '%s'\n\n",argv[2]) ;
	return(0) ;
	}
   }
if (argc > 5)
   {
   if(strcmp(argv[3],"-d") && strcmp(argv[3],"-a") && strcmp(argv[3],"-p"))
   	{
	printf("\nproof 2.02    : unknown option '%s'\n\n",argv[2]) ;
	return(0) ;
	}
   }
if (VERIFILE)
   {
   char *auxFile1 = mbkalloc(strlen(argv[argc-2]) + 2);
   char *auxFile2 = mbkalloc(strlen(argv[argc-1]) + 2);

   sprintf(auxFile1, "%s", argv[argc-2]);
   if(!(fic = fopen(auxFile1, "r")))
	{
	printf("\nproof 2.02    : unable to open '%s'\n\n",argv[argc-2]) ;
	return(0) ;
	}

   sprintf(auxFile2, "%s", argv[argc-1]);
   if(!(fic = fopen(auxFile2, "r")))
	{
	printf("\nproof 2.02    : unable to open '%s'\n\n",argv[argc-1]) ;
	return(0) ;
	}
   }
return(1);
}

/*======================================================================/
/		                MAIN					/
/======================================================================*/

main(argc,argv)
int argc ;
char *argv[];
{
char varaux;
chain_list *ptChain,*order;
befig_list *bef1,*bef2;
pCircuit CC1,CC2;
char *path;
char *auxpath;
char *getenv();
int elimBusBef1,elimBusBef2;

		/* verification de la validite des arguments */

mbkenv();
VHB_HEDFIG = NULL;

path = (char *)malloc(5);
auxpath = (char *) malloc (5);

path = WORK_LIB;

if (!getenv("MBK_CATA_LIB"))
   strcpy(auxpath,".");
else
   auxpath = getenv("MBK_CATA_LIB");

if (argc > 2)
   {
   char *auxFile1 = mbkalloc(strlen(argv[argc-2]) + 5);
   char *auxFile2 = mbkalloc(strlen(argv[argc-1]) + 5);

   sprintf(auxFile1, "%s.vbe",argv[argc-2]);
   argv[argc-2] = auxFile1;

   sprintf(auxFile2, "%s.vbe",argv[argc-1]);
   argv[argc-1] = auxFile2;
   }

if (!verifArgsProof(argc,argv))
   exit(-1);

alliancebanner("Proof","2.02","Formal Proof","90-94","2.1");

printf("================================  Environment  ================================\n");
printf("MBK_WORK_LIB		= %s\n",path);
printf("MBK_CATA_LIB		= %s\n",auxpath);
printf("=======================  Files, Options and Parameters  =======================\n");
printf("First VHDL file		= %s\n",argv[argc-2]);
printf("Second VHDL file	= %s\n",argv[argc-1]);

if(!strcmp(argv[1],"-a") || !strcmp(argv[2],"-a") || 
   (argc > 4 && !strcmp(argv[3],"-a")))
   printf("The common auxiliary signals are kept\n");
else
   printf("The auxiliary signals are erased\n");

if(!strcmp(argv[1],"-d") || !strcmp(argv[2],"-d") || 
   (argc > 4 && !strcmp(argv[3],"-d")))

printf("Errors are displayed\n");

printf("===============================================================================\n");
printf("\n");


if (argc > 2 && argc < 6)
   {

   initializeBdd(2);

		/* compilation du VHDL du premier circuit */
		/* on ne calcule pas les graphes tout de suite */

   if (compileVHDL(argv[argc-2],NULL,0,NULL,1,0,0,0,1,nameIntVect,NULL) == NULL)
      {
      printf("*** Compilation aborted...\n");
      exit(-1);
      }

   bef1 = VHB_HEDFIG;

   if(!strcmp(argv[1],"-a") || !strcmp(argv[2],"-a") || 
      (argc > 4 && !strcmp(argv[3],"-a")))
     varaux = 'A';
   else
     varaux = 'G';

		/* compilation du VHDL du deuxieme circuit */
		/* on ne calcule pas les graphes tout de suite */

   if (compileVHDL(argv[argc-1],NULL,0,NULL,1,0,0,0,1,nameIntVect,NULL) == NULL)
      {
      printf("compilation aborted\n");
      printf("*** Compilation aborted...\n");
      exit(-1);
      }

   bef2 = VHB_HEDFIG;


		/* changement de polarite des BD de bef1 */

   if(!strcmp(argv[1],"-p") || !strcmp(argv[2],"-p") || 
      (argc > 4 && !strcmp(argv[3],"-p")))
      polarBDBeh(bef1);

		/* elimination des INOUT */

   elimInoutBeh(bef1);
   elimInoutBeh(bef2);

		/* calcul d'ordonnancement */

   orderProofBeh(bef1,bef2,varaux);

		/* re-ecriture des bus multiplex */
   
   elimBusBef1 = elimBusMuxBeh(bef1);
   elimBusBef2 = elimBusMuxBeh(bef2);

   if (elimBusBef1 || elimBusBef2) 
      {
			/* re-ordonnancement */

      orderProofBeh(bef1,bef2,varaux);
      }

		/*  renomage des befig */

   {
   char *auxBeh1 = mbkalloc(strlen(path) + strlen(bef1->NAME) + 2);
   char *auxBeh2 = mbkalloc(strlen(path) + strlen(bef2->NAME) + 2);

   sprintf(auxBeh1, "%s/%s",path,bef1->NAME);
   bef1->NAME = auxBeh1;

   sprintf(auxBeh2, "%s/%s",path,bef2->NAME);
   bef2->NAME = auxBeh2;
   }

   if(!strcmp(argv[1],"-d") || !strcmp(argv[2],"-d") || 
      (argc > 4 && !strcmp(argv[3],"-d")))
      {
      int ret = proofBeh(bef1,bef2,1,0); 
      printf("\n\n\n");
      if (ret)
         exit(-1);
      else
         exit(0);
      }
   else
      {
      int ret = proofBeh(bef1,bef2,0,0); 
      printf("                      Formal Proof : ");
      if (ret > 0) 
         {
         printf("%d error%s\n",ret,(ret > 1)?"s":"");
         exit(-1);
         }
      else 
         {
         printf("OK\n\n\n");
         exit(0);
         }
      }
   }
   printf("\n\n\n");
   exit(-1);
}
