/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  Synthetiseur Logique                                       */
/*    Fichier :  clique.c                                                   */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  L. Burgun                            le : 09/03/1994     */
/*                                                                          */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H

#define EXTRACT 15
#define SIMPLIFLOC 1

#define TAILLEGRAPH 500

struct couple {
   pNode on;
   pNode off;
   pNode onp;
   pNode offp;
   chain_list *lst;
   int count;
   } ;

/*-------------------------------------------------------------------------
suptautology    : calcule si il existe une couverture commune a deux
                  fonctions phi-Boolennes 
---------------------------------------------------------------------------
retour          : 0 ou 1 
---------------------------------------------------------------------------*/
int suptautology(on1,off1,on2,off2)
pNode on1,off1,on2,off2;
{
if ((applyBinBdd(AND,on1,off2) == zero) &&
    (applyBinBdd(AND,on2,off1) == zero))
   return 1;
else
   return 0;
}

/*-------------------------------------------------------------------------
simplifONOFF    : simplifie ON et OFF avec des regles :
                       1. assignations a tautologie
                       2. elimination du noeud pere de ON 
---------------------------------------------------------------------------
retour          : si modif 
---------------------------------------------------------------------------*/
int simplifONOFF(pON,pOFF,ppON,ppOFF)
pNode pON,pOFF;
pNode *ppON,*ppOFF;
{
int modifONOFF = 0;

*ppON = pON;
*ppOFF = pOFF;

if (pOFF == zero)	/* assignation a tautologie */
   {
   *ppON = one;
   *ppOFF = zero;
   modifONOFF = 1;
   }
else
   {
   if (pON->index > 1)	/* elimination du noeud racine de pON */
      {
			/* index egaux */
 
      if (pON->index == pOFF->index)
         {
         if ((pON->high == zero) && (pOFF->high == zero))
            {
            *ppON = pON->low;
            *ppOFF = pOFF->low;
            modifONOFF = 1;
            }
         if ((pON->low == zero) && (pOFF->low == zero))
            {
            *ppON = pON->high;
            *ppOFF = pOFF->high;
            modifONOFF = 1;
            }
         }
      }
   }
return modifONOFF;
}

/*-------------------------------------------------------------------------
extractClique    : extrait les cliques  
---------------------------------------------------------------------------
retour          : une liste chainee correspondant a une clique 
---------------------------------------------------------------------------*/
chain_list *extractClique(tabCouple,count)
struct couple *tabCouple;
int count;
{
int cm,cmp;
int max;
chain_list *choix;
chain_list *clique;
int i;

			/* calcul du card. max dans le tableau tabCouple */
max = 0;
cm = 0;
for (i=0; i< count;i++)
   {
   if (max < tabCouple[i].count)
      {
      max = tabCouple[i].count;
      cm = i;
      }
   }
			/* il n'existe plus de clique */

if (max == 0) return NULL;

choix = tabCouple[cm].lst;
clique = addchain(NULL,cm);

while (choix != NULL)
   {				 /* calcul du card. max dans la liste choix */
   chain_list *lst = choix;
   
   max = 0;
   cm = 0;
   while (lst)
      {
      if (max < tabCouple[(int)lst->DATA].count)
         {
         max = tabCouple[(int)lst->DATA].count;
         cm = (int) lst->DATA;
         }
      lst = lst->NEXT;
      } 

   choix = (chain_list *) interLC(choix,tabCouple[cm].lst);
   clique = addchain(clique,cm);
   }
return clique;
}

/*-------------------------------------------------------------------------
calculClique    : extrait les couples ON - OFF a un niveau  
---------------------------------------------------------------------------
retour          : 1 si modif 
---------------------------------------------------------------------------*/
int calculClique(tabCouple,count)
int count;
struct couple *tabCouple;
{
int i,j;
int modifONOFF = 0;
chain_list *lst;

		/* valeur par defaut pour les onp et offp */

for (i=0; i< count;i++)
   {
   tabCouple[i].onp  = tabCouple[i].on;
   tabCouple[i].offp = tabCouple[i].off;
   tabCouple[i].lst = NULL; 
   tabCouple[i].count = 0; 
   }

printf("Algo de clique : nombre de noeuds = %d\n",count);

		/* creation du graphe */

for (i=0; i< count;i++)
   {
   for (j = i+1; j< count; j++)
      {
      if (suptautology(tabCouple[i].on,tabCouple[i].off,
                       tabCouple[j].on,tabCouple[j].off))
         {
         printf("*** Edge between %d and %d\n",i,j);

         tabCouple[i].lst = addchain(tabCouple[i].lst,j);
         tabCouple[i].count = tabCouple[i].count + 1; 
         tabCouple[j].lst = addchain(tabCouple[j].lst,i);
         tabCouple[j].count = tabCouple[j].count + 1; 
         }
      }
   }

		/* calcul de couverture pour les couples isoles 
                   de noeuds relies par une arete */

for (i=0; i< count;i++)
   {
   pNode pONp,pOFFp;

		/* une arete unique entre deux noeuds */

   if ((tabCouple[i].count == 1) &&
       (tabCouple[(int) (tabCouple[i].lst)->DATA].count == 1))
       {
       int j;

       j = (int) (tabCouple[i].lst)->DATA;

/*
       printf("*** Reso : %d - %d\n",i,j);
*/
/*
printf("couple[i].on \n");
displayBdd(tabCouple[i].on,1);
printf("couple[i].off \n");
displayBdd(tabCouple[i].off,1);

printf("couple[j].on \n");
displayBdd(tabCouple[j].on,1);
printf("couple[j].off \n");
displayBdd(tabCouple[j].off,1);
*/

       pONp = applyBinBdd(OR,tabCouple[i].on,tabCouple[j].on);
       pOFFp = applyBinBdd(OR,tabCouple[i].off,tabCouple[j].off);

/*
printf("U-on \n");
displayBdd(pONp,1);
printf("U-off \n");
displayBdd(pOFFp,1);
*/


		/* report de la couverture sur les couples de BDD concernes */ 

      tabCouple[i].onp = pONp;
      tabCouple[i].offp = pOFFp;
      
      tabCouple[j].onp = pONp;
      tabCouple[j].offp = pOFFp;

      freechain(tabCouple[i].lst);
      tabCouple[i].lst = NULL;
      tabCouple[i].count = 0;

      freechain(tabCouple[j].lst);
      tabCouple[j].lst = NULL;
      tabCouple[j].count = 0;

      modifONOFF = 1;
      }
   }

				/* calcul des cliques */

while (lst = (chain_list *) extractClique(tabCouple,count))
   {
   chain_list *lstp;
   pNode pONp,pOFFp;

   modifONOFF = 1;

   printf("clique = [");
   lstp = lst;
   while (lstp)
      {
      printf(" %d",lstp->DATA);
      lstp = lstp->NEXT;
      }
   printf(" ]\n");

		/* elimination des noeuds de la clique et de leurs aretes */

   lstp = lst;
   while (lstp)
      {
      freechain(tabCouple[(int)lstp->DATA].lst);
      tabCouple[(int)lstp->DATA].lst = NULL;
      tabCouple[(int)lstp->DATA].count = 0;
      lstp = lstp->NEXT;
      }

   for (i=0; i< count;i++)
      {
      if (tabCouple[i].lst != NULL)
         {
         lstp = tabCouple[i].lst;
         tabCouple[i].lst = (chain_list *) extractLC(lst,tabCouple[i].lst);
         tabCouple[i].count = countChain_list(tabCouple[i].lst);
         freechain(lstp);
         }
      }
   
		/* calcul de la couverture simple */
   lstp = lst;
   pONp = zero;
   pOFFp = zero;
   while (lstp)
      {
      pONp = applyBinBdd(OR,tabCouple[(int)lstp->DATA].on,pONp);
      pOFFp = applyBinBdd(OR,tabCouple[(int)lstp->DATA].off,pOFFp);
      lstp = lstp->NEXT;
      }   

		/* report de la couverture sur les couples de BDD concernes */ 
   lstp = lst;
   while (lstp)
      {
      tabCouple[(int)lstp->DATA].onp = pONp;
      tabCouple[(int)lstp->DATA].offp = pOFFp;
      lstp = lstp->NEXT;
      }   
   }

		/* simplifications locales directes */

if (SIMPLIFLOC)
   {
   for (i=0; i< count;i++)
      {
      pNode pResON,pResOFF;

      if (simplifONOFF(tabCouple[i].on,tabCouple[i].off,&pResON,&pResOFF))
         {

/*
printf("couple[i].on \n");
displayBdd(tabCouple[i].onp,1);
printf("couple[i].off \n");
displayBdd(tabCouple[i].offp,1);
printf("apres simplif :\n");
*/

         tabCouple[i].onp = pResON;
         tabCouple[i].offp = pResOFF;
         modifONOFF = 1;

/*
printf("couple[i].onp \n");
displayBdd(tabCouple[i].onp,1);
printf("couple[i].offp \n");
displayBdd(tabCouple[i].offp,1);
*/
			/* on verifie ... on ne sait jamais ! */

         if (!suptautology(tabCouple[i].on,tabCouple[i].off,pResON,pResOFF))
            {
            printf("Error simplif loc\n");
            exit(-1);
            }
         }
      }
   }

return(modifONOFF);
}

/*-------------------------------------------------------------------------
setONOFF        : positionne les couples ON - OFF a un niveau  
---------------------------------------------------------------------------
retour          : deux nouveaux pON et pOFF 
---------------------------------------------------------------------------*/
void setONOFF(pON,pOFF,ppONp,ppOFFp,tabCouple,count,level)
pNode pON,pOFF;
pNode *ppONp,*ppOFFp;
struct couple *tabCouple;
int count;
int level;
{
int i;


if ((pON->index == level && pOFF->index <= level) ||
    (pOFF->index == level && pON->index <= level) ||
    (pON->index < level && pOFF->index < level))
   {
   int trouve = 0;

   for (i=0;i<count;i++)
      {
      if (pON == tabCouple[i].on && pOFF == tabCouple[i].off)
         {
         trouve = 1;
         break;
         }
      }
    
   if (trouve == 1)
      {
      *ppONp = tabCouple[i].onp;
      *ppOFFp = tabCouple[i].offp;
      return;
      }
   else
      {
      printf("Error : cannot find the resulting onp - offp\n");
      exit(-1);
      }
   }
else
   {
   pNode pONpLow,pONpHigh,pOFFpLow,pOFFpHigh;

   if (pON->index == pOFF->index)
      {
      setONOFF(pON->high,pOFF->high,&pONpHigh,&pOFFpHigh,tabCouple,count,level);
      setONOFF(pON->low,pOFF->low,&pONpLow,&pOFFpLow,tabCouple,count,level);

      *ppONp = initVertexBdd(pON->index,pONpHigh,pONpLow);
      *ppOFFp = initVertexBdd(pON->index,pOFFpHigh,pOFFpLow);
      }
   else
      {
      if (pON->index > pOFF->index)
         {
         setONOFF(pON->high,pOFF,&pONpHigh,&pOFFpHigh,tabCouple,count,level);
         setONOFF(pON->low,pOFF,&pONpLow,&pOFFpLow,tabCouple,count,level);

         *ppONp = initVertexBdd(pON->index,pONpHigh,pONpLow);
         *ppOFFp = initVertexBdd(pON->index,pOFFpHigh,pOFFpLow);
         }
      else
         {
         setONOFF(pON,pOFF->high,&pONpHigh,&pOFFpHigh,tabCouple,count,level);
         setONOFF(pON,pOFF->low,&pONpLow,&pOFFpLow,tabCouple,count,level);

         *ppONp = initVertexBdd(pOFF->index,pONpHigh,pONpLow);
         *ppOFFp = initVertexBdd(pOFF->index,pOFFpHigh,pOFFpLow);
         }
      }
   }
}

/*-------------------------------------------------------------------------
extractONOFF    : extrait les couples ON - OFF a un niveau  
---------------------------------------------------------------------------
retour          : remplit un tableau et met a jour un compteur 
---------------------------------------------------------------------------*/
void extractONOFF(pON,pOFF,tabCouple,pCount,level)
pNode pON,pOFF;
struct couple *tabCouple;
int *pCount;
int level;
{
pNode pBddLoc;
int i;

		/* couple deja traite */

/*====	il faut creer un cache memoire specifique ...

pBddLoc = searchTabLoc(sysBdd.pMC,pON,pOFF,EXTRACT);
if ((int) pBddLoc == level) return;

========================================================*/


if ((pON->index == level && pOFF->index <= level) ||
    (pOFF->index == level && pON->index <= level) ||
    (pON->index < level && pOFF->index < level)) 
   {
   int trouve = 0;

   for (i=0;i<*pCount;i++)
      {
      if (pON == tabCouple[i].on && pOFF == tabCouple[i].off)
         {
         trouve = 1;
         break;
         }
      }
    
   if (trouve == 0)	/* nouveau couple */
      {
      printf("new : ON = %d  - OFF = %d\n",pON,pOFF); 

      tabCouple[*pCount].on =  pON;
      tabCouple[*pCount].off =  pOFF;
      *pCount = *pCount + 1;
      if (*pCount == TAILLEGRAPH)
         {
         printf("Error : Too many nodes for the cliques algorithm ...\n");
         exit(-1);
         }
      }
   }
else
   {
   if (pON->index == pOFF->index)
      {
      extractONOFF(pON->high,pOFF->high,tabCouple,pCount,level);
      extractONOFF(pON->low,pOFF->low,tabCouple,pCount,level);
      }
   else
      {
      if (pON->index > pOFF->index)
         {
         extractONOFF(pON->high,pOFF,tabCouple,pCount,level);
         extractONOFF(pON->low,pOFF,tabCouple,pCount,level);
         }
      else
         {
         extractONOFF(pON,pOFF->high,tabCouple,pCount,level);
         extractONOFF(pON,pOFF->low,tabCouple,pCount,level);
         }
      }
   }

/* ===========================================

addTabLoc(sysBdd.pMC,pON,pOFF,level,EXTRACT);

=========================================== */

}

/*-------------------------------------------------------------------------
cliqueBdd       : minimise un Bdd avec ses DC 
---------------------------------------------------------------------------
retour          : un BDD simplifie 
---------------------------------------------------------------------------*/
pNode cliqueBdd(pON,pDC)
pNode pON,pDC;
{
pNode pOFF;
int *tabSupport;
chain_list *support, *saveSupport;
struct couple *pCouple;
int i;

		/* on calcule le off-set */

pOFF = notBdd(applyBinBdd(OR,pON,pDC));

printf("pOFF = \n");
displayBdd(pOFF,1);
printf("zero = %d, one = %d\n",zero,one);

		/* support de pON */

support = (chain_list *) supportIndexBdd(pON);
tabSupport = (int *) mbkalloc ((pON->index + 2)*sizeof(int));
saveSupport = support;

for (i=0; i <= pON->index; i++)
   tabSupport[i]=0;

while (support)
   {
   tabSupport[(int) support->DATA] = 1;
   support = support->NEXT;
   } 
freechain(saveSupport);

		/* tableau des couples ON-OFF */

pCouple = (struct couple *) mbkalloc (TAILLEGRAPH*sizeof(struct couple));

		/* on descend 'Z' de l'index de la racine de pON  a 2 */

for (i = pON->index; i >= 2; i--)
   {
   pNode pONp,pOFFp;

   if (tabSupport[i] == 1)		/* la variable i existe dans pON */
      {
      int numCouples = 0;
      int modifONOFF = 0;

printf("niveau = %d\n",i);

      extractONOFF(pON,pOFF,pCouple,&numCouples,i);
      modifONOFF = calculClique(pCouple,numCouples);

		/* on reporte les modifications sur pOn et pOFF */

      if (modifONOFF)
         {
         setONOFF(pON,pOFF,&pONp,&pOFFp,pCouple,numCouples,i);
         pON = pONp;
         pOFF = pOFFp;
         }
      }
   }
printf("Final pOFFp \n");
displayBdd(pOFF,1);
mbkfree(pCouple);
return(pON);
}

/*-------------------------------------------------------------------------
cliqueMultiBdd  : minimise un multi-BDD avec ses DC 
---------------------------------------------------------------------------
retour          : un multi-BDD simplifie 
---------------------------------------------------------------------------*/
void cliqueMultiBdd(pON,pDC,numBdd)
pNode *pON, *pDC;
int numBdd;
{
pNode *pOFF;
int *tabSupport;
chain_list *support, *saveSupport;
int i,j;
int indexMax;
struct couple *pCouple;

pOFF = (pNode *) mbkalloc (numBdd*sizeof(pNode));

		/* on calcule le off-set du multi-Bdd 
                   et l'index maximun */

indexMax = 0;
for (i=0; i<numBdd; i++)
   {
   pOFF[i] = notBdd(applyBinBdd(OR,pON[i],pDC[i]));
   if (indexMax < pON[i]->index)
      indexMax = pON[i]->index;
   }

		/* tableau des couples ON-OFF */

pCouple = (struct couple *) mbkalloc (TAILLEGRAPH*sizeof(struct couple));

		/* on descend 'Z' de l'index indexMax  a 2 */

printf("zero = %d, one = %d\n",zero,one);

for (i=indexMax; i >= 2; i--)
   {
   pNode pONp,pOFFp;
   int numCouples;
   int modifONOFF;

   numCouples = 0;
   modifONOFF = 0;

printf("niveau = %d\n",i);

		/* extraction de tous les couples distincts ON,OFF */

   for (j=0; j < numBdd; j++)
      {
      extractONOFF(pON[j],pOFF[j],pCouple,&numCouples,i);
      }

   modifONOFF = calculClique(pCouple,numCouples);

		/* on reporte les modifications sur pON et pOFF */

   if (modifONOFF)
      {
/*
      for (j=0; j < numCouples; j++)
         {
         printf("pCouple[%d] = \n",j);
         printf("onp ");
         displayBdd(pCouple[j].onp,1);
         printf("offp ");
         displayBdd(pCouple[j].offp,1);
         }
*/
      for (j=0; j < numBdd; j++)
         {
         setONOFF(pON[j],pOFF[j],&pONp,&pOFFp,pCouple,numCouples,i);

/*
         printf("avant simplif pON[%d] = \n",j);
         displayBdd(pON[j],1);
*/

         pON[j] = pONp;

/*
         printf("apres simplif pON[%d] = \n",j);
         displayBdd(pON[j],1);
*/

         pOFF[j] = pOFFp;
         }
      }
   }

mbkfree(pCouple);
mbkfree(pOFF);
}
