/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : synthetiseur FPGA                                           */
/*    Fichier : bool2xil.c                                                  */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : DICTUS N.                             le : 15/07/1994     */
/*                                                                          */
/*    Modifie par : Eudes Prado Lopes                   le : 08/11/1994     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
#include MUT_H
#include MLO_H
#include LOG_H
#include BEH_H
#include ICN_H
#include IAC_H
#include ICU_H
#include "bool2xil.h"
#include "bool2act.h"
#include "./mapp3000.h"
#include "../compil/lax_param.h"
#include "./fp_type.h"

/*------------------------------------------------------------------------------
oibIcon          : genere un icon de buffer pour XILINX (oib)
--------------------------------------------------------------------------------
parametres       : oitag = CONFDIRECT ou CONFNOT (buffer inverseur)
--------------------------------------------------------------------------------
return           : un icon
------------------------------------------------------------------------------*/
struct IconGate *oibIcon(name, nameIn, nameOut, oitag, store_it)
char *name, *nameIn, *nameOut;
int oitag, store_it;        /* 1 if the gate's icon have to be stored, 0 else */{
struct IconGate *gate_icon;

gate_icon = addicongate( name,
                         360, 0,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
                         500, -200,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL,
                         510, 200,
                         SAX_CENTER, SAX_CENTER, SAX_HORIZONTAL);
        /*sortie */
addiconcon(gate_icon, nameOut,
           1000, 0, SAX_EAST, OUT,
           SAX_LEFT, SAX_DOWN, SAX_HORIZONTAL, 1000, -50);

        /* entree */
addiconcon(gate_icon, nameIn,
           0, 0, SAX_WEST, IN,
           SAX_RIGHT, SAX_DOWN, SAX_HORIZONTAL, 0, -50);
 
addiconline( gate_icon, 300, 200, 200, 200);
addiconline( gate_icon, 300, -200, 200, -200);
addiconline( gate_icon, 300, 200, 700, 0);
addiconline( gate_icon, 300, -200, 700, 0);
addiconline( gate_icon, 200, 0, 0, 0);
addiconline( gate_icon, 200, -200, 600, 0);
addiconline( gate_icon, 200, 200, 600, 0);
addiconline( gate_icon, 200, -200, 200, 200);
 
if (oitag == CONFNOT)
   {
   addiconline( gate_icon, 800, 0, 1000, 0);
   addiconcircle( gate_icon, 700, -50, 100, 100);
   }
else
   addiconline( gate_icon, 700, 0, 1000, 0);
    
update_icon( gate_icon);
if (FP_TRACE)
  printf("WORK_LIB is %s \n",WORK_LIB);
if (store_it)
   saveicon( gate_icon);
 
return gate_icon;
}

/*------------------------------------------------------------------------------
createModelXil	 : genere une lofig de type xilinx a partir de son modele VHDL.
--------------------------------------------------------------------------------
parametres 	 : befig.
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *createModelXil(befig, block, oitag)
befig_list *befig;
int block, oitag;
{
lofig_list *liste;
int index = 1;
lofig_list *modelFig;
char *save_WORK_LIB;
bepor_list *port, *auxPort = (bepor_list *)NULL;
berin_list *in, *auxIn = (berin_list *)NULL;

	/* recherche du model en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == befig->NAME)
      return liste;

	/* modele n'existe pas : on le cree */
modelFig = addlofig(befig->NAME);

	/* reverse pas physique */
for(port = befig->BEPOR; port; port = port->NEXT)
   auxPort = beh_addbepor(auxPort, port->NAME,
                          port->DIRECTION, port->TYPE);

	/* ajout des entrees / sorties */
for(port = auxPort; port; port = port->NEXT)
   {
	/* creation du connecteur et du signal de l'entree/sortie */
   if ((port->DIRECTION == IN) || (port->DIRECTION == OUT))
      addlocon(modelFig, port->NAME,
               addlosig(modelFig, index++, 
                        addchain((chain_list *)NULL, port->NAME),
                        EXTERNAL, (float) 0),
               port->DIRECTION);
   else
      printf("Warning : Unknown direction port : %s\n", port->NAME);
   }
if (FP_TRACE)
  printf("WORK_LIB ==%s",WORK_LIB);
save_WORK_LIB = WORK_LIB;
WORK_LIB = FP_XILINX_LIB;
if (FP_TRACE)
  printf("WORK_LIB ==%s",WORK_LIB);

	/* Generation de l'icon corresponsant sur disque */
if (block == CONFIOB)
   delicongate(ICONGATE_MODELNAME(oibIcon(modelFig->NAME, befig->BERIN->NAME,
                                          befig->BEOUT->NAME, oitag, 1)));

savelofig(modelFig);
WORK_LIB = save_WORK_LIB;
return modelFig;
}

/*------------------------------------------------------------------------------
bool2Xil	 : genere une lofig a partir d'une befig et 1 liste de CLB.
		   (plusieurs noeuds du reseau booleen peuvent correspondre a
                    un CLB)
--------------------------------------------------------------------------------
parametres 	 : une befig, une liste des CLB.
--------------------------------------------------------------------------------
return 		 : une lofig.
------------------------------------------------------------------------------*/
lofig_list *bool2Xil(beh, listeCLB)
befig_list *beh;
pCLB listeCLB;
{
lofig_list *fig = (lofig_list *)NULL;
bepor_list *port;
pTH sigTH;
int indexSig = 1;
int cptIOB = 1;
lofig_list *liste;
pCLB clb;

printf("bool2Xil : creating %s\n", beh->NAME);

	/* recherche de la lofig en memoire */
for(liste = HEAD_LOFIG; liste; liste = liste->NEXT)
   if (liste->NAME == beh->NAME)
      return liste;

	/* table hash : nom connecteur externe --> pointeur du signal */
sigTH = createTH(200);

	/* lofig n'existe pas : on la cree */
fig = addlofig(beh->NAME);

for(clb = listeCLB; clb; clb = clb->NEXT)
   {
   lofig_list *model = createModelXil(clb->BEH, clb->BLOCK, clb->OITAG);
   chain_list *sigChain = (chain_list *)NULL;
   locon_list *interface;
   char *insName;

/*
   printf("CLB '%s'\n", clb->NAME);
*/

   if (clb->BLOCK == CONFCLB)
      {
      insName = clb->NAME;
      for(interface = model->LOCON; interface; interface = interface->NEXT)
         {
         short encore = 1;
         ptype_list *assoc;
         int val; 
         losig_list *sig;

/*
printf("sur interface %s\n", interface->NAME);
*/

         for(assoc = clb->ASSOCF; assoc && encore; assoc = assoc->NEXT)
            {
            if (!strcmp((char *)assoc->TYPE, interface->NAME))
               {
               if ((val = searchTH(sigTH, (char *)assoc->DATA)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  sig = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL, assoc->DATA),
                                 INTERNAL, (float) 0);
         
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, assoc->DATA, (int)sig);
                  }
               encore = 0;
               }
            }
         for(assoc = clb->ASSOCG; assoc && encore; assoc = assoc->NEXT)
            {
            if (!strcmp((char *)assoc->TYPE, interface->NAME))
               {
               if ((val = searchTH(sigTH, (char *)assoc->DATA)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  sig = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL, assoc->DATA),
                                 INTERNAL, (float) 0);
         
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, assoc->DATA, (int)sig);
                  }
               encore = 0;
               }
            }
         if (encore)
            {
            char *name;

/*************** TRAITEMENT de l'entree DI ***************/


                 /* on a une entree di */

         if (!strcmp( interface->NAME, "di"))
               {
               printf("va traiter entree di\n");
               if (clb->DXTAG == CONFDI)
                  name = clb->SIGNALDI;
               if (clb->DYTAG == CONFDI)
                  name = clb->SIGNALDI;
               printf("name is %s \n", name);

               if ((val = searchTH(sigTH, name)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  sig = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL, name),
                                 INTERNAL, (float) 0);
         
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, name, (int)sig);
                  }
               encore = 0;
               }
            else


/*************** FIN DU TRAITEMENT de l'entree DI ***************/


		   /* on a une sortie */
            if (!strcmp(interface->NAME, "x"))
               {
               if (clb->XTAG == CONFF)
                  name = clb->SIGNALF;
               else
                  {
                  name = clb->SIGNALQX;
                  insName = clb->SIGNALQX;
                  }
               if ((val = searchTH(sigTH, name)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  sig = addlosig(fig, indexSig++,
                                 addchain((chain_list *)NULL,
                                          name),
/*
                                          genNAME("%sf", clb->NAME)),
*/
                                 INTERNAL, (float) 0);
      
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, name, (int)sig);
                  }
               }
            else
               if (!strcmp(interface->NAME, "y"))
                  {
                  if (clb->YTAG == CONFG)
                     name = clb->SIGNALG;
                  else
                     {
                     name = clb->SIGNALQY;
                     insName = clb->SIGNALQY;
                     }
                  if ((val = searchTH(sigTH, name)) != EMPTYTH)
                     sigChain = addchain(sigChain, (void *)val);
                  else
                     {
                     sig = addlosig(fig, indexSig++,
                                    addchain((chain_list *)NULL,
                                             name),
/*
                                             genNAME("%sg", clb->NAME)),
*/
                                    INTERNAL, (float) 0);
         
                     sigChain = addchain(sigChain, (void *)sig);
                     addTH(sigTH, name, (int)sig);
                     }
                  }
               else
                  if (!strcmp(interface->NAME, "ck"))
                     if ((val = searchTH(sigTH, clb->SIGNALCK)) != EMPTYTH)
                        sigChain = addchain(sigChain, (void *)val);
                     else
                        {
                        sig = addlosig(fig, indexSig++,
                                       addchain((chain_list *)NULL,
                                                genNAME("%sck", clb->NAME)),
                                       INTERNAL, (float) 0);
         
                        sigChain = addchain(sigChain, (void *)sig);
                        addTH(sigTH, clb->SIGNALCK, (int)sig);
                        }
                  else
                     printf("ERROR : unknown bepor '%s' for %s\n",
                            interface->NAME, clb->NAME);
            }
         }
      if (0)
      {
      locon_list *por;
      chain_list *sign;

      for(por = model->LOCON, sign = sigChain; por && sign;
          por = por->NEXT, sign = sign->NEXT)
         printf("  -- por '%s' - sig %ld-\n",
                por->NAME, ((losig_list *)(sign->DATA))->INDEX);
      }
      }
   else
      {		/* clb->BLOCK = CONFIOB */
      insName = (char *)genNAME("%s%d", clb->NAME, cptIOB++);
      for(interface = model->LOCON; interface; interface = interface->NEXT)
         {
         int val;

         if (clb->PADTAG == CONFI)
            {
            if (interface->DIRECTION == IN)
               {
		/* creation du signal pour le connecteur (pad cote externe) */
               losig_list *sig = addlosig(fig, indexSig++,
                                          addchain((chain_list *)NULL,
                                                   (void *)clb->SIGNALPAD), 
                                          EXTERNAL, (float) 0);
               sigChain = addchain(sigChain, sig);
               addlocon(fig, clb->SIGNALPAD, sig, IN);
               }
            else
               {
               if ((val = searchTH(sigTH, clb->SIGNALPAD)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  losig_list *sig = addlosig(fig, indexSig++,
                                             addchain((chain_list *)NULL,
                                                      genNAME("%sb",
                                                              clb->SIGNALPAD)),
                                             INTERNAL, (float) 0);
         
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, clb->SIGNALPAD, (int)sig);
                  }
               }
            }
         else
            {	/* PADTAG = CONFO */
            if (interface->DIRECTION == OUT)
               {
		/* creation du signal pour le connecteur (pad cote externe) */
               losig_list *sig = addlosig(fig, indexSig++,
                                          addchain((chain_list *)NULL,
                                                   clb->SIGNALPAD), 
                                          EXTERNAL, (float) 0);
               sigChain = addchain(sigChain, sig);
               addlocon(fig, clb->SIGNALPAD, sig, OUT);
               insName = clb->SIGNALPAD;
               }
            else
               {
               char *sigName;

               if (clb->OITAG == CONFDIRECT)
                  sigName = VALUE_ATOM(clb->EQUATEO);
               else
                  sigName = VALUE_ATOM(CADR(clb->EQUATEO));

               if ((val = searchTH(sigTH, sigName)) != EMPTYTH)
                  sigChain = addchain(sigChain, (void *)val);
               else
                  {
                  losig_list *sig = addlosig(fig, indexSig++,
                                             addchain((chain_list *)NULL,
                                                      (void *)sigName),
                                             INTERNAL, (float) 0);
         
                  sigChain = addchain(sigChain, (void *)sig);
                  addTH(sigTH, sigName, (int)sig);
                  }
               }
            }
         }
      }	 /* else CONFIOB */
   fig->LOINS = addloins(fig, insName, model, reverse(sigChain));
   }


lofigchain(fig);
if (FP_TRACE)
  displayLofigXil(fig);


destroyTH(sigTH);
return fig;
}

/*---------------------------------------------------------------------------
displayLofigXil	: affiche une lofig XILINX (INSTANCE - CONNECTEUR - SIGNAL - 
                                            CONNECTEUR - INSTANCE)
-----------------------------------------------------------------------------
retour		: void
---------------------------------------------------------------------------*/
void displayLofigXil(lofig)
lofig_list *lofig;
{
loins_list *inst;

	/* parcours des instances de la figure */

printf(" displayLofigXil (in bool2xil.c) ==> parcours des instances de la figure\n");
for(inst = lofig->LOINS; inst; inst = inst->NEXT)
   {
   int nbOut = 0;
   locon_list *con;
   ptype_list *fanout = NULL;

   printf("  INSTANCE : %s [%s]\n", inst->INSNAME, inst->FIGNAME);

	/* parcours des connecteurs de l'instance */
   for(con = inst->LOCON; con; con = con->NEXT)
      {
      ptype_list *liste;
      chain_list *l;

      if (!isvdd(con->NAME) && !isvss(con->NAME))
         {
         printf("   CONNECTOR : %s (%c) --- SIGNAL %s --- ",
                con->NAME,con->DIRECTION, (char *)(con->SIG->NAMECHAIN->DATA));

		/* liste des liaisons signaux --> connecteurs */
         liste = getptype(con->SIG->USER,(long)LOFIGCHAIN);
         for(l = (chain_list *)liste->DATA; l; l = l->NEXT)
            {
            if (((locon_list *)(l->DATA))->ROOT != (void *)inst)
               {
               locon_list *conAux = (locon_list *)l->DATA;

               if (conAux->TYPE == EXTERNAL)
                  {
			/* connecteurs externes : E/S primaires */
                  printf(" Extern %s", conAux->NAME);
                  }
               else
                  {
   			/* connecteur interne */
                  loins_list *cellOut = (loins_list *)conAux->ROOT;

		    /* DETERMINE LE FANOUT DE CHAQUE CELLULE */

                  if (con->DIRECTION == OUT)
                     {
                     nbOut++;
                     printf("  %s[%s],", cellOut->INSNAME, cellOut->FIGNAME);
                     }
                  else
                     if (conAux->DIRECTION == OUT)
                        printf("  %s[%s],", cellOut->INSNAME, cellOut->FIGNAME);
                  }
               }
            }
         printf("\n");
         }
      }		/* fin parcours connecteurs */

   if (nbOut > 0)
      {
      printf("     %s drives %d gate%s\n",
             inst->INSNAME, nbOut, (nbOut > 1)? "s":"");
      }
   printf("\n");
   }
}

/*---------------------------------------------------------------------------
infiX3000	: ecrit un ABL a la sauce XILINX dans le fichier fich
-----------------------------------------------------------------------------
parametre	: expr a ecrire en infixe (AND = *, OR = +, NOT = ~, XOR = @)
-----------------------------------------------------------------------------
retour		: void.
---------------------------------------------------------------------------*/
void infiX3000(fich, expr)
FILE *fich;
chain_list *expr;
{
if (ATOM(expr))
   fprintf(fich, "%s", VALUE_ATOM(expr));
else
   if (OPER(expr) == NOT)
      {
      fprintf(fich, "~");
      infiX3000(fich, CADR(expr));
      }
   else
      {
      int oper = OPER(expr);

      fprintf(fich, "(");
      if ((oper != AND) && (oper != OR) && (oper != XOR))
         {
         printf("Bad X3000 expression :");
         displayExpr(expr);
         }
      while (expr = CDR(expr))
         {
         infiX3000(fich, CAR(expr));
         if (CDR(expr))
            switch (oper)
               {
               case AND : fprintf(fich, "*"); break;
               case OR  : fprintf(fich, "+"); break;
               case XOR : fprintf(fich, "@"); break;
               }
         }
      fprintf(fich, ")");
      }
}


/*---------------------------------------------------------------------------
changeName      Convert internal IO name(vectors) to a .xnf compatible name       -----------------------------------------------------------------------------
parameters	: a signal name possibly with a blank 
-----------------------------------------------------------------------------
return		: a signal name with a backslash 
---------------------------------------------------------------------------*/
char    *changeName( IO_name )
char    *IO_name;
{
    char        *p;
    char        *Found;



    if (FP_TRACE)
       printf("init %s-\n", IO_name);
 
    Found = (char *)strchr( IO_name, ' ' );

    if (Found != NULL )
       {
    printf("init %s, Found =%s-\n", IO_name, Found);
       Found[0] = '_';
       }
}   




/*---------------------------------------------------------------------------
driverX3000	: driver XILINX (.xnf)
-----------------------------------------------------------------------------
parametre	: liste de CLB, nom du fichier, MODE
-----------------------------------------------------------------------------
retour		: void. un fichier .xnf est genere dans MBK_WORK_LIB
---------------------------------------------------------------------------*/
void driverX3000(lstCLB, name, MODE)
pCLB lstCLB;
char *name;
int MODE;
{
FILE *fich;
pCLB liste;
char *fileXNF = (char *)mbkalloc(strlen(WORK_LIB) + strlen(name) +
                                 strlen(OUT_LO) + 4);

sprintf(fileXNF, "%s/%s.xnf", WORK_LIB, name, OUT_LO);

if (fich = fopen(fileXNF, "w"))
   {
   bereg_list *reg;
   bepor_list *con;
   pTH inTH = createTH(50);
   int cptCLB = 0;

   printf("Generating file '%s' ...\n", fileXNF);

   fprintf(fich, "LCANET, 4\n");   /* numero de version */
   fprintf(fich, "USER, LAX, 1.4, Date: , Option: %d\n", MODE);
   fprintf(fich, "PROG, LAX, 1.4, \"Lib = 3000\"\n");
   fprintf(fich, "PART, 3030pc84-125\n");
   fprintf(fich, "PWR, 0, GND\n");
   fprintf(fich, "PWR, 1, VCC\n");

	/* parcours des IOB */
   for(liste = lstCLB; liste; liste = liste->NEXT)
     /* if it is an IOB block */
      if (liste->BLOCK == CONFIOB) 
         {
         char *ext = mbkalloc(80);

         fprintf(fich, "SYM, %s, ", liste->NAME);
         /* if it is an input  IOB block */
         if (liste->PADTAG == CONFI)
            fprintf(fich, "IBUF\n");
         else
           fprintf(fich, "OBUF\n");

	/* reading connectors of related befig */

         for(con = liste->BEH->BEPOR; con; con = con->NEXT)

           /* input-> PADTAG = CONFI = 16 , output-> PADTAG =CONFO =17*/

            if (liste->PADTAG == CONFI) 
               {
               if (con->DIRECTION == IN)
                  {
                  fprintf(fich, "PIN, I, I, %s,\n", changeName(liste->SIGNALPAD));
                  sprintf(ext, "%s, I,", changeName(liste->SIGNALPAD));
                  addTH(inTH, liste->SIGNALPAD,
                        (int)genNAME("_in_%s", liste->SIGNALPAD));
                  }
               else
		{
		if (FP_TRACE) 
                  printf(" 1 o sinal na saida do pad e' %s\n", genNAME("_in_%s", changeName(liste->SIGNALPAD)));
                  fprintf(fich, "PIN, O, O, %s,\n", genNAME("_in_%s", changeName(liste->SIGNALPAD)));
			}
               }
            else
               {	/* PADTAG = CONFO */
               if (con->DIRECTION == OUT)
                  {
		  changeName(liste->SIGNALPAD);
		  if (FP_TRACE) 
                    printf(" 2 o sinal na saida do pad e' %s\n", liste->SIGNALPAD);
                  fprintf(fich, "PIN, O, O, %s,\n", liste->SIGNALPAD);
                  sprintf(ext, "%s, O,", liste->SIGNALPAD);
                  }
               else
                  {
                  char *sigName;

                  if (liste->OITAG == CONFDIRECT)
                     sigName = VALUE_ATOM(liste->EQUATEO);
                  else
                     sigName = VALUE_ATOM(CADR(liste->EQUATEO));

                  fprintf(fich, "PIN, I, I, %s,\n", changeName(sigName));
                  }
               }
         fprintf(fich, "END\n");
         fprintf(fich, "EXT, %s\n", ext);
         }

	/* parcours des CLB */

/* SYM */

   for(liste = lstCLB; liste; liste = liste->NEXT)
      {
      if (liste->BLOCK == CONFCLB)
         {
         ptype_list *assoc;

         cptCLB++;

         fprintf(fich, "SYM, %s, CLB\n", liste->NAME);
            /* Ajouter eventuellement BLKNM=liste->NAME */

/* CFG, BASE */

         fprintf(fich, "CFG, BASE ");
         switch (liste->BASE)
            {
            case BASEF : fprintf(fich, "F\n");
                         break;
            case BASEFG : fprintf(fich, "FG\n");
                         break;
            case BASEFGM : fprintf(fich, "FGM\n");
                         break;
            default : printf("Bad BASE for %s\n", liste->NAME);
            }

/* CFG, CONFIG */

         fprintf(fich, "CFG, CONFIG X:");
         if (liste->XTAG != NIL)
            fprintf(fich, "%s", (liste->XTAG == CONFF) ? "F" : "QX");
         fprintf(fich, " Y:");
         if (liste->YTAG != NIL)
            fprintf(fich, "%s", (liste->YTAG == CONFG) ? "G" : "QY");
         fprintf(fich, " RSTDIR:");
         if (liste->RSTDIRTAG == CONFRD)
            fprintf(fich, "RD");
         fprintf(fich, " ENCLK:");
         if (liste->ENCLKTAG == CONFEC)
            fprintf(fich, "EC");
         fprintf(fich, " DX:");
         if (liste->DXTAG != NIL)
            switch (liste->DXTAG)
               {
               case CONFDI : fprintf(fich, "DI");
                             break;
               case CONFF  : fprintf(fich, "F");
                             break;
               case CONFG  : fprintf(fich, "G");
                             break;
               }
         fprintf(fich, " DY:");
         if (liste->DYTAG != NIL)
            switch (liste->DYTAG)
               {
               case CONFDI : fprintf(fich, "DI");
                             break;
               case CONFF  : fprintf(fich, "F");
                             break;
               case CONFG  : fprintf(fich, "G");
                             break;
               }
         fprintf(fich, " CLK:");
         if (liste->CLKTAG != NIL)
            fprintf(fich, "%s", (liste->CLKTAG == CONFNOT) ? "NOT" : "K");
         if (liste->XTAG == CONFF)
            {
            ptype_list *supportF = liste->ASSOCF;

            fprintf(fich, " F:");
            while (supportF)
               {
               if (supportF->NEXT)
                  fprintf(fich, "%s:", (char *)supportF->TYPE);
               else
                  fprintf(fich, "%s",  (char *)supportF->TYPE);
               supportF = supportF->NEXT;
               }
            }
         if (liste->YTAG == CONFG)
            {
            ptype_list *supportG = liste->ASSOCG;

            fprintf(fich, " G:");
            while (supportG)
               {
               if (supportG->NEXT)
                  fprintf(fich, "%s:", (char *)supportG->TYPE);
               else
                  fprintf(fich, "%s",  (char *)supportG->TYPE);
               supportG = supportG->NEXT;
               }
            }
         fprintf(fich, "\n");


/* CFG, EQUATE */

         if (liste->XTAG == CONFF)
            {
            beout_list *out;

            fprintf(fich, "CFG, EQUATE  F=");
            for(out = liste->BEH->BEOUT; out; out = out->NEXT)
               {
               if (!strcmp(out->NAME, "x"))
                  infiX3000(fich, out->ABL);
               }
            fprintf(fich, "\n");
            }
         if (liste->YTAG == CONFG)
            {
            beout_list *out;

            fprintf(fich, "CFG, EQUATE G=");
            for(out = liste->BEH->BEOUT; out; out = out->NEXT)
               if (!strcmp(out->NAME, "y"))
                  infiX3000(fich, out->ABL);
            fprintf(fich, "\n");
            }
         if (liste->XTAG == CONFQX)
            {
            bereg_list *reg;

            fprintf(fich, "CFG, EQUATE  F=");
            for(reg = liste->BEH->BEREG; reg; reg = reg->NEXT)
               {
               if (!strcmp(reg->NAME, "qx"))
                  infiX3000(fich, reg->BIABL->VALABL);
               }
            fprintf(fich, "\n");
            }
         if (liste->YTAG == CONFQY)
            {
            bereg_list *reg;

            fprintf(fich, "CFG, EQUATE G=");
            for(reg = liste->BEH->BEREG; reg; reg = reg->NEXT)
               if (!strcmp(reg->NAME, "qy"))
                  infiX3000(fich, reg->BIABL->VALABL);
            fprintf(fich, "\n");
            }

/* PIN,  */

/*****************************EPL*******************************************************/



        for(reg = liste->BEH->BEREG; reg; reg = reg->NEXT)
            {
            short encore = 1;
   	       	    if (FP_TRACE)
               printf("Entrou na EPL\n");
            for(assoc = liste->ASSOCF; assoc && encore; assoc = assoc->NEXT)
               if (!strcmp(reg->NAME, (char *)assoc->TYPE))
                  {
                  int val;

                  if ((val = searchTH(inTH, namealloc((char *)assoc->DATA))) != EMPTYTH)
                     {fprintf(fich, "PIN, K, I, _in_%s,\n",
                                    liste->SIGNALCK);
                         	    if (FP_TRACE)
                if (FP_TRACE)
               printf( "PIN, K, I, %s,\n",
                                    liste->SIGNALCK);}
                  else
                     {fprintf(fich, "PIN, K, I, _in_%s,\n",
                                    liste->SIGNALCK);
                        	    if (FP_TRACE)
                if (FP_TRACE)
               printf( "PIN, K, I, %s,\n",
                                    liste->SIGNALCK);}
                  encore = 0;
                  }
            for(assoc = liste->ASSOCG; assoc && encore; assoc = assoc->NEXT)
               if (!strcmp(reg->NAME, (char *)assoc->TYPE))
                  {
                  int val;

                  if ((val = searchTH(inTH, (char *)assoc->DATA)) != EMPTYTH)
                     {fprintf(fich, "PIN, K, I, _in_%s,\n",
                                   liste->SIGNALCK);
                      if (FP_TRACE)
               printf( "PIN, K, I, %s,\n",
                                    liste->SIGNALCK);}
                  else
                     {fprintf(fich, "PIN, K, I, _in_%s,\n",
                                    liste->SIGNALCK);
                      if (FP_TRACE)
               printf(fich, "PIN, K, I, _in_%s,\n",
                                    liste->SIGNALCK);}
                  encore = 0;
                  }
            }

   	    if (FP_TRACE)
               printf("Saiu da EPL\n");

/*****************************EPL*******************************************************/

         for(con = liste->BEH->BEPOR; con; con = con->NEXT)
            {
            short encore = 1;
   
            for(assoc = liste->ASSOCF; assoc && encore; assoc = assoc->NEXT)
               if (!strcmp(con->NAME, (char *)assoc->TYPE))
                  {
                  int val;

                  if ((val = searchTH(inTH, namealloc((char *)assoc->DATA))) != EMPTYTH)
                     fprintf(fich, "PIN, %s, I, %s,\n",
                                   (char *)assoc->TYPE, changeName((char *)val));
                  else
                     fprintf(fich, "PIN, %s, I, %s,\n",
                                   (char *)assoc->TYPE, changeName((char *)assoc->DATA));
                  encore = 0;
                  }
            for(assoc = liste->ASSOCG; assoc && encore; assoc = assoc->NEXT)
               if (!strcmp(con->NAME, (char *)assoc->TYPE))
                  {
                  int val;

                  if ((val = searchTH(inTH, (char *)assoc->DATA)) != EMPTYTH)
                     fprintf(fich, "PIN, %s, I, %s,\n",
                                   (char *)assoc->TYPE, changeName((char *)val));
                  else
                     fprintf(fich, "PIN, %s, I, %s,\n",
                                   (char *)assoc->TYPE, changeName((char *)assoc->DATA));
                  encore = 0;
                  }
            if (encore)
               {
               if (!strcmp(con->NAME, "x"))
                  fprintf(fich, "PIN, X, O, %s,\n", changeName(liste->SIGNALQX));
               else
                  if (!strcmp(con->NAME, "y"))
                     fprintf(fich, "PIN, Y, O, %s,\n", changeName(liste->SIGNALQY));
               }
            }
         fprintf(fich, "END\n");
         }
      }

/* USER */

   fprintf(fich, "USER, LAX, Nb. CLBs = %d\n", cptCLB);
   fprintf(fich, "EOF\n");
   destroyTH(inTH);
   fclose(fich);
   printf("file generated '%s' (%d CLBs)\n", fileXNF, cptCLB);
   }
else
   printf("Unable to open file '%s'\n", fileXNF);
}
