/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :synthetiseur de FPGA					    */
/*    Fichier :mapp3000                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :Burgun L.                              le : 03/06/1994     */
/*                                                                          */
/*    Modifie par :                                     le : 20/07/1994     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
#include MUT_H
#include LOG_H
#include BEH_H
#include <math.h>
#include "../mapping/types.h"
#include "../compil/lax_param.h"
#include "fp_type.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "act1.h"
#include "fpga.h"
#include "mapp3000.h"

/*-------------------------------------------------------------------------
substAssocExpr     : substitution generalisee dans une expression 
---------------------------------------------------------------------------
parameters : 1 expression et une liste associative ((a x)(b y)...)
---------------------------------------------------------------------------*/
chain_list *substAssocExpr(expr,assoc)
chain_list *expr;
ptype_list *assoc;
{
ptype_list *tete;

if (ATOM(expr))
   {
   tete = assoc;
   while (tete)
      {
      if (!strcmp(tete->DATA,VALUE_ATOM(expr)))
         {
         if (tete->TYPE != NULL)
            return(createAtom((char *)tete->TYPE));
         else
            {
            printf("substAssocExpr : Error - NULL substitution for '%s'\n",tete->DATA);
            exit(-1);
            }
         }
      tete = tete->NEXT;
      }
		/* pas de substitution */

   return(copyExpr(expr));
   }
else
   {
   chain_list *res;
   res = createExpr(OPER(expr));
   while ((expr = CDR(expr)))
      addQExpr(res,substAssocExpr(CAR(expr),assoc));
   return(res);
   }  
}

/*-------------------------------------------------------------------------
initCLB     : affiche une liste de CLB 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
pCLB initCLB(block)
int block;
{
pCLB new;

new = (pCLB) mbkalloc(sizeof(struct CLB));

if (block == CONFCLB)
   new->NAME = gensym("clb",coCLB++);
else   
   new->NAME = gensym("iob",coIOB++);
new->BLOCK = block;
new->BASE = NIL;
new->XTAG = NIL;
new->YTAG = NIL;
new->DXTAG = NIL;
new->DYTAG = NIL;

new->CLKTAG = NIL;
new->RSTDIRTAG = NIL;
new->ENCLKTAG = NIL;

new->EQUATEF = NULL;
new->EQUATEG = NULL;

new->ASSOCF = NULL;
new->ASSOCG = NULL;

new->SIGNALF = NULL;
new->SIGNALG = NULL;
new->SIGNALQX = NULL;
new->SIGNALQY = NULL;

new->SIGNALCK = NULL;

new->BEH = NULL;

new->NEXT = NULL;
}

/*-------------------------------------------------------------------------
displayCLB     : affiche une liste de CLB 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void displayCLB(lstCLB)
pCLB lstCLB;
{
pCLB tete;
ptype_list *ptl;

tete = lstCLB;
printf("===== displayCLB (in  mapp3000.c) ==> displays CLB list \n ");
while (tete)
   {
   printf("==================="); 
   if (tete->BLOCK == CONFCLB)
      printf("CLB ================= \n ");
   else
      printf("IOB ================= \n ");
   printf("NAME = %s\n",tete->NAME); 

   if (tete->BLOCK == CONFCLB)
      {
      printf("BASE = ");
      if (tete->BASE == BASEF) printf("F\n");
      if (tete->BASE == BASEFG) printf("FG\n");
      if (tete->BASE == BASEFGM) printf("FGM\n");
   
      if (tete->XTAG == CONFQX)
         printf("X = QX\n");

      if (tete->XTAG == CONFF)
         printf("X = F\n");

      if (tete->YTAG == CONFQY)
         printf("Y = QY\n");

      if (tete->YTAG == CONFG)
         printf("Y = G\n");

      if (tete->DXTAG == CONFDI)
         printf("DX = DI\n");
      if (tete->DXTAG == CONFF)
         printf("DX = F\n");
      if (tete->DXTAG == CONFG)
         printf("DX = G\n");

      if (tete->DYTAG == CONFDI)
         printf("DY = DI\n");
      if (tete->DYTAG == CONFF)
         printf("DY = F\n");
      if (tete->DYTAG == CONFG)
         printf("DY = G\n");

      if (tete->XTAG == CONFQX || tete->YTAG == CONFQY)
         {
         printf("FF configuration : ");
         if (tete->CLKTAG == CONFDIRECT)
            printf("CLK ");
         else 
            printf("not CLK ");
         if (tete->RSTDIRTAG == CONFRD)
            printf("RD selected ");
         if (tete->ENCLKTAG == CONFEC)
            printf("EC selected ");
         printf("\n");
         }
   
      printf("SUPPORT F :");
      ptl = tete->ASSOCF;
      while (ptl)
         {
         printf("%s-%s ",ptl->TYPE,ptl->DATA);
         ptl = ptl->NEXT;
         }
      printf("\n");
      printf("SUPPORT G :");
      ptl = tete->ASSOCG;
      while (ptl)
         {
         printf("%s-%s ",ptl->TYPE,ptl->DATA);
         ptl = ptl->NEXT;
         }
      printf("\n");
   
      if (tete->EQUATEF != NULL)
         displayExpr(tete->EQUATEF);
   
      if (tete->EQUATEG != NULL)
         displayExpr(tete->EQUATEG);
   
      if (tete->BASE == BASEF)
         printf("F <-> %s\n",tete->SIGNALF);
      else 
         printf("F <-> %s,  G <-> %s\n",tete->SIGNALF,tete->SIGNALG);
   
      if (tete->XTAG == CONFQX)
         printf("QX <-> %s\n",tete->SIGNALQX);
      if (tete->YTAG == CONFQY)
         printf("QY <-> %s\n",tete->SIGNALQY);
   
      if (tete->XTAG == CONFQX || tete->YTAG == CONFQY)
         if (tete->CLKTAG == CONFDIRECT)
            printf("CK <-> %s\n",tete->SIGNALCK);
         else
            printf("CK <-> not %s\n",tete->SIGNALCK);
      
      }
   else
      {
      if (tete->PADTAG == CONFO)
         {
         printf("sortie ");
         if (tete->OITAG == CONFNOT)
            printf("inversee : ");
         else
            printf("directe : ");
         printf("%s\n",tete->SIGNALPAD);
         }
      else
         {
         printf("entree : ");
         printf("%s\n",tete->SIGNALPAD);
         }
      }
   if (tete->BEH)
      displayBeh(tete->BEH,0);
   printf("=======================================\n");
   tete = tete->NEXT;
   }
}

/*-------------------------------------------------------------------------
normX3000Signal : creation de CLB 
---------------------------------------------------------------------------
retour 		: rien
                  (variable globales mises a jour : lstCLB et pSignalCLB)
---------------------------------------------------------------------------*/
void normX3000Signal(name,expr,pSignalCLB,beh)
char *name;
chain_list *expr;
pTH pSignalCLB;
befig_list *beh;
{  
ptype_list *tete;

if (FP_TRACE) 
         printf("Entrou na  normX3000Signal\n");
		/* signal non traite */

if (searchTH(pSignalCLB,name) == EMPTYTH)
   {
   int pt = searchTH(pTabVar2,name);

     if (FP_TRACE) 
          printf(" c'est une fonction simple => baseF ou baseFGM \n ");
   if (pt == EMPTYTH)
      {
      pCLB new = initCLB(CONFCLB);

      new->BASE = BASEF;
      new->XTAG = CONFF;
      new->EQUATEF = copyExpr(expr);

      if (FP_TRACE) 
         printf("calcul de la liste d'association des ports\n");

      new->ASSOCF = supportPtype_listExpr(expr);
      tete = new->ASSOCF;
      tete->TYPE = (long) namealloc("a");	/* bestial... mais ca marche */
             
      if (tete->NEXT)
         {
         tete = tete->NEXT;
         tete->TYPE = (long) namealloc("b");
	 }	

      if (tete->NEXT)
         {
         tete = tete->NEXT;
         tete->TYPE = (long) namealloc("c");
	 }	

      if (tete->NEXT)
         {
         tete = tete->NEXT;
         tete->TYPE = (long) namealloc("d");
	 }	

      if (tete->NEXT)
         {
         tete = tete->NEXT;
         tete->TYPE = (long) namealloc("e");
	 }	

      new->SIGNALF = name;
      new->SIGNALQX = name;

      new->NEXT = lstCLB;
      lstCLB = new;
      addTH(pSignalCLB,name,new);
      }
   else			/* c'est une fonction double => baseFG */
      {
      chain_list *global;
      chain_list *suppG, *suppF;
      pCLB new = initCLB(CONFCLB);
      int cov;

      new->BASE = BASEFG;
      new->XTAG = CONFF;
      new->YTAG = CONFG;
      new->EQUATEF = copyExpr(expr);
      new->EQUATEG = copyExpr(exprSignalBeh(beh,pt));
   
      new->ASSOCF = supportPtype_listExpr(new->EQUATEF);
      new->ASSOCG = supportPtype_listExpr(new->EQUATEG);

      suppG = supportChain_listExpr(new->EQUATEG);
      suppF = supportChain_listExpr(new->EQUATEF);

      if (FP_TRACE) 
         printf(" on met a 'd' la premiere variable qui apparait seulement dans F \n");

      tete = new->ASSOCF;
      while (tete)
         {
         if (!inChain_list(suppG,(char *) tete->DATA))
            {
            tete->TYPE = (long) namealloc("d"); 
            tete = NULL;		/* on ne peut le faire qu'une fois */
            }
         else
            tete = tete->NEXT;
         }
      
     if (FP_TRACE) 
         printf(" on met a 'e' la premiere variable qui apparait seulement dans G \n");

      tete = new->ASSOCG;
      while (tete)
         {
         if (!inChain_list(suppF,(char *) tete->DATA))
            {
            tete->TYPE = (long) namealloc("e"); 
            tete = NULL;
            }
         else
            tete = tete->NEXT;
         }
      
	     if (FP_TRACE) 
         	printf(" on remplit le reste \n");

      cov = 1;

      tete = new->ASSOCF;
      while (tete)
         {
         if (tete->TYPE == NULL)
            {
            switch (cov)
               {
               case 1 : tete->TYPE = (long) namealloc("a");break;
               case 2 : tete->TYPE = (long) namealloc("b");break;
               case 3 : tete->TYPE = (long) namealloc("c");break;
               case 4 : tete->TYPE = (long) namealloc("d");break;
               case 5 : tete->TYPE = (long) namealloc("e");break;
               }
            cov++;
            }
         tete = tete->NEXT;
         }
	
      tete = new->ASSOCG;
      while (tete)
         {
         if (tete->TYPE == NULL)
            {
            ptype_list *ptl;
         if (FP_TRACE) 
         printf(" on regarde en premier ASSOCF pour voir si cela n'a pas deja ete traite \n");
            ptl = new->ASSOCF;
            while (ptl)
               {
               if (tete->DATA == ptl->DATA)
                  tete->TYPE = ptl->TYPE;
               ptl = ptl->NEXT;
               }
            
            if (tete->TYPE == NULL)
               {
               switch (cov)
                  {
                  case 1 : tete->TYPE = (long) namealloc("a");break;
                  case 2 : tete->TYPE = (long) namealloc("b");break;
                  case 3 : tete->TYPE = (long) namealloc("c");break;
                  case 4 : tete->TYPE = (long) namealloc("d");break;
                  case 5 : tete->TYPE = (long) namealloc("e");break;
                  }
               cov++;
               }
            }
         tete = tete->NEXT;
         }

      new->SIGNALF = name;
      new->SIGNALG = (char *) pt;
      new->SIGNALQX = name;
      new->SIGNALQY = (char *) pt;

      new->NEXT = lstCLB;
      lstCLB = new;
      addTH(pSignalCLB,name,new);
      addTH(pSignalCLB,pt,new);
      }
   }

     if (FP_TRACE) 
         printf("Saiu da  normX3000Signal\n");
}  
 
/*-------------------------------------------------------------------------
rebouclage     : test de rebouclage des registres  qx et qy 
---------------------------------------------------------------------------
retour 		: rien
---------------------------------------------------------------------------*/
void rebouclage(clb,namereg,regQ)
pCLB clb;
char *namereg;
char *regQ;
{
ptype_list *assoc;


		/* est-ce que namereg est dans le support ASSOCF ? */

assoc = clb->ASSOCF;
if (inPtype_list(assoc,namereg))
   {
   ptype_list *ptl;
   char *namesub;

   ptl = assoc;
   while (ptl)
      {
      if (ptl->DATA == namereg)
         {
         namesub = (char *) ptl->TYPE;
         ptl->TYPE = (long) regQ;
         }
      ptl = ptl->NEXT;
      }
		/* on normalise le suport :
			si Qx apparait, alors b ne doit pas apparaitre
                        si Qy apparait, alors c ne doit pas apparaitre */

   ptl = assoc;
   while (ptl)
      {
      if (ptl->TYPE == (long) namealloc("b") && regQ == namealloc("qx"))
         {
         ptl->TYPE = (long) namesub;
         }
      if (ptl->TYPE == (long) namealloc("c") && regQ == namealloc("qy"))
         {
         ptl->TYPE = (long) namesub;
         }
      ptl = ptl->NEXT;
      }
   }

		/* est-ce que namereg est dans le support ASSOCG ? */

assoc = clb->ASSOCG;
if (inPtype_list(assoc,namereg))
   {
   ptype_list *ptl;
   char *namesub;

   ptl = assoc;
   while (ptl)
      {
      if (ptl->DATA == namereg)
         {
         namesub = (char *) ptl->TYPE;
         ptl->TYPE = (long) regQ;
         }
      ptl = ptl->NEXT;
      }
		/* on normalise le suport :
			si Qx apparait, alors b ne doit pas apparaitre
                        si Qy apparait, alors c ne doit pas apparaitre */

   ptl = assoc;
   while (ptl)
      {
      if (ptl->TYPE == (long) namealloc("b") && regQ == namealloc("qx"))
         {
         ptl->TYPE = (long) namesub;
         }
      if (ptl->TYPE == (long) namealloc("c") && regQ == namealloc("qy"))
         {
         ptl->TYPE = (long) namesub;
         }
      ptl = ptl->NEXT;
      }
   }
}
 
/*-------------------------------------------------------------------------
normX3000Beh     : passage au format bst
---------------------------------------------------------------------------
retour 		: rien
                  (variable globales mises a jour : lstCLB et pSignalCLB)
---------------------------------------------------------------------------*/
void normX3000Beh(beh)
befig_list *beh;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
binode_list *binode;
biabl_list *biabl;
bepor_list *por;
berin_list *in;

pTH pFanout;
char *ident;
pTH pSignalCLB;
pCLB teteCLB;

int numRO = 0;
int numIO = 0;
int numRR = 0;
int numIR = 0;
int numIOB = 0;
int numCLB = 0;
int numBASEF = 0;
int numBASEFG = 0;
int numBASEFGM = 0;

coCLB = 1;
coIOB = 1;
lstCLB = NULL;
pSignalCLB = createTH(100);
 
	/* simplification des expressions par absorption des 0 et 1 */

mapCarExprBeh(beh,simplif10Expr);

                /* passage au format bst */
 
		/* premiere passe : creation des CLB pour le combinatoire */
                /* on utilise pTabVar2 pour connaitre les associations */

aux = beh->BEAUX;
while (aux)
   {
   if (aux->ABL)
      {  
      normX3000Signal(aux->NAME,aux->ABL,pSignalCLB,beh);
      }  
   aux = aux->NEXT;
   }
 
		/* creation des IOB pour les sorties */
		/* deux cas :    out <= aux;
			         out <= not aux;
                 */

out = beh->BEOUT;
while (out)
   {
   if (out->ABL)
      {
      pCLB new = initCLB(CONFIOB); 
 
      new->PADTAG = CONFO;
      if (ATOM(out->ABL))
         {
         new->NAME = namealloc("outbuff");
         new->OITAG = CONFDIRECT;
         }
      else 
         {
         new->NAME = namealloc("outinvbuff");
         new->OITAG = CONFNOT;
         }
      new->EQUATEO = copyExpr(out->ABL);
      new->SIGNALPAD = out->NAME;
      new->NEXT = lstCLB;
      lstCLB = new;
      }
   out = out->NEXT;
   }  

		/* creation des IOB pour les entrees */

por = beh->BEPOR;
while (por)
   {
   if (por->DIRECTION == 'I')
      {
      pCLB new = initCLB(CONFIOB); 
 
      new->PADTAG = CONFI;
      new->SIGNALPAD = por->NAME;
      new->NAME = namealloc("inbuff");
      new->NEXT = lstCLB;
      lstCLB = new;
      }
   por = por->NEXT;
   }

		/* mapping des FF */

if (FP_TRACE)
   printf("Mapping the registers ..\n");

		/* calcul des fanouts des aux, entrees, et registres */

pFanout = createTH(100);
aux = beh->BEAUX;
while (aux)
   {
   addTH(pFanout,aux->NAME,0);
   aux = aux->NEXT;
   }
in = beh->BERIN;
while (in)
   {
   addTH(pFanout,in->NAME,0);
   in = in->NEXT;
   }
reg = beh->BEREG;
while (reg)
   {
   addTH(pFanout,reg->NAME,0);
   reg = reg->NEXT;
   }
remplTHOccBeh(beh,pFanout);

if (FP_TRACE)
   {
   printf("===== Fanout table ===== \n");
   displayTH(pFanout);
   }

		/* mapping trivial des registres : tous les cas ou l'on
                   peut placer le registre avec sa fonction d'entree */

reg = beh->BEREG;
while (reg)
   {
   char *data;
   pCLB clb;

   biabl = reg->BIABL;
   
		/* recherche du fanin (var. aux) du registre */

   data = VALUE_ATOM(biabl->VALABL);
   clb = (pCLB) searchTH(pSignalCLB,data);

	/* si data correspond a un CLB => ce n'est pas une entree directe */

   if (clb != (pCLB) EMPTYTH)
      {
		/* une sortie combinatoire => on met un registre */

      if (clb->BASE == BASEF || clb->BASE == BASEFGM)
         {
         int fanout = searchTH(pFanout,clb->SIGNALF);

         if (FP_TRACE)
            printf("fanout(%s) = %d\n",clb->SIGNALF,fanout); 


         if (fanout == 1) 		/* on met le FF sur QX */
            {
            clb->DXTAG = CONFF;
            clb->SIGNALQX = reg->NAME;
            if (ATOM(biabl->CNDABL))
               {
               clb->SIGNALCK = VALUE_ATOM(biabl->CNDABL);
               clb->CLKTAG = CONFDIRECT;
               }
            else	/* c'est un not ck */
               {
               clb->SIGNALCK = VALUE_ATOM(CADR(biabl->CNDABL));
               clb->CLKTAG = CONFNOT;
               }
            
			/* rebouclage de QX dans la fonction de F */

            if (searchTH(pFanout,reg->NAME) > 1)
               rebouclage(clb,reg->NAME,namealloc("qx"));

            clb->XTAG = CONFQX;
            }
         else				/* on met le FF sur QY */
            {
            clb->DYTAG = CONFF;
            clb->SIGNALQY = reg->NAME;
            clb->CLKTAG = CONFDIRECT;

			/* rebouclage de QY dans la fonction de F */

            if (searchTH(pFanout,reg->NAME) > 1)
               rebouclage(clb,reg->NAME,namealloc("qy"));
            clb->YTAG = CONFQY;
            }

         addTH(pSignalCLB,reg->NAME,clb);
         }
      else	/* deux sorties : BASEFG */
         {
         if (data == clb->SIGNALF)	/* F sur QX  ? */
            {
            int fanout = searchTH(pFanout,clb->SIGNALF);

            if (FP_TRACE)
               printf("fanout(%s) = %d\n",clb->SIGNALF,fanout); 
         
            if (fanout == 1) 		/* on met le FF sur QX */
               {
               clb->DXTAG = CONFF;
               clb->SIGNALQX = reg->NAME;
               if (ATOM(biabl->CNDABL))
                  {
                  clb->SIGNALCK = VALUE_ATOM(biabl->CNDABL);
                  clb->CLKTAG = CONFDIRECT;
                  }
               else	/* c'est un not ck */
                  {
                  clb->SIGNALCK = VALUE_ATOM(CADR(biabl->CNDABL));
                  clb->CLKTAG = CONFNOT;
                  }

               if (searchTH(pFanout,reg->NAME) > 1)
                  rebouclage(clb,reg->NAME,namealloc("qx"));
               clb->XTAG = CONFQX;
               addTH(pSignalCLB,reg->NAME,clb);
               }
            }
         else				/* G sur QY ? */
            {
            int fanout = searchTH(pFanout,clb->SIGNALF);

            if (FP_TRACE)
               printf("fanout(%s) = %d\n",clb->SIGNALF,fanout); 
         
            if (fanout == 1) 		/* on met le FF sur QY */
               {
               clb->DYTAG = CONFG;
               clb->SIGNALQY = reg->NAME;
               clb->CLKTAG = CONFDIRECT;

               if (searchTH(pFanout,reg->NAME) > 1)
                  rebouclage(clb,reg->NAME,namealloc("qy"));
               clb->YTAG = CONFQY;
               addTH(pSignalCLB,reg->NAME,clb);
               }
            }
         }
      } 
   reg = reg->NEXT;
   }

		/* mapping non trivial */
		/* on parcours a nouveau les registres */


reg = beh->BEREG;
while (reg)
   {
   pCLB clb;
   clb = (pCLB) searchTH(pSignalCLB,reg->NAME);

		/* registre non deja traite  - CLB non alloue */

   if (clb == (pCLB) EMPTYTH)
      {
      pCLB tete = lstCLB;
      biabl = reg->BIABL;

			/* recheche d'un CLB BASEF ou BASEFGM sans registre */
      while (tete)
         {   
         if (tete->BLOCK == CONFCLB && tete->BASE != BASEFG && tete->YTAG == NIL)
            { 
            		/* on utilise QY avec DI */

            if (FP_TRACE)
               {
               printf("je place %s avec clb %s\n",reg->NAME,tete->NAME);
               displayExpr(biabl->VALABL);
               }

	    tete->SIGNALDI = VALUE_ATOM(biabl->VALABL);
            tete->DYTAG = CONFDI;
            tete->SIGNALQY = reg->NAME;
            tete->CLKTAG = CONFDIRECT;

            if (searchTH(pFanout,reg->NAME) > 1)
               rebouclage(tete,reg->NAME,namealloc("qy"));
            tete->YTAG = CONFQY;
            addTH(pSignalCLB,reg->NAME,tete);
            break;
            }
         tete = tete->NEXT;
         }

      if (tete == NULL)		/* il faut creer un nouveau CLB specialement */
         {
         ptype_list *plt;
         pCLB new = initCLB(CONFCLB);

         new->BASE = BASEF;
         new->XTAG = CONFF;
         new->EQUATEF = copyExpr(biabl->VALABL);

			/* calcul de la liste d'association des ports */

         new->ASSOCF = supportPtype_listExpr(new->EQUATEF);
         plt = new->ASSOCF;
         plt->TYPE = (long) namealloc("a");	/* bestial... mais ca marche */
             

         new->SIGNALF = VALUE_ATOM(biabl->VALABL);;
         new->SIGNALQX = reg->NAME; 
         if (ATOM(biabl->CNDABL))
            {
            new->SIGNALCK = VALUE_ATOM(biabl->CNDABL);
            new->CLKTAG = CONFDIRECT;
            }
         else	/* c'est un not ck */
            {
            new->SIGNALCK = VALUE_ATOM(CADR(biabl->CNDABL));
            new->CLKTAG = CONFNOT;
            }

         new->NEXT = lstCLB;
         lstCLB = new;
         addTH(pSignalCLB,reg->NAME,new);
         }
      }

   reg = reg->NEXT;
   }

		/* creation des befig et sauvegarde sur disque */


teteCLB = lstCLB;
while (teteCLB)
   {
   teteCLB->BEH = (befig_list *) CLB2Beh(teteCLB);

   if (teteCLB->BLOCK == CONFIOB)
      numIOB++;
   else
      {
      numCLB++;
      if (teteCLB->BASE == BASEF)
         numBASEF++;
      if (teteCLB->BASE == BASEFG)
         numBASEFG++;
      if (teteCLB->BASE == BASEFGM)
         numBASEFGM++;
      }

		/* on ne genere les vbe des clbs que si le format
                   de sortie est different de "xnf" */

   if (strcmp(FP_OUT_LO,"xnf"))
      decompileVHDL(teteCLB->BEH,NULL,0);
   teteCLB = teteCLB->NEXT;
   }

if (FP_TRACE)
   displayCLB(lstCLB);

maxDepthBeh(beh, &numRO, &numIR, &numIO, &numRR);


#ifdef LAX
   PRINTF("FINAL COST :\n");
#else
   printf("==============================  FINAL COST  ===================================\n");
#endif
PRINTF("Number of IOBs   = %d\n",numIOB);
PRINTF("Number of CLBs   :\n");
PRINTF("   BASE F        = %d\n",numBASEF);
PRINTF("   BASE FG       = %d\n",numBASEFG);
PRINTF("   BASE FGM      = %d\n",numBASEFGM);
PRINTF("   TOTAL         = %d\n",numCLB);

PRINTF("Depth in CLB     :\n");
if (numRO > 0)
   PRINTF("   Reg-Out       = %d\n",numRO-1);
if (numIR > 0)
   PRINTF("   In-Reg        = %d\n",numIR-1);
if (numIO > 0)
   PRINTF("   In-Out        = %d\n",numIO-1);
if (numRR > 0)
   PRINTF("   Reg-Reg       = %d\n",numRR-1);

#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif
}

/*----------------------------------------------------------------------------
createAuxX3000   : creation d'une variable aux
------------------------------------------------------------------------------
retour          : le nom de la nouvelle variable
------------------------------------------------------------------------------*/
char *createAuxX3000(beh,pt,pTabAux,expr)
befig_list *beh;
pNode pt;
pTH pTabAux;
chain_list *expr;
{
char *name;
pNode ptInv;
 
if (ATOM(expr))
   {
   printf("Warning : createAuxAct1 - expression is atomic\n",pt);
   displayExpr(expr);
   }
                /* marquage du noeud pt et de son inverse */
 
if (pt->mark == 0)
   {
   pt->mark = 1;
   ptInv = notBdd(pt);
   ptInv->mark = 2;
 
   name = gensym("auxil",coX3000++);
   beh->BEAUX = beh_addbeaux(beh->BEAUX,name,expr,NULL);
 
   addTH(pTabAux,pt,name);
   addTH(pTabAux,ptInv,name);
   }
else            /* creation d'une variable inverse pour les sorties */
   {
   name = gensym("auxil",coX3000++);
   beh->BEAUX = beh_addbeaux(beh->BEAUX,name,expr,NULL);
   }
return (name);
}

/*----------------------------------------------------------------------------
displayCLB4    : affiche la liste des CLB4 
------------------------------------------------------------------------------
retour          : 
------------------------------------------------------------------------------*/
void displayCLB4(lstCLB4)
ptype_list *lstCLB4;
{
chain_list *tete;
ptype_list *lst = lstCLB4;

printf("==== displayCLB4 (mapp3000.c)==>displays 4-input CLBs list \n");
while (lst)
   {
   if (lst->DATA)
      {
      tete = lst->DATA;
      printf("%s : ",lst->TYPE);
      while (tete)
         {
         printf("%d ",tete->DATA);
         tete = CDR(tete);
         }
      printf("\n");
      } 
   lst = CDR(lst);
   }
printf("==== Ending display CLB4 === \n");
}
 
/*----------------------------------------------------------------------------
varRegroupeCLB4    : on regroupe les CLB 4
                     Attention : res est desalloue si elle est fusionnee
------------------------------------------------------------------------------
retour          : la variable qui matche ou NULL
------------------------------------------------------------------------------*/
char *varRegroupeCLB4(pLstCLB4,res,var)
ptype_list **pLstCLB4;
chain_list *res;
char *var;
{
char *regroupe = NULL;
ptype_list *lst;
chain_list *lc;
 
lst = *pLstCLB4;
while (lst && (regroupe == NULL))
   {
   chain_list *fusion;
 
   if (lst->DATA)
      {
      fusion = (chain_list *) fusionLC(res,lst->DATA);
 
                        /* la fusion marche ... */
 
      if (countChain_list(fusion) < 6)
         {
         regroupe = (char *) lst->TYPE;
         freechain(lst->DATA);
         freechain(res);
         lst->DATA = NULL;
         } 
      freechain(fusion);
      }
   lst = lst->NEXT;
   }
                /* pas de fusion possible, ... on ajoute */
 
if (regroupe == NULL)
   {
   *pLstCLB4 = addptype(*pLstCLB4,var,res);
   }

return regroupe;
}


/*----------------------------------------------------------------------------
cutIntX3000	: fonction de synthese des X3000 
                  Elle retourne directement le support (comme la fonction de
                  cout) et indirectement l'expression. 
                  Lorsqu'une coupure est faite, elle se traduit par la 
                  creation d'une variable auxiliaire dans la beh.
                  Les fonctions a 4 entrees sont regroupees dynamiquement de
                  la meme facon que la fonction de cout.
------------------------------------------------------------------------------
retour		: un chain_list* (support) 
                  + un pointeur d'expression chain_list **pExpr
----------------------------------------------------------------------------- */
chain_list *cutIntX3000(beh,pt,pTabAux,pLstCLB4,pExpr,createVI)
befig_list *beh;
pNode pt;
pTH pTabAux;
chain_list **pLstCLB4;
chain_list **pExpr;
int createVI;
{
chain_list *lstLow, *lstHigh;
chain_list *res;
int lenRes;
pNode pBdd, low, high;
chain_list *expr_l, *expr_h;
int inclus = 0;
char *name;
 
if (pt->index == 0)     /* noeud terminal 0 */
   {
   *pExpr = NAME_ZERO;	/* expression '0' */
   return NULL;		/* suport vide */
   }
 
if (pt->index == 1)     /* noeud terminal 1 */
   {
   *pExpr = NAME_ONE;
   return NULL;
   }
 
if ((pt->low)->index < 2 && (pt->high)->index < 2)      /* noeud atomique */
   {
   if (pt->low == zero)
      *pExpr = NAME_ATOM;
   else
      *pExpr = notExpr(NAME_ATOM);
   return(addchain(NULL,pt->index));
   }
 
                        /* noeud deja traite */
 
if (pt->mark == 1)      /* CLB de fonction directe */
   {
   *pExpr = createAtom((char *) searchTH(pTabAux,pt));
   return(addchain(NULL,pt));
   }
 
if (pt->mark == 2)      /* CLB de fonction inverse */
   {
   *pExpr = notExpr(createAtom((char *) searchTH(pTabAux,pt)));
   return(addchain(NULL,notBdd(pt)));
   }
 
                /* simplifications eventuelles avant l'appel
                   recursif sur les fils */
 
		/* inclus est mis a jour :
                   inclus = 0 pas d'inclusion (par defaut)
                   inclus = 1 low est inclus dans high
                   inclus = 2 high est inclus dans low
                 */
if (RULES_DC)
   {
   pBdd = applyBinBdd(AND,pt->high,pt->low);
 
   if (pBdd == pt->low)
      {
      inclus = 1;
      high = simplifDcOneBdd(pt->high,pt->low);
      }
   else
      high = pt->high;
 
   if (pBdd == pt->high)
      {
      inclus = 2;
      low = simplifDcOneBdd(pt->low,pt->high);
      }
   else
      low = pt->low;
   }
else
   {
   high = pt->high;
   low = pt->low;
   }
 
lstLow = cutIntX3000(beh,low,pTabAux,pLstCLB4,&expr_l,1);
lstHigh = cutIntX3000(beh,high,pTabAux,pLstCLB4,&expr_h,1);

while (1)
   {
   pNode ptInv;
   chain_list *toto;
   chain_list *expr;
 
         /* fusion lexico-graphique des listes chainees */

   res = (chain_list *) fusionLC(lstLow,lstHigh);
   lenRes = countChain_list(res);
      

   if (lenRes <= SUPPORTMAX)
      {
      if (inclus == 0)
         expr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM), 
                               createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));

      if (inclus == 1)
         expr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM), 
                                  expr_l);

      if (inclus == 2)
         expr = createBinExpr(OR, expr_h, 
                                  createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));
 
      if (lenRes < SUPPORTMAX)
         {
         freechain(lstLow);
         freechain(lstHigh);
         *pExpr = expr;
         return(addchain(res,pt->index));
         }

      if (lenRes == SUPPORTMAX)    /* CLB */
         {
         freechain(lstLow);
         freechain(lstHigh);
         freechain(res);
      
         if (createVI)
            *pExpr = createAtom(createAuxX3000(beh,pt,pTabAux,expr));
         else
            *pExpr = expr;
         return(addchain(NULL,pt));
         }
      }
   else
      {				 /* on coupe un fils */
      int lenHigh, lenLow;
 
      lenLow  = countChain_list(lstLow);
      lenHigh = countChain_list(lstHigh);
 
      if (lenRes == SUPPORTMAX + 1)
         {
                        /* CLB 2 sorties - on coupe les deux fils et on les
                           regroupe dans le meme CLB */
 
         if (lenLow > 3 && lenHigh > 3)
            {
            chain_list *fils_h, *fils_l;

            fils_l = createAtom(createAuxX3000(beh,low,pTabAux,expr_l));
            fils_h = createAtom(createAuxX3000(beh,high,pTabAux,expr_h));

            if (inclus == 0)
               *pExpr = createBinExpr(OR, 
                          createBinExpr(AND, fils_h, NAME_ATOM),
                          createBinExpr(AND, fils_l, notExpr(NAME_ATOM)));

            if (inclus == 1)
               *pExpr = createBinExpr(OR, 
                          createBinExpr(AND, fils_h, NAME_ATOM), fils_l);

            if (inclus == 2)
               *pExpr = createBinExpr(OR, fils_h, 
                          createBinExpr(AND, fils_l, notExpr(NAME_ATOM)));

            addTH(pTabVar2,VALUE_ATOM(fils_h),VALUE_ATOM(fils_l));
            addTH(pTabVar2,VALUE_ATOM(fils_l),VALUE_ATOM(fils_h));

            freechain(lstLow);
            freechain(lstHigh);
            freechain(res);

            res = addchain(NULL,high);
            res = addchain(res,low);
            res = addchain(res,pt->index);

            return(res);
            }
         }
 
                /* en cas d'egalite, on coupe celui qui a le plus
                   d'entrees primaires dans le support */
 
      if (lenLow == lenHigh)
         {
         chain_list *lc;
         int numInputLow = 0;
         int numInputHigh = 0;
 
         lc = lstLow;
         while (lc)
            {
            if ((long)lc->DATA < 1000)  /* l'index des entrees est < a 1000 ... */
               numInputLow++;
            lc = lc->NEXT;
            }
         lc = lstHigh;
         while (lc)
            {
            if ((long)lc->DATA < 1000)
               numInputHigh++;
            lc = lc->NEXT;
            }
 
                /* on change lenLow et lenHigh */

         lenLow = numInputLow;
         lenHigh = numInputHigh;
         }
 
 
      if (lenLow > lenHigh)
         {
                        /* on cree un CLB pour low */
 
         expr_l = createAtom(createAuxX3000(beh,low,pTabAux,expr_l));

         if (name = varRegroupeCLB4(pLstCLB4,lstLow,VALUE_ATOM(expr_l)))
            {
            addTH(pTabVar2,VALUE_ATOM(expr_l),name);
            addTH(pTabVar2,name,VALUE_ATOM(expr_l));
            }
         else 
            freechain(res);
         lstLow = addchain(NULL,low);
         }
      else
                        /* on cree un CLB pour high */
         {
         expr_h = createAtom(createAuxX3000(beh,high,pTabAux,expr_h));

         if (name = varRegroupeCLB4(pLstCLB4,lstHigh,VALUE_ATOM(expr_h)))
            {
            addTH(pTabVar2,VALUE_ATOM(expr_h),name);
            addTH(pTabVar2,name,VALUE_ATOM(expr_h));
            }
         else
            freechain(res);
         lstHigh = addchain(NULL,high);
         }
      }
   }
}

/*----------------------------------------------------------------------------
cutX3000	: fonction de synthese des X3000 
------------------------------------------------------------------------------
retour		: un chain_list* (expression) 
------------------------------------------------------------------------------*/
chain_list *cutX3000(beh,pt,pTabAux,pLstCLB4,createVI,var)
befig_list *beh;
pNode pt;
pTH pTabAux;
chain_list **pLstCLB4;
int createVI;
char *var;
{
chain_list *expr;
chain_list *supp;
char *name;
chain_list *tete;

supp = cutIntX3000(beh,pt,pTabAux,pLstCLB4,&expr,createVI);

if (FP_TRACE == 2)
   {
   if (supp != NULL)
      {
      printf("cutX3000 : %s - ",var);
      tete = supp;
   while (tete)
         {
         printf("%d ",tete->DATA);
         tete = CDR(tete);
         }
      printf("\n");
      }
   }

	/* on ne cree pas de VI si :
		1. 0 ou 1 (support vide)
		2. 1 variable positive
		3. 1 variable negative et createVI = 2 (pour les outputs
                   que l'on peut inverser dans l'IOB)
	 */

if (supp == NULL || (supp->NEXT == NULL && (pt->mark != 2 || createVI == 2)))
   {
   if (supp) 
      freechain(supp);
   return(expr);
   }
else
   {
		/* on cree une nouvelle variable */

   if (createVI)
      {
      expr = createAtom(createAuxX3000(beh,pt,pTabAux,expr));


      if (countChain_list(supp) < 5) 	/* on essaie de regrouper expr avec 
                                           une autre fonction a 4 entree */
         {
         if (name = varRegroupeCLB4(pLstCLB4,supp,VALUE_ATOM(expr)))
            {
            addTH(pTabVar2,VALUE_ATOM(expr),name);
            addTH(pTabVar2,name,VALUE_ATOM(expr));
            }
         }
      else
         freechain(supp);
      }
		/* on ne cree pas de nouvelle variable */
   else
      {
      if (countChain_list(supp) < 5) 	/* on essaie de regrouper expr avec 
                                           une autre fonction a 4 entree */
         {
         if (name = varRegroupeCLB4(pLstCLB4,supp,var))
            {
            addTH(pTabVar2,var,name);
            addTH(pTabVar2,name,var);
            }
         }
      else
         freechain(supp);
      }
   return(expr);
   }
}

/*-------------------------------------------------------------------------
mappX3000Beh 	: mapping Xilinx X3000 => decomposing in feasible functions 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
pTH mappX3000Beh(beh)
befig_list *beh;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
binode_list *binode;
biabl_list *biabl;

ptype_list *tete;
ptype_list *lstCLB4=NULL;
pTH pTabAux;

pTabAux = createTH(100);
pTabVar2 = createTH(100);	/* table des couples de fns a 4 entrees */

coX3000 = 1;			/* pour la generation des noms de variables */
markAllBdd(0);
tabName = beh->CIRCUI->pNameI;

aux = beh->BEAUX;
while (aux) 
   {
   if (aux->NODE)
      {
      aux->ABL = cutX3000(beh,aux->NODE,pTabAux,&lstCLB4,0,aux->NAME);
      }
   aux = aux->NEXT;
   }

		/* les sorties sont coupees :
                   on ne peut recuperer qu'un litteral "a" ou "(not a)" */
out = beh->BEOUT;
while (out) 
   {
   if (out->NODE)
      {
      out->ABL = cutX3000(beh,out->NODE,pTabAux,&lstCLB4,2,out->NAME);
      }
   out = out->NEXT;
   }

reg = beh->BEREG;
while (reg) 
   {
   binode = reg->BINODE;
   biabl = reg->BIABL;
   while (binode)
      {
      if (binode->VALNODE)
         {
         biabl->VALABL = cutX3000(beh,binode->VALNODE,pTabAux,&lstCLB4,1,NULL);
         }
      binode = binode->NEXT;
      biabl = biabl->NEXT;
      }
   reg = reg->NEXT;
   }


		/* desallocation de lstCLB4 */

if (FP_TRACE)
   {
   printf("CLB4 non regroupes \n");
   displayCLB4(lstCLB4);
   }

tete = lstCLB4;
while (lstCLB4)
   {
   if (lstCLB4->DATA)
      {
      freechain(lstCLB4->DATA);
      }
   lstCLB4 = lstCLB4->NEXT;
   }
freeptype(tete);

destroyTH(pTabAux);
return pTabVar2;
}
