/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :synthetiseur de FPGA					    */
/*    Fichier :mappAct1                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :Burgun L.                              le : 10/06/1994     */
/*                                                                          */
/*    Modifie par :                                     le : 05/07/1994     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H
#include <math.h>
#include "../mapping/types.h"
#include "../compil/lax_param.h"
#include "fp_type.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "../mapping/compile.h"
/*
#include "./mapping/regles.h"
*/
#include "fpga.h"
#include "act1.h"

/*----------------------------------------------------------------------------
createAuxAct1  	: creation d'une variable aux 
------------------------------------------------------------------------------
retour          : le nom de la nouvelle variable 
------------------------------------------------------------------------------*/
char *createAuxAct1(beh,pt,pTabAux,expr)
befig_list *beh;
pNode pt;
pTH pTabAux;
chain_list *expr;
{
char *name;
pNode ptInv;

if (ATOM(expr))
   {
   printf("Warning : createAuxAct1 - expression is atomic\n",pt);
   displayExpr(expr);
   }
		/* marquage du noeud pt et de son inverse */

if (pt->mark == 0)
   {
   pt->mark = 1;
   ptInv = notBdd(pt);
   ptInv->mark = 2;

   name = gensym("auxact",coAct1++);
   beh->BEAUX = beh_addbeaux(beh->BEAUX,name,expr,NULL);

   addTH(pTabAux,pt,name);
   addTH(pTabAux,ptInv,name);
   }
else		/* creation d'une variable inverse pour les sorties */
   {
   name = gensym("auxact",coAct1++);
   beh->BEAUX = beh_addbeaux(beh->BEAUX,name,expr,NULL);
   }

return (name);
}

/*----------------------------------------------------------------------------
normMux :  : normalisation d'un mux simple 

          |       
       _________ 
      |  /  \   |
a  -- | /    \  |
       --------- 
       |      | 
       b0     b1
------------------------------------------------------------------------------
retour          : trois char* mis a jour 
------------------------------------------------------------------------------*/
void normMux(expr,pa,pb0,pb1)
chain_list *expr;
char **pa,**pb0,**pb1;
{
if (FP_TRACE)
   {
   printf("normMux : ");
   displayExpr(expr);
   }

if (ATOM(expr))
   {
   if (!strcmp(VALUE_ATOM(expr), "'0'"))
      {
      *pa = namealloc("'0'");
      *pb0 = namealloc("'0'");
      *pb1 = namealloc("'0'");
      return;
      }
   if (!strcmp(VALUE_ATOM(expr), "'1'"))
      {
      *pa = namealloc("'1'");
      *pb0 = namealloc("'1'");
      *pb1 = namealloc("'1'");
      return;
      }
   *pa = namealloc("'0'");
   *pb0 = (VALUE_ATOM(expr)); 
   *pb1 = namealloc("'0'");
   return ;
   }
else	/* mux complet */
   {
   chain_list *expr_l;
   chain_list *expr_h;
   chain_list *expr_com;

   if (OPER(expr) == NOT)
      {
      *pa = (VALUE_ATOM(CADR(expr))); 
      *pb0 = namealloc("'1'");
      *pb1 = namealloc("'0'");
      return ;
      }

   expr_l = CADR(expr);
   expr_h = CADR(CDR(expr)); 
   expr_com = CADR(expr_h);

   if (ATOM(expr_com))
      {
      *pa = VALUE_ATOM(expr_com);
      }
   else
      {
      printf("normMux : Error - Bad expression\n");
      displayExpr(expr);
      exit(-1);
      }

   *pb0 = VALUE_ATOM(CADR(CDR(expr_l)));
   *pb1 = VALUE_ATOM(CADR(CDR(expr_h)));
   }
}

	
/*----------------------------------------------------------------------------
normAct1Expr :  : normalisation une expression ACT1 pour la bst

 			|
                     _________     __
                    |  /  \   |___/ /-- s0
                    | /    \  |   \_\-- s1
                     ---------
           __________|      |__________
          |                            |
       _________                  ___________
      |  /  \   |                |   /   \   |
sa -- | /    \  |                |  /     \  | -- sb
       ---------                  -----------
       |      |                    |        |
       a0     a1                   b0       b1

	
	donne :

		(cm8a s0 s1 sa sb a0 a1 b0 b1)


------------------------------------------------------------------------------
retour          : une expression du genre : (cm8a <lst_connecteur>) 
------------------------------------------------------------------------------*/
chain_list *normAct1Expr(expr)
chain_list *expr;
{
chain_list *res;

if (FP_TRACE)
   printf("normAct1 : ");

if (ATOM(expr))
   return expr;
else
   {
   char *s0,*s1,*sa,*sb;
   char *a0,*a1,*b0,*b1;
   chain_list *expr_h, *expr_l;
   chain_list *expr_com;
   chain_list *sexpr;
   int profAOExpr();

   if (OPER(expr) == NOT)	/* (inv X) */
      {
      res = createExpr(INV);
      addHExpr(res,copyExpr(CADR(expr)));
      return res;
      }

   if (OPER(expr) != OR)	
      {
      printf("normAct1Expr : Error - Bad expression\n");
      displayExpr(expr);
      exit(-1);
      }

   expr_l = CADR(expr);
   expr_h = CADR(CDR(expr));

   expr_com = CADR(expr_h);

   if (ATOM(expr_com))
      {
      s0 = VALUE_ATOM(expr_com);
      s1 = s0;
      }
   else			/* commande OU */
      {		
      s0 = VALUE_ATOM(CADR(expr_com));
      s1 = VALUE_ATOM(CADR(CDR(expr_com)));
      }
   normMux(CADR(CDR(expr_l)),&sa,&a0,&a1); 
   normMux(CADR(CDR(expr_h)),&sb,&b0,&b1); 

			/* detection des macros */

/*
   sexpr = simplif10Expr(expr);
   flatArityExpr(sexpr);
   if (!anyExpr(profAOExpr,sexpr))
      {
      int length = 0;
      int numNot = 0;
      chain_list *tete = sexpr;

      length = lengthExpr(sexpr);
      while (tete = CDR(tete))
         {
         if (OPER(CAR(tete)) == NOT)
            numNot++;
         }
      if (OPER(sexpr) == AND)
         {
         if (length == 2)
            {
            if (numNot == 0)
               res = createExpr(AND2);
            if (numNot == 1)
               res = createExpr(AND2A);
            if (numNot == 2)
               res = createExpr(AND2B);
            }
         }
      }
*/

   res = createExpr(CM8A);
   addHExpr(res,createAtom(b1));
   addHExpr(res,createAtom(b0));
   addHExpr(res,createAtom(a1));
   addHExpr(res,createAtom(a0));
   addHExpr(res,createAtom(sb));
   addHExpr(res,createAtom(sa));
   addHExpr(res,createAtom(s1));
   addHExpr(res,createAtom(s0));

   if (FP_TRACE)
      {
      displayExpr(expr);
      printf("         : ");
      displayExpr(res);
      }

   freeExpr(expr);
   return res;
   }
}

/*-------------------------------------------------------------------------
macroAct1 	: sauvegarde sur disque des descriptions
			comportementale
			logique
			icone 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void macroAct1()
{
befig_list *beh;

/*
beh = (befig_list *) cm8a();
operBeh[CM8A] = beh;
if (strcmp(FP_OUT_LO,"adl"))
   decompileVHDL(beh,NULL,0);

beh = (befig_list *) inv(); 
operBeh[INV] = beh;
if (strcmp(FP_OUT_LO,"adl"))
   decompileVHDL(beh,NULL,0);

beh = (befig_list *) dl1(); 
operBeh[DL1] = beh;
if (strcmp(FP_OUT_LO,"adl"))
   decompileVHDL(beh,NULL,0);

beh = (befig_list *) df1(); 
operBeh[DF1] = beh;
if (strcmp(FP_OUT_LO,"adl"))
   decompileVHDL(beh,NULL,0);
printf("End macroAct1\n");
*/

operName[CM8A] = namealloc("cm8a");
operName[INV] = namealloc("inv");
operName[DF1] = namealloc("df1");
operName[DL1] = namealloc("dl1");
}

/*-------------------------------------------------------------------------
normAct1Beh 	: passage au format bst 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void normAct1Beh(beh)
befig_list *beh;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
binode_list *binode;
biabl_list *biabl;
bepor_list *por;

char *ident;

int numRO = 0;
int numIO = 0;
int numRR = 0;
int numIR = 0;
int numIOB = 0;
int sAct1 = 0;
int cAct1 = 0;
int cm8a = 0;
int inv = 0;


		/* passage au format bst */

aux = beh->BEAUX;
while (aux)
   {
   if (aux->ABL)
      {
      aux->ABL = normAct1Expr(aux->ABL);
      if (!ATOM(aux->ABL))
         {
         switch (OPER(aux->ABL))
            {
            case CM8A : cm8a++;
            case INV : inv++;
            }
         cAct1++;
         }
      }
   aux = aux->NEXT;
   }

out = beh->BEOUT;
while (out)
   {
   if (out->ABL)
      {
      out->ABL = normAct1Expr(out->ABL);
      if (!ATOM(out->ABL))
         {
         switch (OPER(out->ABL))
            {
            case CM8A : cm8a++;
            case INV : inv++;
            }
         cAct1++;
         }
      }
   out = out->NEXT;
   }

reg = beh->BEREG;
while (reg)
   {
   biabl = reg->BIABL;
   if (biabl->VALABL)
      {
      chain_list *expr;

      if (biabl->USER)		/* bascule D --> (df1 data command) */
         {
         sAct1++;
         sAct1++;
         expr = createExpr(DF1);

         if (ATOM(biabl->CNDABL))	/* ck = 1 and not ck stable */
            addHExpr(expr,biabl->CNDABL);
         else				/* ck = 0 and not ck stable */
            {
            addHExpr(expr,copyExpr(CADR(biabl->CNDABL)));
            }

         addHExpr(expr,biabl->VALABL);
         biabl->VALABL = expr ;
         }
      else			/* latch --> (dl1 data command) */
         {
         sAct1++;
         expr = createExpr(DL1);

         if (ATOM(biabl->CNDABL))	/* ck = 1 */
            addHExpr(expr,biabl->CNDABL);
         else				/* ck = 0 */
            {
            addHExpr(expr,copyExpr(CADR(biabl->CNDABL)));
            }

         addHExpr(expr,biabl->VALABL);
         biabl->VALABL = expr ;
         }
      }
   reg = reg->NEXT;
   }

maxDepthBeh(beh, &numRO, &numIR, &numIO, &numRR);

por = beh->BEPOR;
while (por)
   {
   numIOB++;   
   por = por->NEXT;
   }

#ifdef LAX
   PRINTF("FINAL COST :\n");
#else
   printf("==============================  FINAL COST  ===================================\n");
#endif
PRINTF("Number of IOBs          = %d\n",numIOB);
PRINTF("Number of ACT1s  :\n");
PRINTF("   Sequential ACT1      = %d\n",sAct1);
PRINTF("   Combinationnal ACT1  = %d\n",cAct1);
PRINTF("   TOTAL                = %d\n",cAct1 + sAct1);

PRINTF("Depth in ACT1    :\n");
if (numRO > 0)
   PRINTF("   Reg-Out              = %d\n",numRO-1);
if (numIR > 0)
   PRINTF("   In-Reg               = %d\n",numIR-1);
if (numIO > 0)
   PRINTF("   In-Out               = %d\n",numIO-1);
if (numRR > 0)
   PRINTF("   Reg-Reg              = %d\n",numRR-1);

#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif
}

/*----------------------------------------------------------------------------
cutIntAct1     	: fonction de decomposition d'un BDD pour ACT1
------------------------------------------------------------------------------
retour          : un int  (pour le type)

les expressions retournees doivent imperativement etre normalisees :

MUX : createBinExpr(OR,
          createBinExpr(AND, fils_droit, commande),
          createBinExpr(AND, fils_gauche, notExpr(commande)))
------------------------------------------------------------------------------*/
int cutIntAct1(beh,pt,pTabAux,pExpr,markOR,createVI)
befig_list *beh;
pNode pt;
pTH pTabAux;
chain_list **pExpr;
int markOR;
int createVI;
{
int type_l, type_h;
int somme;
pNode ptInv, pBdd, low, high;
chain_list *expr_l, *expr_h;
chain_list *expr;

if (pt->index == 0)	/* noeud terminal 0 */
   {
   *pExpr = NAME_ZERO;
   return TYPE_T;
   }

if (pt->index == 1)	/* noeud terminal 1 */
   {
   *pExpr = NAME_ONE;
   return TYPE_T;
   }

			/* noeud deja traite */


if (pt->mark == 1)	/* ACT1 de fonction directe */
   {
   *pExpr = createAtom((char *) searchTH(pTabAux,pt));
   return(TYPE_A);
   }

if (pt->mark == 2)	/* ACT1 de fonction inverse */
   {
   *pExpr = notExpr(createAtom((char *) searchTH(pTabAux,pt)));
   return(TYPE_I);
   }

		/* simplifications eventuelles avant l'appel
                   recursif sur les fils */

if (RULES_DC)
   {
   pBdd = applyBinBdd(AND,pt->high,pt->low);

   if (pBdd == pt->low || pBdd == pt->high)
      {
      if (pBdd == pt->low)
         high = simplifDcOneBdd(pt->high,pt->low);
      else
         high = pt->high;

      if (pBdd == pt->high)
         low = simplifDcOneBdd(pt->low,pt->high);
      else
         low = pt->low;

		/* si un fils a ete simplifie ... */

      if (low != pt->low || high != pt->high)
         {

         expr_l = cutAct1(beh,low,pTabAux,0);
         if (!ATOM(expr_l))
            expr_l = createAtom(createAuxAct1(beh,low,pTabAux,expr_l));

         expr_h = cutAct1(beh,high,pTabAux,0);
         if (!ATOM(expr_h))
            expr_h = createAtom(createAuxAct1(beh,high,pTabAux,expr_h));

         if (pBdd == pt->low)	/* L'.(a'.(0) + a.H) + L.(1) */
            {
            expr = createBinExpr(OR,
                       createBinExpr(AND,expr_h,NAME_ATOM),
                       createBinExpr(AND,NAME_ZERO,notExpr(NAME_ATOM)));

            expr = createBinExpr(OR,
                      createBinExpr(AND,NAME_ONE,copyExpr(expr_l)),
                      createBinExpr(AND,expr,notExpr(expr_l)));
            }
         else			/* H'.(a'.L + a.(0)) + H.(1) */
            {
            expr = createBinExpr(OR,
                       createBinExpr(AND,NAME_ZERO,NAME_ATOM),
                       createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));

            expr = createBinExpr(OR,
                      createBinExpr(AND,NAME_ONE,copyExpr(expr_h)),
                      createBinExpr(AND,expr,notExpr(expr_h)));
            }

         if (createVI)
            *pExpr = createAtom(createAuxAct1(beh,pt,pTabAux,expr));
         else
            *pExpr = expr;
         return(TYPE_A);
         }
      }
   }

high = pt->high;
low = pt->low;

		/* specifique aux ACTEL 1 */
		/* detection du OU */

if (markOR)
   type_l = cutIntAct1(beh,low,pTabAux,&expr_l,0,1);
else
   {
   if (high == low->high)
      type_l = cutIntAct1(beh,low,pTabAux,&expr_l,1,1);
   else
      type_l = cutIntAct1(beh,low,pTabAux,&expr_l,0,1);
   }

type_h = cutIntAct1(beh,high,pTabAux,&expr_h,0,1);

somme = type_l + type_h;

if (somme > 3)		/* coupure */
   {

   if (markOR)
      {
      *pExpr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM),
                                 createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));
      return(TYPE_G);
      }
		/* commande OU */

	/* cette commande n'est possible que si le fils gauche 
           n'a pas deja ete coupe */

   if (high == low->high && low->mark == 0)
      {
      chain_list *exprOR;
      char *NAME_LOW = *(tabName + low->index - 2);

      freeExpr(expr_h);
      exprOR = createExpr(OR);
      addHExpr(exprOR,NAME_ATOM);
      addHExpr(exprOR,createAtom(NAME_LOW));
      substPhyExpr(expr_l,NAME_LOW,exprOR);
      expr = expr_l;

      }
   else
      expr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM),
                               createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));
   if (createVI)
      *pExpr = createAtom(createAuxAct1(beh,pt,pTabAux,expr));
   else
      *pExpr = expr;
   return(TYPE_A);
   }
else
   {
   if (somme == 0)		/* TYPE_T - TYPE_T  */
      {
      freeExpr(expr_l);
      freeExpr(expr_h);

      if (low == zero)
         {
         *pExpr = NAME_ATOM;
         return(TYPE_A);
         }
      else
         {
         *pExpr = notExpr(NAME_ATOM);
         return(TYPE_I);
         }
      }

   if (somme == 3)		/* TYPE_T - TYPE_I */
      {
      if (high == zero || low == one)	/* a'.b'  ou  a' + b'  =>  coupure */
         {
         expr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM),
                                  createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));

			/* commande OU */

         if (markOR)
            {
            *pExpr = expr;
            return(TYPE_G);
            }

         if (createVI)
            *pExpr = createAtom(createAuxAct1(beh,pt,pTabAux,expr));
         else
            *pExpr = expr;
         return(TYPE_A);
         }
      else		 /* a.b'  ou  a + b'  */
         {

			/* pour eviter les problemes avec la commande OU, 
                           on realise b' avec un mux complet */
 
         if (markOR)
            {
            *pExpr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM),
                                       createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));
            return(TYPE_G);
            }

			/* attention : b devient la commande de selection */

         if (high == one)		/* b'.1 + b.a */
            *pExpr = createBinExpr(OR, 
                         createBinExpr(AND,NAME_ATOM,notExpr(copyExpr(expr_l))),
                         createBinExpr(AND,expr_h,expr_l));

         else				/* b'.a + b.0 */ 
            *pExpr = createBinExpr(OR, 
                         createBinExpr(AND,expr_l,notExpr(copyExpr(expr_h))),
                         createBinExpr(AND,NAME_ATOM,expr_h));

         return(TYPE_G);
         }
      }

			/* Dans les autres cas :
				TYPE_T - TYPE_A
				TYPE_A - TYPE_A
			*/

   *pExpr = createBinExpr(OR, createBinExpr(AND,expr_h,NAME_ATOM),
                              createBinExpr(AND,expr_l,notExpr(NAME_ATOM)));
   return(TYPE_G);
   }
}

/*----------------------------------------------------------------------------
cutAct1     	: fonction de decomposition d'un BDD pour ACT1
------------------------------------------------------------------------------
retour          : une expression 
------------------------------------------------------------------------------*/
chain_list *cutAct1(beh,pt,pTabAux,createVI)
befig_list *beh;
pNode pt;
pTH pTabAux;
int createVI;
{
chain_list *expr;
int type;
pNode ptInv;

type = cutIntAct1(beh,pt,pTabAux,&expr,0,createVI);

if (type < TYPE_I)	/* TYPE_T ou TYPE_A */
   {
   return (expr);
   }
else
   {
   if (createVI)
      {
      expr = createAtom(createAuxAct1(beh,pt,pTabAux,expr));
      }
   return(expr);
   }
}

/*-------------------------------------------------------------------------
mappAct1Beh 	: mapping ACT1 => decomposing in feasible functions 
---------------------------------------------------------------------------
---------------------------------------------------------------------------*/
void mappAct1Beh(beh)
befig_list *beh;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
binode_list *binode;
biabl_list *biabl;

pTH pTabAux;

pTabAux = createTH(100);
coAct1 = 1;			/* pour la generation de nom de variables */
markAllBdd(0);
tabName = beh->CIRCUI->pNameI;


aux = beh->BEAUX;
while (aux)
   {
   if (aux->NODE)
      {
      aux->ABL = cutAct1(beh,aux->NODE,pTabAux,0);
      }
   aux = aux->NEXT;
   }
 
out = beh->BEOUT;
while (out)
   {
   if (out->NODE)
      {
      out->ABL = cutAct1(beh,out->NODE,pTabAux,0);
      }
   out = out->NEXT;
   }
 
reg = beh->BEREG;
while (reg)
   {
   binode = reg->BINODE;
   biabl = reg->BIABL;
   while (binode)
      {
      if (binode->CNDNODE) 
         {
         biabl->CNDABL = cutAct1(beh,binode->CNDNODE,pTabAux,1);
         }
      if (binode->VALNODE)
         {
         biabl->VALABL = cutAct1(beh,binode->VALNODE,pTabAux,1);
         }
      binode = binode->NEXT;
      biabl = biabl->NEXT;
      }
   reg = reg->NEXT;
   }

destroyTH(pTabAux);

}
