
#include MUT_H
#include MPH_H
#include MLO_H
#include BEH_H
#include "c4.h"
#include "c4_stretch.h"
#include ICN_H
#include IAC_H
#include ICU_H

/******************************************************************************
 ***  Version 1.0b02                                            14/06/1991  ***
 ******************************************************************************

 ******************************************************************************
 ******************************************************************************
 ***                                                                        ***
 ***                            c.c.c.c tools                               ***
 ***                    Complex Cmos Cells Compiler                         ***
 ***                                                                        ***
 ******************************************************************************
 ******************************************************************************

 ******************************************************************************
 ***                  STRETCHED CELLS GENERATOR MODULE                      ***
 ******************************************************************************/

/*----------------------------------------------------------------------------*
  -------- very low-level func -------------------------------------------[]--
  ----------------------------------------------------------------------------
  SYNTAX : scaleTrans(loFig_list *, int, int)
  ----------------------------------------------------------------------------
  EXEMPLE : scaleTrans(lFig, 1, 3)
  ----------------------------------------------------------------------------
  FONCTION :
  ----------------------------------------------------------------------------
  ERROR : none
 *----------------------------------------------------------------------------*/

void scaleTrans(lFig, dn, dp)
lofig_list *lFig;
int dn, dp;
{
  lotrs_list *lTr;

  for (lTr = lFig->LOTRS; lTr; lTr = lTr->NEXT)
    if (lTr->X == (-SCALE_X)) {
      lTr->WIDTH += (4 * SCALE_X * ((lTr->TYPE == TRANSN) ? dn : dp));
      lTr->X = 0;
    }
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*
  =-=-=-=- low level func =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-[]=-
  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  SYNTAX : c4_genPCells(short, char *, chain_list *, char *, char *)
  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  EXEMPLE : c4_genPCells(NT1, "nt1_y", lc_io, "s", "io")
  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  FONCTION :
  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  ERROR : none
 *=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

char *c4_genPCells(cellType, name, lc_io, out, in1, in2, in3, in4, in5)
short cellType;
char *name;
chain_list *lc_io;
char *out;
char *in1, *in2, *in3, *in4, *in5;
{
lofig_list *lFig;
struct IconGate *not1icon();
struct IconGate *not1Picon();
struct IconGate *buficon();
struct IconGate *high1icon();
struct IconGate *low1icon();
struct IconGate *bus1icon();
struct IconGate *tieicon();
struct IconGate *latch1icon();
struct IconGate *latch1nicon();
struct IconGate *latch2icon();
struct IconGate *latch2nicon();
struct IconGate *bd1icon();
struct IconGate *bd1nicon();

(void)dellofig(name);
switch (cellType)
   {
   case N1 :
      n1_p(name, out, in1);
      lofigNot(name, lc_io, out, in1);
      delicongate((char *) ICONGATE_MODELNAME(not1icon(name, 1)));
      break;

		/* inverseur puissant */
   case N1P :
      n1p_p(name, out, in1);
      lofigNotP(name, lc_io, out, in1);
      delicongate((char *) ICONGATE_MODELNAME(not1Picon(name, 1)));
      break;
		/* buffer */
   case BUF :
      b1_p(name, out, in1);
      lofigBuf(name, lc_io, out, in1);
      delicongate((char *) ICONGATE_MODELNAME(buficon(name, 1)));
      break;

   case TRI :
      bus_p(name, out, in1, in2);
      lofigBus(name,lc_io,out, in1, in2);
      delicongate((char *) ICONGATE_MODELNAME(bus1icon(name, 1)));
      break;

   case L1 :
      l1_p(name, out,in1,in2);
      lofigLatch1(name,lc_io,out,in1,in2,in3);
      delicongate((char *) ICONGATE_MODELNAME(latch1icon(name, 1)));
      break;

   case L1N :
      l1n_p(name, out, in1, in2);
      lofigLatch1N(name,lc_io,out,in1,in2,in3);
      delicongate((char *) ICONGATE_MODELNAME(latch1nicon(name, 1)));
      break;

   case L2 :
      l2_p(name, out, in1, in2, in3, in4);
      lofigLatch2(name,lc_io,out,in1,in2,in3,in4,in5);
      delicongate((char *) ICONGATE_MODELNAME(latch2icon(name, 1)));
      break;

   case L2N :
      l2n_p(name, out, in1, in2, in3, in4);
      lofigLatch2N(name,lc_io,out,in1,in2,in3,in4,in5);
      delicongate((char *) ICONGATE_MODELNAME(latch2nicon(name, 1)));
      break;

   case BD1 :
      bd1_p(name,out,in1,in2);
      lofigBd1(name,lc_io,out,in1,in2,in3,in4);
      delicongate((char *) ICONGATE_MODELNAME(bd1icon(name, 1)));
      break;

   case BD1N :
      bd1n_p(name,out,in1,in2);
      lofigBd1n(name,lc_io,out,in1,in2,in3,in4);
      delicongate((char *) ICONGATE_MODELNAME(bd1nicon(name, 1)));
      break;

   case TIE :
      tie_p(name);
      lofigTie(name,lc_io);
      delicongate((char *) ICONGATE_MODELNAME(tieicon(name, 1)));
      break;

   case HIGH :
      high_p(name,out);
      lofigHigh(name, lc_io, out);
      delicongate((char *) ICONGATE_MODELNAME(high1icon(name, 1)));
      break;

   case LOW :
      low_p(name, out);
      lofigLow(name, lc_io, out);
      delicongate((char *) ICONGATE_MODELNAME(low1icon(name, 1)));
      break;
   }

(void)dellofig(name);
lFig = getlofig(name, 'A');
/* Supprime depuis le 31/01/1995
scaleTrans(lFig, dn, dp);
*/
savelofig(lFig);

return name;
}
