/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  Synthetiseur Logique                                       */
/*    Fichier :  jaco.c                                                   */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  L. Burgun                            le : 18/03/1994     */
/*                                                                          */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../..       */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H

#define TAILLE_X 100

struct eqNode {
   pNode on;
   pNode dc;
   chain_list *assign;
   struct eqNode *NEXT;
   };

struct X_fathers {
   pNode on;
   pNode dc;
   struct eqNode *lst;		/* la liste des noeuds equivalents */
   };

/*-------------------------------------------------------------------------
extractX_fathers    : calcule les couples on-off correspondant a des peres de X 
---------------------------------------------------------------------------
retour          : 
---------------------------------------------------------------------------*/
void extractX_fathers(pON,pDC,fathers,pCount)
pNode pON;
pNode pDC;
struct X_fathers *fathers;
int *pCount;
{
int i;

if (pDC->index < 2)
   return;

if (pON->index == pDC->index || pON->index < 2)
   {
   int trouve = 0;

   if (pDC->low == one || pDC->high == one || pON->index < 2)
      {
      for (i=0;i<*pCount;i++)
         {
         if (pON == fathers[i].on && pDC == fathers[i].dc)
            {
            trouve = 1;
            break;
            }
         }
         
      if (trouve == 0)	/* nouveau pere de X */
         {
         fathers[*pCount].on = pON;
         fathers[*pCount].dc = pDC;
         *pCount = *pCount + 1;
			/* on avance dans l'autre branche */

         if (pON->index > 1)
            {
            if (pDC->low == one)
               extractX_fathers(pON->high,pDC->high,fathers,pCount);
            else
               extractX_fathers(pON->low,pDC->low,fathers,pCount);
            }
         }

      return;
      }
   else			/* on avance dans les deux BDD */
      {
      extractX_fathers(pON->high,pDC->high,fathers,pCount);
      extractX_fathers(pON->low,pDC->low,fathers,pCount);
      }
   }
else
   {
   if (pON->index > pDC->index)
      {
      extractX_fathers(pON->high,pDC,fathers,pCount);
      extractX_fathers(pON->low,pDC,fathers,pCount);
      }
   else
      {
      extractX_fathers(pON,pDC->high,fathers,pCount);
      extractX_fathers(pON,pDC->low,fathers,pCount);
      }
   }
}

/*-------------------------------------------------------------------------
computeBrothers : calcule les couples on-off de meme index 
---------------------------------------------------------------------------
retour          : 
---------------------------------------------------------------------------*/
void computeBrothers(pON,pDC,brothers)
pNode pON;
pNode pDC;
struct eqNode **brothers;
{
struct eqNode *lst;
int trouve = 0;
int index;

printf("computeBrothers %d %d\n",pON,pDC);

if (pDC->index < 2 && pON->index < 2) return;


		/* on prend l'index max */

if (pON->index < pDC->index)
   index = pDC->index;
else
   index = pON->index;

lst = brothers[index];
while (lst && trouve == 0)
   {
   if ((lst->on == pON) && (lst->dc == pDC))
      trouve = 1;
   lst = lst->NEXT;
   }
         
if (!trouve)
   {
   struct eqNode *newEqNode;

   newEqNode = (struct eqNode *) mbkalloc (sizeof (struct eqNode));
   newEqNode->on = pON;
   newEqNode->dc = pDC;
   newEqNode->NEXT = brothers[index];
   brothers[index] = newEqNode;
   if (pON->index < 2)
      return;
   }
else
   return;

if (pON->index ==  pDC->index)
   {
   computeBrothers(pON->high,pDC->high,brothers);
   computeBrothers(pON->low,pDC->low,brothers);
   }
else
   {
   if (pON->index > pDC->index)
      {
      computeBrothers(pON->high,pDC,brothers);
      computeBrothers(pON->low,pDC,brothers);
      }
   else
      {
      computeBrothers(pON,pDC->high,brothers);
      computeBrothers(pON,pDC->low,brothers);
      }
   }
}
/*-------------------------------------------------------------------------
matchBdd       : teste si deux couples peuvent etre equivalents 
---------------------------------------------------------------------------
retour          : une liste d'association 
---------------------------------------------------------------------------*/
chain_list *matchBdd(pON1,pDC1,pON2,pDC2,pMatch)
pNode pON1,pDC1,pON2,pDC2;
int *pMatch;
{
int index1,index2;

if (pON1->index > pDC1->index)
   index1 = pON1->index;
else
   index1 = pDC1->index;

if (pON2->index > pDC2->index)
   index2 = pON2->index;
else
   index2 = pDC2->index;

if (index1 != index2)
   {
   *pMatch = 0;
   return NULL;
   }

if (pON1 == pON2)
   {
   *pMatch = 1;
   return NULL;
   }

if (pDC1 == zero && pDC2 == zero)
   {
   if (pON1 != pON2)
      {
      *pMatch = 0;
      return NULL; 
      }
   else
      {
      *pMatch = 1;
      return NULL; 
      }
   }
}

/*-------------------------------------------------------------------------
jacoBdd       : minimise un Bdd avec ses DC 
---------------------------------------------------------------------------
retour          : un BDD simplifie 
---------------------------------------------------------------------------*/
pNode jacoBdd(pON,pDC)
pNode pON,pDC;
{
struct X_fathers *fathers;
struct eqNode **brothers;
int i;
int count=0;

		/* initialisation de la structure fathers */

fathers = (struct X_fathers *) mbkalloc (TAILLE_X*sizeof(struct X_fathers));
for (i=0;i<TAILLE_X;i++)
   fathers[i].lst = NULL;

		/* recherche des noeuds ayant un fils a X */

extractX_fathers(pON,pDC,fathers,&count);

		/* calcul des freres */

brothers = (struct eqNode **) mbkalloc ((pON->index+1)*sizeof(struct eqNode *));
for (i=0;i<=pON->index;i++)
   brothers[i] = NULL;

computeBrothers(pON,pDC,brothers);

/*
for (i=0;i<=pON->index;i++)
   {
   struct eqNode *lst;

   lst = brothers[i];
   printf("brother[%d]\n",i);
   while (lst)
      {
      printf("on : ");displayBdd(lst->on,0);
      printf("dc : ");displayBdd(lst->dc,0);
      lst = lst->NEXT;
      }
   }
*/

		/* calcul des noeuds equivalents pour chaque fathers */

for (i=0;i<count;i++)
   {
   struct eqNode *lst;

   printf("============================ fathers[%d] :\n",i);
   printf("ON :\n");
   displayBdd(fathers[i].on,1);
   printf("DC :\n");
   displayBdd(fathers[i].dc,1);

   lst = brothers[(fathers[i].on)->index]; 
   while (lst)
      {
      chain_list *lstMatch = NULL;
      int match = 0;

      lstMatch = matchBDD(fathers[i].on,fathers[i].dc,lst->on,lst->off,&match);

		/* fathers[i] et lst matchent ... */

      if (match)
         {
         struct eqNode *newEqNode;
         newEqNode = (struct eqNode *) mbkalloc (sizeof (struct eqNode));
         newEqNode->on = lst->on;
         newEqNode->dc = lst->dc;
         newEqNode->assign = lstMatch;
         newEqNode->NEXT = fathers[i].lst;
         fathers[i].lst = newEqNode;
         }
      lst = lst->NEXT;
      }
   }

return pON;
}

/*-------------------------------------------------------------------------
jacoMultiBdd  : minimise un multi-BDD avec ses DC 
---------------------------------------------------------------------------
retour          : un multi-BDD simplifie 
---------------------------------------------------------------------------*/
void jacoMultiBdd(pON,pDC,numBdd)
pNode *pON, *pDC;
int numBdd;
{
}
