/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  synthese FPGA                                              */
/*    Fichier :  x3000.c                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  Eudes Prado Lopes                       le : 20 /10/1993 */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include MUT_H
#include LOG_H
#include BEH_H
#include "../mapping/types.h"
#include "../compil/lax_param.h"
#include "../compil/sl_util.h"
#include "../bddorder/bdd_order.h"
#include "fp_type.h"
#include "fpga.h"

/*----------------------------------------------------------------------------
costIntX3000	: fonction interne de cout des X3000 pour guider 
                  le re-odonnancement
------------------------------------------------------------------------------
retour		: un pointeur de liste chainee (support)
                  un cout (entier) -> par adresse
------------------------------------------------------------------------------*/
chain_list *costIntX3000(pt,pCount,pLstCLB4)
pNode pt;
int *pCount;
chain_list **pLstCLB4;
{
chain_list *lstLow, *lstHigh;
chain_list *res;
int lenRes;
pNode pBdd, low, high;


if (pt->index < 2)	/* noeud terminal */
   return (NULL);

if ((pt->low)->index < 2 && (pt->high)->index < 2)	/* noeud atomique */
   {
   return(addchain(NULL,pt->index));
   }

			/* noeud deja traite */

if (pt->mark == 1)	/* CLB de fonction directe */
   {
   return(addchain(NULL,pt));
   }

if (pt->mark == 2)	/* CLB de fonction inverse */
   {
   return(addchain(NULL,notBdd(pt)));
   }

		/* simplifications eventuelles avant l'appel
                   recursif sur les fils */

if (RULES_DC)
   {
   pBdd = applyBinBdd(AND,pt->high,pt->low);

   if (pBdd == pt->low)
      high = simplifDcOneBdd(pt->high,pt->low);
   else
      high = pt->high;

   if (pBdd == pt->high)
      low = simplifDcOneBdd(pt->low,pt->high);
   else
      low = pt->low;
   }
else
   {
   high = pt->high;
   low = pt->low;
   }

lstLow = costIntX3000(low,pCount,pLstCLB4);
lstHigh = costIntX3000(high,pCount,pLstCLB4);

while (1)
   {
   pNode ptInv;
   chain_list *toto;

	 /* fusion lexico-graphique des listes chainees */	

   res = (chain_list *) fusionLC(lstLow,lstHigh);

   lenRes = countChain_list(res);

   if (lenRes < SUPPORTMAX)
      {
      freechain(lstLow);
      freechain(lstHigh);
      return(addchain(res,pt->index));
      }
   if (lenRes == SUPPORTMAX)	/* CLB */
      {
      freechain(lstLow);
      freechain(lstHigh);
      freechain(res);
      pt->mark = 1;
      ptInv = notBdd(pt);
      ptInv->mark = 2; 
      *pCount = *pCount + 1;
      return(addchain(NULL,pt));
      }
   if (lenRes > SUPPORTMAX)	/* on coupe un fils */
      {
      int lenHigh, lenLow;

      lenLow  = countChain_list(lstLow);
      lenHigh = countChain_list(lstHigh);

      if (lenRes == SUPPORTMAX + 1)
         {
                        /* CLB 2 sorties */

         if (lenLow > 3 && lenHigh > 3)
            {
            low->mark = 1;
            ptInv = notBdd(low);
            ptInv->mark = 2; 

            high->mark = 1;
            ptInv = notBdd(high);
            ptInv->mark = 2; 

            *pCount = *pCount + 1;
            freechain(lstLow);
            freechain(lstHigh);
            freechain(res);
            res = addchain(NULL,high);
            res = addchain(res,low);
            res = addchain(res,pt->index);
            return(res);
            }
         }


                /* en cas d'egalite, on coupe celui qui a le plus
                   d'entrees primaires dans le support */
 

      if (lenLow == lenHigh)
         {
         chain_list *lc;
         int numInputLow = 0;
         int numInputHigh = 0;
            
         lc = lstLow;
         while (lc)
            {			
            if ((long)lc->DATA < 1000)   /* l'index des entrees est < a 1000 ... */
               numInputLow++;
            lc = lc->NEXT;
            }
         lc = lstHigh;
         while (lc)
            {
            if ((long)lc->DATA < 1000)
               numInputHigh++;
            lc = lc->NEXT;
            }

		/* on change lenLow et lenHigh */

         lenLow = numInputLow;
         lenHigh = numInputHigh;
         }  


      if (lenLow > lenHigh)
         {
			/* on cree un CLB pour low */

         if (regroupeCLB4(pLstCLB4,lstLow))
            *pCount = *pCount - 1;

         low->mark = 1;
         ptInv = notBdd(low);
         ptInv->mark = 2; 
         *pCount = *pCount + 1;
         freechain(res);
         lstLow = addchain(NULL,low);
         }
      else
			/* on cree un CLB pour high */
         {
         if (regroupeCLB4(pLstCLB4,lstHigh))
            *pCount = *pCount - 1;
	
         high->mark = 1;
         ptInv = notBdd(high);
         ptInv->mark = 2; 
         *pCount = *pCount + 1;
         freechain(res);
         lstHigh = addchain(NULL,high);
         }
      }
   }
}
/*----------------------------------------------------------------------------
costX3000	: fonction de cout des X3000 pour guider le re-ordonnancement
------------------------------------------------------------------------------
retour		: un cout (entier)
------------------------------------------------------------------------------*/
int costX3000(pt,pLstCLB4)
pNode pt;
chain_list **pLstCLB4;
{
chain_list *supp;
pNode ptInv;
int count = 0;		/* nombre de CLB utilises */

supp = costIntX3000(pt,&count,pLstCLB4);

if (supp == NULL || (supp->NEXT == NULL && pt->mark != 2))
   {
   if (supp) 
      freechain(supp);
   return(count);
   }
else
   {
   if (countChain_list(supp) < 5)
      {
      if (regroupeCLB4(pLstCLB4,supp))
         count = count - 1;
      }

   pt->mark = 1;
   ptInv = notBdd(pt);
   ptInv->mark = 2; 
   return(1 + count);
   }
}

/*----------------------------------------------------------------------------
costIntDepthX3000	: fonction interne de cout des X3000 
------------------------------------------------------------------------------
retour		: un pointeur de liste chainee (support)
                  un cout (entier) -> par adresse
------------------------------------------------------------------------------*/
chain_list *costIntDepthX3000(pt,pCount,pLstCLB4,pDepth)
pNode pt;
int *pCount;
chain_list **pLstCLB4;
int *pDepth;
{
chain_list *lstLow, *lstHigh;
chain_list *res;
int lenRes;
pNode pBdd, low, high;
int depthLow,depthHigh;


if (pt->index < 2)	/* noeud terminal */
   {
   *pDepth = 0;
   return (NULL);
   }

if ((pt->low)->index < 2 && (pt->high)->index < 2)	/* noeud atomique */
   {
		/* est-ce une variable auxiliaire ? */
   *pDepth = 0;
   return(addchain(NULL,pt->index));
   }

			/* noeud deja traite */

if (pt->mark == 1)	/* CLB de fonction directe */
   {
   *pDepth = searchTH(depthTH,pt);
   return(addchain(NULL,pt));
   }

if (pt->mark == 2)	/* CLB de fonction inverse */
   {
   *pDepth = searchTH(depthTH,pt);
   return(addchain(NULL,notBdd(pt)));
   }

		/* simplifications eventuelles avant l'appel
                   recursif sur les fils */

if (RULES_DC)
   {
   pBdd = applyBinBdd(AND,pt->high,pt->low);

   if (pBdd == pt->low)
      high = simplifDcOneBdd(pt->high,pt->low);
   else
      high = pt->high;

   if (pBdd == pt->high)
      low = simplifDcOneBdd(pt->low,pt->high);
   else
      low = pt->low;
   }
else
   {
   high = pt->high;
   low = pt->low;
   }

lstLow = costIntDepthX3000(low,pCount,pLstCLB4,&depthLow);
lstHigh = costIntDepthX3000(high,pCount,pLstCLB4,&depthHigh);

while (1)
   {
   pNode ptInv;
   chain_list *toto;

	 /* fusion lexico-graphique des listes chainees */	

   res = (chain_list *) fusionLC(lstLow,lstHigh);

   lenRes = countChain_list(res);

   if (lenRes < SUPPORTMAX)
      {
      freechain(lstLow);
      freechain(lstHigh);

      if (depthLow > depthHigh)
         *pDepth = depthLow;
      else
         *pDepth = depthHigh;

      return(addchain(res,pt->index));
      }
   if (lenRes == SUPPORTMAX)	/* CLB */
      {
      freechain(lstLow);
      freechain(lstHigh);
      freechain(res);
      pt->mark = 1;
      ptInv = notBdd(pt);
      ptInv->mark = 2; 
      *pCount = *pCount + 1;

      if (depthLow > depthHigh)
         *pDepth = 1 + depthLow;
      else
         *pDepth = 1 + depthHigh;
      addTH(depthTH,pt,*pDepth);

      return(addchain(NULL,pt));
      }
   if (lenRes > SUPPORTMAX)	/* on coupe un fils */
      {
      int lenHigh, lenLow;

      lenLow  = countChain_list(lstLow);
      lenHigh = countChain_list(lstHigh);

      if (lenRes == SUPPORTMAX + 1)
         {
                        /* CLB 2 sorties */

         if (lenLow > 3 && lenHigh > 3)
            {
            low->mark = 1;
            ptInv = notBdd(low);
            ptInv->mark = 2; 

            high->mark = 1;
            ptInv = notBdd(high);
            ptInv->mark = 2; 

            *pCount = *pCount + 1;

            if (depthLow > depthHigh)
               *pDepth = 1 + depthLow;
            else
               *pDepth = 1 + depthHigh;
            addTH(depthTH,low,depthLow + 1);
            addTH(depthTH,high,depthHigh + 1);

            freechain(lstLow);
            freechain(lstHigh);
            freechain(res);
            res = addchain(NULL,high);
            res = addchain(res,low);
            res = addchain(res,pt->index);
            return(res);
            }
         }


                /* en cas d'egalite, on coupe celui qui a le plus
                   d'entrees primaires dans le support */
 
      if (lenLow == lenHigh)
         {
         chain_list *lc;
         int numInputLow = 0;
         int numInputHigh = 0;
            
         lc = lstLow;
         while (lc)
            {
            if ((long)lc->DATA < 1000)
               numInputLow++;
            lc = lc->NEXT;
            }
         lc = lstHigh;
         while (lc)
            {
            if ((long)lc->DATA < 1000)
               numInputHigh++;
            lc = lc->NEXT;
            }
         lenLow = numInputLow;
         lenHigh = numInputHigh;
         }  


      if (lenLow > lenHigh)
         {
			/* on cree un CLB pour low */

         if (regroupeCLB4(pLstCLB4,lstLow))
            *pCount = *pCount - 1;

         low->mark = 1;
         ptInv = notBdd(low);
         ptInv->mark = 2; 
         *pCount = *pCount + 1;

         depthLow++;
         addTH(depthTH,low,depthLow);

         freechain(res);
         lstLow = addchain(NULL,low);
         }
      else
			/* on cree un CLB pour high */
         {
         if (regroupeCLB4(pLstCLB4,lstHigh))
            *pCount = *pCount - 1;
	
         high->mark = 1;
         ptInv = notBdd(high);
         ptInv->mark = 2; 
         *pCount = *pCount + 1;

         depthHigh++;
         addTH(depthTH,high,depthHigh);

         freechain(res);
         lstHigh = addchain(NULL,high);
         }
      }
   }
}
/*----------------------------------------------------------------------------
costDepthX3000	: fonction de cout des X3000 
------------------------------------------------------------------------------
retour		: un cout (entier)
------------------------------------------------------------------------------*/
int costDepthX3000(pt,pLstCLB4,pDepth)
pNode pt;
chain_list **pLstCLB4;
int *pDepth;
{
chain_list *supp;
pNode ptInv;
int count = 0;		/* nombre de CLB utilises */

supp = costIntDepthX3000(pt,&count,pLstCLB4,pDepth);

if (supp == NULL || (supp->NEXT == NULL && pt->mark != 2))
   {
   if (supp) freechain(supp);
   return(count);
   }
else
   {
   if (countChain_list(supp) < 5)
      {
      if (regroupeCLB4(pLstCLB4,supp))
         count = count - 1;
      }

   pt->mark = 1;
   ptInv = notBdd(pt);
   ptInv->mark = 2; 
   return(1 + count);
   }
}

/*----------------------------------------------------------------------------
regroupeCLB4	: on regroupe les CLB 4 
                  Attention : res est desalloue si elle est fusionnee
------------------------------------------------------------------------------
retour		: 1 si possible, 0 sinon 
------------------------------------------------------------------------------*/
int regroupeCLB4(pLstCLB4,res)
chain_list **pLstCLB4;
chain_list *res;
{
int regroupe = 0;
chain_list *lst;
chain_list *lc;

lst = *pLstCLB4;
while (lst && (regroupe == 0))
   {
   chain_list *fusion;

   if (lst->DATA)
      {
      fusion = (chain_list *) fusionLC(res,lst->DATA);

			/* la fusion marche ... */

      if (countChain_list(fusion) < 6)
         {
         regroupe = 1;
         freechain(lst->DATA);
         freechain(res);
         lst->DATA = NULL;
         } 
      freechain(fusion);
      }
   lst = lst->NEXT;
   }
		/* pas de fusion possible, ... on ajoute */

if (regroupe == 0)
   {
   *pLstCLB4 = addchain(*pLstCLB4,res);
   }
return regroupe;
}

/*-------------------------------------------------------------------------
costX3000Beh     : calcule le cout X3000 d'une befig 
---------------------------------------------------------------------------
retour          : un void.
---------------------------------------------------------------------------*/

void costX3000Beh(beh,numCLB,depthCLB)
befig_list *beh;
int *numCLB;
int *depthCLB;
{
beout_list *out;
beaux_list *aux;
bereg_list *reg;
bebus_list *bus;
bebux_list *bux;
binode_list *binode;
biabl_list *biabl;
chain_list *lstCLB4;
chain_list *tete;
int depth;
int maxDepth = 0;


*numCLB = 0;
*depthCLB = 0;
lstCLB4 = NULL;
depthTH = createTH(100);

markAllBdd(0);

aux = beh->BEAUX;
while (aux) 
   {
   if (aux->NODE)
      {
      *numCLB = *numCLB + costDepthX3000(aux->NODE,&lstCLB4,&depth);
      if (maxDepth < depth)
         maxDepth = depth;
      }
   aux = aux->NEXT;
   }

out = beh->BEOUT;
while (out) 
   {
   if (out->NODE)
      {
      *numCLB = *numCLB + costDepthX3000(out->NODE,&lstCLB4,&depth);
      if (maxDepth < depth)
         maxDepth = depth;
      }
   out = out->NEXT;
   }
reg = beh->BEREG;
while (reg) 
   {
   binode = reg->BINODE;
   while (binode)
      {
      if (binode->CNDNODE && binode->VALNODE)
         {
         *numCLB = *numCLB + costDepthX3000(binode->VALNODE,&lstCLB4,&depth);
         if (maxDepth < depth)
            maxDepth = depth;
         *numCLB = *numCLB + costDepthX3000(binode->CNDNODE,&lstCLB4,&depth);
         if (maxDepth < depth)
            maxDepth = depth;
         }
      binode = binode->NEXT;
      }
   reg = reg->NEXT;
   }
		/* desallocation de lstCLB4 */

tete = lstCLB4;
while (lstCLB4)
   {
   if (lstCLB4->DATA)
      freechain(lstCLB4->DATA);
   lstCLB4 = lstCLB4->NEXT;
   }
freechain(tete);
*depthCLB = maxDepth;
destroyTH(depthTH);
}
