/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  Synthetiseur Logique                                       */
/*    Fichier :  sl_systemp.c                                                */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) :  L. Burgun                            le : 30/09/1993     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

#include MUT_H
#include LOG_H
#include BEH_H
#include MLO_H
#include "ctype.h"
#include "../menu/sl_corps.h"
#include "../ablorder/sl_order.h"
#include "../compil/lax_param.h"
#include "../mapping/types.h"
#include "../mapping/compile.h"
#include "../mapping/util.h"
#include "../mbk/sl_mbk.h"
#include "../cccc/c4_debug.h"
#include "../cccc/c4.h"
#include "sl_type.h"
#include "../compil/sl_util.h"
 
chain_list *compileVHDL();

/*----------------------------------------------------------------------------
initializeParam : initialise les parametres de la synthese.
------------------------------------------------------------------------------
retour          : un pointeur sur une structure param.
------------------------------------------------------------------------------*/
param *initializeParam(optim,aux,level,delayPI,intermediate,optimPO,numTransN,numTransP)
int optim;
int aux;
int level;
ptype_list *delayPI;
chain_list *intermediate;
chain_list *optimPO;
int numTransN,numTransP;
{
param *par;

par = (param *) malloc (sizeof(param));
par->mode = optim;
par->aux = aux;
par->level = level;
par->delayPI = delayPI;
par->intermediate = intermediate;
par->earlyPO = optimPO;
par->numTransN = numTransN;
par->numTransP = numTransP;

par->maxSlopeTime = 1000;
par->impedancePI = NULL;
par->capaPO = NULL;
par->capaPI = NULL;

return(par);
}

/*----------------------------------------------------------------------------
initializeSLSystem:  alloue sys 
------------------------------------------------------------------------------
retour		: un pointeur sur une structure systeme. 
------------------------------------------------------------------------------*/

sl_system *initializeSLSystem()
{
sl_system *sl;

sl = (sl_system *) malloc (sizeof(sl_system));

sl->orderPI	= NULL;
sl->order	= 0;
sl->version	= (char *) malloc (100);
sl->fileVhdl	= (char *) malloc (100);
sl->fileOut	= (char *) malloc (100);
sl->file	= (char *) malloc (100);
sl->fileParam	= (char *) malloc (100);
sl->filePar	= (char *) malloc (100);

sl->file_lib	= (char *) malloc (10);
sl->cata_lib	= (char *) malloc (200);
sl->cata_save	= (char *) malloc (200);
sl->c4_lib	= (char *) malloc (10);
sl->in_lo	= (char *) malloc (5);
sl->out_lo	= (char *) malloc (5);
sl->out_ph	= (char *) malloc (5);

return( sl );
}

/* ===========================================================================
msgBiblio : on trappe les exit -1 lors de la compilation de la bibliotheque 
------------------------------------------------------------------------------
retour : void 
============================================================================ */
msgBiblio()
{
printf("Fatal Error\n");
exit(-1);
}


/* ================================================================================
mappingSC : fonction de mapping SC
---------------------------------------------------------------------------------------------
retour : une lofig
=================================================================================== */

lofig_list *mappingSC()
{
chain_list *fileLib;
int lit,maxLit,litRed,litReg,profMax,delaisMax,operBin;
char *sauve_cata_lib;
lofig_list * fig;
char *sauve_work_lib;

PRINTF("Running Standard Cell Mapping...\n");

		/* compilation de la bibliotheque */
fileLib = (chain_list *) SLgetFile(SL_CATA_LIB,"vbe");
if (!fileLib)
   {
   PRINTF("Mapping SC : Error\n--> There is no cell in directory '%s'\n",SL_CATA_LIB);
#ifndef LAX
   exit(-1);
#endif
   return(NULL);
   }
 
VHB_HEDFIG = NULL;

coutAblBeh(SL_BEH,&lit,&maxLit,&litRed,&profMax,&delaisMax,&operBin,
           &litReg,0,SL_DELAYPI);

#ifdef LAX
   PRINTF("INITIAL COST :\n");
#else
printf("=============================  INITIAL COST  ==================================\n");
#endif
 

PRINTF("Total number of literals    = %d\n",lit);
PRINTF("Number of reduced literals  = %d\n",litRed);
PRINTF("Number of latches           = %d\n",litReg);
PRINTF("Maximum logical depth       = %d\n",profMax);
PRINTF("Maximum delay               = %.3f\n",((float) delaisMax)/1000);


#ifdef LAX
   PRINTF("\n");
#else
printf("===============================================================================\n");
#endif

if (SL_TRACE)
   printf("Removing the auxiliary signals...\n");

mapCarExprBeh(SL_BEH,simplif10Expr);

if (SL_AUX == 0)
   compactExprBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO,1,SL_GUARDEDAUX);

mapCarExprBeh(SL_BEH,simplif10Expr);

if (SL_TRACE)
   {
   coutAblBeh(SL_BEH,&lit,&maxLit,&litRed,&profMax,&delaisMax,&operBin,
              &litReg,0,SL_DELAYPI);
   printf("==============================  INITIAL COST  ==================================\n");

   printf("Total number of literals    = %d\n",lit);
   printf("Number of reduced literals  = %d\n",litRed);
   printf("Number of latches           = %d\n",litReg);
   printf("Maximum logical depth       = %d\n",profMax);
   printf("Maximum delay               = %.3f\n",((float) delaisMax)/1000);

   printf("===============================================================================\n");
   }

if (SL_TRACE && SL_AUX == 0)
   {
   printf("apres decompactage\n");
   displayBeh(SL_BEH,1);
   }

	/* on sauvegarde et on positionne MBK_CATA_LIB a SL_CATA_LIB */

sauve_cata_lib = (char *) mbkalloc (200);
sauve_work_lib = WORK_LIB;
WORK_LIB = *CATA_LIB;

if (!getenv("MBK_CATA_LIB"))
   strcpy(sauve_cata_lib,".");
else
   strcpy(sauve_cata_lib, (char *)getenv("MBK_CATA_LIB"));

put_CATA_LIB((char *) genNAME("%s",SL_CATA_LIB));

PRINTF("Compiling library '%s'\n",SL_CATA_LIB);

/*
signal(SIGTERM,msgBiblio);
*/

while (fileLib)		
   {
   if (SL_TRACE)
      printf("compile %s\n",fileLib->DATA);
   if (compileVHDL(fileLib->DATA,NULL,0,NULL,1,0,1,1,0,renameSig,NULL) != NULL)
      {
      removeAuxBeh(VHB_HEDFIG,NULL);
      bddToAblBeh(VHB_HEDFIG, 0, 0);
      }
   fileLib = fileLib->NEXT;
   }

WORK_LIB = sauve_work_lib;

if (SL_TRACE)
   printf("Init Mapping \n");

mappSys = initMapping();

PRINTF("Generating Expert System ...\n");
if (compileLibrary() == 0)
   return(NULL);

mapper();

PRINTF("\n");

	/* Generation de la netlist en memoire */

fig = genMBKBeh(SL_BEH, 1, 1, SL_FILEOUT);

changeAblBeh(SL_BEH);

endMapping();

if (SL_TRACE)
   printf("End of mappingSC\n");
else
   PRINTF("\n");

		/* on restore MBK_CATA_LIB */

put_CATA_LIB(sauve_cata_lib);
mbkfree(sauve_cata_lib);

return fig;
}



/* ==========================================================================
mappingC4	: fonction de mapping avec C4
-----------------------------------------------------------------------------
retour		: une lofig
============================================================================= */
lofig_list *mappingC4()
{
char *carint;
lofig_list *fig;
int lit,maxLit,litRed,litReg,profMax,delaisMax,operBin;
char *save_WORK_LIB;
char *sauve_cata_lib;
chain_list *lstvbe;
FILE *fich;
chain_list *filevbe;

/* ATTENTION : 27/01/95
if (SL_BEH->TYPE & BEH_STABLE)
   {
   PRINTF("Mapping C4 : Error \n");
   displayBehMap(SL_BEH, 1);
   return(NULL); 
   }
*/

PRINTF("Running Mapping with C4 ...\n");

coutAblBeh(SL_BEH,&lit,&maxLit,&litRed,&profMax,&delaisMax,&operBin,
           &litReg,0,SL_DELAYPI);

#ifdef LAX
   PRINTF("INITIAL COST :\n");
#else
   printf("=============================  INITIAL COST  ==================================\n");
#endif
 
PRINTF("Total number of literals    = %d\n",lit);
PRINTF("Number of reduced literals  = %d\n",litRed);
PRINTF("Number of latches           = %d\n",litReg);
PRINTF("Maximum logical depth       = %d\n",profMax);
PRINTF("Maximum delay               = %.3f\n",((float) delaisMax)/1000);

#ifdef LAX
   PRINTF("\n");
#else
   printf("===============================================================================\n");
#endif
PRINTF("Decomposing in feasible functions...\n");

PRINTF(".");
fflush(stdout);
mapCarExprBeh(SL_BEH,simplif10Expr);
devXorBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO);

PRINTF(".");
fflush(stdout);

if (SL_TRACE)
   printf("\nRemoving the auxiliary signals...\n");

mapCarExprBeh(SL_BEH,simplif10Expr);

if (SL_AUX == 0)
   compactExprBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO,1,SL_GUARDEDAUX);

mapCarExprBeh(SL_BEH,simplif10Expr);

if (SL_TRACE == 2 && SL_AUX == 0)
   {
   printf("apres decompactage\n");
   displayBeh(SL_BEH,1);
   }

PRINTF(".");
fflush(stdout);
flatPolarityBeh(SL_BEH);

PRINTF(".");
fflush(stdout);
decompBoolBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO,SL_NUMTRANSP,SL_NUMTRANSN);

		/* on appelle une deuxieme fois (les temps ont change) */
PRINTF(".");
fflush(stdout);
decompBoolBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO,SL_NUMTRANSP,SL_NUMTRANSN);

PRINTF(".");
fflush(stdout);
mapBeh(SL_BEH,SL_OPTIM,SL_DELAYPI,SL_OPTIMPO,SL_NUMTRANSP,SL_NUMTRANSN);

PRINTF(".");
fflush(stdout);
postBeh(SL_BEH,SL_AUX);

PRINTF(".");
fflush(stdout);

PRINTF("\n");

		/* ==== appel a C4 ==== */

PRINTF("Generating CMOS Cells in '%s'...\n",SL_CCCC_LIB);

/*
signal(SIGTERM,msgBiblio);
*/

	/* on modifie WORK_LIB a SL_CCCC_LIB pour sauver les fichiers de C4 */

save_WORK_LIB = WORK_LIB;

WORK_LIB = (char *) genNAME("%s",SL_CCCC_LIB);

	/* on fait un putenv() a cause du mbkenv() de GENLIB */

putenv((char *) genNAME("MBK_WORK_LIB=%s",SL_CCCC_LIB));

c4MappBeh(SL_BEH,SL_TRACE,SL_DELAYPI);

	/* on remet WORK_LIB a sa valeur initiale */

WORK_LIB = save_WORK_LIB;


	/* on sauvegarde et on positionne MBK_CATA_LIB a SL_CCCC_LIB */

sauve_cata_lib = (char *) mbkalloc (2);
if (!getenv("MBK_CATA_LIB"))
   strcpy(sauve_cata_lib,".");
else
   sauve_cata_lib = (char *)getenv("MBK_CATA_LIB");
put_CATA_LIB((char *) genNAME("%s",SL_CCCC_LIB));

	/* on cree un fichier CATAL dans SL_CCCC_LIB */

filevbe = (chain_list *) SLgetFile(SL_CCCC_LIB,"vbe");
system(genNAME("rm %s/CATAL",SL_CCCC_LIB));
fich = fopen((char *)genNAME("%s/CATAL",SL_CCCC_LIB),"w");

if (fich)
   {
   lstvbe = filevbe;
   while (lstvbe)
      {
      char *nameCell = (char *) getIdent(lstvbe->DATA);

	/* 16/02/95 = pour scr : tie   F */
      if (!strcmp(nameCell, namealloc("tie")))
         fprintf(fich,"%s\tF\n", nameCell); 

      fprintf(fich,"%s\tC\n", nameCell); 
      lstvbe = lstvbe->NEXT;
      }
   fclose(fich);
   }
else
   {
   printf("mappingC4 : Error \n---> Can't open file 'CATAL'\n");
   return(NULL);
   }
freechain(filevbe);

fig = genMBKBeh(SL_BEH, 0, 0, SL_FILEOUT);

changeAblBeh(SL_BEH);

		/* on restore MBK_CATA_LIB */

put_CATA_LIB(sauve_cata_lib);
mbkfree(sauve_cata_lib);
return fig;
}
 
/*----------------------------------------------------------------------------
laxInfoVBE	: envoie les info de la befig sur ecran pour lax 
------------------------------------------------------------------------------
retour		: 
------------------------------------------------------------------------------*/
void laxInfoVBE(beh, delay_pi)
befig_list *beh;
ptype_list *delay_pi;
{
int lit,maxLit,litRed,litReg,profMax,delaisMax,operBin;

   mapExprBeh(beh,flatArityExpr);

   coutAblBeh(beh,&lit,&maxLit,&litRed,&profMax,&delaisMax,&operBin,
              &litReg,0,delay_pi);

   PRINTI("\n");
   PRINTI("\n");

   PRINTI("\tTotal number of literals    = %d\n",lit);
   PRINTI("\tNumber of reduced literals  = %d\n",litRed);
   PRINTI("\tNumber of latches           = %d\n",litReg);
   PRINTI("\tNumber of binary operators  = %d\n",operBin);
   PRINTI("\tMaximum cone                = %d\n",maxLit);
   PRINTI("\tMaximum logical depth       = %d\n",profMax);
   PRINTI("\tMaximum delay               = %.3f\n",((float) delaisMax)/1000);

}

/*----------------------------------------------------------------------------
laxOptimize	: Encapsulation de la fonction d'optimisation pour LAX 
------------------------------------------------------------------------------
retour		: une befig optimisee
------------------------------------------------------------------------------*/

befig_list *laxOptimize(beh,trace,optim,aux,intermediate,level,optimPO,delayPI)
befig_list *beh;
int trace;
int optim;
int aux;
chain_list *intermediate;
int level;
chain_list *optimPO;
ptype_list *delayPI;
{
int vhb_reorder = 0;

	/* initialisation de la structure sl_sys */

sl_sys->beh = beh;

sl_sys->fileVhdl = NULL;
sl_sys->fileOut = NULL;
sl_sys->file = NULL;
sl_sys->file_lib = NULL;

sl_sys->auxOF = 1;
sl_sys->mode = 1;
sl_sys->version = "3.02";
sl_sys->trace = trace;
sl_sys->option = 'o';

/* appel aux fonctions d'ordonnancement - 0 rien - 1 ABL - 2 ABL + BDD */
 
if (!getenv("MBK_ORDER"))
   sl_sys->makeOrder = 2;
else
   if (!strcmp((char *)getenv("MBK_ORDER"),"0"))
      sl_sys->makeOrder = 0;
   else
      if (!strcmp((char *)getenv("MBK_ORDER"),"1"))
         sl_sys->makeOrder = 1;
      else
         sl_sys->makeOrder = 2;
 
if (sl_sys->makeOrder >= 1)
   sl_sys->order = 1;


sl_sys->par = initializeParam(optim,aux,level,delayPI,intermediate,optimPO,4,4);

if (SL_MAKEORDER >= 1)
   vhb_reorder = 2;
       
	/* la befig a deja ete compile, 
           on ne cree que les graphes avec compileVHDL */

sl_sys->orderPI = compileVHDL(SL_VHDL, SL_ORDERPI, SL_ORDER, SL_PAR,
                              SL_AUX, 1, vhb_reorder, 1, trace, nameInt, beh);
if (SL_ORDERPI == NULL)
   {
   PRINTF("Aborted compilation\n");
   return NULL;
   }
else
   {
   annealingLOGIC(SL_TRACE);
   }
resetBdd();
return beh;
}

/*----------------------------------------------------------------------------
laxMap	: Encapsulation des fonctions de mapping pour LAX 
------------------------------------------------------------------------------
retour	: une lofig
------------------------------------------------------------------------------*/
lofig_list *laxMap(beh,trace,optim,aux,level,optimPO,delayPI,typeMap,c4_lib,target_lib)
befig_list *beh;
int trace;
int optim;
int aux;
int level;
chain_list *optimPO;
ptype_list *delayPI;
char typeMap;
char *c4_lib;
char *target_lib;
{
lofig_list *fig;

	/* initialisation de la structure sl_sys */

sl_sys->beh = beh;

sl_sys->fileVhdl = NULL;
sl_sys->file = NULL;
sl_sys->fileOut = NULL;
sl_sys->file_lib = NULL;

sl_sys->mode = 1;
sl_sys->version = "3.02";
sl_sys->trace = trace;
sl_sys->option = typeMap; 
sl_sys->cata_lib = target_lib;
sl_sys->c4_lib = c4_lib;

sl_sys->par = initializeParam(optim,aux,level,delayPI,NULL,optimPO,4,4);

	/* la befig a deja ete compile, 
           on ne cree que les graphes avec compileVHDL */

if (compileVHDL(SL_VHDL,SL_ORDERPI,SL_ORDER,SL_PAR,
                SL_AUX,1,1,0,trace,nameInt,beh) == NULL)
   {
   PRINTF("Compile VHDL : Error \n---> Compilation Aborted\n");
   return NULL;
   }
else
   {
	/* elimination des DC */
   removeDCBeh(SL_BEH);
   mapCarExprBeh(SL_BEH,simplif10Expr);

   if (SL_OPTION == 's')
      fig = mappingSC();
   else
      fig = mappingC4();
   }

resetBdd();

return fig;
}
