/* ###--------------------------------------------------------------### */
/*                                                                      */
/* file        : syf_beh_addbeasg.c                                     */
/* date        : Aug 28 1992                                            */
/* version     : v102                                                   */
/* authors     : Pirouz BAZARGAN SABET                                  */
/* content     : contains a specific function used to create a          */
/*            behaviour data structure                                  */
/*                                                                      */
/* ###--------------------------------------------------------------### */

#include MUT_H 
#include LOG_H 
#include "syf_beh.h" 

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbeasg                                      */
/* description     : create a BEASG structure at the top the list       */
/* called func.     : addptype, mbkalloc, namealloc                     */
/* ###--------------------------------------------------------------### */

struct ptype *syf_beh_addbeasg (lastptype, name, abl, type)

struct ptype  *lastptype;     /* pointer on the last ptype structure     */
char          *name;          /* signal's name                           */
struct chain  *abl;          /* condition's expression                   */
unsigned char  type;          /* signal's type (cannot be 0)             */

  {
  struct ptype *ptptype;
  struct syf_beasg *ptbeasg;

  name              = namealloc (name);

  ptbeasg           = (struct syf_beasg *) mbkalloc (sizeof(struct syf_beasg));
  ptbeasg->NAME     = name;
  ptbeasg->ABL      = abl;
  ptbeasg->TYPE     = type;

  ptptype           = addptype (lastptype, BEH_BEASG, (void *)ptbeasg);

  return (ptptype);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbeaux                              */
/* description     : create a BEAUX structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_beaux *syf_beh_addbeaux (lastbeaux, name, abl_expr, bdd_expr, type)

struct syf_beaux *lastbeaux;     /* pointer on the last beaux structure     */
char         *name;          /* signal's name               */
struct chain *abl_expr;          /* signal's expression (ABL)          */
struct node  *bdd_expr;          /* signal's expression (BDD)          */
unsigned char  type;          /* signal's type                */

  {
  struct syf_beaux *ptaux;

  name        = namealloc (name);

  ptaux       = (struct syf_beaux *) mbkalloc (sizeof(struct syf_beaux));
  ptaux->NAME = name;
  ptaux->ABL  = abl_expr;
  ptaux->NODE = bdd_expr;
  ptaux->TYPE = type;
  ptaux->NEXT = lastbeaux;

  return (ptaux);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbebus                              */
/* description     : create a BEBUS structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bebus *syf_beh_addbebus (lastbebus, name, biabl, binode, type)

struct syf_bebus  *lastbebus;     /* pointer on the last bebus structure     */
char          *name;          /* port's name                    */
struct biabl  *biabl;          /* port's drivers (ABL)               */
struct binode *binode;          /* port's drivers (BDD)               */
char           type;          /* port type mark (M or W)          */

  {
  struct syf_bebus *ptbus;

  name          = namealloc (name);

  ptbus         = (struct syf_bebus *) mbkalloc (sizeof(struct syf_bebus));
  ptbus->NAME   = name;
  ptbus->BIABL  = biabl;
  ptbus->BINODE = binode;
  ptbus->TYPE   = type;
  ptbus->NEXT   = lastbebus;

  return (ptbus);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbebux                              */
/* description     : create a BEBUX structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bebux *syf_beh_addbebux (lastbebux, name, biabl, binode, type)

struct syf_bebux  *lastbebux;     /* pointer on the last bebux structure     */
char          *name;          /* signal's name               */
struct biabl  *biabl;          /* signal's expression (ABL)          */
struct binode *binode;          /* signal's expression (BDD)          */
char           type;          /* signal's type mark (M or W)          */

  {
  struct syf_bebux *ptbux;

  name          = namealloc (name);

  ptbux         = (struct syf_bebux *) mbkalloc (sizeof(struct syf_bebux));
  ptbux->NAME   = name;
  ptbux->BIABL  = biabl;
  ptbux->BINODE = binode;
  ptbux->TYPE   = type;
  ptbux->NEXT   = lastbebux;

  return (ptbux);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbecas                              */
/* description     : create a BECAS structure at the top the list          */
/* called func.     : addptype, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct ptype *syf_beh_addbecas (lastptype, abl,type)

struct ptype *lastptype;     /* pointer on the last ptype structure     */
struct chain *abl;          /* condition's expression          */
unsigned char type;          /*  expression's type     i               */

  {
  struct ptype *ptptype;
  struct syf_becas *ptbecas;

  ptbecas           = (struct syf_becas *) mbkalloc (sizeof(struct syf_becas));
  ptbecas->ABL      = abl;
  ptbecas->CHOICE   = NULL;
  ptbecas->USER     = NULL;
  ptbecas->SIZE     =  0;
  ptbecas->TYPE     = type;

  ptptype           = addptype (lastptype, BEH_BECAS, (void *)ptbecas);

  return (ptptype);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbefig                              */
/* description     : create an empty BEFIG structure at the top of the list*/
/*            The BEH_HEDFIG list is not modified.               */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_befig *syf_beh_addbefig (lastbefig, name)

struct syf_befig *lastbefig;     /* pointer on the last befig structure     */
char         *name;          /* figure's name               */

  {
  struct syf_befig *ptfig;

  name          = namealloc (name);

  ptfig         = (struct syf_befig *) mbkalloc (sizeof(struct syf_befig));
  ptfig->NAME   = name;
  ptfig->BEREG  = NULL;
  ptfig->BEMSG  = NULL;
  ptfig->BERIN  = NULL;
  ptfig->BEOUT  = NULL;
  ptfig->BEBUS  = NULL;
  ptfig->BEAUX  = NULL;
  ptfig->BEDLY  = NULL;
  ptfig->BEBUX  = NULL;
  ptfig->BEPOR  = NULL;
  ptfig->BEPCS  = NULL;
  ptfig->BETYP  = NULL;
  ptfig->BEPGM  = NULL;
  ptfig->CIRCUI = NULL;
  ptfig->USER   = NULL;
  ptfig->FLAG   = 0;
  ptfig->NEXT   = lastbefig;

  return (ptfig);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbegen                              */
/* description     : create a BEGEN structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_begen *syf_beh_addbegen (lastbegen, name, type, value)

struct syf_begen   *lastbegen;     /* pointer on the last begen structure     */
char           *name;          /* generic's name               */
void           *value;          /* generic's value               */
unsigned char  type;          /* generic's type                */

  {

  struct syf_begen *ptgen;

  name         = namealloc (name);
  
  ptgen        = (struct syf_begen *) mbkalloc (sizeof(struct syf_begen));
  ptgen->NAME  = name;
  ptgen->TYPE  = type;
  ptgen->VALUE = value;
  ptgen->NEXT  = lastbegen;

  return (ptgen);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbeifs                              */
/* description     : create a BEIFS structure at the top the list          */
/* called func.     : addptype, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct ptype *syf_beh_addbeifs (lastptype, abl)

struct ptype *lastptype;     /* pointer on the last ptype structure     */
struct chain *abl;          /* condition's expression          */

  {
  struct ptype *ptptype;
  struct syf_beifs *ptbeifs;

  ptbeifs           = (struct syf_beifs *) mbkalloc (sizeof(struct syf_beifs));
  ptbeifs->CND      = abl;
  ptbeifs->CNDTRUE  = NULL;
  ptbeifs->CNDFALSE = NULL;
  ptptype           = addptype (lastptype, BEH_BEIFS, (void *)ptbeifs);

  return (ptptype);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbemsg                              */
/* description     : create BEMSG structure at the top the list          */
/* called func.     : mbkalloc, namealloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bemsg *syf_beh_addbemsg (lastbemsg,label,level,message,abl_expr,bdd_expr)

struct syf_bemsg *lastbemsg;     /* pointer on the last bemsg structure     */
char         *label;          /* assert's label               */
char          level;          /* severity level (E or W)          */
char         *message;          /* reported message               */
struct chain *abl_expr;          /* assert's condition (ABL)          */
struct node  *bdd_expr;          /* assert's condition (BDD)          */

  {

  struct syf_bemsg *ptmsg;

  label          = namealloc (label);

  ptmsg          = (struct syf_bemsg *) mbkalloc (sizeof(struct syf_bemsg));
  ptmsg->LABEL   = label;
  ptmsg->LEVEL   = level;
  ptmsg->MESSAGE = message;
  ptmsg->ABL     = abl_expr;
  ptmsg->NODE    = bdd_expr;
  ptmsg->NEXT    = lastbemsg;

  return (ptmsg);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbeout                              */
/* description     : create a BEOUT structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_beout *syf_beh_addbeout (lastbeout, name, abl_expr, bdd_expr,type)

struct syf_beout *lastbeout;     /* pointer on the last beout structure     */
char         *name;          /* port's name                    */
struct chain *abl_expr;          /* port's expression (ABL)          */
struct node  *bdd_expr;          /* port's expression (BDD)          */
unsigned char type;          /* port's type                     */

  {
  struct syf_beout *ptout;

  name         = namealloc (name);

  ptout        = (struct syf_beout *) mbkalloc (sizeof(struct syf_beout));
  ptout->NAME  = name;
  ptout->ABL   = abl_expr;
  ptout->NODE  = bdd_expr;
  ptout->TYPE  = type;
  ptout->NEXT  = lastbeout;

  return (ptout);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbepcs                              */
/* description     : create a BEPCS structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bepcs *syf_beh_addbepcs (lastbepcs, label, sens, inst, outs, flag)

struct syf_bepcs *lastbepcs;     /* pointer on the last bepcs structure     */
char         *label;          /* process's label               */
struct chain *sens;          /* sensitivity list               */
struct ptype *inst;          /* list of instructions               */
struct chain *outs;          /* list of outputs               */
char          flag;          /* flag                         */

  {
  struct syf_bepcs *ptbepcs;

  label                = namealloc (label);

  ptbepcs              = (struct syf_bepcs *) mbkalloc (sizeof(struct syf_bepcs));
  ptbepcs->LABEL       = label;
  ptbepcs->SENSITIVITY = sens;
  ptbepcs->OUTPUTS     = outs;
  ptbepcs->INSTRUCTION = inst;
  ptbepcs->FLAG        = flag;

  ptbepcs->NEXT        = lastbepcs;

  return (ptbepcs);
  }
/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbepgm                              */
/* description     : add an BEPGM structure at the top of a list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bepgm *syf_beh_addbepgm (lastbepgm, tool , type , name , value)
struct syf_bepgm  *lastbepgm;     /* pointer on the last bepgm          */
char          *tool;          /* name's of target tool          */
char          *type;          /* pragma's type                         */
char          *name;          /* pragma's name               */
void          *value;          /* pragma's content                              */
  {
  struct syf_bepgm *ptbepgm;

  if(tool)
       tool            = namealloc (tool);
  if(type)
       type            = namealloc (type);
  if(name)
       name            = namealloc (name);

  ptbepgm         = (struct syf_bepgm *) mbkalloc (sizeof(struct syf_bepgm));

  ptbepgm->TOOL   = tool;
  ptbepgm->TYPE   = type;
  ptbepgm->NAME   = name;
  ptbepgm->VALUE  = value;

  ptbepgm->NEXT   = lastbepgm;

  return (ptbepgm);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbepor                              */
/* description     : create a BEPOR structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bepor *syf_beh_addbepor (lastbepor, name, dir, type)

struct syf_bepor  *lastbepor;     /* pointer on the last bepor structure     */
char          *name;          /* signal's name               */
char           dir;              /* signal's mode (I, O, B, Z, or T)     */
unsigned char  type;          /* signal's type (M or W)          */

  {
  struct syf_bepor *ptbepor;

  name               = namealloc (name);

  ptbepor            = (struct syf_bepor *) mbkalloc (sizeof(struct syf_bepor));
  ptbepor->NAME      = name;
  ptbepor->DIRECTION = dir;
  ptbepor->TYPE      = type;
  ptbepor->NEXT      = lastbepor;

  return (ptbepor);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbereg                              */
/* description     : create a BEREG structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_bereg *syf_beh_addbereg (lastbereg, name, biabl, binode)

struct syf_bereg  *lastbereg;     /* pointer on the last bereg structure     */
char          *name;          /* register's name               */
struct biabl  *biabl;          /* register's drivers (ABL)          */
struct binode *binode;          /* register's drivers (BDD)          */
/*unsigned char  type;           register's type               */

  {
  struct syf_bereg *ptreg;

  name          = namealloc (name);

  ptreg         = (struct syf_bereg *) mbkalloc (sizeof(struct syf_bereg));
  ptreg->NAME   = name;
  ptreg->BIABL  = biabl;
  ptreg->BINODE = binode;
  ptreg->NEXT   = lastbereg;

  return (ptreg);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addberin                              */
/* description     : create a BERIN structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_berin *syf_beh_addberin (lastberin, name)

struct syf_berin *lastberin;     /* pointer on the last berin structure     */
char         *name;          /* signal's name               */

  {
  struct syf_berin *ptrin;

  name         = namealloc (name);

  ptrin        = (struct syf_berin *) mbkalloc (sizeof(struct syf_berin));
  ptrin->NAME  = name;
  ptrin->NEXT  = lastberin;

  return (ptrin);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbetyp                              */
/* description     : add an BETYP structure at the top of a list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct syf_betyp *syf_beh_addbetyp (lastbetyp, name , left , right , value ,
                            size     , byte , base , index, resolv, class)

struct syf_betyp  *lastbetyp;     /* pointer on the last betyp          */
char          *name;          /* type's name                    */
int            left;          /* left bound                    */
int            right;          /* right bound                    */
char         **value;          /* list of values               */
unsigned int   size;          /* number of values               */
unsigned int   byte;          /* number of bytes needed to code values*/
unsigned char  base;          /* the base type or element's type     */
unsigned char  index;          /* the type of the index for an array     */
unsigned char  resolv;          /* index of resolution function          */
char           class;          /* type's class (E, I, A, S)          */

  {
  struct syf_betyp *ptbetyp;

  name            = namealloc (name);

  ptbetyp         = (struct syf_betyp *) mbkalloc (sizeof(struct syf_betyp));

  ptbetyp->CLASS  = class;
  ptbetyp->NAME   = name;
  ptbetyp->LEFT   = left;
  ptbetyp->RIGHT  = right;
  ptbetyp->VALUE  = value;
  ptbetyp->SIZE   = size;
  ptbetyp->BYTE   = byte;
  ptbetyp->BASE   = base;
  ptbetyp->INDEX  = index;
  ptbetyp->RESOLV = resolv;

  ptbetyp->NEXT   = lastbetyp;

  return (ptbetyp);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbiabl                              */
/* description     : create a BIABL structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct biabl *syf_beh_addbiabl (lastbiabl, label, condition, value)

char         *label;          /* block's label               */
struct biabl *lastbiabl;     /* pointer on the last biabl structure     */
struct chain *condition;     /* guard expression (ABL)          */
struct chain *value;          /* value expression (ABL)          */

  {
  struct biabl  *ptbiabl;

  ptbiabl         = (struct biabl *) mbkalloc (sizeof(struct biabl));
  label           = namealloc (label);

  ptbiabl->LABEL  = label;
  ptbiabl->CNDABL = condition;
  ptbiabl->VALABL = value;
  ptbiabl->NEXT   = lastbiabl;

  return (ptbiabl);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_addbinode                              */
/* description     : create a BINODE structure at the top the list          */
/* called func.     : namealloc, mbkalloc                         */
/* ###--------------------------------------------------------------### */

struct binode *syf_beh_addbinode (lastbinode, condition, value)

struct binode *lastbinode;     /* pointer on the last binode structure     */
struct node   *condition;     /* guard expression (BDD)          */
struct node   *value;          /* value expression (BDD)          */

  {
  struct binode *ptbinode;

  ptbinode          = (struct binode *) mbkalloc (sizeof(struct binode));
  ptbinode->CNDNODE = condition;
  ptbinode->VALNODE = value;
  ptbinode->NEXT    = lastbinode;

  return (ptbinode);
  }

/* ###--------------------------------------------------------------### */
/* function     : syf_beh_prdeftyp                              */
/* description     : create BETYPs for predefined types               */
/* called func.     : syf_beh_addbetyp, namealloc                    */
/* ###--------------------------------------------------------------### */

struct syf_betyp *syf_beh_prdeftyp ()

  {
  static char         *bit_vl[2];
  static char         *boolean_vl[2];
  static char         *severity_vl[2];
  struct syf_betyp        *t = NULL;

  boolean_vl[0] = namealloc ("false");
  boolean_vl[1] = namealloc ("true");

  bit_vl[0] = namealloc ("'0'");
  bit_vl[1] = namealloc ("'1'");

  severity_vl[0] = namealloc ("warning");
  severity_vl[1] = namealloc ("error");

     /* ###------------------------------------------------------### */
     /*    Enumeration types                              */
     /* ###------------------------------------------------------### */

  t = syf_beh_addbetyp (t,"severity_level",0,1,severity_vl,2,1,0,0,0,'E');
  t = syf_beh_addbetyp (t,"boolean"       ,0,1,boolean_vl ,2,1,0,0,0,'E');
  t = syf_beh_addbetyp (t,"bit"           ,0,1,bit_vl     ,2,1,0,0,0,'E');

     /* ###------------------------------------------------------### */
     /*    Integer types                              */
     /* ###------------------------------------------------------### */

  t = syf_beh_addbetyp (t,"integer",0x80000000,0x7fffffff,NULL,0,0,0,0,'I');

     /* ###------------------------------------------------------### */
     /*    Subtypes                                   */
     /* ###------------------------------------------------------### */

  t = syf_beh_addbetyp (t,"mux_bit",0,1         ,NULL,0,1,2,0,1,'S');
  t = syf_beh_addbetyp (t,"wor_bit",0,1         ,NULL,0,1,2,0,2,'S');
  t = syf_beh_addbetyp (t,"reg_bit",0,1         ,NULL,0,1,2,0,3,'S');
  t = syf_beh_addbetyp (t,"natural",0,0x7fffffff,NULL,0,1,3,0,0,'S');

     /* ###------------------------------------------------------### */
     /*    Unconstraint array types                         */
     /* ###------------------------------------------------------### */

  t = syf_beh_addbetyp (t,"bit_vector",0,0x7fffffff,NULL,0,1,2,7,0,'U');
  t = syf_beh_addbetyp (t,"mux_vector",0,0x7fffffff,NULL,0,1,4,7,0,'U');
  t = syf_beh_addbetyp (t,"wor_vector",0,0x7fffffff,NULL,0,1,5,7,0,'U');
  t = syf_beh_addbetyp (t,"reg_vector",0,0x7fffffff,NULL,0,1,6,7,0,'U');

  return (t);
  }
