/* ###--------------------------------------------------------------### */
/*                                     */
/* file        : syf_beh_delbeaux.c                    */
/* date        : Aug 24 1992                        */
/* version    : v102                            */
/* authors    : Pirouz BAZARGAN SABET                    */
/* content    : contains a specific function used to delete a        */
/*          behaviour data structure                */
/*                                    */
/* ###--------------------------------------------------------------### */

#include MUT_H 
#include LOG_H 
#include "syf_beh.h"

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbeaux                        */
/* description    : delete a BEAUX structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_beaux *syf_beh_delbeaux (listbeaux, ptbeaux, mode)

struct syf_beaux *listbeaux;    /* list of beaux containing the object    */
struct syf_beaux *ptbeaux;        /* pointer of the BEAUX to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_beaux  headaux;
  struct syf_beaux *ptlastaux;

  if ((listbeaux != NULL) && (ptbeaux != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headaux.NEXT = listbeaux;
    ptlastaux    = &headaux;
    while ((ptlastaux != NULL) && (ptlastaux->NEXT != ptbeaux))
      ptlastaux = ptlastaux->NEXT;

    if (ptlastaux != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if (ptbeaux->ABL != NULL)
          (void) fprintf (stderr,"BEH_warning : beaux `%s` not empty\n",
                          ptbeaux->NAME);
        }
      else
        freeExpr (ptbeaux->ABL);

      ptlastaux->NEXT = ptbeaux->NEXT;
      mbkfree (ptbeaux);
      }

    listbeaux = headaux.NEXT;
    }

  return(listbeaux);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbebus                        */
/* description    : delete a BEBUS structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_bebus *syf_beh_delbebus (listbebus, ptbebus, mode)

struct syf_bebus *listbebus;    /* list of bebus containing the object    */
struct syf_bebus *ptbebus;        /* pointer of the BEBUS to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_bebus  headbus;
  struct syf_bebus *ptlastbus;

  if ((listbebus != NULL) && (ptbebus != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headbus.NEXT = listbebus;
    ptlastbus    = &headbus;
    while ((ptlastbus != NULL) && (ptlastbus->NEXT != ptbebus))
      ptlastbus = ptlastbus->NEXT;

    if (ptlastbus != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbebus->BIABL != NULL) || (ptbebus->BINODE != NULL))
          (void) fprintf (stderr,"BEH_warning : bebus `%s` not empty\n",
                          ptbebus->NAME);
        }
      else
        {
        syf_beh_frebiabl  (ptbebus->BIABL);
        syf_beh_frebinode (ptbebus->BINODE);
        }

      ptlastbus->NEXT = ptbebus->NEXT;
      mbkfree (ptbebus);
      }

    listbebus = headbus.NEXT;
    }

  return(listbebus);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbebux                        */
/* description    : delete a BEBUX structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_bebux *syf_beh_delbebux (listbebux, ptbebux, mode)

struct syf_bebux *listbebux;    /* list of bebux containing the object    */
struct syf_bebux *ptbebux;        /* pointer of the BEBUX to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_bebux  headbux;
  struct syf_bebux *ptlastbux;

  if ((listbebux != NULL) && (ptbebux != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headbux.NEXT = listbebux;
    ptlastbux    = &headbux;
    while ((ptlastbux != NULL) && (ptlastbux->NEXT != ptbebux))
      ptlastbux = ptlastbux->NEXT;

    if (ptlastbux != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbebux->BIABL != NULL) || (ptbebux->BINODE != NULL))
          (void) fprintf (stderr,"BEH_warning : bebux `%s` not empty\n",
                          ptbebux->NAME);
        }
      else
        {
        syf_beh_frebiabl  (ptbebux->BIABL);
        syf_beh_frebinode (ptbebux->BINODE);
        }

      ptlastbux->NEXT = ptbebux->NEXT;
      mbkfree (ptbebux);
      }

    listbebux = headbux.NEXT;
    }

  return(listbebux);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbefig                        */
/* description    : delete a BEFIG structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N.    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_befig *syf_beh_delbefig (listbefig, ptbefig, mode)

struct syf_befig *listbefig;    /* list of befig containing the object    */
struct syf_befig *ptbefig;        /* pointer of the BEFIG to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_befig  headfig;
  struct syf_befig *ptlastfig;
  struct ptype *ptptype;

  if ((listbefig != NULL) && (ptbefig != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headfig.NEXT = listbefig;
    ptlastfig    = &headfig;
    while ((ptlastfig != NULL) && (ptlastfig->NEXT != ptbefig))
      ptlastfig = ptlastfig->NEXT;

    if (ptlastfig != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbefig->BEREG != NULL) || (ptbefig->BEMSG != NULL) ||
            (ptbefig->BERIN != NULL) || (ptbefig->BEOUT != NULL) ||
            (ptbefig->BEBUS != NULL) || (ptbefig->BEAUX != NULL) ||
            (ptbefig->BEDLY != NULL) || (ptbefig->BEBUX != NULL) || 
        (ptbefig->BEPOR != NULL) || (ptbefig->USER  != NULL))

          (void) fprintf (stderr,"BEH_warning : befig `%s` not empty\n",
                          ptbefig->NAME);
        }
      else
        {
        syf_beh_frebereg (ptbefig->BEREG);
        syf_beh_frebemsg (ptbefig->BEMSG);
        syf_beh_freberin (ptbefig->BERIN);
        syf_beh_frebeout (ptbefig->BEOUT);
        syf_beh_frebebus (ptbefig->BEBUS);
        syf_beh_frebeaux (ptbefig->BEAUX);
        syf_beh_frebeaux (ptbefig->BEDLY);
        syf_beh_frebebux (ptbefig->BEBUX);
        syf_beh_frebepor (ptbefig->BEPOR);
        if ((ptptype = getptype (ptbefig->USER,BEH_GENERIC)) != NULL)
          syf_beh_frebegen (ptptype->DATA);
        }

      ptlastfig->NEXT = ptbefig->NEXT;
      mbkfree (ptbefig);
      }

    listbefig = headfig.NEXT;
    }

  return(listbefig);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbegen                        */
/* description    : delete a BEGEN structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_begen *syf_beh_delbegen (listbegen, ptbegen, mode)

struct syf_begen *listbegen;    /* list of begen containing the object    */
struct syf_begen *ptbegen;        /* pointer of the BEGEN to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_begen  headgen;
  struct syf_begen *ptlastgen;

  if ((listbegen != NULL) && (ptbegen != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headgen.NEXT = listbegen;
    ptlastgen    = &headgen;
    while ((ptlastgen != NULL) && (ptlastgen->NEXT != ptbegen))
      ptlastgen = ptlastgen->NEXT;

    if (ptlastgen != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if (ptbegen->VALUE != NULL)
          (void) fprintf (stderr,"BEH_warning : begen `%s` not empty\n",
                          ptbegen->NAME);
        }
      else
        mbkfree (ptbegen->VALUE);

      ptlastgen->NEXT = ptbegen->NEXT;
      mbkfree (ptbegen);
      }

    listbegen = headgen.NEXT;
    }

  return(listbegen);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbemsg                        */
/* description    : delete a BEMSG structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_bemsg *syf_beh_delbemsg (listbemsg, ptbemsg, mode)

struct syf_bemsg *listbemsg;    /* list of bemsg containing the object    */
struct syf_bemsg *ptbemsg;        /* pointer of the BEMSG to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_bemsg  headmsg;
  struct syf_bemsg *ptlastmsg;

  if ((listbemsg != NULL) && (ptbemsg != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headmsg.NEXT = listbemsg;
    ptlastmsg    = &headmsg;
    while ((ptlastmsg != NULL) && (ptlastmsg->NEXT != ptbemsg))
      ptlastmsg = ptlastmsg->NEXT;

    if (ptlastmsg != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if (ptbemsg->ABL != NULL)
          (void) fprintf (stderr,"BEH_warning : bemsg `%s` not empty\n",
                          ptbemsg->LABEL);
        }
      else
        freeExpr (ptbemsg->ABL);

      ptlastmsg->NEXT = ptbemsg->NEXT;
      mbkfree (ptbemsg);
      }

    listbemsg = headmsg.NEXT;
    }

  return(listbemsg);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbeout                        */
/* description    : delete a BEOUT structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_beout *syf_beh_delbeout (listbeout, ptbeout, mode)

struct syf_beout *listbeout;    /* list of beout containing the object    */
struct syf_beout *ptbeout;        /* pointer of the BEOUT to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_beout  headout;
  struct syf_beout *ptlastout;

  if ((listbeout != NULL) && (ptbeout != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headout.NEXT = listbeout;
    ptlastout    = &headout;
    while ((ptlastout != NULL) && (ptlastout->NEXT != ptbeout))
      ptlastout = ptlastout->NEXT;

    if (ptlastout != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if (ptbeout->ABL != NULL)
          (void) fprintf (stderr,"BEH_warning : beout `%s` not empty\n",
                          ptbeout->NAME);
        }
      else
        freeExpr (ptbeout->ABL);

      ptlastout->NEXT = ptbeout->NEXT;
      mbkfree (ptbeout);
      }

    listbeout = headout.NEXT;
    }

  return(listbeout);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbepor                        */
/* description    : delete a BEPOR structure and return the pointer of    */
/*          the next object.                    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_bepor *syf_beh_delbepor (listbepor, ptbepor)

struct syf_bepor *listbepor;    /* list of bepor containing the object    */
struct syf_bepor *ptbepor;        /* pointer of the BEPOR to be deleted    */

  {
  struct syf_bepor  headpor;
  struct syf_bepor *ptlastpor;

  if ((listbepor != NULL) && (ptbepor != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headpor.NEXT = listbepor;
    ptlastpor    = &headpor;
    while ((ptlastpor != NULL) && (ptlastpor->NEXT != ptbepor))
      ptlastpor = ptlastpor->NEXT;

    if (ptlastpor != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /* ###------------------------------------------------------### */

      ptlastpor->NEXT = ptbepor->NEXT;
      mbkfree (ptbepor);
      }

    listbepor = headpor.NEXT;
    }

  return(listbepor);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbereg                        */
/* description    : delete a BEREG structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_bereg *syf_beh_delbereg (listbereg, ptbereg, mode)

struct syf_bereg *listbereg;    /* list of bereg containing the object    */
struct syf_bereg *ptbereg;        /* pointer of the BEREG to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct syf_bereg  headreg;
  struct syf_bereg *ptlastreg;

  if ((listbereg != NULL) && (ptbereg != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headreg.NEXT = listbereg;
    ptlastreg    = &headreg;
    while ((ptlastreg != NULL) && (ptlastreg->NEXT != ptbereg))
      ptlastreg = ptlastreg->NEXT;

    if (ptlastreg != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbereg->BIABL != NULL) || (ptbereg->BINODE != NULL))
          (void) fprintf (stderr,"BEH_warning : bereg `%s` not empty\n",
                          ptbereg->NAME);
        }
      else
        {
        syf_beh_frebiabl  (ptbereg->BIABL);
        syf_beh_frebinode (ptbereg->BINODE);
        }

      ptlastreg->NEXT = ptbereg->NEXT;
      mbkfree (ptbereg);
      }

    listbereg = headreg.NEXT;
    }

  return(listbereg);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delberin                        */
/* description    : delete a BERIN structure and return the pointer of    */
/*          the next object.                    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct syf_berin *syf_beh_delberin (listberin, ptberin)

struct syf_berin *listberin;    /* list of berin containing the object    */
struct syf_berin *ptberin;        /* pointer of the BERIN to be deleted    */

  {
  struct syf_berin  headrin;
  struct syf_berin *ptlastrin;

  if ((listberin != NULL) && (ptberin != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headrin.NEXT = listberin;
    ptlastrin    = &headrin;
    while ((ptlastrin != NULL) && (ptlastrin->NEXT != ptberin))
      ptlastrin = ptlastrin->NEXT;

    if (ptlastrin != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /* ###------------------------------------------------------### */

      ptlastrin->NEXT = ptberin->NEXT;
      mbkfree (ptberin);
      }

    listberin = headrin.NEXT;
    }

  return(listberin);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbiabl                        */
/* description    : delete a BIABL structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct biabl *syf_beh_delbiabl (listbiabl, ptbiabl, mode)

struct biabl *listbiabl;    /* list of biabl containing the object    */
struct biabl *ptbiabl;        /* pointer of the BIABL to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct biabl  headabl;
  struct biabl *ptlastabl;

  if ((listbiabl != NULL) && (ptbiabl != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headabl.NEXT = listbiabl;
    ptlastabl    = &headabl;
    while ((ptlastabl != NULL) && (ptlastabl->NEXT != ptbiabl))
      ptlastabl = ptlastabl->NEXT;

    if (ptlastabl != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbiabl->VALABL != NULL) || (ptbiabl->CNDABL != NULL))
          (void) fprintf (stderr,"BEH_warning : biabl not empty\n");
        }
      else
        {
        freeExpr (ptbiabl->VALABL);
        freeExpr (ptbiabl->CNDABL);
        }

      ptlastabl->NEXT = ptbiabl->NEXT;
      mbkfree (ptbiabl);
      }

    listbiabl = headabl.NEXT;
    }

  return(listbiabl);
  }

/* ###--------------------------------------------------------------### */
/* function    : syf_beh_delbinode                        */
/* description    : delete a BINODE structure and return the pointer of    */
/*          the next object. A warning is printed out if the    */
/*          object to be deleted is not empty when the mode is N    */
/* called func.    : mbkfree                        */
/* ###--------------------------------------------------------------### */

struct binode *syf_beh_delbinode (listbinode, ptbinode, mode)

struct binode *listbinode;    /* list of binode containing the object    */
struct binode *ptbinode;    /* pointer of the BINODE to be deleted    */
char          mode;        /* recursive delete or not (Y or N)    */

  {
  struct binode  headnode;
  struct binode *ptlastnode;

  if ((listbinode != NULL) && (ptbinode != NULL))
    {

    /* ###------------------------------------------------------### */
    /*    Search the object to be deleted                */
    /* ###------------------------------------------------------### */

    headnode.NEXT = listbinode;
    ptlastnode    = &headnode;
    while ((ptlastnode != NULL) && (ptlastnode->NEXT != ptbinode))
      ptlastnode = ptlastnode->NEXT;

    if (ptlastnode != NULL)
      {

    /* ###------------------------------------------------------### */
    /*    If the object doesn't exist return the list without    */
    /* modification.                        */
    /*    If the object has been found check the mode and, if asked    */
    /* delete pointed objects recursively.                */
    /* ###------------------------------------------------------### */

      if (mode == 'N')
        {
        if ((ptbinode->VALNODE != NULL) || (ptbinode->CNDNODE != NULL))
          (void) fprintf (stderr,"BEH_warning : binode not empty\n");
        }

      ptlastnode->NEXT = ptbinode->NEXT;
      mbkfree (ptbinode);
      }

    listbinode = headnode.NEXT;
    }

  return(listbinode);
  }
