/* ###--------------------------------------------------------------### */
/* 									*/
/* file		: syf_beh_frebeaux.c					*/
/* date		: Aug 24 1992						*/
/* version	: v102							*/
/* authors	: Pirouz BAZARGAN SABET					*/
/* content	: contains a specific function used to delete a list	*/
/*		  of behaviour data structures				*/
/*									*/
/* ###--------------------------------------------------------------### */

#include MUT_H 
#include LOG_H 
#include "syf_beh.h"

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebeaux						*/
/* description	: delete a list of BEAUX structures and all objects	*/
/*		  pointed by any os BEAUXs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebeaux (listbeaux)

struct syf_beaux *listbeaux;	/* list of beaux to be deleted		*/

  {
  struct syf_beaux *ptbeaux;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbeaux != NULL)
    {
    freeExpr (listbeaux->ABL);

    ptbeaux   = listbeaux;
    listbeaux = listbeaux->NEXT;
    mbkfree (ptbeaux);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebebus						*/
/* description	: delete a list of BEBUS structures and all objects	*/
/*		  pointed by any os BEBUSs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebebus (listbebus)

struct syf_bebus *listbebus;	/* list of bebus to be deleted		*/

  {
  struct syf_bebus *ptbebus;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbebus != NULL)
    {
    syf_beh_frebiabl  (listbebus->BIABL);
    syf_beh_frebinode (listbebus->BINODE);

    ptbebus   = listbebus;
    listbebus = listbebus->NEXT;
    mbkfree (ptbebus);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebebux						*/
/* description	: delete a list of BEBUX structures and all objects	*/
/*		  pointed by any os BEBUXs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebebux (listbebux)

struct syf_bebux *listbebux;	/* list of bebux to be deleted		*/

  {
  struct syf_bebux *ptbebux;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbebux != NULL)
    {
    syf_beh_frebiabl  (listbebux->BIABL);
    syf_beh_frebinode (listbebux->BINODE);

    ptbebux   = listbebux;
    listbebux = listbebux->NEXT;
    mbkfree (ptbebux);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebefig						*/
/* description	: delete a list of BEFIG structures and all objects	*/
/*		  pointed by any os BEFIGs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebefig (listbefig)

struct syf_befig *listbefig;	/* list of befig to be deleted		*/

  {
  struct syf_befig *ptbefig;
  struct ptype *ptptype;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbefig != NULL)
    {
    syf_beh_frebereg (ptbefig->BEREG);
    syf_beh_frebemsg (ptbefig->BEMSG);
    syf_beh_freberin (ptbefig->BERIN);
    syf_beh_frebeout (ptbefig->BEOUT);
    syf_beh_frebebus (ptbefig->BEBUS);
    syf_beh_frebeaux (ptbefig->BEAUX);
    syf_beh_frebeaux (ptbefig->BEDLY);
    syf_beh_frebebux (ptbefig->BEBUX);
    syf_beh_frebepor (ptbefig->BEPOR);
    if ((ptptype = getptype (ptbefig->USER,BEH_GENERIC)) != NULL)
      syf_beh_frebegen (ptptype->DATA);

    ptbefig   = listbefig;
    listbefig = listbefig->NEXT;
    mbkfree (ptbefig);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebegen						*/
/* description	: delete a list of BEGEN structures and all objects	*/
/*		  pointed by any os BEGENs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebegen (listbegen)

struct syf_begen *listbegen;	/* list of begen to be deleted		*/

  {
  struct syf_begen *ptbegen;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbegen != NULL)
    {
    mbkfree (listbegen->VALUE);

    ptbegen   = listbegen;
    listbegen = listbegen->NEXT;
    mbkfree (ptbegen);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebemsg						*/
/* description	: delete a list of BEMSG structures and all objects	*/
/*		  pointed by any os BEMSGs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebemsg (listbemsg)

struct syf_bemsg *listbemsg;	/* list of bemsg to be deleted		*/

  {
  struct syf_bemsg *ptbemsg;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbemsg != NULL)
    {
    freeExpr (listbemsg->ABL);

    ptbemsg   = listbemsg;
    listbemsg = listbemsg->NEXT;
    mbkfree (ptbemsg);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebeout						*/
/* description	: delete a list of BEOUT structures and all objects	*/
/*		  pointed by any os BEOUTs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebeout (listbeout)

struct syf_beout *listbeout;	/* list of beout to be deleted		*/

  {
  struct syf_beout *ptbeout;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbeout != NULL)
    {
    freeExpr (listbeout->ABL);

    ptbeout   = listbeout;
    listbeout = listbeout->NEXT;
    mbkfree (ptbeout);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebepor						*/
/* description	: delete a list of BEAUX structures and all objects	*/
/*		  pointed by any os BEAUXs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebepor (listbepor)

struct syf_bepor *listbepor;	/* list of bepor to be deleted		*/

  {
  struct syf_bepor *ptbepor;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, delete the object itself	*/
	/* ###------------------------------------------------------### */

  while (listbepor != NULL)
    {
    ptbepor   = listbepor;
    listbepor = listbepor->NEXT;
    mbkfree (ptbepor);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebereg						*/
/* description	: delete a list of BEAUX structures and all objects	*/
/*		  pointed by any os BEAUXs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebereg (listbereg)

struct syf_bereg *listbereg;	/* list of bereg to be deleted		*/

  {
  struct syf_bereg *ptbereg;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbereg != NULL)
    {
    syf_beh_frebiabl  (listbereg->BIABL);
    syf_beh_frebinode (listbereg->BINODE);

    ptbereg   = listbereg;
    listbereg = listbereg->NEXT;
    mbkfree (ptbereg);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_freberin						*/
/* description	: delete a list of BEAUX structures and all objects	*/
/*		  pointed by any os BEAUXs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_freberin (listberin)

struct syf_berin *listberin;	/* list of berin to be deleted		*/

  {
  struct syf_berin *ptberin;

	/* ###------------------------------------------------------### */
	/*    for each object of the list delete the object itself	*/
	/* ###------------------------------------------------------### */

  while (listberin != NULL)
    {
    ptberin   = listberin;
    listberin = listberin->NEXT;
    mbkfree (ptberin);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebetyp						*/
/* description	: delete a list of BETYP structures and all objects	*/
/*		  pointed by any os BETYPs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebetyp (listbetyp)

struct syf_betyp *listbetyp;	/* list of betyp to be deleted		*/

  {
  struct syf_betyp *ptbetyp;

	/* ###------------------------------------------------------### */
	/*    for each object of the list delete the object itself	*/
	/* ###------------------------------------------------------### */

  while (listbetyp != NULL)
    {
    ptbetyp   = listbetyp;
    listbetyp = listbetyp->NEXT;
    mbkfree (ptbetyp);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebiabl						*/
/* description	: delete a list of BIABL structures and all objects	*/
/*		  pointed by any os BIABLs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebiabl (listbiabl)

struct biabl *listbiabl;	/* list of biabl to be deleted		*/

  {
  struct biabl *ptbiabl;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbiabl != NULL)
    {
    freeExpr (listbiabl->VALABL);
    freeExpr (listbiabl->CNDABL);

    ptbiabl   = listbiabl;
    listbiabl = listbiabl->NEXT;
    mbkfree (ptbiabl);
    }

  }

/* ###--------------------------------------------------------------### */
/* function	: syf_beh_frebinode						*/
/* description	: delete a list of BINODE structures and all objects	*/
/*		  pointed by any os BINODEs in the list			*/
/* called func.	: mbkfree						*/
/* ###--------------------------------------------------------------### */

void syf_beh_frebinode (listbinode)

struct binode *listbinode;	/* list of binode to be deleted		*/

  {
  struct binode *ptbinode;

	/* ###------------------------------------------------------### */
	/*    for each object of the list, first delete pointed objects	*/
	/* then, delete the object itself				*/
	/* ###------------------------------------------------------### */

  while (listbinode != NULL)
    {
    ptbinode   = listbinode;
    listbinode = listbinode->NEXT;
    mbkfree (ptbinode);
    }

  }
