#include ELP_H
extern FILE *yyin                                                      ;

char   elpTechnoName[elpSTRINGSIZE]                                    ;
double elpTechnoVersion                                                ;
char   elpEsimName[elpSTRINGSIZE]                                      ;
double elpEsimLevel                                                    ;
double elpShrink[elpSHRINKNUM]             = {1.0,1.0,0.0,0.0,0.0,0.0} ;
double elpModel[elpTRANSNUM][elpMODELNUM]                              ;
double elpVoltage[elpVOLTNUM]                                          ;
double elpCapa[elpTRANSNUM][elpCAPANUM]                                ;
double elpGeneral[elpGENERALNUM]           = {0.0,0.0,1000.0}          ;
char   elpLang                             = elpDEFLANG                ;
int    yylineno                                                        ;


/*****************************************************************************/
/*                        function LoadElp()                                 */
/* chargement du fichier techno sous forme structure ELP. En cas d'erreur la */
/* fonction affiche un message et renvoie le numero d'erreur. Sinon le code  */
/* de retour est 0.                                                          */
/*                                                                           */
/* Parametres en entree:                                                     */
/* --------------------                                                      */
/*    1) filename: nom du fichier a charger AVEC EXTENSION. C'EST LE CHEMIN  */
/*                 COMPLET QU'IL FAUT DONNER. PAS DE CATA_LIB ni WORK_LIB.   */
/*                                                                           */
/* Parametre en sortie:                                                      */
/* -------------------                                                       */
/*    La fonction renvoie 0 si le chargement s'est bien effectue, le numero  */
/*    d'erreur sinon.                                                        */
/*****************************************************************************/
elpFCT short elpLoadElp(filename)
char *filename ; /* nom du fichier avec extension, avec chemin complet */

{
/* ouverture du fichier techno */
if((yyin = fopen(filename,"r")) == NULL)
    {
    elpError(1000,filename) ; /* si fichier pas ouvert => erreur 1000 */
    return 1000 ; /* arret de la procedure elp */
    }

yylineno = 1 ;

/* appel au parser pour chargement */
if(yyparse() != 0)
    {
    elpError(1002,filename) ; /* si erreur chergement => erreur 1002 */
    return 1002 ; /* arret de la procedure elp */
    }

/* fermeture du fichier techno */
if(fclose(yyin) != 0)
elpError(1003,filename) ; /* si fermeture ko => erreur 1003 */

/*-------------------------------------------------------------------*/
/* verification des parametres obligatoires et calcul des parametres */
/* optionnels.                                                       */
/*-------------------------------------------------------------------*/
if( /* verification des parametres obligatoires */
   (elpModel[elpNMOS][elpVT] == 0.0 ) ||
   (elpModel[elpNMOS][elpA]  == 0.0 ) ||
   (elpModel[elpNMOS][elpB]  == 0.0 ) ||
   (elpModel[elpNMOS][elpRT] == 0.0 ) ||
   (elpModel[elpPMOS][elpVT] == 0.0 ) ||
   (elpModel[elpPMOS][elpA]  == 0.0 ) ||
   (elpModel[elpPMOS][elpB]  == 0.0 ) ||
   (elpModel[elpPMOS][elpRT] == 0.0 ) ||
   (elpVoltage[elpVDDMAX]    == 0.0 ) ||
   (elpCapa[elpNMOS][elpCGS] == 0.0 ) ||
   (elpCapa[elpNMOS][elpCGP] == 0.0 ) ||
   (elpCapa[elpPMOS][elpCGS] == 0.0 ) ||
   (elpCapa[elpPMOS][elpCGP] == 0.0 ) ||
   (elpEsimName[0]           == '\0') ||
   (elpEsimLevel             == 0.0 ) ||
   (elpGeneral[elpTEMP]      == 0.0 )
  )
    {
    elpError(1004,filename) ; /* parametre manquant => erreur 1004 */
    return 1004 ; /* arret de la procedure elp */
    }

/* calcul des parametres optionnels ne se trouvant pas dans le fichier tec */
/*-------------------------------------------------------------------------*/
/* les niveaux degrades par defaut: VSSdeg = 1.5*VTP                       */
/*                                  VDDdeg = VDDmax-1.5*VTN                */
/* tension de seuil de calcul     : VTHR   = VDDmax / 2                    */
/*-------------------------------------------------------------------------*/
if(elpVoltage[elpVSSDEG] == 0.0)
elpVoltage[elpVSSDEG] = 1.5*elpModel[elpPMOS][elpVT] ;

if(elpVoltage[elpVDDDEG] == 0.0)
elpVoltage[elpVDDDEG] = elpVoltage[elpVDDMAX] - 1.5*elpModel[elpNMOS][elpVT] ;

if(elpVoltage[elpVTHR] == 0.0)
elpVoltage[elpVTHR] = elpVoltage[elpVDDMAX] / 2 ;

return 0 ; /* retourne 0 si tout s'est bien passe */
}


/****************************************************************************/
/*                           fonction LotrsShrink()                         */
/* modifie les dimensions du transistor en fonction des parametres de       */
/* shrink et du parametre ACM.                                              */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lotrs: transistor logique MBK dont on veut modifier les dimensions */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    Aucun!                                                                */
/****************************************************************************/
elpFCT int elpLotrsShrink(lotrs)
struct lotrs *lotrs ;

{
double deltal ,
       deltaw ;

if(lotrs->TYPE == TRANSN)
    {
    deltal = elpShrink[elpDLN] * (double)SCALE_X ;
    deltaw = elpShrink[elpDWN] * (double)SCALE_X ;
    }
else
    {
    deltal = elpShrink[elpDLP] * (double)SCALE_X ;
    deltaw = elpShrink[elpDWP] * (double)SCALE_X ;
    }

/* modification de la longueur */
lotrs->LENGTH = (long)(elpShrink[elpLMLT]*(double)lotrs->LENGTH + deltal) ;

/* modification de la largeur */
lotrs->WIDTH = (long)(elpShrink[elpWMLT]*(double)lotrs->WIDTH +deltaw) ;

/*--------------------------------------------------------------------------*/
/* si la largeur est modifiee, il faut egalement modifier les largeurs      */
/* laterales XS et XD pour la multiplication de XS ou de XD par la largeur  */
/* ne bouge pas. En effet cette multiplication donne l'aire du drain ou de  */
/* la source qui n'est absolument pas affectee par les parametres de shrink.*/
/*--------------------------------------------------------------------------*/
lotrs->XS = (long)((elpShrink[elpWMLT] + deltaw/lotrs->WIDTH) * lotrs->XS) ;
lotrs->XD = (long)((elpShrink[elpWMLT] + deltaw/lotrs->WIDTH) * lotrs->XD) ;

/*--------------------------------------------------------------------------*/
/* pour ce qui est de l'aire et du perimetre du transistor. Si le parametre */
/* ACM est a 1, l'aire et le perimetre du transistor sont calcules unique-  */
/* ment en fonction de la largeur (quelles que soient leurs valeurs initi-  */
/* ales. Dans ce cas:                                                       */
/*                                                                          */
/*    Aire      = Weffectif*WMLT                                            */
/*    Perimetre = Weffectif                                                 */
/*                                                                          */
/* Or ce qu'on a dans le transistor logique, ce n'est pas la surface, mais  */
/* la largeur laterale X, telle que Aire = X*Weffectif. La transformation   */
/* dans le cas ACM=1 est donc:                                              */
/*                                                                          */
/*    X         = WMLT*SCALE_X                                              */
/*    Perimetre = Weffectif                                                 */
/*--------------------------------------------------------------------------*/
if(elpGeneral[elpACM] == 1.0)
    {
    lotrs->XS = lotrs->XD = (long)(elpShrink[elpWMLT] * SCALE_X) ;
    lotrs->PS = lotrs->PD = lotrs->WIDTH ;
    }

return ;
}


/****************************************************************************/
/*                           fonction LotrsCapaDrain()                      */
/* calcule la capacite de drain d'un transistor MBK passe en parametre.     */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lotrs: transistor logique MBK dont on veut calculer la capacite de */
/*              drain.                                                      */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    la fonction renvoie la capcite du drain du transistor en picoF.       */
/****************************************************************************/
elpFCT float elpLotrsCapaDrain(lotrs)
struct lotrs *lotrs ;

{
short lotrstype ; /* variable indiquant le type du transistor a traiter */

/* si les parametres CDxx sont manquants => erreur 1005 */
if(
   (elpCapa[elpNMOS][elpCDS] == 0.0) &&
   (elpCapa[elpNMOS][elpCDP] == 0.0) &&
   (elpCapa[elpPMOS][elpCDS] == 0.0) &&
   (elpCapa[elpPMOS][elpCDP] == 0.0)
  )
    {
    elpError(1005,"drain") ;
    return 0.0 ;
    }

/*---------------------------------------------------------------------------*/
/* Methode de calcul des capacites: si le perimetre est manquant, la contri- */
/* bution en perimetre sera remplacee par la contribution en largeur. Sinon, */
/* seul le perimetre sera pris en compte. La variable 'lotrstype' contiendra */
/* le type du transistor qu'on traite.                                       */
/*---------------------------------------------------------------------------*/
lotrstype = (lotrs->TYPE == TRANSN) ? elpNMOS : elpPMOS ;

return
/* contribution en surface   */
(lotrs->XD*lotrs->WIDTH / (SCALE_X*SCALE_X)) * elpCapa[lotrstype][elpCDS] +
/* contribution en perimetre */
(lotrs->PD              /          SCALE_X ) * elpCapa[lotrstype][elpCDP] +
/* contribution en largeur   */
(lotrs->WIDTH          /           SCALE_X ) * elpCapa[lotrstype][elpCGP] ;
}


/****************************************************************************/
/*                           fonction LotrsCapaSource()                     */
/* calcule la capacite de source d'un transistor MBK passe en parametre.    */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lotrs: transistor logique MBK dont on veut calculer la capacite de */
/*              source.                                                     */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    la fonction renvoie la capcite de la source du transistor en picoF.   */
/****************************************************************************/
elpFCT float elpLotrsCapaSource(lotrs)
struct lotrs *lotrs ;

{
short lotrstype ; /* variable indiquant le type du transistor a traiter */

/* si les parametres CSxx sont manquants => erreur 1005 */
if(
   (elpCapa[elpNMOS][elpCSS] == 0.0) &&
   (elpCapa[elpNMOS][elpCSP] == 0.0) &&
   (elpCapa[elpPMOS][elpCSS] == 0.0) &&
   (elpCapa[elpPMOS][elpCSP] == 0.0)
  )
    {
    elpError(1005,"source") ;
    return 0.0 ;
    }

/*---------------------------------------------------------------------------*/
/* Methode de calcul des capacites: si le perimetre est manquant, la contri- */
/* bution en perimetre sera remplacee par la contribution en largeur. Sinon, */
/* seul le perimetre sera pris en compte. La variable 'lotrstype' contiendra */
/* le type du transistor qu'on traite.                                       */
/*---------------------------------------------------------------------------*/
lotrstype = (lotrs->TYPE == TRANSN) ? elpNMOS : elpPMOS ;

return
/* contribution en surface   */
(lotrs->XS*lotrs->WIDTH / (SCALE_X*SCALE_X)) * elpCapa[lotrstype][elpCSS] +
/* contribution en perimetre */
(lotrs->PS              /          SCALE_X ) * elpCapa[lotrstype][elpCSP] +
/* contribution en largeur   */
(lotrs->WIDTH          /           SCALE_X ) * elpCapa[lotrstype][elpCGP] ;
}


/****************************************************************************/
/*                           fonction LotrsCapaGrid()                       */
/* calcule la capacite de grille d'un transistor MBK passe en parametre.    */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lotrs: transistor logique MBK dont on veut calculer la capacite de */
/*              grille.                                                     */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    la fonction renvoie la capcite de la grille du transistor en picoF.   */
/****************************************************************************/
elpFCT float elpLotrsCapaGrid(lotrs)
struct lotrs *lotrs ;

{
/* variable indiquant le type du transistor */
short lotrstype = (lotrs->TYPE == TRANSN) ? elpNMOS : elpPMOS ;

/* les parametres CGxx sont obligatoires et ne sont donc pas manquant */

return
/* contribution en surface   */
(lotrs->WIDTH*lotrs->LENGTH / (SCALE_X*SCALE_X))*elpCapa[lotrstype][elpCGS] +
/* contribution en perimetre */
(2*lotrs->WIDTH             / SCALE_X          )*elpCapa[lotrstype][elpCGP] ;
}


/****************************************************************************/
/*                    fonction LofigCapaDiff()                              */
/* calcule les capas de diffusion des transistors de la figure lofig        */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lofig: figure logique MBK dont dans laquelle il faut ajouter les   */
/*              capas de diffusion.                                         */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    Aucun!                                                                */
/****************************************************************************/
elpFCT int elpLofigCapaDiff(lofig)
struct lofig*lofig ;

{
lotrs_list *lotrs ;

for(lotrs = lofig->LOTRS ; lotrs != NULL ; lotrs = lotrs->NEXT)
if((lotrs->PS != 0.0) && (lotrs->XS != 0.0))
    {
    lotrs->DRAIN->SIG->CAPA  += elpLotrsCapaDrain(lotrs)  ;
    lotrs->SOURCE->SIG->CAPA += elpLotrsCapaSource(lotrs) ;
    }

return ;
}


/****************************************************************************/
/*                    fonction LofigCapaGrid()                              */
/* calcule les capas de grille.                                             */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lofig: figure logique MBK dont dans laquelle il faut ajouter les   */
/*              capas de grille.                                            */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    Aucun!                                                                */
/****************************************************************************/
elpFCT int elpLofigCapaGrid(lofig)
struct lofig *lofig ;

{
lotrs_list *lotrs  ;

for(lotrs = lofig->LOTRS ; lotrs != NULL ; lotrs = lotrs->NEXT)
lotrs->GRID->SIG->CAPA += elpLotrsCapaGrid(lotrs) ;

return ;
}


/****************************************************************************/
/*                        fonction LofigShrink()                            */
/* modifie les dimensions de tous les transistors en fonction des parametres*/
/* de shrink.                                                               */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) lofig: figure logique MBK dont dans laquelle il faut modifier les  */
/*              dimensions des transistors.                                 */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    Aucun!                                                                */
/****************************************************************************/
elpFCT int elpLofigShrink(lofig)
struct lofig *lofig ;

{
lotrs_list *lotrs  ;


for(lotrs = lofig->LOTRS ; lotrs != NULL ; lotrs = lotrs->NEXT)
elpLotrsShrink(lotrs) ;

return ;
}
