/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.50                                            */
/*    Fichier : pre.c                                                       */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Amjad HAJJAR et Payam KIANI           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 26/06/1991     */
/*    Modifie par : Payam KIANI                         le : 16/10/1991     */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*                                                                          */
/****************************************************************************/
/* fichier pre.c : les pre-traitements avant le calcul des temps (les       */
/*                 switch-cmos ou chemins paralleles, etc...)               */
/****************************************************************************/
#include "tas_all.h"


/****************************************************************************/
/*                           fonction test_p()                              */
/* recherche la chaine '_p' dans le nom passe en parametre. Elle sert a     */
/* detecter la precharge. Renvoie 1 si la chaine existe, 0 sinon.           */
/****************************************************************************/
short tas_test_p(name)
char *name ;

{
long i = strlen(name) - 1 ;

if(i == 0)
return(0) ;

if(name[i] == ']')
    {
    while((i != 0) && (name[i--] == ']'))
    while((i != 0) && (name[i--] != '[')) ;

    i++ ;
    }

if((i > 1) && (name[i] == 'p') && (name[i-1] == '_'))
i = 0 ;

return(!i) ;
}


/****************************************************************************/
/*                           fonction CorrectHZType()                       */
/* pour un signal precharge, cette fonction enleve l'attribut HZ du cone et */
/* du connecteur eventuellement associe.                                    */
/****************************************************************************/
int tas_CorrectHZType(cone)
cone_list *cone ;

{
locon_list *locon = (locon_list *)getptype(cone->USER,EXT) ;

cone->TYPE &= ~HZ ;
if(locon != NULL)
    {
    locon = (locon_list *)((ptype_list *)locon)->DATA ;

    if(locon->DIRECTION == 'T')
    locon->DIRECTION = 'B' ;
    if(locon->DIRECTION == 'Z')
    locon->DIRECTION = 'O' ;
    }

return ;
}


/****************************************************************************/
/*                           fonction TreatInfSig()                         */
/* type le cone s'il appartient a la liste des collages.                    */
/****************************************************************************/
int tas_TreatInfSig(cone)
cone_list *cone ;

{
list_list   *list        ;
static char flag   = 'N' ;

for(list = INF_SIGLIST ; list != NULL ; list = list->NEXT)
if((list->TYPE != 0) && (list->TYPE != 1))
if(list->DATA == cone->NAME)
    {
    /* 0 et 1 collage => traite plus loin => ici ignores */
    if(list->TYPE == 2) /* signal precharge */
        {
        cone->TYPE |= PRECHARGE ; list->DATA = NULL ;
        tas_CorrectHZType(cone) ;
        if((PRFTYPE != 'N') && (flag == 'N'))
            {
            tas_error(41,NULL,TAS_WARNING) ;
            PRFTYPE = 'N' ;
            flag = 'Y' ;
            }
        }
    if(list->TYPE == 3) /* signal non-precharge */
        {
        cone->TYPE |= tas_NOPRECH ;
        cone->TYPE &= ~PRECHARGE ;
        }
    if(list->TYPE == 4) /* noeud dont les chemins entrants sont ignores */
    cone->TYPE |= tas_BYPASSIN ;
    if(list->TYPE == 5) /* noeud dont les chemins sortants sont ignores */
    cone->TYPE |= tas_BYPASSOUT ;
    if(list->TYPE == 6) /* noeud dont tous les chemins sont ignores */
    cone->TYPE |= tas_BYPASSIN | tas_BYPASSOUT ;

    list->DATA = NULL ;
    }

return ;
}


/****************************************************************************/
/*                           fonction init_timing()                         */
/* initialisiation pour la phase de calcul des temps elementaires           */
/****************************************************************************/
int init_timing(ptfig,inflist)
desafig_list *ptfig   ;
list_list    *inflist ;

{
cone_list *cone ;
list_list *list ;

/*-------------------------------------------------------------------------*/
/* tout d'abord on est en manque de champ pour les maillons. On prend donc */
/* le champ USER pour y coller un ptype_list, ce qui nous donnera trois    */
/* champs supplementaires par maillon. Dans ce dernier champ, le TYPE est  */
/* en long. On l'utilisera pour la largeur equivalente du transistor. Le   */
/* champ DATA sera pris pour la capacite equivalente du maillon (c'est le  */
/* champ CAPARA qui avait ce role jusqu'a aujourd'hui, mais il va etre eli-*/
/* mine). Enfin le champ NEXT sera utilise pour ranger la capacite ramenee */
/* vue par le maillon. C'est TAS qui le rempliera.                         */
/* Evidemment tous ces nouveaux champs seront initialises a NULL ou 0.     */
/* Si l'option precharge n'est pas active, tous les cones types precharges */
/* seront detypes pour etre consideres comme des cones combinatoires.      */
/*-------------------------------------------------------------------------*/
for(cone = ptfig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    list_list  *path         ;
    locon_list *locon = NULL ;

    /* revectorisation du nom de cone */
    cone->NAME = tas_ReVect(cone->NAME) ;

    /* si option desactive et cone precharge => detypage */
    if(TAS_TREATPRECH == 'N')
    cone->TYPE &= ~PRECHARGE ;

    /*-------------------------------------------------------------------*/
    /* traitement du cone suivant le fichier '.inf': si le cone est dans */
    /* la liste des signaux du fichier, on fait le traitement adequat:   */
    /*         type = 0 => collage a 0                                   */
    /*         type = 1 => collage a 1                                   */
    /*         type = 2 => signal precharge                              */
    /*         type = 3 => signal non precharge                          */
    /*         type = 4 => signal insignifiant                           */
    /*-------------------------------------------------------------------*/
    tas_TreatInfSig(cone) ;

    /* sinon on cherche le '_p' */
    if(TAS_TREATPRECH != 'N')
    if((tas_test_p(cone->NAME,'p') == 1) &&
            ((cone->TYPE & tas_NOPRECH) != tas_NOPRECH))
        {
        cone->TYPE |= PRECHARGE ;
        tas_CorrectHZType(cone) ;
        }

#ifdef BULL
    if((cone->TYPE & PRECHARGE) == PRECHARGE)
        { /* debut BULL */
        list_list *auxpath = NULL ;

        /*-------------------------------------------------------------*/
        /* pour le cone precharge, il faut en plus enlever les fausses */
        /* branches. Pour cette version et pour BULL, les fausses      */
        /* branches seront celles ayant des transistors de type diffe- */
        /* rents et ayant comme premier transistor un NMOS.            */
        /*-------------------------------------------------------------*/
        for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        if((path->TYPE & (BLEEDER | VDD)) == VDD)
            {
            link_list *link = (link_list *)path->DATA ;

            if((link->TYPE & TN) == TN)
                {
                link_list *link ;
    
                for(link = (link_list *)path->DATA ;
                    (link != NULL) && ((link->TYPE & TP) != TP) ;
                    link = link->NEXT) ;
    
                if(link != NULL)
                path->TYPE |= BLEEDER ;
                }
            /* on recherche s'il y a precharge avec de TP */
            else if((link->TYPE & TP) == TP)
            auxpath = path ;
            } /* fin du parcours des branches */

        /*-----------------------------------------------------*/
        /* on reparcourt les branches pour eliminer celles qui */
        /* prechargent avec de NMOS, alors qu'il y a precharge */
        /* avec des PMOS.                                      */
        /*-----------------------------------------------------*/
        if(auxpath != NULL)
        for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        if((path->TYPE & (BLEEDER | VDD)) == VDD)
        if((((link_list *)path->DATA)->TYPE & TN) == TN)
        path->TYPE |= BLEEDER ;

        } /* fin BULL */
#endif

    /* determination du connecteur de sortie eventuel */
    if((cone->TYPE & (EXT | CONE_CONNECT)) == EXT)
        {
        ptype_list *ptype = getptype(cone->USER,EXT) ;

        if(ptype != NULL)
            {
            list_list  *incone ;

            locon = (locon_list *)ptype->DATA ;

            /*-------------------------------------------------------------*/
            /* si un cone est monte sur un connecteur INOUT, ce connecteur */
            /* apparait en entree et en sortie de ce cone. On le type donc */
            /* en tant qu'entree pour ne pas le prendre en compte.         */
            /*-------------------------------------------------------------*/
            for(incone = cone->INCONE ; incone != NULL ; incone = incone->NEXT)
            if(incone->DATA == (char *)locon)
                {
                incone->TYPE |= PATH_INOUT ;
                break ;
                }
            } 
        }

    for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        {
        link_list *link = (link_list *)path->DATA ;

        /*--------------------------------------------------------------------*/
        /* si un connecteur INOUT est en entree et en sortie de cone, il doit */
        /* etre ignore (meme chose que pour INCONE un peu plus haut).         */
        /*--------------------------------------------------------------------*/
        if(link->TRANS == (lotrs_list *)locon)
        if(locon->DIRECTION != 'I')
        path->TYPE |= PATH_INOUT ;

        for( /* initialise */ ; link != NULL ; link = link->NEXT)
        /* initialisation pour capa ramenees, ... */
        link->USER = addptype(NULL,0,NULL) ;
        }
    }

/* recherche si toute la liste sauf les collages a ete traitee */
for(list = INF_SIGLIST ; list != NULL ; list = list->NEXT)
if((list->DATA != NULL) && (list->TYPE != 0) && (list->TYPE != 1))
    {
    tas_error(9,list->DATA,TAS_WARNING) ;
    list->DATA = NULL ;
    }

return ;
}


/****************************************************************************/
/*                           fonction tas_capara()                          */
/* calcule la capacite ramenee des maillons qui en voient                   */
/****************************************************************************/
int tas_capara(ptfig)
desafig_list *ptfig ;

{
cone_list  *cone  ;
locon_list *locon ;

for(locon = ptfig->LOCON ; locon != NULL ; locon = locon->NEXT)
    {
    locon->SIG = NULL ;
    locon->ROOT = NULL ;
    locon->TYPE = '\0' ;
    }

for(cone = ptfig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    list_list *incone ;
    list_list *path   ;

    for(path = cone->PATH ; path != NULL ; path = path->NEXT)
    if((path->TYPE & BLEEDER) == 0)
        {
        link_list *link ;

        /* on tag les transistors et les connecteurs du cone */
        for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
            {
            /* cas du connecteur: champ ROOT pour tager */
            if((link->TYPE & (IN | INOUT)) != 0)
            ((locon_list *)link->TRANS)->ROOT = (char *)cone ;
            else /* cas du transistor: champ SOURCE pour tager */
            link->TRANS->SOURCE = (locon_list *)cone ;
            }
        }

    /*------------------------------------------------------------------------*/
    /* l'idee c'est, une fois les transistors tages, parcourir la liste des   */
    /* sorties des entrees du cone courant. On obtient ainsi la liste des     */
    /* cones susceptibles d'avoir des transistors en commun avec le cone      */
    /* courant, ce qui est source de capacite ramenee. On parcourt alors pour */
    /* chacune de ces sorties, sa liste de transistors. Si a un moment donne  */
    /* on arrive sur des transistors tages, c'est qu'on en a en commun. Alors */
    /* il suffit de ramener les capacites des maillons non-tages sur celui    */
    /* qui l'est.                                                             */
    /*------------------------------------------------------------------------*/
    for(incone = cone->INCONE ; incone != NULL ; incone = incone->NEXT)
    if((incone->TYPE & (LOOP | BLEEDER | EXT)) == 0)
        {
        list_list *outcone ;

        /*----------------------------------------------------------------*/
        /* on cherche les capacites ramenees parmi les autres cones autre */
        /* que celui etudie et qui n'est pas un connecteur                */
        /*----------------------------------------------------------------*/
        for(outcone = ((cone_list *)incone->DATA)->OUTCONE ; outcone != NULL ;
            outcone = outcone->NEXT)
        if(((outcone->TYPE & (LOOP | BLEEDER | EXT)) == 0) &&
           (outcone->DATA != (char *)cone))
            {
            /* recherche des transistors en commun */
            for(path = ((cone_list *)outcone->DATA)->PATH ; path != NULL ;
                path = path->NEXT)
            if((path->TYPE & BLEEDER) == 0)
                {
                link_list *link           ,
                          *auxlink = NULL ;

                for(link = (link_list *)path->DATA ; link != NULL ;
                    link = link->NEXT)
                    {
                    /* si on tombe sur un connecteur tage */
                    if((link->TYPE & (IN | INOUT)) != 0)
                        {
                        if(((locon_list *)link->TRANS)->ROOT == (char *)cone)
                            {
                            if(auxlink != NULL)
                                {
                                locon = (locon_list *)link->TRANS ;
                                if(locon->TYPE != tas_TAG)
                                locon->SIG = (losig_list *)
                                             ((long)locon->SIG     +
                                              (long)auxlink->CAPA) ;
                                }
                            break ;
                            }
                        }
                    /* si transistor tage */
                    else if(link->TRANS->SOURCE == (locon_list *)cone)
                        {
                        if(auxlink != NULL)
                            {
                            if(link->TRANS->Y != (long)tas_TAG)
                            if((link->TRANS->DRAIN == NULL) ||
                               (link->TRANS->DRAIN == (locon_list *)outcone))
                                {
                                link->TRANS->X += (long)auxlink->CAPA ;
                                link->TRANS->DRAIN = (locon_list *)outcone ;
                                }
                            }
                        break ;
                        }

                    auxlink = link ;
                    }
                } /* fin de parcours des branches des cones de sortie */
            } /* fin de parcours des sorties */
        } /* fin de parcours des entrees */

    /* on ramene la capa ramenee sur le maillon du cone */
    for(path = cone->PATH ; path != NULL ; path = path->NEXT)
    if((path->TYPE & BLEEDER) == 0)
        {
        link_list *link ;

        for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
            {
            if((link->TYPE & (IN | INOUT)) == 0)
                {
                long LinkCapa = (long)(CAPARAPREC * (double)link->TRANS->X) ;

                link->USER->NEXT = (ptype_list *)LinkCapa ;
                /* on type le transistor pour ne pas le reprendre en compte */
                link->TRANS->Y = (long)tas_TAG ;
                }
            else
                {
                long LinkCapa = (long)(CAPARAPREC *
                (double)((long)((locon_list *)link->TRANS)->SIG)) ;

                link->USER->NEXT = (ptype_list *)LinkCapa ;
                ((locon_list *)link->TRANS)->TYPE = tas_TAG ;
                }
            }
        }
    } /* fin de parcours des cones */

return ;
}


/****************************************************************************/
/*                           fonction DetectConeConnect()                   */
/* type CONE_CONNECT les cones connecteurs                                  */
/****************************************************************************/
int tas_DetectConeConnect(desbfig)
desafig_list *desbfig ;

{
locon_list *locon ;

/*--------------------------------------------------------------------------*/
/* un cone connecteur est un cone qui a en entree un connecteur de direct.  */
/* 'I' mais qui a dans son champ USER un pointeur qui pointe dessus avec le */
/* mot clef EXT.                                                            */
/*--------------------------------------------------------------------------*/
for(locon = desbfig->LOCON ; locon != NULL ; locon = locon->NEXT)
    {
    locon->NAME = tas_ReVect(locon->NAME) ;

    if(locon->DIRECTION == 'I')
        {
        cone_list *cone = (cone_list *)getptype(locon->USER,EXT)->DATA ;

        if(cone != NULL) cone->TYPE |= CONE_CONNECT ;
        }
    }

return ;
}


/****************************************************************************/
/*                           fonction prelink()                             */
/* range les dimensions du transistor du maillon dans le champ USER du      */
/* maillon. La structure utilisee est la structure bilong qui est la meme   */
/* pour FUP et FDOWN. Retourne le nombre de maillons traites.               */
/****************************************************************************/
long prelink(ptfig)
desafig_list *ptfig ;

{
cone_list *cone ;
lotrs_list *lotrs ;

init_timing(ptfig) ;

/*--------------------------------------------------------------------------*/
/* on commence par initialiser les champs des transistors et connecteurs. Le*/
/* champ   X   du transistor et SIG du connecteur vont servir pour ranger le*/
/* capacite ramenee calculee par TAS. Le champ SOURCE du transistor et ROOT */
/* du connecteur vont servir pour tager le maillon.                         */
/* Pour tager un element, on utilise l'adresse du cone courant, car, comme  */
/* on le verra, ceci nous permettra de ne pas a chaque fois les reinitiliser*/
/*--------------------------------------------------------------------------*/
for(lotrs = ptfig->TRANS ; lotrs != NULL ; lotrs = lotrs->NEXT)
    {
    /* initialisation des champs */
    lotrs->DRAIN = lotrs->SOURCE = NULL ;
    lotrs->X = lotrs->Y = 0 ;
    TAS_INFO.nb_trans ++ ;
    }

/* pour tous les cones de la figure */
for(cone = ptfig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    list_list *path ;

    TAS_INFO.nb_cone ++ ;
    /* pour toutes les banches du cone */
    for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        {
        link_list *link ;

        /* pour tous les maillons de la branche */
        for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
            {
            /*--------------------------------------------------------------*/
            /* Les dimensions du transistor correspondant au maillon dev-   */
            /* raient etre rangees dans le maillon car pour le Switches et  */
            /* branches paralleles, les tailles des transistors doivent etre*/
            /* modifiees. Mais un transistor appartient a plusieurs elements*/
            /* donc risque d'erreur si les dimensions sont modifiees direc- */
            /* tement dans la structure lotrs (si pour un maillon elles     */
            /* doivent etre modifiees, il se peut que ca ne soit pas vrai   */
            /* pour un autre maillon). Le probleme c'est qu'il n'y a pas de */
            /* champs libres pour les dimensions dans link_list: on ne dis- */
            /* pose que du champ USER. L'idee est donc de ne modifier qu'une*/
            /* seule  dimension, en cherchant l'autre directement sur le    */
            /* transistor. La dimension fixe choisie est la longueur. La    */
            /* largeur de chaque transistor est donc porte dans le champ    */
            /* USER du maillon, et s'il faut modifier les dimensions, se    */
            /* sera ce champ la qui, cette fois, est unique.                */
            /*--------------------------------------------------------------*/
            if((link->TYPE & (IN | INOUT)) == 0) /* test si maillon connect.*/
            link->USER->TYPE = link->TRANS->WIDTH ;
            } /* fin des maillons */
        } /* fin des branches */
    } /* fin des cones */

return ;
}


/*****************************************************************************
*                              function  delpath()                           *
*                           destruction d'un chemin                          *
*****************************************************************************/
int delpath(cone,path)
cone_list *cone ; /* cone ou il y a la branche */
list_list *path ; /* branche a detruire        */

{
link_list *pt_nextlink = NULL ;
int       resultat = 0 ;

/* on detruit la branche */
if(path == cone->PATH) /* si c'est la premiere */
    {
    cone->PATH = path->NEXT ;
    resultat = 1 ;
    }
else 
    {
    list_list *pt_path ;

    for(pt_path = cone->PATH ; pt_path != NULL ; pt_path = pt_path->NEXT)
        { /* pour chaque chemin */
        if(pt_path->NEXT == path) 
            {
            pt_path->NEXT = path->NEXT ;
            resultat = 1 ;
            break ;
            }
        }
    }

return( resultat ) ;
}
        

/******************************************************************************
*                               del_para()                                    *
* elimine les branches paralleles dans un cone                                *
******************************************************************************/
int del_para(cone,chaine1,chaine2)
cone_list *cone ;
chain_list *chaine1 ;
chain_list *chaine2 ;

{
list_list *chemin2;
link_list *maillon1,*maillon2;
trans_list *trans1,*trans2;
char transfor='n';

chemin2=(list_list *)chaine2->DATA;
maillon1=(link_list *)((list_list *)chaine1->DATA)->DATA;
maillon2=(link_list *)chemin2->DATA;

if((maillon1->TYPE & (IN | INOUT)) == 0)
/* on ne calcule pas une largeur equivalente entre deux memes transistors */
if ((trans1=maillon1->TRANS)!=(trans2=maillon2->TRANS))
    {
    double largeur = (double)maillon1->USER->TYPE ;

    largeur += (double)maillon2->USER->TYPE*((double)trans1->LENGTH/
                                             (double)trans2->LENGTH);
    maillon1->USER->TYPE = (long)largeur ;
    transfor='o';
    }

maillon1=maillon1->NEXT;
maillon2=maillon2->NEXT;

while((maillon1 != NULL) && ((maillon1->TYPE & (IN | INOUT))== 0))
    {
    if((trans1=maillon1->TRANS)!=(trans2=maillon2->TRANS))
        {
        double largeur = (double)maillon1->USER->TYPE ;

        largeur += (double)maillon2->USER->TYPE*((double)trans1->LENGTH/
                                                 (double)trans2->LENGTH);
        maillon1->USER->TYPE = (long)largeur ;

        if (transfor=='o' )
            {
            maillon1->CAPA += maillon2->CAPA;
            transfor='o';
            }
        }
    else transfor ='n';

    maillon1=maillon1->NEXT;
    maillon2=maillon2->NEXT;
    }

delpath(cone,chemin2);
}

/******************************************************************************
*                               number_maillon()                              *
******************************************************************************/
long number_maillon(chemin0) 
list_list *chemin0;

{
link_list *maillon0;
long num=0L;

for(maillon0=(link_list *)chemin0->DATA;maillon0;maillon0=maillon0->NEXT) num++;

if(TAS_TRACE_MODE == 'Y') printf("--- tas --- number_maill      :%d\n",num);   

return(num);
}


/******************************************************************************
*                               para()                                        *
******************************************************************************/
int para(cone0)
cone_list *cone0;

{
chain_list *chaine0,*chaine1,*chaine2,*chaine_active;  
chain_list *chbl = (chain_list *)getptype(cone0,BLEEDER) ;
list_list  *chemin1;
long       a=0L,min=0L;

if(TAS_TRACE_MODE == 'Y' ) 
printf ("--- tas --- para      : cone(%d)\n",cone0->INDEX);

/*****************************************************************************/
/* dans la liste des branches paralleles, il y a des branches non fonction-  */
/* nelles: en fait soit elles sont toutes fonctionnelles, soit aucune. On    */
/* traite alors seulement la premiere branche. Si elle est fonctionnelle on  */
/* fait le traitement sinon on sort de la premiere boucle. D'autre part ce   */
/* n'est pas la peine de tester si getptype retourne NULL ou non: cette      */
/* fonction n'est appelee que si le cone a des branches paralleles.          */
/*****************************************************************************/

for(chaine0 = (chain_list *)getptype(cone0->USER,PATH_PARA)->DATA ;
    chaine0 != NULL ; chaine0 = chaine0->NEXT)
    {
    chain_list *auxbl = chbl ;

    chaine1=(chain_list *)chaine0->DATA;
    chaine2=chaine1;

    /* initialisation */
    chemin1=(list_list *) chaine2->DATA;
    
    /* recherche de la fonctionnalite de la branche */ /* modif. du 26/06/91 */
    for( /* initialise */ ; (auxbl != NULL) &&
        ((list_list *)auxbl->DATA != chemin1) ; auxbl = auxbl->NEXT) ;

    if(((chemin1->TYPE & BLEEDER) == BLEEDER) && (auxbl == NULL)) continue ;

    min=number_maillon(chemin1);
    chemin1->USER=(ptype_list *) min;
    chaine_active=chaine2;
    chaine2=chaine2->NEXT;                

    /* recherche du chemin posedant le moins de maillons*/
    while(chaine2 != NULL)
        {
        chemin1=(list_list *)chaine2->DATA;  
        a=number_maillon(chemin1);
        chemin1->USER=(ptype_list *)a;

        if(a < min) 
            {
            min=a;
            chaine_active=chaine2;
            }

        chaine2=chaine2->NEXT;
        }

    chaine2=chaine1;

    while(chaine2 != NULL)
        {
        if(chaine2==chaine_active) 
            {
            chaine2=chaine2->NEXT;
            continue;
            }
        chemin1=(list_list *)chaine2->DATA;
        a=(long) (chemin1->USER); 
        if(a==min) del_para(cone0,chaine_active,chaine2);
        chaine2=chaine2->NEXT;
        }
    }
}

                  
/******************************************************************************
*                               switch_cmos()                                 *
******************************************************************************/
int switch_cmos(cone0)
cone_list *cone0;

{
list_list *chemin0,*chemin1;
link_list *maillon0;
char change='n'; 

if (TAS_TRACE_MODE=='Y')
printf("--- tas --- switch_cmos  : cone(%d)\n",cone0->INDEX);

chemin0=cone0->PATH;

while (chemin0 != NULL)
    {
    maillon0=(link_list *) chemin0->DATA;

    while(maillon0 != NULL)
        {
        if((maillon0->TYPE & SWITCH)==SWITCH) 
            {
            /* AJOUT DU 19/05/1994: traitement des switches en entree */
            if((chemin0->TYPE & EXT) == EXT)
            chemin0->TYPE |= SWITCH ;

            else if(((maillon0->TYPE & TP)==TP) && ((chemin0->TYPE &VSS)==VSS))
                {
                chemin1=chemin0;
                chemin0=chemin0->NEXT;        
                delpath(cone0,chemin1);
                change='o';
                break;
                }

            else if(((maillon0->TYPE &TN)==TN) &&((chemin0->TYPE &VDD)==VDD))
                {
                chemin1=chemin0;
                chemin0=chemin0->NEXT;        
                delpath(cone0,chemin1);
                change='o';
                break;
                }
            }
        maillon0=maillon0->NEXT;
        }

    if(change=='o') change='n';
    else chemin0=chemin0->NEXT;
    }
}


/*****************************************************************************/
/*                            fonction AddCapaOut()                          */
/* rajoute la capacite de sortie passee en option cout, sur tous les connec- */
/* teurs du bloc. Plus tard, apres le desassemblage, une fonction la ret-    */
/* ranche pour les connecteurs IN et UNKNOWN.                                */
/*****************************************************************************/
int tas_AddCapaOut(lofig)
lofig_list *lofig ;

{
locon_list *locon       ;
char       flag   = 'N' ;

for(locon = lofig->LOCON ; locon != NULL ; locon = locon->NEXT)
    {
    /*----------------------------------------------------------------*/
    /* si capacite nulle => alors l'extraction s'est mal passee =>    */
    /* on calucle la geometrie du transistor avec ACM=1 et on repasse */
    /* le calcul des capas de diff.                                   */
    /* On regarde seuls les connecteurs, car les connecteurs de sortie*/
    /* ne voient pas les capas de grille et donc sont les plus suscep-*/
    /* tibles a ne pas avoir de capa.                                 */
    /*----------------------------------------------------------------*/
    if(locon->SIG->CAPA < 0.001)
    flag = 'Y' ;

    locon->SIG->CAPA += TAS_CAPAOUT ;
    }

if(flag == 'Y')
    {
    /*---------------------------------------------------------------*/
    /* si les parametres n'existent pas, on ne peut rien calculer => */
    /* le probleme est juste signale. Sinon on signale et on calcule.*/
    /*---------------------------------------------------------------*/
    if(elpCapa[elpNMOS][elpCDS] == 0.0)
    tas_error(15,NULL,TAS_WARNING) ;
    else
        {
        struct lotrs *lotrs ;

        tas_error(16,NULL,TAS_WARNING) ;

        /* on modifie l'aire et le perimetre pour etre equivalent ACM=1 */
        for(lotrs = lofig->LOTRS ; lotrs != NULL ; lotrs = lotrs->NEXT)
            {
            lotrs->XS = lotrs->XD = elpShrink[elpWMLT]*SCALE_X ;
            lotrs->PS = lotrs->PD = lotrs->WIDTH ;
            }

        elpLofigCapaDiff(lofig) ;
        }
    }

return ;
}


/*****************************************************************************/
/*                          fonction DelCapaOut()                            */
/* retranche la capacite de sortie passee par cout, des connecteurs d'entree */
/* et inconnus (voir fonction AddCapaOut()).                                 */
/*****************************************************************************/
int tas_DelCapaOut(desbfig)
desafig_list *desbfig ;

{
locon_list *locon ;

for(locon = desbfig->LOCON ; locon != NULL ; locon = locon->NEXT)
    {
    if(locon->DIRECTION == 'I')
        {
        chain_list *chain = (chain_list *)getptype(locon->USER,CONE_TYPE) ;

        /* si pas de cone associe au connecteur => erreur */
        if(chain == NULL)
        tas_error(4,(char *)locon->NAME,TAS_ERROR) ;

        for(chain = (chain_list *)((ptype_list *)chain)->DATA ;
            chain != NULL ; chain = chain->NEXT)
            {
            cone_list *cone = (cone_list *)chain->DATA ;
            list_list *path                            ;
    
            for(path = cone->PATH ; path != NULL ; path = path->NEXT)
            if((path->TYPE & EXT) == EXT)
                {
                link_list *link ;

                /* recherche du maillon connecteur */
                for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
                    link = link->NEXT) ;
    
                /* si maillon non-connecteur => erreur */
                if((link->TYPE & (IN | INOUT)) == 0)
                tas_error(24,(char *)cone,TAS_ERROR) ;

                /* si le maillon rencontre est le connecteur => retrait capa */
                if(link->TRANS == (lotrs_list *)locon)
                link->CAPA -= TAS_CAPAOUT*1000.0 ;
                } /* fin de la liste des branches */
            } /* fin de la liste des chain */
        } /* fin du cas 'I' */
    } /* fin de la liste des connecteurs */

return ;
}


/*****************************************************************************/
/*                               fonction NewConeAlim()                      */
/* pour un cone alim donne, il type les entrees de ses sorties en VDD ou VSS.*/
/*****************************************************************************/
int tas_NewConeAlim(cone,level)
cone_list *cone ;
long      level ;

{
list_list *outcone                            ,
          *incone                             ;
long      type     = (level == 0) ? VSS : VDD ;

cone->TYPE |= type ;

for(outcone = cone->OUTCONE ; outcone != NULL ;
    outcone = outcone->NEXT)
if((outcone->TYPE & (EXT | LOOP)) == 0)
    {
    list_list *incone = ((cone_list * )outcone->DATA)->INCONE ;

    for( /* initialise */ ; (incone != NULL) && (incone->DATA != (char *)cone) ;
        incone = incone->NEXT) ;

    /* si entree pas trouvee => erreur */
    if(incone == NULL)
    tas_error(12,(char *)outcone->DATA,TAS_ERROR) ;

    incone->TYPE &= ~CONE_TYPE ;
    incone->TYPE |= type | LOOP ;
    } /* fin de parcours des sorties */

return ;
}


/*****************************************************************************/
/*                               fonction StuckAt()                          */
/* propage les collages du circuit, soit cables, soit donnes par l'utilisa-  */
/* teur.                                                                     */
/*****************************************************************************/
int tas_StuckAt(desbfig)
desafig_list *desbfig ;

{
list_list *list ;
cone_list *cone ;

/*--------------------------------------------------------------------*/
/* Pretraitement: si dans la liste donnee par l'utilisateur le nom    */
/* correspond a un connecteur, on traite tout de suite. Sinon on      */
/* remplace dans le champ DATA le nom par le cone associe.            */
/* Important: on suppose que si un signal a le meme nom qu'un locon,  */
/* il est forcement le signal associe a celui-ci.                     */
/*--------------------------------------------------------------------*/
for(list = INF_SIGLIST ; list != NULL ; list = list->NEXT)
if(list->DATA != NULL)
    {
    locon_list *locon ;

    for(locon = desbfig->LOCON ;
        (locon != NULL) && (locon->NAME != list->DATA) ;
        locon = locon->NEXT) ;

    if(locon != NULL)
        {
        chain_list *chain =
        (chain_list *)getptype(locon->USER,CONE_TYPE)->DATA ;

        /*--------------------------------------------------------------*/
        /* si chain = NULL, c'est un connecteur de sortie. Sa valeur ne */
        /* peut pas etre imposee => avertissement et passage au suivant.*/
        /*--------------------------------------------------------------*/
        if(chain == NULL)
        tas_error(7,(char *)locon->NAME,TAS_WARNING) ;
        else
        for( /* initialise */ ; chain != NULL ; chain = chain->NEXT)
            {
            list_list *path ;

            cone = (cone_list *)chain->DATA ;

            /* on recherche la branche ayant le connecteur en queue */
            for( path = cone->PATH ; path != NULL ; path = path->NEXT)
            if((path->TYPE & (BLEEDER | PATH_INOUT | EXT)) == EXT)
                {
                link_list *link    = (link_list *)path->DATA ,
                          *endlink = NULL                    ;
                short     typetr   = 0                       ;

                     if((link->TYPE & TN) == TN) typetr = 1 ;
                else if((link->TYPE & TP) == TP) typetr = 2 ;

                /* recherche du dernier maillon qui est un connecteur */
                for( /* initialise */ ; link->NEXT != NULL ;
                    link = link->NEXT)
                    {
                    /*-----------------------------------------------*/
                    /* typetr dit si dans une branche il n'y a que   */
                    /* des N, ou des P ou des deux. On peut comme    */
                    /* ca controler le caractere degrade d'une       */
                    /* branche si le connecteur a son bout est colle */
                    /*-----------------------------------------------*/
                    if(((typetr == 1) && ((link->TYPE & TP) == TP)) ||
                       ((typetr == 2) && ((link->TYPE & TN) == TN))   )
                    typetr = 0 ;

                    endlink = link ;
                    }

                /* si maillon pas connecteur => erreur */
                if((link->TYPE & (IN | INOUT)) == 0)
                tas_error(34,(char *)cone,TAS_ERROR) ;

                if(((locon_list *)link->TRANS)->NAME == (char *)list->DATA)
                    {
                    /*-----------------------------------------------*/
                    /* si le connecteur est le premier maillon de la */
                    /* liste, le cone, s'il n'a pas deja une valeur, */
                    /* prendra la valeur du connecteur, ainsi que les*/
                    /* INCONE des cones qu'il attaque. S'il y a deja */
                    /* une valeur, si elle est differente de celle   */
                    /* qui va etre imposee, il y aura warning.       */
                    /* Note: le type du connecteur devient inconnu,  */
                    /* s'il est colle. A priori donc, il faut ranger */
                    /* dans son champ USER, avec le mot CAPA_TSP, sa */
                    /* capacite, puisqu'il est alors comme un connect*/
                    /* de transparence. Or cette capa n'intervient   */
                    /* plus dans les temps de commutation, puisqu'   */
                    /* elle est sous tension d'alimentation, due au  */
                    /* collage du connecteur. Rien ne sera range ce  */
                    /* qui entraine la valeur nulle pour cette capa  */
                    /* dans les perfmodules.                         */
                    /*-----------------------------------------------*/
                    if(endlink == NULL)
                        {
                        /* cone pas ALIM */
                        if((cone->TYPE & (VDD | VSS)) == 0)
                            {
                            /* on type le cone et les incones */
                            tas_NewConeAlim(cone,list->TYPE) ;
                            locon->DIRECTION = UNKNOWN ;
                            }
                        else /* cone deja ALIM => on verifie son niveau */
                        if((cone->TYPE &
                           ((list->TYPE == 0) ? VSS : VDD)) == 0)
                            tas_error(7,cone->NAME,TAS_WARNING) ;
                        } /* fin du cas si connecteur et cone sur le meme sig */
                    else /* si connecteur pas sur le meme signal que le cone */
                        {
                        list_list *incone ;

                        endlink->NEXT = NULL ; /* on coupe le connecteur */
                        path->TYPE &= ~EXT ; /* on enleve l'attribut EXT */
                        locon->DIRECTION = UNKNOWN ;

                        if(list->TYPE == 0)
                            {
                            path->TYPE |= VSS ;
                            if(typetr == 1) path->TYPE &= ~TNINVDD ;
                            }
                        else
                            {
                            path->TYPE |= VDD ;
                            if(typetr == 2) path->TYPE &= ~TPINVSS ;
                            }

                        /* on type l'entree */
                        for(incone = cone->INCONE ; (incone != NULL) &&
                            (incone->DATA != (char *)locon) ;
                            incone = incone->NEXT) ;

                        if(incone == NULL)
                        tas_error(12,(char *)cone,TAS_ERROR) ;

                        incone->TYPE &= ~EXT ;
                        incone->TYPE |= (path->TYPE & (VDD | VSS)) | LOOP ;
                        } /* fin du cas connecteur et cone pas sur le mem sig */
                    } /* fin du traitement du maillon connecteur */
                } /* fin de parcours des branches externes */
            } /* fin de parcours des cones associes au connecteur */

        list->DATA = NULL ; /* element traite => on ne le traitera plus */
        } /* fin du traitement du cas connecteur trouve */
    else /* alors le signal correspond a priori a un cone */
        {
        for(cone = desbfig->CONE ;
            (cone != NULL) && (cone->NAME != (char *)list->DATA) ;
            cone = cone->NEXT) ;

        if(cone != NULL) /* cone trouve */
            {
            /* si le cone n'est pas deja une alim, on traite */
            if((cone->TYPE & (VDD | VSS)) == 0)
            list->DATA = (char *)cone ; 
            else /* sinon ca depend de son niveau */
                {
                if((cone->TYPE &
                   ((list->TYPE == 0) ? VSS : VDD)) == 0)
                tas_error(7,cone->NAME,TAS_WARNING) ;

                list->DATA = NULL ;
                }
            } /* fin du cas cone trouve */
        } /* fin du cas du cone */
    } /* fin du premier parcours de la liste des collages */

/*--------------------------------------------------------------------*/
/* si dans le circuit il y a des cones VDD ou VSS, on les empile pour */
/* propager le collage.                                               */
/*--------------------------------------------------------------------*/
for(cone = desbfig->CONE ; cone != NULL ; cone = cone->NEXT)
if((cone->TYPE & (VDD | VSS)) != 0)
infAddList((void *)cone,((cone->TYPE & VSS) == 0) ? 0 : 1,'t') ;

/*--------------------------------------------------------------------*/
/* a partir de cet instant, l'initialisation est terminee. On peut ne */
/* s'occuper que des cones. On propage la valeur de ceux-ci. Si au    */
/* moment de la propagation des cones prennenent des valeurs VDD ou   */
/* VSS, on les empile pour propager cette valeur a son tour.          */
/* On rappelle que l'ajout a la liste se fait par la queue.           */
/*--------------------------------------------------------------------*/
for(list = INF_SIGLIST ; list != NULL ; list = list->NEXT)
if(list->DATA != NULL)
    {
    list_list *outcone ;

    cone = (cone_list *)list->DATA ;
    cone->TYPE |= ((list->TYPE == 0) ? VSS : VDD) ;

    /*---------------------------------------------------------------*/
    /* Pour un cone VDD ou VSS, on parcourt les sorties. On regarde  */
    /* ensuite si les transistors de ce dernier ne sont pas attaques */
    /* par VDD ou VSS. Si tous les transistors d'une branche sont    */
    /* montes en resistance, le cone prend le type de la branche. Si */
    /* un seul des transistors est bloque, la branche est bloquee.   */
    /*---------------------------------------------------------------*/
    for(outcone = cone->OUTCONE ; outcone != NULL ; outcone = outcone->NEXT)
    if((outcone->TYPE & (LOOP | EXT)) == 0)
        {
        list_list *path ;

        cone = (cone_list *)outcone->DATA ;
        if((cone->TYPE & (VDD | VSS)) != 0)
        continue ;

        for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        if((path->TYPE & (BLEEDER | PATH_INOUT)) == 0)
            {
            link_list *link    = (link_list *)path->DATA ;
            long      outlevel = 2                       ;

                 if((path->TYPE & VDD) == VDD) outlevel = 1 ;
            else if((path->TYPE & VSS) == VSS) outlevel = 0 ;

            for( /* initialise */ ; link != NULL ; link = link->NEXT)
            if((link->TYPE & (RESIST | IN | INOUT)) == 0)
                {
                cone = (cone_list *)link->TRANS->GRID ;

                if((link->TYPE & TN) == TN)
                    {
                    /* NMOS attaque par VSS => branche bloquee */
                    if((cone->TYPE & VSS) == VSS)
                    break ;

                    /* NMOS attaque par VDD => resistance */
                    else if((cone->TYPE & VDD) == VDD)
                        {
                        link->TYPE |= RESIST ;
                        link->TRANS->TYPE |= RESIST ;
                        }
                    else /* branche contenant un transistor non resist */
                    outlevel = 2 ;
                    } /* fin du transistor N */
                else /* cas du PMOS */
                    {
                    /* PMOS attaque par VDD => branche bloquee */
                    if((cone->TYPE & VDD) == VDD)
                    break ;

                    /* NMOS attaque par VSS => resistance */
                    else if((cone->TYPE & VSS) == VSS)
                        {
                        link->TYPE |= RESIST ;
                        link->TRANS->TYPE |= RESIST ;
                        }
                    else /* branche contenant un transistor non resist */
                    outlevel = 2 ;
                    } /* fin du transistor N */
                } /* fin du parcours des maillons */

            if(link != NULL) /* maillon non-nul => branche non-fonctionnelle */
            path->TYPE |= BLEEDER ;
            else if(outlevel != 2) /* alors cone atteignant VDD ou VSS */
                {
                infAddList((char *)outcone->DATA,outlevel,'t') ;
                tas_NewConeAlim((cone_list *)outcone->DATA,outlevel) ;
                }
            } /* fin de parcours des branches */
        } /* fin de parcours des sorties */
    } /* fin de parcours de la liste des collages */

return ;
}


/****************************************************************************/
/*                            fonction AddList()                            */
/* rajoute un list_list a la liste pointee par head.                        */
/****************************************************************************/
list_list *tas_AddList(head,type,data)
list_list  *head ;
long       type  ;
char       *data ;

{
list_list *aux = (list_list *)mbkalloc(sizeof(list_list)) ;

aux->NEXT = head ;
aux->DATA = data ;
aux->TYPE = type ;

return(aux) ;
}


/****************************************************************************/
/*                          fonction GetAliasSig()                          */
/* remplace les noms du fichier '.inf' par l'alias le plus significatif.    */
/****************************************************************************/
list_list *tas_GetAliasSig(lofig,list)
lofig_list *lofig ;
list_list  *list  ;

{
losig_list *losig          ;
list_list  *auxlist = NULL ;
char       flc      = 'N'  ;

if(lofig != NULL)
while(list != NULL)
    {
    char      *name              ,
              flag  = 'N'        ;
    list_list *aux  = list->NEXT ;

    for(losig = lofig->LOSIG ; losig != NULL ; losig = losig->NEXT)
        {
        chain_list *namechain ;

        /* recherche du signal ayant le meme nom */
        for(namechain = losig->NAMECHAIN ; namechain != NULL ;
            namechain = namechain->NEXT)
        if((namechain->DATA == tas_DeVect(list->DATA))                  ||
           (tas_JokerSubst(namechain->DATA,tas_DeVect(list->DATA)) == 0)  )
            {
            /* si signal externe, on prend le nom du connecteur */
            if(losig->TYPE == 'E')
                {
                locon_list *locon ;

                for(locon = lofig->LOCON ;
                    (locon != NULL) && (locon->SIG != losig) ;
                    locon = locon->NEXT) ;

                if(locon == NULL)
                tas_error(15,list->DATA,TAS_ERROR) ;

                name = locon->NAME ;
                }
            else name = get_name(losig) ;

            auxlist = tas_AddList(auxlist,list->TYPE,tas_ReVect(name)) ;
            flag = 'Y' ;

            /* ce signal ne nous interesse plus => break */
            break ;
            }

        /*-----------------------------------------------------------*/
        /* s'il n'y a pas de joker dans le nom, le signal est trouve */
        /* et il n'y en a pas d'autre: ce n'est pas la peine de con- */
        /* tinuer a en chercher dans les autres signaux.             */
        /*-----------------------------------------------------------*/
        if((namechain != NULL) && (namechain->DATA == list->DATA))
        break ;
        } /* fin de la boucle sur les signaux */

    if(flag == 'N') /* alors pas de signal trouve */
    tas_error(9,list->DATA,TAS_WARNING) ;

    free(list) ;
    list = aux ;
    } /* fin du parcours de la liste des noms du '.inf'*/

return(auxlist) ;
}


/****************************************************************************/
/*                         fonction BullitLatch()                           */
/* traitement des latches BULLIT: les commandes ne sont pas sur le premier  */
/* maillon des branches, mais sur le dernier maillon afin de limiter les    */
/* bruits. Cette fonction traite les entrees commandes.                     */
/****************************************************************************/
int tas_BullitLatch(desbfig)
desafig_list *desbfig ;

{
chain_list *reg ;

for(reg = desbfig->REG ; reg != NULL ; reg = reg->NEXT)
    {
    cone_list *cone   = (cone_list *)reg->DATA ;
    list_list *path                            ,
              *incone                          ;

    /* deselection des commandes existantes */
    for(incone = cone->INCONE ; incone != NULL ; incone = incone->NEXT)
    incone->TYPE &= ~COMMAND ;

    for(path = ((cone_list *)reg->DATA)->PATH ; path != NULL ;
        path = path->NEXT)
    if((path->TYPE & BLEEDER) != BLEEDER)
        {
        link_list *link ;
        cone_list *cmd  ;

        for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
            link = link->NEXT) ;

        /* si maillon connecteur => erreur: impossible */
        if((link->TYPE & (IN | INOUT)) != 0)
        tas_error(13,cone->NAME,TAS_ERROR) ;

        if((incone = tas_GetInLink(cone,link)) != NULL)
        incone->TYPE |= COMMAND ;
        } /* fin de parcours des branches */
    } /* fin de parcours de la liste des registres */

return ;
}
