/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.54                                            */
/*    Fichier : tas_util.c                                                  */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Payam KIANI                           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 21/10/1991     */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par : Payam KIANI                         le : 07/10/1992     */
/*                                                                          */
/****************************************************************************/
/* fichier tas_util.c : traitement des options de TAS:                      */
/*                          -d : genere le fichier .deb                     */
/*                          -in: donne le type du format d'entree           */
/*                          -t : generation du fichier ".int"               */
/*                          -t$: idem "-t" mais le programme s'arrete juste */
/*                               apres la generation du ".int"              */
/*                          -x : composition des chemins critiques          */
/*                          -e : generation du fichier ".slo"               */
/*                          -l : fichier de caracterisation pour SL         */
/*                          -tec=<fichier> : fichier technologie            */
/*                          -pch      : prise en compte des precharges. Elle*/
/*                                      ne peut etre active qu'avec la      */
/*                                      nouvelle chaine (fichier ttv)       */
/*                          -opc=<valeur>  : pourcentage de prise en compte */
/*                                           des capas ramenees             */
/*                          -swc=<valeur>  : pourcentage pour ramener les   */
/*                                           capacites sur des switches a   */
/*                                           l'entree du bloc               */
/*                          -nhr           : pas d'hierarchie               */
/*                          -pwl           : generateur tgh                 */
/*                          -pwl$          : comme -pwl, mais sortie apres  */
/*                          -bullit        : latches bullit                 */
/*                          -cout=<valeur> : capas des connecteurs OUT      */
/*                          -uk            : TAS anglophone                 */
/*                          -fr            : TAS anglophone                 */
/*                          -i : option DESB. Permet de lire le fichier .inf*/
/*                          -b : option DESB. Active l'orientation des trs. */
/*                          -a : option DESB. Prend en compte les "_s"      */
/*                          -p=: option DESB. Donne la profondeur de foconf */
/*                          -nf: option DESB. Empeche l'analyse fonction.   */
/****************************************************************************/
#ident "@(#) Timing Analyzer with Slopes"
#ident "@(#) Version 3.84"

#include "tas_all.h"

char        TAS_TRACE_MODE             ;
char        TECFILE[80]                ;
char        VERSION[80]                ;
char        DATE_VER[80]               ;
long        TAS_SEUIL_MIN = 0x7FFFFFFF ;
long        TAS_SEUIL_MAX = 0x80000000 ;
char        SLOFILE       = 'N'        ;
char        PERFINT       = 'N'        ;
char        PERFORD       = 'N'        ;
char        PRFTYPE       = 'N'        ;
char        INT_END       = 'N'        ;
char        NOTAS         = 'N'        ;
double      CAPARAPREC    = 1.0        ;
double      CAPASWITCH    = 1.0        ;
char        *FILENAME     = NULL       ;
char        TAS_BULLIT    = 'N'        ;
char        TAS_LANG      = 'E'        ;
char        TAS_PWL       = 'N'        ;
double      TAS_CAPAOUT   = 0.0        ;
short       tas_LEVEL     = 1          ;
char        TAS_GARBAGE   = 'N'        ;
char        DEB_FILE      = 'N'        ;
char        TAS_TREATPRECH = 'N'       ;
char        TAS_SILENTMODE = 'N'       ;
char        TAS_HIER       = 'N'       ;
#ifndef THOMSON
char        TAS_MAIL       = 'Y'       ;
#else
char        TAS_MAIL       = 'N'       ;
#endif
chemcr_list *TAS_MAXPATH               ;
ptype_list  *HEADMEMO                  ;
struct information TAS_INFO            ;


/****************************************************************************/
/*                         fonction tas_MotifInName()                       */
/* fonction heritee de DESB.                                                */
/****************************************************************************/
 /*------------------------------------------------------------------*
  | test l'inclusion de la chaine pointee par motif                  |
  | dans la chaine pointee par nom                                   |
  |   entree : nom et motif;                                         |
  |   sortie : NUll si non trouve                                    |
  |            char * pt sur le caractere suivant le motif dans name |
  *------------------------------------------------------------------*/
static char * tas_MotifInName(nom,motif)
char * nom   ;
char * motif;
{
char * pt1 = NULL;
char * pt2 = NULL;
char * pt3 = NULL;

   if((motif!=NULL)&&(nom!=NULL)) 
   for(pt1=nom;(*pt1)!='\0';pt1++)
   {
   pt3=motif;
      for(pt2=pt1;( ((*pt2)!='\0') && ((*pt3)!='\0') && (*pt2==*pt3) );pt3++,pt2++);
      if(*pt3=='\0') return (pt2);
   }

return(NULL);
}


/****************************************************************************/
/*                         fonction tas_SubElt()                            */
/* fonction heritee de DESB.                                                */
/****************************************************************************/
static char * tas_SubElt(sub,addSub,jok)
char * sub ;
char ** addSub ;
char   jok ;
{
char buff[100] ;
char * res=buff ;

   while((*sub!=jok)&&(*sub!='\0'))
   {
   *res++ = *sub++ ;
   (*addSub)++;
   }

*res='\0' ;

res=(char*)malloc(strlen(buff)+1);
strcpy(res,buff);
return(res);
}


/****************************************************************************/
/*                         fonction TestJoker()                             */
/* 0 si pas de joker, 1 sinon.                                              */
/****************************************************************************/
short tas_TestJoker(name)
char *name ;

{
short i ;

for(i = strlen(name)-1 ; (i != -1 ) && (name[i] != INF_JOK) ; i--) ;
return((i != -1) ? 1 : 0) ;
}


/****************************************************************************/
/*                         fonction version()                               */
/* fixe la date et le numero de version.                                    */
/****************************************************************************/
int tas_version()

{
strcpy(VERSION,"3.85") ;

if(TAS_LANG == 'E')
strcpy(DATE_VER,"March 10th, 1995") ;
else if(TAS_LANG == 'F')
strcpy(DATE_VER,"10 mars 1995") ;
}


/****************************************************************************/
/*                         fonction GenPwl()                                */
/* ecrit un signal de la forme tangente hyperbolique pour la commutation    */
/* d'entree.                                                                */
/****************************************************************************/
int tas_GenPwl(file,vt,vinit,front,name)
FILE      *file ;
double    vinit ,
          vt    ,
          front ;
char      *name ;

{
double vfin = VDDmax - vinit ,
       t0                    ,
       temps                 ;

/* si front nul => echelon */
if(front == 0.0)
    {
    tas_error(25,name,TAS_WARNING) ;
    fprintf(file,"\n.IC V(%s)=%.2f\nVin %s 0 DC %.2fV\n",name,vinit,
            name,vfin) ;
    }

/*-------------------------------------------------------------------------*/
/* de 0 a la tension de seuil le signal est continu et au-dela, il devient */
/* de la forme tgh, de maniere continu et derivable.                       */
/* La derivee du signal a l'instant t=t0 ou le niveau atteint est Vt est:  */
/*                                                                         */
/*                        S'=(Vfin-Vt)/front                               */
/* D'ou l'equation de la tangente au point (t0,Vt):                        */
/*                        V-Vt = ((Vfin-Vt)/front)*(t-t0)                  */
/* Si on dit que cette droite passe par Vinit:                             */
/*                       t0 = -front*((Vinit-Vt)/(Vfin-Vt))                */
/*-------------------------------------------------------------------------*/
t0 = front*((vt-vinit)/(vfin-vt)) ;
fprintf(file,"Vin %s 0 PWL(\n+0NS %.4fV",name,vinit) ;

/* a partir de la => tgh */
for(temps = t0 ; temps < t0+(5*front) ; temps += (front/10.0))
fprintf(file,"\n+%.2fNS %.4fV",temps,(vt+(vfin-vt)*tas_TGH((temps-t0)/front))) ;

fprintf(file,"\n+%.2fNS %.4fV\n+)\n",temps,vfin) ;
}


/****************************************************************************/
/*                          fonction PwlFile()                              */
/* cree deux fichier PwlRise et PwlFall l'un contenant un signal tgh montant*/
/* et l'autre descendant.                                                   */
/****************************************************************************/
int tas_PwlFile(desbfig)
desafig_list *desbfig ;

{
FILE *fup = mbkfopen("PwlRise",NULL,WRITE_TEXT) ,
     *fdw = mbkfopen("PwlFall",NULL,WRITE_TEXT) ;

if((fup == NULL) || (fdw == NULL))
    {
    char straux[100] ;

    strcpy(straux,"'PwlRise' or 'PwlFall'") ;
    tas_error(8,straux,TAS_WARNING) ;
    return ;
    }

tas_GenPwl(fup,VTN,0.0,(FRONT_CON/1000.0),"<noeud>") ;
tas_GenPwl(fdw,VDDmax-VTP,VDDmax,(FRONT_CON/1000.0),"<noeud>") ;

if((fclose(fup) != 0) || (fclose(fdw) != 0))
    {
    char straux[100] ;

    strcpy(straux,"'PwlRise' or 'PwlFall'") ;
    tas_error(10,straux,TAS_WARNING) ;
    }

return ;
}


/****************************************************************************/
/*                          fonction CompCapa()                             */
/* Calcule une capacite equivalente a une porte NAND dans la technologie    */
/* utilisee. Elle sera rajoutee a la cellule etudiee, puisque cette fonction*/
/* ne sert que pour la caracterisation des cellules de base. Si une capa    */
/* a deja ete donnee avec l'option -cout, celle-ci sera gardee.             */
/****************************************************************************/
double tas_CompCapa(lofig)
lofig_list *lofig ;

{
double capa = TAS_CAPAOUT ;

if((capa == 0.0) && (lofig->LOTRS != NULL))
    {
    lotrs_list *tn, *tp ;

    /* recherche du transistor NMOS */
    for(tn = lofig->LOTRS ; (tn != NULL) && (tn->TYPE != TRANSN) ;
        tn = tn->NEXT) ;

    /* recherche du transistor PMOS */
    for(tp = lofig->LOTRS ; (tp != NULL) && (tp->TYPE != TRANSP) ;
        tp = tp->NEXT) ;

    /*-----------------------------------------------------------------*/
    /* si un des deux transistors n'existe pas, on va rendre 2 fois la */
    /* meme chose.                                                     */
    /*-----------------------------------------------------------------*/
    if(tn == NULL)
    tn = tp ;

    if(tp == NULL)
    tp = tn ;

    /* capa de charge = capa de grille des deux transistors */
    capa = elpLotrsCapaGrid(tn)+elpLotrsCapaGrid(tp) ;
    }
 
/* capa est en fF, on le transforme en pF */
return(capa) ;
}

/*****************************************************************************
*                           fonction tas_setenv()                            *
*****************************************************************************/
int tas_setenv()

{
char *str ;

/* fichier techno par defaut */
str = getenv("MBK_TECHNO_NAME") ;
if(str != NULL)
strcpy(TECFILE,str) ;
else
strcpy(TECFILE,TECHNOLOGIE) ;

/* variable mode trace */
str = getenv("TAS_TRACE_MODE") ;
if(str != NULL)
    {
    TAS_TRACE_MODE = (strcmp(str,"yes") == 0 ) ? 'Y' : 'N' ;
    }

/* variable langage */
DESB_LANG = elpLang = TAS_LANG = 'E' ;
str = getenv("TAS_LANGUAGE") ;
if(str != NULL)
    {
    DESB_LANG = elpLang = TAS_LANG =
    (strcmp(str,"english") == 0 ) ? 'E' : 'F' ;
    }

return ;
}


/*****************************************************************************
*                            function perfpath()                             *
*****************************************************************************/
void perfpath(pathopt)
char *pathopt ;

{
short i ;

pathopt += 3 ;

/* recherche du carcatere ':' dans l'argument */
for(i = 0 ; (i != strlen(pathopt)) && (pathopt[i] != ':') ; i++) ;

if(i == strlen(pathopt))
    {
    TAS_SEUIL_MIN = atol(pathopt) ;
    TAS_SEUIL_MAX = 0x7FFFFFFF ;

    if((TAS_SEUIL_MIN == 0) && (strcmp(pathopt,"0") != 0) &&
       (strcmp(pathopt,"00") != 0))
    tas_error(26,NULL,TAS_NULL) ;
    }
else /* alors il y a un ':' */
    {
    char *aux = pathopt ;

    aux[i] = '\0' ; /* isolation du seuil MIN */

    if(i != 0)
        {
        TAS_SEUIL_MIN = atol(aux) ;

        if((TAS_SEUIL_MIN == 0) && (strcmp(aux,"0") != 0) &&
           (strcmp(aux,"00") != 0))
        tas_error(26,NULL,TAS_NULL) ;
        }
    else TAS_SEUIL_MIN = 0x80000000 ;

    aux += i+1 ; /* isolation du seuil MAX */

    if(aux[0] != '\0')
        {
        TAS_SEUIL_MAX = atol(aux) ;

        if((TAS_SEUIL_MAX == 0) && (strcmp(aux,"0") != 0) &&
           (strcmp(aux,"00") != 0))
        tas_error(26,NULL,TAS_NULL) ;
        }
    else TAS_SEUIL_MAX = 0x7FFFFFFF ;
    }
 
return ;
}


/*****************************************************************************
*                            fonction retkey()                               *
* renvoie la position de '=' dans chaine si il existe, -1 sinon              *
*****************************************************************************/
short retkey(chaine)
char *chaine ;

{
short i ;

for(i = 0 ; (i<strlen(chaine)) && (chaine[i] != '=') ; i ++) ;

return((i == strlen(chaine)) ? -1 : i) ;
}


/*****************************************************************************
*                            function optionword()                           *
* pour les options sur plusieurs lettres.                                    *
*****************************************************************************/
int optionword(chaine)
char *chaine ;

{
short i ;

if(chaine[1] == 'x' && chaine[2] == '=') perfpath(chaine) ;
else /* autres cas */
    {
    short eq ;

    if((eq=retkey(chaine)) == -1) /* les options n'ayant pas de '=' */
        {
             if(strcmp(chaine,"-pch") == 0) TAS_TREATPRECH = 'Y' ;
        else if(strcmp(chaine,"-hr") == 0) TAS_HIER = 'Y' ;
        else if(strcmp(chaine,"-bullit") == 0) TAS_BULLIT = 'Y' ;
        else if(strcmp(chaine,"-pwl") == 0) TAS_PWL = 'Y' ;
        else if(strcmp(chaine,"-pwl$") == 0)
            {
            TAS_PWL = 'Y' ;
            NOTAS = 'Y' ;
            }
        else if(strcmp(chaine,"-uk") == 0)
            {
            DESB_LANG = elpLang = TAS_LANG = 'E' ;
            tas_version() ;
            }
        else if(strcmp(chaine,"-fr") == 0)
            {
            DESB_LANG = elpLang = TAS_LANG = 'F' ;
            tas_version() ;
            }
        else if(strcmp(chaine,"-t$") == 0)
        PERFINT = INT_END = 'Y' ;
       else if(strcmp(chaine,"-nomail") == 0)
       TAS_MAIL = 'N' ;

        else for(i = 1 ; i < strlen(chaine) ; i ++)
        switch(chaine[i])
            {
            case 'd' : DEB_FILE = 'Y' ; break ;
            case 'l' : PRFTYPE = 'C' ; break ;
            case 'e' : SLOFILE = 'Y' ; break ;
            case 't' : PERFINT = 'Y' ; break ;
            case 'x' : TAS_SEUIL_MAX = 0x7FFFFFFF ;
                       TAS_SEUIL_MIN = 0x80000000 ;
                       break ;
            case 's' : tas_CloseTerm() ;
                       TAS_SILENTMODE = 'Y' ;
                       break ;

            /* options DESB */
#ifdef BULL
            case 'm' : DSB_HELP_L = 1 ; break ; /* prise en compte _l */
#else
            case 'a' : DSB_HELP_S++ ; break ;
#endif
            case 'b' : DSB_ORIENT = 0 ; break ;
            case 'i' : DSB_INF = 1 ; break ;
            /* fin des options DESB */

            default  : tas_error(26,NULL,TAS_NULL) ;
            }
        } /* fin de autres options melangees */
    else /* cas des options ayant '=' dans la ligne de commande  */
        {
        char aux[150] ;

        strcpy(aux,chaine) ;
        aux[eq] = '\0' ; /* on isole le mot avant '=' */

        if(eq != 0) /* premier caractere pas '=' */
            {
            short prec = 0 ;

            if(strcmp(aux,"-tec") == 0) strcpy(TECFILE,chaine+eq+1) ;
            else if(strcmp(aux,"-opc") == 0)
                {
                CAPARAPREC = ((double)atoi(chaine+eq+1)) / 100.0 ;

                if(((CAPARAPREC == 0.0) && (strcmp(chaine+eq+1,"0") != 0)) ||
                   (CAPARAPREC < 0.0)                                        )
                tas_error(26,NULL,TAS_NULL) ;
                }
            else if(strcmp(aux,"-swc") == 0)
                {
                CAPASWITCH = ((double)atoi(chaine+eq+1)) / 100.0 ;

                if(((CAPASWITCH == 0.0) && (strcmp(chaine+eq+1,"0") != 0)) ||
                   (CAPASWITCH < 0.0)                                        )
                tas_error(26,NULL,TAS_NULL) ;
                }
            else if(strcmp(aux,"-cout") == 0) /* capas de charge */
                {
                if((TAS_CAPAOUT = atof(chaine+eq+1)) == 0.0)
                tas_error(26,NULL,TAS_NULL) ;
                }
            else if(strcmp(aux,"-in") == 0) /* type du fichier d'entree */
                {
                /* test du format. TAS n'accepte pas de VHDL, EDIF ou HILO */
                if((strcmp(chaine+eq+1,"al" ) != 0) &&  /* ALLIANCE */
                   (strcmp(chaine+eq+1,"fdn") != 0) &&  /* DAX      */
                   (strcmp(chaine+eq+1,"fne") != 0) &&  /* VTI      */
                   (strcmp(chaine+eq+1,"hns") != 0) &&  /* VTI      */
                   (strcmp(chaine+eq+1,"hdn") != 0) &&  /* DAX      */
                   (strcmp(chaine+eq+1,"spi") != 0)   ) /* SPICE    */
                tas_error(26,NULL,TAS_NULL) ;

                strcpy(IN_LO,chaine+eq+1) ;
                }

            /* options DESB */
            else if(strcmp(aux,"-p") == 0)
                {
                DSB_PROF = atoi(chaine+eq+1) ;
                DSB_USE_FCF = 1 ;

                if((DSB_PROF == 0) && (strcmp(chaine+eq+1,"0") != 0))
                tas_error(26,NULL,TAS_NULL) ;

                if(DSB_PROF == 0)
                DSB_USE_FCF = 0 ;
                }

            else tas_error(26,NULL,TAS_NULL) ;
            }
        else tas_error(26,NULL,TAS_NULL) ;
        }
    } /* fin des autres cas */
}


/*****************************************************************************
*                            function option()                               *
*****************************************************************************/
int option(argc,argv)
int argc;
char *argv[];

{
short i          ;
char  help = 'n' ;

tas_version() ;
if(argc == 1) tas_error(26,NULL,TAS_NULL) ;

#ifdef BULL
DSB_HELP_S = 1 ;
DSB_PROF = 7 ;
#else
DSB_HELP_S = 0 ;
DSB_PROF = 5 ;
#endif
DSB_USE_FCF = 5 ;
DSB_HELP_L = 0 ;
DSB_ORIENT = 1 ;

for(i = 1 ; i != argc ; i++)
    {
    if(argv[i][0] != '-')
        {
        FILENAME = mbkalloc(strlen(argv[i]) + 1) ;
        strcpy(FILENAME,argv[i]) ;
        }
    else
        {
             if(strcmp(argv[i],"-e") == 0) SLOFILE   = 'Y' ;
        else if(strcmp(argv[i],"-t") == 0) PERFINT   = 'Y' ;
        else if(strcmp(argv[i],"-l") == 0) PRFTYPE = 'C' ;
        else if(strcmp(argv[i],"-h") == 0) help = 'y' ;
        else if(strcmp(argv[i],"-x") == 0)
            {
            TAS_SEUIL_MAX = 0x7FFFFFFF ;
            TAS_SEUIL_MIN = 0x80000000 ;
            }
        else if(strcmp(argv[i],"-d") == 0) DEB_FILE = 'Y' ;
        else if(strcmp(argv[i],"-s") == 0)
            {
            tas_CloseTerm() ;
            TAS_SILENTMODE = 'Y' ;
            }

        /* options DESB */
        else if(strcmp(argv[i],"-i") == 0) DSB_INF = 1 ;
        else if(strcmp(argv[i],"-b") == 0) DSB_ORIENT = 0 ;
#ifdef BULL
        else if(strcmp(argv[i],"-na") == 0)
        DSB_HELP_S = DSB_HELP_L = 0 ;
        else if(strcmp(argv[i],"-m") == 0)
        DSB_HELP_L = 1 ;
#else
        else if(strcmp(argv[i],"-a") == 0)
        DSB_HELP_S = 1 ;
#endif
        /* fin des options DESB */

        else optionword(argv[i]) ;
        }
    }

if(help == 'y')
tas_error(0,NULL,TAS_NULL) ;

if((FILENAME == NULL) && (NOTAS == 'N'))
tas_error(26,NULL,TAS_NULL) ;

/*-----------------------------------------------------------------*/
/* l'option -l est incompatible avec un chargement du fichier deb. */
/* La sortie deb est d'ailleurs inhibee lorsque -l est demandee.   */
/*-----------------------------------------------------------------*/
if(PRFTYPE == 'C')
    {
    DEB_FILE = 'N' ;
    TAS_HIER = 'N' ;
    }

return ;
}


/*****************************************************************************/
/*                            fonction tas_FreeCrPath()                      */
/* libere les noeuds du chemin passe en parametre.                           */
/*****************************************************************************/
int tas_FreeCrPath(crpath)
chemcr_list *crpath ;

{
while(crpath->PATH != NULL)
    {
    critic_list *critic = crpath->PATH->NEXT ;

    free(crpath->PATH) ;
    crpath->PATH = critic ;
    }
}


/*****************************************************************************/
/*                            fonction CloseTerm()                           */
/* renvoie les messages envoyes sur stdout vers /dev/null (mode silencieux). */
/*****************************************************************************/
int tas_CloseTerm()

{
int  out ;

out = open("/dev/null",1) ;
close(1) ;
dup(out) ;

return ;
}


/*****************************************************************************/
/*                            fonction RestoreTerm()                         */
/* retablit le renvoie des messages envoyes sur stdout vers l'ecran.         */
/*****************************************************************************/
int tas_RestoreTerm()

{
char sout[50] ;
int  out      ;

strcpy(sout,"/dev/tty") ;
out = open(sout,1) ;
close(1) ;
dup(out) ;
}


/*****************************************************************************/
/*                            fonction GetKill()                             */
/* lorsque le signal ^C est envoye au process, cette fonction retablit la    */
/* sortie stdout, si elle a ete detrournee.                                  */
/*****************************************************************************/
int *tas_GetKill()

{
if(TAS_SILENTMODE == 'Y')
tas_RestoreTerm() ;

exit(1) ;
}


/*****************************************************************************/
/*                            fonction tasalloc()                            */
/* fonction d'allocation memoire. S'il n'y a plus de place, malloc() renvoie */
/* NULL. Alors on vide les chemins critiques. A partir du parcours de graphe */
/* c'est cette fonction qui sera utilisee pour l'allocation memoire.         */
/*****************************************************************************/
char *tasalloc(nbytes)
unsigned int nbytes ;

{
char *pt = (char *)malloc(nbytes) ;

if(pt == NULL)
    {
    chemcr_list *couple ;

    /*------------------------------------------------------------------*/
    /* si la place n'a pas pu etre allouee, deux possibilites: soit le  */
    /* vidange a deja ete fait soit non. Si oui, ce n'est plus possible */
    /* et alors on jette. Sinon on y procede et on alloue.              */
    /*------------------------------------------------------------------*/
    tas_error(27,NULL,TAS_WARNING) ;

    /* vidange des chemins critiques. Seul le chemin critique sera garde. */
    for(couple = PERFFIG->COUPLE ; couple != NULL ; couple = couple->NEXT)
    tas_FreeCrPath(couple) ;

    for(couple = PERFFIG->CHAINE ; couple != NULL ; couple = couple->NEXT)
    tas_FreeCrPath(couple) ;

    /* on annule l'option -x */
    TAS_SEUIL_MAX = 0x80000000 ;
    TAS_SEUIL_MIN = 0x7FFFFFFF ;

    /* nouvelle allocation */
    if((pt = (char *)malloc(nbytes)) == NULL)
    tas_error(28,NULL,TAS_ERROR) ;
    }

return(pt) ;
}


/*****************************************************************************/
/*                          fonction Mail()                                  */
/* envoie un mail au compte P.Kiani@frcl.bull.fr dans lequel il y a le infos */
/* sur le circuit lance. Comme ca on sait exactement le nbre de fois et les  */
/* circuits traites par TAS.                                                 */
/*****************************************************************************/
int tas_Mail(info)
struct information info ;

{
FILE  *file        ;
char  mailfic[150] ,
      sys[200]     ,
      mail[50]     ;
short i            ;

strcpy(mailfic,"00tas00.tmp") ;
/* si fichier pas ouvert => retour */
if((file = fopen(mailfic,"w")) == NULL)
return ;

/* selection de l'adresse mail */
#ifdef BULL
strcpy(mail,"P.Kiani@frcl.bull.fr") ;
#else
strcpy(mail,"kiani@masi.ibp.fr") ;
#endif

fprintf(file,"TAS: Fichier d'information pour envoi au compte PARIS6.\n") ;
fprintf(file,"\n") ;
fprintf(file,"Ce fichier contient les renseignements relatifs au circuit que vous avez\n") ;
fprintf(file,"traite. Je vous prie d'accepter leur communication au compte PARIS6 pour\n") ;
fprintf(file,"des statistiques et de l'avancement des theses concernant ce programme.\n") ;
fprintf(file,"\n") ;
fprintf(file,"Si vous avez des remarques envoyez-les au:\n") ;
fprintf(file,"\n") ;
fprintf(file,"                           %s\n",mail) ;
fprintf(file,"\n") ;
fprintf(file,"Si toutefois vous n'etes pas d'accord avec ce procede, vous pouvez empecher\n") ;
fprintf(file,"l'envoi de mail en lancant le programme avec l'option -nomail.\n") ;
fprintf(file,"\n") ;
fprintf(file,"Merci de votre comprehension...\t\tP. KIANI\n") ;
fprintf(file,"\n") ;

/* ecriture de la ligne de commande */
fprintf(file,"ligne de commande: ") ;
for(i = 0 ; i != info.argc ; i++)
fprintf(file,"%s ",info.argv[i]) ;

/* infos du circut */
fprintf(file,"\n\ninformations sur le circuit:\n") ;
fprintf(file,"delai critique                         : %ldpS\n",info.maxdelay) ;
fprintf(file,"nombre de transistors du bloc          : %ld\n",info.nb_trans) ;
fprintf(file,"nombre de cones du bloc                : %ld\n",info.nb_cone) ;
fprintf(file,"complexite du circuit                  : %ld\n\n",info.comp) ;

/* les temps de traitement */
fprintf(file,"les temps d'execution CPU:\n") ;
fprintf(file,"temps de parsing du fichier logique    : %lds\n",info.lo_t) ;
fprintf(file,"temps de desassemblage                 : %lds\n",info.db_t) ;
fprintf(file,"temps de calcul des delais elementaires: %lds\n",info.el_t) ;
fprintf(file,"temps de parcours de graphe            : %lds\n",info.gr_t) ;
fprintf(file,"temps de generation du perfmodule      : %lds\n\n",info.pr_t) ;

fclose(file) ;

sprintf(sys,"mail -s \"INFO TAS%s\" %s < %s",VERSION,mail,mailfic) ;
system(sys) ;

sprintf(sys,"rm %s",mailfic) ;
system(sys) ;
return ;
}


/****************************************************************************/
/*                         fonction tas_JokerSubst()                        */
/* fonction heritee de DESB: test si une chaine de caracteres correspond au */
/* motif. 0 si oui, -1 sinon.                                               */
/****************************************************************************/
/*--------------------------------------------------------------------------+
| Teste si pt_name respecte les conditions de substitution prevues dans     |
| ptSub;                                                                    |
| entree : ptName - Le nom devant eventuellement etre substitue             |
|          ptSub  - La chaine de caratere contenant les jokers '*'          |
| Sortie : 0        Substitution possible                                   |
|         -1        Pas de correspondance                                   |
+--------------------------------------------------------------------------*/
short tas_JokerSubst(ptName,ptSub)
char * ptName ;
char * ptSub ;
{
char * name = ptName ;
char * sub = ptSub ;
char * pt ;

   if(*sub!=INF_JOK) 
   {
      while(*sub == *name) 
      {
/* MODIF PAYAM: Pour que les chaines sans joker puissent etre comparees */
/* Le 20/05/1993                                                        */
      if((*sub == '\0') && (*name == '\0'))
      return(0) ;
/* fin MODIF */

      sub++  ;
      name++ ;
      }
      if(*sub!=INF_JOK) return(-1);
   sub=ptSub ;
   name=ptName ;
   }

   if(sub[strlen(sub)]!=INF_JOK)
   {
   short i = strlen(sub) ;
   short j = strlen(name);
      while(sub[i]==name[j])
      {
      j--;
      i--;
      }
      if(sub[i]!=INF_JOK) return(-1) ;
   }

   while(*sub!='\0')
   {
   char * motif ;
      if(*sub==INF_JOK) sub++ ;
      if(*sub=='\0') return (0) ;
   motif=tas_SubElt(sub,&sub,INF_JOK);
   name=tas_MotifInName(name,motif);
   mbkfree(motif);
      if(name==NULL) return(-1) ;
   }

return(0);   
}


/****************************************************************************/
/*                         fonciton FreePtypeList()                         */
/* libere une liste de ptype dont la tete est head.                         */
/****************************************************************************/
int tas_FreePtypeList(head)
ptype_list *head ;

{
while(head != NULL)
head = delptype(head,head->TYPE) ;

return ;
}


/****************************************************************************/
/*                         fonction FreeMbkFig()                            */
/* vide completement la structure MBK a plat et renvoie NULL.               */
/****************************************************************************/
lofig_list *tas_FreeMbkFig(lofig)
lofig_list *lofig ;

{
char       *aux ;

/* figure vide => on ne fait rien */
if(lofig == NULL)
return(NULL) ;

/* liberation des transistors */
while(lofig->LOTRS != NULL)
    {
    aux = (char *)lofig->LOTRS->NEXT ;
    mbkfree(lofig->LOTRS) ;
    lofig->LOTRS = (lotrs_list *)aux ;
    }

while(lofig->LOCON != NULL)
    {
    aux = (char *)lofig->LOCON->NEXT ;
    mbkfree(lofig->LOCON) ;
    lofig->LOCON = (locon_list *)aux ;
    }

/* suppression des signaux */
while(lofig->BKSIG != NULL)
    {
    mbkfree((losig_list *)lofig->BKSIG->DATA) ;
    lofig->BKSIG = delptype(lofig->BKSIG,lofig->BKSIG) ;
    }

return(NULL) ;
}


/****************************************************************************/
/*                          fonction ReVect()                               */
/* revectorise le nom passe en parametre: s'il contient des blancs, ils     */
/* seront remplaces par des '[]'.                                           */
/****************************************************************************/
char *tas_ReVect(str)
char *str ;

{
char  name[500]               ;
short i         = strlen(str) ,
      j         = i - 1       ;

strcpy(name,str) ;

do
    {
    /*-----------------------------------------------------------------*/
    /* recherche de ' ' s'il y en a. D'autre part si le nom est eclate */
    /* le parser rajoute '.<index>' a la fin du nom. Il faut dans ce   */
    /* cas, mettre le '.<index>' a la fin de la chaine apres revecto-  */
    /* risation. Donc le ']' vient juste avant le '.'.                 */
    /* Attention: si la vectorisation se fait dans un cheminon qui     */
    /* n'est pas le dernier, il y a probleme.                          */
    /*-----------------------------------------------------------------*/
    while((name[--i] != ' ') && (i != -1))
    if(name[i] == '.')
    j = i-1 ;

    if(i != -1)
        {
        short k ;

        name[i] = '[' ;

        for(k = strlen(name) ; k != j ; k--)
        name[k+1] = name[k] ;

        name[j+1] = ']' ;
        j = i - 1 ;
        }
    }
while(i != -1) ;

return(namealloc(name)) ;
}


/****************************************************************************/
/*                          fonction DeVect()                               */
/* devectorise un nom.                                                      */
/****************************************************************************/
char *tas_DeVect(str)
char *str ;

{
char  name[500]                ;
short i         = strlen(str) ;

strcpy(name,str) ;

while(i-- != 0)
    {
    short j ;

    /* si vecteur fermant rencontre */
    if(name[i] == ']')
        {
        /* on garde la position du caratere */
        j = i ;

        /*---------------------------------------------------------------*/
        /* on recule jusqu'a erreur ou vecteur ouvrant (attention: Joker */
        /* accepte):                                                     */
        /*       toto[*] est valide                                      */
        /*       *toto[1] est valide                                     */
        /*       *toto[*] est valide                                     */
        /*       toto[x] n'est pas valide                                */
        /*---------------------------------------------------------------*/
        while((--i != 0) && (name[i] != '[') &&
              (((name[i] >= '0') && (name[i] <= '9')) || (name[i] == '*'))) ;

        if(name[i] == '[') ;
            {
            while(name[j] != '\0')
            name[j] = name[++j] ;

            name[i] = ' ' ;
            }
        }
    } /* fin de la garnde boucle */

return(namealloc(name)) ;
}


/****************************************************************************/
/*                           fonction GetInLink()                           */
/* Renvoie l'entree correspondant au signal attaquant le maillon passe en   */
/* parametre.                                                               */
/****************************************************************************/
list_list *tas_GetInLink(cone,link)
cone_list *cone ;
link_list *link ;

{
list_list *incone ;

for(incone = cone->INCONE ; incone != NULL ; incone = incone->NEXT)
if((link->TYPE & (IN | INOUT)) == 0)
    {
    if(link->TRANS->GRID == (locon_list *)incone->DATA)
    break ;
    }
else
    {
    if(link->TRANS == (lotrs_list *)incone->DATA)
    break ;
    }

return(incone) ;
}
