/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.82                                            */
/*    Fichier : tastec.c                                                    */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Amjad HAJJAR et Payam KIANI           le : 10/09/1994     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier techno.c : calcul des parametres TAS.                            */
/****************************************************************************/
#include "tas_all.h"
double  RN        ,
        RP        ,
        FRONT_CON ,
        AN        ,
        BN        ,
        AP        ,
        BP        ,
        RNT       ,
        RPT       ,
        VDDmax    ,
        VTP       ,
        VTN       ,
        SEUIL     ,
        RNDD      ,
        RPDD      ,
        KNDD      ,
        KPDD      ,
        RNDF      ,
        RPDF      ,
        VSSdeg    ,
        VDDdeg    ,
        RNUD      ,
        RPUD      ,
        KNUD      ,
        KPUD      ,
        RNUF      ,
        RPUF      ,
        QN        ,
        QP        ,
        KGUD      ,
        KGDD      ,
        INFB      ,
        IPFB      ;


/****************************************************************************/
/*                            fonction TechnoParameters()                   */
/* fonction faisant appel au fichier techno et calculant les parametres     */
/* technos de TAS.                                                          */
/*                                                                          */
/* Parametres en entree:                                                    */
/* --------------------                                                     */
/*    1) tecname: nom complet du fichier techno a charger.                  */
/*                                                                          */
/* Parametre en sortie:                                                     */
/* -------------------                                                      */
/*    La fonction renvoie -1 en cas de probleme, 0 sinon.                   */
/****************************************************************************/
int tasTechnoParameters(tecname)
char *tecname ;

{
if(elpLoadElp(tecname)) /* si probleme dans la lecture techno */
return -1 ;

VTN = elpModel[elpNMOS][elpVT] ;
AN  = elpModel[elpNMOS][elpA] ;
BN  = elpModel[elpNMOS][elpB] ;
RNT = elpModel[elpNMOS][elpRT] ;
VTP = elpModel[elpPMOS][elpVT] ;
AP  = elpModel[elpPMOS][elpA] ;
BP  = elpModel[elpPMOS][elpB] ;
RPT = elpModel[elpPMOS][elpRT] ;

VDDmax = elpVoltage[elpVDDMAX] ;
VDDdeg = elpVoltage[elpVDDDEG] ;
VSSdeg = elpVoltage[elpVSSDEG] ;
SEUIL  = elpVoltage[elpVTHR] ;
FRONT_CON = elpGeneral[elpSLOPE] ;

RN = (1.0+BN*0.90*(VDDmax-VTN)) / (AN*0.90*(VDDmax-VTN)) ; 
RP = (1.0+BP*0.90*(VDDmax-VTP)) / (AP*0.90*(VDDmax-VTP)) ;
RNDD = (VDDmax-SEUIL)*(BN+(1.0/(VDDmax-VTN)))/(AN*(VDDmax-VTN)) ;
RPUD = SEUIL * (BP+(1.0/(VDDmax-VTP)))/(AP*(VDDmax-VTP)) ;
KNUD = 1.1 ;
INFB = 250*VDDmax/RNT ;
IPFB = 250*VDDmax/RPT ;


/* calcul de RPDD */
{
double VT = (VTP + VSSdeg)/2.0 ;
double b=BP*(VDDmax-VT);
double Rsat = 1.0 / (AP*(VDDmax-VT)) ;

RPDD = VDDmax / (VDDmax-VT-VT);
RPDD += b * log((2*(VDDmax-VT))/(VDDmax-VT-VT));
RPDD *= Rsat;
}

RNDF = RN ;
RPDF = RPDD ;

/* calcul de KNDD */
{
double BU=BN*(VDDmax-VTN);
KNDD = BU/(BU-1.0);
KNDD *= log((2*BU)/(BU+1.0));
KNDD -= 0.5*log(3.0-(4.0*VTN/VDDmax)) ;
}

/* calcul de KPDD */
{
double VT = (VTP + 2*VSSdeg)/3.0 ;
double b = 1.0+BP*(VDDmax-VT);
KPDD = RPDD*((VDDmax-2*VT)*(1.0+b)) / (RPT*(2*(VDDmax-VT) + b*(VDDmax-2*VT))) ;
}

/* calcul de RNUD */
{
double VT = (VDDmax + VTN - VDDdeg)/2.0 ;
double b=BN*(VDDmax-VT);
RNUD = VDDmax / (VDDmax-VT-VT);
RNUD += b * log((2*(VDDmax-VT))/(VDDmax-VT-VT));
RNUD /= AN*(VDDmax-VT) ;
}

RNUF = RNUD ;
RPUF = RPT ;

/* calcul de KPUD */
{
double BU=BN*(VDDmax-VTP);
KPUD = BU/(BU-1.0);
KPUD *= log((2*BU)/(BU+1.0));
KPUD -= 0.5*log(3.0-(4.0*VTP/VDDmax)) ;
}

/* calcul de QP */
{
double imp,u,b;
imp=AP*(VDDmax-VTP)*(VDDmax-VTP)/(1.0+BP*(VDDmax-VTP));
b=BP*(VDDmax-VTP);
u=0.5*(VDDmax-2*VTP)/(VDDmax-VTP);
QP = 500*imp*( ((1.0+b)*(log(1.0+u))/(1.0-b))
               -log(1-u)-2*(log(1+b*u))/(b-b*b) );
}

/* calcul de QN */
{
double imn,u,b;
imn=AN*(VDDmax-VTN)*(VDDmax-VTN)/(1.0+BN*(VDDmax-VTN));
b=BN*(VDDmax-VTN);
u=0.5*(VDDmax-2*VTN)/(VDDmax-VTN);
QN = 500*imn*( ((1.0+b)*(log(1.0+u))/(1.0-b))
               -log(1-u)-2*(log(1+b*u))/(b-b*b) );
}

/* calcul de KGUD */
{
double VT = (VTN + 2*(VDDmax - VDDdeg)) / 3.0 ,
       var1 = 0.5*VDDmax / (VDDmax-VT) ,
       var2 = (0.5*VDDmax -VT) / (VDDmax-VT) ;

var1 = 0.5*log( (1.0+var1)/(1.0-var1) );
var2 = 0.5*log( (1.0+var2)/(1.0-var2) );
KGUD = var1 - var2;
}

/* calcul de KGDD */
{
double VT = (VTP + 2*VSSdeg) / 3.0 ,
       var1 = 0.5*VDDmax / (VDDmax-VT) ,
var2 = (0.5*VDDmax -VT)/(VDDmax -VT) ;

var1 = 0.5*log( (1.0+var1)/(1.0-var1) );
var2 = 0.5*log( (1.0+var2)/(1.0-var2) );
KGDD = var1 - var2;
}

return 0 ;
}
