/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.50                                            */
/*    Fichier : tp.c                                                        */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Amjad HAJJAR et Payam KIANI           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 21/10/1991     */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier tas_tp.c : calcul des temps de propagation entre une sortie de   */
/*                    cone et toutes ses entrees                            */
/****************************************************************************/
#include "tas_all.h"

extern list_list *decision() ;
long FHH ;
long FLL ;
long CCUSED;

/*****************************************************************************/
/*                          fonction PathFonc()                              */
/* dit si la branche de type BLEEDER du cone est fonctionnelle ou non, c'est */
/* a dire si elle appartient au champ USER du cone ou non. Renvoie '1' si la */
/* branche est fonctionnelle, 0 sinon.                                       */
/*****************************************************************************/
short tas_PathFonc(cone,path)
cone_list *cone ;
list_list *path ;

{
short      resultat = 0 ;
ptype_list *ptype       ;
chain_list *chain       ;

/* branche pas typee BLEEDER => elle est fonctionnelle */
if((path->TYPE & BLEEDER) != BLEEDER) return(1) ;
/* si le cone n'est pas de type BLEEDER, alors la branche est non-fonc. */
else if((cone->TYPE & (BLEEDER | LATCH)) == 0) return(0) ;

/* sinon on cherche les branches rebouclees. Si liste vide => erreur */
ptype = getptype(cone->USER,BLEEDER) ; /* liste des bleeders */

if(ptype == NULL) tas_error(33,(char *)cone,TAS_ERROR) ;

/* recherche de la branche path dans la liste des feedback */
for(chain = (chain_list *)ptype->DATA ; chain != NULL ; chain = chain->NEXT)
if(chain->DATA == (char *)path) break ;

/* on renvoie 1 si chain est != NULL (branche existante), '0' sinon */
return((chain == NULL) ? 0:1) ;
}


/*-----------------------------------------------------------------------------
*				affiche_maillon()			      *
-----------------------------------------------------------------------------*/
/* fonction de debuggage: affiche le numero de l'index du cone qui attaque la
   grille de transistor du maillon					*/

int affiche_maillon(maillon0)
link_list *maillon0;

{
if (maillon0==NULL) printf("maillon : NULL\n");
else printf("maillon : %ld  type : %ld  L=%ld W=%ld  (%ld)\n",((cone_list *)(maillon0->TRANS->GRID))->INDEX,maillon0->TRANS->TYPE,maillon0->TRANS->LENGTH,maillon0->USER->TYPE,maillon0);
}


/*-----------------------------------------------------------------------------
*				affiche_time()			     	      *
-----------------------------------------------------------------------------*/
/* fonction d'affichage de la valeur du temps de propagation de l'input_grid0
   de cone0  et de type TPxx 						*/

int affiche_time(cone0,input_grid0,type,valeur) 
cone_list *cone0,*input_grid0;
char *type;
long valeur;

{
if (valeur==NOTIME)
printf("---tas--- affiche_time	: cone (%d) input_cone (%d) %s = NOTIME\n",
         cone0->INDEX,input_grid0->INDEX,type);
else   
printf("---tas--- affiche_time	: cone (%d) input_cone (%d) %s = %d\n",
        cone0->INDEX,input_grid0->INDEX,type,valeur);
}


/*-----------------------------------------------------------------------------
*				existe_tpd_maillon()		  	      *
* recherche l'existence d'un transistor de type type_transistor dont la       *
*  grille de transistor est attaque par l'input_grid0 et qui appartient a la  *
*  branche dont le premier element est maillon0                               *
*  retourne le maillon ou se trouve le transistor si ce maillon existe        *
*  sinon retourne NULL							      *
-----------------------------------------------------------------------------*/
link_list *existe_tpd_maillon(cone0,maillon0,input_grid0,type_transistor)
cone_list *cone0 ;
link_list *maillon0;
cone_list *input_grid0;
long      type_transistor;

{
link_list *maillon1 = maillon0 ;

while (maillon1 != NULL)
    { 
    if((maillon1->TYPE & (IN | INOUT)) != 0)
        {
        maillon1=NULL;
        break;
        }

    if( (((long)maillon1->TRANS->TYPE & type_transistor) == type_transistor)
    &&((cone_list *)maillon1->TRANS->GRID == input_grid0) ) break;

    maillon1=maillon1->NEXT;
    }
return(maillon1);
}


/*-----------------------------------------------------------------------------
*				existe_tpd()				      *
* recherche l'existence d'un transistor de type type_transistor dont la       *
*  grille de transistor est attaque par l'input_grid0 et qui appartient aux   *
*  branches  de type type_branche du cone0                                    *
*  retourne le 1 er maillon de la branche ou se trouve le maillon du          *
*  transistor s'il exite sinon retourne NULL                                  *
*  retourne par adresse dans le champs maillon_trans le maillon du transistor *
*  en question                                                                *
-----------------------------------------------------------------------------*/
list_list *existe_tpd(cone0,input_grid0,type_branche,type_transistor,
                      maillon_trans)
cone_list *cone0 ;
cone_list *input_grid0 ;
link_list **maillon_trans ;
long      type_branche ;
long      type_transistor;

{
list_list  *chemin0;
list_list  *chemin_ext,*chemin_v,*chemin_final;
link_list  *maillon0,*maillon1,*maillon2, *maillon3;
link_list *existe_tpd_maillon();
list_list *decision();
char       existe_ext='n',existe_v='n';

for(chemin0=cone0->PATH;chemin0;chemin0=chemin0->NEXT)
if((chemin0->TYPE & BLEEDER) != BLEEDER)
    {
    if((chemin0->TYPE & EXT) == EXT)
      {
      maillon0=(link_list *)chemin0->DATA ;
      maillon1=existe_tpd_maillon(cone0,maillon0,input_grid0,type_transistor);
      if( (maillon1 != NULL) &&
          ((chemin0->TYPE & (BLEEDER|CR)) == 0L) )
        {
         existe_ext='o';
         chemin_ext=chemin0;
         break;
        }
      }
    }

for(chemin0=cone0->PATH;chemin0;chemin0=chemin0->NEXT)
if((chemin0->TYPE & BLEEDER) != BLEEDER)
    {
    if( ((chemin0->TYPE & type_branche) == type_branche) &&
        ((chemin0->TYPE & (BLEEDER|CR)) == 0L) )
      {
       maillon2=(link_list *) (chemin0->DATA);
       maillon3=existe_tpd_maillon(cone0,maillon2,input_grid0,type_transistor);

       if (maillon3 != NULL)
        {
        existe_v='o';
        chemin_v=chemin0;
        break;
        }
      }
    }

chemin_final=decision(existe_ext,existe_v,chemin_ext,chemin_v);

if(chemin_final != NULL)
    {
    if ((chemin_final->TYPE & EXT)==EXT) (*maillon_trans)=maillon1;
    else (*maillon_trans)=maillon3;
    }
if(chemin_final==NULL) return (NULL);
else return(chemin_final);
}


/*-----------------------------------------------------------------------------
*				existe_conflit()			      *
-----------------------------------------------------------------------------*/
/* presque identique a existe_tpd() sinon qu'on a ajoute un index qui permet
   de recuperer la n ieme branche ou se trouve le transistor pour le conflit
   les arguments sont donc les memes que pour existe_tpd() plus l'index     */

link_list *existe_conflit(cone0,input_grid0,type_branche,
                          type_transistor,index, maillon_trans)
int index;
cone_list *cone0 ;
cone_list *input_grid0 ;
link_list **maillon_trans;
long      type_branche ;
long      type_transistor ;

{
list_list *chemin0 ;
link_list *maillon0=NULL,*maillon1=NULL;

for(chemin0 = cone0->PATH ; chemin0 ; chemin0 = chemin0->NEXT)
if ( ((chemin0->TYPE & type_branche) == type_branche ) &&
     ((chemin0->TYPE & (CR|BLEEDER)) == 0L) ) break;

while(chemin0 != NULL )
    {
    if ( ((chemin0->TYPE & type_branche) != type_branche) ||
         ((chemin0->TYPE & (CR|BLEEDER)) != 0L) )
        {
        chemin0=NULL;
        break;
        }

    maillon0=(link_list *) chemin0->DATA;
    maillon1=existe_tpd_maillon(cone0,maillon0,input_grid0,type_transistor);
    if(maillon1 != NULL) index--;
    if(index==0) break;
    chemin0=chemin0->NEXT;
    }

if(chemin0==NULL) return (NULL);
else 
    {
    *maillon_trans=maillon1;
    return(maillon0);
    }
}

/* i_br(m0,m1): Calcul du courant maximal dans une branche homogene */
/* maillon0 est le premier maillon de la branche, maillon1 commute  */
/* renvoie vers i_branch en cas particuliers de circuit non dual    */

double i_br(maillon0,maillon1)
link_list *maillon0;
link_list *maillon1;

{
link_list *maillonc;
cone_list *input;
double i_branch();
double i0,bx=0.0,Rtr=0.0,Rs=0.0,imax=0.0,k;
double ugs,uds,usat,fsat,fi,ui,vi,Ri,ut,vth,vtmax,vti;
/* ui */
/* ui-1 c'est vi */

for(;maillon0;maillon0=maillon0->NEXT)
    {
     if((maillon0->TYPE&(TN|TP))==(maillon1->TYPE&(TN|TP))) break;
    }
if(maillon0==NULL) return(0);

 /* test si ambiguite sur la branche (switch, degrade, etc..) */

k=0.0;
for(maillonc=maillon0;maillonc->NEXT;maillonc=maillonc->NEXT)
   {
     if((maillonc->TYPE&(TN|TP))!=(maillonc->NEXT->TYPE&(TN|TP))) k=1.1;
   }
for(maillonc=maillon0;maillonc;maillonc=maillonc->NEXT)
   {
     if ((maillonc->TYPE & SWITCH)==SWITCH) k=1.1;
     input=(cone_list *) maillonc->TRANS->GRID;
     if( ( (maillonc->TYPE & TN)==TN) &&
         ((input->TYPE & TPINVSS)==TPINVSS) ) k=1.1;
     if( ( (maillonc->TYPE & TP)==TP) &&
         ((input->TYPE & TNINVDD)==TNINVDD) ) k=1.1;
   }

if (k==1.1) /* ambiguite, on renvoie vers i_branch() */
   {
     imax=i_branch(maillon0,maillon1);
     return(imax);
   }

if ( (maillon1->TYPE & TN)==TN)
   {
     ut = VTN / (VDDmax-VTN);
     i0 = AN *  (VDDmax-VTN) * (VDDmax-VTN);
     i0 /= (1.0 + BN * (VDDmax-VTN));
     bx = BN*(VDDmax-VTN);
     k = (RNT * i0) / (VDDmax-VTN);
     vtmax = VDDmax - VDDdeg; 
     vth = (vtmax-VTP) / (VDDmax-VTN);
   }

else if ( (maillon1->TYPE & TP)==TP)
   {
     ut = VTP / (VDDmax-VTP);
     i0 = AP *  (VDDmax-VTP) * (VDDmax-VTP);
     i0 /= (1.0 + BP * (VDDmax-VTP));
     bx = BP*(VDDmax-VTP);
     k = (RPT * i0) / (VDDmax-VTP);
     vtmax = VSSdeg;
     vth = (vtmax-VTP) / (VDDmax-VTP);
   }
else return (0);


Rs=0.0;
Rtr=0.0;
for(maillonc=maillon0;maillonc;maillonc=maillonc->NEXT)
   {
     Rtr =  (double)maillonc->TRANS->LENGTH /
            (double)maillonc->USER->TYPE ;
     Rs += Rtr;
   }

Ri=Rs;
ui=1.0;
vi=0.0;
imax=0.0;
for(maillonc=maillon0;maillonc;maillonc=maillonc->NEXT)
   {
     if(maillonc!=maillon0)
      {
         ui = (1.0-sqrt(1.0-Ri/Rs)); 
      }
     Rtr  =  (double)maillonc->TRANS->LENGTH /
             (double)maillonc->USER->TYPE ;
     Ri -= Rtr;
     vi = (1.0-sqrt(1.0-(Ri/Rs))); 
     /*vti = ut + (vth * vi); */
     vti = ut;
     ugs = (1.0 - vi - vti) / (1.0 - ut);
     uds = ui - vi;
     usat=k*ugs;
     fsat = ((1.0+bx)*ugs*ugs)/(1.0+(bx*ugs));
     fi=fsat;
     if(uds < usat) fi *= (uds/usat);
     imax += (i0 * fi)/Rs;
/*
if (TAS_TRACE_MODE=='Y')
    {
    printf("---tas--- i_br	: imax=%.2e   i0=%.2e\n",imax,i0);
    printf("---tas--- i_br	: vi=%.2e   ui=%.2e\n",vi,ui);
    printf("---tas--- i_br	: ugs=%.2e   uds=%.2e\n",ugs,uds);
    printf("---tas--- i_br	: fi=%.2e   Rs=%.2e\n",fi,Rs);
    printf("---tas--- i_br	: vth=%.2e   vti=%.2e   ut=%.2e\n",
                                  vth,vti,ut);
    fflush(stdout);
    }
*/
   }

return(imax);
}

/* i_branch(m0,m1): Calcul du courant maximal dans une branche             */
/* maillon0 est le premier maillon de la branche, maillon1 commute  */

double i_branch(maillon0,maillon1)
link_list *maillon0;
link_list *maillon1;

{
link_list *maillonc;
cone_list *input;
double ax=0.0,bx=0.0,Rtr=0.0,Rs=0.0,imax=0.0,k;
double v=VDDmax,vin=VDDmax;

for(;maillon0;maillon0=maillon0->NEXT)
    {
     if((maillon0->TYPE&(TN|TP))==(maillon1->TYPE&(TN|TP))) break;
    }
if(maillon0==NULL)
return(0);

maillonc=maillon0->NEXT;
while (maillonc != NULL)
{
 if( (maillonc->TYPE & TN)==TN)
        {
        Rtr = RNT * (double)maillonc->TRANS->LENGTH /
                    (double)maillonc->USER->TYPE ;

        input=(cone_list *) maillonc->TRANS->GRID;
        vin = VDDmax-VTN;
        if( ((input->TYPE & TPINVSS)==TPINVSS ) &&
            (maillonc==maillon1) ) vin = VDDdeg-VTN;
        v=VDDmax-VTN;
        Rtr *= (v/(1.0+BN*v)) / (vin/(1.0+BN*vin)) ;
        }
 if( (maillonc->TYPE & TP)==TP)
        {
        Rtr = RPT * (double)maillonc->TRANS->LENGTH /
                    (double)maillonc->USER->TYPE ;
        input=(cone_list *) maillonc->TRANS->GRID;
        vin = VDDmax-VTP;
        if( ((input->TYPE & TPINVSS)==TPINVSS) &&
            (maillonc==maillon1) ) vin = VDDmax-VSSdeg-VTP;
        v=VDDmax-VTP;
        Rtr *= (v/(1.0+BP*v)) / (vin/(1.0+BP*vin)) ;
        }
  if ((maillonc->TYPE & SWITCH)==SWITCH)
     {
     if(maillonc != maillon1) Rtr /= 1.7 ;
     else Rtr /= 1.2 ;
     }
Rs += Rtr;
maillonc = maillonc->NEXT;
}
if( (maillon0->TYPE & TN)==TN)
    {
    ax=AN*(double)maillon0->USER->TYPE /
          (double)maillon0->TRANS->LENGTH ;
    bx=BN;
    input=(cone_list *) maillon0->TRANS->GRID;
    if( ((input->TYPE & TNINVDD)==TNINVDD) &&
        (maillon0==maillon1) ) vin = VDDdeg-VTN;
    else vin = VDDmax-VTN;
    }
if( (maillon0->TYPE & TP)==TP)
    {
    ax=AP*(double)maillon0->USER->TYPE /
          (double)maillon0->TRANS->LENGTH ;
    bx=BP;
    input=(cone_list *) maillon0->TRANS->GRID;
    if( ((input->TYPE & TPINVSS)==TPINVSS)  && 
        (maillon0==maillon1) ) vin = VDDmax-VSSdeg-VTP;
    else vin = VDDmax-VTP;
    }

k=tas_F1(ax,bx,Rs);
bx=tas_F2(ax,bx,Rs);
ax=k;
imax=(ax*vin*vin)/ (1.0+bx*vin); 
if (TAS_TRACE_MODE=='Y')
    {
    printf("---tas--- i_branch	: imax=%.2e\n",imax);
    fflush(stdout);
    }
return(imax);
}

/*--------------------------------------------------------------------------*/
double f5(U,F,an1,bn1,vddin,c,fin,VT)
double U,F,an1,bn1,vddin,fin,VT;
long c;

{
double b=0.0,z=0.0,tt=0.0,ttt=0.0,k=0.0;
long a;
double c1=0.0;

if (TAS_TRACE_MODE=='Y')
    {
    printf("---tas--- f5	: U=%.2e F=%.2e an1=%.2e\n",U,F,an1);
    printf("---tas--- f5	: bn1=%.2e vddin=%.2e c=%ld\n",bn1,vddin,c);
    fflush(stdout);
    }

an1=an1/bn1;
bn1=1/(U*bn1);
c1=c;
z=((SEUIL/1000)*c1)/(F*an1*U); 
tt=(1.0+bn1)*z;
tt += ((1.0+bn1)*0.693) - (bn1*bn1*(log(0.5*((bn1+1.0) / bn1))) / (1.0-bn1));

for(a=1;a<7;a++)
    {
    b=exp(-2.0*tt);
    ttt=(1.0+bn1) *log(2.0/(1.0+b))
        -((bn1*bn1)/(1.0-bn1))*log(1.0+((1.0-bn1)*(1.0-b)/(2.0*bn1)));
    tt=(1.0+bn1)*z + ttt;
    }

tt *= F;
k= (SEUIL-VT)/vddin;
if(TAS_TRACE_MODE=='Y') printf("---tas--- f5	: tt=%.2e\n",tt);
tt = tt - fin*0.5*log((1.0+k)/(1.0-k));
return(tt);
}
/*-----------------------------------------------------------------------------*
* 				CAPACITE DE CONFLIT 			       *
------------------------------------------------------------------------------*/
/* calcule la capacite de conflit() 
   ramene la capacite dans l'unitee donnee apres DESA (scale_capa)	*/

long capa_de_conflit (cone0,input_grid0,type_branche,type_transistor)
cone_list *cone0 ;
cone_list *input_grid0;
long      type_branche ;
long      type_transistor;

{
list_list *chemin0;
link_list *maillon0,*maillon1;
int index=1;
long capa=0L;
double capa1=0.0,capa2=0.0;
long FX=0L;
double QX=0.0,QY=0.0;
front_list *front0;
long ct,mt;

CCUSED=0L;
for(chemin0=cone0->PATH;chemin0;chemin0=chemin0->NEXT)
   for(maillon0=(link_list *)chemin0->DATA;
       maillon0;maillon0=maillon0->NEXT)
       if((maillon0->TYPE & (IN | INOUT)) == 0)
       maillon0->TRANS->USER = NULL;

for(chemin0=cone0->PATH;chemin0;chemin0=chemin0->NEXT)
    {
    ct=chemin0->TYPE ;
    if( (ct & type_branche)==type_branche )
        for(maillon0=(link_list *)chemin0->DATA;
            maillon0;maillon0=maillon0->NEXT)
        if((maillon0->TYPE & (IN | INOUT)) == 0L)
        maillon0->TRANS->USER = (ptype_list *)(ct|UCONFL|BTOPP);
             /*maillon marque conflictuel*/
    }
for(chemin0=cone0->PATH;chemin0;chemin0=chemin0->NEXT)
    {
    ct=chemin0->TYPE ;
    if( ((ct & type_branche) != type_branche) ||
        ((ct & (EXT|BLEEDER|CR))!=0L) )
        for(maillon0=(link_list *)chemin0->DATA;
            maillon0;maillon0=maillon0->NEXT)
        if((maillon0->TYPE & (IN | INOUT)) == 0L)
        maillon0->TRANS->USER = (ptype_list *)(
        (long)(maillon0->TRANS->USER) & (~UCONFL) );
    }

{ chain_list *sw ;
  ptype_list *paux = getptype(cone0->USER,SWITCH) ;
  if(paux != NULL)
  for(sw=(chain_list *)paux->DATA;sw;sw=sw->NEXT)
    {
    chain_list *tr = (chain_list *)sw->DATA ;
    ((lotrs_list *)tr->DATA)->USER = (ptype_list *) (BTSWITCH|BTCONFL);
    ((lotrs_list *)tr->NEXT->DATA)->USER = (ptype_list *) (BTSWITCH|BTCONFL);
    }
}

while((maillon0=existe_conflit(cone0,input_grid0,type_branche,
       type_transistor,index,&maillon1) ) != NULL )
    {
    index++;
    for(;maillon0!=maillon1;maillon0=maillon0->NEXT)
        { 
        mt = (long)maillon0->TRANS->USER ;
        mt = mt | BTCONFL;
        if( ((mt & UCONFL)==UCONFL) &&
            (((long)maillon0->NEXT->TRANS->USER & UCONFL)==UCONFL) )
            { 
            capa += ((long)maillon0->NEXT->CAPA + GetCapara(maillon0->NEXT)) ;
            mt = mt & (~UCONFL);
            maillon0->TRANS->USER=(ptype_list *) mt; 
            }
        }
        mt = (long)maillon1->TRANS->USER ;
        mt = mt | BTCONFL ;
        if( (mt & UCONFL)==UCONFL )
        {
        front0=(front_list *) ((getptype(input_grid0->USER,SLOPE))->DATA);
        if ((maillon1->TYPE & TP)==TP)
            { 
            QX=QP/(VDDmax-SEUIL);
            QY=elpCapa[elpPMOS][elpCDW]*1000.0 ;
            FX=front0->FUP;
            }
        else 
            {
            QX=QN/SEUIL;
            QY=elpCapa[elpNMOS][elpCDW]*1000.0 ;
            FX=front0->FDOWN;
            }
    
        if (FX==((long) (NOFRONT))) break;
/*NEW 6 1 91
        if(type_branche==VDD)
        FX *= 1.6*FX/(FX+FLL);
        if(type_branche==VSS )
        FX *= 1.6*FX/(FX+FHH);
NEW 6 1 91*/
        mt = mt & (~UCONFL);
	maillon0->TRANS->USER=(ptype_list *) mt; 
        capa1 =QX*FX*
               (double)maillon1->USER->TYPE /
               (double)maillon1->TRANS->LENGTH ;
        capa2 =QY*(double)maillon1->USER->TYPE / SCALE_X ;
        capa += (long)capa1 + (long)capa2 ;
    
        if (TAS_TRACE_MODE == 'Y') 
        printf("---tas--- capa_confl\t: cconf=%.2e cc=%.2e capa_tot=%ld\n",
               capa1,capa2,capa);
        }
    }	  
if (FX==((long) (NOFRONT))) capa=0L;
return(capa);
}
/*-----------------------------------------------------------------------------
*				tpd_dual()				      *
-----------------------------------------------------------------------------*/
/*calcule le temps dans une branche non degrade
  maillon0 = 1er maillon de la branche ou on veut calculer
  maillon1 = maillon ou se trouve le transistor qui varie
  cconf = capa de conflit
  retourne le temps en pico secondes*/



long tpd_dual(maillon0,maillon1,cconf)
link_list *maillon0,*maillon1;
long cconf;


{ double lw2=0.0,lw=0.0,AX=0.0,BX=0.0,RX=0.0,QY=0.0,r2=0.0;
  double imax=0.0,i_br(),i_branch();
  double an1=0.0,bn1=0.0,U=0.0,F=0.0,Rbr;
  double temps=0.0,k=0.0,VT=0.0,vddin=0.0,fin=0.0;
  long c3=0L,fin0=0L;
  link_list *maillonc;
  cone_list *cone_avant;
  front_list *front0;

if (maillon0==NULL) return(NOTIME);
cone_avant=(cone_list *)maillon1->TRANS->GRID;
front0=(front_list *) ((getptype(cone_avant->USER,SLOPE))->DATA);

if ((maillon1->TYPE &TP)==TP)
    { 
    if ((cone_avant->TYPE & TPINVSS) != TPINVSS) vddin=( VDDmax - VTP );
    else vddin= ( VDDmax- VSSdeg - VTP );

    fin0=front0->FDOWN;
    AX=AP;
    BX=BP;
    RX=RPT;
    VT=VTP;
    QY=elpCapa[elpPMOS][elpCDW]*1000.0 ;
    } 
else
    { 
    if ((cone_avant->TYPE & TNINVDD) != TNINVDD) vddin= (VDDmax - VTN );
    else vddin= (VDDdeg - VTN);

    fin0=front0->FUP;
    AX=AN;
    BX=BN;
    RX=RNT;
    VT=VTN;
    QY=elpCapa[elpNMOS][elpCDW]*1000.0 ;
    }

imax=i_br(maillon0,maillon1);
if (TAS_TRACE_MODE=='Y') 
	printf("---tas--- tpd_dual	: imax de la branche=%.2e\n",imax);
fflush(stdout);

if (fin0==NOFRONT) return(NOTIME);
else fin=fin0;
maillonc=maillon0;
while(maillonc!=maillon1)
{
	if (maillonc==NULL)
		{
		printf ("Erreur 1  dans l'apport des arguments de valtpd()");
      		exit(1);
		}
        c3 += (long)( (maillonc->CAPA + (double)GetCapara(maillonc)) /
               ((double)(i_br(maillonc,maillon1)/imax)) );
        if( (((long)maillonc->TRANS->USER & BTCONFL) != BTCONFL) &&
            (CCUSED==0L) )
          {
           c3 += (long)( (double)cconf/
               ((double)(i_br(maillonc,maillon1)/imax)) );
           CCUSED=1L;
          }
        maillonc=maillonc->NEXT;
}
c3 += (long)( (maillonc->CAPA + (double)GetCapara(maillonc) 
                                      + (QY*(double)maillonc->USER->TYPE  /
                                            SCALE_X)) /
            (double)(i_br(maillonc,maillon1)/imax) );
if(CCUSED==0L)
   { c3 += (long)( (double)cconf/
            ((double)(i_br(maillonc,maillon1)/imax)) );
   CCUSED=1L;
   }

maillonc=maillonc->NEXT;
while((maillonc != NULL) && ((maillonc->TYPE & (IN | INOUT)) == 0))
	{
	lw= (double)maillonc->TRANS->LENGTH /
            (double)maillonc->USER->TYPE ;
	if ((maillonc->TYPE & SWITCH)==SWITCH) lw/=1.7;
	lw2+=lw;
	maillonc=maillonc->NEXT;
	}

lw=(double)maillon1->USER->TYPE /
   (double)maillon1->TRANS->LENGTH ;

	if ((maillon1->TYPE & SWITCH)==SWITCH) 
	lw *= 1.2;

an1=AX*lw;
bn1=BX;

if (lw2 != 0) 
	{
	r2=RX * lw2 ;
   	k=tas_F1(an1,bn1,r2);
	bn1=tas_F2(an1,bn1,r2);
	an1=k;
	}

U=tas_F3(an1,bn1,imax);
F=tas_F4(U,vddin,fin);
temps=f5(U,F,an1,bn1,vddin,c3,fin,VT); 
if (TAS_TRACE_MODE=='Y') 
	printf("---tas--- tpd_dual	: time=%ld\n",((long) temps) );
fflush(stdout);
return((long) temps);
}

/* FIN de tpd_dual()        */                    


/*-----------------------------------------------------------------------------
*				val_dual()			      *
-----------------------------------------------------------------------------*/

/* fonction qui calcule  les  4 temps pour les cones en dual cmos
   les mets dans la structure delay_list				*/
   

delay_list *val_dual(cone0,input_grid0)
cone_list *cone0,*input_grid0;

{
delay_list *delay0;
link_list *maillon0,*maillon1;
list_list *chemin0;
long capa=0L;

delay0=(delay_list *)(mbkalloc(sizeof(delay_list)));
chemin0=existe_tpd(cone0,input_grid0,VSS,TN,&maillon1);

if (chemin0==NULL) maillon0=NULL;
	else maillon0=(link_list *) (chemin0->DATA);
capa=capa_de_conflit(cone0,input_grid0,VDD,TP);
delay0->TPHL=tpd_dual(maillon0,maillon1,capa);
if(TAS_TRACE_MODE=='Y')
	affiche_time(cone0,input_grid0,"TPHL",delay0->TPHL);

capa=0L;
chemin0=existe_tpd(cone0,input_grid0, VDD,TP,&maillon1);
if (chemin0==NULL) maillon0=NULL;
	else maillon0=(link_list *) (chemin0->DATA);
capa=capa_de_conflit(cone0,input_grid0,VSS,TN);
delay0->TPLH=tpd_dual(maillon0,maillon1,capa);
if(TAS_TRACE_MODE=='Y')
	affiche_time(cone0,input_grid0,"TPLH",delay0->TPLH);


delay0->TPHH=NOTIME;
if(TAS_TRACE_MODE=='Y')
	affiche_time(cone0,input_grid0,"TPHH",delay0->TPHH);

delay0->TPLL=NOTIME;
if(TAS_TRACE_MODE=='Y')
	affiche_time(cone0,input_grid0,"TPLL",delay0->TPLL);


return(delay0);

}

/* FIN DE val_dual  */


/*-----------------------------------------------------------------------------
*				capa_equi()				      *
-----------------------------------------------------------------------------*/
/* calcule pour chaque maillon la capacite equivalente pour la branche qui le
   separe de la sortie. (on rajoute cconf a la capacite de sortie)           */

int capa_equi(maillon0,cconf)
link_list *maillon0;
long cconf;
{
double k2=0.0,cti=0.0;
link_list *maillon1;

if (maillon0==NULL) return ;
maillon0->USER->DATA = (char *)((long)maillon0->CAPA + GetCapara(maillon0)) ;
if( (((long)maillon0->TRANS->USER & BTCONFL) != BTCONFL) &&
    (CCUSED==0L) )
    {
    maillon0->USER->DATA = (char *)((long)maillon0->CAPA + GetCapara(maillon0) +
                                    cconf) ;
    cconf=0L;
    CCUSED=1L;
    }

if(TAS_TRACE_MODE=='Y')
   printf("---tas--- capa_equi	: cconf= %ld new_capa=%ld\n",
	  cconf,(long)maillon0->USER->DATA+GetCapara(maillon0));

maillon1=maillon0->NEXT;
while(maillon1 != NULL)
    {
    if( (maillon1->TYPE & (IN|INOUT)) !=0L ) return;
    if ((maillon0->TYPE & TN)==TN) k2= 1.0 * (double)FHH / RNT;
    else k2= 1.0 * (double)FLL / RPT;

    k2 *= 1000;
    cti=k2 * (double)maillon0->USER->TYPE /
             (double)maillon0->TRANS->LENGTH ;
    maillon1->USER->DATA = (char *)((long)((long)maillon1->CAPA+
                                           GetCapara(maillon1) +
                                    (long)((double)((long)maillon0->USER->DATA)*
                                            cti                                /
                                    ((double)((long)maillon0->USER->DATA) + cti)
                                    ))) ;
    if( (((long)maillon1->TRANS->USER & BTCONFL) != BTCONFL) &&
        (CCUSED==0L) )
        {
        maillon1->USER->DATA = (char *)((long)maillon1->USER->DATA + cconf) ;
        CCUSED=1L;
        }
    if(TAS_TRACE_MODE=='Y')
        {
        fprintf(stdout,"---tas--- capa_equi: capa_trans=%ld capa ramenee=%ld\n",
                (long)cti,(long)maillon1->USER->DATA) ;
        fflush(stdout) ;
        }
    maillon1=maillon1->NEXT;
    maillon0=maillon0->NEXT;
    }
}
/*-----------------------------------------------------------------------------
*				tpd_path()				      *
-----------------------------------------------------------------------------*/
/* calcule le temps de propagation entre le drain et la source de chaque 
   transistor de la liste qui debute par maillon0;
   sert dans 2 cas .dans le cas ou on calcule le tpd d'un connecteur en entree
   jusqu'a la sortie,et dans le cas des 2 calculs de temps des branches degrades
   la valeur du temps est en ps  */

long tpd_path(maillon0,front,type_front,cconf)
link_list *maillon0;
double front;
char type_front;
long cconf;

{
link_list *maillonc;
long nombre,p=0L;
double val=0.0,cap=0.0,k3=0.0,k4=0.0,k5=0.0;


if(maillon0 == NULL) return(0L) ;

capa_equi(maillon0,cconf) ;

for(maillonc = maillon0, nombre = 0 ; maillonc != NULL ;
    maillonc = maillonc->NEXT, nombre ++) ;

while(nombre > 0)
    {
    maillonc=maillon0; 
    p=nombre;

    while (p >1) 
        {
        maillonc=maillonc->NEXT;
        p--;
        }

    nombre-- ;

    if(type_front == 'D') /* si front descendant */
        {
        if((maillonc->TYPE & TN) != 0) /* si transistor NMOS */
            {
            k3 = RNDD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            k4 = KNDD ;
            k5 = RNDF * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            }  
        else /* si transistor PMOS */
            {
            k3 = RPDD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            k4 = KPDD ;
            k5 = RPDF * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            }
        }
    else /* si front montant */
        {
        if((maillonc->TYPE & TN) != 0) /* si transistor NMOS */
            {
            k3 = RNUD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            k4 = KNUD ;
            k5 = RNUD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            }
        else /* si transistor PMOS */
            {
            k3 = RPDD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            k4 = KPDD ;
            k5 = RPDD * (double)maillonc->TRANS->LENGTH /
                        (double)maillonc->USER->TYPE ;
            }
        }

    cap = (double)((long)maillonc->USER->DATA) / 1000.0 ;
	 	
    /* val += (k3*k3*cap*cap)/((k3*cap)+(k4*front)); */
    val += k3*cap*((k3*cap)+front)/((k3*cap)+(k4*front)) ;
    front += k5*cap ;

    if(TAS_TRACE_MODE == 'Y')
        {
        fprintf(stdout,"---tas--- tpd_path: val=%.2e front=%.2e\n",val,front) ;
        fflush(stdout) ;
        }
    }

if(TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"---tas--- tpd_path: valeur finale = %.2e\n",val) ;
    fflush(stdout) ;
    }

return(val) ;

}
/* FIN de tpd_path()    */



/*-----------------------------------------------------------------------------
*				affich_conec()				      *
-----------------------------------------------------------------------------*/
/*fonction d'affichage du temps pour un connecteur HR :f */

int affiche_conec(cone0,conec,type,valeur)
cone_list *cone0;
locon_list *conec;
char *type;
long valeur;

{
if (valeur==NOTIME)
	printf("---tas--- affich_conec	: cone(%d) conecteur( %s ) %s = NOTIME\n",cone0->INDEX,conec->NAME,type);
else
	printf("---tas--- affich_conec	: cone(%d) conecteur( %s ) %s = %d\n",cone0->INDEX,conec->NAME,type,valeur);

}


/*****************************************************************************/
/*                             fonction GiveActivExtPath()                   */
/* renvoie la branche externe la mieux placee pour commuter.                 */
/*****************************************************************************/
list_list *tas_GiveActivExtPath(cone,locon,level)
cone_list  *cone  ;
locon_list *locon ;
char       level  ;

{
list_list *path ;

for(path = cone->PATH ; (path != NULL) && ((path->TYPE & EXT) == EXT) ;
    path = path->NEXT)
if((path->TYPE & BLEEDER) != BLEEDER)
    {
    link_list *link ;

    if((path->TYPE & SWITCH) != SWITCH)
        {
        for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
            link = link->NEXT) ;
        if(link->TRANS == (lotrs_list *)locon)
        break ;
        }
    else /* si branche contenant des switches-cmos */
        {
        char prevtype = '\0' ;

        for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
            link = link->NEXT)
            {
            if((link->TYPE & SWITCH) == SWITCH)
                {
                if(
                   (((link->TYPE & TN) == TN) && (prevtype == 'P'))
                  ||
                   (((link->TYPE & TP) == TP) && (prevtype == 'N'))
                  )
                break ;

                if((link->TYPE & TN) == TN)
                prevtype = 'N' ;
                else
                prevtype = 'P' ;
                } /* fin du maillon switch */
            } /* fin de parcours des maillons */

        if((link->NEXT == NULL) && (link->TRANS == (lotrs_list *)locon))
        if(
           ((prevtype == 'P') && (level == 'U'))
          ||
           ((prevtype == 'N') && (level == 'D'))
          )
        break ;
        } /* fin des branches contenant des switches-cmos */
    }

if(path == NULL)
return(NULL) ;
else
return(((path->TYPE & EXT) == EXT) ? path : NULL) ;
}


/*-----------------------------------------------------------------------------
*				val_con()				      *
-----------------------------------------------------------------------------*/
delay_list *val_con(cone,locon,cconf)
cone_list *cone;
locon_list *locon;
long cconf;

{
list_list  *path                                                 ;
delay_list *delay   = (delay_list *)mbkalloc(sizeof(delay_list)) ;
link_list  *auxlink = NULL                                       ,
           *link                                                 ;

delay->TPHL = delay->TPLH = delay->TPHH = delay->TPLL = NOTIME ;

if((path = tas_GiveActivExtPath(cone,locon,'U')) != NULL)
    {
    for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
        link = link->NEXT)
    auxlink = link ;

    if(auxlink != NULL)
        {
        auxlink->NEXT = NULL ;
        delay->TPHH = tpd_path(path->DATA,FRONT_CON,'U',cconf);
        auxlink->NEXT = link ;
        }
    else
    delay->TPHH = 0 ;
    }

auxlink = NULL ;

if((path = tas_GiveActivExtPath(cone,locon,'D')) != NULL)
    {
    for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
        link = link->NEXT)
    auxlink = link ;

    if(auxlink != NULL)
        {
        auxlink->NEXT = NULL ;
        delay->TPLL = tpd_path(path->DATA,FRONT_CON,'D',cconf);
        auxlink->NEXT = link ;
        }
    else
    delay->TPLL = 0 ;
    }


return(delay) ;
}


/*-----------------------------------------------------------------------------
*					f6()			 	      *
-----------------------------------------------------------------------------*/
double f6(Imax,c)
double Imax,c;
{
double result=0.0;

if (TAS_TRACE_MODE=='Y')
	{
	printf("---tas--- f6		: Imax = %.2e c = %.2e\n",Imax,c); 
	}
result=(1.2 * (VDDmax-((VTN+VTP)/2.0))) * c /(Imax*1000);
if (TAS_TRACE_MODE=='Y')  
	printf("---tas--- f6()		: new_front= %.2e \n",result); 
return (result);
}

/*-----------------------------------------------------------------------------
*				tpd_trans_good()			      *
-----------------------------------------------------------------------------*/
long tpd_trans_good(maillon0,maillon_actif,cconf)
link_list *maillon0,*maillon_actif;
long cconf;


{ double lw2=0.0,AX=0.0,BX=0.0,RX=0.0,QY=0.0,r2=0.0,imax=0.0;
  double an1,bn1,U=0.0,F=0.0,Rbr=0.0;
  double ca=0.0,temps=0.0,k=0.0,VT=0.0,vddin=0.0,fin=0.0,new_front=0.0;
  long c3=0L,temps2=0L,total_temps=0L;
  link_list *maillon_var,*maillon_recol,*maillon1,*maillonc;
  cone_list *cone_avant;
  front_list *front0;
  short type_maillon;

if (TAS_TRACE_MODE=='Y')
printf("---tas--- tpd_trs_good\t: input(%ld)\n",
      ((cone_list *)(maillon_actif->TRANS->GRID))->INDEX);

if (maillon0 != maillon_actif)
{
maillon_var=maillon0;
if ((maillon_var->TYPE & TP) == TP ) 
		  type_maillon=TP;
          else 
		 type_maillon=TN;
while (maillon_var != maillon_actif)
    {
    if ((maillon_var->TYPE & TP) == TP ) type_maillon=TP;
    else type_maillon=TN;
    maillon1=maillon_var;
    for(;(((maillon_var->TYPE & type_maillon)== type_maillon) &&
      (maillon_var != maillon_actif));maillon_var=maillon_var->NEXT);
    }
if ((maillon_actif->TYPE & type_maillon)!= type_maillon) 
   maillon1=maillon_actif;
}
else maillon1=maillon0;

maillon_var=maillon1->NEXT;
maillon1->NEXT=NULL;
capa_equi(maillon0,cconf);
maillon1->NEXT=maillon_var;

maillon_var=maillon0;
if (maillon_var != maillon1)
  { while(maillon_var->NEXT !=maillon1)
         maillon_var=maillon_var->NEXT;
         maillon_recol=maillon_var;
         maillon_var->NEXT=NULL;
         maillon_var=maillon0;
  }
else maillon_var=NULL;

cone_avant=(cone_list *)maillon_actif->TRANS->GRID;
front0=(front_list *)((getptype(cone_avant->USER,SLOPE))->DATA);
if ((maillon_actif->TYPE & TP ) ==TP)
	{ 
	if ((cone_avant->TYPE & TPINVSS) != TPINVSS)
        	vddin= ( VDDmax - VTP );
        else
        	vddin= ( VDDmax- VSSdeg - VTP );

	fin=front0->FDOWN;
	AX=AP;
	BX=BP;
	VT=VTP;
        QY=elpCapa[elpPMOS][elpCDW]*1000.0 ;
        } 
  
	else
	{ 
	if ((cone_avant->TYPE & TNINVDD) != TNINVDD)
        	vddin= (VDDmax - VTN );
        else
        	vddin= (VDDdeg - VTN);

	fin=front0->FUP;
	AX=AN;
	BX=BN;
	VT=VTN;
        QY=elpCapa[elpPMOS][elpCDW]*1000.0 ;
	}
if (fin==(double)(NOFRONT)) return(NOTIME);

maillonc=maillon1;
imax=i_br(maillonc,maillon_actif);
c3=(long)maillonc->USER->DATA ;

while(maillonc!=maillon_actif)
   {  
    maillonc=maillonc->NEXT;

    if(maillonc==NULL)
    tas_error(38,NULL,TAS_ERROR) ;

    c3 += (long)( (maillonc->CAPA + (double)GetCapara(maillonc)
                   + (QY*(double)maillonc->USER->TYPE)/ SCALE_X) /
                  ((double)(i_br(maillonc,maillon_actif)/imax)) );
    if( (((long)maillonc->TRANS->USER & BTCONFL)!=BTCONFL) &&
        (CCUSED==0L) )
      {
      c3 += (long)( (cconf) /
            ((double)(i_br(maillonc,maillon_actif)/imax)) );
      CCUSED=1L;
      }
    }

for(maillonc = maillonc->NEXT ; maillonc != NULL ; maillonc = maillonc->NEXT)
    {
    if((maillonc->TYPE &  TP) != 0) RX=RP ;
    else RX=RN ;

    if((maillonc->TYPE & (IN | INOUT)) == 0)
    r2 = RX * (double)maillonc->TRANS->LENGTH /
              (double)maillonc->USER->TYPE ;
    }

an1 = AX*(double)maillon_actif->USER->TYPE /
         (double)maillon_actif->TRANS->LENGTH ;
bn1=BX ;

if (r2 != 0) 
    {
    k=tas_F1(an1,bn1,r2);
    bn1=tas_F2(an1,bn1,r2);
    an1=k;
    }

U=tas_F3(an1,bn1,imax);
F=tas_F4(U,vddin,fin);
temps=f5(U,F,an1,bn1,vddin,c3,fin,VT); 
ca=(long)maillon1->USER->DATA ;
new_front=f6(imax,(double)c3);
if ((maillon_actif->TYPE & TP ) == TP )
	temps2=tpd_path(maillon_var,new_front,'U',cconf);
else
	temps2=tpd_path(maillon_var,new_front,'D',cconf);

total_temps=(long) (temps) +temps2;
if (TAS_TRACE_MODE=='Y') 
	{
	printf("---tas--- tpd_trs_good	:t1=%ld  t2=%ld total=%ld\n",
	(long) temps,(long) temps2,total_temps);
	}
fflush(stdout);
if (maillon_var != NULL ) maillon_recol->NEXT=maillon1;
return(total_temps);
}

/*-----------------------------------------------------------------------------
*				tpd_trans_false()		     	      *
-----------------------------------------------------------------------------*/
long tpd_trans_false(maillon0,maillon1,capaCR)
link_list *maillon0,*maillon1;
long capaCR;

{
list_list *chemin0;
link_list *maillon_var,*maillon3,*maillon2,*maillon_recol;
cone_list *cone_avant;
front_list *front0;
double Rf=0.0,Rg=0.0,rtot=0.0,fin=0.0,K=0.0,
       te1=0.0,te2=0.0,final=0.0,newfront=0.0;

maillon2=maillon1->NEXT;
maillon1->NEXT=NULL;
capa_equi(maillon0,capaCR);
maillon1->NEXT=maillon2;

maillon_var=maillon0;
if (maillon_var != maillon1)
	{ while(maillon_var->NEXT !=maillon1) maillon_var=maillon_var->NEXT;
	  maillon_recol=maillon_var;
  	  maillon_var->NEXT=NULL;
	  maillon_var=maillon0;
	}
else maillon_var=NULL;

cone_avant=(cone_list *)maillon1->TRANS->GRID;
front0=(front_list *) ((getptype(cone_avant->USER,SLOPE))->DATA);

if((maillon1->TYPE & TP) != 0) /* cas du PMOS */
    {
    Rf = RPDD ;
    Rg = RNT ;
    rtot = Rf * (double)maillon1->TRANS->LENGTH /
                (double)maillon1->USER->TYPE ;
    fin = front0->FDOWN ;
    K = KGDD ;
    }

/* rajoute le 10 juillet 1991 */
else if((maillon1->TYPE & TN) != 0) /* cas du NMOS */
    {
    Rf = RNUD ;
    Rg = RPT;
    rtot = Rf * (double)maillon1->TRANS->LENGTH /
                (double)maillon1->USER->TYPE ;
    fin = front0->FUP ;
    K = KGUD ;
    }

if(fin == (double)NOFRONT) return(NOTIME) ;

for(maillon2 = maillon1->NEXT ; maillon2 != NULL ; maillon2 = maillon2->NEXT)
    {
    if((maillon2->TYPE & (TP|TN)) ==   (maillon1->TYPE & (TP|TN)))
    rtot += Rf * (double)maillon2->TRANS->LENGTH /
                 (double)maillon2->USER->TYPE ;

/* ajoute le 10 juillet 1991 */
    else if((maillon2->TYPE & (IN | INOUT)) == 0)
    rtot += Rg * (double)maillon2->TRANS->LENGTH /
                 (double)maillon2->USER->TYPE ;
    }

te1 = (rtot * (long)maillon1->USER->DATA) / 1000;
te1 += K*fin ;
if(TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"---tas--- tpd_trs_fals: rtot=%lf,c=%ld,K=%lf\n",
            rtot,(long)maillon1->USER->DATA,K) ;
    fprintf(stdout,"---tas--- tpd_trs_fals: Rg=%.2e,Rf=%.2e,fin=%.2e\n",
            Rg,Rf,fin) ;
    fflush(stdout) ;
    }

newfront= fin + (rtot * (long)maillon1->USER->DATA / 1000);

if ((maillon1->TYPE & TP)==TP) te2 = tpd_path(maillon_var,newfront,'D',0L);
else te2=tpd_path ( maillon_var,newfront,'U',0L);
final=te1 + te2;

if (TAS_TRACE_MODE == 'Y')
	{
	printf("---tas--- tpd_trs_fals	: cconf=%ld tot= %.2e newfront= %.2e \n",
		capaCR,rtot,newfront);
	printf("---tas--- tpd_trs_fals	: te1= %.2e te2=%.2e \n",te1,te2);
	printf ("---tas--- tpd_trs_fals	:time=%ld\n",(long)final);
	}
if (maillon_var != NULL) maillon_recol->NEXT=maillon1;
return((long) final);
}

/*-----------------------------------------------------------------------------
*			calcul_FB(chemin0,'U ou D')			      *
*    calcule pour le cone qui commence par ch0 la capa de FeedBack Up ou Dn   *
-----------------------------------------------------------------------------*/
long calcul_FB(cone,chemin0,commutation)
cone_list *cone ;
list_list *chemin0;
char commutation;
{
list_list *cheminc;
link_list *maillonc;
double Cch=0.0,cm=0.0;
double Cup=0.0,Cdown=0.0;
long w,l,fr;

for(cheminc=chemin0;cheminc;cheminc=cheminc->NEXT)
if(tas_PathFonc(cone,cheminc) != 0)
    {
    Cch=0.0;
    if((cheminc->TYPE & BLEEDER)==BLEEDER)
        {
        for(maillonc=(link_list *)cheminc->DATA;
           maillonc;maillonc=maillonc->NEXT)
            {
            cone_list *cone0 = (cone_list *)(maillonc->TRANS->GRID) ;
            w=maillonc->USER->TYPE ;
            l=maillonc->TRANS->LENGTH;
            if((cheminc->TYPE & VDD) == VDD)
                {
                fr=((front_list*)(getptype(cone0->USER,SLOPE)->DATA))->FUP;

                if( (maillonc->TYPE & TN)==TN)
                   {
                   cm = ((double)w/(double)l) * (double)fr * INFB ;
                   if(Cch==0.0) Cch=cm;
                   else Cch = (cm*Cch)/(cm+Cch);
                   }
                if( (maillonc->TYPE & TP)==TP)
                   {
                   cm = ((double)w/(double)l) * (double)fr * IPFB ;
                   if(Cch==0.0) Cch=cm;
                   else Cch = (cm*Cch)/(cm+Cch);
                   }
                }
            if((cheminc->TYPE & VSS) == VSS)
                {
                fr=((front_list*)(getptype(cone0->USER,SLOPE)->DATA))->FDOWN;

                if( (maillonc->TYPE & TN)==TN)
                   {
                   cm = ((double)w/(double)l) * (double)fr * INFB ;
                   if(Cch==0.0) Cch=cm;
                   else Cch = (cm*Cch)/(cm+Cch);
                   }
                if( (maillonc->TYPE & TP)==TP)
                   {
                   cm = ((double)w/(double)l) * (double)fr * IPFB ;
                   if(Cch==0.0) Cch=cm;
                   else Cch = (cm*Cch)/(cm+Cch);
                   }
                }
            }
        }
            if((cheminc->TYPE & VDD) == VDD) Cup += Cch; 
            if((cheminc->TYPE & VSS) == VSS) Cdown += Cch; 
    }
Cdown *= 2.0/VDDmax;
Cup *= 2.0/VDDmax;

if(TAS_TRACE_MODE=='Y')
printf("--- tas --- FEEDBACK\t: CRup=%ld, CRdn=%ld\n",(long)Cup,(long)Cdown);

return((commutation == 'U') ? (long)Cup : (long)Cdown) ;
}



/*-----------------------------------------------------------------------------
*				val_non_dual()			     	      *
-----------------------------------------------------------------------------*/
delay_list *val_non_dual(desafig0,cone0,input_grid0)
desafig_list *desafig0 ;
cone_list *cone0,*input_grid0;

{
delay_list *delay0;
list_list *chemin0;
link_list *maillon0,*maillon1;
long capa=0L;
long Cconfup=0L,Cconfdown=0L ; /* capa de contre reaction */
list_list *existe_tpd();

if (TAS_TRACE_MODE=='Y') 
printf("---tas--- val_non_dual	: cone(%d) attack by input_cone(%d)\n",
        cone0->INDEX,input_grid0->INDEX);


if(((cone0->TYPE & LATCH)==LATCH) || ((cone0->TYPE & BLEEDER) == BLEEDER)) 
	{
        chemin0=(list_list *)cone0->PATH;
	Cconfup=calcul_FB(cone0,chemin0,'U');
	Cconfdown=calcul_FB(cone0,chemin0,'D');
	}

delay0=(delay_list *) mbkalloc(sizeof(delay_list));
chemin0=existe_tpd(cone0,input_grid0, VSS,TN,&maillon1 );

if (chemin0==NULL) delay0->TPHL=NOTIME;
else 
    { 
    capa=capa_de_conflit(cone0,input_grid0,VDD,TP)+Cconfup ;

    if (((chemin0->TYPE & TPINVSS)  !=TPINVSS) &&
        ((chemin0->TYPE & TNINVDD)!=  TNINVDD))
    delay0->TPHL=tpd_dual((link_list *) (chemin0->DATA),maillon1,capa);
    else
    delay0->TPHL=tpd_trans_good((link_list *)(chemin0->DATA),maillon1,capa);
    }

if(TAS_TRACE_MODE=='Y') affiche_time(cone0,input_grid0,"TPHL",delay0->TPHL);

chemin0=existe_tpd(cone0,input_grid0, VDD,TP,&maillon1 );

if (chemin0==NULL) delay0->TPLH=NOTIME;
else 
    { 
    capa=capa_de_conflit(cone0,input_grid0,VSS,TN)+Cconfdown ;

    if(((chemin0->TYPE & TPINVSS)  !=TPINVSS) &&
        ((chemin0->TYPE & TNINVDD)!=  TNINVDD))
    delay0->TPLH=tpd_dual((link_list *) (chemin0->DATA),maillon1,capa);
    else
    delay0->TPLH=tpd_trans_good((link_list *)(chemin0->DATA),maillon1,capa);
    }

if(TAS_TRACE_MODE=='Y') affiche_time(cone0,input_grid0,"TPLH",delay0->TPLH);

chemin0=existe_tpd(cone0,input_grid0,VDD,TN,&maillon1);

if (chemin0==NULL) delay0->TPHH=NOTIME;
else 
    {
    capa=capa_de_conflit(cone0,input_grid0,VSS,TP)+Cconfdown ;
    delay0->TPHH=tpd_trans_false((link_list *) (chemin0->DATA),maillon1,capa);
    }
if(TAS_TRACE_MODE=='Y') affiche_time(cone0,input_grid0,"TPHH",delay0->TPHH);


chemin0=existe_tpd(cone0,input_grid0, VSS,TP,&maillon1 );
if(chemin0==NULL) delay0->TPLL=NOTIME;
else 
    { 
    capa=capa_de_conflit(cone0,input_grid0,VDD,TN)+Cconfup ;
    if (((chemin0->TYPE & TPINVSS)  !=TPINVSS) &&
        ((chemin0->TYPE & TNINVDD)!=  TNINVDD))
    delay0->TPLL=tpd_dual((link_list *) (chemin0->DATA),maillon1,capa);
    else
    delay0->TPLL=tpd_trans_false((link_list *) (chemin0->DATA),maillon1,capa);
    }

if(TAS_TRACE_MODE=='Y') affiche_time(cone0,input_grid0,"TPLL",delay0->TPLL);
return(delay0);
}


/*-----------------------------------------------------------------------------
*				tpd()				     	      *
-----------------------------------------------------------------------------*/
int tpd(desafig0,cone0)
cone_list *cone0;
desafig_list *desafig0 ;

{
link_list  *maillon0,maillon1;
list_list  *input0;
cone_list  *input_grid0;
locon_list      *conec;
delay_list *delay0;
front_list *front0 ;

if((cone0->TYPE & (VDD | VSS)) != 0)
    {
    for(input0 = cone0->INCONE ; input0 != NULL ; input0 = input0->NEXT)
    input0->USER = NULL ;
    return ;
    }

input0=  cone0->INCONE;
fflush(stdout);
front0 = (front_list *)(getptype(cone0->USER,SLOPE))->DATA ;
FHH = front0->FUP ;
FLL = front0->FDOWN ;
while (input0 != NULL )
    { 
    if (TAS_TRACE_MODE=='Y') 
        {
        printf("\n---tas--- tpd( )\t: cone(%ld) %s",cone0->INDEX,
               cone0->NAME);

        if ((input0->TYPE &CONE_TYPE)==CONE_TYPE)
        printf(" attaque par %ld (%s)\n",((cone_list *)input0->DATA)->INDEX,
               ((cone_list *)input0->DATA)->NAME);

        else
        printf(" attaque par le connecteur %s\n",
               ((locon_list *)input0->DATA)->NAME) ;

        printf("\n---tas--- tpd()\t: la sortie monte en %ld ps", FHH);
        printf("\n---tas--- tpd()\t: la sortie descend en %ld ps\n", FLL);
        }

    delay0 = NULL ;

    if((input0->TYPE &LOOP )!=LOOP)
        {
        if ((input0->TYPE &CONE_TYPE)==CONE_TYPE)
            {
            if((cone0->TYPE & DUAL) == DUAL) 
            delay0=val_dual(cone0,(cone_list *)input0->DATA);
            else
            delay0=val_non_dual(desafig0,cone0,(cone_list *) input0->DATA);
            }

        else if ((input0->TYPE & EXT)==EXT)
        delay0=val_con(cone0,(locon_list *) input0->DATA,0L); 

        if(delay0 != NULL)
            {
            /* si temps nul et entree et sorte differents => warning */
            if((delay0->TPLH == 0) || (delay0->TPHL == 0) ||
               (delay0->TPHH == 0) || (delay0->TPLL == 0))
                {
                char *name ;

                if((input0->TYPE & CONE_TYPE) == CONE_TYPE)
                name = ((cone_list *)input0->DATA)->NAME ;
                else name = ((locon_list *)input0->DATA)->NAME ;

                if(name != cone0->NAME)
                    {
                    char straux[100] ;

                    sprintf(straux,"%s et %s",name,cone0->NAME) ;
                    tas_error(37,straux,TAS_WARNING) ;
                    if(delay0->TPHH == 0) delay0->TPHH = DELAI_MIN ;
                    if(delay0->TPLL == 0) delay0->TPLL = DELAI_MIN ;
                    if(delay0->TPLH == 0) delay0->TPLH = DELAI_MIN ;
                    if(delay0->TPHL == 0) delay0->TPHL = DELAI_MIN ;
                    }
                }
            }
        }

    input0->USER=(ptype_list *) delay0;
    input0=input0->NEXT;
    }
}
