/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Verison 3.54                                            */
/*    Fichier : class.c                                                     */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Amjad HAJJAR et Payam KIANI           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par : Payam KIANI                         le : 07/10/1992     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier class.c : fonctions relatives aux calculs des fronts des cones   */
/****************************************************************************/
#include "tas_all.h"


/*****************************************************************************
*                                   visu_branche()                           *
*    fonction de debuggage: affiche le type et la resistance du chemin       *
*****************************************************************************/
int visu_branche(path)
list_list *path ;

{
link_list *link ;

if(path != NULL) 
    {
    fprintf(stdout,"path type = %ld - ",(long)path->TYPE)  ;
    if((path->TYPE & VDD) == VDD) fprintf(stdout,"VDD ") ;      /*************/
    else if((path->TYPE & VSS) == VSS) fprintf(stdout,"VSS ") ; /* type des  */
    else if((path->TYPE & EXT) == EXT) fprintf(stdout,"EXT ") ; /* branches  */

    if((path->TYPE & BLEEDER) == BLEEDER) fprintf(stdout,"non-functional!") ;
 
    fprintf(stdout,"\n\tresistance value = %ld\n",(long)path->USER) ;

    fflush(stdout) ;
    for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT) 
    affiche_maillon(link) ;
    }
}


/*****************************************************************************
*                            function res_branche()                          *
*          calcul de la resistance equivalente de chaque branche             *
*****************************************************************************/
int res_branche(cone)
cone_list    *cone ;

{
list_list   *path ;

/*---------------------------------------------------------------------------*/
/* si c'est une branche fonctionnelle, on rajoute la resistance equivalente  */
/* de tous ses transistors. Si c'est une branche non-fonctionnelle, on lui   */
/* donne une resistance nulle pour ne pas la prendre en compte.              */
/*---------------------------------------------------------------------------*/
for(path = cone->PATH ; path != NULL ; path = path->NEXT)
if((path->TYPE & BLEEDER) != BLEEDER) /* si branche fonctionnelle */
    {
    link_list   *link = (link_list *)path->DATA ;
    long        res=0 ;
    double      resist = 0.0 ;

    /* somme des resistances jusqu'a l'avant-dernier maillon */
    for( /* initialise */ ; link->NEXT != NULL ; link = link->NEXT)
        {
        if((link->TYPE & TP) == TP) resist = RP ;
        else resist = RN ;
        res += (long)(resist*(double)link->TRANS->LENGTH /
                             (double)link->USER->TYPE) ;
        }

    /* si le dernier maillon est un trans., on rajoute la res. equivalente */
    if((link->TYPE & (IN | INOUT)) == 0)
        {
        if((link->TYPE & TP) == TP) resist = RP ;
        else resist = RN ;
        res += (long)(resist*(double)link->TRANS->LENGTH/
                             (double)link->USER->TYPE) ;
        }

    path->USER = (ptype_list *)res ;
    }
else /* si branche non fonctionnelle */
    {
    path->USER = (ptype_list *)0L ; /* resistance nulle */
    }

if(TAS_TRACE_MODE == 'Y')
for(path = cone->PATH ; path != NULL ; path = path->NEXT)
    {
    fprintf(stdout,"---tas--- res_branche : to ") ;

    if((path->TYPE & EXT) == EXT) fprintf(stdout,"EXT") ;     /***************/
    else if((path->TYPE & VDD) == VDD) fprintf(stdout,"VDD") ;/* type de la  */
    else fprintf(stdout,"VSS") ;                              /* branche     */

    fprintf(stdout," in cone %ld (%s) is ",cone->INDEX,cone->NAME) ;
    if((path->TYPE & BLEEDER) == BLEEDER)
    fprintf(stdout,"%ld (false path)\n",(long)path->USER) ;
    else fprintf(stdout,"%ld unit(s)\n",(long)path->USER) ;
    fflush(stdout) ;
    }

return ;
}


/*****************************************************************************
*                     function clas_liste_branche()                          *
*           classement de la liste des branches d'un type donne              *
*****************************************************************************/
int clas_liste_branche(path)
list_list *path ;

{
list_list *aux_path ;
short     test=0    ;

if(path != NULL) do /* classement si chemin existe */
for(aux_path = path ; aux_path->NEXT != NULL ; aux_path = aux_path->NEXT)
    {
    list_list *nxpath ;

    test = 0 ;
    for(nxpath = aux_path->NEXT ; nxpath != NULL ; nxpath = nxpath->NEXT)
        {
        if((long)nxpath->USER > (long)aux_path->USER)
            {
            union tampon /* variable tampon auxiliaire */
                {
                char       *link ;
                ptype_list *user ;
                long       type  ;
                } chemin ;

            chemin.type = aux_path->TYPE ;  /*****************************/
            aux_path->TYPE = nxpath->TYPE ; /* echange des types         */
            nxpath->TYPE = chemin.type ;    /*****************************/
    
            chemin.link = aux_path->DATA ;  /*****************************/
            aux_path->DATA = nxpath->DATA ; /* ecahnge des maillons      */
            nxpath->DATA = chemin.link ;    /*****************************/

            chemin.user = aux_path->USER ;  /*****************************/
            aux_path->USER = nxpath->USER ; /* echange des resistances   */
            nxpath->USER = chemin.user ;    /*****************************/

            test = 1 ;
            }
        }
    }
while(test != 0) ;

if(TAS_TRACE_MODE == 'Y')
    {
    if(path == NULL) fprintf(stdout,"---tas--- clas_list_br : no path\n") ;
    else
        {
        fprintf(stdout,"---tas--- clas_list_br : the greater resistor is %ld\n",
                (long)path->USER) ;
        fprintf(stdout,"\tpath type = %ld\n",path->TYPE ) ;
        }
    fflush(stdout) ;
    }
}


/*****************************************************************************
*                           function clas_branche()                          *
* classement des branches du cone par ordre de resistivite croissante        *
* par type de branche (d'abord celles  allant vers les connecteurs externes  * 
* puis celle allant vers vdd et enfin celles menant vers vss                 *
*****************************************************************************/
void clas_branche(vdd_path,vss_path,ext_path,cone)
list_list *vdd_path ;
list_list *vss_path ;
list_list *ext_path ;
cone_list *cone ;

{
list_list *path ;
list_list *chemin_vdd = vdd_path ;
list_list *chemin_vss = vss_path ;
list_list *chemin_ext = ext_path ;
link_list *link ;

if(cone->PATH == NULL) return ;

/* separation de toutes les branches en trois listes VDD, VSS et EXT */
for(path = cone->PATH ; path != NULL ; path = path->NEXT)
    {
    if((path->TYPE & VDD) == VDD) chemin_vdd = chemin_vdd->NEXT = path ;
    if((path->TYPE & VSS) == VSS) chemin_vss = chemin_vss->NEXT = path ;
    if((path->TYPE & EXT) == EXT) chemin_ext = chemin_ext->NEXT = path ;
    }

chemin_vdd->NEXT = NULL ;
chemin_vss->NEXT = NULL ;
chemin_ext->NEXT = NULL ;


if (TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"---tas--- clas_branche : cone %ld (%s)\n",cone->INDEX,
            cone->NAME) ; 
    fflush(stdout) ;
    }
clas_liste_branche(ext_path->NEXT) ;
clas_liste_branche(vdd_path->NEXT) ;
clas_liste_branche(vss_path->NEXT) ;

if (ext_path != chemin_ext)
    {
    cone->PATH = ext_path->NEXT ;

    if(vdd_path != chemin_vdd)
        {
        chemin_ext->NEXT = vdd_path->NEXT ;
        if(vss_path != chemin_vss) chemin_vdd->NEXT = vss_path->NEXT ;
        }
    else
        {
        if(vss_path != chemin_vss) chemin_ext->NEXT = vss_path->NEXT ;
        }
}
else
    {
    if(vdd_path != chemin_vdd)
        {
        cone->PATH = vdd_path->NEXT;
        if(vss_path != chemin_vss) chemin_vdd->NEXT = vss_path->NEXT ;
        }

    else
        {
        if(vss_path != chemin_vss) cone->PATH = vss_path->NEXT ;
        else  cone->PATH = NULL ;
        }
    }
}


/*****************************************************************************
*                            function valfupfdown()                          *
*                 calcul de FUP ou FDOWN sur la branche choisie              *
*****************************************************************************/
long valfupfdown(path)
list_list    *path ;

{
link_list *link ;
long      slope = 0,capa = 0,restrans = 0 ;
double    res ;

/*----------------------------------------------------------------------------*/
/* si il n'y a pas de maillon, il n'y a pas de front=>NOFRONT.                */
/* si il n`y a qu'un seul maillon et que c'est un connecteur => FRONT_CON.    */
/* sinon on calcule le front du a la branche.                                 */
/*----------------------------------------------------------------------------*/
if(path == NULL) /* aucun maillon */
    {
    if(TAS_TRACE_MODE == 'Y')
    fprintf(stdout,"---tas--- valfupfdown : no slope\n" ) ;
    fflush(stdout) ;
    return(NOFRONT) ; 
    }

link = (link_list *)path->DATA ;

if(link->NEXT == NULL) /* si un seul maillon sur la branche */
    {
    if((link->TYPE & (IN | INOUT)) != 0) /* si connecteur => front connecteur */
        {
        if(TAS_TRACE_MODE == 'Y') 
            {
            fprintf(stdout,"---tas--- valfupfdown : value of slope is %ld",
                    (long)FRONT_CON) ;
            fprintf(stdout," -connector slope-\n") ;
            fflush(stdout) ;
            }
        return((long)FRONT_CON) ;
        }
    }

while(link->NEXT != NULL) /* il existe des maillons transistors */
    {
    if((link->TYPE & TP) == TP) res = RP ;
    else res = RN ;

    restrans = (long)(res*(double)link->TRANS->LENGTH /
                          (double)link->USER->TYPE) ;

    if(slope != 0)
    restrans = (long)(restrans * 0.95) ;

    if((link->TYPE & SWITCH) == SWITCH)
    restrans = (long)(restrans * 0.5) ; 

    if(slope == 0)
    capa += (long)link->CAPA + GetCapara(link) ;

    else
    capa += (long)(0.95*((long)link->CAPA + GetCapara(link))) ;

    slope += restrans*capa ;
    link = link->NEXT ;
    }

if((link->TYPE & (IN | INOUT)) == 0) /* si dernier maillon PAS connecteur */
    {
    if((link->TYPE & TP) == TP) res = RP ;
    else res = RN ;

    restrans = (long)(res*(double)link->TRANS->LENGTH /
                          (double)link->USER->TYPE) ;

    if(slope != 0)
    restrans = (long)(restrans * 0.95) ;

    if((link->TYPE & SWITCH) == SWITCH)
    restrans = (long)(restrans * 0.5) ;

    if(slope==0L)
    capa += (long)link->CAPA + GetCapara(link) ;

    else
    capa += (long)(0.95*((long)link->CAPA + GetCapara(link))) ;

    slope += restrans*capa ;
    }
/* si dernier maillon connecteur, alors pas touche! */

slope = (long)(slope  / 1000) ;

if(slope == 0) /* front nul => erreur: plus tard cause des problemes ! */
tas_error(1,NULL,TAS_ERROR) ;

if(TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"---tas--- valfupfdown : value of slope is %ld\n",slope) ;
    fflush(stdout) ;
    }

return(slope) ;
}


/*****************************************************************************
*                           function decision()                              *
* decide de prendre soit la branche EXT soit la branche VSS,VDD suivant      *
* qu'on ait trouve des  branches qui convenaient,et si les 2 branches        *
* existent prend la branche la plus resistive                                *  
*****************************************************************************/
list_list *decision(ext,alim,ext_path,alim_path)
char      ext ;
char      alim ;
list_list *ext_path ;
list_list *alim_path ;

{
list_list *path ; /* branche decide qui doit etre renvoyee */

if(ext == 'n') /* si pas de branche externe */
    {
    if(alim == 'n') path = NULL ; /* si non plus branche alim */
    else path = alim_path ; /* sinon branche alim */
    }
else /* si branche externe est oui */
    {
    if(alim == 'n') path = ext_path ; /* si pas de branche alim=>externe */
    else /* si les deux existent, il faut renvoyer la plus resistive */
    path = ((long)ext_path->USER > (long)alim_path->USER) ?
           ext_path : alim_path ;
    }

return(path) ;
}


/*****************************************************************************
*                               function front()                             *
*                               calcul de front                              *
*****************************************************************************/
long front(cone)
cone_list *cone ;

{
list_list  *ext_path = NULL,
           *vdd_path = NULL,
           *vss_path = NULL ;
char       ext='n',
           vdd='n',
           vss='n' ;
list_list  *path ;
front_list *slope ;

/*---------------------------------------------------------------------------*/
/* a ce niveau, les branches de chaque cone sont classees dans l'ordre       */
/* decroissant et par type : EXT, VDD et VSS. Dans un premier temps on       */
/* la branche la plus resistive pour chaque type (premiere branche de chaque */
/* liste).
/*---------------------------------------------------------------------------*/
for(path = cone->PATH ; path != NULL ; path = path->NEXT)
if((path->TYPE & BLEEDER) != BLEEDER)
    {
    if(((path->TYPE & EXT) == EXT) && (ext_path == NULL))
        {
        ext_path = path ; /* tete des branches externes (apres classement) */
        ext = 'o' ;
        }
    if(((path->TYPE & VDD) == VDD) && (vdd_path == NULL))
        {
        vdd_path = path ; /* tete de VDD (apres classement) */
        vdd = 'o' ;
        }
    if(((path->TYPE & VSS) == VSS) && (vss_path == NULL))
        {
        vss_path = path ; /* tete des VSS (apres classement) */
        vss = 'o' ;
        }
    if((ext != 'n') && (vdd != 'n') && (vss != 'n')) break ;
    }
        
/*---------------------------------------------------------------------------*/
/* pour calculer FUP il faut prendre la branche la plus resistive entre      */
/* une externe et une VDD. Idem pour FDOWN, mais avec une branche VSS.       */
/* C'est la fonction decision() qui choisit la branche.                      */
/*---------------------------------------------------------------------------*/
slope = (front_list *)mbkalloc(sizeof(front_list)) ;

slope->FUP = valfupfdown(decision(ext,vdd,ext_path,vdd_path)) ;
slope->FDOWN = valfupfdown(decision(ext,vss,ext_path,vss_path)) ;

/* le resultat est range dans le champ USER du cone */
cone->USER = addptype(cone->USER,SLOPE,(char *)slope) ;

if(TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"---tas--- front : ") ;
    if(cone->PATH == NULL) fprintf(stdout,"no path\n") ;
    else
        {
        fprintf(stdout,"type of pathes are ") ;
        if(ext_path != NULL) fprintf(stdout,"extern ") ;
        if(vdd_path != NULL) fprintf(stdout,"VDD ") ;
        if(vss_path != NULL) fprintf(stdout,"VSS") ;
        fprintf(stdout,"\n") ;
        }
    fflush(stdout) ;
    }
return ;
}


/****************************************************************************/
/*                                  interface()                             */
/* calcule les capacites devant etre donnees pour chaque connecteur.        */
/****************************************************************************/
int interface(desbfig)
desafig_list *desbfig ;

{
locon_list *locon ;

/*-------------------------------------------------------------------*/
/* les resistances du fichier '.prf', je crois qu'elles ne servent a */
/* rien. Alors je ne calcule rien du tout jusqu'a avis contraire.    */
/* Ici on ne calcule que des capacites.                              */
/*-------------------------------------------------------------------*/
for(locon = desbfig->LOCON ; locon != NULL ; locon = locon->NEXT)
if((locon->DIRECTION != 'X') && (locon->DIRECTION != 'A'))
    {
    chain_list *hchain = getptype(locon->USER,CONE_TYPE)->DATA ,
               *chain  = hchain                                ;
    double     capa    = 0.0                                   ,
               clo     = 0.0                                   ;

    if(hchain != NULL)
    for( /* initialise */ ; chain != NULL ; chain = chain->NEXT)
        {
        list_list *path = ((cone_list *)chain->DATA)->PATH ;

        for( /* initialise */ ; path != NULL ; path = path->NEXT)
        if((path->TYPE & EXT) == EXT)
            {
            link_list *link ;

            /* recherche de la branche contenant le connecteur */
            for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
                link = link->NEXT) ;

            /* si dernier maillon pas connecteur => avertissement */
            if((link->TYPE & (IN | INOUT)) == 0)
            tas_error(34,(char *)chain->DATA,TAS_WARNING) ;

            if(link->TRANS == (lotrs_list *)locon)
                {
                clo = link->CAPA ;

                if(CAPASWITCH != 0.0)
                for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
                    link = link->NEXT)
                    {
                    if(link->TRANS->SOURCE != (locon_list *)hchain)
                        {
                        capa += link->CAPA ;
                        /*---------------------------------------------------*/
                        /* on tag pour ne pas prendre la meme capa plusieurs */
                        /* fois.                                             */
                        /*---------------------------------------------------*/
                        link->TRANS->SOURCE = (locon_list *)hchain ;
                        }
                    }
                else break ;
                }
            } /* fin des branches */

        if(path != NULL) break ;
        } /* fin des chains */
    else
        {
        hchain = getptype(locon->USER,EXT)->DATA ;

        if(hchain == NULL)
        tas_error(4,locon->NAME,TAS_WARNING) ;
        else
        clo = ((link_list *)((cone_list *)hchain)->PATH->DATA)->CAPA ;
        }

    clo += capa*CAPASWITCH ;
    locon->USER = addptype(locon->USER,CAPA_TSP,(long)clo) ;
    } /* fin du parcours des connecteurs */

return ;
}


/*****************************************************************************
*                                function timing()                           *
*            depart du clacul des fronts et des temps elementaires           *
*****************************************************************************/
int timing(desafig)
desafig_list *desafig ;

{
FILE      *file ;
cone_list *cone ;
list_list *vdd  ;
list_list *vss  ;
list_list *ext  ;

tas_DetectConeConnect(desafig) ;
prelink(desafig) ;
tas_capara(desafig) ;

if(TAS_BULLIT != 'Y')
tas_StuckAt(desafig) ;

if(TAS_BULLIT == 'Y')
tas_BullitLatch(desafig) ;

vdd = (list_list *)mbkalloc(sizeof(list_list)) ;
vss = (list_list *)mbkalloc(sizeof(list_list)) ;
ext = (list_list *)mbkalloc(sizeof(list_list)) ;

if (TAS_TRACE_MODE == 'Y') 
    {
    fprintf(stdout,"\n---tas--- TIMING FOR FRONT AND TPxx\n");
    fprintf(stdout,"---tas--- 1ST STAPE PRETRAITEMENT & FRONT\n\n\n");
    fflush(stdout) ;
    }

for(cone = desafig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    if(TAS_TRACE_MODE == 'Y')
    fprintf(stdout,"---tas--- timing : cone %ld (%s)\n",cone->INDEX,cone->NAME);

    if((cone->TYPE & PATH_PARA) == PATH_PARA) para(cone) ;
    if((cone->TYPE & SWITCH ) == SWITCH) switch_cmos(cone) ;

    res_branche(cone) ;
    clas_branche(vdd,vss,ext,cone) ;

    front(cone) ;

    if(TAS_TRACE_MODE == 'Y')
        {
        front_list *slope = (front_list *)((getptype(cone->USER,SLOPE))->DATA) ;

        fprintf(stdout,"---tas--- timing : cone %ld (%s) FUP=",cone->INDEX,
                cone->NAME) ;
        if(slope->FUP == NOFRONT) fprintf(stdout,"NOFRONT ") ;
        else fprintf(stdout,"%ld ",slope->FUP) ;
        fprintf(stdout,"FDOWN=") ;
        if(slope->FDOWN == NOFRONT) fprintf(stdout,"NOFRONT\n\n") ;
        else fprintf(stdout,"%ld\n\n",slope->FDOWN) ;
        }
    }

interface(desafig) ;

if(TAS_TRACE_MODE == 'Y')
    {
    fprintf(stdout,"\n\n---tas--- 2ND STAPE TPxx\n\n\n") ;
    fflush(stdout) ;
    }

for(cone = desafig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    if(TAS_TRACE_MODE =='Y')
        {
        fprintf(stdout,"\n---tas--- timing : cone %ld (%s)\n",cone->INDEX,
                cone->NAME);
        fflush(stdout) ;
        }
    tpd(desafig,cone) ;
    }
}
