/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Verison 3.54                                            */
/*    Fichier : dricar.c                                                    */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Payam KIANI                           le : 28/04/1992     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier dricar.c: driver du format .car, fichier en entree de addgen pour*/
/* la caracterisation des cellules de base                                  */
/****************************************************************************/
#include "tas_all.h"

/****************************************************************************/
/*                         fonction RetCapa()                               */
/* renvoie la somme des capacites vues par la branche la plus resistive     */
/* passee en parametre.                                                     */
/****************************************************************************/
long tas_RetCapa(cone,path)
cone_list *cone ;
list_list *path ;

{
list_list *auxp     ;
link_list *link     ;
long      capa  = 0 ;

/* activation des cones de la branche active */
for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
if((link->TYPE & (IN | INOUT)) == 0)
    {
    link->TRANS->DRAIN = (locon_list *)1 ;
    if((link->TYPE & TN) == TN) /* transistor NMOS => activation '1' */
    ((cone_list *)link->TRANS->GRID)->X1 = 11 ;
    else ((cone_list *)link->TRANS->GRID)->X1 = 10 ;
    }

for(auxp = cone->PATH ; auxp != NULL ; auxp = auxp->NEXT)
if(auxp != path)
    {
    link_list *auxl ;

    for(auxl = (link_list *)path->DATA , link = auxl->NEXT ; link != NULL ;
        auxl = auxl->NEXT , link = link->NEXT)
        {
        if((((auxl->TYPE & TN) == TN) && /* si NMOS pas commande par '0'... */
           (((cone_list *)auxl->TRANS->GRID)->X1 != 10))
          || /* ...ou PMOS pas commande par '1', on rajoute la capa */
           (((auxl->TYPE & TP) == TP) &&
           (((cone_list *)auxl->TRANS->GRID)->X1 == 11)))
            {
            if(link->TRANS->DRAIN == NULL) /* si ce n'est pas un trans commun */
            if(auxl->TRANS->DRAIN == NULL) 
            /* a voir s'il faut rajouter la capa ramenee ou non */
            capa += (long)link->CAPA ;
            }
        else /* sinon, les capas au-dela ne sont pas vues */
        break ;
        }
    }

/* reinitialisation */
for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
if((link->TYPE & (IN | INOUT)) == 0)
    {
    link->TRANS->DRAIN = NULL ;
    ((cone_list *)link->TRANS->GRID)->X1 = 10 ;
    }

return(capa) ;
}


/****************************************************************************/
/*                         fonction dricar()                                */
/* genere un fichier equivalent au perfmodule intermediaire mais dans le    */
/* format .car.                                                             */
/****************************************************************************/
int dricar(desbfig,perfig)
desafig_list *desbfig ;
perf_list    *perfig ;

{
char        fic[100]        ;
FILE        *file           ;
chemcr_list *couple  = NULL ;
long        Rup             ,
            Rdown           ;
locon_list  *locon          ;
cone_list   *cone           ;
lotrs_list  *lotrs          ;

if((file = mbkfopen(FILENAME,"car",WRITE_TEXT)) == NULL)
    {
    char straux[100] ;

    sprintf(straux,"%s.car",FILENAME) ;
    tas_error(8,straux,TAS_ERROR) ;

    return ;
    }

/* si figure vide => on ne met rien dans le fichier */
if(desbfig->TRANS == NULL)
fprintf(file,"c\n*\n%s\n*\n$",FILENAME) ;
else
    {
    /* initialisation */
    for(cone = desbfig->CONE ; cone != NULL ; cone = cone->NEXT)
        {
        cone->X1 = 0 ;
        cone->USER = NULL ;
        }
    
    for(lotrs = desbfig->TRANS ; lotrs != NULL ; lotrs = lotrs->NEXT)
    lotrs->DRAIN = NULL ;
    
    /* determination de l'element sequentiel ou combinatoire */
    if(desbfig->REG == NULL) fprintf(file,"c\n*\n") ;
    else fprintf(file,"s\n*\n") ;
    
    /* ecriture du nom de la cellule */
    fprintf(file,"%s\n*\n",desbfig->NAME) ;

    for(locon = desbfig->LOCON ; locon != NULL ; locon = locon->NEXT)
    if((locon->DIRECTION == 'O') || (locon->DIRECTION == 'B') ||
       (locon->DIRECTION == 'Z') || (locon->DIRECTION == 'T')) break ;

    /* ecriture de Cout */
    fprintf(file,"%ld",(long)getptype(locon->USER,CAPA_TSP)->DATA -
                       (long)(TAS_CAPAOUT*1000.0)                  ) ;
    
    /* ecriture de ROFF et de RON si pas de chemin dans le circuit */
    if(TAS_MAXPATH->UP == NULL)
        {
        list_list *path = (list_list *)getptype(locon->USER,EXT)->DATA ;

        if(path == NULL)
        fprintf(file,"\n*\n%ld\n*\n%ld",ROFF,ROFF) ;
        else
            {
            long rup = ROFF ,
                 rdw = ROFF ;

            path = ((cone_list *)path)->PATH ;

            for( /* initialise */ ; path != NULL ; path = path->NEXT)
                {
                if(((path->TYPE & VDD) == VDD) && (rup == ROFF))
                rup = (long)path->USER ;
                if(((path->TYPE & VSS) == VSS) && (rdw == ROFF))
                rdw = (long)path->USER ;
                if((path->TYPE & EXT) == EXT)
                    {
                    if(rup == ROFF) rup = (long)path->USER ;
                    if(rdw == ROFF) rdw = (long)path->USER ;
                    }
                } /* fin des chemins */

            fprintf(file,"\n*\n%ld\n*\n%ld",rup,rdw) ;
            } /* fin de else */
        } /* fin de pas de chemin */
    
    cone = (cone_list *)getptype(locon->USER,EXT)->DATA ;

    if(cone == NULL)
    tas_error(4,locon->NAME,TAS_ERROR) ;
    
    /*-----------------------------------------------------------------------*/
    /* si on est dans le cas d'un latch, on ecrit les resistances de sortie: */
    /* une RON et une ROFF. Mais ATTENTION!!!!!!! On suppose que la sortie de*/
    /* la cellule sequentielle est toujours un inverseur. On calucle donc le */
    /* courant max dans cette branche et on divise par VDD/2.                */
    /*-----------------------------------------------------------------------*/
    if(desbfig->REG != NULL)
        {
        link_list  *link1    = (link_list *)cone->PATH->DATA       ,
                   *link2    = (link_list *)cone->PATH->NEXT->DATA ;
        double     ron       = VDDmax/(2*i_branch(link1,link1))    ,
                   roff      = VDDmax/(2*i_branch(link2,link2))    ;
        locon_list *auxlocon                                       ;
    
        if((cone->PATH->TYPE & VSS) == VSS) /* determination de ron et roff */
            {
            double res = ron ;
    
            ron = roff ;
            roff = res ;
            }
    
        fprintf(file,"\n*\n%ld\n*\n%ld",(long)ron,(long)roff) ;
    
        /* liste des capa d'entree: que pour les connecteurs IN et TRANSCVR */
        for(auxlocon = desbfig->LOCON ; auxlocon != NULL ;
            auxlocon = auxlocon->NEXT)
        if(
           (auxlocon->DIRECTION != 'A') &&
           (auxlocon->DIRECTION != 'O') &&
           (auxlocon->DIRECTION != 'B') &&
           (auxlocon->DIRECTION != 'Z')
          )
        if(auxlocon != locon) /* pas de parametres pour le connect de sortie */
            {
            long capa = (long)getptype(auxlocon->USER,CAPA_TSP)->DATA ;

            /*--------------------------------------------------------------*/
            /* sur les connecteurs de type 'I', aucune capa n'a ete ajoutee */
            /* La capa associee est donc directement la capacite lue. Par   */
            /* contre pour les connecteurs de type 'T', TAS_CAPAOUT a ete   */
            /* ajoutee: il faut la retrancher.                              */
            /*--------------------------------------------------------------*/
            if(auxlocon->DIRECTION == 'T')
            capa -= (long)(TAS_CAPAOUT*1000.0) ;

            fprintf(file,"\n*\n%s\n%ld",auxlocon->NAME,capa) ;
            }
        } /* fin du cas des registres */
    
    /*-----------------------------------------------------------------*/
    /* si on est dans le cas d'un element combinatoire, il faut donner */
    /* tous les renseignements: resistance par entree, temps de propa- */
    /* gation, etc...                                                  */
    /* Si par contre, c'est un element sequentiel, il faut seulement   */
    /* les capas d'entree et de sortie et une RON et une ROFF.         */
    /*-----------------------------------------------------------------*/
    else
    for(couple = perfig->COUPLE ; couple != NULL ; couple = couple->NEXT)
    if(couple->USER == NULL)
        {
        char *firstname                      ;
        chemcr_list *aux                     ;
        event_list *nodein  = couple->UP     ;
        event_list *nodeout = couple->BOTTOM ;
        long       capa     = 0              ;

        /* nom de l'entree */
        if((nodein->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
            {
            firstname = ((locon_list *)nodein->CONE)->NAME ;
            /* capacite d'entree */
            capa = (long)getptype(((locon_list *)nodein->CONE)->USER,
                                  CAPA_TSP)->DATA ;

            /* meme raisonnement que pour les registres (voir plus haut) */
            if(((locon_list *)nodein->CONE)->DIRECTION == 'T')
            capa -= (long)(TAS_CAPAOUT * 1000.0) ;
            }
        else /* si l'entree est un cone */
            {
            firstname = nodein->CONE->NAME ;
            /* capacite d'entree */
            capa = (long)((link_list *)nodein->CONE->PATH->DATA)->CAPA ;
            }
        fprintf(file,"\n*\n%s\n%ld",firstname,capa) ;
    
        for(aux = couple ; aux != NULL ; aux = aux->NEXT)
        if(aux->USER == NULL)
            {
            event_list *auxnin  = aux->UP                                     ;
            event_list *auxnout = aux->BOTTOM                                 ;
            long       res      = getrs(aux)                                  ,
                       delai    = aux->DELAY - (long)(res*TAS_CAPAOUT)        ;
    
            if(
               (auxnin->CONE  == nodein->CONE ) &&
               (auxnout->CONE == nodeout->CONE)
              )
                {
                /* temps de propagation */
                if(LEVEL(auxnout) == HIGHLEVEL) /* sortie niv haut: LH ou HH */
                    {
                    if(LEVEL(auxnin) != 0) /* cas TPHH */
                    fprintf(file,"\nTPHH\n%ld",delai) ;
                    else /* cas TPLH */
                    fprintf(file,"\nTPLH\n%ld",delai) ;
                    }
                else /* cas TPLL ou TPHL */
                    {
                    if(LEVEL(auxnin) != 0) /* cas TPHL */
                    fprintf(file,"\nTPHL\n%ld",delai) ;
                    else /* cas TPLL */
                    fprintf(file,"\nTPLL\n%ld",delai) ;
                    }
    
                /* ecriture de la resistance de sortie */
                fprintf(file,"\n%ld",res) ;
    
                aux->USER = (ptype_list *)1 ;
                }
            }
        }
    
    for(couple = perfig->COUPLE ; couple != NULL ; couple = couple->NEXT)
    couple->USER = NULL ;

    fprintf(file,"\n*\n$\n") ;
    }

if(fclose(file) != 0)
    {
    char straux[100] ;

    sprintf(straux,"%s.car",FILENAME) ;
    tas_error(10,straux,TAS_WARNING) ;
    }

return ;
}
