/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Verison 3.54                                            */
/*    Fichier : drittv.c                                                    */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Payam KIANI                           le : 28/05/1992     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier drittv.c : driver du perfmodule general dans le nouveau format   */
/****************************************************************************/
#include "tas_all.h"
list_list *pathcomm = NULL ; /* branche commutante */


/******************************************************************************/
/*                               function criticpath()                        */
/* ecrit le chemin critique complet dans le perfmodule en commentaire.        */
/******************************************************************************/
int criticpath(file,chemcr,comment)
FILE        *file    ;
chemcr_list *chemcr  ;
char        *comment ;

{
critic_list *previous ,
            *next     ;

if(chemcr->PATH != NULL)
for(previous = chemcr->PATH , next = previous->NEXT ;
    previous->NODE->CONE != chemcr->BOTTOM->CONE ;
    previous = next , next = next->NEXT)
    {
    event_list  *nodein  = previous->NODE ,
                *nodeout = next->NODE     ;

    /* on ne met pas le cone connecteur */
    if(
       ((nodeout->TYPE & tas_CONNECTOR)      != 0) ||
       ((nodeout->CONE->TYPE & CONE_CONNECT) == 0)
      )
        {
        /* pour mettre en commentaire dans le ".int" ou le ".dtv" */
        fprintf(file,"\n%s ",comment) ;
 
        /* ecriture du noeud entree */
        if(LEVEL(nodein) != 0) fprintf(file,"UP(") ;
        else fprintf(file,"DOWN(") ;
        fprintf(file,"%s) => ",tas_NODENAME(nodein)) ;
 
        /* ecriture du noeud sortie */
        if(LEVEL(nodeout) != 0) fprintf(file,"UP(") ;
        else fprintf(file,"DOWN(") ;
        fprintf(file,"%s), Tp=%ld pS",tas_NODENAME(nodeout),
                previous->DELAY-next->DELAY) ;

        fflush(file) ;
        }
    }

return ;
}


/******************************************************************************/
/*                               foction getrs()                              */
/* renvoie la resistance de sortie pour un couple dont la sortie est un connec*/
/******************************************************************************/
long getrs(couple)
chemcr_list *couple ;

{
long typetr = (LEVEL(couple->BEFORE) != 0) ? TN : TP ;
long typebr = (LEVEL(couple->BOTTOM) != 0) ? VDD : VSS ;
cone_list *cone = couple->BOTTOM->CONE ;
list_list *path ;
link_list *link ;
link_list *maillon ;
double    rs = 0.0 ;

/*--------------------------------------------------------------------*/
/* si c'est une branche alim, on fait le calcul pour RS. Si c'est une */
/* branche externe, on renvoie la resistance de la branche.           */
/*--------------------------------------------------------------------*/
if((couple->BEFORE->TYPE & tas_CONNECTOR) != tas_CONNECTOR)
    {
    pathcomm = path =
    existe_tpd(cone,couple->BEFORE->CONE,typebr,typetr,&maillon) ;

    if(path == NULL)
    return(0) ;

    /*------------------------------------------------------------------*/
    /* le calcul de la resistance se fait de la maniere suivante: si la */
    /* branche ne degrade pas le niveau, on calcule imax de la branche  */
    /* (fonction i_branch) et la resistance est VDD/2Imax. Par contre   */
    /* si la branche degrade le niveau, on suppose que les transistors  */
    /* degradant sont vers le sommet du cone. Alors tant qu'on ne tombe */
    /* pas sur un bon transistor, on fait pour chaque transistor:       */
    /*                   Req = RNUD*L/W pour le NMOS                    */
    /*                   Req = RPDD*L/W pour le PMOS                    */
    /* Arrive au bon transistor, on calcule le Imax et on ajoute Req    */
    /* a partir de la qui est VDD/2Imax.                                */
    /*------------------------------------------------------------------*/
    for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
        link = link->NEXT) maillon = link ;
    if((link->TYPE & (IN | INOUT)) != 0) link = maillon ;

    if((path->TYPE & (TPINVSS | TNINVDD | EXT)) == 0) /* pas degradant */
    rs = VDDmax/(2*i_branch((link_list *)path->DATA,link)) ;

    else /* branche degradante ou externe */
        {
        for(maillon = (link_list *)path->DATA ; maillon != NULL ;
            maillon = maillon->NEXT)
            {
            /*---------------------------------------------------------*/
            /* des qu'on trouve un transistor homogene, il faut que la */
            /* resistance soit calculee par mesure de courant, sinon   */
            /* par des resistances equivalentes.                       */
            /*---------------------------------------------------------*/
            if(((typebr == VDD) && ((maillon->TYPE & TP) == TP)) ||
               ((typebr == VSS) && ((maillon->TYPE & TN) == TN)))
            break ;

            if((maillon->TYPE & TP) == TP)
            rs += RPDD*(double)maillon->TRANS->LENGTH/
                       (double)maillon->USER->TYPE ;
            else if((maillon->TYPE & TN) == TN)
            rs += RNUD*(double)maillon->TRANS->LENGTH/
                       (double)maillon->USER->TYPE ;
            }

        if(maillon != NULL)
        rs += (VDDmax/(2*i_branch(maillon,link))) ;
        }
    }

else /* entree connecteur */
    {
    for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        {
        for(link = (link_list *)path->DATA ; link->NEXT != NULL ;
            link = link->NEXT) ;
        if(link->TRANS == (lotrs_list *)couple->BEFORE->CONE) break ;
        }
    rs = (double)((long)path->USER) ;
    }

return((long)rs) ;
}


/***************************************************************************/
/*                         fonction getcmd()                               */
/* renvoie, si elle existe, la commande qui concerne la donnee arrivant a  */
/* un point latch par une entree donnee.                                   */
/***************************************************************************/
link_list *getcmd(couple)
chemcr_list *couple ;

{
list_list *path                     ;
event_list *before = couple->BEFORE ,
           *bottom = couple->BOTTOM ;

if((before->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    {
    char lev = (LEVEL(before) == HIGHLEVEL) ? 'U' : 'D' ;

    if((path = tas_GiveActivExtPath(bottom->CONE,before->CONE,lev)) != NULL)
        {
        link_list *cmd ;

        for(cmd = (link_list *)path->DATA ;
            (cmd != NULL) &&
            ((tas_GetInLink(bottom->CONE,cmd)->TYPE & COMMAND) != COMMAND) ;
            cmd = cmd->NEXT) ;

        if(cmd != NULL)
        return(cmd) ;
        }
    }
else
for(path = bottom->CONE->PATH ; path != NULL ; path = path->NEXT)
if((path->TYPE & BLEEDER) != BLEEDER)
if(
   ((path->TYPE & EXT ) == EXT)                                  ||
   (((path->TYPE & VDD) == VDD) && (LEVEL(bottom) == HIGHLEVEL)) ||
   (((path->TYPE & VSS) == VSS) && (LEVEL(bottom) != HIGHLEVEL))
  )
    {
    if((before->TYPE & tas_CONNECTOR) != tas_CONNECTOR)
        {
        link_list *link ,
                  *cmd  ;

        for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
        if((link->TYPE & (IN | INOUT)) == 0)
        if(link->TRANS->GRID == (locon_list *)before->CONE)
            {
            /* recherche de l'entree commande de la branche */
            for(cmd = (link_list *)path->DATA ;
                (cmd != NULL) &&
                ((tas_GetInLink(bottom->CONE,cmd)->TYPE & COMMAND) != COMMAND) ;
                cmd = cmd->NEXT) ;

            if(cmd != NULL)
            return(cmd) ;

            break ;
            }
        }
    else
        {
        link_list *cmd  ,
                  *link ;

        for(link = (link_list *)path->DATA ; link != NULL ; link = link->NEXT)
        if(link->TRANS == (lotrs_list *)before->CONE)
            {
            /* recherche de l'entree commande de la branche */
            for(cmd = (link_list *)path->DATA ;
                (cmd != NULL) &&
                ((tas_GetInLink(bottom->CONE,cmd)->TYPE & COMMAND) != COMMAND) ;
                cmd = cmd->NEXT) ;

            if(cmd != NULL)
            return(cmd) ;

            break ;
            }
        }
    } /* fin du parcours des branches */

return(NULL) ;
}


/***************************************************************************/
/*                         fonction getdir()                               */
/* elle renvoie le caractere corresondant a la direction du connecteur     */
/***************************************************************************/
char getdir(locon)
locon_list *locon ;

{
char direction = (locon->DIRECTION == 'X') ? 'U' : locon->DIRECTION ;

return(direction) ;
}


/***************************************************************************/
/*                         fonction tas_tvheader()                         */
/* ecrit l'entete (article G), les connecteurs (article X), les latches    */
/* (article L) et les points precharges (article P) de la figure.          */
/***************************************************************************/
int tas_tvheader(desbfig,file,type)
desafig_list *desbfig ;
FILE         *file    ;
char         type     ; /* dit si c'est un ttv ou un dtv */

{
struct tm   *tm     ;
long        datesys ;
int         i       ;
locon_list  *locon  ;
chain_list  *reg    ;
cone_list   *cone   ;
precharge_list *precharge ;

/* recherche de la date de la creation du fichier */
time(&datesys) ;
tm = gmtime(&datesys) ;

/*********************      entete du perfmodule general *********************/
if(type == 't') /* fichier ttv */
    {
    fprintf(file,"#Perfmodule %s\n",desbfig->NAME) ;
    fprintf(file,"#TAS version %s ",VERSION) ;
 
    if(TAS_LANG == 'E')
        {
        fprintf(file,"of %s\n",DATE_VER) ;
        fprintf(file,"#out of path capacitance factor   (opc)  = %.2lf%%\n",
                CAPARAPREC*100.0) ;
        fprintf(file,"#input switch capacitance factor  (swc)  = %.2lf%%\n",
                CAPASWITCH*100.0) ;
        fprintf(file,"#output capacitance               (cout) = %.3lfpF\n",
                TAS_CAPAOUT) ;
        fprintf(file,"#command line:") ;
        for(i = 0 ; i != TAS_INFO.argc ; i++)
        fprintf(file," %s",TAS_INFO.argv[i]) ;
        fprintf(file,"\n#technology: %s -- V%.2f\n",elpTechnoName,
                elpTechnoVersion) ;
        fprintf(file,"#worst case delay = %ld\n",TAS_MAXPATH->DELAY) ;
        }
    else if(TAS_LANG == 'F')
        {
        fprintf(file,"du %s\n",DATE_VER) ;
        fprintf(file,"#facteur de capacite hors-chemin  (opc)  = %.2lf%%\n",
                CAPARAPREC*100.0) ;
        fprintf(file,"#facteur de switch d'antree       (swc)  = %.2lf%%\n",
                CAPASWITCH*100.0) ;
        fprintf(file,"#capacite de charge               (cout) = %.3lfpF\n",
                TAS_CAPAOUT) ;
        fprintf(file,"#ligne de commande:") ;
        for(i = 0 ; i != TAS_INFO.argc ; i++)
        fprintf(file," %s",TAS_INFO.argv[i]) ;
        fprintf(file,"\n#technologie: %s -- V%.2f\n",elpTechnoName,
                elpTechnoVersion) ;
        fprintf(file,"#plus long delai = %ld\n",TAS_MAXPATH->DELAY) ;
        }

    /* ecriture du chemin critique */
    if(TAS_MAXPATH->UP != NULL)
        {
        if(LEVEL(TAS_MAXPATH->UP) == HIGHLEVEL)
        fprintf(file,"#UP(") ;
        else fprintf(file,"#DOWN(") ;
        
        fprintf(file,"%s) ---> ",tas_NODENAME(TAS_MAXPATH->UP)) ;
        
        if(LEVEL(TAS_MAXPATH->BOTTOM) == HIGHLEVEL)
        fprintf(file,"UP(") ;
        else fprintf(file,"DOWN(") ;
        
        fprintf(file,"%s) --- Tp=%ld pS",tas_NODENAME(TAS_MAXPATH->BOTTOM),
                TAS_MAXPATH->DELAY) ;
        criticpath(file,TAS_MAXPATH,"#\t") ;
        fprintf(file,"\n\n") ;
        }
    } 
else /* alors c'est le dtv */
    {
    if(TAS_LANG == 'E')
    fprintf(file,"#intermediate Perfmodule %s\n",desbfig->NAME) ;
    else if(TAS_LANG == 'F')
    fprintf(file,"#Perfmodule intermediaire %s\n",desbfig->NAME) ;
    fprintf(file,"#TAS version %s ",VERSION) ;
    if(TAS_LANG == 'E')
        {
        fprintf(file,"of %s\n",DATE_VER) ;
        fprintf(file,"#out of path capacitance factor   (opc)  = %.2lf%%\n",
                CAPARAPREC*100.0) ;
        fprintf(file,"#input switch capacitance factor  (swc)  = %.2lf%%\n",
                CAPASWITCH*100.0) ;
        fprintf(file,"#output capacitance               (cout) = %.3lfpF\n",
                TAS_CAPAOUT) ;
        fprintf(file,"#command line:") ;
        for(i = 0 ; i != TAS_INFO.argc ; i++)
        fprintf(file," %s",TAS_INFO.argv[i]) ;
        fprintf(file,"\n#technology: %s -- V%.2f\n",elpTechnoName,
                elpTechnoVersion) ;
        }
    else if(TAS_LANG == 'F')
        {
        fprintf(file,"du %s\n",DATE_VER) ;
        fprintf(file,"#facteur de capacite hors-chemin  (opc)  = %.2lf%%\n",
                CAPARAPREC*100.0) ;
        fprintf(file,"#facteur de switch d'antree       (swc)  = %.2lf%%\n",
                CAPASWITCH*100.0) ;
        fprintf(file,"#capacite de charge               (cout) = %.3lfpF\n",
                TAS_CAPAOUT) ;
        fprintf(file,"#ligne de commande:") ;
        for(i = 0 ; i != TAS_INFO.argc ; i++)
        fprintf(file," %s",TAS_INFO.argv[i]) ;
        fprintf(file,"\n#technologie: %s -- V%.2f\n",elpTechnoName,
                elpTechnoVersion) ;
        }
    }

/*-----------------------------------------------------------------------*/
/* entete: nom du fichier, nom de la techno utilisee, le front suppose   */
/* sur les connecteurs d'entree,nom du programme ayant genere ce fichier,*/
/* son numero de version, la date et l'heure de creation, le niveau de   */
/* finesse et le nom de user                                             */
/*-----------------------------------------------------------------------*/
fprintf(file,"\nG %s %s %ld TAS %s (%d %d %d %d %d %d) %d *;\n\n",desbfig->NAME,
        elpTechnoName,(long)FRONT_CON,VERSION,tm->tm_mday,tm->tm_mon,
        tm->tm_year,tm->tm_hour,tm->tm_min,tm->tm_sec,tas_LEVEL) ;
 
/*********************   liste des connecteurs           *********************/
for(locon = desbfig->LOCON ; locon != NULL ; locon = locon->NEXT)
if(locon->DIRECTION != 'A')
    {
    long capa = (long)getptype(locon->USER,CAPA_TSP) ;

    if(capa == (long)NULL)
    capa = 0 ;
    else
    capa = (long)((ptype_list *)capa)->DATA ;

    /* direction du connecteur, son nom et sa capacite */
    fprintf(file,"X %c %s %ld;\n",getdir(locon),locon->NAME,capa) ;
    }
fprintf(file,"\n") ;

/*********************   liste des precharges            *********************/
if(type == 't') /* dans le cas d'un ttv */
for(precharge = tas_PRECHR ; precharge != NULL ; precharge = precharge->NEXT)
    {
    chain_list *cmd ;

    fprintf(file,"P %s ( ",precharge->CONE->NAME) ;

    for(cmd = precharge->CMD ; cmd != NULL ; cmd = cmd->NEXT)
    fprintf(file,"%s ",(char *)cmd->DATA) ;

    fprintf(file,");\n") ;
    }
else /* dans le cas d'un dtv */
for(cone = desbfig->CONE ; cone != NULL ; cone = cone->NEXT)
if((cone->TYPE & PRECHARGE) == PRECHARGE)
fprintf(file,"P %s ( );\n",cone->NAME) ;

fprintf(file,"\n") ;

/*********************   liste des latches               *********************/
for(reg = desbfig->REG ; reg != NULL ; reg = reg->NEXT)
    {
    list_list *input                          ;

    cone  = (cone_list *)reg->DATA ;
    /* nom du latch suivi de son type et de la liste des comandes */
/*    fprintf(file,"L S %s ( ",cone->NAME) ;*/
    fprintf(file,"L %s ( ",cone->NAME) ;

    /*--------------------------------------------------------------*/
    /* ecirutre des commandes du latch. Si la commande est NMOS, la */
    /* commande est ecrite telle quelle. Sinon, si c'est un PMOS,   */
    /* il faut l'ecrireavec un back quote. Il faut donc parcourir   */
    /* les maillons pour savoir quel est le type du transistor.     */
    /*--------------------------------------------------------------*/
    for(input = cone->INCONE ; input != NULL ; input = input->NEXT)
    if((input->TYPE & (COMMAND | BLEEDER)) == COMMAND)
        {
        list_list *path ;

        for(path = cone->PATH ; path != NULL ; path = path->NEXT)
        if((path->TYPE & BLEEDER) != BLEEDER)
            {
            link_list *link ;

            for(link = (link_list *)path->DATA ; link != NULL ;
                link = link->NEXT)
            if((link->TYPE & (TN | TP)) != 0) /* si c'est bien un trans */
            if(link->TRANS->GRID == (locon_list *)input->DATA)
                {
                if((link->TYPE & TP) == TP)
                fprintf(file,"`") ;
                fprintf(file,"%s ",((cone_list *)input->DATA)->NAME) ;
                break ; /* arret de la boucle des maillons */
                }

            /* si le maillon est trouve, pas la peine d'aller + loin */
            if(link != NULL)
            break ; /* arret de la boucle des branches */
            } /* fin de la boucle des branches */
        } /* fin du test entree commande */

    fprintf(file,");\n") ;
    }

fprintf(file,"\n") ;

return ;
}


/***************************************************************************/
/*                         fonction WriteTran()                            */
/* ecrit les transitions possibles pour une liste donnee de chemins.       */
/***************************************************************************/
int tas_WriteTran(cple,file)
chemcr_list *cple ;
FILE        *file ;

{
chemcr_list *couple    ;
char        *firstname ;
event_list  *nodein    ;
event_list  *nodeout   ;
cone_list   *coneout   ;
chemcr_list *couplebis ;

for(couple = cple ; couple != NULL ; couple = couple->NEXT)
if(couple->USER == NULL)
    {
    nodein = couple->UP ;
    nodeout = couple->BOTTOM ;
    coneout = nodeout->CONE ;
    firstname = tas_NODENAME(nodein) ;
    
    if((nodeout->TYPE & LCOMMAND) == LCOMMAND)
        {
        link_list *link = (link_list *)nodeout->CONE->PATH->DATA ;
    
        if((link->TYPE & (IN | INOUT)) != 0)
        if(strcmp(((locon_list *)link->TRANS)->NAME,firstname) == 0) continue ;
        }

    fprintf(file,"T %c %s %s (",couple->TYPETR,firstname,
            tas_NODENAME(nodeout)) ;
/* TOTO */
/*    fprintf(file,"T %c %c %s %s (",couple->TYPETR,couple->TYPETR,firstname,
            tas_NODENAME(nodeout)) ;*/

    /*----------------------------------------------------------------------*/
    /* les couples sot ordonnes. En effet, ils sont par liste, chaque liste */
    /* etant reconnue par la sortie du chemin. Si dans le parcours des      */
    /* chemins on n'a plus sorite = coneout, on ne l'aura plus jamais!      */
    /*----------------------------------------------------------------------*/
    for(couplebis = couple ;
        (couplebis != NULL) && (couplebis->BOTTOM->CONE == coneout) ;
        couplebis = couplebis->NEXT)
    /* il faut que les couples aient le meme type */
    if(couplebis->TYPETR == couple->TYPETR)
    /* et il faut tester que le chemin n'a pas ete traite */
    if(couplebis->USER == NULL)
        {
        if(couplebis->UP->CONE == nodein->CONE)
            {
            float rs    = (float)getrs(couplebis) / 1000.0                ;
            long  trc   = (long)
                          (((link_list *)coneout->PATH->DATA)->CAPA * rs) ;
            char  *cmd                                                    ;
    
    
            /* recherche de la commande eventuellement associee */
            if((nodeout->TYPE & LATCH) == LATCH)
            cmd = ((cone_list *)getcmd(couplebis)->TRANS->GRID)->NAME ;
            else cmd = namealloc("*") ;
    
            /* si la sortie n'est pas un connecteur => pas res, frnt, etc... */
            if(((coneout->TYPE & EXT) != EXT) || (pathcomm == NULL))
/* TOTO */
/*            fprintf(file,"\n\t(%s (%c%c MAX %ld %ld))",cmd,
                    couplebis->FRIN,couplebis->FROUT,couplebis->DELAY,trc) ;*/
            fprintf(file,"\n\t(%s (%c%c %ld))",cmd,
                    couplebis->FRIN,couplebis->FROUT,couplebis->DELAY) ;
            else /* si connecteur => res, front, cte de front */
/*            fprintf(file,"\n\t(%s (%c%c MAX %ld %ld %.2f 0 %ld))",cmd,
                    couplebis->FRIN,couplebis->FROUT,couplebis->DELAY,trc,rs,
                    valfupfdown(pathcomm)) ;*/
            fprintf(file,"\n\t(%s (%c%c %ld %.2f %ld))",cmd,
                    couplebis->FRIN,couplebis->FROUT,couplebis->DELAY,rs,
                    valfupfdown(pathcomm)) ;

            pathcomm = NULL ;
    
            /* si le chemin entier est demande (option "-x") */
            if(couplebis->PATH != NULL)
            criticpath(file,couplebis,"#") ;
    
            couplebis->USER = (ptype_list *)1;
            }
        }

    fprintf(file,"\n);\n") ;
    }
}


/***************************************************************************/
/*                         fonction drittv()                               */
/* driver du nouveau format de perfmodule.                                 */
/***************************************************************************/
int drittv(ptfig,perfig)
desafig_list *ptfig ;
perf_list *perfig ;

{
FILE        *file   = mbkfopen(ptfig->NAME,"ttv",WRITE_TEXT) ;
long        counter                                          ;
chemcr_list *couple                                          ;

/* test d'ouverture de fichier */
if(file == NULL)
    {
    char straux[100] ;

    sprintf(straux,"%s.ttv",ptfig->NAME) ;
    tas_error(8,straux,TAS_ERROR) ;
    }

/* pour avoir le temps d'ecriture du fichier */
time(&counter) ;

/* en-tete, connecteur, latch et precharge */
tas_tvheader(ptfig,file,'t') ;

/*********************  ecriture des couples et chaines  *********************/
tas_WriteTran(perfig->COUPLE,file) ;

if(PRFTYPE != '2')
tas_WriteTran(perfig->CHAINE,file) ;
else
for(couple = perfig->CHAINE ; couple != NULL ; couple = couple->NEXT)
    {
    event_list *nodeout = couple->BOTTOM ;
    char      *cmd                      ;

    if((nodeout->TYPE & LATCH) == LATCH)
    cmd = ((cone_list *)getcmd(couple)->TRANS->GRID)->NAME ;
    else cmd = namealloc("*") ;

    /*-----------------------------------------------------------------------*/
    /* si l'ancien format de perfmodule est egalement genere, les delais des */
    /* chemins complets seront de la forme XX. Sinon les commutations seront */
    /* ecrite, puisqu'elles sont connues.                                    */
    /*-----------------------------------------------------------------------*/
    /* TOTO */
/*    fprintf(file,"T X X %s %s (\n\t(%s (XX MAX %ld)));\n",
            couple->UP->CONE->NAME,nodeout->CONE->NAME,cmd,
            couple->DELAY) ;*/
    fprintf(file,"T X %s %s (\n\t(%s (XX %ld)));\n",
            couple->UP->CONE->NAME,nodeout->CONE->NAME,cmd,
            couple->DELAY) ;
    }

/*********************  fin et fermeture du fichier      *********************/
fprintf(file,"E;\n") ;

if(fclose(file) != 0)
    {
    char straux[100] ;

    sprintf(straux,"%s.ttv") ;
    tas_error(10,straux,TAS_WARNING) ;
    }

return ;
}
