/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.50                                            */
/*    Fichier : tas.h                                                       */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Amjad HAJJAR et Payam KIANI           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier tas.h : les structures et base de donnees de TAS                 */
/****************************************************************************/

#ifdef THOMSON
#define BULL
#endif

/* defines utilises pour des affectations */
#define NOTIME          ((long) 0x80000000)  /* pas de TPxx  */
#define NOFRONT         ((long) 0x80000000)  /* pas de front */
#define ROFF            ((long)  999990   )
#define DELAI_MIN       ((long) 1         ) /* temps minimal: 1pS */
#define MAX_BLOC_NODE   512
#define tas_ZE          ((char) '\1')
#define tas_ZP          ((char) '\2')

/* mots clefs des champs USER */
#define SLOPE           ((long) 0x0000FFFF)

/* defines utilises pour masquage pour cones ou noeuds */
#define MARQUE          0x80000000
#define DEMARQUE        (~(MARQUE))
#define OUTFIFO         (~(INFIFO))
#define LCOMMAND         ((long) 0x20000000)
#define DEJAEMPILE       ((long) 0x10000000)
#define REFERE           ((long) 0x08000000)
#define CONE_CONNECT     ((long) 0x04000000) /* pour typer un cone connect. */
#define PATH_INOUT       ((long) 0x02000000) /* branches INOUT              */
#define tas_BYPASSIN     ((long) 0x01000000) /* cone non-important pour IN  */
#define tas_BYPASSOUT    ((long) 0x00200000) /* cone non-important pour OUT */

/* TYPE DES NOEUDS */
#define HIGHLEVEL     ((long) 0x00000001 )   /* noeud niveau haut */
#define DEBUT         ((long) 0x00000002 )   /* debut de chaine   */
#define CONE_TYPE     ((long) 0x00000008 )   /*        C        */
#define LATCH         ((long) 0x00001000 )   /*        L        */
#define tas_CONNECTOR ((long) 0x80000000 )   /* noeud connecteur  */
#define tas_NOPRECH   ((long) 0x00400000 )   /* noeud connecteur  */
#define tas_PRECHARGE ((long) 0x00800000 )   /* noeud precharge   */
#define INFIFO        ((long) 0x40000000 )   /* noeud deja empile */

/* MARAQUAGE DES TRANSISTORS */
#define BTSWITCH        ((long) 0x01000000) /* part of switch         */
#define BTCONFL         ((long) 0x02000000) /* in conflictual branch  */
#define UCONFL          ((long) 0x04000000) /* used in conflict       */
#define BTOPP           ((long) 0x08000000) /* in opposite branch     */
#define BTFB            ((long) 0x10000000)

/* LES PSEUDO-FONCTIONS */
#define LEVEL(ptr)           ((ptr)->TYPE & HIGHLEVEL)
#define ETAT_DEB(ptr)        ((ptr)->TYPE & DEBUT)
#define GetCapara(link)      ((long)link->USER->NEXT)
#define MAX(a,b)             ((a > b) ? a : b )
#define tas_F1(a,b,r)        ((double)(a*sqrt( b/(b+(r*a)) )))
#define tas_F2(a,b,r)        ((double)sqrt( b*(b+(r*a)) ))
#define tas_F3(a,b,i)        ((double) (((i*b)+sqrt(i*b*i*b+(4*a*i))) / (2*a)) )
#define tas_F4(u,vdd,fin)    ((double)(fin*u/vdd))
#define tas_TGH(x)           ((double)((exp(x)-exp((-x)))/(exp(x)+exp((-x)))))
#define tas_NODENAME(node)   (((node->TYPE & tas_CONNECTOR) == tas_CONNECTOR) ?\
                             ((locon_list *)node->CONE)->NAME                 :\
                             node->CONE->NAME                                 )


typedef struct delay
        {
        long    TPHH,TPHL;   /* delay time                     */
        long    TPLL,TPLH;  
        }
delay_list;


typedef struct front
        {
        long      FUP,FDOWN;            /* front time                      */
        }
front_list;


typedef struct event
    {
    long         DELAY ;
    long         TYPE ;
    struct cone  *CONE ;
    struct line  *INNODE ;
    struct event *OUTNODE ;
    struct ptype *USER ;
    }
event_list ;

typedef struct line
    {
    struct line  *NEXT ;
    struct event *NODE ;
    long         VAL ;
    }
line_list ;

typedef struct Nbloc 
    {
    struct Nbloc    *NEXT ;
    struct event    NODE[MAX_BLOC_NODE][2] ;
    }
Nbloc_list ;

typedef struct critic
    {
    struct critic *NEXT ;
    struct event  *NODE ;
    long          DELAY ;
    }
critic_list ;

typedef struct chemcr
    {
    struct chemcr *NEXT ;
    struct critic *PATH ;
    struct event  *UP ;
    struct event  *BOTTOM ;
    struct event  *BEFORE ;
    char          TYPETR ;
    char          FRIN ;
    char          FROUT ;
    long          DELAY ;
    struct ptype  *USER ;
    }
chemcr_list ;

typedef struct perf
    {
    struct chemcr *COUPLE ;
    struct chemcr *CHAINE ;
    }
perf_list ;

typedef struct biname
    {
    char          *Fname ;
    char          *Lname ;
    struct biname *NEXT ;
    }
biname_list ;

typedef struct precharge
    {
    struct precharge *NEXT ; /* precharge suivant                      */
    struct cone      *CONE ; /* cone correspondant                     */
    struct chain     *CMD  ; /* liste des commandes de latch associees */
    }
precharge_list ;

typedef struct information /* informations relatives au circuit */
    {
    long lo_t     ; /* temps du chargement de le netlist logique */
    long db_t     ; /* temps du desassemblage                    */
    long el_t     ; /* temps du calcul des temps elementaires    */
    long gr_t     ; /* temps du parcours de graphe               */
    long pr_t     ; /* temps de generation du perfmodule         */
    long nb_trans ; /* nombre de transistors                     */
    long nb_cone  ; /* nombre de cones                           */
    long comp     ; /* complexite du circuit                     */
    long maxdelay ; /* delai max du bloc                         */
    short argc    ; /* nombre d'arguments de la ligne de commande*/
    char **argv   ; /* arguments de la ligne de commande         */
    }
info_list ;
