/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 3.50                                            */
/*    Fichier : tasgraph.c                                                  */
/*                                                                          */
/*    (c) copyright 1991-1992 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Payam KIANI et Amjad HAJJAR           le : 28/05/1991     */
/*                                                                          */
/*    Modifie par : Payam KIANI                         le : 22/07/1992     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/
/* fichier tasgraph.c : parcours de graphe des cones; son dedoublement et   */
/*                      detection des boucles par un parcours en profondeur */
/*                      et parcours du graphe dedouble en largeur pour de-  */
/*                      tection des chaines critiques.                      */
/****************************************************************************/
#include "tas_all.h"

Nbloc_list  *HEAD_BLOC_NODE = NULL ; /* tete des blocs des noeuds        */
chain_list  *HEAD_GRAPH     = NULL ; /* le noeud fictif pour loop_2      */
chain_list  *FIFO_UP               ; /* tete de la pile                  */
chain_list  *FIFO_B                ; /* queue de la pile                 */
long        PBN             = 0    ; /* nombre total de noeuds alloues   */
long        NP              = 0    ; /* nombre de chemins=>complexite    */
char        PARSER_MODE     = 'N'  ; /* dit si parser a ete lance ou non */
char        TAS_TRACE_MODE  = 'N'  ; /* variable pour la mode trace      */
long        LEN_FIFO               ; /* taille de la fifo                */
long        FIFO_USED              ; /* nombre de cases fifo utilisees   */
perf_list   *PERFFIG        = NULL ; /* list des chaines et couples      */
long        NB_COUPLE       = 0    ;
long        NB_CHAINE       = 0    ;
struct precharge *tas_PRECHR    = NULL ; /* liste des points precharges */


/*****************************************************************************/
/*                               fonction AddPrecharge()                     */
/* rajoute une porte prechargee a la liste existante.                        */
/*****************************************************************************/
struct precharge *tas_AddPrecharge(cone)
struct cone *cone ;

{
struct precharge *precharge = (struct precharge *)tasalloc(
                              sizeof(struct precharge)    ) ;

precharge->CONE = cone ;
precharge->CMD = NULL ;
precharge->NEXT = tas_PRECHR ;

return(precharge) ;
}


/*****************************************************************************/
/*                               fonction GetPrecharge()                     */
/* renvoie un point precharge trouve par son cone.                           */
/*****************************************************************************/
struct precharge *tas_GetPrecharge(cone)
struct cone *cone ;

{
struct precharge *precharge ;

for(precharge = tas_PRECHR ; (precharge != NULL) && (precharge->CONE != cone) ;
    precharge = precharge->NEXT) ;

return(precharge) ;
}


/*****************************************************************************
*                                function getnode()                          *
*       permet de recuperer un bloc de deux noeuds, d'index "index",         * 
*         correspondant aux niveux 0 et 1 du cone de meme index              * 
*****************************************************************************/
event_list *getnode(ptfig,index) 
desafig_list    *ptfig ; /* pointeur de la figure courante    */
long            index  ; /* index du double noeuds a chercher */

{
Nbloc_list  *aux                     ; /* bloc auxiliaire de double-noeuds */
long        nb = PBN - MAX_BLOC_NODE ; /* permet de trouver le bloc desire */

/*----------------------------------------------------------------------------*/
/* si "index" est plus grand que l'index du dernier element alloue, alors err.*/
/*----------------------------------------------------------------------------*/
if(PBN < index) tas_error(29,(char *)index,TAS_ERROR) ;

/*----------------------------------------------------------------------------*/
/* on parcourt les blocs des noeuds jusqu'a trouver le bloc de l'"index"      */
/*----------------------------------------------------------------------------*/
for(aux = HEAD_BLOC_NODE ; index < nb ; aux = aux->NEXT) nb -= MAX_BLOC_NODE ;

if (TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- getnode : nodes %ld in figure %s\n",index,
            ptfig->NAME) ;
    fflush(stdout) ;
    }

return(aux->NODE[index - nb]) ;/* on retourne le double noeuds d'ind. "index" */
}


/*****************************************************************************
*                             function addline()                             *
* permet de relier le noeud in au noeud out avec une arete de longueur val   *
*****************************************************************************/
int addline(in,out,val) 
event_list       *in  ; /* noeud initial     */
event_list       *out ; /* noeud final       */
long            val  ; /* valeur de l'arete */

{
line_list   *line    ; /* arete auxiliaire */

line = (line_list *)mbkalloc(sizeof(line_list)) ;
line->VAL = val ;
line->NODE = in ;
line->NEXT = out->INNODE ;
out->INNODE = line ;

if(TAS_TRACE_MODE == 'Y') /* mode  trace */
    {
    fprintf(stdout,"---tas--- addline : ") ;

    if(LEVEL(in) != 0) fprintf(stdout,"UP(") ; else fprintf(stdout,"DOWN(") ;

    if((in->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    fprintf(stdout,"connector %s) to ",((locon_list *)in->CONE)->NAME) ;
    else fprintf(stdout,"cone %ld - %s) to ",in->CONE->INDEX,in->CONE->NAME) ;

    if(LEVEL(out) != 0) fprintf(stdout,"UP(") ; else fprintf(stdout,"DOWN(") ;
    if((out->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    fprintf(stdout,"connector %s)",((locon_list *)in->CONE)->NAME) ;
    else fprintf(stdout,"cone %ld - %s)",out->CONE->INDEX,out->CONE->NAME) ;

    fprintf(stdout,", with %ld value\n",out->INNODE->VAL) ;
    fflush(stdout) ;
    }

return ; /* on ne  renvoie rien du tout ! */
}


/*****************************************************************************
*                           function linknode()                              *
*  relie deus ensembles de deux noeuds suivant l'existence des temps TPxx    *
*****************************************************************************/
int linknode(out,in,user)
event_list *in   ; /* noeud initial         */
event_list *out  ; /* noeud final           */
ptype_list *user ; /* l'arete quadri-valuee */

{
delay_list *delay = (delay_list *)user ; /* les delais */

if(delay == NULL) return ;

/*----------------------------------------------------------------------------*/
/* liaison des doubles noeuds initiaux et finaux suivant l'existance des temps*/
/*----------------------------------------------------------------------------*/
if(delay->TPHH != NOTIME) addline(in + 1,out + 1,delay->TPHH) ;
if(delay->TPLH != NOTIME) addline(in, out + 1,delay->TPLH) ;
if(delay->TPLL != NOTIME) addline(in,out,delay->TPLL) ;
if(delay->TPHL != NOTIME) addline(in + 1,out,delay->TPHL) ;

NP ++ ;

if(TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- linknode : linking ") ;

    if(LEVEL(in) != 0) fprintf(stdout,"UP(") ; else fprintf(stdout,"DOWN(") ;

    if((in->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    fprintf(stdout,"connector %s) to ",((locon_list *)in->CONE)->NAME) ;
    else fprintf(stdout,"cone %ld - %s) to ",in->CONE->INDEX,in->CONE->NAME) ;

    if(LEVEL(out) != 0) fprintf(stdout,"UP(") ; else fprintf(stdout,"DOWN(") ;
    if((out->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    fprintf(stdout,"connector %s)",((locon_list *)in->CONE)->NAME) ;
    else fprintf(stdout,"cone %ld - %s)",out->CONE->INDEX,out->CONE->NAME) ;

    fprintf(stdout,", with %ld value\n",out->INNODE->VAL) ;
    fflush(stdout) ;
    }

return ; /* on ne renvoie rien du tout ! */
}


/*****************************************************************************
*                           function pushnode()                              *
*     met le noeud "node" dans une pile FIFO dont le dernier element         *
*                       est pointe par bottom                                * 
*****************************************************************************/
chain_list *pushnode(bottom,node)
chain_list  *bottom ; /* dernier element de la pile */
event_list   *node   ; /* noeud a empiler            */

{
static chain_list *fifo ; /* initialise a NULL, car static */
static chain_list *next ;

if(FIFO_USED == (LEN_FIFO-1))
    {
    chain_list *aux ;

    aux = (chain_list *)mbkalloc(sizeof(chain_list)) ;
    aux->NEXT = NULL ;
    FIFO_B = FIFO_B->NEXT = aux ;
    LEN_FIFO ++ ;
    }

if(bottom == NULL)
    {
    if(FIFO_USED != 0) fifo->NEXT = next ;
    fifo = FIFO_UP ;
    FIFO_USED = 0 ;
    }
else fifo = bottom->NEXT = next ;

fifo->DATA = (char *)node ;
next = fifo->NEXT ;
fifo->NEXT = NULL ;
FIFO_USED ++ ;

node->TYPE |= INFIFO ;

if(TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- pushnode : add a node in the bottom of FIFO\n") ;
    fflush(stdout) ;
    }

return(fifo) ; /* on renvoie le dernier element a ajouter */
}


/*****************************************************************************
*                               function initial()                           *
*       pretraitement du graphe en cone pour mutation en graphe en noeuds    *
*****************************************************************************/
chain_list *initial(ptfig)
desafig_list    *ptfig ; /* pointeur de la figure courante */

{
Nbloc_list  *blocN          ; /* bloc auxiliaire de double-noeuds */
chain_list  *fifo    = NULL ; /* tete de la pile                  */
chain_list  *reg            ; /* registre auxiliaire              */
cone_list   *cone           ; /* cone auxiliaire                  */
locon_list  *connect        ; /* connecteur auxiliaire            */
delay_list  *delai          ; /* delai auxiliaire                 */
long        nb_cone  = 0    ; /* pour calcul du nombre de cones   */
long        nb_conn         ;
long        i               ;

FIFO_UP = (chain_list *)mbkalloc(sizeof(chain_list)) ;
FIFO_B  = (chain_list *)mbkalloc(sizeof(chain_list)) ;
FIFO_UP->NEXT = FIFO_B ;
FIFO_B->NEXT = NULL ;
FIFO_USED = 0 ; /* aucun element de la pile encore utilisee */
LEN_FIFO = 2 ;  /* longueur de la pile est a 2              */

/*---------------------------------------------------------------------------*/
/* initialisation des cones : X1 pour marquer un cone, Y1 pour adresse de    */
/* retour. On compte le nombre de cones pour allocation des noeuds           */
/*---------------------------------------------------------------------------*/
for(cone = ptfig->CONE ; cone != NULL ; cone = cone->NEXT)
    {
    cone->X1 = cone->Y1 = cone->Y2 = 0 ;
    cone->X2 = (long)cone->INCONE ;
    nb_cone ++ ;
    }

/*---------------------------------------------------------------------------*/
/*          allocation des cases double-noeuds suivant nb_cone               */
/*---------------------------------------------------------------------------*/
for(i = nb_cone ; i >= 0 ; i -= MAX_BLOC_NODE)
    {
    short j ;

    blocN = (Nbloc_list *)mbkalloc(sizeof(Nbloc_list)) ;
    blocN->NEXT = HEAD_BLOC_NODE ;
    HEAD_BLOC_NODE = blocN ;
    PBN += MAX_BLOC_NODE ;
    }

nb_conn = nb_cone + 1 ;

/*---------------------------------------------------------------------------*/
/* determination des sorties du graphe en cone pour effectuer le parcours en */
/* profondeur: pour chaque connecteur de direction 'T', 'O', 'B' ou 'Z' on   */
/* prend le champ EXT de son USER qui pointe sur le cone qui nous interesse. */
/* On peut alors creer le sommet fictif qui sera HEAD_GRAPH.                 */
/*---------------------------------------------------------------------------*/
for(connect = ptfig->LOCON ; connect != NULL ; connect = connect->NEXT)
    {
    if((connect->DIRECTION == 'O') || (connect->DIRECTION == 'B') ||
       (connect->DIRECTION == 'Z') || (connect->DIRECTION == 'T')   )
        {
        cone_list  *cone ;
        event_list *node ; /* noeud correspondant au cone */
        list_list  *in   ; /* liste des entrees du cone   */

        /* cone monte sur le connecteur */
        cone = (cone_list *)getptype(connect->USER,EXT)->DATA ;

        if(connect->DIRECTION == 'T')
            {
            connect->SIG = (losig_list *)nb_conn ;
            nb_conn ++ ;
            }

        /* si pas de cone EXT, alors connecteur suivant */
        if(cone == NULL) continue ;

        /* on empile le cone dans la liste des sorties graphe */
        HEAD_GRAPH = addchain(HEAD_GRAPH,(char *)cone) ;

        /*----------------------------------------------------*/
        /* on garde le connecteur de sortie dans le noeud pour*/
        /*              le perfmodule                         */
        /*----------------------------------------------------*/
        node = getnode(ptfig,cone->INDEX);
        node[0].USER = node[1].USER = (ptype_list *)connect ;

        /*----------------------------------------------------*/
        /* on empile les noeuds de sortie pour le parcours en */
        /* largeur.                                           */
        /*----------------------------------------------------*/
        fifo = addchain(fifo,(char *)node) ;
        fifo = addchain(fifo,(char *)(node + 1)) ;
        }
    else
        {
        connect->SIG = (losig_list *)nb_conn ;
        nb_conn ++ ;
        }
    }

while(nb_conn >= PBN)
    {
    short j ;

    blocN = (Nbloc_list *)mbkalloc(sizeof(Nbloc_list)) ;
    blocN->NEXT = HEAD_BLOC_NODE ;
    HEAD_BLOC_NODE = blocN ;
    PBN += MAX_BLOC_NODE ;
    nb_conn -= MAX_BLOC_NODE ;
    }

/*---------------------------------------------------------------------------*/
/* s'il n'y a pas de connecteur de sortie du circuit (OUT ou INOUT en sortie)*/
/* c'est genant pour sortir les donnees! => erreur.                          */
/*---------------------------------------------------------------------------*/
if(HEAD_GRAPH == NULL)
    {
    /*------------------------------------------------------------------------*/
    /* si c'est pour la caracterisation, un circuit sans connecteur de sortie */
    /* ne doit pas poser de probleme. Un '.car' doit quand meme etre genere.  */
    /* Ce n'est pas un message d'erreur qu'il faut envoyer, mais d'avertis-   */
    /* sement.                                                                */
    /*------------------------------------------------------------------------*/
    if(PRFTYPE == 'C')
    tas_error(30,NULL,TAS_WARNING) ;
    else
    tas_error(30,NULL,TAS_ERROR) ;
    }

/*---------------------------------------------------------------------------*/
/* les latchs sont des points d'arrets. Ils sont donc consideres comme entree*/
/* et en meme temps sortie du graphe (un peu comme les INOUT!)               */
/*---------------------------------------------------------------------------*/
for(reg = ptfig->REG ; reg != NULL ; reg = reg->NEXT)
    {
    event_list *node ;
    list_list *in   ;

    /*-----------------------------------------------------------------------*/
    /* on initialise les double-noeuds correspondant aux latchs              */
    /*-----------------------------------------------------------------------*/
    node = getnode(ptfig,((cone_list *)reg->DATA)->INDEX) ;
    fifo = addchain(fifo,(char *)node) ;/* empilement du noeud niveau 0 */
    fifo = addchain(fifo,(char *)(node + 1)) ;/* noeud niveau 1 */

    HEAD_GRAPH = addchain(HEAD_GRAPH,reg->DATA) ; /* empilement du cone */

    /* il faut ajouter l'entree commande comme un point d'arret */
    for(in = ((cone_list *)reg->DATA)->INCONE ; in != NULL ;
        in = in->NEXT)
    if((in->TYPE & COMMAND) == COMMAND)
        {
        cone = (cone_list *)in->DATA ;

        if((cone->TYPE & DEJAEMPILE) == DEJAEMPILE) continue ;

        node = getnode(ptfig,cone->INDEX) ;
        fifo = addchain(fifo,(char *)node) ;
        fifo = addchain(fifo,(char *)(node + 1)) ;

        cone->TYPE |= (LCOMMAND | DEJAEMPILE) ;

        HEAD_GRAPH = addchain(HEAD_GRAPH,in->DATA) ;
        }
    }

if(TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- initial : initialize cone diagram of figure %s\n",
            ptfig->NAME) ;
    fflush(stdout) ;
    }

return(fifo) ; /*  on renvoie la pile des sorties du graphe en noeud */
}


/*****************************************************************************
*                               function loop_2()                            *
*  fonction du parcours en profondeur du graphe permettant la detection      *
*   des boucles (donc erreurs) et la mutation (dedoublement) du graphe       *
*                           de cones en noeuds                               *
*****************************************************************************/
chain_list *loop_2(ptfig)
desafig_list    *ptfig ; /* pointeur de la figure courante */

{
cone_list   *cone             ; /* cone auxiliaire                 */
chain_list  *list_loop = NULL ; /* liste de toutes les boucles     */
chain_list  *loop      = NULL ; /* liste des cones dans une boucle */
chain_list  *reg              ; /* registre auxiliaire             */
chain_list  *fifo_up          ; /* tete de la pile en noeud        */
event_list  *node1            ; /* pour le noeud final             */
event_list  *node2            ; /* pour le noeud initial           */
short       nb_loop    = 0    ; /* le nombre de boucles            */

fifo_up = initial(ptfig) ; /* recherche des sorties du graphe */

while(HEAD_GRAPH != NULL) /* tant que le "noeud fictif" n'est pas vide */
    {
    cone_list   *sommet = (cone_list *)HEAD_GRAPH->DATA ; /* cone de sortie */

    /*-----------------------------------------------------------------------*/
    /* si le cone n'a pas encore ete parcouru, le double-noeud correspondant */
    /* n'est pas encore initialise => initialisation                         */
    /*-----------------------------------------------------------------------*/
    if(sommet->Y2 != MARQUE)
        {
        node1 = getnode(ptfig,sommet->INDEX) ;
        node1[0].CONE = node1[1].CONE = sommet ;
        node1[0].TYPE = node1[1].TYPE = CONE_TYPE ;

        if((sommet->TYPE & tas_BYPASSIN) == tas_BYPASSIN)
        node1[0].TYPE = node1[1].TYPE |= tas_BYPASSIN ;

        if((sommet->TYPE & tas_BYPASSOUT) == tas_BYPASSOUT)
        node1[0].TYPE = node1[1].TYPE |= tas_BYPASSOUT ;

        if((sommet->TYPE & (LATCH | MEMORY)) != 0)
            {
            node1[0].TYPE = node1[1].TYPE |= LATCH ;
            }
        if((sommet->TYPE & LCOMMAND) == LCOMMAND)
            {
            node1[0].TYPE = node1[1].TYPE |= LCOMMAND ;
            }
/* AJOUT DU 16/04/93 PAR PAYAM: TRAITEMENT DES SIGNAUX REENTRANTS */
/* AJOUT des 5 ligne qui suivent ('}' compris). Avant: rien       */
        if((sommet->TYPE & EXT) == EXT)
        if(TAS_HIER == 'Y')
            {
            node1[0].TYPE = node1[1].TYPE |= EXT ;
            }

        if((sommet->TYPE & PRECHARGE) == PRECHARGE)
            {
            node1[0].TYPE = node1[1].TYPE |= tas_PRECHARGE ;

            /* ajout dans la liste des points precharges */
            tas_PRECHR = tas_AddPrecharge(sommet) ;
            }

/* AJOUT DU 16/04/93 PAR PAYAM: TRAITEMENT DES SIGNAUX REENTRANTS */

        node1[0].DELAY = node1[1].DELAY = NOTIME ;
        node1[0].INNODE = node1[1].INNODE = NULL ;
        node1[0].OUTNODE = node1[1].OUTNODE = NULL ;
        node1[1].TYPE |= HIGHLEVEL ;
        sommet->Y2 = MARQUE ;
        }

    /*-----------------------------------------------------------------------*/
    /* si le cone courant (sommet) a une adresse de retour vide, on est sur  */
    /* une sortie du graphe. On boucle tant que toutes ses entrees ne sont   */
    /* pas traites                                                           */
    /*-----------------------------------------------------------------------*/
    while((sommet->Y1 != (long)NULL) || (sommet->INCONE != NULL))
        {
        cone_list   *aux_sommet ;

        /*-------------------------------------------------------------------*/
        /* sur le cone courant : si l'entree n'est ni un connecteur, ni un   */
        /* cone VDD, ni un cone VSS, ni un element REGISTER (second inverseur*/
        /* du latch), si le cone suivant n'est pas un latch et si le cone    */
        /* courant est ouvert (il existe au moins une entree pas encore      */
        /* traitee, alors on peut passer du cone i au cone j (voir rapport)  */
        /*-------------------------------------------------------------------*/
        if(                               (sommet->INCONE != NULL)  &&
        ((((cone_list *)sommet->INCONE->DATA)->TYPE & LATCH) == 0)  &&
        ((((cone_list *)sommet->INCONE->DATA)->TYPE & MEMORY) == 0)  &&
        ((sommet->INCONE->TYPE & (EXT | LOOP | VDD | VSS | COMMAND | PATH_INOUT)) == 0)  )
            {
            aux_sommet = (cone_list *)sommet->INCONE->DATA ;
            node2 = getnode(ptfig,aux_sommet->INDEX) ;
    
            if(aux_sommet->Y2 != MARQUE) /* voir pareil plus haut! */
                {
                node2[0].CONE = node2[1].CONE = aux_sommet ;
                node2[0].TYPE = node2[1].TYPE = CONE_TYPE ;

                if((aux_sommet->TYPE & tas_BYPASSIN) == tas_BYPASSIN)
                node2[0].TYPE = node2[1].TYPE |= tas_BYPASSIN ;

                if((aux_sommet->TYPE & tas_BYPASSOUT) == tas_BYPASSOUT)
                node2[0].TYPE = node2[1].TYPE |= tas_BYPASSOUT ;

                if((aux_sommet->TYPE & LCOMMAND) == LCOMMAND)
                node2[0].TYPE = node2[1].TYPE |= LCOMMAND ;

                if((aux_sommet->TYPE & PRECHARGE) == PRECHARGE)
                    {
                    node2[0].TYPE = node2[1].TYPE |= tas_PRECHARGE ;

                    /* ajout dans la liste des points precharges */
                    tas_PRECHR = tas_AddPrecharge(aux_sommet) ;

                    /* empilement pour le parcours en largeur */
                    fifo_up = addchain(fifo_up,(char *)node2) ;
                    fifo_up = addchain(fifo_up,(char *)(node2+1)) ;
                    }
/* AJOUT DU 16/04/93 PAR PAYAM: TRAITEMENT DES SIGNAUX REENTRANTS    */
/* AJOUT des 5 ligne qui suivent (accolades comprises). Avant: rien  */
                if((aux_sommet->TYPE & (CONE_CONNECT | EXT)) == EXT)
                if(TAS_HIER == 'Y')
                    {
                    node2[0].TYPE = node2[1].TYPE |= EXT ;
                    }

                node2[0].INNODE = node2[1].INNODE = NULL ;
                node2[0].OUTNODE = node2[1].OUTNODE = NULL;
                node2[0].DELAY = node2[1].DELAY = NOTIME ;
                node2[1].TYPE |= HIGHLEVEL ;
                aux_sommet->Y2 = MARQUE ;
                }

            /*---------------------------------------------------------------*/
            /* si le sommet initial est ouvert mais marque, c'est qu'il y a  */
            /* une boucle                                                    */
            /*---------------------------------------------------------------*/
            if(aux_sommet->X1 == MARQUE && aux_sommet->INCONE != NULL)
                {
                nb_loop ++ ; /* on incremente le nombre de boucle */

                tas_error(31,ptfig->NAME,TAS_WARNING) ;

                /*-----------------------------------------------------------*/
                /* dans loop on range le chemin complet de la boucle         */
                /*-----------------------------------------------------------*/
                for(cone=sommet ; cone!=aux_sommet ; cone=(cone_list *)cone->Y1)
                    {   
                    loop = addchain(loop,(char *)cone) ;
                    }
    
                /*-----------------------------------------------------------*/
                /* cette boucle est ajoutee a la liste des boucles           */
                /*-----------------------------------------------------------*/
                loop = addchain(loop,(char *)aux_sommet) ;
                }
        
            /* on fait la liaison que s'il n'y a pas de boucles */
            else
            linknode(node1,node2,sommet->INCONE->USER) ;

            /*---------------------------------------------------------------*/
            /* si boucle detectee, on elimine l'arete cause de la boucle et  */
            /* on continue. Ca permet de detecter toutes les boucles du      */
            /* circuit. Sinon on garde l'adresse de retour.                  */ 
            /*---------------------------------------------------------------*/
            if((nb_loop != 0) && (loop != NULL))
                {
                list_loop = addchain(list_loop,(char *)loop) ;
                loop = NULL ;

                /*-----------------------------------------------------------*/
                /* en typant LOOP l'entree de rebouclage, on permet a TAS de */
                /* ne pas aller dans la boucle lors du parcours en largeur.  */
                /* Cela permet d'avoir un perfmodule meme s'il y a des cycles*/
                /*-----------------------------------------------------------*/
                sommet->INCONE->TYPE |= LOOP ;
                sommet->INCONE = sommet->INCONE->NEXT ;
                aux_sommet = sommet ;
                }
            else aux_sommet->Y1 = (long)sommet ;
    
            aux_sommet->X1 = MARQUE ; /* marquage du cone traite */
            node1 = node2 ; /* noeud suivant pour nouvelle iteration */
            sommet = aux_sommet ;
            }

        /*-------------------------------------------------------------------*/
        /* si on ne doit pas passer au noeud j, alors fin d'un chemin.       */
        /* Plusieurs cas se presentent :                                     */
        /*-------------------------------------------------------------------*/
        else
            {
            /*---------------------------------------------------------------*/
            /* si le cone est ferme (toutes les entrees traitees), on passe  */
            /* au cone precedent sur le chemin, pareil pour le noeud. On eli-*/
            /* mine le cone courant de la liste des entrees du cone precedent */
            /*---------------------------------------------------------------*/
            if(sommet->INCONE == NULL)
                {
                cone_list   *aux_cone = sommet ;

                sommet = (cone_list *)sommet->Y1 ;
                aux_cone->Y1 = 0 ;
                node1 = getnode(ptfig,sommet->INDEX) ;
                }

            else  /* sinon... */
            if((sommet->INCONE->TYPE & LOOP) != LOOP)
                {
                list_list   *aux_in = sommet->INCONE ;

                /*-----------------------------------------------------------*/
                /* si l'entree est un connecteur, on fait la liaison en      */
                /* gardant l'information. C'est utile pour perfmodule        */
                /*-----------------------------------------------------------*/
                if((aux_in->TYPE & EXT) == EXT)
                    {
                    if((aux_in->TYPE & (LOOP | PATH_INOUT)) == 0)
                    {
                    event_list   *node ;
                    long        ind = (long)((locon_list *)aux_in->DATA)->SIG ;
                    ptype_list  *ptype = ((locon_list *)aux_in->DATA)->USER ;

                    node = getnode(ptfig,ind) ;
                    node[0].CONE = node[1].CONE = (cone_list *)aux_in->DATA ;
                    node[0].TYPE = node[1].TYPE = tas_CONNECTOR ;
                    node[0].INNODE = node[1].INNODE = NULL ;
                    node[0].OUTNODE = node[1].OUTNODE = NULL;
                    node[0].DELAY = node[1].DELAY = NOTIME ;
                    node[0].USER = node[1].USER = ptype ;
                    node[1].TYPE |= HIGHLEVEL ;

                    linknode(node1,node,aux_in->USER) ;
                    }
                    }

                /*-----------------------------------------------------------*/
                /* si l'entree est un latch, il suffit de faire la liaison   */
                /*-----------------------------------------------------------*/
                else if((((cone_list *)aux_in->DATA)->TYPE & (LATCH | MEMORY))
                         != 0)
                    {
                    event_list   *node ;

                    node = getnode(ptfig,((cone_list *)aux_in->DATA)->INDEX) ;
                    linknode(node1,node,aux_in->USER) ;
                    }
                }

            /* on passe a l'entree suivante */
            if(sommet->INCONE != NULL) sommet->INCONE =  sommet->INCONE->NEXT ;
            }
        }

    HEAD_GRAPH = HEAD_GRAPH->NEXT ; /* sortie suivante du graphe */
    }

for(cone = HEAD_DESBFIG->CONE ; cone != NULL ; cone = cone->NEXT)
cone->INCONE = (list_list *)cone->X2 ;

/*---------------------------------------------------------------------------*/
/* s'il existe des boucles dans les parties combinatoires (comme on s'arrete */
/* sur les boucles et les bleeders etant "ignores", les boucles ne peuvent   */
/* etre que sur les parties combinatoires), il y a erreur de conception =>   */
/* on sort apres avoir donne toutes les boucles et leurs chemins.            */
/*---------------------------------------------------------------------------*/
if(nb_loop != 0)
    {
    FILE    *file ;
    char    *f ;
    int i = 1 ;

    f = mbkalloc(sizeof(strlen(ptfig->NAME)) + 6) ;
    strcpy(f,ptfig->NAME) ; 
    strcat(f,".LOOP") ;

    if((file = fopen(f,"w")) == NULL) /*  erreur en ouverture du fichier */
    tas_error(8,f,TAS_WARNING) ;

    if(TAS_LANG == 'E')
        {
        fprintf(file,"********** FILE OF LOOPS OF CIRCUIT %s\n",ptfig->NAME) ;
        fprintf(file,"----TAS V.%s of %s\n\n",VERSION,DATE_VER) ;
        if(nb_loop == 1) fprintf(file,"one loop was detected :\n") ;
        else fprintf(file,"%ld loops were detected :\n",nb_loop) ;
        }
    else if(TAS_LANG == 'F')
        {
        fprintf(file,"********** FICHIER DES BOUCLES DE LA FIGURE %s\n",
                ptfig->NAME) ;
        fprintf(file,"----TAS V.%s du %s\n\n",VERSION,DATE_VER) ;
        if(nb_loop == 1) fprintf(file,"une boucle a ete detectee :\n") ;
        else fprintf(file,"%ld boucles ont ete detectees :\n",nb_loop) ;
        }
 
    while(list_loop != NULL)
        {
        chain_list  *aux_l ;

        if(TAS_LANG == 'E')
        fprintf(file,"loop no %ld :\n",i) ;
        else if(TAS_LANG == 'F')
        fprintf(file,"boucle no %ld :\n",i) ;
        i++ ;

        for(aux_l=(chain_list *)list_loop->DATA;aux_l;aux_l=aux_l->NEXT)
            {
            fprintf(file,"%ld (%s)\n",((cone_list *)aux_l->DATA)->INDEX,
                    ((cone_list *)aux_l->DATA)->NAME) ;
            }

        fprintf(file,"\n") ;
        fflush(file) ;
        list_loop = list_loop->NEXT ;
        }

    if(fclose(file) != 0) tas_error(10,f,TAS_WARNING) ;
    /*----------------------------------------------------------------------*/
    /* a la fin de la declaration des boucles, on continue puisque le cycle */
    /* a ete rompu.                                                         */
    /*----------------------------------------------------------------------*/
/*    exit(1) ;*/
    }

/* on renvoie la liste des noeuds de sortie pour le parcours en largeur */
return(fifo_up) ; 
}


/******************************************************************************
*                          function addcritic()                               *
* sert pour memoriser les cones du chemin.                                    *
******************************************************************************/
critic_list *addcritic(bottom,node)
critic_list *bottom ; /* queue de la pile des elements du chemin critique */
event_list   *node   ; /* l'element a ajouter dans ce chemin               */

{
critic_list *aux_cr ;

aux_cr = (critic_list *)mbkalloc(sizeof(critic_list)) ;
aux_cr->NODE = node ;
aux_cr->DELAY = node->DELAY ;
aux_cr->NEXT = NULL ;
if(bottom != NULL) bottom->NEXT = aux_cr ; /* ajout de l'element en queue */

if(TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- addcritic : add ") ;
    if(LEVEL(node) != 0) fprintf(stdout,"UP(") ;
    else fprintf(stdout,"DOWN(") ;

    if((node->TYPE & tas_CONNECTOR) == tas_CONNECTOR)
    fprintf(stdout,"connector %s)",((locon_list *)node->CONE)->NAME) ;
    else fprintf(stdout,"cone %ld - %s)",node->CONE->INDEX,node->CONE->NAME) ;

    fprintf(stdout,"in the critical chain\n") ;
    fflush(stdout) ;
    }

return(aux_cr) ; /* renvoie l'element ajoute => queue de la pile */
}


/******************************************************************************
*                           function addlist()                                *
*       ajoute une chaine critique a la liste des chaines critiques           *
******************************************************************************/
int addlist(node,goal,type)
event_list *node ; /* le premier element de la chaine a trouver   */
event_list *goal ; /* sortie du chemin critique                   */
char       type  ; /* donne le type du chemin                     */

{
event_list   *avd_node         ; /* avant-dernier noeud          */
critic_list  *i_ch_o_up = NULL ; /* premier element de la chaine */
critic_list  *i_ch_o_b  = NULL ; /* premier element de la chaine */
event_list   *aux_node         ; /* noeud auxiliaire             */
event_list   *nodeaux   = NULL ;
char         typetr     = 'X'  ; /* type de la transition        */
char         frin              ; /* front d'entree               */
char         frout             ; /* front de sortie              */
chain_list   *intprech  = NULL ; /* liste des precharges interm. */

/* si l'entree est  BYPASSOUT ou la sortie BYPASSIN, on arrete */
if(((node->TYPE & tas_BYPASSOUT) == tas_BYPASSOUT) ||
   ((goal->TYPE & tas_BYPASSIN) == tas_BYPASSIN)     )
return ;

/*-------------------------------------------------------------------*/
/* pour les latches a ecriture differentielle, il ne faut pas garder */
/* la chaine qui va du latch a lui-meme de l'autre cote! Pour cela on*/
/* teste le champ USER d'un latch MEMORY.                            */
/*-------------------------------------------------------------------*/
if(((node->TYPE & tas_CONNECTOR) != tas_CONNECTOR) &&
   ((node->CONE->TYPE & MEMORY) == MEMORY))
    {
    ptype_list *ptype = getptype(node->CONE->USER,MEMORY) ;

    if(ptype != NULL)
    ptype = (ptype_list *)ptype->DATA ;

    if(goal->CONE == (cone_list *)ptype)
    return ;
    }

node->TYPE &= ~(DEBUT) ; /* pour un nouveau parcours de graphe */

/* il faut tester le rebouclage des connecteurs INOUT
if(((node->TYPE & tas_CONNECTOR) == tas_CONNECTOR) &&
   ((goal->TYPE & (LATCH | LCOMMAND | tas_PRECHARGE)) == 0) &&
   (node->CONE == (cone_list *)goal->USER) && (node->DELAY <= 0)) return ;*/
/* simplification: des que l'entree = sortie et temps <= 0, on saute */
if((node->CONE == (cone_list *)goal->USER) && (node->DELAY <= 0)) return ;


if(type == '\0') /* determination de l'entree et de la sortie */
    {
    if((goal->TYPE & tas_PRECHARGE) == tas_PRECHARGE)
        {
        if(LEVEL(goal) != 0) typetr = 'P' ;
        else typetr = 'E' ;
        }
    else typetr = 'X' ;

    frout = (LEVEL(goal) != 0) ? 'U':'D' ;
    }
else /* sortie haute impedance */
    {
    if(type == tas_ZE) typetr = 'E' ;
    else typetr = 'P' ;

    frout = 'Z' ;
    }
frin = (LEVEL(node) != 0) ? 'U':'D' ;

/*---------------------------------------------------------------------------*/
/* on reconstitue le chemin critique en gardant sa tete et sa queue          */
/*---------------------------------------------------------------------------*/
for(aux_node = node ; aux_node != NULL ; aux_node = aux_node->OUTNODE)
    {
    if(aux_node->OUTNODE == NULL) avd_node = nodeaux ;
    nodeaux = aux_node ;

    if((node->DELAY > TAS_SEUIL_MIN) && (node->DELAY < TAS_SEUIL_MAX))
        {
        i_ch_o_b = addcritic(i_ch_o_b,aux_node) ;
        if(i_ch_o_up == NULL) i_ch_o_up = i_ch_o_b ;
        }
    }

if(avd_node->OUTNODE == NULL)
avd_node = NULL ;

/* dans l'ancien perfmodule on met une seule chaine entre deux memes points */
if(((node->TYPE & LATCH) == LATCH) && ((goal->TYPE & (LATCH | LCOMMAND)) != 0))
    {
    chemcr_list *chaine ;

    /* on teste si l'ancien format du perfmodule sera genere */
    if((PRFTYPE == 'O') || (PRFTYPE == '2'))
    for(chaine = PERFFIG->CHAINE ; chaine != NULL ; chaine = chaine->NEXT)
        {
        if((node->CONE == chaine->UP->CONE) &&
           (goal->CONE == chaine->BOTTOM->CONE))
        /* alors chaine deja existante. Il faut garder la plus grande */
            {
            if(chaine->DELAY < node->DELAY)
                {
                chaine->PATH = i_ch_o_up ;
                chaine->DELAY = node->DELAY ;
                }
            return ;
            }
        }

    /* si on arrive ici, c'est qu'il faut ajouter une nouvelle chaine */
    chaine = (chemcr_list *)mbkalloc(sizeof(chemcr_list)) ;
    chaine->UP = node ;
    chaine->BOTTOM = goal ;
    chaine->BEFORE = avd_node ;
    chaine->USER = NULL ;
    chaine->PATH = i_ch_o_up ;
    chaine->TYPETR = typetr ;
    chaine->FRIN = frin ;
    chaine->FROUT = frout ;
    chaine->DELAY = node->DELAY ;
    chaine->NEXT = PERFFIG->CHAINE ;
    PERFFIG->CHAINE = chaine ;
    NB_CHAINE ++ ;

/* Le 23/06/94: le type provisoire PP pour les latches est enleve */
/*
    if((goal->TYPE & LATCH) == LATCH)
    if((getcmd(chaine)->TYPE & TP) == TP)
    chaine->TYPETR = 'P' ;*/
    } /* fin traitement des chaines */
else /* alors on a des couples. Note: les precharges seront dans les couples */
    {
    chemcr_list *cple ;

    /*---------------------------------------------------------------*/
    /* pour les portes prechargees avec l'etat HZ en sortie, il faut */
    /* verifier si le couple n'existe pas deja. Si oui on garde le   */
    /* plus grand.                                                   */
    /*---------------------------------------------------------------*/
    if((frout == 'Z') && ((goal->TYPE & tas_PRECHARGE) == tas_PRECHARGE))
    for(cple = PERFFIG->COUPLE ; cple != NULL ; cple = cple->NEXT)
        {
        if((node->CONE == cple->UP->CONE) &&
           (goal->CONE == cple->BOTTOM->CONE) &&
           (frin == cple->FRIN) && (frout == cple->FROUT))
        /* alors couple deja existante. Il faut garder la plus grande */
            {
            if(cple->DELAY < node->DELAY)
                {
                cple->PATH = i_ch_o_up ;
                cple->DELAY = node->DELAY ;
                }
            return ;
            }
        }

    cple = (chemcr_list *)mbkalloc(sizeof(chemcr_list)) ;

    cple->UP = node ;
    cple->BOTTOM = goal ;
    cple->BEFORE = avd_node ;
    cple->USER = NULL ;
    cple->PATH = i_ch_o_up ;
    cple->TYPETR = typetr ;
    cple->FRIN = frin ;
    cple->FROUT = frout ;
    cple->DELAY = node->DELAY ;
    cple->NEXT = PERFFIG->COUPLE ;
    PERFFIG->COUPLE = cple ;
    NB_COUPLE ++ ;

/* Payam, le 23/06/94: j'ai enleve le type de transition PP pour les latches */
/*
    if((goal->TYPE & LATCH) == LATCH)
    if((getcmd(cple)->TYPE & TP) == TP)
    cple->TYPETR = 'P' ;*/
    }

/* on garde le chemin critique */
if(node->DELAY > TAS_MAXPATH->DELAY)
    {
    critic_list  *path ;

    for(aux_node = node , path = TAS_MAXPATH->PATH ;
        aux_node != NULL ;
        aux_node = aux_node->OUTNODE , path = path->NEXT)
        {
        path->NODE = aux_node ;
        path->DELAY = aux_node->DELAY ;

        if((path->NEXT == NULL) && (aux_node->OUTNODE != NULL))
            {
            path->NEXT = (critic_list *)mbkalloc(sizeof(critic_list)) ;
            path->NEXT->NEXT = NULL ;
            }
        }

    TAS_MAXPATH->NEXT = NULL ;
    TAS_MAXPATH->UP = node ;
    TAS_MAXPATH->BOTTOM = goal ;
    TAS_MAXPATH->BEFORE = avd_node ;
    TAS_MAXPATH->DELAY = node->DELAY ;
    TAS_MAXPATH->TYPETR = typetr ;
    TAS_MAXPATH->FRIN = frin ;
    TAS_MAXPATH->FROUT = frout ;
    TAS_MAXPATH->USER = NULL ;
    }

/*------------------------------------------------------------------------*/
/* si l'entree est prechargee et la sortie latchee, il faut rajouter dans */
/* la liste des commades de latch associees a la precharge, la commande   */
/* correspondante de la sortie.                                           */
/*------------------------------------------------------------------------*/
if(((node->TYPE & tas_PRECHARGE) == tas_PRECHARGE) &&
   ((goal->TYPE & LATCH) == LATCH)                   )
    {
    struct precharge *precharge = tas_GetPrecharge(node->CONE) ;
    char             *cmd       = ((cone_list *)getcmd(PERFFIG->COUPLE)->
                                   TRANS->GRID)->NAME          ;
    chain_list       *auxcmd                                   ;

    for(auxcmd = precharge->CMD ; (auxcmd != NULL) && (auxcmd->DATA != cmd) ;
        auxcmd = auxcmd->NEXT) ;

    if(auxcmd == NULL)
    precharge->CMD = addchain(precharge->CMD,(void *)cmd) ;
    }

if(TAS_TRACE_MODE == 'Y') /* mode trace */
    {
    fprintf(stdout,"---tas--- addlist : add a critical chain in the list\n") ;
    fflush(stdout) ;
    }

return ; /* on renvoie la liste des chaines critiques */
}


/*****************************************************************************/
/*                           fonction BreadthFirst()                         */
/* parcours en largeur d'abord du cone d'influence d'un noeud donne.Avant de */
/* sortir, les noeuds sont reinitialises.                                    */
/*****************************************************************************/
chemcr_list *tas_BreadthFirst(root,type,metaroot)
struct event *root     ; /* sommet du cone d'influence                 */
char         type      ; /* dit si le traitement est de type HZ ou non */
struct event *metaroot ; /* pour une liaison eventuelle avec root      */

{
chain_list  *head_fifo        ; /* tete de la pile FIFO            */
chain_list  *fifo_up          ; /* element courant de la pile FIFO */
chain_list  *fifo_b    = NULL ; /* queue de la pile FIFO           */
chain_list  *in_Gi     = NULL ; /* queue des entrees du graphe Gi  */

/* si l'arrivee est un cone VDD ou VSS, on ignore la chaine */
if((root->CONE->TYPE & (VDD | VSS)) != 0)
return ;

root->DELAY = 0 ; /* reset du premier temps */
head_fifo = fifo_up = fifo_b = pushnode(fifo_b,root) ;

while(fifo_up != NULL) /* debut du parcours en largeur */
    {
    /* sommet de la pile */
    event_list *node = (event_list *)fifo_up->DATA ;
    line_list  *in ; /* liste des entrees du noeud courant */

    /* on garde dans la pile les entrees du graphe Gi pour ajouter */
/* MODIF du 16/04/93 par PAYAM: ajout de EXT dans la liste. Avant:      */
/*  if(((node->TYPE & (LATCH | tas_CONNECTOR | tas_PRECHARGE)) != 0) && */
    if(((node->TYPE & (LATCH | tas_CONNECTOR | tas_PRECHARGE | EXT)) != 0) &&
       (node->OUTNODE != NULL))
        {
        if(ETAT_DEB(node) == 0)
            {
            in_Gi = addchain(in_Gi,(char *)node) ;
            node->TYPE |= DEBUT ;
            }
        }
    /*-------------------------------------------------------------------*/
    /* si on arrive sur un noeud dont on ne veut pas traiter les entrees */
    /* on passe au noeud suivant.                                        */
    /*-------------------------------------------------------------------*/
    else if((node->TYPE & tas_BYPASSIN) != tas_BYPASSIN)
    for(in = node->INNODE ; in != NULL ; in = in->NEXT) 
        {
        long        time ;

        if(in->NODE == root)
            {
            tas_error(32,(char *)in->NODE->CONE,TAS_WARNING) ;
            continue ;
            }

        /*---------------------------------------------------------------*/
        /* si on arrive sur un noeud dont on ne veut pas traiter les     */
        /* sortie, on continue. Sinon:                                   */
        /* rafraichissement de l'entree (voir compte-rendu) :            */
        /*           delai(Xi) = SUP[delai(Xi),delai(Y)+di]              */
        /*---------------------------------------------------------------*/
        if((in->NODE->TYPE & tas_BYPASSOUT) != tas_BYPASSOUT)
        if((time = node->DELAY + in->VAL) > in->NODE->DELAY)
            {
            in->NODE->DELAY = time ;
            in->NODE->OUTNODE = node ;

            if((in->NODE->TYPE & INFIFO) != INFIFO)
                { 
                /* empilement de l'entree */
                fifo_b = pushnode(fifo_b,in->NODE) ; 
                in->NODE->TYPE |= INFIFO ;
                }
            }
        }

    node->TYPE &= OUTFIFO ;
    fifo_up = fifo_up->NEXT ; /* element suivant de la pile */
    }

root->OUTNODE = metaroot ;
if(metaroot != NULL) root = metaroot ;

/*-------------------------------------------------------------------*/
/* creation de la liste des chemins critiques du graphe Gi           */
/*-------------------------------------------------------------------*/
while(in_Gi != NULL)
    {
    addlist((event_list *)in_Gi->DATA,root,type) ;
    in_Gi = in_Gi->NEXT ;
    }

/*--------------------------------------------------------------------*/
/* initialisation des noeuds traites pour nouveau parcours de graphe  */
/*--------------------------------------------------------------------*/
while(head_fifo != NULL)
    {
    event_list   *node = (event_list *)head_fifo->DATA ;

    node->OUTNODE = NULL ;
    node->DELAY = NOTIME ;
    head_fifo = head_fifo->NEXT ;
    }
}


/*****************************************************************************/
/*                           fonction TreatZPre()                            */
/* traitement de l'etat HZ de la sortie de la porte prechargee.              */
/*****************************************************************************/
int tas_TreatZPre(node)
struct event *node ;

{
if((node->TYPE & tas_PRECHARGE) == tas_PRECHARGE)
    {
    char        type  = (LEVEL(node) != 0) ? tas_ZP : tas_ZE ;
    struct line *line                                        ;

    node->DELAY = 0 ;

    for(line = node->INNODE ; line != NULL ; line = line->NEXT)
        {
        struct line  *auxline ;

        for(auxline = node->INNODE ; auxline != NULL ; auxline = auxline->NEXT)
        if((line->NODE->CONE == auxline->NODE->CONE) &&
           (LEVEL(line->NODE) != LEVEL(auxline->NODE)))
        break ;

        if(auxline == NULL)
            {
            /*-----------------------------------------------------*/
            /* si le noeud en question est au niveau haut, il faut */
            /* effectuer la recherche a partir du niveau bas et    */
            /* vice-versa.                                         */
            /*-----------------------------------------------------*/
            if(LEVEL(line->NODE) != 0) /* alors traitement pour niv bas */
            tas_BreadthFirst(line->NODE-1,type,node) ;
            else /* alors traitement pour niveau haut */
            tas_BreadthFirst(line->NODE+1,type,node) ;
            }
        }
    }
}


/******************************************************************************
*                            function timecat()                               * 
* parcours en largeur du graphe : determination de toutes ses chaines longues *
******************************************************************************/
perf_list *timecat(ptfig)
desafig_list    *ptfig ; /* pointeur de la figure courante */

{
chain_list  *heads ; /* liste de toutes les sorties du graphe */

heads = loop_2(ptfig) ; /* dedoublement et detection des boucles */

PERFFIG = (perf_list *)mbkalloc(sizeof(perf_list)) ;

/* initialisation de la figure des performances */
PERFFIG->COUPLE = NULL ;
PERFFIG->CHAINE = NULL ;

/* initialisation du representant du chemin critique du circuit */
TAS_MAXPATH = (chemcr_list *)mbkalloc(sizeof(chemcr_list)) ;
TAS_MAXPATH->UP = NULL ;
TAS_MAXPATH->NEXT = NULL ;
TAS_MAXPATH->DELAY = 0x80000000 ;
TAS_MAXPATH->PATH = (critic_list *)mbkalloc(sizeof(critic_list)) ;
TAS_MAXPATH->PATH->NEXT = NULL ;

if(TAS_LANG == 'E')
fprintf(stdout,"complexity = %ld\n",NP) ;
else if(TAS_LANG == 'F')
fprintf(stdout,"indice de complexite : %ld\n",NP) ;
fflush(stdout) ;

while(heads != NULL) /* tant que toutes les sorties ne sont pas traitees */
    {
    struct event *node = (struct event *)heads->DATA ;

    /* traitement des etats HZ des portes prechargees */
    if((node->TYPE & tas_PRECHARGE) == tas_PRECHARGE)
    tas_TreatZPre(node) ;

    tas_BreadthFirst(node,'\0',NULL) ;
    heads = heads->NEXT ;
    }

/* on retourne la liste des chemins critiques pour ediperf et perfmodule */
return(PERFFIG) ;
}
